window.fc = window.fc || {};
window.fc.zsqq = window.fc.zsqq || {};

(function (exports, Laya) {
    'use strict';
    var ZsSdk = window.zs.sdk;
    class Channel {

        static init() {
            return true;
        }

        static initAdService(info) {
            console.log("qqChannel.init");

            if (!info) {
                console.warn("[adapter]Channel.init:", "info is invalid");
                return true;
            }

            if (info.videoAdUnitId) {
                this.initVideoAd(info.videoAdUnitId);
            }

            if (info.bannerAdUnitId) {
                this.initBannerAd(info.bannerAdUnitId, 1);
            }

            if (info.insertAdUnitId) {
                this.initInsertAd(info.insertAdUnitId);
            }

            if (info.gameBoxUnitId) {
                this.initGameBox(info.gameBoxUnitId);
            }

            if (info.blockAdUnitId) {
                this.initGameBanner(info.blockAdUnitId);
            }
        }

        static initBannerAd(bannerAdUnitId, bannerAdScale) {
            this.bannerAdUnitId = bannerAdUnitId;
            this.bannerAdScale = bannerAdScale;
            this.bannerAdIndex = Math.floor( Math.random() * this.bannerAdUnitId.length);
            return true;
        }

        static initGameBanner(blockAdUnitId) {
            this.blockAdUnitId = blockAdUnitId;
            return true;
        }

        static initInsertAd(adUnitId) {
            this.insertAdUnitId = adUnitId;
            this.insertLastShowTime = Date.now();
            return true;
        }

        static initGameBox(appBoxAdUnitId) {
            console.log("initGameBox");
            if (typeof qq == "undefined") {
                console.warn("[adapter]Channel.initGameBox:", "qq is undefine");
                return false;
            }
            this.appBoxAdReady = false;
            this.appBoxAdUnitId = appBoxAdUnitId;
            var self = this;
            this.appBoxAd = qq.createAppBox({ adUnitId: self.appBoxAdUnitId });
            this.appBoxAd.load().then(() => {
                self.appBoxAdReady = true;
            });
            return true;
        }

        static initVideoAd(videoAdUnit) {
            console.log("initVideoAd");
            this.rewardedVideoAd = null;
            if (typeof qq == "undefined") {
                console.warn("[adapter]Channel.initVideoAd:", "qq is undefine");
                return;
            }
            if (videoAdUnit == null || videoAdUnit == "") {
                tconsole.warn("[adapter]Channel.initVideoAd:", "videoAdUnit is invalid");
                return false;
            }
            this.rewardedVideoAd = qq.createRewardedVideoAd({ adUnitId: videoAdUnit });
            if (this.rewardedVideoAd == null) {
                console.warn("[adapter]Channel.initVideoAd:", "rewardedVideoAd is invalid");
                return false;
            }
            var self = this;
            this.rewardedVideoAd.onLoad(function () {
                self.videoLoad = true;
                console.debug("video加载成功" + self.videoLoad);
            })
            this.rewardedVideoAd.onError(function (err) {
                console.debug("视频报错:" + JSON.stringify(err));
                self.videoLoad = false;
                self.rewardedVideoAd = null;
                if (self.videoErrorHandler) {
                    self.videoErrorHandler.runWith(err);
                }
            });
            this.rewardedVideoAd.onClose(function (res) {
                // 用户点击了【关闭广告】按钮
                // 小于 2.1.0 的基础库版本，res 是一个 undefined
                self.videoLoad = false;
                self.rewardedVideoAd.load();
                if (res && res.isEnded || res === undefined) {
                    // 正常播放结束，可以下发游戏奖励
                    if (self.videoCompletedHandler) {
                        self.videoCompletedHandler.run();
                    }
                } else {
                    // 播放中途退出，不下发游戏奖励
                    if (self.videoInterruptHandler) {
                        self.videoInterruptHandler.run();
                    }
                }
            });
            return true;
        }

        static isGameBoxValid() {
            return this.appBoxAd != null;
        }

        static login(successHandler, failedHandler) {

            if (typeof qq == "undefined") {
                console.warn("[adapter]Channel.login:", "qq is undefine");
                return false;
            }

            this.loginSuccess = successHandler;
            this.loginFailed = failedHandler;
            this.loginNum++;
            // var self = this;
            // qq.login({
            //     success: function (res) {
            //         if (res.code) {
            //             if (self.loginSuccess) self.loginSuccess.runWith({ identityId: res.code });
            //         }
            //         else {
            //             if (self.loginNum <= 3) {
            //                 self.login(self.loginSuccess, self.loginFailed);
            //             } else {
            //                 self.loginNum = 0;
            //                 if (self.failedHandler) self.failedHandler.runWith({ code: 2, desc: "Code不存在" });
            //             }
            //         }
            //     },
            //     fail: function () {
            //         if (self.loginNum <= 3) {
            //             self.login(self.loginSuccess, self.loginFailed);
            //         } else {
            //             self.loginNum = 0;
            //             if (self.failedHandler) self.failedHandler.runWith({ code: 2, desc: "login platform error" });
            //         }
            //     },
            //     complete: function () {
            //     }
            // });

            var self = this;
            ZsSdk.login(function (openid) {
                if (openid) {
                    ZsSdk.init(openid);
                    self.openid = openid;
                    if (self.loginSuccess) self.loginSuccess.runWith({ identityId: openid });
                }
                else {
                    if (self.failedHandler) self.failedHandler.runWith({ code: 2, desc: "Code不存在" });
                }
            },
                function () {
                    if (self.failedHandler) self.failedHandler.runWith({ code: 2, desc: "login platform error" });
                }
            );
            return true;
        }

        static loadCfg(success, fail) {
            ZsSdk.loadCfg(
                function (data) {
                    if (data) {
                        success(data);
                    }
                },
                function () {
                    fail();
                });
        }

        static loadSubpackage(pkgName, progressHandler, successHandler, failedHandler) {

            if (typeof qq == "undefined") {
                console.warn("[adapter]Channel.loadSubpackage:", "qq is undefine");
                return false;
            }

            var loadTask = qq.loadSubpackage({
                name: pkgName, // name 可以填 name 或者 root
                success(res) {
                    // 分包加载成功后通过 success 回调
                    successHandler && successHandler.runWith(1);
                },
                fail(res) {
                    // 分包加载失败通过 fail 回调
                    successHandler && successHandler.runWith(1);
                }
            });
            if (!loadTask) {
                console.warn("[adapter]Channel.loadSubpackage:", "loadTask is invalid");
                return false;
            }
            // if (progressHandler) {
            //     loadTask.onProgressUpdate(function (res) {
            //         progressHandler.runWith(res.progress);
            //     })
            // }
            return true;
        }

        static showModal(title, content, confirmTxt, cancelTxt, success, failed, complete) {

            if (typeof qq == "undefined") {
                console.warn("[adapter]Channel.showModal:", "qq is undefine");
                return false;
            }

            qq.showModal({
                title: title,
                content: content,
                cancelText: cancelTxt,
                confirmText: confirmTxt,
                success(res) {
                    success && success.runWith(res);
                },
                fail() {
                    failed && failed.run();
                },
                complete() {
                    complete && complete.run();
                }
            });

            return true;
        }

        static shareAppMessage(data, success, failed) {
            if (typeof qq == "undefined") {
                console.warn("[adapter]Channel.openShare:", "qq is undefine");
                return false;
            }

            var shareInfo = data || {};
            shareInfo.success = function () {
                success && success.run();
            };
            shareInfo.fail = function () {
                failed && failed.run();
            };

            qq.shareAppMessage(shareInfo);
            return true;
        }

        static playVideo(completedHandler, interruptHandler, errorHandler) {
            if (typeof wx == "undefined") {
                console.warn("[adapter]Channel.playVideo:", "wx is undefine");
                return false;
            }
            if (this.rewardedVideoAd == null) {
                console.warn("[adapter]Channel.playVideo:", "video is undefine");
                return false;
            }
            this.videoErrorHandler = errorHandler;
            this.videoCompletedHandler = completedHandler;
            this.videoInterruptHandler = interruptHandler;

            var self = this;
            if (self.videoLoad) {
                self.rewardedVideoAd.show();
                self.videoLoad = false;
            }
            else {
                console.debug("视频还未准备好");
                self.rewardedVideoAd.load()
                    .then(function () {
                        console.debug("加载后展示");
                        self.rewardedVideoAd.show();
                        self.videoLoad = false;
                    })
            }
        }

        static isVideoEnable() {
            return this.rewardedVideoAd != null;
        }

        static loadInsertAd(loadedHandler, errorHandler) {
            if (typeof qq == "undefined") {
                console.warn("[adapter]Channel.loadInsertAd:", "qq is undefine");
                return false;
            }

            if (this.insertAdUnitId == null || this.insertAdUnitId == '') {
                console.warn("[adapter]Channel.initInsertAd:", "insertAdUnitId is undefine");
                return false;
            }

            if (this.insertLastShowTime > 0 && Date.now() - this.insertLastShowTime < 30000) {
                console.warn("[adapter]Channel.loadInsertAd:", "load time short insert ad");
                return false;
            }

            this.insertAdLoadedHandler = loadedHandler;
            this.insertAdErrorHandler = errorHandler;

            if (this.insertAd == null) {
                this.insertAd = qq.createInterstitialAd({ adUnitId: this.insertAdUnitId });
                if (this.insertAd == null) {
                    console.warn("[adapter]Channel.loadInsertAd:", "insertAd is undefine");
                    return false;
                }
                var self = this;
                this.insertAd.onLoad(function () {
                    self.insertAdReady = true;
                    if (self.insertAdLoadedHandler) {
                        self.insertAdLoadedHandler.run();
                    }
                    self.insertAdLoadedHandler = null;
                });
                this.insertAd.onError(function (err) {
                    console.log(err);
                    if (self.insertAdErrorHandler) {
                        self.insertAdErrorHandler.runWith(err);
                    }
                });
                this.insertAd.onClose(function () {
                    self.insertAdReady = false;
                    if (self.insertAdCloseHandler) {
                        self.insertAdCloseHandler.run();
                    }
                    self.insertAdLoadedHandler = null;
                    self.insertAdErrorHandler = null;
                    self.insertAdCloseHandler = null;
                });
            }
            if (this.insertAdReady) {
                if (this.insertAdLoadedHandler) {
                    this.insertAdLoadedHandler.run();
                }
                this.insertAdLoadedHandler = null;
            }
            return true;
        }

        static showInsertAd(closeHandler, errorHandler) {
            if (this.insertAd == null) {
                console.warn("[adapter]Channel.showInsertAd:", "insertAd is undefine");
                return false;
            }

            this.insertAdCloseHandler = closeHandler;
            this.insertAdErrorHandler = errorHandler;

            this.insertAd.show();
            this.insertLastShowTime = Date.now();
            return true;
        }

        static isInsertAdEnable() {
            return true;//this.insertAd != null;
        }

        static createGameBanner(errorHandler, config) {
            this.blockAdReady = false;
            this.blockAdVisible = false;
            if (typeof qq == "undefined") {
                console.warn("[adapter]Channel.createGameBanner:", "qq is undefine");
                return false;
            }

            if (this.blockAdUnitId == null || this.blockAdUnitId == "") {
                console.warn("[adapter]Channel.createGameBanner:", "blockAdUnitId is invalid");
                return false;
            }

            if (this.blockAd != null) {
                console.warn("[adapter]Channel.createGameBanner:", "bannerAd is exist");
                return false;
            }

            this.blockAdConfig.x = config.x ? config.x : 20;
            this.blockAdConfig.y = config.y ? config.y : 20;
            this.blockAdConfig.size = config.size ? config.size : 5;
            this.blockAdConfig.orientation = config.orientation ? config.orientation : "landscape";
            this.blockAd = qq.createBlockAd({
                adUnitId: this.blockAdUnitId,
                style: { left: this.blockAdConfig.x, top: this.blockAdConfig.y },
                size: this.blockAdConfig.size,
                orientation: this.blockAdConfig.orientation
            });

            if (this.blockAd == null) {
                console.warn("[adapter]Channel.createGameBanner:", "bannerAd is undefine");
                return false;
            }

            var self = this;
            this.blockAd.onError(function (err) {
                console.log(err);
                self.blockAd = null;
                if (self.blockAdErrorHandler) {
                    self.blockAdErrorHandler.run();
                }
            });
            this.blockAd.onLoad(function () {
                this.blockAdReady = true;
                if (self.blockAdVisible) {
                    self.blockAd.show();
                }
            });
            var systemInfo = qq.getSystemInfoSync();
            this.blockAd.onResize(function (size) {
                console.log(size);
                if (size.width > 0 && self.blockAd) {
                    self.blockAd.style.left = (systemInfo.windowWidth - size.width) * 0.5;
                }
            });
            return true;
        }

        static showGameBanner() {
            console.log("showGameBanner");
            if (!this.blockAd) {
                this.createGameBanner(this.blockAdErrorHandler, this.blockAdConfig);
                if (!this.blockAd) {
                    return false;
                }
            }
            if (this.blockAdReady && !this.blockAdVisible) {
                this.blockAd.show();
            }
            this.blockAdVisible = true;
            return true;
        }

        static hideGameBanner() {
            console.log("hideGameBanner");
            if (!this.blockAd) {
                console.warn("[adapter]Channel.hideGameBanner:", "blockAd is undefine");
                return false;
            }
            this.blockAd.hide();
            this.blockAd.destroy();
            this.blockAd = null;
            this.blockAdVisible = false;
            return true;
        }

        static destroyGameBanner() {
            console.log("destroyGameBanner");
            if (!this.blockAd) {
                console.warn("[adapter]Channel.destroyGameBanner:", "blockAd is undefine");
                return false;
            }
            this.blockAd.destroy();
            this.blockAd = null;
            return true;
        }

        static createBannerAd(errorHandler) {
            if (typeof qq == "undefined") {
                console.warn("[adapter]Channel.createBannerAd:", "qq is undefine");
                return false;
            }

            if (this.bannerAdUnitId == null || this.bannerAdUnitId == "" || this.bannerAdUnitId.length == 0) {
                console.warn("[adapter]Channel.createBannerAd:", "bannerAdUnitId is invalid");
                return false;
            }

            if (this.bannerAd != null) {
                console.warn("[adapter]Channel.createBannerAd:", "bannerAd is exist");
                return false;
            }

            this.bannerAdIsReady = false;
            this.bannerAdVisible = false;
            this.bannerErrorHandler = errorHandler;
            var slideScale = (1 - this.bannerAdScale) * 0.5;
            var systemInfo = qq.getSystemInfoSync();
            this.bannerAd = qq.createBannerAd({
                adUnitId: this.bannerAdUnitId[this.bannerAdIndex],
                style: {
                    top: systemInfo.windowHeight - 100,
                    width: systemInfo.windowWidth * this.bannerAdScale,
                    left: systemInfo.windowWidth * slideScale
                }
            });
            this.bannerAdIndex = (this.bannerAdIndex + 1) % this.bannerAdUnitId.length;
            if (this.bannerAd == null) {
                console.warn("[adapter]Channel.createBannerAd:", "bannerAd is undefine");
                return false;
            }

            var self = this;
            // 720 * 250
            this.bannerAd.onResize(function (size) {
                console.log(systemInfo);
                if (self.bannerAd) {
                    self.bannerAd.style.top = systemInfo.windowHeight - size.height;
                    self.bannerAd.style.left = (systemInfo.windowWidth - size.width) * 0.5;
                }
            });

            this.bannerAd.onError(function (err) {
                console.log(err);
                self.bannerAd = null;
                if (self.bannerErrorHandler) {
                    self.bannerErrorHandler.run();
                }
            });

            this.bannerAd.onLoad(function () {
                self.bannerAdIsReady = true;
                if (self.bannerAdVisible) {
                    self.bannerAd.show();
                }
            });

            return true;
        }

        static showBannerAd() {
            console.log("showBannerAd");
            if (!this.bannerAd) {
                console.warn("[adapter]Channel.showBannerAd:", "bannerAd is undefine");
                return false;
            }
            if (this.bannerAdIsReady && !this.bannerAdVisible) {
                this.bannerAd.show();
            }
            this.bannerAdVisible = true;
            return true;
        }

        static hideBannerAd() {
            console.log("hideBannerAd");
            if (!this.bannerAd) {
                console.warn("[adapter]Channel.hideBannerAd:", "bannerAd is undefine");
                return false;
            }
            if (this.bannerAdIsReady && this.bannerAdVisible) {
                this.bannerAd.hide();
            }
            this.bannerAdVisible = false;
            return true;
        }

        static destroyBannerAd() {
            console.log("destroyBannerAd");
            this.bannerAdIsReady = false;
            this.bannerAdVisible = false;
            if (!this.bannerAd) {
                console.warn("[adapter]Channel.destroyBannerAd:", "bannerAd is undefine");
                return false;
            }
            this.bannerAd.destroy();
            this.bannerAd = null;
            return true;
        }

        static isBannerAdValid() {
            return !!this.bannerAd;
        }

        static showGameBox(success, fail, close) {
            console.log("showGameBox");
            if (typeof qq == "undefined") {
                console.warn("[adapter]Channel.showGameBox:", "qq is undefine");
                return false;
            }

            if (!this.appBoxAd) {
                console.warn("[adapter]Channel.showGameBox:", "appBoxAd is undefine");
                return false;
            }

            var self = this;
            this.appBoxAd.onClose(function () {
                self.appBoxAd.destroy().then(function () {
                    self.initGameBox(self.appBoxAdUnitId);
                });
                close && close.run();
            });

            if (this.appBoxAdReady) {
                this.appBoxAd.show().then(function () {
                    success && success.run();
                }).catch(function () {
                    fail && fail.run();
                });
                return true;
            }

            this.appBoxAd.load().then(function () {
                self.appBoxAd.show().then(function () {
                    success && success.run();
                }).catch(function () {
                    fail && fail.run();
                });
            });
            return true;
        }
    }
    Channel.loginNum = 0;
    Channel.rewardedVideoAd = null;
    Channel.videoCompletedHandler = null;
    Channel.videoInterruptHandler = null;
    Channel.videoErrorHandler = null;
    Channel.videoLoad = false;
    Channel.insertAd = null;
    Channel.insertAdUnitId = null;
    Channel.insertAdReady = false;
    Channel.insertAdLoadedHandler = null;
    Channel.insertAdErrorHandler = null;
    Channel.insertAdCloseHandler = null;
    Channel.blockAd = null;
    Channel.blockAdConfig = { x: 0, y: 0, size: 5 };
    Channel.blockAdUnitId = null;
    Channel.blockAdReady = false;
    Channel.blockAdVisible = false;
    Channel.blockAdErrorHandler = null;
    Channel.bannerAd = null;
    Channel.bannerAdUnitId = null;
    Channel.bannerAdScale = 1;
    Channel.bannerLiveTime = 0;
    Channel.bannerLastShowTime = 0;
    Channel.bannerErrorHandler = null;
    Channel.bannerReady = false;
    Channel.bannerVisible = false;
    Channel.realSize = {};

    Channel.insertLastShowTime = 0;

    Channel.bannerAdIndex = 0;

    Channel.appBoxAd = null;
    Channel.appBoxAdUnitId = null;
    Channel.appBoxAdReady = false;

    var validNetConnectType = ["2g", "3g", "4g", "5g", "wifi"];

    class Device {

        static get statusBarHeight() {
            return this.sDeviceInfo ? this.sDeviceInfo.statusBarHeight : 0;
        }

        static get screenWidth() {
            return this.sDeviceInfo ? this.sDeviceInfo.screenWidth : 1;
        }

        static get screenHeight() {
            return this.sDeviceInfo ? this.sDeviceInfo.screenHeight : 1;
        }

        static init(onShowHandler, onHideHandler) {
            if (typeof qq == "undefined") {
                console.warn("[adapter]Device.init:", "qq is undefine");
                return false;
            }
            this.sDeviceInfo = qq.getSystemInfoSync();
            console.log(this.sDeviceInfo);
            console.log("qqDeviceAdapter.init");
            qq.onShow(function (res) {
                console.log("qqDeviceAdapter.show:" + Date.now());
                if (onShowHandler) onShowHandler.runWith(res);
            });
            qq.onHide(function () {
                console.log("qqDeviceAdapter.hide:" + Date.now());
                if (onHideHandler) onHideHandler.run();
            });
            var self = this;
            qq.onNetworkStatusChange(function (res) {
                self.sNetworkType = res.networkType;
                self.sIsNetConnected = res.isConnected;
            });
            qq.getNetworkType({
                success(res) {
                    self.sNetworkType = res.networkType;
                }
            });
            return true;
        }

        static vibrateShort() {
            if (typeof qq == "undefined") {
                console.warn("[adapter]Device.vibrateShort:", "qq is undefine");
                return false;
            }
            qq.vibrateShort({
                fail: function () {
                    console.log("vibrateShort failed");
                },
            });
            return true;
        }

        static vibrateLong() {
            if (typeof qq == "undefined") {
                console.warn("[adapter]Device.vibrateLong:", "qq is undefine");
                return false;
            }
            qq.vibrateLong({
                fail: function () {
                    console.log("vibrateLong failed");
                },
            });
            return true;
        }

        static isNetValid() {
            return this.sIsNetConnected && validNetConnectType.indexOf(this.sNetworkType) != -1;
        }

        // static playMusic(res) {
        //     if (this.Instance && this.Instance.playMusic) {
        //         this.Instance.playMusic(res);
        //     }
        //     else {
        //         Laya.SoundManager.playMusic(res, 0);
        //     }
        // }

        // static stopMusic(res) {
        //     if (this.Instance && this.Instance.stopMusic) {
        //         this.Instance.stopMusic(res);
        //     }
        //     else {
        //         Laya.SoundManager.stopMusic(res);
        //     }
        // }

        // static playSound(res, loop, compHandler) {
        //     var audio = null;
        //     let self = this;
        //     if (this.sSoundInfo[res]) {
        //         audio = this.sAudioArr[this.sSoundInfo[res]["idx"]];
        //         console.debug("取到" + res + "的音频");
        //         if (!this.sSoundInfo[res]["isStop"]) {
        //             console.debug("" + res + "已在播放中");
        //             self.sContinue = true;
        //             return;
        //         }
        //     }
        //     if (!audio) {
        //         audio = wx.createInnerAudioContext();
        //         this.sAudioArr.push(audio);
        //         audio.onError(function (eCode) {
        //             console.debug("音效播放异常  code " + JSON.stringify(eCode));
        //             self.sAudioArr.splice(self.sSoundInfo[audio.src]["idx"]);
        //             delete self.sSoundInfo[audio.src];
        //             audio.destroy();
        //         });
        //     }
        //     audio.src = res;
        //     audio.loop = loop;
        //     this.sSoundInfo[res] = { "idx": this.sAudioArr.indexOf(audio), "ptime": new Date().getTime(), "isStop": false, "curTime": 0 };
        //     audio.onEnded(function () {
        //         if (compHandler) compHandler.run();
        //     });
        //     audio.play();
        // }

        // static pauseSound(res) {
        //     var audio = this.sAudioArr[this.sSoundInfo[res]["idx"]];
        //     if (!audio) return;
        //     audio.pause();
        //     audio.offEnded();
        // }

        // static stopSound(res) {
        //     if (!this.sSoundInfo[res]) return;
        //     this.sContinue = false;
        //     var a = new Date().getTime() - this.sSoundInfo[res]["ptime"]
        //     if (a < 300) {
        //         console.debug("间隔过短，延时关闭");
        //         let self = this;
        //         setTimeout(function () {
        //             if (!self.sSoundInfo[res]) return;
        //             var audio = self.sAudioArr[self.sSoundInfo[res]["idx"]];
        //             if (!audio || self.sSoundInfo[res]["isStop"] || self.sContinue) return;
        //             self.sSoundInfo[res]["isStop"] = true;
        //             audio.stop();
        //             audio.offEnded();
        //             console.debug("进入延时关闭");
        //         }, 300 - a);
        //         return;
        //     }
        //     var audio = this.sAudioArr[this.sSoundInfo[res]["idx"]];
        //     audio.stop();
        //     audio.offEnded();
        //     this.sSoundInfo[res]["isStop"] = true;
        //     console.debug("进入关闭");
        // }

        // static stopAll() {
        //     this.stopMusic();
        //     this.stopAllSound();
        // }

        // static stopAllSound() {
        //     for (const key in this.sSoundInfo) {
        //         this.stopSound(key);
        //     }
        // }
    }
    Device.sAudioArr = [];
    Device.sSoundInfo = {};
    Device.sContinue = false;
    Device.sDeviceInfo = null;
    Device.sNetworkType = null;
    Device.sIsNetConnected = false;

    exports.Channel = Channel;
    exports.Device = Device;

}(window.fc.zsqq, Laya))

window.fc.adapter = window.fc.zsqq;
