window.fc = window.fc || {};
window.fc.zsvivo = window.fc.zsvivo || {};
(function (exports, Laya) {
    'use strict';
    var ZsSdk = window.zs.sdk;
    class Channel {

        static init() {
            if (typeof qg == "undefined") {
                console.warn("[adapter]Channel.init:", "qg is undefine");
                return false;
            }
            console.log("vivo.init");
            qg.onError(function (data) {
                console.error(`vivo error message is ${data.message}`);
            });
            return true;
        }

        static initAdService(info) {
            if (!info) {
                console.warn("[adapter]Channel.init:", "info is invalid");
                return true;
            }
            if (info.videoInterval) {
                this.videoInterval = info.videoInterval;
            }

            if (info.videoAdUnitId) {
                this.initVideoAd(info.videoAdUnitId);
            }

            if (info.bannerAdUnitId) {
                this.initBannerAd(info.bannerAdUnitId, 1);
            }

            if (info.insertAdUnitId) {
                this.initInsertAd(info.insertAdUnitId);
            }

            if (info.nativeAdUnitId) {
                this.initNativeAd(info.nativeAdUnitId);
            }

            if (info.gameBoxUnitId) {
                this.initGameBox(info.gameBoxUnitId);
            }

            if (info.gameBannerUnitId) {
                this.initGameBanner(info.gameBannerUnitId);
            }
            return true;
        }
    
        /**初始化banner */
        static initBannerAd(bannerAdUnit, scale) {
            console.log("初始化banner", bannerAdUnit);
            this.bannerAdUnitId = bannerAdUnit;
            return true;
        }

        static initNativeAd(nativeAdUnitId) {
            this.nativeAdUnitId = nativeAdUnitId;
            this.nativeAdIndex = Math.floor( Math.random() * this.nativeAdUnitId.length);
            return true;
        }
    
        static initInsertAd(insertAdUnitId) {
            this.insertAdUnitId = insertAdUnitId;
            return true;
        }
    
        /**初始化视频广告 */
        static initVideoAd(videoAdUnit) {
            this.rewardedVideoAd = null;
            if (typeof qg == "undefined") {
                console.warn("[adapter]Channel.initVideoAd:", "qg is undefine");
                return false;
            }
            if (videoAdUnit == null || videoAdUnit == "") {
                console.warn("[adapter]Channel.initVideoAd:", "videoAdUnit is invalid");
                return false;
            }
            this.rewardedVideoAd = qg.createRewardedVideoAd({ posId: videoAdUnit, style: {} });
            if (this.rewardedVideoAd == null) {
                console.warn("[adapter]Channel.initVideoAd:", "rewardedVideoAd is invalid");
                return false;
            }
            var self = this;
            this.rewardedVideoAd.onError(function (err) {
                console.log("激励广告展示失败", err.errCode, JSON.stringify(err));
                if (self.videoErrorHandler) {
                    self.videoErrorHandler.runWith(err);
                }
                self.videoErrorHandler = null;
                self.videoPlay = false;
                self.videoReady = false;
            });
            this.rewardedVideoAd.onLoad(function() {
                self.videoReady = true;
                if (self.videoPlay) {
                    self.rewardedVideoAd.show();
                }
            });

            this.rewardedVideoAd.onClose(function (res) {
                if (res && res.isEnded || res === undefined) {
                    console.log('激励视频广告完成');
                    if (self.videoCompletedHandler) {
                        console.log(" 执行激励视频广告完成回调");
                        self.videoCompletedHandler.run();
                    }
                } else {
                    console.log("播放中途退出，不下发游戏奖励");
                    if (self.videoInterruptHandler) {
                        self.videoInterruptHandler.run();
                    }
                }
                self.videoPlay = false;
                self.videoReady = false;
                self.videoErrorHandler = null;
                self.videoInterruptHandler = null;
                self.videoInterruptHandler = null;
                self.rewardedVideoAd.load();
            })
            return true;
        }

        static initGameBox(gameBoxUnitId) {
            console.log("initGameBox:", gameBoxUnitId);
            this.gameBoxUnitId = gameBoxUnitId;
 
            return true;
        }

        static isGameBoxValid() {
            return this.gameBoxAd != null;
        }

        static initGameBanner(gameBannerUnitId) {
            this.gameBannerUnitId = gameBannerUnitId;
        }

        static createGameBanner(errorHandle, config) {
            console.log("createGameBanner");
            if (typeof qg == "undefined") {
                console.warn("[adapter]Channel.initGameBanner:", "qg is undefine");
                return false;
            }
            if (!qg.createBoxBannerAd) {
                console.warn("[adapter]Channel.initGameBanner:", "createBoxBannerAd is unsupported");
                return false;
            }
            if (typeof qg.createBoxBannerAd != 'function') {
                console.warn("[adapter]Channel.initGameBanner:", "createBoxBannerAd is unsupported");
                return false;
            }

            this.gameBannerAd = qg.createBoxBannerAd({
                posId: this.gameBannerUnitId
              });
              return true;
        }

        static showGameBanner() {
            console.log("showGameBanner");
            if (this.gameBannerAd) {
                this.gameBannerAd.show().then(function() {
                    console.log('show success')
                }).catch(function(error) {
                    console.log('show fail with:' + error.errCode + ',' + error.errMsg)
                })
                return true;
            }
            else {
                return false;
            }
        }

        static hideGameBanner() {
            console.log("hideGameBanner");
            if (this.gameBannerAd) {
                this.gameBannerAd.hide().then(function() {
                    console.log('hide success')
                }).catch(function(error) {
                    console.log('hide fail with:' + error.errCode + ',' + error.errMsg)
                })
                return true;
            }
            else {
                return false;
            }
        }

        static destroyGameBanner() {
            console.log("destroyGameBanner");
            if (this.gameBannerAd) {
                gameBannerAd.destroy().then(function() {
                    console.log('destroy success')
                }).catch(function(error) {
                    console.log('destroy fail with:' + error.errCode + ',' + error.errMsg)
                });
                this.gameBannerAd = null;
                return true;
            }
            else {
                return false;
            }
        }

        static showGameBox(success, fail, close,marginTop) {
            console.log("showGameBox",marginTop);
            if (typeof qg == "undefined") {
                console.warn("[adapter]Channel.initGameBox:", "qg is undefine");
                return false;
            }
            if (!qg.createBoxPortalAd) {
                console.warn("[adapter]Channel.createBoxPortalAd:", "createBoxPortalAd is unsupported");
                return false;
            }
            if (typeof qg.createBoxPortalAd != 'function') {
                console.warn("[adapter]Channel.createBoxPortalAd:", "createBoxPortalAd is unsupported");
                return false;
            }
            this.gameBoxAd = qg.createBoxPortalAd({
                posId: this.gameBoxUnitId,
                marginTop: marginTop
            });
            
            if (!this.gameBoxAd) {
                console.warn("[adapter]Channel.showGameBox:", "gameBoxAd is undefine");
                return false;
            }

            var self = this;

            this.gameBoxAd.onError(function (err) {
                console.log("盒子九宫格广告加载失败", err)
                fail && fail.runWith(err);
            })
            this.gameBoxAd.onClose(function () {
                console.log('close')
                if (self.gameBoxAd.isDestroyed) {
                    return
                }
                // 当九宫格关闭之后，再次展示Icon
                Laya.stage.event(fc.define.Event.SHOW_NATIVEBANNER);
                self.gameBoxAd.show();
                
            })
            this.gameBoxAd.onShow(function () {
                Laya.stage.event(fc.define.Event.HIDE_NATIVEBANNER);
                console.log('show gamebox ad')
            })
            // 广告数据加载成功后展示
            this.gameBoxAd.show().then(function () {
                console.log('show success');
               
            })

            return true;
        }

        static hideGameBox(){
            if (this.gameBoxAd != null) {
                this.gameBoxAd.isDestroyed = true;
                this.gameBoxAd.destroy();
            }
        }
    
        static login(successHandler, failedHandler) {
            if (typeof qg == "undefined") {
                console.warn("[adapter]Channel.login:", "qg is undefine");
                return false;
            }

            // const sysInfo = qg.getSystemInfoSync();  
            // if (sysInfo.platformVersionCode >= 1053) {
            //     qg.login().then(function(res) {
            //         if (res.data.token) {
            //             var data = JSON.stringify(res.data);
            //             console.log(data);
            //             if (successHandler) successHandler.runWith({ identityId: res.data.token });
            //         }
            //     }, function (err) {
            //         if (failedHandler) failedHandler.runWith(JSON.stringify(err));
            //         console.log("vivo login fail", JSON.stringify(err));
            //     });
            // } else {
            //     qg.authorize({
            //         type: "code",
            //         success: function (data) {
            //             console.log(data);
            //             if (successHandler) successHandler.runWith({ identityId: data.code });
            //         },
            //         fail: function (data, code) {
            //             if (failedHandler) failedHandler.runWith(JSON.stringify(code));
            //             console.log("vivo login fail", JSON.stringify(data));
            //         }
            //     })
            // }

            var self = this;
            ZsSdk.login(function (openid) {
                if (openid) {
                    // ZsSdk.init(openid);
                    self.openid = openid;
                    if (self.loginSuccess) self.loginSuccess.runWith({ identityId: openid });
                }
                else {
                    if (self.failedHandler) self.failedHandler.runWith({ code: 2, desc: "Code不存在" });
                }
            },
                function () {
                    if (self.failedHandler) self.failedHandler.runWith({ code: 2, desc: "login platform error" });
                }
            );
            return true;
        }
    
        static loadCfg(success,fail) {
            ZsSdk.loadCfg(
                function (data) {
                    if (data) {
                        success(data);
                    }
                },
                function () {
                    fail();
                });
        }

        static loadSubpackage(pkgName, progressHandler, successHandler, failedHandler) {
            
            if (typeof qg == "undefined") {
                console.warn("[adapter]Channel.loadSubpackage:", "qg is undefine");
                return false;
            }

            const sysInfo = qg.getSystemInfoSync();    
            if (sysInfo.platformVersionCode < 1052) {
                console.warn("[adapter]Channel.loadSubpackage:", "platformVersionCode < 1052");
                return false;
            }

            console.log("---------------loadSubpackage: " + pkgName);
            // 引擎版本号大于等于1052，使用qg.loadSubpackage加载分包
            var loadTask = qg.loadSubpackage({
                name: pkgName, // name 可以填 name 或者 root
                success(res) {
                    // 分包加载成功后通过 success 回调
                    successHandler && successHandler.runWith(1);
                },
                fail(res) {
                    // 分包加载失败通过 fail 回调
                    failedHandler && failedHandler.runWith(1);
                }
            });
            if (!loadTask) {
                console.warn("[adapter]Channel.loadSubpackage:", "loadTask is invalid");
                return false;
            }
            // if (progressHandler) {
            //     loadTask.onProgressUpdate(function (res) {
            //         progressHandler.runWith(res.progress);
            //     })
            // }
            return true;
        }

        static showModal(title, content, confirmTxt, cancelTxt, success, failed, completed) {
            if (typeof qg == "undefined") {
                console.warn("[adapter]Channel.showModal:", "qg is undefine");
                return false;
            }
            qg.showDialog({
                title: title,
                message: content,
                buttons: [
                  {
                    text: confirmTxt,
                    color: '#33dd44'
                  },
                  {
                    text: cancelTxt,
                    color: '#33dd44'
                  }
                ],
                success (res) {
                    res = res || {};
                    res.cancel = res.index == 1;
                    res.confirm = res.index == 0;
                    success && success.runWith(res);
                },
                cancel () {
                    var res = {};
                    res.cancel = true;
                    success && success.runWith(res);
                }, 
                fail () {
                    failed && failed.run();
                },
                complete (data, code) {
                    completed && completed.run();
                }
              });
            return true;
        }
        
        static shareAppMessage(data, success, failed) {
            if (typeof qg == "undefined") {
                console.warn("[adapter]Channel.shareAppMessage:", "qg is undefine");
                return false;
            }
            
            const sysInfo = qg.getSystemInfoSync();    
            if (sysInfo.platformVersionCode < 1056) {
                console.warn("[adapter]Channel.shareAppMessage:", "platformVersionCode < 1056");
                return false;
            }

            qg.share({
                success() {
                    success && success.run();
                },
                fail() {
                    failed && failed.run();
                },
                cancel() {
                    failed && failed.run();
                }
            });
            return true;
        }

        static createBannerAd(errorHandler) {
            if (typeof qg == "undefined") {
                console.warn("[adapter]Channel.createBannerAd:", "qg is undefine");
                return false;
            }

            if (this.bannerAdUnitId == null || this.bannerAdUnitId == "") {
                console.warn("[adapter]Channel.createBannerAd:", "bannerAdUnitId is invalid");
                return false;
            }

            if (this.bannerAd != null) {
                console.warn("[adapter]Channel.createBannerAd:", "bannerAd is exist");
                return false;
            }

            this.bannerAdIsReady = false;
            this.bannerAdVisible = false;
            this.bannerErrorHandler = errorHandler;

            this.bannerAd = qg.createBannerAd({
                posId: this.bannerAdUnitId,
                style: {}
            });
    
            if (this.bannerAd == null) {
                console.warn("[adapter]Channel.createBannerAd:", "bannerAd is undefine");
                return false;
            }
            console.log("banner广告创建成功");
            var self = this;
            this.bannerAd.onError(function (err) {
                console.log("banner广告展示失败onError", JSON.stringify(err));
                if (self.bannerErrorHandler) self.bannerErrorHandler.runWith(err);
            });
            return true;
        }
    
        /**显示banner */
        static showBannerAd() {  
            console.log("showBannerAd");
            if (!this.bannerAd) {
                console.warn("[adapter]Channel.showBannerAd:", "bannerAd is undefine");
                return false;
            }

            if (this.bannerAdVisible) {
                return true;
            }
            var self = this;
            this.bannerAd.show().then(function() {
                if (self.bannerAdVisible == false) {
                    self.hideBannerAd();
                }
            });
    
            this.bannerAdVisible = true;
            return true;
        }
    
        /**隐藏banner */
        static hideBannerAd() {
            console.log("hideBannerAd");
            if (this.bannerAdVisible == false) {
                return true;
            }
            this.bannerAdVisible = false;
            if (!this.bannerAd) {
                console.warn("[adapter]Channel.hideBannerAd:", "bannerAd is undefine");
                return true;
            }
            this.bannerAd.hide().then(function () {
                console.log("banner广告隐藏成功");
            }).catch(function(err) {
                console.log("banner广告隐藏失败", err);
            });
            return true;
        }

        static destroyBannerAd() {
            console.log("destroyBannerAd");
            this.bannerAdIsReady = false;
            this.bannerAdVisible = false;
            if (!this.bannerAd) {
                console.warn("[adapter]Channel.destroyBannerAd:", "bannerAd is undefine");
                return false;
            }
            this.bannerAd.destroy();
            this.bannerAd = null;
            return true;
        }

        static isBannerAdValid() {
            return !!this.bannerAd;
        }
    
        /**播放视频广告 */
        static playVideo(completedHandler, interruptHandler, errorHandler) {
            // 第一次creat后广告可以在onload里面直接show
            // 后续的加载必须要load才能触发onload接着才能show出广告
            if (this.rewardedVideoAd == null) {
                console.warn("[adapter]Channel.playVideo:", "video is undefine");
                return false;
            }

            // if (this.videoLastShowTime > 0 && Date.now() - this.videoLastShowTime < this.videoInterval) {
            //     console.warn("[adapter]Channel.playVideo:", "no valid video");
            //     return false;
            // }

            this.videoErrorHandler = errorHandler;
            this.videoCompletedHandler = completedHandler;
            this.videoInterruptHandler = interruptHandler;
        
            if (this.videoReady) {
                this.rewardedVideoAd.show();
                this.videoReady = false;
            }
            else {
                this.videoPlay = true;
                this.rewardedVideoAd.load();
            }
            this.videoLastShowTime = Date.now();
            
            return true;
        }
    
        static isVideoEnable() {
            return this.rewardedVideoAd != null;
        }
    
        /**加载插屏广告 */
        static loadInsertAd(loadedHandler, errorHandler) {
            
            if (typeof qg == "undefined") {
                console.warn("[adapter]Channel.loadInsertAd:", "qg is undefine");
                return false;
            }
            var systemInfo = qg.getSystemInfoSync();
            if (systemInfo.platformVersionCode < 1031) {
                if (errorHandler) {
                    errorHandler.runWith("platformVersionCode < 1031");
                }
                console.warn("[adapter]Channel.loadInsertAd:", "platformVersionCode < 1031");
                return false;
            }
            if (this.insertAdUnitId == null) {
                console.warn("[adapter]Channel.loadInsertAd:", "insertAdUnitId is undefine");
                return false;
            }
        
            if (this.insertAd != null) {
                this.insertAd.destroy();
                this.insertAd = null;
            }
            this.insertAd = qg.createInterstitialAd({
                posId: this.insertAdUnitId,
                style: {}
            });
            if (this.insertAd == null) {
                console.warn("[adapter]Channel.loadInsertAd:", "insertAd is undefine");
                return false;
            }

            this.insertAd.onLoad(function() {
                if (loadedHandler) {
                    loadedHandler.run();
                }
            });
            var self = this;
            this.insertAd.onError(function(err) {
                console.log("插屏广告错误事件", JSON.stringify(err));
                self.insertAd.destroy();
                self.insertAd = null;
                if (errorHandler) {
                    errorHandler.runWith(err);
                }
            });
            this.insertAd.onClose(function() {
                self.insertAd.destroy();
                self.insertAd = null;
            })

            return true;

        }
    
        /**显示插屏广告 */
        static showInsertAd(closeHandler, errorHandler) {
            if (this.insertAd == null) {
                console.warn("[adapter]Channel.showInsertAd:", "insertAd is undefine");
                return false;
            }

            var self = this;
            this.insertAd.onClose(function () {
                self.insertAd = null;
                if (closeHandler) {
                    closeHandler.run();
                }
            });
            this.insertAd.onError(function(err) {
                console.log("插屏广告错误事件", JSON.stringify(err));
                if (errorHandler) {
                    errorHandler.runWith(err);
                }
            });

            this.insertAd.show().then(function () {
                console.log("插屏显示成功");
            });
            return true;
        }

        static isInsertAdEnable() {
            return true;
        }
    
        static createNativeAd(errorHandler) {

            if (typeof qg == "undefined") {
                console.warn("[adapter]Channel.initNativeAd:", "qg is undefine");
                return false;
            }

            var systemInfo = qg.getSystemInfoSync();
            if (systemInfo.platformVersionCode < 1053) {
                console.warn("[adapter]Channel.initNativeAd:", "platformVersionCode < 1053");
                return false;
            }

            if (this.nativeAdUnitId == null || this.nativeAdUnitId.length == 0) {
                console.warn("[adapter]Channel.initNativeAd:", "nativeAdUnitId is undefine");
                return false;
            }

            this.nativeAdErrorHandler = errorHandler;
            this.nativeAd = qg.createNativeAd({
                posId: this.nativeAdUnitId[this.nativeAdIndex]
            });

            this.nativeAdIndex = (this.nativeAdIndex + 1) % this.nativeAdUnitId.length;

            if (this.nativeAd == null) {
                console.warn("[adapter]Channel.initNativeAd:", "nativeAd is undefine");
                return false;
            }
    
            var self = this;
            this.nativeAd.onError(function (err){
                console.log('原生广告加载失败', JSON.stringify(err));
                if (self.nativeAdErrorHandler) self.nativeAdErrorHandler.runWith(err);
            });
            

            return true;
        }
    
        /**加载原生广告 */
        static loadNativeAd(completedHandler, errorHandler) {
            if (typeof qg == "undefined") {
                console.warn("[adapter]Channel.loadNativeAd:", "qg is undefine");
                return false;
            }

            if (this.nativeAd == null) {
                console.warn("[adapter]Channel.loadNativeAd:", "nativeAd is invalid");
                return false;
            }
    
            this.nativeAdErrorHandler = errorHandler;
            this.nativeAd.onLoad(function (res){
                console.log('原生广告加载完成', JSON.stringify(res));
                if (res && res.adList) {
                    if (completedHandler) {
                        completedHandler.runWith(res);
                    }
                }
            });
    
            this.nativeAd.load();
            // adLoad && adLoad.then(function () {
            //     console.log('原生广告加载完成', JSON.stringify(res));
            // }).catch(function(err) {
            //     if (errorHandler) errorHandler.run();
            // })
            return true;
        }
    
        /**上报原生广告显示 */
        static reportNativeAdShow(adId) {
            if (!this.nativeAd) {
                console.warn("[adapter]Channel.loadNativeAd:", "nativeAd is invalid");
                return false;
            }
            console.log("reportNativeAdShow:" + Date.now());
            this.nativeAd.reportAdShow({ adId: adId.toString() });
            return true;
        }
    
        /**上报原生广告点击 */
        static reportNativeAdClick(adId) {
            if (!this.nativeAd) {
                console.warn("[adapter]Channel.loadNativeAd:", "nativeAd is invalid");
                return false;
            }
            console.log("reportNativeAdClick:" + Date.now());
            this.nativeAd.reportAdClick({ adId: adId.toString() });
            return true;
        }
    
        //判断是否有桌面图标
        static hasDesktopIcon(successHandler) {
            if (typeof qg == "undefined") {
                console.warn("[adapter]Channel.hasDesktopIcon:", "qg is undefine");
                return false;
            }
            qg.hasShortcutInstalled({
                success: function (res) {
                    if (successHandler) successHandler.runWith({ hasIcon: res });
                },
                fail: function (err) {
                    console.log("request desktop icon error:" + JSON.stringify(err));
                }
            });
            return true;
        }
    
        //创建桌面图标
        static createDesktopIcon(successHandler) {
            if (typeof qg == "undefined") {
                console.warn("[adapter]Channel.createDesktopIcon:", "qg is undefine");
                return false;
            }
            qg.installShortcut({
                success: function () {
                    console.log("create DesktopIcon success", successHandler);
                    if (successHandler) successHandler.runWith({ hasIcon: true });
                },
                fail: function (err) {
                    console.log("create desktop icon fail", err);
                },
                complete: function () {
                    console.log("create desktop icon complete");
                }
            });
            return true;
        }
    
        static getNetworkType(complete) {
            if (typeof qg == "undefined") {
                console.warn("[adapter]Channel.getNetworkType:", "qg is undefine");
                return false;
            }
            qg.getNetworkType({
                success: function (data) {
                    console.log(`handling success: ${data.type}`)
                    if (data.type == "none") {
                        qg.showDialog({
                            title: '提示',
                            message: '当前网络不可用，请检查是否连接网络',
                            buttons: [
                                {
                                    text: '退出游戏',
                                    color: '#33dd44'
                                }
                            ],
                            success: function (data) {
                                console.log('handling callback')
                                qg.exitApplication();
                            },
                            cancel: function () {
                                qg.exitApplication();
                            },
                            fail: function (data, code) {
                                qg.exitApplication();
                            }
                        })
                    } else {
                        if(complete) complete.run();
                    }
                }
            });
            return true;
        }
    }
    Channel.rewardedVideoAd = null;
    Channel.videoPlay = false;
    Channel.videoReady = false;
    Channel.videoCompletedHandler = null;
    Channel.videoInterruptHandler = null;
    Channel.videoErrorHandler = null;
    // Channel.firstPlay = true;
    Channel.videoInterval = 10000;
    Channel.videoLastShowTime = 0;
    Channel.bannerAd = null;
    Channel.bannerAdUnitId = null;
    Channel.bannerAdVisible = false;
    Channel.bannerAdIsReady = false;
    Channel.bannerErrorHandler = null;
    Channel.insertAd = null;
    Channel.insertAdUnitId = null;
    Channel.nativeAd = null;
    Channel.nativeAdUnitId = null;
    Channel.nativeAdIndex = 0;
    Channel.nativeAdErrorHandler = null;
    Channel.nativeAdLoadedHandler = null;
    Channel.gameBoxUnitId = null;
    Channel.gameBoxAd = null;
    Channel.gameBannerUnitId = null;
    Channel.gameBannerAd = null;
    
    class Device {
    
        static get statusBarHeight() {
            if (typeof qg == "undefined") {
                console.warn("[adapter]Device.statusBarHeight:", "qg is undefine");
                return 0;
            }
            var data = qg.getNotchHeightSync();
            console.log("现在的缺口高度为：", data, data.height);
            return data ? data.height : 0;
        }
    
        static get screenWidth() {
            return this.sDeviceInfo ? this.sDeviceInfo.screenWidth : 1;
        }
    
        static get screenHeight() {
            return this.sDeviceInfo ? this.sDeviceInfo.screenHeight : 1;
        }
    
        static init(onShowHandler, onHideHandler) {
            if (typeof qg == "undefined") {
                console.warn("[adapter]Device.init:", "qg is undefine");
                return false;
            }
            this.sDeviceInfo = qg.getSystemInfoSync();
            console.log(this.sDeviceInfo);
            console.log("fcDevice.init");
            qg.onShow(function (res) {
                console.log("fcDevice.show:" + Date.now());
                if (onShowHandler) onShowHandler.runWith(res);
            });
            qg.onHide(function () {
                console.log("fcDevice.hide:" + Date.now());
                if (onHideHandler) onHideHandler.run();
            });
            return true;
        }
    
        static vibrateShort() {
            if (typeof qg == "undefined") {
                console.warn("[adapter]Device.vibrateShort:", "qg is undefine");
                return false;
            }
            qg.vibrateShort({
                fail: function () {
                    console.log("vibrateShort failed");
                },
                success: function () { },
                complete: function () { }
            });
            return true;
        }
    
        static vibrateLong() {
            if (typeof qg == "undefined") {
                console.warn("[adapter]Device.vibrateLong:", "qg is undefine");
                return false;
            }
            qg.vibrateLong({
                fail: function () {
                    console.log("vibrateLong failed");
                },
                success: function () { },
                complete: function () { }
            });
            return true;
        }
    
        static isNetValid() {
            return true;
        }
    
    }
    Device.sDeviceInfo = null;

    exports.Channel = Channel;
    exports.Device = Device;
}(window.fc.zsvivo = window.fc.zsvivo || {}, Laya));    

window.fc.adapter = window.fc.zsvivo;