window.fc = window.fc || {};
window.fc.framework = window.fc.framework || {};

(function (exports, Laya) {
    'use strict';

    var Event = fc.define.Event;
    var SceneId = fc.define.SceneId;
    var LoadingStep = fc.define.LoadingStep;

    class Md5Utils {

        static hex_md5(s) { return this.binl2hex(this.core_md5(this.str2binl(s), s.length * this.chrsz)); }
        static b64_md5(s) { return this.binl2b64(this.core_md5(this.str2binl(s), s.length * this.chrsz)); }
        static str_md5(s) { return this.binl2str(this.core_md5(this.str2binl(s), s.length * this.chrsz)); }
        static hex_hmac_md5(key, data) { return this.binl2hex(this.core_hmac_md5(key, data)); }
        static b64_hmac_md5(key, data) { return this.binl2b64(this.core_hmac_md5(key, data)); }
        static str_hmac_md5(key, data) { return this.binl2str(this.core_hmac_md5(key, data)); }
        /*
        * A JavaScript implementation of the RSA Data Security, Inc. MD5 Message
        * Digest Algorithm, as defined in RFC 1321.
        * Version 2.1 Copyright (C) Paul Johnston 1999 - 2002.
        * Other contributors: Greg Holt, Andrew Kepert, Ydnar, Lostinet
        * Distributed under the BSD License
        * See http://pajhome.org.uk/crypt/md5 for more info.
        */

        /*
        * Configurable variables. You may need to tweak these to be compatible with
        * the server-side, but the defaults work in most cases.
        */

        /*
        * These are the functions you'll usually want to call
        * They take string arguments and return either hex or base-64 encoded strings
        */
        static hex_md5(s) { return this.binl2hex(this.core_md5(this.str2binl(s), s.length * this.chrsz)); }
        static b64_md5(s) { return this.binl2b64(this.core_md5(this.str2binl(s), s.length * this.chrsz)); }
        static str_md5(s) { return this.binl2str(this.core_md5(this.str2binl(s), s.length * this.chrsz)); }
        static hex_hmac_md5(key, data) { return this.binl2hex(this.core_hmac_md5(key, data)); }
        static b64_hmac_md5(key, data) { return this.binl2b64(this.core_hmac_md5(key, data)); }
        static str_hmac_md5(key, data) { return this.binl2str(this.core_hmac_md5(key, data)); }

        /*
        * Calculate the MD5 of an array of little-endian words, and a bit length
        */
        static core_md5(x, len) {
            /* append padding */
            x[len >> 5] |= 0x80 << ((len) % 32);
            x[(((len + 64) >>> 9) << 4) + 14] = len;

            var a = 1732584193;
            var b = -271733879;
            var c = -1732584194;
            var d = 271733878;

            for (var i = 0; i < x.length; i += 16) {
                var olda = a;
                var oldb = b;
                var oldc = c;
                var oldd = d;

                a = this.md5_ff(a, b, c, d, x[i + 0], 7, -680876936);
                d = this.md5_ff(d, a, b, c, x[i + 1], 12, -389564586);
                c = this.md5_ff(c, d, a, b, x[i + 2], 17, 606105819);
                b = this.md5_ff(b, c, d, a, x[i + 3], 22, -1044525330);
                a = this.md5_ff(a, b, c, d, x[i + 4], 7, -176418897);
                d = this.md5_ff(d, a, b, c, x[i + 5], 12, 1200080426);
                c = this.md5_ff(c, d, a, b, x[i + 6], 17, -1473231341);
                b = this.md5_ff(b, c, d, a, x[i + 7], 22, -45705983);
                a = this.md5_ff(a, b, c, d, x[i + 8], 7, 1770035416);
                d = this.md5_ff(d, a, b, c, x[i + 9], 12, -1958414417);
                c = this.md5_ff(c, d, a, b, x[i + 10], 17, -42063);
                b = this.md5_ff(b, c, d, a, x[i + 11], 22, -1990404162);
                a = this.md5_ff(a, b, c, d, x[i + 12], 7, 1804603682);
                d = this.md5_ff(d, a, b, c, x[i + 13], 12, -40341101);
                c = this.md5_ff(c, d, a, b, x[i + 14], 17, -1502002290);
                b = this.md5_ff(b, c, d, a, x[i + 15], 22, 1236535329);

                a = this.md5_gg(a, b, c, d, x[i + 1], 5, -165796510);
                d = this.md5_gg(d, a, b, c, x[i + 6], 9, -1069501632);
                c = this.md5_gg(c, d, a, b, x[i + 11], 14, 643717713);
                b = this.md5_gg(b, c, d, a, x[i + 0], 20, -373897302);
                a = this.md5_gg(a, b, c, d, x[i + 5], 5, -701558691);
                d = this.md5_gg(d, a, b, c, x[i + 10], 9, 38016083);
                c = this.md5_gg(c, d, a, b, x[i + 15], 14, -660478335);
                b = this.md5_gg(b, c, d, a, x[i + 4], 20, -405537848);
                a = this.md5_gg(a, b, c, d, x[i + 9], 5, 568446438);
                d = this.md5_gg(d, a, b, c, x[i + 14], 9, -1019803690);
                c = this.md5_gg(c, d, a, b, x[i + 3], 14, -187363961);
                b = this.md5_gg(b, c, d, a, x[i + 8], 20, 1163531501);
                a = this.md5_gg(a, b, c, d, x[i + 13], 5, -1444681467);
                d = this.md5_gg(d, a, b, c, x[i + 2], 9, -51403784);
                c = this.md5_gg(c, d, a, b, x[i + 7], 14, 1735328473);
                b = this.md5_gg(b, c, d, a, x[i + 12], 20, -1926607734);

                a = this.md5_hh(a, b, c, d, x[i + 5], 4, -378558);
                d = this.md5_hh(d, a, b, c, x[i + 8], 11, -2022574463);
                c = this.md5_hh(c, d, a, b, x[i + 11], 16, 1839030562);
                b = this.md5_hh(b, c, d, a, x[i + 14], 23, -35309556);
                a = this.md5_hh(a, b, c, d, x[i + 1], 4, -1530992060);
                d = this.md5_hh(d, a, b, c, x[i + 4], 11, 1272893353);
                c = this.md5_hh(c, d, a, b, x[i + 7], 16, -155497632);
                b = this.md5_hh(b, c, d, a, x[i + 10], 23, -1094730640);
                a = this.md5_hh(a, b, c, d, x[i + 13], 4, 681279174);
                d = this.md5_hh(d, a, b, c, x[i + 0], 11, -358537222);
                c = this.md5_hh(c, d, a, b, x[i + 3], 16, -722521979);
                b = this.md5_hh(b, c, d, a, x[i + 6], 23, 76029189);
                a = this.md5_hh(a, b, c, d, x[i + 9], 4, -640364487);
                d = this.md5_hh(d, a, b, c, x[i + 12], 11, -421815835);
                c = this.md5_hh(c, d, a, b, x[i + 15], 16, 530742520);
                b = this.md5_hh(b, c, d, a, x[i + 2], 23, -995338651);

                a = this.md5_ii(a, b, c, d, x[i + 0], 6, -198630844);
                d = this.md5_ii(d, a, b, c, x[i + 7], 10, 1126891415);
                c = this.md5_ii(c, d, a, b, x[i + 14], 15, -1416354905);
                b = this.md5_ii(b, c, d, a, x[i + 5], 21, -57434055);
                a = this.md5_ii(a, b, c, d, x[i + 12], 6, 1700485571);
                d = this.md5_ii(d, a, b, c, x[i + 3], 10, -1894986606);
                c = this.md5_ii(c, d, a, b, x[i + 10], 15, -1051523);
                b = this.md5_ii(b, c, d, a, x[i + 1], 21, -2054922799);
                a = this.md5_ii(a, b, c, d, x[i + 8], 6, 1873313359);
                d = this.md5_ii(d, a, b, c, x[i + 15], 10, -30611744);
                c = this.md5_ii(c, d, a, b, x[i + 6], 15, -1560198380);
                b = this.md5_ii(b, c, d, a, x[i + 13], 21, 1309151649);
                a = this.md5_ii(a, b, c, d, x[i + 4], 6, -145523070);
                d = this.md5_ii(d, a, b, c, x[i + 11], 10, -1120210379);
                c = this.md5_ii(c, d, a, b, x[i + 2], 15, 718787259);
                b = this.md5_ii(b, c, d, a, x[i + 9], 21, -343485551);

                a = this.safe_add(a, olda);
                b = this.safe_add(b, oldb);
                c = this.safe_add(c, oldc);
                d = this.safe_add(d, oldd);
            }
            return Array(a, b, c, d);

        }

        /*
        * These functions implement the four basic operations the algorithm uses.
        */
        static md5_cmn(q, a, b, x, s, t) {
            return this.safe_add(this.bit_rol(this.safe_add(this.safe_add(a, q), this.safe_add(x, t)), s), b);
        }
        static md5_ff(a, b, c, d, x, s, t) {
            return this.md5_cmn((b & c) | ((~b) & d), a, b, x, s, t);
        }
        static md5_gg(a, b, c, d, x, s, t) {
            return this.md5_cmn((b & d) | (c & (~d)), a, b, x, s, t);
        }
        static md5_hh(a, b, c, d, x, s, t) {
            return this.md5_cmn(b ^ c ^ d, a, b, x, s, t);
        }
        static md5_ii(a, b, c, d, x, s, t) {
            return this.md5_cmn(c ^ (b | (~d)), a, b, x, s, t);
        }

        /*
        * Calculate the HMAC-MD5, of a key and some data
        */
        static core_hmac_md5(key, data) {
            var bkey = this.str2binl(key);
            if (bkey.length > 16) bkey = this.core_md5(bkey, key.length * this.chrsz);

            var ipad = Array(16), opad = Array(16);
            for (var i = 0; i < 16; i++) {
                ipad[i] = bkey[i] ^ 0x36363636;
                opad[i] = bkey[i] ^ 0x5C5C5C5C;
            }

            var hash = this.core_md5(ipad.concat(this.str2binl(data)), 512 + data.length * this.chrsz);
            return this.core_md5(opad.concat(hash), 512 + 128);
        }

        /*
        * Add integers, wrapping at 2^32. This uses 16-bit operations internally
        * to work around bugs in some JS interpreters.
        */
        static safe_add(x, y) {
            var lsw = (x & 0xFFFF) + (y & 0xFFFF);
            var msw = (x >> 16) + (y >> 16) + (lsw >> 16);
            return (msw << 16) | (lsw & 0xFFFF);
        }

        /*
        * Bitwise rotate a 32-bit number to the left.
        */
        static bit_rol(num, cnt) {
            return (num << cnt) | (num >>> (32 - cnt));
        }

        /*
        * Convert a string to an array of little-endian words
        * If chrsz is ASCII, characters >255 have their hi-byte silently ignored.
        */
        static str2binl(str) {
            var bin = Array();
            var mask = (1 << this.chrsz) - 1;
            for (var i = 0; i < str.length * this.chrsz; i += this.chrsz)
                bin[i >> 5] |= (str.charCodeAt(i / this.chrsz) & mask) << (i % 32);
            return bin;
        }

        /*
        * Convert an array of little-endian words to a string
        */
        static binl2str(bin) {
            var str = "";
            var mask = (1 << this.chrsz) - 1;
            for (var i = 0; i < bin.length * 32; i += this.chrsz)
                str += String.fromCharCode((bin[i >> 5] >>> (i % 32)) & mask);
            return str;
        }

        /*
        * Convert an array of little-endian words to a hex string.
        */
        static binl2hex(binarray) {
            var hex_tab = this.hexcase ? "0123456789ABCDEF" : "0123456789abcdef";
            var str = "";
            for (var i = 0; i < binarray.length * 4; i++) {
                str += hex_tab.charAt((binarray[i >> 2] >> ((i % 4) * 8 + 4)) & 0xF) +
                    hex_tab.charAt((binarray[i >> 2] >> ((i % 4) * 8)) & 0xF);
            }
            return str;
        }

        /*
        * Convert an array of little-endian words to a base-64 string
        */
        static binl2b64(binarray) {
            var tab = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
            var str = "";
            for (var i = 0; i < binarray.length * 4; i += 3) {
                var triplet = (((binarray[i >> 2] >> 8 * (i % 4)) & 0xFF) << 16)
                    | (((binarray[i + 1 >> 2] >> 8 * ((i + 1) % 4)) & 0xFF) << 8)
                    | ((binarray[i + 2 >> 2] >> 8 * ((i + 2) % 4)) & 0xFF);
                for (var j = 0; j < 4; j++) {
                    if (i * 8 + j * 6 > binarray.length * 32) str += this.b64pad;
                    else str += tab.charAt((triplet >> 6 * (3 - j)) & 0x3F);
                }
            }
            return str;
        }
    }
    Md5Utils.hexcase = 0;  /* hex output format. 0 - lowercase; 1 - uppercase        */
    Md5Utils.b64pad = ""; /* base-64 pad character. "=" for strict RFC compliance   */
    Md5Utils.chrsz = 8;  /* bits per input character. 8 - ASCII; 16 - Unicode      */
    Laya.ILaya.regClass(Md5Utils);
    Laya.ClassUtils.regClass("fc.framework.Md5Utils", Md5Utils);

    class MiscUtils {

        static compareVersion(v1, v2) {//比较版本
            v1 = v1.split('.');
            v2 = v2.split('.');
            var len = Math.max(v1.length, v2.length);
            while (v1.length < len) {
                v1.push('0');
            }
            while (v2.length < len) {
                v2.push('0');
            }
            for (var i = 0; i < len; i++) {
                var num1 = parseInt(v1[i]);
                var num2 = parseInt(v2[i]);
                if (num1 > num2) {
                    return 1;
                } else if (num1 < num2) {
                    return -1;
                }
            }
            return 0;
        }

        static isToday(date) {
            var now = new Date(Date.now());
            var target = new Date(date);
            if (now.getFullYear() != target.getFullYear() || now.getMonth() != target.getMonth() || now.getDate() != target.getDate()) {
                return false;
            }
            else {
                return true;
            }
        }

        /** 获取范围内的随机数 [min,max) */
        static random(min, max) {
            return Math.random() * (max - min) + min << 0;
        }

        /**是否为数字 包括字符串数字*/
        static isNumber(val) {
            var regPos = /^\d+(\.\d+)?$/; //非负浮点数
            var regNeg = /^(-(([0-9]+\.[0-9]*[1-9][0-9]*)|([0-9]*[1-9][0-9]*\.[0-9]+)|([0-9]*[1-9][0-9]*)))$/; //负浮点数
            if (regPos.test(val) || regNeg.test(val)) {
                return true;
            } else {
                return false;
            }
        }

        static number2String(val, length) {
            var str = val.toString();
            if (str.length == length) {
                return;
            }
            if (str.length > length) {
                return str.substring(str.length - length);
            }
            var zero = "";
            for (let index = length - str.length; index > 0; index--) {
                zero = zero.concat("0");
            }
            str = zero.concat(str);
            return str;
        }

        static array2UrlStrArgs(arr) {
            if (!arr || arr.length == 0) {
                return "";
            }
            if (arr.length == 1) {
                return arr[0];
            }
            var params = arr[0] + "=" + arr[1];
            for (var index = 2; index + 1 < arr.length; index += 2) {
                params += "&" + arr[index] + "=" + arr[index + 1];
            }
            return params;
        }

        static object2UrlStrArgs(obj) {
            if (obj) {
                var params = "";
                for (var key in obj) {
                    if (obj.hasOwnProperty(key)) {
                        params += key + "=" + obj[key] + "&";
                    }
                }
                return params.substr(0, params.length - 1);
            }
            else {
                return "";
            }
        }

        static isEmptyString(str) {
            return !str || str.length == 0;
        }
    }
    Laya.ILaya.regClass(MiscUtils);
    Laya.ClassUtils.regClass("fc.framework.MiscUtils", MiscUtils);

    class Channel {

        static get Instance() {
            if (this.sInitialized == false) {
                this.sInitialized = true;
                this.sInstance = fc.adapter.Channel;
            }
            return this.sInstance;
        }

        static get bannerHeight() {
            if (this.sInstance && this.sInstance.bannerHeight) {
                return this.sInstance.bannerHeight;
            }
            else {
                return this.sBannerHeight;
            }
        }

        static init() {
            return this.callFunc("init");
        }

        static initAdService(info) {
            this.sShareDesc = info.shareDesc ? info.shareDesc : "";
            this.sShareTitle = info.shareTitle ? info.shareTitle : "";
            this.sShareImgUrl = info.shareImgUrl ? info.shareImgUrl : "";
            return this.callFunc("initAdService", null, [info]);
        }

        static pushGameData(json) {
            return this.callFunc("pushGameData", null, [json]);
        }

        static login(successHandler, failedHandler) {
            return this.callFunc("login", failedHandler, [successHandler, failedHandler]);
        }

        static loadSubpackage(pkgName, progressHandler, successHandler, failedHandler) {
            return this.callFunc("loadSubpackage", successHandler, [pkgName, progressHandler, successHandler, failedHandler]);
        }

        static navigate2Mini(appId, path, extraData, success, failed, completed) {
            return this.callFunc("navigate2Mini", failed, [appId, path, extraData, success, failed, completed]);
        }

        static showModal(title, content, confirmTxt, cancelTxt, success, failed, completed) {
            return this.callFunc("showModal", failed, [title, content, confirmTxt, cancelTxt, success, failed, completed]);
        }

        static shareAppMessage(data, success, failed) {
            var info = data || {};
            info.title = info.title || this.sShareTitle;
            info.imageUrl = info.imageUrl || this.sShareImgUrl;
            info.desc = info.desc || this.sShareDesc;
            return this.callFunc("shareAppMessage", failed, [info, success, failed]);
        }

        static isShareEnable() {
            return this.Instance && this.Instance.shareAppMessage;
        }

        static createGameBanner(errorHandler, config) {
            return this.callFunc("createGameBanner", errorHandler, [errorHandler, config]);
        }

        static showGameBanner() {
            return this.callFunc("showGameBanner");
        }

        static hideGameBanner() {
            return this.callFunc("hideGameBanner");
        }

        static destroyGameBanner() {
            return this.callFunc("destroyGameBanner");
        }

        static createBannerAd(errorHandler) {
            return this.callFunc("createBannerAd", null, [errorHandler]);
        }

        static showBannerAd() {
            return this.callFunc("showBannerAd");
        }

        static hideBannerAd() {
            return this.callFunc("hideBannerAd");
        }

        static destroyBannerAd() {
            return this.callFunc("destroyBannerAd");
        }

        static isBannerAdValid() {
            return this.callFunc("isBannerAdValid");
        }

        static createNativeAd(errorHandler) {
            if (GameFrame.isGMMode) {
                return true;
            }
            return this.callFunc("createNativeAd", null, [errorHandler]);
        }

        static loadNativeAd(successHandler, errorHandler) {
            if (GameFrame.isGMMode) {
                var data = { "adList": [{ "adId": "8b92d67f-fee3-450a-96fe-81648ae42ecb", "clickBtnTxt": "点击查看", "creativeType": 6, "desc": "渠道请勿使用", "iconUrlList": [], "icon": "", "interactionType": 1, "logoUrl": "", "title": "拼多多", "imgUrlList": ["http://images.pinduoduo.com/marketing_api/2020-05-17/5243f066-6924-4a2b-b8d4-d23f6a4edda7.jpg"] }], "code": 0, "msg": "ok" };
                successHandler.runWith(data);
                return true;
            }
            return this.callFunc("loadNativeAd", errorHandler, [successHandler, errorHandler]);
        }

        static destroyNativeAd() {
            return this.callFunc("destroyNativeAd");
        }

        static createCustomAd(adUnitId,errorHandler, x, y,type) {
            if (GameFrame.isGMMode) {
                return true;
            }
            return this.callFunc("createCustomAd", null, [adUnitId,errorHandler, x, y,type]);
        }
        
        static loadCustomAd(successHandler, errorHandler) {
            if (GameFrame.isGMMode) {
                return true;
            }
            return this.callFunc("loadCustomAd", errorHandler, [successHandler, errorHandler]);
        }
        
        static destroyCustomAd() {
            return this.callFunc("destroyCustomAd");
        }
        
        static reportNativeAdShow(adId) {
            return this.callFunc("reportNativeAdShow", null, [adId]);
        }

        static reportNativeAdClick(adId) {
            return this.callFunc("reportNativeAdClick", null, [adId]);
        }

        static playVideo(completedHandler, interuptHandler, errorHandler) {
            if (GameFrame.isGMMode) {
                Laya.stage.event(this.EVENT_AD_VIDEO_CLOSED);
                completedHandler && completedHandler.run();
                return true;
            }
            if (this.Instance && this.Instance.playVideo) {
                Laya.stage.event(this.EVENT_AD_VIDEO_PLAY);
                var ret = this.Instance.playVideo(
                    Laya.Handler.create(this, function () { Laya.stage.event(this.EVENT_AD_VIDEO_CLOSED); completedHandler && completedHandler.run() }, null, false),
                    Laya.Handler.create(this, function () { Laya.stage.event(this.EVENT_AD_VIDEO_CLOSED); interuptHandler && interuptHandler.run() }, null, false),
                    Laya.Handler.create(this, function (error) { Laya.stage.event(this.EVENT_AD_VIDEO_CLOSED); errorHandler && errorHandler.runWith(error) }, null, false)
                );
                if (ret == false && errorHandler) {
                    Laya.stage.frameOnce(1, this, function (errorHandler) {
                        errorHandler.run();
                    }, [errorHandler])
                }
            }
            else {
                Laya.stage.event(this.EVENT_AD_VIDEO_PLAY);
                if (completedHandler) {
                    Laya.stage.frameOnce(1, this, function (completedHandler) {
                        Laya.stage.event(this.EVENT_AD_VIDEO_CLOSED);
                        completedHandler.run();
                    }, [completedHandler])
                }
                console.warn("Channel", "playVideo is not exist");
                return true;
            }
        }

        static isVideoEnable() {
            if (GameFrame.isGMMode) {
                return true;
            }
            return this.callFunc("isVideoEnable");
        }

        static loadInsertAd(loadedHandler, errorHandler) {
            return this.callFunc("loadInsertAd", errorHandler, [loadedHandler, errorHandler]);
        }

        static showInsertAd(closeHandler, errorHandler) {
            return this.callFunc("showInsertAd", errorHandler, [closeHandler, errorHandler]);
        }

        static isInsertAdEnable() {
            return this.callFunc("isInsertAdEnable");
        }

        static loadFullScreenAd(loadedHandler, errorHandler) {
            return this.callFunc("loadFullScreenAd", loadedHandler, [loadedHandler, errorHandler]);
        }

        static showFullScreenAd(closeHandler, errorHandler) {
            return this.callFunc("showFullScreenAd", errorHandler, [closeHandler, errorHandler]);
        }

        static isFullScreenAdEnable() {
            return this.callFunc("isFullScreenAdEnable");
        }

        static isRecorderEnable() {
            return this.Instance && this.Instance.startRecorder;
        }

        static startRecorder(iMaxTime) {
            return this.callFunc("startRecorder", null, [iMaxTime]);
        }

        static stopRecorder() {
            return this.callFunc("stopRecorder");
        }

        static pauseRecorder() {
            return this.callFunc("pauseRecorder");
        }

        static resumeRecorder() {
            return this.callFunc("resumeRecorder");
        }

        static canShareRecorder() {
            return this.callFunc("canShareRecorder");
        }

        static isGameBoxValid() {
            return this.callFunc("isGameBoxValid");
        }

        static showGameBox(success, fail, close, navSuccess, navFailed) {
            return this.callFunc("showGameBox", fail, [success, fail, close, navSuccess, navFailed]);
        }

        static hideGameBox() {
            return this.callFunc("hideGameBox");
        }

        static hasDesktopIcon(successHandler) {
            return this.callFunc("hasDesktopIcon", null, [successHandler]);
        }

        static createDesktopIcon(successHandler) {
            return this.callFunc("createDesktopIcon", null, [successHandler]);
        }

        static reportMonitor(name, value) {
            return this.callFunc("reportMonitor", null, [name, value]);
        }

        static loadAd(callback) {
            return this.callFunc("loadAd", null, [callback]);
        }
        static loadCfg(success) {
            return this.callFunc("loadCfg", null, [success]);
        }

        static pay(callback) {
            return this.callFunc("pay", null, [callback]);
        }

        static callFunc(funcName, onFailedCb = null, funcArgs = null) {
            if (this.Instance && this.Instance[funcName]) {
                var method = this.Instance[funcName]
                var result = method.apply(this.Instance, funcArgs);
                if (!result && onFailedCb) {
                    Laya.stage.frameOnce(1, this, function (onFailedCb) {
                        onFailedCb.run();
                    }, [onFailedCb]);
                }
                return result;
            }
            else {
                if (onFailedCb) {
                    Laya.stage.frameOnce(1, this, function (onFailedCb) {
                        onFailedCb.run();
                    }, [onFailedCb]);
                }
                console.warn("Channel", funcName, " is not exist");
                return false;
            }
        }
    }
    Channel.sInstance = null;
    Channel.sInitialized = false;
    Channel.sShareImgUrl = null;
    Channel.sShareTitle = "";
    Channel.sShareDesc = "";
    Channel.sBannerHeight = 228;
    Channel.EVENT_AD_VIDEO_PLAY = Event.AD_VIDEO_PLAY;
    Channel.EVENT_AD_VIDEO_CLOSED = Event.AD_VIDEO_CLOSED;
    Laya.ILaya.regClass(Channel);
    Laya.ClassUtils.regClass("fc.framework.Channel", Channel);

    class Joystick {

        static get Instance() {
            if (this.sInitialized == false) {
                this.sInitialized = true;
                this.sInstance = fc.adapter.Joystick;
            }
            return this.sInstance;
        }

        static init() {
            var onKeyPressHandle = Laya.Handler.create(this, function (code) { Laya.stage.event(Event.JOYSTICK_KEY_PRESS, code) }, null, false);
            var onKeyDownHandle = Laya.Handler.create(this, function (code) { Laya.stage.event(Event.JOYSTICK_KEY_DOWN, code) }, null, false);
            var onKeyUpHandle = Laya.Handler.create(this, function (code, isClick) { Laya.stage.event(Event.JOYSTICK_KEY_UP, [code, isClick]) }, null, false);
            return this.callFunc("init", [onKeyPressHandle, onKeyDownHandle, onKeyUpHandle]);
        }

        static callFunc(funcName, funcArgs = null) {
            if (this.Instance && this.Instance[funcName]) {
                var method = this.Instance[funcName]
                var result = method.apply(this.Instance, funcArgs);
                return result;
            }
            else {
                console.warn("Joystick", funcName, " is not exist");
                return false;
            }
        }
    }
    Joystick.sInstance = null;
    Joystick.sInitialized = false;

    class Device {

        static get Instance() {
            if (this.sInitialized == false) {
                this.sInitialized = true;
                this.sInstance = fc.adapter.Device;
            }
            return this.sInstance;
        }

        static init(onAppResume, onAppPause) {
            if (this.Instance && this.Instance.init) {
                this.Instance.init(onAppResume, onAppPause);
            }
        }

        static get statusBarHeight() {
            if (this.Instance && this.Instance.statusBarHeight) {
                return this.Instance.statusBarHeight;
            }
            else {
                return 0;
            }
        }

        static get screenWidth() {
            if (this.Instance && this.Instance.screenWidth) {
                return this.Instance.screenWidth;
            }
            else {
                return Laya.stage.width;
            }
        }

        static get screenHeight() {
            if (this.Instance && this.Instance.screenHeight) {
                return this.Instance.screenHeight;
            }
            else {
                return Laya.stage.height;
            }
        }

        static get soundVolume() {
            if (this.Instance && this.Instance.soundVolume) {
                return this.Instance.soundVolume;
            }
            else {
                return Laya.SoundManager.soundVolume;
            }
        }

        static set soundVolume(val) {
            if (this.Instance && this.Instance.soundVolume) {
                this.Instance.soundVolume = val;
            }
            else {
                Laya.SoundManager.soundVolume = val;
            }
        }

        static get soundMuted() {
            if (this.Instance && this.Instance.soundMuted) {
                return this.Instance.soundMuted;
            }
            else {
                return Laya.SoundManager.soundMuted;
            }
        }

        static set soundMuted(val) {
            if (this.Instance && this.Instance.soundMuted) {
                this.Instance.soundMuted = val;
            }
            else {
                Laya.SoundManager.soundMuted = val;
            }
        }

        static get musicVolume() {
            if (this.Instance && this.Instance.musicVolume) {
                return this.Instance.musicVolume;
            }
            else {
                return Laya.SoundManager.musicVolume;
            }
        }

        static set musicVolume(val) {
            if (this.Instance && this.Instance.musicVolume) {
                this.Instance.musicVolume = val;
            }
            else {
                Laya.SoundManager.musicVolume = val;
            }
        }

        static get musicMuted() {
            if (this.Instance && this.Instance.musicMuted) {
                return this.Instance.musicMuted;
            }
            else {
                return Laya.SoundManager.musicMuted;
            }
        }

        static set musicMuted(val) {
            if (this.Instance && this.Instance.musicMuted) {
                this.Instance.musicMuted = val;
            }
            else {
                Laya.SoundManager.musicMuted = val;
            }
        }

        static playSound(res, loop, compHandler) {
            if (this.Instance && this.Instance.playSound) {
                this.Instance.playSound(res, loop, compHandler);
            }
            else {
                Laya.SoundManager.playSound(res, loop ? 0 : 1, compHandler);
            }
        }

        static stopSound(res) {
            if (this.Instance && this.Instance.stopSound) {
                this.Instance.stopSound(res);
            }
            else {
                Laya.SoundManager.stopSound(res);
            }
        }

        static playMusic(res) {
            if (this.Instance && this.Instance.playMusic) {
                this.Instance.playMusic(res);
            }
            else {
                Laya.SoundManager.playMusic(res, 0);
            }
        }

        static stopMusic(res) {
            if (res == null) {
                return;
            }
            if (this.Instance && this.Instance.stopMusic) {
                this.Instance.stopMusic(res);
            }
            else {
                Laya.SoundManager.stopMusic(res);
            }
        }

        static stopAll() {
            this.stopMusic();
            this.stopAllSound();
        }

        static stopAllSound() {
            if (this.Instance && this.Instance.stopAllSound) {
                this.Instance.stopAllSound();
            }
            else {
                Laya.SoundManager.stopAllSound();
            }
        }

        static vibrateShort() {
            if (Date.now() - this.sShortVibrateTime < 100) {
                return ;
            }
            this.sShortVibrateTime = Date.now();
            if (this.Instance && this.Instance.vibrateShort) {
                this.Instance.vibrateShort();
            }
            else if (typeof navigator !== "undefined" && "vibrate" in navigator) {
                navigator.vibrate(500);
            }
            else {
                console.log("vibrateShort");
                console.warn("Device", "vibrateShort is not exist");
            }
        }

        static vibrateLong() {
            if (this.Instance && this.Instance.vibrateLong) {
                this.Instance.vibrateLong();
            }
            else if (typeof navigator !== "undefined" && "vibrate" in navigator) {
                navigator.vibrate(1000);
            }
            else {
                console.log("vibrateLong");
                console.warn("Device", "vibrateLong is not exist");
            }
        }

        static isNetValid() {
            if (this.Instance && this.Instance.isNetValid) {
                return this.Instance.isNetValid();
            }
            return navigator.onLine;
        }
    }
    Device.sShortVibrateTime = 0;
    Device.sInstance = null;
    Device.sInitialized = false;
    Laya.ILaya.regClass(Device);
    Laya.ClassUtils.regClass("fc.framework.Device", Device);

    class Audio {

        static get IsBgmPlaying() {
            return this.sIsBgmPlaying;
        }
        static get soundVolume() {
            return Device.soundVolume;
        }

        static set soundVolume(val) {
            Device.soundVolume = val;
        }

        static get soundMuted() {
            return Device.soundMuted;
        }

        static set soundMuted(val) {
            Device.soundMuted = val;
        }

        static get musicVolume() {
            return Device.musicVolume;
        }

        static set musicVolume(val) {
            Device.musicVolume = val;
        }

        static get musicMuted() {
            return Device.musicMuted;
        }

        static set musicMuted(val) {
            Device.musicMuted = val;
        }

        static playMusic(music) {
            if (music == null) {
                return;
            }
            if (this.sCurMusicUrl != music || this.sIsBgmPlaying == false) {
                if (this.sCurMusicUrl != music && this.sCurMusicUrl != null) {
                    Device.stopMusic(this.sCurMusicUrl);
                }
                this.sCurMusicUrl = music;
                this.sIsBgmPlaying = true;
                Device.playMusic(this.sCurMusicUrl);
            }
        }

        static stopMusic() {
            Device.stopMusic(this.sCurMusicUrl);
            this.sIsBgmPlaying = false;
        }

        static playSound(soundId, isLoop) {
            if (isLoop && this.sLoopSounds.indexOf(soundId) != -1) {
                return;
            }
            if (isLoop) {
                this.sLoopSounds.push(soundId);
                Device.playSound(soundId, false, Laya.Handler.create(this, this.onSoundCompleted, [soundId]));
            }
            else {
                Device.playSound(soundId, false, null);
            }
        }

        static stopSound(soundId) {
            var soundIdx = this.sLoopSounds.indexOf(soundId);
            if (soundIdx != -1) {
                this.sLoopSounds.splice(soundIdx, 1);
            }
            Device.stopSound(soundId);
        }

        static onSoundCompleted(soundId) {
            if (this.sLoopSounds.indexOf(soundId) != -1) {
                Device.playSound(soundId, 1, Laya.Handler.create(this, this.onSoundCompleted, [soundId]));
            }
        }

        static stopAll() {
            this.stopMusic();
            this.stopAllSound();
        }

        static stopAllSound() {
            Device.stopAllSound();
            this.sLoopSounds.splice(0);
        }
    }
    Audio.sCurMusicUrl = null;
    Audio.sIsBgmPlaying = false;
    Audio.sLoopSounds = [];
    Laya.ILaya.regClass(Audio);
    Laya.ClassUtils.regClass("fc.framework.Audio", Audio);

    class LoaderAsyn {
        static async loadSubpackageAsyn(pkgName, onProgress, onSuccess, onFailed) {
            return new Promise(function (resolve, reject) {
                Channel.loadSubpackage(
                    pkgName,
                    onProgress,
                    Laya.Handler.create(null, function () { resolve(pkgName); }),
                    Laya.Handler.create(null, function () { reject(pkgName); })
                );
            }).then(function (pkgName) {
                onSuccess && onSuccess.runWith(pkgName);
            }, function (pkgName) {
                onFailed && onFailed.runWith(pkgName);
            });
        }

        static async loadFontAsyn(font, fontUrl, onCompleted) {
            return new Promise(function (resolve, reject) {
                font.loadFont(fontUrl, Laya.Handler.create(null, function () { resolve(); }));
            }).then(function () {
                onCompleted && onCompleted.run();
            })
        }

        static async loadResAsyn(resPath, onCompleted, onProgress) {
            return new Promise(function (resolve, reject) {
                Laya.loader.load(resPath, Laya.Handler.create(null, function (res) { resolve(res); }), onProgress);
            }).then(function (res) {
                onCompleted && onCompleted.runWith(res);
            });
        }

        static async createResAsyn(resPath, onCompleted, onProgress) {
            return new Promise(function (resolve, reject) {
                Laya.loader.create(resPath, Laya.Handler.create(null, function (res) { resolve(res); }), onProgress);
            }).then(function (res) {
                onCompleted && onCompleted.runWith(res);
            });
        }
    }

    class LoaderOld {
        static loadSubpackageAsyn(pkgName, onProgress, onSuccess, onFailed) {
            Channel.loadSubpackage(
                pkgName,
                onProgress,
                onSuccess,
                onFailed);
        }

        static loadFontAsyn(font, fontUrl, onCompleted) {
            font && font.loadFont(fontUrl, onCompleted);
        }

        static loadResAsyn(resPath, onCompleted, onProgress) {
            Laya.loader.load(resPath, onCompleted, onProgress);
        }

        static createResAsyn(resPath, onCompleted, onProgress) {
            Laya.loader.create(resPath, onCompleted, onProgress);
        }
    }
    var Loader = LoaderOld;

    class Resource {

        static registerSubPkg(pkgName, pkgPath) {
            this.subPkgMap[pkgName] = pkgPath;
        }

        static get3dResUrl(filename, subPkg) {
            if (!subPkg || !this.subPkgMap[subPkg]) {
                return this.threeDResRootPath + filename;
            }
            return this.subPkgMap[subPkg] + filename;
        }

        static get3dPrefabUrl(filename, subPkg) {
            if (!subPkg || !this.subPkgMap[subPkg]) {
                return this.threeDResRootPath + filename + ".lh";
            }
            return this.subPkgMap[subPkg] + filename + ".lh";
        }

        static get3dSceneUrl(filename, subPkg) {
            if (!subPkg || !this.subPkgMap[subPkg]) {
                return this.threeDResRootPath + filename + ".ls";
            }
            return this.subPkgMap[subPkg] + filename + ".ls";
        }

        static get curScene3dUrl() {
            return this.curScene3d ? this.curScene3d["fc_res_url"] : null;
        }

        static loadSubpackage(subpkg, onCompleted, onProgress) {
            Loader.loadSubpackageAsyn(subpkg, onProgress, onCompleted, Laya.Handler.create(this, function () {
                console.error("subPkgError:", subpkg);
                onCompleted && onCompleted.run();
            }));
        }

        static loadFont(fontUrl, onCompleted, onProgress) {
            var font = new Laya.BitmapFont();
            if (fontUrl instanceof Array) {
                Loader.loadFontAsyn(font, `font/${fontUrl[0]}.fnt`, Laya.Handler.create(this, this.onFontReady, [font, fontUrl[0], fontUrl, onCompleted, onProgress]));
            }
            else {
                Loader.loadFontAsyn(font, `font/${fontUrl}.fnt`, Laya.Handler.create(this, this.onFontReady, [font, fontUrl, [], onCompleted, onProgress]));
            }
        }

        static onFontReady(font, fontUrl, fontList, onCompleted, onProgress) {
            Laya.Text.registerBitmapFont(fontUrl, font);
            if (fontList.length == 0) {
                if (onCompleted) {
                    onCompleted.run();
                }
                if (onProgress) {
                    onProgress.runWith(1);
                }
            }
            else {
                var index = fontList.indexOf(fontUrl) + 1;
                if (index == fontList.length) {
                    if (onCompleted) {
                        onCompleted.run();
                    }
                    if (onProgress) {
                        onProgress.runWith(1);
                    }
                }
                else {
                    Loader.loadFontAsyn(font, `font/${fontUrl[index]}.fnt`, Laya.Handler.create(this, this.onFontReady, [font, fontUrl[index], fontUrl, onCompleted, onProgress]));//font.loadFont(`font/${fontList[index]}.fnt`, Laya.Handler.create(this, this.onFontReady, [font, fontList[index], fontList, onCompleted, onProgress]));
                    if (onProgress) {
                        onProgress.runWith(index / fontList.length);
                    }
                }
            }
        }

        static loadRes(resPath) {
            return Laya.loader.getRes(resPath);
        }

        static loadResAsyn(resPath, onCompleted, onProgress, pkgName) {
            if (!pkgName || this.loadedPkg.indexOf(pkgName) >= 0) {
                //Laya.loader.load(resPath, onCompleted, onProgress);
                Loader.loadResAsyn(resPath, onCompleted, onProgress);
                return;
            }

            if (this.loadingPkg[pkgName] == null) {
                this.loadingPkg[pkgName] = [{ cb: onCompleted, step: onProgress, res: resPath, type: "load" }];
                Loader.loadSubpackageAsyn(pkgName, null, Laya.Handler.create(this, this.onSubPkgReady, [pkgName]), Laya.Handler.create(this, this.onSubPkgError, [pkgName]));
                return;
            }

            this.loadingPkg[pkgName].push({ cb: onCompleted, step: onProgress, res: resPath });
        }

        static load3dRes(resPath) {
            return Laya.loader.getRes(resPath);
        }

        static load3dResAsyn(resPath, onCompleted, onProgress, pkgName) {
            if (!pkgName || this.loadedPkg.indexOf(pkgName) >= 0) {
                //Laya.loader.create(resPath, onCompleted, onProgress);
                Loader.createResAsyn(resPath, onCompleted, onProgress);
                return;
            }

            if (this.loadingPkg[pkgName] == null) {
                this.loadingPkg[pkgName] = [{ cb: onCompleted, step: onProgress, res: resPath, type: "create" }];
                Loader.loadSubpackageAsyn(pkgName, null, Laya.Handler.create(this, this.onSubPkgReady, [pkgName]), Laya.Handler.create(this, this.onSubPkgError, [pkgName]));
                return;
            }

            this.loadingPkg[pkgName].push({ cb: onCompleted, step: onProgress, res: resPath });
        }

        static loadSprite3d(resPath, pkgName) {
            return this.load3dRes(this.get3dPrefabUrl(resPath, pkgName));
        }

        static loadSprite3dAsyn(resPath, onCompleted, onProgress, pkgName) {
            if (resPath instanceof Array) {
                var spriteUrls = [];
                for (let index = 0; index < resPath.length; index++) {
                    spriteUrls.push(this.get3dPrefabUrl(resPath[index], pkgName));
                }
                this.load3dResAsyn(spriteUrls, onCompleted, onProgress, pkgName);
            }
            else {
                var spriteUrl = this.get3dPrefabUrl(resPath, pkgName);
                this.load3dResAsyn(spriteUrl, onCompleted, onProgress, pkgName);
            }
        }

        static loadScene3d(resPath, pkgName) {
            var scene3d = this.curScene3d;
            this.curScene3d = null;
            var lastSceneUrl = scene3d != null ? scene3d["fc_scene_url"] : null;
            if (scene3d) {
                scene3d.removeSelf();
                scene3d.destroy();
                scene3d = null;
            }
            this.curScene3d = Laya.loader.getRes(this.get3dSceneUrl(resPath, pkgName));
            if (this.curScene3d) {
                this.curScene3d["fc_scene_url"] = resPath;
            }
            if (lastSceneUrl) {
                this.unloadSceneRes(lastSceneUrl);
            }
            return this.curScene3d;
        }

        static loadScene3dAsyn(resPath, onCompleted, onProgress, pkgName) {
            if (!pkgName || this.loadedPkg.indexOf(pkgName) >= 0) {
                this.initScene3d(resPath, onCompleted, onProgress, pkgName);
                return;
            }
            Loader.loadSubpackageAsyn(pkgName, null, Laya.Handler.create(this, function () {
                this.initScene3d(resPath, onCompleted, onProgress, pkgName);
            }), Laya.Handler.create(this, function () {
                console.error("subPkgError:", pkgName);
                console.error("loadSceneError:", resPath);
            }));
            return null;
        }

        static initScene3d(resPath, onCompleted, onProgress, pkgName) {
            if (this.curDownloadSceneUrl != resPath) {
                Laya.loader.cancelLoadByUrl(this.curDownloadSceneUrl);
            }
            this.curDownloadSceneUrl = null;
            var scene3d = this.curScene3d;
            this.curScene3d = null;
            var lastSceneUrl = scene3d != null ? scene3d["fc_scene_url"] : null;
            if (scene3d) {
                scene3d.removeSelf();
                scene3d.destroy();
                scene3d = null;
                Laya.loader.clearRes(lastSceneUrl);
            }
            var sceneUrl = this.get3dSceneUrl(resPath, pkgName);
            this.curScene3d = Laya.loader.getRes(sceneUrl);
            if (this.curScene3d) {
                this.curScene3d["fc_scene_url"] = resPath;
                if (lastSceneUrl) {
                    this.unloadSceneRes(lastSceneUrl);
                }
                if (onProgress) {
                    onProgress.runWith(1);
                }
                if (onCompleted) {
                    onCompleted.runWith(this.curScene3d);
                }
                return this.curScene3d;
            }

            this.curDownloadSceneUrl = sceneUrl;
            Loader.createResAsyn(sceneUrl, Laya.Handler.create(null, function (s) {
                Resource.curDownloadSceneUrl = null;
                Resource.curScene3d = s;
                if (Resource.curScene3d) {
                    Resource.curScene3d["fc_scene_url"] = resPath;
                }
                if (lastSceneUrl) {
                    Resource.unloadSceneRes(lastSceneUrl);
                }
                if (onCompleted) {
                    onCompleted.runWith(Resource.curScene3d);
                }
            }), onProgress);
        }

        static unloadSceneRes(sceneUrl, subPkg) {
            Laya.loader.load(this.get3dResUrl(sceneUrl + ".json", subPkg), Laya.Handler.create(this, this.destroyScene3dRes, [sceneUrl + ".json", subPkg]));
        }

        static printSceneDependences(sceneUrl, subPkg) {
            Laya.loader.load(this.get3dResUrl(sceneUrl + ".json", subPkg), Laya.Handler.create(this, this.printScene3dRes, [sceneUrl + ".json", subPkg]));
        }

        static printScene3dRes(resListUrl, subPkg) {
            var arr = Laya.loader.getRes(this.get3dResUrl(resListUrl, subPkg));
            if (arr == null) {
                return;
            }
            var log = {};
            for (var i = arr.length - 1; i > -1; i--) {
                //根据资源路径获取资源
                var resUrl = this.get3dResUrl(arr[i], subPkg)
                var resource = Laya.loader.getRes(resUrl);
                if (resource == null) {
                    //console.log("invalid resUrl:" + resUrl);
                }
                else {
                    log[resUrl] = resource.referenceCount;
                }
            }
            //console.log(JSON.stringify(log));
        }

        static destroyScene3dRes(resListUrl, subPkg) {
            var arr = Laya.loader.getRes(this.get3dResUrl(resListUrl, subPkg));
            if (arr == null) {
                return;
            }
            for (var i = arr.length - 1; i > -1; i--) {
                //根据资源路径获取资源
                var resUrl = this.get3dResUrl(arr[i], subPkg)
                var resource = Laya.loader.getRes(resUrl);
                if (resource == null) {
                    //console.log("invalid resUrl:" + resUrl);
                }
                else if (resource.referenceCount > 0) {
                    //console.log("using resUrl:" + resUrl + ",refCount:" + resource.referenceCount);
                }
                else {
                    //资源释放
                    resource.destroy();
                    Laya.loader.clearRes(resUrl);
                    //console.log("release resUrl:" + resUrl);
                }
            }
        }

        static onSubPkgReady(pkgName) {
            console.log("onSubPkgReady:", pkgName);
            this.loadedPkg.push(pkgName);
            var cbArray = this.loadingPkg[pkgName];
            this.loadingPkg[pkgName] = null;
            if (cbArray == null) {
                return;
            }
            for (let index = 0; index < cbArray.length; index++) {
                const element = cbArray[index];
                if (element.type == "load") {
                    //Laya.loader.load(element.res, element.cb, element.step);
                    Loader.loadResAsyn(element.res, element.cb, element.step);
                }
                else {
                    //Laya.loader.create(element.res, element.cb, element.step);
                    Loader.createResAsyn(element.res, element.cb, element.step);
                }
            }
        }

        static onSubPkgError(pkgName) {
            console.error("subPkgError:", pkgName);
            this.loadedPkg.push(pkgName);
            var cbArray = this.loadingPkg[pkgName];
            this.loadingPkg[pkgName] = null;
            if (cbArray == null) {
                return;
            }
            for (let index = 0; index < cbArray.length; index++) {
                const element = cbArray[index];
                if (element.type == "load") {
                    //Laya.loader.load(element.res, element.cb, element.step);
                    Loader.loadResAsyn(element.res, element.cb, element.step);
                }
                else {
                    //Laya.loader.create(element.res, element.cb, element.step);
                    Loader.createResAsyn(element.res, element.cb, element.step);
                }
            }
        }
    }
    Resource.curScene3d = null;
    Resource.curDownloadSceneUrl = null;
    Resource.threeDResRootPath = "3dres/Conventional/";
    Resource.loadedPkg = [];
    Resource.loadingPkg = {};
    Resource.subPkgMap = {};
    Laya.ILaya.regClass(Resource);
    Laya.ClassUtils.regClass("fc.framework.Resource", Resource);

    class WebRequest {

        static getArgsSign(agrs, secret) {
            var sortedKeys = Object.keys(agrs).sort();
            var signParam = '';
            for (var i = 0; i < sortedKeys.length; i++) {
                signParam += sortedKeys[i] + "=" + agrs[sortedKeys[i]];
            }
            if (secret) {
                signParam += secret;
            }
            var md5sign = Md5Utils.hex_md5(signParam);
            return md5sign.toLowerCase();
        }

        static xhrPost(api, args) {
            if (this.sXHRList[api] != null) {
                return;
            }
            this.sXHRList[api] = Date.now();

            var params = MiscUtils.object2UrlStrArgs(args);
            var xhr = new Laya.HttpRequest();
            xhr.http.timeout = 10000;//设置超时时间；
            xhr.once(Laya.Event.ERROR, this, this.onXHRError, [api, args]);
            xhr.once(Laya.Event.COMPLETE, this, this.onXHRCompleted, [api, args]);
            xhr.send(api, params, "post", "text");
        }

        static xhrPostWithSign(api, args, secret) {
            if (this.sXHRList[api] != null) {
                return;
            }
            this.sXHRList[api] = Date.now();

            // 添加sign
            args.sign = this.getArgsSign(args, secret);
            var params = MiscUtils.object2UrlStrArgs(args);
            var xhr = new Laya.HttpRequest();
            xhr.http.timeout = 10000;//设置超时时间；
            xhr.once(Laya.Event.ERROR, this, this.onXHRError, [api, args]);
            xhr.once(Laya.Event.COMPLETE, this, this.onXHRCompleted, [api, args]);
            xhr.send(api, params, "post", "text");
        }

        static xhrPostWithSignAndHeader(api, args, secret, headerInfo) {
            if (this.sXHRList[api] != null) {
                return;
            }
            this.sXHRList[api] = Date.now();

            // 添加sign
            args.sign = this.getArgsSign(args, secret);

            // 头数据header
            var headers = ["Content-Type", "application/x-www-form-urlencoded"];
            var headerKeys = Object.keys(headerInfo)
            for (var i = 0; i < headerKeys.length; i++) {
                headers.push(headerKeys[i]),
                    headers.push(headerInfo[headerKeys[i]])
            }

            var params = MiscUtils.object2UrlStrArgs(args);
            var xhr = new Laya.HttpRequest();
            xhr.http.timeout = 10000;//设置超时时间；
            xhr.once(Laya.Event.ERROR, this, this.onXHRError, [api, args]);
            xhr.once(Laya.Event.COMPLETE, this, this.onXHRCompleted, [api, args]);
            xhr.send(api, params, "post", "text", headers);
        }

        static onXHRError(api, params, data) {
            this.sXHRList[api] = null;
            console.log("onXHRError:" + api + ",params:" + JSON.stringify(params) + ",response:" + data);
            Laya.stage.event(this.NET_XHR_RESPONSE, [0, api, params, data]);
        }

        static onXHRCompleted(api, params, data) {
            this.sXHRList[api] = null;
            var response = JSON.parse(data);

            if (response.code) {

                if (response.code != 200) {
                    Laya.stage.event(this.NET_XHR_RESPONSE, [0, api, params, response.msg]);
                    return;
                }

                if (response.data == null) {
                    Laya.stage.event(this.NET_XHR_RESPONSE, [0, api, params, "登陆失败，数据体为空"]);
                    return;
                }

                if (response.data.status != 1) {
                    Laya.stage.event(this.NET_XHR_RESPONSE, [0, api, params, response.data.msg]);
                    return;
                }
            }
            else {
                Laya.stage.event(this.NET_XHR_RESPONSE, [0, api, params, response.msg]);
                return;
            }

            Laya.stage.event(this.NET_XHR_RESPONSE, [1, api, params, response]);
        }
    }
    WebRequest.XHR_RESPONSE = Event.XHR_RESPONSE;
    WebRequest.sXHRList = {};
    Laya.ILaya.regClass(WebRequest);
    Laya.ClassUtils.regClass("fc.framework.WebRequest", WebRequest);

    class ViewManager {

        static init(viewCfg, sceneCfg) {
            this.viewCfg = viewCfg;
            this.sceneCfg = sceneCfg;
        }

        static registerView(viewId, config) {
            if (!this.viewCfg) {
                this.viewCfg = {};
            }
            this.viewCfg[viewId] = config;
        }

        static registerScene(sceneId, configs, type) {
            if (!this.sceneCfg) {
                this.sceneCfg = {};
            }
            this.sceneCfg[sceneId] = {
                type: type,
                array: configs
            }
        }

        static isSceneEmpty() {
            return this.sequence.length == 0 && this.openedList.length == 0;
        }

        static unloadScene() {
            this.sequence.splice(0);
            this.syncList.splice(0);
            this.taskList.splice(0);
            var keys = Object.keys(this.viewCfg);
            for (var index = 0; index < keys.length; index++) {
                this.closeView(keys[index]);
            }
            if (this.sceneId != null) {
                Laya.timer.callLater(this, this.broadcastUnloadScene, [this.sceneId, null]);
            }
            this.sceneId = null;
        }

        static loadScene(sceneId, data, callback) {
            console.log("loadScene:" + sceneId);
            this.sceneId = null;
            this.sequence.splice(0);
            this.syncList.splice(0);
            this.taskList.splice(0);
            var keys = Object.keys(this.viewCfg);
            for (var index = 0; index < keys.length; index++) {
                this.closeView(keys[index]);
            }

            this.sceneId = sceneId;
            this.sceneData = data;
            this.sceneCb = callback;

            if (this.sceneCfg == null) {
                console.warn("[ViewManager]loadScene:", "sceneCfg is undefine");
                return;
            }

            var sceneInfo = this.sceneCfg[sceneId];
            if (sceneInfo == null) {
                console.warn("[ViewManager]loadScene:", "sceneId is undefine,", sceneId);
                Laya.timer.callLater(this, this.broadcastUnloadScene, [sceneId, callback]);
                return;
            }

            if (!sceneInfo.array || sceneInfo.array.length == 0) {
                console.warn("[ViewManager]loadScene:", "array is invalid");
                Laya.timer.callLater(this, this.broadcastUnloadScene, [sceneId, callback]);
                return;
            }

            if (sceneInfo.type == "sequence") {
                for (let index = 0; index < sceneInfo.array.length; index++) {
                    const element = sceneInfo.array[index];
                    if (!element.checkCode || !this.checkFun || this.checkFun(element.checkCode)) {
                        this.sequence.push(element);
                    }
                }
                if (this.sequence.length == 0) {
                    console.warn("[ViewManager]loadScene:", "no valid view");
                    Laya.timer.callLater(this, this.broadcastUnloadScene, [sceneId, callback]);
                    return;
                }
                this.openView(this.sequence[0].viewId, this.sceneData, null);
            }
            else {
                for (let index = 0; index < sceneInfo.array.length; index++) {
                    const element = sceneInfo.array[index];
                    if (!element.checkCode || !this.checkFun || this.checkFun(element.checkCode)) {
                        this.syncList.push(element);
                    }
                }
                if (this.syncList.length == 0) {
                    console.warn("[ViewManager]loadScene:", "no valid view");
                    Laya.timer.callLater(this, this.broadcastUnloadScene, [sceneId, callback])
                    return;
                }
                this.openView(this.syncList[0].viewId, this.sceneData, null);
            }
            console.log("loadScene over");
        }

        static openView(viewId, data, callback) {
            if (this.viewCfg == null) {
                console.warn("[ViewManager]openView:", "viewCfg is undefine");
                return;
            }
            var viewInfo = this.viewCfg[viewId];
            if (viewInfo == null) {
                console.warn("[ViewManager]openView:", "invalid param, ", viewId);
                if (callback) {
                    callback.runWith(null, viewId);
                }
                return;
            }
            data = data || {};
            if (viewInfo.status) {
                console.warn("[ViewManager]openView:", "view has opened", viewId);
                if (callback) {
                    callback.runWith(null, viewId);
                }
                return;
            }
            // 是否在按顺序打开的窗口队列中
            var hasInTask = false;
            if (this.sequence.length != 0) {
                for (let index = 0; index < this.sequence.length; index++) {
                    const element = this.sequence[index];
                    if (element.viewId == viewId && index != 0) {
                        console.warn("[ViewManager]openView:", "view has in sequence", viewId);
                        if (callback) {
                            callback.runWith(null, viewId);
                        }
                        return;
                    }
                    else {
                        hasInTask = true;
                    }
                }
            }
            // 是否在同时可显示的窗口队列中
            if (this.syncList.length != 0) {
                for (let index = 0; index < this.syncList.length; index++) {
                    const element = this.syncList[index];
                    if (element.viewId == viewId && index != 0) {
                        console.warn("[ViewManager]openView:", "view has in sync list", viewId);
                        if (callback) {
                            callback.runWith(null, viewId);
                        }
                        return;
                    }
                    else {
                        hasInTask = true;
                    }
                }
            }
            // 是否在任务列表中
            if (!hasInTask && this.taskList.length != 0) {
                for (let index = 0; index < this.taskList.length; index++) {
                    const element = this.taskList[index];
                    if (element.viewId == viewId) {
                        console.warn("[ViewManager]openView:", "view has in task list", viewId);
                        if (callback) {
                            callback.runWith(null, viewId);
                        }
                        return;
                    }
                }
                this.taskList.push({ viewId: viewId, data: data, callback: callback });
                return;
            }
            viewInfo.status = 1;
            Laya.Scene.open(viewInfo.url, false, null,
                Laya.Handler.create(this, this.onViewLoaded, [viewId, data, callback]));
        }

        static onViewLoaded(viewId, data, callback, view) {
            if (view) {
                view.viewId = viewId;
            }
            if (this.viewCfg[viewId].status != 1) {
                view.visible = false;
                this.viewCfg[viewId].status = 0;
                Laya.Scene.close(this.viewCfg[viewId].url);
                if (callback) {
                    callback.runWith(null, viewId);
                }
                return;
            }
            if (this.viewCfg[viewId].script) {
                var scriptType = Laya.ClassUtils.getClass(this.viewCfg[viewId].script);
                var script = view.getComponent(scriptType);
                if (script == null) {
                    script = view.addComponent(scriptType);
                }
            }
            var components = view.getComponents(Laya.Script);
            for (var index = 0; components && index < components.length; index++) {
                if (components[index].initView) {
                    components[index].initView(data);
                }
                components[index].viewId = viewId;
            }
            this.viewCfg[viewId].view = view;
            this.viewCfg[viewId].status = 2;

            this.openedList.push(viewId);

            if (callback) {
                callback.runWith(view, viewId);
            }

            var viewName = this.viewCfg[viewId].url;
            viewName = viewName.substring(viewName.lastIndexOf('/') + 1, viewName.lastIndexOf('.'));

            Laya.stage.frameOnce(1, this, function (viewName, viewId, view) {
                Laya.stage.event(Event.UI_VIEW_OPENED, [viewName, viewId, view]);
            }, [viewName, viewId, view], false);

            if (this.sequence.length == 1 && this.sequence[0].viewId == viewId) {
                if (this.sceneCb) {
                    this.sceneCb.run();
                }
                Laya.stage.event(Event.UI_SCENE_LOADED, this.sceneId);
            }

            if (data.zOrder != null) {
                view.zOrder = data.zOrder;
            }
            else {
                view.zOrder = this.openedList.length;
            }
            if (this.syncList.length > 0) {
                var element = this.syncList.shift();
                if (element.zOrder != null) {
                    view.zOrder = element.zOrder;
                }
                if (this.syncList.length > 0) {
                    if (!!this.syncList[0].delay) {
                        Laya.stage.timerOnce(this.syncList[0].delay, this, this.openView, [this.syncList[0].viewId, this.sceneData, null], false);
                    }
                    else {
                        this.openView(this.syncList[0].viewId, this.sceneData, null);
                    }
                }
                else {
                    if (this.sceneCb) {
                        this.sceneCb.run();
                    }
                    Laya.stage.event(Event.UI_SCENE_LOADED, this.sceneId);
                }
            }
            else if (this.taskList.length > 0) {
                var taskInfo = this.taskList.shift();
                this.openView(taskInfo.viewId, taskInfo.data, taskInfo.callback);
            }
        }

        static closeView(viewInfo) {
            var viewId = "";
            if (typeof viewInfo === "string") {
                viewId = viewInfo;
                if (this.viewCfg[viewId] == null) {
                    return;
                }
                if (!this.viewCfg[viewId].status) {
                    return;
                }
                if (this.viewCfg[viewId].status == 1) {
                    this.viewCfg[viewId].status = 0;
                    return;
                }
            }
            else if (viewInfo instanceof Laya.View) {
                var keys = Object.keys(this.viewCfg);
                for (var index = 0; index < keys.length; index++) {
                    const element = keys[index];
                    if (this.viewCfg[element] && this.viewCfg[element].view == viewInfo) {
                        viewId = element;
                        break;
                    }
                }
                if (viewId == "") {
                    var viewUrl = viewInfo.url;
                    var viewRes = viewUrl.substring(viewUrl.lastIndexOf('/') + 1, viewUrl.lastIndexOf('.'));
                    console.warn("[ViewManager]closeView:", "view has closed, ", viewRes);
                    return;
                }
            }
            else {
                console.warn("[ViewManager]closeView:", "Invalid param");
                return;
            }
            this.viewCfg[viewId].view = null;
            this.viewCfg[viewId].status = 0;
            Laya.Scene.close(this.viewCfg[viewId].url);

            var idx = this.openedList.indexOf(viewId);
            if (idx >= 0) {
                this.openedList.splice(idx, 1);
            }

            var viewName = this.viewCfg[viewId].url;
            viewName = viewName.substring(viewName.lastIndexOf('/') + 1, viewName.lastIndexOf('.'));

            Laya.stage.frameOnce(1, this, function (viewName, viewId) {
                Laya.stage.event(Event.UI_VIEW_CLOSED, [viewName, viewId]);
            }, [viewName, viewId], false);

            if (this.sequence.length > 0 && this.sequence[0].viewId == viewId) {
                this.sequence.shift();
                if (this.sequence.length > 0) {
                    this.openView(this.sequence[0].viewId, this.sceneData, null);
                }
            }

            if (this.sequence.length == 0 && this.syncList.length == 0 &&
                this.openedList.length == 0 && this.sceneId != null) {
                Laya.timer.callLater(this, this.broadcastUnloadScene, [this.sceneId, null]);
                this.sceneId = null;
            }
        }

        static broadcastUnloadScene(sceneId, callback) {
            console.log(sceneId);
            Laya.stage.frameOnce(1, this, function (sceneId, sceneCb) {
                if (sceneCb) {
                    sceneCb.run();
                }
                Laya.stage.event(Event.UI_SCENE_UNLOADED, sceneId);
            }, [sceneId, callback]);
        }

        static hideViews() {

        }

        static showViews() {

        }

        static showToastMsg(msg) {
            if (this.toastMsg == null) {
                this.toastMsg = 1;
                Laya.loader.create("fc-layout/prefab/toastMsg.prefab", Laya.Handler.create(this, this.onToastPrefabReady, [msg]), null, Laya.Loader.PREFAB);
            }
            else if (this.toastMsg != 1) {
                this.popToast(msg);
            }
        }

        static onToastPrefabReady(msg, prefab) {
            this.toastMsg = Laya.stage.addChild(prefab.create());
            this.popToast(msg);
        }

        static popToast(msg) {
            this.toastMsg.visible = true;
            this.toastMsg.text = msg;
            this.toastMsg.centerY = 400;
            this.toastMsg.zOrder = 100;
            Laya.timer.clear(this, this.hideToast);
            this.mesTween && Laya.Tween.clear(this.mesTween);
            this.mesTween = Laya.Tween.to(this.toastMsg, { centerY: 0 }, 500, Laya.Ease.elasticOut);

            Laya.timer.once(1000, this, this.hideToast);
        }

        static hideToast() {
            this.toastMsg.visible = false;
        }

        static showMessageBox(msg, callback) {
            this.messageList.push({ message: msg, confirm: callback });
            if (this.messageBox == null) {
                Laya.loader.create("fc-layout/prefab/msgBox.json", Laya.Handler.create(this, this.onMessagePrefabReady), null, Laya.Loader.PREFAB);
            }
            else if (this.messageList.length == 1 && this.messageCompleted) {
                this.popMessageBox();
            }
        }

        static onMessagePrefabReady(prefab) {
            this.messageBox = Laya.stage.addChild(prefab.create());
            var confirmBtn = this.messageBox.getChildByName("msgDialog").getChildByName("confirmBtn");
            confirmBtn.clickHandler = Laya.Handler.create(this, this.onMsgBoxConfirm, null, false);
            this.popMessageBox();
        }

        static popMessageBox() {
            if (this.messageList.length == 0) {
                this.messageBox.visible = false;
                return;
            }

            this.messageCompleted = false;
            this.messageData = this.messageList.shift();
            var msgDialog = this.messageBox.getChildByName("msgDialog");
            this.messageBox.visible = true;
            this.messageBox.centerY = 0;
            this.messageBox.centerX = 0;
            msgDialog.getChildByName("content").text = this.messageData.message;
            this.messageBox.zOrder = 100;
            msgDialog.scaleX = 0.1;
            msgDialog.scaleY = 0.1;
            var msgBg = this.messageBox.getChildByName("bg");
            msgBg.width = Laya.stage.width;
            msgBg.height = Laya.stage.height;
            Laya.Tween.to(msgDialog, { scaleX: 1, scaleY: 1 }, 500, null);
        }

        static onMsgBoxConfirm() {
            if (this.messageData.confirm) {
                this.messageData.confirm.run();
            }
            this.messageData = null;
            var msgDialog = this.messageBox.getChildByName("msgDialog");
            Laya.Tween.to(msgDialog, { scaleX: 0.1, scaleY: 0.1 }, 500, null, Laya.Handler.create(this, function () {
                this.messageCompleted = true;
                this.messageBox.visible = false;
                Laya.timer.once(500, this, this.popMessageBox);
            }));
        }
    }
    ViewManager.viewCfg = {};
    ViewManager.sceneCfg = {};
    ViewManager.syncList = [];
    ViewManager.sequence = [];
    ViewManager.taskList = [];
    ViewManager.openedList = [];
    ViewManager.sceneId = null;
    ViewManager.sceneData = null;
    ViewManager.sceneCb = null;
    ViewManager.checkFun = null;
    ViewManager.toastList = [];
    ViewManager.toastMsg = null;
    ViewManager.toastCompleted = false;
    ViewManager.messageList = [];
    ViewManager.messageBox = null;
    ViewManager.messageCompleted = false;
    ViewManager.messageData = null;

    class StartUpResLoader {

        static get appCfgUrl() {
            return this.sAppCfgUrl;
        }

        static set appCfgUrl(val) {
            if (val) {
                this.sAppCfgUrl = val;
            }
        }

        static loadAppCfg(completedCb) {
            console.log("loadAppCfg:" + this.appCfgUrl);
            Resource.loadResAsyn(this.sAppCfgUrl, Laya.Handler.create(this, this.onAppCfgLoaded, [completedCb]));
        }

        static onAppCfgLoaded(completedCb, config) {
            console.log("onAppCfgLoaded", config);
            this.startUpCfg = config.startUpCfg;
            this.startUpCfg.bgm = config.appInfo.bgm;
            this.startUpCfg.default3dScene = config.appInfo.default3dScene;
            this.startUpCfg.subPkgCfg = config.subPkgCfg;
            if (completedCb) {
                completedCb.runWith(config);
            }
        }

        static loadAppRes(completedCb) {
            console.log("loadAppRes");
            this.onAppResReady = completedCb;
            if (this.startUpCfg.subPkgCfg != null) {
                for (let index = 0; index < this.startUpCfg.subPkgCfg.length; index++) {
                    const element = this.startUpCfg.subPkgCfg[index];
                    Resource.registerSubPkg(element.pkgName, element.pkgPath);
                }
            }
            Laya.stage.event(Event.LOADING_PROGRESS, (LoadingStep.LOAD_SUBPACKAGE + 1) / LoadingStep.TOTAL_STEPS);
            var subpackages = [];
            for (var index = 0; index < this.startUpCfg.subpackage.length; index++) {
                subpackages.push(this.startUpCfg.subpackage[index]);
            }
            console.log("loadSubpackage");
            this.loadSubpackage(subpackages);
        }

        static loadSubpackage(subpackages) {
            if (subpackages && subpackages.length > 0) {
                var pkgName = subpackages.pop();
                Resource.loadSubpackage(pkgName, Laya.Handler.create(this, this.loadSubpackage, [subpackages]));
            }
            else {
                this.loadBaseConfig();
            }
        }

        static loadBaseConfig() {
            console.log("loadBaseConfig");
            Laya.stage.event(Event.LOADING_PROGRESS, (LoadingStep.LOAD_CONFIG + 1) / LoadingStep.TOTAL_STEPS);
            this.startUpCfg.configList = this.startUpCfg.configList ? this.startUpCfg.configList : [];
            if (this.startUpCfg.pluginCfg) {
                this.startUpCfg.configList.push(this.startUpCfg.pluginCfg);
            }
            if (this.startUpCfg.languageCfg) {
                this.startUpCfg.configList.push(this.startUpCfg.languageCfg);
            }
            if (this.startUpCfg.configList.length == 0) {
                this.loadBaseRes();
            }
            else {
                Resource.loadResAsyn(this.startUpCfg.configList,
                    Laya.Handler.create(this, this.loadBaseRes));
            }
        }

        static loadBaseRes() {
            console.log("loadBaseRes");
            Laya.stage.event(Event.LOADING_PROGRESS, (LoadingStep.LOAD_3DRES + 1) / LoadingStep.TOTAL_STEPS);
            if (this.startUpCfg.bgm) {
                console.log("loadMusic", this.startUpCfg.bgm);
                Audio.playMusic(this.startUpCfg.bgm);
            }
            if (this.startUpCfg.fontList && this.startUpCfg.fontList.length > 0) {
                console.log("loadFont");
                Resource.loadFont(this.startUpCfg.fontList);
            }
            if (this.startUpCfg.baseResList == null || this.startUpCfg.baseResList.length == 0) {
                this.loadScene3d();
            } else {
                var resPaths = [];
                for (var index = 0; index < this.startUpCfg.baseResList.length; index++) {
                    var info = this.startUpCfg.baseResList[index].split(",");
                    info.length == 2 ? resPaths.push(Resource.get3dResUrl(info[1], info[0])) : resPaths.push(Resource.get3dResUrl(info[0]));
                    //resPaths.push(Resource.get3dResUrl(this.startUpCfg.baseResList[index]));
                }
                var resInfo = this.startUpCfg.baseResList.shift();
                var info = resInfo.split(",");
                if (info.length == 2) {
                    Resource.load3dResAsyn(Resource.get3dResUrl(info[1], info[0]), Laya.Handler.create(this, this.onBaseResReady), null, info[0]);
                }
                else {
                    Resource.load3dResAsyn(Resource.get3dResUrl(info[0]), Laya.Handler.create(this, this.onBaseResReady));
                }
            }
        }

        static onBaseResReady() {
            console.log("onBaseResReady");
            if (this.startUpCfg.baseResList.length == 0) {
                this.loadScene3d();
            }
            else {
                var resInfo = this.startUpCfg.baseResList.shift();
                var info = resInfo.split(",");
                if (info.length == 2) {
                    Resource.load3dResAsyn(Resource.get3dResUrl(info[1], info[0]), Laya.Handler.create(this, this.onBaseResReady), null, info[0]);
                }
                else {
                    Resource.load3dResAsyn(Resource.get3dResUrl(info[0]), Laya.Handler.create(this, this.onBaseResReady));
                }
            }
        }

        static loadScene3d() {
            console.log("loadScene3d");
            Laya.stage.event(Event.LOADING_PROGRESS, (LoadingStep.LOAD_3DSCENE + 1) / LoadingStep.TOTAL_STEPS);
            if (this.startUpCfg.default3dScene == null || this.startUpCfg.default3dScene.length == 0) {
                this.onLaunchResReady(null);
                return;
            }

            Resource.loadScene3dAsyn(
                this.startUpCfg.default3dScene,
                Laya.Handler.create(this, this.onLaunchResReady)
            );
        }

        static onLaunchResReady(scene3d) {
            console.log("onLaunchResReady");
            Audio.stopMusic();
            if (this.onAppResReady) {
                this.onAppResReady.runWith(scene3d);
            }
        }
    }
    StartUpResLoader.sAppCfgUrl = "config/appConfig.json";
    StartUpResLoader.onAppCfgReady = null;
    StartUpResLoader.onAppResReady = null;

    class GameFrame {

        static startUp(appCfgUrl, channelTag, viewCfg, sceneCfg, playerCfg) {

            this.channelTag = channelTag;

            if (Laya.Browser.onIOS || Laya.Browser.onIPad || Laya.Browser.onIPhone) {
                if (Laya.Browser.onWeiXin) {
                    // Laya.stage.frameRate = Laya.Stage.FRAME_MOUSE;
                }
            }

            Laya.stage.on(Event.APP_SHOW, this, this.onGameShow);
            Laya.stage.on(Event.APP_HIDE, this, this.onGameHide);

            Laya.stage.on(Event.UI_SCENE_UNLOADED, this, this.onSceneUnloaded);

            this.viewCfg = viewCfg;
            this.sceneCfg = sceneCfg;
            this.playerInfo = playerCfg || {};

            Device.init(Laya.Handler.create(this, this.onGameShow,null,false), Laya.Handler.create(this, this.onGameHide,null,false));
            Channel.init();
            StartUpResLoader.appCfgUrl = appCfgUrl;
            StartUpResLoader.loadAppCfg(Laya.Handler.create(this, this.onAppCfgReady));
        }

        static onGameShow(res) {
            // Laya.timer.scale = 1;
            Laya.stage.renderingEnabled = true;
            if (this.appInfo && this.appInfo.bgm && !Audio.IsBgmPlaying && GameFrame.gameSceneId == fc.define.SceneId.Game) {
                Audio.playMusic(this.appInfo.bgm);
            }
            this.resume();
            console.log("onGameShow");
            Laya.stage.event(Event.DEVICE_SHOW, res)
        }

        static onGameHide() {
            // Laya.timer.scale = 0;
            this.pause();
            Laya.stage.renderingEnabled = false;
            Audio.stopAll();
            console.log("onGameHide");
            Laya.stage.event(Event.DEVICE_HIDE)
        }

        static onSceneUnloaded(sceneId) {
            if (sceneId != this.curSceneId || this.curSceneId == this.nextSceneId) {
                return;
            }
            this.curSceneId = this.nextSceneId;
            ViewManager.loadScene(this.nextSceneId, this.sceneData);
            if (this.sceneEvent) {
                Laya.stage.event(this.sceneEvent, this.sceneData);
            }
            if (this.curSceneId == SceneId.ShowFailed) {
                this.reportFailed(this.sceneData);
            }
            else if (this.curSceneId == SceneId.ShowSuccess) {
                this.reportSuccess(this.sceneData);
            }
            else if (this.curSceneId == SceneId.Over) {
                this.go2Home();
            }
            else if (this.curSceneId == SceneId.Home) {
                this.resume();
            }
        }

        static onAppCfgReady(config) {
            console.log("onAppCfgReady");
            this.appInfo = config.appInfo;
            this.startUpCfg = config.startUpCfg;
            this.gameCfg = config.gameCfg;
            this.goodsCfg = config.goodsCfg;

            if (this.startUpCfg.logoPage) {
                this.viewCfg["Logo"] = this.startUpCfg.logoPage;
                // this.sceneCfg[SceneId.Loading].type = "sync";
                // this.sceneCfg[SceneId.Loading].array.push({viewId: "Logo"});
            }
            if (config.playerCfg) {
                var keys = Object.keys(config.playerCfg);
                for (var index = 0; index < keys.length; index++) {
                    this.playerInfo[keys[index]] = config.playerCfg[keys[index]]
                }
            }
            this.loadPlayerInfo();
            if (config.viewCfg) {
                var keys = Object.keys(config.viewCfg);
                for (var index = 0; index < keys.length; index++) {
                    this.viewCfg[keys[index]] = config.viewCfg[keys[index]];
                    GameFrame.registerView(keys[index], config.viewCfg[keys[index]]);
                }
            }

            if (config.sceneCfg) {
                var keys = Object.keys(config.sceneCfg);
                for (var index = 0; index < keys.length; index++) {
                    const item = config.sceneCfg[keys[index]];
                    this.sceneCfg[keys[index]] = item;
                    GameFrame.registerScene(keys[index], item.array, item.type);
                }
            }
            console.log("openLoading");
            ViewManager.init(this.viewCfg, this.sceneCfg);
            Laya.stage.once(Event.UI_VIEW_OPENED, this, this.onLoadingOpened);
            ViewManager.loadScene(SceneId.Loading, null, null);
        }

        static onLoadingOpened() {
            Laya.stage.off(Event.UI_VIEW_OPENED, this, this.onLoadingOpened);
            StartUpResLoader.loadAppRes(Laya.Handler.create(this, this.onAppResReady));
        }

        static onAppResReady(scene3d) {
            if (scene3d) {
                this.scene3d = Laya.stage.addChildAt(scene3d, 0);
            }
            if (this.startUpCfg.languageCfg) {
                this.langCfg = Laya.loader.getRes(this.startUpCfg.languageCfg);
            }
            if (this.startUpCfg.pluginCfg) {
                this.pluginCfg = Laya.loader.getRes(this.startUpCfg.pluginCfg);

                if (this.pluginCfg.viewCfg) {
                    var keys = Object.keys(this.pluginCfg.viewCfg);
                    for (var index = 0; index < keys.length; index++) {
                        this.viewCfg[keys[index]] = this.pluginCfg.viewCfg[keys[index]];
                        GameFrame.registerView(keys[index], this.pluginCfg.viewCfg[keys[index]]);
                    }
                }

                if (this.pluginCfg.sceneCfg) {
                    var keys = Object.keys(this.pluginCfg.sceneCfg);
                    for (var index = 0; index < keys.length; index++) {
                        const config = this.pluginCfg.sceneCfg[keys[index]];
                        this.sceneCfg[keys[index]] = config;
                        GameFrame.registerScene(keys[index], config.array, config.type);
                    }
                }

                if (this.pluginCfg.gameCfg) {
                    var keys = Object.keys(this.pluginCfg.gameCfg);
                    for (var index = 0; index < keys.length; index++) {
                        this.gameCfg[keys[index]] = this.pluginCfg.gameCfg[keys[index]];
                    }
                }

                if (this.pluginCfg.goodsCfg) {
                    this.goodsCfg = this.pluginCfg.goodsCfg
                }

                if (this.pluginCfg.playerCfg) {
                    var keys = Object.keys(this.pluginCfg.playerCfg);
                    for (var index = 0; index < keys.length; index++) {
                        if (this.playerInfo[keys[index]] == null) {
                            this.playerInfo[keys[index]] = this.pluginCfg.playerCfg[keys[index]]
                        }
                    }
                    if (this.pluginCfg.playerCfg.goodsInUse) {
                        for (var index = this.playerInfo.goodsInUse.length; index < this.pluginCfg.playerCfg.goodsInUse.length; index++) {
                            this.playerInfo.goodsInUse.push(config.playerCfg.goodsInUse[index]);
                        }
                    }
                    if (this.pluginCfg.playerCfg.goodsInBag) {
                        for (var index = this.playerInfo.goodsInBag.length; index < this.pluginCfg.playerCfg.goodsInBag.length; index++) {
                            this.playerInfo.goodsInBag.push(config.playerCfg.goodsInBag[index]);
                        }
                    }
                }
            }
            Laya.stage.event(Event.LAUNCH_COMPLETED);
        }

        static get gameSceneId() {
            return this.curSceneId;
        }

        static get isGamePaused() {
            return !this.isRunning;
        }

        static pause() {
            this.isRunning = false;
            Laya.stage.event(Event.GAME_SLEEP);
        }

        static resume() {
            this.isRunning = true;
            Laya.stage.event(Event.GAME_WAKEUP);
        }

        static go2Scene(sceneInfo) {
            this.isRelive = !!sceneInfo.isRelive;
            this.isRestart = !!sceneInfo.isRestart;
            this.sceneData = sceneInfo.sceneData;
            this.sceneEvent = sceneInfo.sceneEvent;
            this.nextSceneId = sceneInfo.nextSceneId;
            if (sceneInfo.relivedTime != null) {
                this.relivedTime = sceneInfo.relivedTime;
            }
        }

        static finishLoading() {
            this.go2Home();
            Laya.stage.event(Event.UI_LOADING_COMPLETED);
        }

        static go2Home(data) {
            this.go2Scene({
                isRelive: false,
                isRestart: false,
                sceneData: data,
                sceneEvent: Event.GAME_HOME,
                nextSceneId: SceneId.Home,
                relivedTime: 0
            });
        }

        static go2Game(data) {
            this.go2Scene({
                isRelive: false,
                isRestart: false,
                sceneData: data,
                sceneEvent: Event.GAME_PREPARE,
                nextSceneId: SceneId.Prepare
            });
        }

        static startGame(data) {
            this.go2Scene({
                isRelive: false,
                isRestart: false,
                sceneData: data,
                sceneEvent: Event.GAME_START,
                nextSceneId: SceneId.Game
            });
        }

        static relive2Game(data) {
            this.go2Scene({
                isRelive: true,
                isRestart: false,
                sceneData: data,
                sceneEvent: Event.GAME_RELIVE_START,
                nextSceneId: SceneId.Game
            });
        }

        static resetGame(data) {
            this.go2Scene({
                isRelive: false,
                isRestart: true,
                sceneData: data,
                sceneEvent: Event.GAME_RESET,
                nextSceneId: SceneId.Prepare
            });
        }

        static go2Relive(data) {
            this.go2Scene({
                isRelive: true,
                isRestart: false,
                sceneData: data,
                sceneEvent: Event.GAME_RELIVE,
                nextSceneId: SceneId.Prepare,
                relivedTime: this.relivedTime + 1
            });
        }

        static showSuccessed(data) {
            this.go2Scene({
                isRelive: false,
                isRestart: false,
                sceneData: data,
                sceneEvent: Event.GAME_SHOW_WIN,
                nextSceneId: SceneId.ShowSuccess
            });
        }

        static showFailed(data) {
            this.go2Scene({
                isRelive: false,
                isRestart: false,
                sceneData: data,
                sceneEvent: Event.GAME_SHOW_FAIL,
                nextSceneId: SceneId.ShowFailed
            });
        }

        static reportSuccess(data) {
            this.go2Scene({
                isRelive: false,
                isRestart: false,
                sceneData: data,
                sceneEvent: Event.GAME_WIN,
                nextSceneId: SceneId.Success
            });
        }

        static reportFailed(data) {
            this.go2Scene({
                isRelive: false,
                isRestart: false,
                sceneData: data,
                sceneEvent: Event.GAME_FAILED,
                nextSceneId: SceneId.Failed
            });
        }

        static overGame(data) {
            this.savePlayerInfo(data);
            this.go2Scene({
                isRelive: false,
                isRestart: false,
                sceneData: data,
                sceneEvent: Event.GAME_OVER,
                nextSceneId: SceneId.Over
            });
        }

        static back2Home(data) {
            this.go2Scene({
                isRelive: false,
                isRestart: false,
                sceneData: data,
                sceneEvent: Event.GAME_BACK_HOME,
                nextSceneId: SceneId.Home
            });
        }

        static exitApplication() {
            if (window["conch"]) {
                window["conch"].exit();
            }
        }

        static savePlayerInfo(data) {
            if (data == null) {
                return;
            }
            if (data != this.playerInfo) {
                var keys = Object.keys(data);
                for (var index = 0; index < keys.length; index++) {
                    this.playerInfo[keys[index]] = data[keys[index]];
                }
            }
            Laya.LocalStorage.setItem(`${this.appInfo.appName}:${this.channelTag}`, JSON.stringify(this.playerInfo));
        }

        static loadPlayerInfo() {
            var historyData = Laya.LocalStorage.getItem(`${this.appInfo.appName}:${this.channelTag}`);
            if (historyData == null || historyData.length == 0) {
                return;
            }
            var historyInfo = JSON.parse(historyData);
            var keys = Object.keys(historyInfo);
            for (var index = 0; index < keys.length; index++) {
                this.playerInfo[keys[index]] = historyInfo[keys[index]];
            }
        }

        static playClickSound() {
            Audio.playSound(this.appInfo.soundClick, false);
        }

        static showToastMsg(msg) {
            ViewManager.showToastMsg(msg);
        }

        static showMessageBox(msg, callback) {
            ViewManager.showMessageBox(msg, callback);
        }

        static showModal(title, content, confirmTxt, cancelTxt, success, failed, completed) {
            return Channel.showModal(title, content, confirmTxt, cancelTxt, success, failed, completed);
        }

        static openView(viewId, data, callback) {
            ViewManager.openView(viewId, data, callback);
        }

        static closeView(viewInfo) {
            ViewManager.closeView(viewInfo);
        }

        static closeAllViews() {
            ViewManager.unloadScene();
        }

        static setViewValidCheckFun(fun) {
            ViewManager.checkFun = fun;
        }

        static registerView(viewId, config) {
            ViewManager.registerView(viewId, config);
        }

        static registerScene(sceneId, configs, type) {
            ViewManager.registerScene(sceneId, configs, type);
        }

        static getLocalStorage(key) {
            return Laya.LocalStorage.getItem(`${this.appInfo.appName}:${this.channelTag}:${key}`);
        }

        static setLocalStorage(key, val) {
            if (val == null) {
                return;
            }
            if (val instanceof Object) {
                Laya.LocalStorage.setItem(`${this.appInfo.appName}:${this.channelTag}:${key}`, JSON.stringify(val));
            }
            else if (val instanceof Array) {
                Laya.LocalStorage.setItem(`${this.appInfo.appName}:${this.channelTag}:${key}`, JSON.stringify(val));
            }
            else {
                Laya.LocalStorage.setItem(`${this.appInfo.appName}:${this.channelTag}:${key}`, val.toString());
            }
        }
    }
    GameFrame.appInfo = {};
    GameFrame.startUpCfg = {};
    GameFrame.langCfg = {
        gettxt: "恭喜获得",
        getcointxt: "恭喜获得金币",
        novideotxt: "暂无可看视频，请稍后再试",
        novideotxt2: "现在没有视频看了",
        nocointxt: "金币不足!",
        nokeytxt: "钥匙用完了，请看视频兑换!",
        sharetimeshort: "录屏失败：录屏时长低于 3 秒!",
        sharefailtxt: "分享失败!",
        ok: "确定",
        cancel: "取消",
        videocanceltitle: "未观看完视频",
        videocanceldesc: "观看完视频才能获得奖励哦",
        unlock: "未解锁",
        level: "关卡"
    };

    GameFrame.nicknameList = {};//昵称
    GameFrame.gameCfg = {};
    GameFrame.goodsCfg = {};
    GameFrame.pluginCfg = {};
    GameFrame.playerInfo = {};
    GameFrame.viewCfg = {};
    GameFrame.sceneCfg = {};
    GameFrame.scene3d = null;
    GameFrame.channelTag = "unknown";
    GameFrame.isGMMode = false;
    GameFrame.isRunning = true;
    GameFrame.isRestart = false;
    GameFrame.isRelive = false;
    GameFrame.relivedTime = 0;
    GameFrame.curSceneId = SceneId.Loading;
    GameFrame.nextSceneId = SceneId.Loading;
    GameFrame.sceneData = null;
    GameFrame.sceneEvent = null;
    Laya.ILaya.regClass(GameFrame);
    Laya.ClassUtils.regClass("fc.framework.GameFrame", GameFrame);

    exports.Audio = Audio;
    exports.Device = Device;
    exports.Channel = Channel;
    exports.Joystick = Joystick;
    exports.Resource = Resource;
    exports.Md5Utils = Md5Utils;
    exports.MiscUtils = MiscUtils;
    exports.WebRequest = WebRequest;
    exports.GameFrame = GameFrame;

}(window.fc.framework, Laya))
