window.fc = window.fc || {};
window.fc.plugin = window.fc.plugin || {};

(function (exports, Laya) {
    'use strict';

    var Event = fc.define.Event;
    var Layout = fc.define.Layout;
    var JoystickCode = fc.define.JoystickCode;
    var Device = fc.framework.Device;
    var Channel = fc.framework.Channel;
    var Resource = fc.framework.Resource;
    var GameFrame = fc.framework.GameFrame;

    class ObjectPool {

        static printPoolInfo() {
            console.log("ObjectPool.poolStatus:" + JSON.stringify(this.poolStatus));
            for (var key in this.poolStatus) {
                if (this.poolStatus.hasOwnProperty(key)) {
                    Laya.Pool.clearBySign(key);
                }
            }
            this.poolStatus = {};
        }

        static recoverSprite3d(sprite3d) {
            sprite3d.removeSelf();
            if (sprite3d["_fc_sign"] == null) {
                return;
            }
            if (sprite3d["__InPool"]) return;
            if (!this.poolStatus[sprite3d["_fc_sign"]]) this.poolStatus[sprite3d["_fc_sign"]] = {};
            if (this.poolStatus[sprite3d["_fc_sign"]]["free"]) {
                this.poolStatus[sprite3d["_fc_sign"]]["free"] += 1;
            }
            else {
                this.poolStatus[sprite3d["_fc_sign"]]["free"] = 1;
            }
            Laya.Pool.recover(sprite3d["_fc_sign"], sprite3d);
        }

        static getSprite3d(original, sign) {
            var sprite_sign = sign ? sign : original.url;
            if (this.poolStatus[sprite_sign]) {
                this.poolStatus[sprite_sign]["total"] += 1;
            }
            else {
                this.poolStatus[sprite_sign] = {};
                this.poolStatus[sprite_sign]["total"] = 1;
            }
            var self = this;
            return Laya.Pool.getItemByCreateFun(
                sprite_sign,
                function () { return self.createSprite3d(original, sprite_sign); })
        }

        static createSprite3d(original, sign) {
            var sprite3d = Laya.Sprite3D.instantiate(original, null, false, Laya.Vector3._ZERO.clone());
            sprite3d["_fc_sign"] = sign;
            return sprite3d;
        }    

        static getEmptySprite3d(sign) {
            var sprite_sign = sign;
            if (this.poolStatus[sprite_sign]) {
                this.poolStatus[sprite_sign]["total"] += 1;
            }
            else {
                this.poolStatus[sprite_sign] = {};
                this.poolStatus[sprite_sign]["total"] = 1;
            }
            var self = this;
            return Laya.Pool.getItemByCreateFun(
                sprite_sign,
                function () { return self.createEmptySprite3d(sign); });
        }

        static createEmptySprite3d(sign) {
            var sprite3d = new Laya.Sprite3D();
            sprite3d["_fc_sign"] = sign;
            return sprite3d;
        }
    }
    ObjectPool.poolStatus = {};

    class GameObj extends Laya.Script3D {

        constructor() {
            super();
        }
    
        get Sprite() {
            return this.owner;
        }
    
        get Animator() {
            return this.owner.getComponent(Laya.Animator);
        }
    
        get Rigidbody() {
            return this.owner.getComponent(Laya.Rigidbody3D);
        }
    
        get PhysicsCollider() {
            return this.owner.getComponent(Laya.PhysicsCollider);
        }

        findChild(path) {
            if (path == null || path == "") {
                return null;
            }
            var layers = path.split("/");
            var child = this.Sprite;
            for (let index = 0; index < layers.length; index++) {
                child = child.getChildByName(layers[index]);
                if (child == null) {
                    return null;
                }
            }
            return child;
        }
    }
    Laya.ClassUtils.regClass("fc.plugin.GameObj", GameObj);

    class BaseView extends Laya.Script {

        initView(data) {

        }

        get View() {
            return this.owner;
        }

        
        get FullUI() {
            return this.fullUI;
        }
        get TopUI() {
            return this.topUI;
        }
        get MiddleUI() {
            return this.middleUI;
        }
        get BottomUI() {
            return this.bottomUI;
        }
        get LeftFloatUI() {
            return this.leftFloatUI;
        }
        get RightFloatUI() {
            return this.rightFloatUI;
        }

        constructor() {
            super();
            this.active = false;

            this.topUI = null;
            this.middleUI = null;
            this.bottomUI = null;
            this.leftFloatUI = null;
            this.rightFloatUI = null;
            this.fullUI = null;

            this.applyStatusBar = true;
            this.isFull = true;
            this.vLayout = Layout.VERTICAL_MIDDLE;
            this.hLayout = Layout.HORIZONTAL_CENTER;
            
            this.viewName = "";
        }

        applyLayout() {
            var view = this.owner;
            if (this.isFull) {
                view.top = 0;
                view.bottom = 0;
                view.left = 0;
                view.right = 0;
            }
            else {

                if (this.vLayout == 0) {
                    view.top = 0;
                }
                else if (this.vLayout == 1) {
                    view.centerY = 0;
                }
                else {
                    view.bottom = 0;
                }

                if (this.hLayout == 0) {
                    view.left = 0;
                }
                else if (this.hLayout == 1) {
                    view.centerX = 0;
                }
                else {
                    view.right = 0;
                }
            }

            if (this.topUI) {
                this.topUI.top = Device.statusBarHeight * Laya.stage.height / Device.screenHeight;
                this.topUI.centerX = 0;
            }

            if (this.middleUI) {
                this.middleUI.centerY = 0;
                this.middleUI.centerX = 0;
            }

            if (this.bottomUI) {
                this.bottomUI.bottom = 0;
                this.bottomUI.centerX = 0;
            }

            if (this.leftFloatUI) {
                this.leftFloatUI.left = 0;
                this.leftFloatUI.bottom = 0;
                this.leftFloatUI.top = 0;
            }

            if (this.rightFloatUI) {
                this.rightFloatUI.left = Laya.stage.width - this.rightFloatUI.width;
                this.rightFloatUI.centerY = 0;
                this.rightFloatUI.bottom = 0;
                this.rightFloatUI.top = 0;
            }

            if (this.fullUI) {
                this.fullUI.top = 0;
                this.fullUI.bottom = 0;
                this.fullUI.left = 0;
                this.fullUI.right = 0;
            }
        }

        onAwake() {
            this.topUI = this.owner.getChildByName("topUI");
            this.middleUI = this.owner.getChildByName("middleUI");
            this.bottomUI = this.owner.getChildByName("bottomUI");
            this.leftFloatUI = this.owner.getChildByName("leftFloatUI");
            this.rightFloatUI = this.owner.getChildByName("rightFloatUI");
            this.fullUI = this.owner.getChildByName("fullUI");
            
            this.viewName = this.owner.url;
            this.viewName = this.viewName.substring(this.viewName.lastIndexOf('/') + 1, this.viewName.lastIndexOf('.'));

            //this.applyStatusBar = platformType != "wx"
        }

        onStart() {
            this.applyLayout();
        }

        onEnable() {
            this.applyLayout();
        }
        
    }
    Laya.ClassUtils.regClass("fc.plugin.BaseView", BaseView);

    class Effect3d extends Laya.Script3D {

        static createEffect(effectInfo) {

            if (effectInfo == null) {
                return ;
            }
            
            effectInfo.createTime = Laya.timer.currTimer
            this.effectWaitingList.push(effectInfo);
            if (this.effectWaitingList.length == 1) {
                this.instanceEffectDelay(2);
            }
        }

        static instanceEffectDelay(delayFrame) {
            Laya.timer.frameOnce(delayFrame, this, this.instanceEffect, [delayFrame]);
        }

        static instanceEffect(delayFrame) {
            var effectInfo = this.effectWaitingList.shift();
            if (Laya.timer.currTimer - effectInfo.createTime > effectInfo.duaration) {
                if (this.effectWaitingList.length > 0) {
                    this.instanceEffectDelay(delayFrame);
                }
                return;
            }

            Resource.loadSprite3dAsyn(effectInfo.sign, Laya.Handler.create(this, this.onEffectReady, [effectInfo]), null, effectInfo.subPkg)

            if (this.effectWaitingList.length > 0) {
                this.instanceEffectDelay(delayFrame);
            }
        }

        static onEffectReady(effectInfo, sprite) {

            if (Laya.timer.currTimer - effectInfo.createTime > effectInfo.duaration) {
                return;
            }

            var effect = Laya.Sprite3D.instantiate(sprite, null, false, Laya.Vector3._ZERO.clone());
            if (effect == null || effectInfo.parent.destroyed || effectInfo.parent.parent == null) {
                return;
            }

            effectInfo.parent.addChild(effect);
            effect.name = effectInfo.customId;
            if (effectInfo.isWorldSpace == null) {

            }
            else if (effectInfo.isWorldSpace) {
                effectInfo.position ? effect.transform.position = effectInfo.position : 1;
                effectInfo.rotation ? effect.transform.rotation = effectInfo.rotation : 1;
            }
            else {
                effectInfo.position ? effect.transform.localPosition = effectInfo.position : 1;
                effectInfo.rotation ? effect.transform.localRotation = effectInfo.rotation : 1;
            }
            var delayRecover = effect.getComponent(Effect3d);
            if (delayRecover == null) {
                delayRecover = effect.addComponent(Effect3d);
            }
            delayRecover.delay = effectInfo.duaration;

            if (effectInfo.disableDepthTest) {
                this.diableDepthTest(effect);
            }
        }

        static diableDepthTest(sprite3d) {
            if (sprite3d instanceof Laya.ShuriKenParticle3D) {
                var mat = sprite3d.particleRenderer.sharedMaterial;
                mat.depthTest = Laya.RenderState.DEPTHTEST_ALWAYS;
                sprite3d.particleRenderer.sharedMaterial = mat;
            }
            for (let index = 0; index < sprite3d.numChildren; index++) {
                this.diableDepthTest(sprite3d.getChildAt(index));
            }
        }

        constructor() {
            super();
            this.delay = Number.MAX_VALUE;
        }

        /**
         * cleanNode
         */
        cleanNode() {
            this.owner.destroy(true);
        }

        onUpdate() {
            if (this.delay < 0) {
                return;
            }

            this.delay -= Laya.timer.delta;
            if (this.delay <= 0) {
                this.delay = -1;
                this.owner.destroy(true);
            }
        }

        onDestroy() {
            this.delay = -1;
        }

    }
    Effect3d.effectWaitingList = [];
    
    class Trigger3dListener extends Laya.Script3D {

        constructor() {
            super();
            this.listener = null;
            this.triggerComponent = null;
        }

        static add(triggerComponent, listener) {
            if (triggerComponent == null || listener == null) {
                return ;
            }
            var triggerListener = triggerComponent.owner.getComponent(Trigger3dListener);
            if (!triggerListener) {
                triggerListener = triggerComponent.owner.addComponent(Trigger3dListener);
            }
            triggerListener.listener = listener;
            triggerListener.triggerComponent = triggerComponent;
            if (triggerListener.listener == triggerListener) {
                console.warn("Trigger3dListener", "listener can't be self")
            }
        }

        static remove(triggerComponent, listener) {
            if (triggerComponent == null) {
                return ;
            }
            var triggerListener = triggerComponent.owner.getComponent(Trigger3dListener);
            if (!triggerListener) {
                return ;
            }
            triggerListener.listener = null;
            triggerListener.triggerComponent = null;
        }

        onTriggerEnter(other, self, contact) {
            if (this.listener == this) {
                return ;
            }
            if (this.listener != null && this.listener.on3dTriggerEnter) {
                this.listener.on3dTriggerEnter(other, self ? self : this.triggerComponent, contact);
            }
        }

        onTriggerExit(other, self, contact) {
            if (this.listener == this) {
                return ;
            }
            if (this.listener != null && this.listener.on3dTriggerEnter) {
                this.listener.on3dTriggerExit(other, self ? self : this.triggerComponent, contact);
            }
        }

        onTriggerStay(other, self, contact) {
            if (this.listener == this) {
                return ;
            }
            if (this.listener != null && this.listener.on3dTriggerStay) {
                this.listener.on3dTriggerStay(other, self ? self : this.triggerComponent, contact);
            }
        }

        onDestroy() {
            this.listener = null;
            this.triggerComponent = null;
        }
    }

    class Collision3dListener extends Laya.Script3D {
        
        constructor() {
            super();
            this.listener = null;
            this.collisionComponent = null;
        }

        static add(collisionComponent, listener) {
            if (collisionComponent == null || listener == null) {
                return ;
            }
            var collisionListener = collisionComponent.owner.getComponent(Collision3dListener);
            if (!collisionListener) {
                collisionListener = collisionComponent.owner.addComponent(Collision3dListener);
            }
            collisionListener.listener = listener;
            collisionListener.collisionComponent = collisionComponent;
            if (collisionListener.listener == collisionListener) {
                console.warn("Trigger3dListener", "listener can't be self")
            }
        }

        static remove(collisionComponent, listener) {
            if (collisionComponent == null) {
                return ;
            }
            var collisionListener = collisionComponent.owner.getComponent(Collision3dListener);
            if (!collisionListener) {
                return ;
            }
            collisionListener.listener = null;
            collisionListener.collisionComponent = null;
        }

        		/**
		 * 开始碰撞时执行
		 * 此方法为虚方法，使用时重写覆盖即可
		 */
		onCollisionEnter(collision) {
            if (this.listener == this) {
                return ;
            }
            if (this.listener != null && this.listener.on3dCollisionEnter) {
                this.listener.on3dCollisionEnter(collision);
            }
        }

		/**
		 * 持续碰撞时执行
		 * 此方法为虚方法，使用时重写覆盖即可
		 */
		onCollisionStay(collision) {
            if (this.listener == this) {
                return ;
            }
            if (this.listener != null && this.listener.on3dCollisionStay) {
                this.listener.on3dCollisionStay(collision);
            }
        }

		/**
		 * 结束碰撞时执行
		 * 此方法为虚方法，使用时重写覆盖即可
		 */
		onCollisionExit(collision) {
            if (this.listener == this) {
                return ;
            }
            if (this.listener != null && this.listener.on3dCollisionExit) {
                this.listener.on3dCollisionExit(collision);
            }
        }

        onDestroy() {
            this.listener = null;
            this.collisionComponent = null;
        }
    }

    class AnimEventListener extends Laya.Script3D {

        static add(animator, listener) {
            var animEvtListener = animator.owner.getComponent(AnimEventListener);
            if (animEvtListener == null) {
                animEvtListener = animator.owner.addComponent(AnimEventListener);
            }
            animEvtListener.listener = listener;
        }


        static remove(animator, listener) {
            var animEvtListener = animator.owner.getComponent(AnimEventListener);
            if (animEvtListener == null) {
                return ;
            }
            animEvtListener.listener = null;
        }

        constructor() {
            super();
            this.listener = null;
        }
    
        onAnimEvt(...args)
        {
            if (this.listener && this.listener.onAnimEvt) {
                this.listener.onAnimEvt(...args);
            }
        }
    
    }
    
    class AnimStateListener extends Laya.AnimatorStateScript {

        constructor() {
            super();
            this.stateName = "";
            this.animator = null;
            this.listener = null;
        }

        static add(animator, stateName, listener) {
            var controller = animator.getControllerLayer(0);
            var animState = controller.getAnimatorState(stateName);
            if (animState == null) {
                return;
            }
            var animStatelistener = animState.getScript(AnimStateListener);
            if (animStatelistener == null) {
                animStatelistener = animState.addScript(AnimStateListener);
            }
            animStatelistener.stateName = stateName;
            animStatelistener.animator = animator;
            animStatelistener.listener = listener;
        }

        static remove(animator, stateName, listener) {
            var controller = animator.getControllerLayer(0);
            var animState = controller.getAnimatorState(stateName);
            if (animState == null) {
                return;
            }
            var listener = animState.getScript(AnimStateListener);
            if (listener == null) {
                return;
            }
            listener.stateName = null;
            listener.animator = null;
            listener.listener = null;
        }

        onStateEnter() {
            if (this.listener && this.listener.onAnimStateEnter) {
                this.listener.onAnimStateEnter(this.stateName, this.animator);
            }
        }

		onStateUpdate() {
            if (this.listener && this.listener.onAnimStateUpdate) {
                this.listener.onAnimStateUpdate(this.stateName, this.animator);
            }
        }

        onStateExit() {
            if (this.listener && this.listener.onAnimStateExit) {
                this.listener.onAnimStateExit(this.stateName, this.animator);
            }
        }
    }

    class TextureCamera extends Laya.Script3D {

        constructor() {
            super();
            
            this.mRenderCamera = null; //: Laya.Camera;
            this.mRenderModelSlot = null; //: Laya.Sprite3D;
            this.mAnimator = null; //: Laya.Animator;
            this.mModel = null; //: Laya.Sprite3D;
        
            this.mRenderTexture = null; //: Laya.RenderTexture;
            this.mPrefab = null; //: string;
            this.mAnimState = null; //: string;
        
            this.mRenderLayer = 5; //: number = 5;
            this.mIdx = 0;
        }
    
        onAwake() {
            this.mIdx = TextureCamera.id;
            TextureCamera.id ++;
            this.mRenderCamera = this.owner.getChildByName("renderCamera");
            this.mRenderModelSlot = this.owner.getChildByName("renderModelSlot");
            this.mRenderCamera.active = true;
            this.mRenderModelSlot.active = true;
            this.mRenderTexture = new Laya.RenderTexture(TextureCamera.width, TextureCamera.height, Laya.RenderTexture.FORMAT_R8G8B8A8);
            this.mRenderCamera.renderTarget = this.mRenderTexture;
            this.mRenderCamera.renderingOrder = -1;
            this.mRenderCamera.cullingMask = 0;
            this.mRenderCamera.cullingMask = (1 << this.mRenderLayer);
            this.mRenderCamera.clearFlag = Laya.BaseCamera.CLEARFLAG_SOLIDCOLOR;
            this.mRenderCamera.clearColor = new Laya.Vector4(0, 0, 0, 0);
            if (this.mIdx == 0) {
                TextureCamera.RenderTexture = this.mRenderTexture;
            }
            TextureCamera.RenderTextureArr.push(this.mRenderTexture);
            Laya.stage.on(Event.TEXTURE_CAMERA_INIT, this, this.onInitTextureCamera);
            Laya.stage.on(Event.TEXTURE_CAMERA_CLEAR, this, this.onClearTextureCamera);
        }

        onDestroy() {
            Laya.stage.off(Event.TEXTURE_CAMERA_INIT, this, this.onInitTextureCamera);
            Laya.stage.off(Event.TEXTURE_CAMERA_CLEAR, this, this.onClearTextureCamera);
        }

        onInitTextureCamera(subPkg, prefab, animation, id) {
            if (!id && this.mIdx == 0) {
                this.renderModel(subPkg, prefab, animation);
            }
            else if (id == this.mIdx) {
                this.renderModel(subPkg, prefab, animation);
            }
        }
    
        onClearTextureCamera(id) {
            if (!id && this.mIdx == 0) {
                this.clearModel();
            }
            else if (id == this.mIdx) {
                this.clearModel();
            }
        }
    
        renderModel(subPkg, prefab, animation) {
            if (prefab == null || prefab.length == 0) {
                this.onClearTextureCamera();
                return ;
            }
            if (prefab == this.mPrefab) {
                return ;
            }
            this.playAnimation(animation);
            this.mPrefab = prefab;
            Resource.loadSprite3dAsyn(prefab, Laya.Handler.create(this, this.onModelReady, [prefab]), null, subPkg);
        }
    
        clearModel() {
            this.mAnimState = null;
            this.mAnimator = null;
            this.mPrefab = null;
            if (this.mModel) {
                this.mModel.destroy();
            }
            this.mModel = null;
        }
    
        playAnimation(animation) {
            this.mAnimState = animation;
            if (this.mAnimator && this.mAnimState) {
                this.mAnimator.play(this.mAnimState);
            }
        }
    
        onModelReady(prefab, sprite3d) {
            if (this.mPrefab != prefab) {
                return ;
            }
            var animtion = this.mAnimState;
            this.clearModel();
            this.mModel = ObjectPool.getSprite3d(sprite3d, prefab);
            this.mRenderModelSlot.addChild(this.mModel);
            this.mModel.transform.localPosition.setValue(0, 0, 0);
            this.mModel.transform.localPosition = this.mModel.transform.localPosition;
            this.mModel.transform.localRotationEuler.setValue(0, 0, 0);
            this.mModel.transform.localRotationEuler = this.mModel.transform.localRotationEuler;
            this.updateLayer(this.mModel);
            this.mAnimState = animtion;
            if (this.mAnimState) {
                this.mAnimator = this.findAnimator(this.mModel);
            }
            this.playAnimation(this.mAnimState);
        }
    
        updateLayer(model) {
            model.layer = this.mRenderLayer;
            for (let index = 0; index < model.numChildren; index++) {
                const child = model.getChildAt(index);
                this.updateLayer(child);
            }
        }
    
        findAnimator(model) {
            var animator = model.getComponent(Laya.Animator);
            if (animator) {
                return animator;
            }
            
            for (let index = 0; index < model.numChildren; index++) {
                const child = model.getChildAt(index);
                var animator = this.findAnimator(child);
                if (animator != null) {
                    return animator;
                }
            }
            return null;
        }
    
    }
    TextureCamera.RenderTexture = null;
    TextureCamera.width = 512;
    TextureCamera.height = 512;
    TextureCamera.id = 0;
    TextureCamera.RenderTextureArr = [];

    var SceneLoopMode;
    (function (SceneLoopMode) {
        SceneLoopMode[SceneLoopMode["Static"] = 0] = "Static";
        SceneLoopMode[SceneLoopMode["Dynamic"] = 1] = "Dynamic";
    })(SceneLoopMode || (SceneLoopMode = {}));

    class LoopScene extends Laya.Script3D {

        constructor() {
            super();
            this.mSubScenes = [];
            this.mLoopMode = SceneLoopMode.Static;
            this.mInterval = 0;
            this.mIncDist = 0;
            this.mInitialized = false;
            this.mLoopThreshold = 0;
            this.mPassedDist = 0;
        }
    
        setLoopMode(mode) {
            this.mLoopMode = mode;
        }
    
        setLoopThreshold(threshold) {
            this.mLoopThreshold = threshold;
        }
    
        reset() {
            this.mPassedDist = 0;
            for (let index = 0; index < this.mSubScenes.length; index++) {
                const element = this.mSubScenes[index];
                element.transform.localPositionZ = (index - 1) * this.mInterval;
            }
        }
    
        updateScene(passedDist) {
            if (!this.mInitialized || !passedDist) {
                return ;
            }
            if (this.mLoopMode == SceneLoopMode.Static) {
                if (passedDist > 0) {
                    for (let index = this.mSubScenes.length - 1; index >= 0; index--) {
                        const element = this.mSubScenes[index];
                        if (element.transform.localPositionZ - passedDist < -this.mLoopThreshold) {
                            element.transform.localPositionZ += this.mIncDist - passedDist;
                            this.mSubScenes.splice(index, 1);
                            this.mSubScenes.push(element);
                        }
                        else {
                            element.transform.localPositionZ -= passedDist;
                        }
                    }
                }
                else {
                    for (let index = 0; index < this.mSubScenes.length; index++) {
                        const element = this.mSubScenes[index];
                        if (element.transform.localPositionZ - passedDist > this.mIncDist - this.mLoopThreshold) {
                            element.transform.localPositionZ -= this.mIncDist + passedDist;
                            this.mSubScenes.splice(index, 1);
                            this.mSubScenes.unshift(element);
                        }
                        else {
                            element.transform.localPositionZ -= passedDist;
                        }
                    }
                }
            }
            else {
                this.mPassedDist += passedDist;
                if (passedDist > 0) {
                    for (let index = this.mSubScenes.length - 1; index >= 0; index--) {
                        const element = this.mSubScenes[index];
                        if (this.mPassedDist - element.transform.localPositionZ > this.mLoopThreshold) {
                            element.transform.localPositionZ += this.mIncDist;
                            this.mSubScenes.splice(index, 1);
                            this.mSubScenes.push(element);
                        }
                    }
                }
                else {
                    for (let index = 0; index < this.mSubScenes.length; index++) {
                        const element = this.mSubScenes[index];
                        if (element.transform.localPositionZ - this.mPassedDist > this.mIncDist - this.mLoopThreshold) {
                            element.transform.localPositionZ -= this.mIncDist;
                            this.mSubScenes.splice(index, 1);
                            this.mSubScenes.unshift(element);
                        }
                    }
                }
            }
        }
    
        onAwake() {
            for (let index = 0; index < this.owner.numChildren; index++) {
                const element = this.owner.getChildAt(index);
                this.mSubScenes.push(element);
            }
            if (this.mSubScenes.length <= 2) {
                console.warn("LoopScene:", "sub scenes num less or equal 2");
                return ;
            }
            this.mInitialized = true;
            this.mInterval = this.mSubScenes[1].transform.localPositionZ - this.mSubScenes[0].transform.localPositionZ;
            this.mIncDist = (this.mSubScenes.length - 1) * this.mInterval;
            this.mLoopThreshold = this.mInterval;
        }
    
        onDestroy() {
    
        }
    }

    class LoopBoard extends Laya.Script3D {

        constructor() {
            super();
            this.mMaterial = null;
            this.mScale = 1;
        }
    
        reset() {
            if (this.mMaterial) {
                this.mMaterial.tilingOffsetZ = 0;
            }
        }
    
        setMoveScale(scale) {
            if (!scale) {
                return ;
            }
            this.mScale = scale;
        }
    
        updateBoard(passedDist) {
            if (this.mMaterial) {
                this.mMaterial.tilingOffsetZ += passedDist * this.mScale;
            }
        }
    
        onAwake() {
            this.mMaterial = this.owner.meshRenderer.sharedMaterial;
        }
    
        onDestroy() {
            this.mMaterial = null;
        }
    }

    class BannerView extends Laya.Script {

        constructor () {
            super();
            this.config = null;
            this.otherViews = [];
            this.passedTime = 0;
            this.uiReady = false;
            this.uiVisible = true;
            this.uiHasShow = false;
            
            this.showNativeHandler = null;
            this.hideNativeHandler = null;
        }

        onStart() {     
            if (!this.owner.viewId) {
                return ;
            }          
            this.config = BannerCtrl.bannerConfig[this.owner.viewId];
            this.uiReady = this.config && !this.config.delay;
            this.uiVisible = this.config && this.config.visible != false;

            if (!this.config) {
                return ;
            }

            var userSetting = BannerCtrl.userSetting[this.owner.viewId];
            if (userSetting) {
                this.uiVisible = this.uiVisible && !!userSetting.visible;
                BannerCtrl.userSetting[this.owner.viewId] = null;
            }
            
            if (BannerCtrl.bannerRefreshInterval) {
                var now = Date.now();
                if (now - BannerCtrl.bannerCreateTime >= BannerCtrl.bannerRefreshInterval) {
                    Channel.destroyBannerAd();
                    Channel.createBannerAd();
                    BannerCtrl.bannerCreateTime = now;
                }
            }
            else {
                Channel.destroyBannerAd();
                Channel.createBannerAd();
            }
        }

        onEnable() { 
            Laya.stage.on(Event.UI_VIEW_CLOSED, this, this.onViewClosed);
            Laya.stage.on(Event.UI_VIEW_OPENED, this, this.onViewOpened);   
            Laya.stage.on(BannerCtrl.EVENT_SHOW_BANNER, this, this.onShowBanner); 
            Laya.stage.on(BannerCtrl.EVENT_HIDE_BANNER, this, this.onHideBanner); 
            this.showBanner();
        }

        onDisable() {     
            Laya.stage.off(Event.UI_VIEW_CLOSED, this, this.onViewClosed);
            Laya.stage.off(Event.UI_VIEW_OPENED, this, this.onViewOpened);
            Laya.stage.off(BannerCtrl.EVENT_SHOW_BANNER, this, this.onShowBanner); 
            Laya.stage.off(BannerCtrl.EVENT_HIDE_BANNER, this, this.onHideBanner);
            this.hideBanner();
        }

        onDestroy() {
            Laya.stage.off(Event.UI_VIEW_CLOSED, this, this.onViewClosed);
            Laya.stage.off(Event.UI_VIEW_OPENED, this, this.onViewOpened);
            Laya.stage.off(BannerCtrl.EVENT_SHOW_BANNER, this, this.onShowBanner); 
            Laya.stage.off(BannerCtrl.EVENT_HIDE_BANNER, this, this.onHideBanner);
            this.hideBanner();
        }

        onUpdate() {  
            if (!this.config || !this.owner.viewId) {
                return ;
            }        
            if (this.uiReady) {
                return ;
            }
            if (!this.config.delay) {
                this.uiReady = true;
                this.showBanner();
            }
            else if (this.config.delay >= 0) {
                this.passedTime += Laya.timer.delta;
                if (this.passedTime > this.config.delay) {
                    this.uiReady = true;
                    this.showBanner();
                }
            }
        }

        onViewOpened(viewName, viewId, view) {
            if (view.zOrder < this.owner.zOrder) {
                return ;
            }
            this.otherViews.push(viewId);
            this.hideBanner();
        }

        onViewClosed(viewName, viewId) {
            var idx = this.otherViews.indexOf(viewId);
            if (idx < 0) {
                return ;
            }
            this.otherViews.splice(idx, 1);
            this.showBanner();
        }

        onShowBanner(viewId) {
            if (viewId != this.owner.viewId) {
                return ;
            }
            this.uiVisible = true;
            this.showBanner();
        }

        onHideBanner(viewId) {
            if (viewId != this.owner.viewId) {
                return ;
            }
            this.uiVisible = false;
            this.hideBanner();
        }

        showBanner() {
            if (this.otherViews.length == 0 && this.uiVisible && this.uiHasShow == false && this.uiReady) {
                this.uiHasShow = true;
                Channel.showBannerAd();
            }
        }

        hideBanner() {
            if (this.uiHasShow) {
                Channel.hideBannerAd();
                this.uiHasShow = false;
            }
        }
    }

    class BannerCtrl extends Laya.Script {

        static showBanner(viewId) {
            this.userSetting[viewId] = this.userSetting[viewId] || {};
            this.userSetting[viewId].visible = true;
            Laya.stage.event(BannerCtrl.EVENT_SHOW_BANNER, viewId);
        }

        static hideBanner(viewId) {
            if (!this.bannerConfig) {
                return ;
            }
            this.userSetting[viewId] = this.bannerConfig[viewId] || {};
            this.userSetting[viewId].visible = false;
            Laya.stage.event(BannerCtrl.EVENT_HIDE_BANNER, viewId);
        }

        constructor () {
            super();
        }

        onAwake() {     
            Laya.stage.once(Event.LAUNCH_COMPLETED, this, this.onLaunchResReady);
            Laya.stage.on(Event.UI_VIEW_OPENED, this, this.onViewOpened);
        }

        onDestroy() {
            Laya.stage.off(Event.UI_VIEW_OPENED, this, this.onViewOpened);
        }

        onLaunchResReady() {
            var pluginCfg = GameFrame.pluginCfg;
            if (pluginCfg) {
                if (pluginCfg.bannerConfig) {
                    BannerCtrl.bannerConfig = pluginCfg.bannerConfig;
                }
                BannerCtrl.bannerRefreshInterval = pluginCfg.pg_banner_refresh_time;
            }
        }

        onViewOpened(viewName, viewId, view) {
            view.addComponent(BannerView);
        }
    }
    BannerCtrl.EVENT_SHOW_BANNER = "BANNERCTRL_EVENT_SHOW_BANNER";
    BannerCtrl.EVENT_HIDE_BANNER = "BANNERCTRL_EVENT_HIDE_BANNER";
    BannerCtrl.userSetting = {};
    BannerCtrl.bannerConfig = {};
    BannerCtrl.bannerCreateTime = 0;
    BannerCtrl.bannerRefreshInterval = 30000;
    Laya.ClassUtils.regClass("fc.plugin.BannerCtrl", BannerCtrl);
    
    class BlockAdView extends Laya.Script {

        constructor () {
            super();
            this.config = null;
            this.otherViews = [];
            this.passedTime = 0;
            this.uiReady = false;
            this.uiVisible = true;
            this.uiHasShow = false;
            
            this.showNativeHandler = null;
            this.hideNativeHandler = null;
        }

        onStart() {     
            if (!this.owner.viewId) {
                return ;
            }          
            this.config = BlockAdCtrl.blockConfig[this.owner.viewId];
            this.uiReady = this.config && !this.config.delay;
            this.uiVisible = true;

            if (!this.config) {
                return ;
            }

            var userSetting = BlockAdCtrl.userSetting[this.owner.viewId];
            if (userSetting) {
                this.uiVisible = !!userSetting.visible;
                BlockAdCtrl.userSetting[this.owner.viewId] = null;
            }

            Channel.createGameBanner(null, this.config);
        }

        onEnable() { 
            Laya.stage.on(Event.UI_VIEW_CLOSED, this, this.onViewClosed);
            Laya.stage.on(Event.UI_VIEW_OPENED, this, this.onViewOpened);   
            Laya.stage.on(BlockAdCtrl.EVENT_SHOW_BLOCK, this, this.onShowBlock); 
            Laya.stage.on(BlockAdCtrl.EVENT_HIDE_BLOCK, this, this.onHideBlock); 
            this.showBlock();
        }

        onDisable() {     
            Laya.stage.off(Event.UI_VIEW_CLOSED, this, this.onViewClosed);
            Laya.stage.off(Event.UI_VIEW_OPENED, this, this.onViewOpened);
            Laya.stage.off(BlockAdCtrl.EVENT_SHOW_BLOCK, this, this.onShowBlock); 
            Laya.stage.off(BlockAdCtrl.EVENT_HIDE_BLOCK, this, this.onHideBlock);
            this.hideBlock();
        }

        onDestroy() {     
            Laya.stage.off(Event.UI_VIEW_CLOSED, this, this.onViewClosed);
            Laya.stage.off(Event.UI_VIEW_OPENED, this, this.onViewOpened);
            Laya.stage.off(BlockAdCtrl.EVENT_SHOW_BLOCK, this, this.onShowBlock); 
            Laya.stage.off(BlockAdCtrl.EVENT_HIDE_BLOCK, this, this.onHideBlock);
            this.hideBlock();
        }

        onUpdate() {  
            if (!this.config || !this.owner.viewId) {
                return ;
            }        
            if (this.uiReady) {
                return ;
            }
            if (!this.config.delay) {
                this.uiReady = true;
                this.showBlock();
            }
            else if (this.config.delay >= 0) {
                this.passedTime += Laya.timer.delta;
                if (this.passedTime > this.config.delay) {
                    this.uiReady = true;
                    this.showBlock();
                }
            }
        }

        onViewOpened(viewName, viewId, view) {
            if (view.zOrder < this.owner.zOrder) {
                return ;
            }
            this.otherViews.push(viewId);
            this.hideBlock();
        }

        onViewClosed(viewName, viewId) {
            var idx = this.otherViews.indexOf(viewId);
            if (idx < 0) {
                return ;
            }
            this.otherViews.splice(idx, 1);
            this.showBlock();
        }

        onShowBlock(viewId) {
            if (viewId != this.owner.viewId) {
                return ;
            }
            this.uiVisible = true;
            this.showBlock();
        }

        onHideBlock(viewId) {
            if (viewId != this.owner.viewId) {
                return ;
            }
            this.uiVisible = false;
            this.hideBlock();
        }

        showBlock() {
            if (this.otherViews.length == 0 && this.uiVisible && this.uiHasShow == false && this.uiReady) {
                this.uiHasShow = true;
                Channel.showGameBanner();
            }
        }

        hideBlock() {
            if (this.uiHasShow) {
                Channel.hideGameBanner();
                this.uiHasShow = false;
            }
        }
    }

    class BlockAdCtrl extends Laya.Script {

        static showBlock(viewId) {
            this.userSetting[viewId] = this.userSetting[viewId] || {};
            this.userSetting[viewId].visible = true;
            Laya.stage.event(BlockAdCtrl.EVENT_SHOW_BLOCK, viewId);
        }

        static hideBlock(viewId) {
            if (!this.blockConfig) {
                return ;
            }
            this.userSetting[viewId] = this.blockConfig[viewId] || {};
            this.userSetting[viewId].visible = false;
            Laya.stage.event(BlockAdCtrl.EVENT_HIDE_BLOCK, viewId);
        }

        constructor () {
            super();
        }

        onAwake() {     
            Laya.stage.once(Event.LAUNCH_COMPLETED, this, this.onLaunchResReady);
            Laya.stage.on(Event.UI_VIEW_OPENED, this, this.onViewOpened);
        }

        onDestroy() {
            Laya.stage.off(Event.UI_VIEW_OPENED, this, this.onViewOpened);
        }

        onLaunchResReady() {
            var pluginCfg = GameFrame.pluginCfg;
            if (pluginCfg && pluginCfg.blockConfig) {
                BlockAdCtrl.blockConfig = pluginCfg.blockConfig;
            }
        }

        onViewOpened(viewName, viewId, view) {
            view.addComponent(BlockAdView);
        }
    }
    BlockAdCtrl.EVENT_SHOW_BLOCK = "BLOCKCTRL_EVENT_SHOW_BLOCK";
    BlockAdCtrl.EVENT_HIDE_BLOCK = "BLOCKCTRL_EVENT_HIDE_BLOCK";
    BlockAdCtrl.userSetting = {};
    BlockAdCtrl.blockConfig = {};
    Laya.ClassUtils.regClass("fc.plugin.BlockAdCtrl", BlockAdCtrl);

    
    class AdditiveUIView extends Laya.Script {

        constructor() {
            super();
            this.config = null;
            this.adView = null;
            this.monitorOtherPageOpen = false;
            this.visibleArr = null;
        }

        onEnable() {
        }

        onDisable() {
            if (this.monitorOtherPageOpen) {
                Laya.stage.off(Event.UI_VIEW_OPENED, this, this.onViewOpened);
                Laya.stage.off(Event.UI_VIEW_CLOSED, this, this.onViewClosed);
            }
        }

        onDestroy() {
            if (this.adView == null) {
                return;
            }
            for (var index = 0; index < this.adView.length; index++) {
                if (this.adView[index] != null) {
                    if (this.config[index].scriptType) {
                        var scriptType = Laya.ClassUtils.getClass(this.config[index].scriptType);
                        var script = this.adView[index].getComponent(scriptType);
                        if (script && script.releaseView) {
                            script.releaseView();
                        }
                    }
                    this.adView[index].destroy();
                }
            }
            this.adView = null;
        }

        onStart() {
            if (this.adView || AdditiveUICtrl.additiveUIConfig == null || !this.owner.viewId) {
                return;
            }

            this.config = AdditiveUICtrl.additiveUIConfig[this.owner.viewId];
            if (!this.config) {
                return;
            }

            this.monitorOtherPageOpen = false;
            for (var index = 0; index < this.config.length; index++) {
                var element = this.config[index];
                if (!element.active || element.active()) {
                    this.monitorOtherPageOpen = this.monitorOtherPageOpen || element.isHide;
                }
            }
            if (this.monitorOtherPageOpen) {
                Laya.stage.on(Event.UI_VIEW_OPENED, this, this.onViewOpened);
                Laya.stage.on(Event.UI_VIEW_CLOSED, this, this.onViewClosed);
            }

            this.adView = [];
            for (var index = 0; index < this.config.length; index++) {
                var element = this.config[index];
                if (!element.parentRoot || !this.owner.getChildByName(element.parentRoot)) {
                    console.warn(this.owner.viewId, "invalid additiveUIConfig.parentRoot", index);
                    this.adView.push(null);
                }
                else if (!element.viewUrl || element.viewUrl.length == 0) {
                    console.warn(this.owner.viewId, "invalid additiveUIConfig.prefab", index);
                    this.adView.push(null);
                }
                else if (element.scriptType && !Laya.ClassUtils.getClass(element.scriptType)) {
                    console.warn(this.owner.viewId, "invalid additiveUIConfig.scriptType", index);
                    this.adView.push(null);
                }
                else if (!element.checkCode || !AdditiveUICtrl.checkFun || AdditiveUICtrl.checkFun(element.checkCode)) {
                    Laya.loader.create(element.viewUrl, Laya.Handler.create(this, this.onPrefabReady, [this.owner.viewId]), null, Laya.Loader.PREFAB);
                    break;
                }
                else {
                    console.warn(this.owner.viewId, "invalid additiveUIConfig", index);
                    this.adView.push(null);
                }
            }
        }

        onPrefabReady(viewId, prefab) {
            if (this.destroyed) {
                return;
            }
            var params = this.config[this.adView.length];
            var view = prefab.create();
            this.owner.getChildByName(params.parentRoot).addChild(view);
            view.pos(params.x, params.y);
            // view.visible = this.owner.visible;
            if (params.scriptType) {
                var scriptType = Laya.ClassUtils.getClass(params.scriptType);
                var script = view.getComponent(scriptType);
                if (script == null) {
                    script = view.addComponent(scriptType);
                }
                if (script.initView) {
                    script.initView(params);
                }
            }

            this.adView.push(view);
            
            for (var index = this.adView.length; index < this.config.length; index++) {
                var element = this.config[index];
                if (!element.parentRoot || !this.owner.getChildByName(element.parentRoot)) {
                    console.warn(viewId, "invalid additiveUIConfig.parentRoot", index);
                    this.adView.push(null);
                }
                else if (!element.viewUrl || element.viewUrl.length == 0) {
                    console.warn(viewId, "invalid additiveUIConfig.prefab", index);
                    this.adView.push(null);
                }
                else if (element.scriptType && !Laya.ClassUtils.getClass(element.scriptType)) {
                    console.warn(viewId, "invalid additiveUIConfig.scriptType", index);
                    this.adView.push(null);
                }
                else if (!element.checkCode || !AdditiveUICtrl.checkFun || AdditiveUICtrl.checkFun(element.checkCode)) {
                    Laya.loader.create(element.viewUrl, Laya.Handler.create(this, this.onPrefabReady, [viewId]), null, Laya.Loader.PREFAB);
                    break;
                }
                else {
                    console.warn(viewId, "invalid additiveUIConfig", index);
                    this.adView.push(null);
                }
            }

            if (this.adView.length == this.config.length) {
                this.owner.timerOnce(100, this, function() {
                    this.owner.event(AdditiveUICtrl.ADDITIVE_UI_LOAD_COMPLETED);
                })
            }
        }

        onViewOpened(viewName) {
            if (viewName && this.adView) {
                this.visibleArr = [];
                for (var index = 0; index < this.adView.length; index++) {
                    if (this.adView[index] != null && this.config[index].isHide) {
                        this.visibleArr[index] = this.adView[index].visible;
                        this.adView[index].visible = false;
                    }
                }
            }
        }

        onViewClosed(viewName) {
            if (viewName && this.adView) {
                if (!this.visibleArr) {
                    return;
                }
                for (var index = 0; index < this.adView.length; index++) {
                    if (this.adView[index] != null && this.config[index].isHide) {
                        if (this.visibleArr[index]) {
                            this.adView[index].visible = this.visibleArr[index];
                        }
                    }
                }
            }
        }
    }

    class AdditiveUICtrl extends Laya.Script {

        constructor() {
            super();
        }

        onAwake() {
            Laya.stage.once(Event.LAUNCH_COMPLETED, this, this.onLaunchResReady);
            Laya.stage.on(Event.UI_VIEW_OPENED, this, this.onViewOpened);
        }

        onDestroy() {
            Laya.stage.off(Event.UI_VIEW_OPENED, this, this.onViewOpened);
        }

        onLaunchResReady() {
            var pluginCfg = GameFrame.pluginCfg;
            if (pluginCfg.additiveUIConfig) {
                var config = AdditiveUICtrl.additiveUIConfig || {};
                var keys = Object.keys(pluginCfg.additiveUIConfig);
                for (var index = 0; index < keys.length; index++) {
                    config[keys[index]] = pluginCfg.additiveUIConfig[keys[index]];
                }
                AdditiveUICtrl.additiveUIConfig = config;
            }
        }

        onViewOpened(viewName, viewId, view) {
            view.addComponent(AdditiveUIView);
        }
    }
    AdditiveUICtrl.ADDITIVE_UI_LOAD_COMPLETED = "ADDITIVE_UI_LOAD_COMPLETED";
    AdditiveUICtrl.additiveUIConfig = null;
    AdditiveUICtrl.checkFun = null;
    Laya.ClassUtils.regClass("fc.plugin.AdditiveUICtrl", AdditiveUICtrl);

    class UIJoystickView extends Laya.Script {

        constructor() {
            super();
            this.selectedItem = null;
            this.comps = null;
            this.layerIdx = 0;
            this.compIdx = 0;
            this.layerInfo = null;
            this.backComp = null;
            this.backView = null;
            this.confirmComp = null;
            this.confirmView = null;
            this.otherViews = [];
            this.guideFinger = null;
            this.localPoint = new Laya.Point();
            this.listIdx = 0;
            this.openTime = 0;
            this.lastClick = 0;
            this.jumpNum = 0;
        }

        onAwake() {
            this.openTime = Date.now();
            this.lastClick = Date.now();
        }

        onStart() {     
            if (!this.owner.viewId) {
                return ;
            }          
            var uiConfig = UIJoystickCtrl.viewConfig[this.owner.viewId];
            if (uiConfig == null) {
                return ;
            }
            this.compIdx = uiConfig.defaultCompIdx ? uiConfig.defaultCompIdx : 0;
            this.layerIdx = uiConfig.defaultLayerIdx ? uiConfig.defaultLayerIdx : 0;
            this.backComp = uiConfig.backComp;
            this.backView = uiConfig.backView;
            this.confirmComp = uiConfig.confirmComp;
            this.confirmView = uiConfig.confirmView;
            if (uiConfig.comps && uiConfig.comps.length > 0 && uiConfig.comps[0].length > 0) {
                this.comps = uiConfig.comps;
                if (this.layerIdx >= this.comps.length) {
                    this.layerIdx = 0;
                }
                if (this.compIdx >= this.comps[this.layerIdx].length) {
                    this.compIdx = 0;
                }
                this.updateFocus(this.findItem(this.comps[this.layerIdx][this.compIdx].viewPath));
                this.owner.once(AdditiveUICtrl.ADDITIVE_UI_LOAD_COMPLETED, this, this.onAdditiveUIReady);
                Laya.loader.create(UIJoystickCtrl.guideFinger, Laya.Handler.create(this, this.onPrefabReady), null, Laya.Loader.PREFAB);
            }
            else {
                this.comps = null;
            }
            Laya.stage.on(Event.JOYSTICK_KEY_DOWN, this, this.onJoystickKeyUp);
        }

        onEnable() { 
            Laya.stage.on(Event.UI_VIEW_CLOSED, this, this.onViewClosed);
            Laya.stage.on(Event.UI_VIEW_OPENED, this, this.onViewOpened);   
        }

        onDisable() {     
            Laya.stage.off(Event.UI_VIEW_CLOSED, this, this.onViewClosed);
            Laya.stage.off(Event.UI_VIEW_OPENED, this, this.onViewOpened);
        }

        onDestroy() {     
            Laya.stage.off(Event.UI_VIEW_CLOSED, this, this.onViewClosed);
            Laya.stage.off(Event.UI_VIEW_OPENED, this, this.onViewOpened);
            Laya.stage.off(Event.JOYSTICK_KEY_DOWN, this, this.onJoystickKeyUp);
        }

        onPrefabReady(prefab) {
            if (this.destroyed) {
                return;
            }
            this.guideFinger = prefab.create();
            this.owner.addChild(this.guideFinger);
            this.updateFocus(this.findItem(this.comps[this.layerIdx][this.compIdx].viewPath));
            this.guideFinger.visible = this.otherViews.length == 0;
        }

        onAdditiveUIReady() {
            this.updateFocus(this.findItem(this.comps[this.layerIdx][this.compIdx].viewPath));
        }

        onViewOpened(viewName, viewId, view) {
            if (view.zOrder < this.owner.zOrder) {
                return ;
            }
            this.otherViews.push(viewId);
            if (this.guideFinger) {
                this.guideFinger.visible = false;
            }
        }

        onViewClosed(viewName, viewId) {
            var idx = this.otherViews.indexOf(viewId);
            if (idx < 0) {
                return ;
            }
            this.otherViews.splice(idx, 1);
            if (this.otherViews.length == 0 && this.guideFinger) {
                this.guideFinger.visible = true;
                this.freshFocus();
            }
        }

        onJoystickKeyUp(keyCode, isClick) {
            if(!Laya.MouseManager.enabled)return;
            if (isClick == false) {
                return ;
            }
            if (this.otherViews.length > 0 || keyCode == JoystickCode.UNDEFINE) {
                return ;
            }

            switch (keyCode) {
                case JoystickCode.ESCAPE:
                    this.doneBack();
                    break;
                case JoystickCode.OK:
                    this.confirmSelected();
                    break;
                case JoystickCode.LEFT:
                    this.selectLeftComp();
                    break;
                case JoystickCode.RIGHT:
                    this.selectRightComp();
                    break;
                case JoystickCode.UP:
                    this.selectPreLayer();
                    break;
                case JoystickCode.DOWN:
                    this.selectNextLayer();
                    break;
            }
        }

        doneBack() {
            if (Date.now() - this.openTime < 400) {
                console.debug("click too quickly");
                return ;
            }
            if (Date.now() - this.lastClick < 400) {
                console.debug("click too quickly");
                return ;
            }
            this.lastClick = Date.now();
            // if (this.comps) {
            //     var compInfo = this.comps[this.layerIdx][this.compIdx];
            //     var selectedItem = this.findItem(compInfo.clickPath?compInfo.clickPath:compInfo.viewPath);
            //     if (selectedItem instanceof Laya.List) {
            //         this.selectPreLayer(true);
            //         return ;
            //     }
            // }
            if (this.backComp) {
                var backComp = this.findItem(this.backComp);
                if (backComp) {
                    backComp.event(Laya.Event.CLICK);
                    return ;
                }
            }
            
            if (this.backView) {
                GameFrame.openView(this.backView);
                return ;
            }

            GameFrame.closeView(this.owner);
        }

        confirmSelected() {
            if (Date.now() - this.lastClick < 400) {
                console.debug("click too quickly");
                return ;
            }
            this.lastClick = Date.now();
            if (this.comps) {
                var compInfo = this.comps[this.layerIdx][this.compIdx];
                var selectedItem = this.findItem(compInfo.clickPath?compInfo.clickPath:compInfo.viewPath);
                if (selectedItem instanceof Laya.List) {
                    selectedItem.selectedIndex = -1;
                    selectedItem.selectedIndex = this.listIdx;
                    this.selectNextLayer(true);
                    return ;
                }
                if (selectedItem) {
                    selectedItem.event(Laya.Event.CLICK, {type: Laya.Event.CLICK});
                }
                else {
                    console.log("selectedItem is undefine");
                }
                return ;
            }

            if (this.confirmComp) {
                var confirmComp = this.findItem(this.confirmComp);
                if (confirmComp) {
                    confirmComp.event(Laya.Event.CLICK);
                    return ;
                }
            }
            
            if (this.confirmView) {
                GameFrame.openView(this.confirmView);
                return ;
            }
        }

        confirmSelectListItem() {
            if (Date.now() - this.lastClick < 400) {
                console.debug("click too quickly");
                return ;
            }
            this.lastClick = Date.now();
            if (this.comps) {
                var compInfo = this.comps[this.layerIdx][this.compIdx];
                var selectedItem = this.findItem(compInfo.clickPath?compInfo.clickPath:compInfo.viewPath);
                if (selectedItem instanceof Laya.List) {
                    selectedItem.selectedIndex = -1;
                    selectedItem.selectedIndex = this.listIdx;
                    this.selectNextLayer(true);
                    return ;
                }
                if (selectedItem) {
                    selectedItem.event(Laya.Event.CLICK, {type: Laya.Event.CLICK});
                }
                else {
                    console.log("selectedItem is undefine");
                }
                return ;
            }

            if (this.confirmComp) {
                var confirmComp = this.findItem(this.confirmComp);
                if (confirmComp) {
                    confirmComp.event(Laya.Event.CLICK);
                    return ;
                }
            }
            
            if (this.confirmView) {
                GameFrame.openView(this.confirmView);
                return ;
            }
        }

        selectLeftComp() {
            if (!this.comps) {
                return ;
            }
            var curItem = this.findItem(this.comps[this.layerIdx][this.compIdx].viewPath);
            if (curItem instanceof Laya.List) {
                if (this.listIdx - 1 >= 0){
                    this.listIdx--;
                    curItem.scrollTo(this.listIdx);
                    this.updateFocus(curItem.getCell(this.listIdx));
                    curItem.selectedIndex = this.listIdx;
                    return ;
                }
            }

            var newCompIdx = this.compIdx - 1;
            if (newCompIdx < 0) {
                newCompIdx = this.comps[this.layerIdx].length - 1;
            }
            var newItem = this.findItem(this.comps[this.layerIdx][newCompIdx].viewPath);
            if (newItem.visible == false) {
                if (this.jumpNum > this.comps[this.layerIdx].length) {
                    return ;
                }
                this.jumpNum ++;
                this.compIdx = newCompIdx;
                this.selectLeftComp();
                return ;
            }
            this.compIdx = newCompIdx;

            if (newItem instanceof Laya.List) {
                this.listIdx = newItem.selectedIndex;
                if (this.listIdx == -1) {
                    this.listIdx = 0;
                }
                this.updateFocus(newItem.getCell(this.listIdx));
                newItem.selectedIndex = this.listIdx;
                return ;
            }
            this.updateFocus(newItem);
        }

        selectRightComp() {
            if (!this.comps) {
                return ;
            }
            var curItem = this.findItem(this.comps[this.layerIdx][this.compIdx].viewPath);
            if (curItem instanceof Laya.List) {
                if (this.listIdx + 1 < curItem.array.length){
                    this.listIdx++;
                    curItem.scrollTo(this.listIdx);
                    this.updateFocus(curItem.getCell(this.listIdx));
                    curItem.selectedIndex = this.listIdx;
                    return ;
                }
            }
            
            var newCompIdx = this.compIdx + 1;
            if (newCompIdx == this.comps[this.layerIdx].length) {
                newCompIdx = 0;
            }
            var newItem = this.findItem(this.comps[this.layerIdx][newCompIdx].viewPath);
            if (newItem.visible == false) {
                if (this.jumpNum > this.comps[this.layerIdx].length) {
                    return ;
                }
                this.jumpNum ++;
                this.compIdx = newCompIdx;
                this.selectRightComp();
                return ;
            }
            this.compIdx = newCompIdx;

            if (newItem instanceof Laya.List) {
                this.listIdx = newItem.selectedIndex;
                if (this.listIdx == -1) {
                    this.listIdx = 0;
                }
                this.updateFocus(newItem.getCell(this.listIdx));
                newItem.selectedIndex = this.listIdx;
                return ;
            }
            this.updateFocus(newItem);
        }

        selectPreLayer(jumpList=false) {
            if (!this.comps) {
                return ;
            }
            if (!jumpList) {
                var curItem = this.findItem(this.comps[this.layerIdx][this.compIdx].viewPath);
                if (curItem instanceof Laya.List) {
                    if (curItem.repeatX && this.listIdx - curItem.repeatX >= 0){
                        this.listIdx-=curItem.repeatX;
                        curItem.scrollTo(this.listIdx);
                        this.updateFocus(curItem.getCell(this.listIdx));
                        curItem.selectedIndex = this.listIdx;
                        return ;
                    }
                }
            }
            if (this.layerIdx == 0) {
                return ;
            }

            var newLayerIdx = this.layerIdx - 1;
            var newCompIdx = 0;
            var newItem = this.findItem(this.comps[newLayerIdx][newCompIdx].viewPath);
            if (newItem.visible == false) {
                this.layerIdx = newLayerIdx;
                this.compIdx = newCompIdx;
                this.selectRightComp();
                return ;
            }
            this.layerIdx = newLayerIdx;
            this.compIdx = newCompIdx;

            if (newItem instanceof Laya.List) {
                this.listIdx = newItem.selectedIndex;
                if (this.listIdx == -1) {
                    this.listIdx = 0;
                }
                this.updateFocus(newItem.getCell(this.listIdx));
                newItem.selectedIndex = this.listIdx;
                return ;
            }
            this.updateFocus(newItem);
        }

        selectNextLayer(jumpList=false) {
            if (!this.comps) {
                return ;
            }
            if (!jumpList) {
                var curItem = this.findItem(this.comps[this.layerIdx][this.compIdx].viewPath);
                if (curItem instanceof Laya.List) {
                    if (curItem.repeatX && this.listIdx + curItem.repeatX < curItem.array.length){
                        this.listIdx+=curItem.repeatX;
                        curItem.scrollTo(this.listIdx);
                        this.updateFocus(curItem.getCell(this.listIdx));
                        curItem.selectedIndex = this.listIdx;
                        return ;
                    }
                }
            }
            if (this.layerIdx == this.comps.length - 1) {
                return ;
            }
            
            var newLayerIdx = this.layerIdx + 1;
            var newCompIdx = 0;
            var newItem = this.findItem(this.comps[newLayerIdx][newCompIdx].viewPath);
            if (newItem.visible == false) {
                this.layerIdx = newLayerIdx;
                this.compIdx = newCompIdx;
                this.selectRightComp();
                return ;
            }
            this.layerIdx = newLayerIdx;
            this.compIdx = newCompIdx;
            
            if (newItem instanceof Laya.List) {
                this.listIdx = newItem.selectedIndex;
                if (this.listIdx == -1) {
                    this.listIdx = 0;
                }
                this.updateFocus(newItem.getCell(this.listIdx));
                newItem.selectedIndex = this.listIdx;
                return ;
            }
            this.updateFocus(newItem);
        }

        freshFocus() {
            var curItem = this.findItem(this.comps[this.layerIdx][this.compIdx].viewPath);
            if (curItem.visible == false) {
                return ;
            }
            
            if (curItem instanceof Laya.List) {
                this.listIdx = curItem.selectedIndex;
                if (this.listIdx == -1) {
                    this.listIdx = 0;
                }
                this.updateFocus(curItem.getCell(this.listIdx));
                curItem.selectedIndex = this.listIdx;
                return ;
            }
            this.updateFocus(curItem);
        }

        updateFocus(comp) {
            if (comp == null || this.guideFinger == null) {
                return ;
            }
            this.localPoint.x = comp.width * 0.75;
            this.localPoint.y = comp.height * 0.75;
            comp.localToGlobal(this.localPoint, false, this.owner);
            this.guideFinger.pos(this.localPoint.x, this.localPoint.y);
            this.guideFinger.ef_finger.stop();
            this.guideFinger.ef_finger.play(0, true);
            comp.event(Event.JOYSTICK_FOCUS_ON, [comp]);
            this.jumpNum = 0;
        }

        findItem(compPath) {
            var layers = compPath.split("/");
            var curUI = this.owner;
            for (let index = 0; index < layers.length; index++) {
                curUI = curUI.getChildByName(layers[index]);
            }
            return curUI;
        }
    }

    class UIJoystickCtrl extends Laya.Script {

        constructor() {
            super();
        }


        onAwake() { 
            fc.framework.Joystick.init();
            Laya.stage.once(Event.LAUNCH_COMPLETED, this, this.onLaunchResReady);
            Laya.stage.on(Event.UI_VIEW_OPENED, this, this.onViewOpened);   
        }

        onDestroy() {     
            Laya.stage.off(Event.UI_VIEW_OPENED, this, this.onViewOpened);
        }

        onViewOpened(viewName, viewId, view) {
            view.addComponent(UIJoystickView);
        }

        onLaunchResReady() {
            var pluginCfg = GameFrame.pluginCfg;
            if (pluginCfg.uiJoystickConfig) {
                var uiJoystickConfig = UIJoystickCtrl.viewConfig;
                var keys = Object.keys(pluginCfg.uiJoystickConfig);
                for (var index = 0; index < keys.length; index++) {
                    uiJoystickConfig[keys[index]] = pluginCfg.uiJoystickConfig[keys[index]];
                }
                UIJoystickCtrl.viewConfig = uiJoystickConfig;
            }
        }
    }
    UIJoystickCtrl.viewConfig = {};
    UIJoystickCtrl.guideFinger = "fc-layout/prefab/guide/ef_finger.json";
    Laya.ClassUtils.regClass("fc.plugin.UIJoystickCtrl", UIJoystickCtrl);

    class GameJoystickCtrl {

        constructor() {
            this.mJoyStickWorking = false;
            this.mJoyStickDirection = new Laya.Vector3();
            this.mJoyStickStatus = 0;
        }

        init() {        
            Laya.stage.on(Event.JOYSTICK_KEY_DOWN, this, this.onJoystickKeyDown);
            Laya.stage.on(Event.JOYSTICK_KEY_UP, this, this.onJoystickKeyUp);
        }
    
        release() {
            Laya.stage.off(Event.JOYSTICK_KEY_DOWN, this, this.onJoystickKeyDown);
            Laya.stage.off(Event.JOYSTICK_KEY_UP, this, this.onJoystickKeyUp);
        }
    
        update() {
            if (fc.framework.GameFrame.isGamePaused) {
                return ;
            }
            if (this.mJoyStickStatus != 0) {
                this.mJoyStickDirection.setValue(0, 0);
                if ((this.mJoyStickStatus & JoystickCode.UP) != 0) {
                    this.mJoyStickDirection.y -= 1;
                }
                if ((this.mJoyStickStatus & JoystickCode.DOWN) != 0) {
                    this.mJoyStickDirection.y += 1;
                }
                if ((this.mJoyStickStatus & JoystickCode.LEFT) != 0) {
                    this.mJoyStickDirection.x -= 1;
                }
                if ((this.mJoyStickStatus & JoystickCode.RIGHT) != 0) {
                    this.mJoyStickDirection.x += 1;
                }
                Laya.stage.event(Event.GAMEPAD_DIRECTION_UPDATE, [this.mJoyStickDirection.x, this.mJoyStickDirection.y, 1]);
            }
            if (this.mJoyStickWorking) {
                return ;
            }
        }
    
        onJoystickKeyDown(btnCode) {
            if (btnCode == JoystickCode.UP || btnCode == JoystickCode.DOWN || btnCode == JoystickCode.LEFT || btnCode == JoystickCode.RIGHT) {
                this.mJoyStickStatus |= btnCode;
                this.mJoyStickDirection.setValue(0, 0);
                if ((this.mJoyStickStatus & JoystickCode.UP) != 0) {
                    this.mJoyStickDirection.y -= 1;
                }
                if ((this.mJoyStickStatus & JoystickCode.DOWN) != 0) {
                    this.mJoyStickDirection.y += 1;
                }
                if ((this.mJoyStickStatus & JoystickCode.LEFT) != 0) {
                    this.mJoyStickDirection.x -= 1;
                }
                if ((this.mJoyStickStatus & JoystickCode.RIGHT) != 0) {
                    this.mJoyStickDirection.x += 1;
                }
                Laya.stage.event(Event.GAMEPAD_DIRECTION_UPDATE, [this.mJoyStickDirection.x, this.mJoyStickDirection.y, 1]);
            }
        }
    
        onJoystickKeyUp(btnCode) {
            if (btnCode == JoystickCode.UP || btnCode == JoystickCode.DOWN || btnCode == JoystickCode.LEFT || btnCode == JoystickCode.RIGHT) {
                if ((this.mJoyStickStatus & btnCode)) {
                    this.mJoyStickStatus ^= btnCode;
                }
                if (this.mJoyStickStatus == 0) {
                    Laya.stage.event(Event.GAMEPAD_DIRECTION_UPDATE, [0, 0, 0]);
                }
                else {
                    this.mJoyStickDirection.setValue(0, 0);
                    if ((this.mJoyStickStatus & JoystickCode.UP) != 0) {
                        this.mJoyStickDirection.y -= 1;
                    }
                    if ((this.mJoyStickStatus & JoystickCode.DOWN) != 0) {
                        this.mJoyStickDirection.y += 1;
                    }
                    if ((this.mJoyStickStatus & JoystickCode.LEFT) != 0) {
                        this.mJoyStickDirection.x -= 1;
                    }
                    if ((this.mJoyStickStatus & JoystickCode.RIGHT) != 0) {
                        this.mJoyStickDirection.x += 1;
                    }
                    Laya.stage.event(Event.GAMEPAD_DIRECTION_UPDATE, [this.mJoyStickDirection.x, this.mJoyStickDirection.y, 1]);
                }
            }
        }
    }

    class GameTouchCtrl {

        constructor() {
            this.mArrowPad = null;
            this.mJoystick = null;
            this.mMaxDist = 100;
            this.mCurPos = new Laya.Vector3();
            this.mNewPos = new Laya.Vector3();
        }
    
        init(gamePad) {
            this.mArrowPad = gamePad;
            this.mJoystick = gamePad.getChildByName("JoyStick");
            this.mArrowPad.alpha = 0;
            Laya.stage.on(Laya.Event.MOUSE_DOWN, this, this.onTouchOn);
            Laya.stage.on(Laya.Event.MOUSE_MOVE, this, this.onTouchMove);
            Laya.stage.on(Laya.Event.MOUSE_UP, this, this.onTouchRelease);
        }
    
        release() {
            Laya.stage.off(Laya.Event.MOUSE_DOWN, this, this.onTouchOn);
            Laya.stage.off(Laya.Event.MOUSE_MOVE, this, this.onTouchMove);
            Laya.stage.off(Laya.Event.MOUSE_UP, this, this.onTouchRelease);
        }
    
        onTouchOn() {
            this.mArrowPad.alpha = 0.5;
            this.mArrowPad.x = Laya.stage.mouseX;
            this.mArrowPad.y = Laya.stage.mouseY;
            this.mJoystick.x = this.mArrowPad.pivotX;
            this.mJoystick.y = this.mArrowPad.pivotY;
            this.mCurPos.setValue(this.mArrowPad.x, this.mArrowPad.y, 0);
            Laya.stage.event(Event.GAMEPAD_DIRECTION_UPDATE, [0, 0, 1]);
        }
    
        onTouchMove() {
            this.mNewPos.setValue(Laya.stage.mouseX, Laya.stage.mouseY, 0);
            Laya.stage.event(Event.GAMEPAD_DIRECTION_UPDATE, [this.mNewPos.x - this.mCurPos.x, this.mNewPos.y - this.mCurPos.y, 1]);
            var dist = Laya.Vector3.distance(this.mCurPos, this.mNewPos);
            if (dist < this.mMaxDist) {
                this.mJoystick.x = this.mNewPos.x - this.mCurPos.x + this.mArrowPad.pivotX;
                this.mJoystick.y = this.mNewPos.y - this.mCurPos.y + this.mArrowPad.pivotY;
            }
            else {
                this.mArrowPad.x += (this.mNewPos.x - this.mCurPos.x) * (1 - this.mMaxDist / dist);
                this.mArrowPad.y += (this.mNewPos.y - this.mCurPos.y) * (1 - this.mMaxDist / dist);
                this.mJoystick.x = (this.mNewPos.x - this.mCurPos.x) * this.mMaxDist / dist + this.mArrowPad.pivotX;
                this.mJoystick.y = (this.mNewPos.y - this.mCurPos.y) * this.mMaxDist / dist + this.mArrowPad.pivotY;
                
                this.mCurPos.setValue(this.mArrowPad.x, this.mArrowPad.y, 0);
            }
        }
    
        onTouchRelease() {
            this.mArrowPad.alpha = 0;
            Laya.stage.event(Event.GAMEPAD_DIRECTION_UPDATE, [0, 0, 0]);
        }
    }

    class GameCtrlPad extends Laya.Script {

        constructor() {
            super();
            this.joystickCtrl = new GameJoystickCtrl();
            this.touchCtrl = new GameTouchCtrl();
        }

        onAwake() {
            this.joystickCtrl.init();
            this.touchCtrl.init(this.owner);
        }

        onDestroy() {
            this.joystickCtrl.release();
            this.touchCtrl.release();
        }

        onUpdate() {
            this.joystickCtrl.update();
        }
    }
    Laya.ClassUtils.regClass("fc.plugin.GameCtrlPad", GameCtrlPad);
    
    exports.ObjectPool = ObjectPool;
    exports.GameObj = GameObj;
    exports.BaseView = BaseView;
    exports.Effect3d = Effect3d;
    exports.Collision3dListener = Collision3dListener;
    exports.Trigger3dListener = Trigger3dListener;
    exports.AnimEventListener = AnimEventListener;
    exports.AnimStateListener = AnimStateListener;
    exports.TextureCamera = TextureCamera;
    exports.SceneLoopMode = SceneLoopMode;
    exports.LoopScene = LoopScene;
    exports.LoopBoard = LoopBoard;
    exports.BannerCtrl = BannerCtrl;
    exports.BlockAdCtrl = BlockAdCtrl;
    exports.AdditiveUICtrl = AdditiveUICtrl;
    exports.UIJoystickCtrl = UIJoystickCtrl;
    exports.GameCtrlPad = GameCtrlPad;

}(window.fc.plugin, Laya))