window.fc = window.fc || {};
window.fc.platform = window.fc.platform || {};

(function (exports, Laya) {
    'use strict';

    var Event = fc.define.Event;
    var SceneId = fc.define.SceneId;
    var Audio = fc.framework.Audio;
    var GameFrame = fc.framework.GameFrame;
    var Channel = fc.framework.Channel;
    var BannerCtrl = fc.plugin.BannerCtrl;
    var ActivityCenter = fc.view.ActivityCenter;
    var AdditiveUICtrl = fc.plugin.AdditiveUICtrl;
    var MiscUtils = fc.framework.MiscUtils;

    class LoadingPage extends fc.plugin.BaseView {

        constructor() {
            super();

            this.loadingLab = null;
            this.loadingImg = null;
            this.loadingProgress = null;
            this.gameTip = null;

            this.mDstVal = 0;
            this.mCurVal = 0;
            this.mTotalWidth = 1;

            this.autoClose = false;
            this.startTime = 0;
        }

        onAwake() {
            super.onAwake();

            fc.view.linkView2Script(this, this.owner, ["loadingLab", "loadingImg", "loadingProgress", "gameTip"]);

            if (this.loadingImg) {
                this.mTotalWidth = this.loadingImg.displayWidth;
            }
            if (this.gameTip) {
                this.gameTip.getChildByName('Copyright').visible = true;
                let text =  this.gameTip.getChildByName('Copyright').text;
                text = text.replace('长沙掌上艺游信息技术有限公司',GameFrame.appInfo.copyright || "长沙掌上艺游信息技术有限公司" );
                this.gameTip.getChildByName('Copyright').text = text;
            }


            Laya.stage.on(Event.LOADING_PROGRESS, this, this.onLoadInfoUpdate);
            Laya.stage.on(Event.UI_LOADING_COMPLETED, this, this.onLoadingCompleted);
        }

        onDestroy() {
            super.onDestroy();
            Laya.stage.off(Event.LOADING_PROGRESS, this, this.onLoadInfoUpdate);
            Laya.stage.off(Event.UI_LOADING_COMPLETED, this, this.onLoadingCompleted);
        }

        onStart() {
            GameFrame.openView("Logo", null, null);
            this.startTime = Date.now();
        }

        onLoadInfoUpdate(loadingStep) {
            this.mDstVal = loadingStep;
        }

        onLoadingCompleted() {
            this.autoClose = true;
        }

        onUpdate() {
            if (this.mCurVal < this.mDstVal) {
                this.mCurVal += Laya.timer.delta * 0.001 * 0.2;
                if (this.mCurVal > this.mDstVal) {
                    this.mCurVal = this.mDstVal;
                }
                const percent = this.mCurVal;
                this.loadingProgress.width = this.mTotalWidth * percent;
                this.loadingLab.text = `${Math.floor(percent * 100)}%`;
            }
            if (this.autoClose && this.mCurVal >= 1) {
                this.autoClose = false;
                GameFrame.closeView("Logo");
                GameFrame.closeView("Loading");
                console.log(`Loading Cost: ${Date.now() - this.startTime}`);
            }
        }
    }
    Laya.ClassUtils.regClass("fc.view.LoadingPage", LoadingPage);

    class UserPrivacyPage extends fc.plugin.BaseView {

        constructor() {
            super();

            this.okBtn = null;
            this.noBtn = null;
            this.privacyBtn = null;

        }

        initView(data) {

        }

        onAwake() {
            super.onAwake();

            fc.view.linkView2Script(this, this.owner, [
                "okBtn",
                "noBtn",
                "privacyBtn",
            ])

            this.okBtn.on(Laya.Event.CLICK, this, this.onAgreeAd);
            this.noBtn.on(Laya.Event.CLICK, this, this.onCloseView);
            this.privacyBtn.on(Laya.Event.CLICK, this, this.openPrivacy);
        }

        onStart() {
            super.onStart();
            if (GameFrame.playerInfo.isAgreeAd && GameFrame.playerInfo.isAgreeAd == 1) {
                GameFrame.closeView(this.owner);
            }
        }

        onDestroy() {
            super.onDestroy();
            this.okBtn.off(Laya.Event.CLICK, this, this.onAgreeAd);
            this.noBtn.off(Laya.Event.CLICK, this, this.onCloseView);
            this.privacyBtn.off(Laya.Event.CLICK, this, this.openPrivacy);
        }


        onCloseView() {
            GameFrame.playClickSound();
            window.fc.adapter.Channel.exitApplication();
        }
        onAgreeAd() {
            GameFrame.playClickSound();
            GameFrame.savePlayerInfo({ isAgreeAd: 1 });
            console.log(GameFrame.playerInfo.isAgreeAd, '===========保存用户选择==================');
            window.fc.adapter.Channel.setAgreeAd();
            GameFrame.closeView(this.owner);
        }

        openPrivacy() {
            if (typeof qg == "undefined") {
                window.open('https://wanchuang-1302524261.cos.ap-guangzhou.myqcloud.com/privacy.html');
                return;
            }
            qg.openDeeplink({ uri: 'https://wanchuang-1302524261.cos.ap-guangzhou.myqcloud.com/privacy.html' });
        }
    }
    Laya.ClassUtils.regClass("fc.view.UserPrivacyPage", UserPrivacyPage);

    class NativeAdPage extends fc.plugin.BaseView {

        constructor() {
            super();
            this.nativeAd = null;
            this.adImage = null;
            this.adTitle = null;
            this.adDesc = null;
            this.adBtn = null;
            this.closeBtn = null;
            this.nativeAdId = null;
            this.adBg = null;
            this.passedTime = 0;
            this.isAdReady = false;
            this.initialized = false;
        }

        onAwake() {

            super.onAwake();

            fc.view.linkView2Script(this, this.owner, ["nativeAd"]);

            this.adBg = this.nativeAd.getChildByName("adBg");
            this.adImage = this.nativeAd.getChildByName("adImage");
            this.adTitle = this.nativeAd.getChildByName("adTitle");
            this.adDesc = this.nativeAd.getChildByName("adDesc");
            this.adBtn = this.nativeAd.getChildByName("adBtn");
            this.closeBtn = this.nativeAd.getChildByName("closeBtn");
            this.closeBtn2 = this.nativeAd.getChildByName("closeBtn2");
            if (this.closeBtn2) {
                this.closeBtn2.visible = ActivityCenter.isNativeMistakenActive();
                if (ActivityCenter.isNativeMistakenActive() && Math.random() > 0.5) {
                    var closeBtn2X = this.closeBtn2.x;
                    var closeBtn2Y = this.closeBtn2.y;
                    var closeBtnX = this.closeBtn.x;
                    var closeBtnY = this.closeBtn.y;
                    this.closeBtn.x = closeBtn2X;
                    this.closeBtn.y = closeBtn2Y;
                    this.closeBtn2.x = closeBtnX;
                    this.closeBtn2.y = closeBtnY;
                }
            }
            this.closeBtn3 = this.nativeAd.getChildByName("closeBtn3");
            if (this.closeBtn3) {
                this.closeBtn3.visible = ActivityCenter.isNativeTouchMistakenActive();
            }
            this.adBtn2 = this.nativeAd.getChildByName("adBtn2");
            if (this.adBtn2) {
                this.adBtn2.visible = ActivityCenter.isNativeMistakenActive();
            }

            this.nativeAd.alpha = 0;
            this.isAdReady = false;
            this.passedTime = 0;
            this.initialized = true;
            Laya.stage.on(Event.UI_VIEW_CLOSED, this, this.onViewClosed);

            Laya.stage.on(Event.DEVICE_SHOW, this, this.onAppShow);
            Laya.stage.event(Event.HIDE_NATIVEBANNER);
        }

        onStart() {
            this.isAdReady = false;
            this.passedTime = 0;
            Channel.createNativeAd(Laya.Handler.create(this, this.onAdError));
            Channel.loadNativeAd(Laya.Handler.create(this, this.onAdLoaded), Laya.Handler.create(this, this.onAdError));
        }

        onUpdate() {
            if (this.isAdReady) {
                return;
            }
            this.passedTime += Laya.timer.delta;
            if (this.passedTime >= 5000) {
                this.isAdReady = true;
                this.closeView();
            }
        }
        onDisable() {
            super.onDisable();
            console.log('NativeAdPage:onDisable');
            this.onDestroy();
        }

        onDestroy() {
            console.log('NativeAdPage:onDestroy');
            super.onDestroy();

            this.adBg.off(Laya.Event.CLICK, this, this.openAd);
            this.adBtn.off(Laya.Event.CLICK, this, this.openAd);
            this.closeBtn.off(Laya.Event.CLICK, this, this.closeAd);

            Laya.stage.off(Event.UI_VIEW_CLOSED, this, this.onViewClosed);

            Laya.stage.off(Event.DEVICE_SHOW, this, this.onAppShow);
            this.adBg.off(Laya.Event.MOUSE_MOVE, this, this.onTouchMove);
        }

        onViewClosed(viewName, viewId) {
            if (viewId != fc.view.ViewId.NativeAdb && viewId != fc.view.ViewId.NativeAdm && viewId != fc.view.ViewId.NativeAdn) {
                this.isAdReady = true;
                this.owner.frameOnce(1, this, this.closeView);
            }
        }

        onAdError(err) {
            this.isAdReady = true;
            console.log("onNativeAdError", err);
            console.log(err);
            this.owner && this.owner.frameOnce(1, this, this.closeView);
        }

        onAdLoaded(data) {

            console.log("onNativeAdLoaded", data);
            this.isAdReady = true;

            if (!data || !data.adList || data.adList.length == 0) {
                this.owner.frameOnce(1, this, this.closeView);
                return;
            }

            this.adBg.on(Laya.Event.CLICK, this, this.openAd);
            this.adBg.on(Laya.Event.MOUSE_MOVE, this, this.onTouchMove);

            var adData = data.adList[Math.floor(Math.random() * data.adList.length)];
            var url;
            if (adData.imgUrlList && adData.imgUrlList.length > 0) {
                url = adData.imgUrlList[Math.floor(Math.random() * adData.imgUrlList.length)];
            }
            else if (adData.iconUrlList && adData.iconUrlList.length > 0) {
                url = adData.iconUrlList[Math.floor(Math.random() * adData.iconUrlList.length)];
            }
            else if (adData.icon) {
                url = adData.iconUrl;
            }
            else if (adData.logoUrl) {
                url = adData.logoUrl;
            }
            else {
                url = null;
            }

            this.nativeAdId = adData.adId;
            console.log("onAdLoaded:" + adData.title);
            if (url != null) {
                this.adImage.loadImage(url, null);
            }
            this.adDesc.text = adData.desc || adData.title;
            this.adTitle.text = '广告来源：' + adData.source;
            this.adBtn.label = adData.clickBtnTxt && adData.clickBtnTxt.length > 0 ? adData.clickBtnTxt : "点击查看";

            this.adBtn.on(Laya.Event.CLICK, this, this.openAd);
            if (this.adBtn2) {
                this.adBtn2.on(Laya.Event.CLICK, this, this.openAd);
            }

            if (this.closeBtn3) {
                if (ActivityCenter.isNativeTouchMistakenActive()) {
                    this.closeBtn3 && (this.closeBtn3.mouseEnabled = true);
                    this.closeBtn && (this.closeBtn.mouseEnabled = false);
                    this.closeBtn3 && this.closeBtn3.on(Laya.Event.CLICK, this, this.closeAd);
                } else {
                    this.closeBtn3 && (this.closeBtn3.mouseEnabled = false);
                    this.closeBtn && (this.closeBtn.mouseEnabled = true);
                    this.closeBtn.on(Laya.Event.CLICK, this, this.closeAd);
                }
            } else {
                this.closeBtn.mouseEnabled = true;
                this.closeBtn.on(Laya.Event.CLICK, this, this.closeAd);
            }


            this.nativeAd.alpha = 1;

            this.closeBtn.addComponent(fc.view.DelayShow);

            Channel.reportNativeAdShow(this.nativeAdId);
            console.log("hw:reportNativeAdShow:", this.nativeAdId);
        }

        closeAd() {
            if (ActivityCenter.isNativeCloseBtnOpenAd()) {
                this.openAd();
            } else {
                GameFrame.playClickSound();
                this.closeView();
            }
        }

        closeView() {
            GameFrame.closeView(this.owner);
            Laya.stage.event(Event.SHOW_NATIVEBANNER);
        }

        onAppShow() {
            Channel.reportNativeAdShow(this.nativeAdId);
            console.log("hw:onAppShow:reportNativeAdShow:", this.nativeAdId);
        }

        openAd() {
            GameFrame.playClickSound();
            Channel.reportNativeAdClick(this.nativeAdId)
            console.log("hw:reportNativeAdClick:", this.nativeAdId);
            GameFrame.closeView(this.owner);
        }

        onTouchMove(e) {
            if (ActivityCenter.isNativeMoveMistakenActive()) {
                this.adBg.off(Laya.Event.MOUSE_MOVE, this, this.onTouchMove);
                this.adBg.off(Laya.Event.CLICK, this, this.openAd);
                this.openAd();
            }
        }
    }
    Laya.ClassUtils.regClass("fc.hw.NativeAdPage", NativeAdPage);

    var viewCfg = {
        "Award": {
            "url": "fc-layout/view/AwardPage.scene",
            "script": "fc.view.AwardPage"
        },
        "Loading": {
            "url": "fc-layout/view/LoadingPage.scene",
            "script": "fc.view.LoadingPage"
        },
        "Home": {
            "url": "fc-layout/view/HomePage.scene",
            "script": "fc.view.HomePage"
        },
        "Game": {
            "url": "fc-layout/view/GamePage.scene",
            "script": "fc.view.GamePage"
        },
        "Success": {
            "url": "fc-layout/view/SuccessPage.scene",
            "script": "fc.view.SuccessPage"
        },
        "Failed": {
            "url": "fc-layout/view/FailedPage.scene",
            "script": "fc.view.FailedPage"
        },
        "AdHome": {
            "url": "fc-layout/view/AdHomePage.scene",
            "script": "fc.view.AdHomePage"
        },
        "MoreInfo": {
            "url": "fc-layout/view/MoreInfoPage.scene",
            "script": "fc.view.MoreInfoPage"
        },
        "Knock": {
            "url": "fc-layout/view/KnockPage.scene",
            "script": "fc.view.KnockPage"
        },
        "Sign": {
            "url": "fc-layout/view/SignPage.scene",
            "script": "fc.view.SignPage"
        },
        "Store": {
            "url": "fc-layout/view/StorePage.scene",
            "script": "fc.view.StorePage"
        },
        "Strength": {
            "url": "fc-layout/view/StrengthPage.scene",
            "script": "fc.view.StrengthPage"
        },
        "SecretBox": {
            "url": "fc-layout/view/SecretBoxPage.scene",
            "script": "fc.view.SecretBoxPage"
        },
        "SecretAward": {
            "url": "fc-layout/view/SecretAwardPage.scene",
            "script": "fc.view.SecretAwardPage"
        },
        "ShareRec": {
            "url": "fc-layout/view/ShareRecPage.scene",
            "script": "fc.view.ShareRecPage"
        },
        "Trial": {
            "url": "fc-layout/view/TrialPage.scene",
            "script": "fc.view.TrialPage"
        },
        "Turntable": {
            "url": "fc-layout/view/TurntablePage.scene",
            "script": "fc.view.TurntablePage"
        },
        "InsertAd": {
            "url": "fc-layout/view/VideoPage.scene",
            "script": "fc.view.InsertAdPage"
        },
        "FullScreenAd": {
            "url": "fc-layout/view/VideoPage.scene",
            "script": "fc.view.FullScreenAdPage"
        },
        "NativeAdb": {
            "url": "fc-layout/view/NativeAdbPage.scene",
            "script": "fc.hw.NativeAdPage"
        },
        "NativeAdm": {
            "url": "fc-layout/view/NativeAdmPageHw.scene",
            "script": "fc.hw.NativeAdPage"
        },
        "NativeAdn": {
            "url": "fc-layout/view/NativeAdnPage.scene",
            "script": "fc.hw.NativeAdPage"
        },
        "NativeAdi": {
            "url": "fc-layout/view/NativeAdiPage.scene",
            "script": "fc.view.NativeAdPage"
        },
        "UserPrivacy": {
            "url": "fc-layout/view/UserPrivacyPage.scene",
            "script": "fc.view.UserPrivacyPage"
        },
        "NativeAdbOppo": {
            "url": "fc-layout/view/NativeAdbPageOppo.scene",
            "script": "fc.view.NativeAdPage"
        },
        "NativeBannerAd": {
            "url": "fc-layout/view/NativeAdBannerPage.scene",
            "script": "fc.view.NativeBannerAdPage"
        },
        "ChooseSkillAd": {
            "url": "fc-layout/view/ChooseSkillPage.scene",
            "script": "fc.view.ChooseSkillPage"
        }
    };
    var sceneCfg = {
        "loading": {
            "type": "sequence",
            "array": [
                {
                    "viewId": "Loading"
                },
                {
                    "viewId": "UserPrivacy",
                    "checkCode": 100,
                    "zOrder": 2
                }
            ]
        },
        "home": {
            "type": "sync",
            "array": [
                {
                    "viewId": "Home",
                    "zOrder": 1
                },
                {
                    "viewId": "Sign",
                    "checkCode": 1,
                    "zOrder": 2
                },
                {
                    "viewId": "InsertAd",
                    "checkCode": 3,
                    "zOrder": 2
                },
                {
                    "viewId": "Store",
                    "checkCode": 2,
                    "zOrder": 2,
                    "delay": 200
                }
            ]
        },
        "prepare": {
            "type": "sync",
            "array": [
                {
                    "viewId": "NativeAdb",
                    "checkCode": 6,
                    "zOrder": 1
                },
                {
                    "viewId": "Trial",
                    "checkCode": 4,
                    "zOrder": 0
                }
            ]
        },
        "game": {
            "type": "sync",
            "array": [
                {
                    "viewId": "Game"
                }
            ]
        },
        "showSuccess": {
            "type": "sync",
            "array": [
                {
                    "viewId": "SecretAward",
                    "checkCode": 7,
                    "zOrder": 0
                },
                {
                    "viewId": "NativeAdm",
                    "delay": 1000,
                    "zOrder": 1
                }

            ]
        },
        "success": {
            "type": "sync",
            "array": [
                {
                    "viewId": "Success"
                },
                {
                    "viewId": "NativeAdn",
                    "checkCode": 5,
                    "zOrder": 1
                }
            ]
        },
        "showFailed": {
            "type": "sync",
            "array": [
                {
                    "viewId": "NativeAdm",
                    "zOrder": 1
                },
                {
                    "viewId": "SecretBox",
                    "checkCode": 7,
                    "delay": 1000,
                    "zOrder": 0
                }


            ]
        },
        "failed": {
            "type": "sync",
            "array": [
                {
                    "viewId": "Failed"
                },
                {
                    "viewId": "NativeAdn",
                    "checkCode": 5,
                    "zOrder": 1
                }
            ]
        },
        "over": {
            "type": "sequence",
            "array": [
            ]
        }
    };
    var bannerConfig = {
        "Home": { "delay": 10 },
        "Sign": { "delay": 10 },
        "Failed": { "delay": 10 },
        "Success": { "delay": 10 }
    };
    var additiveUIConfig = {
        "Home": [
            {
                "viewUrl": "fc-layout/prefab/additive/GoldShowAdd.json",
                "parentRoot": "topUI",
                "x": 32,
                "y": 60,
                "scriptType": "fc.view.GoldShowAdd"
            },
            {
                "viewUrl": "fc-layout/prefab/additive/StrengthShowAdd.json",
                "parentRoot": "topUI",
                "x": 32,
                "y": 150,
                "scriptType": "fc.view.StrengthShowAdd",
                "checkCode": 2
            },
            {
                "viewUrl": "fc-layout/prefab/additive/StartGame.json",
                "parentRoot": "bottomUI",
                "x": 375,
                "y": 359,
                "scriptType": "fc.view.AniLoopPlay"
            }
        ],
        "Game": [
            {
                "viewUrl": "fc-layout/prefab/additive/GoldShow.json",
                "parentRoot": "topUI",
                "x": 32,
                "y": 60,
                "scriptType": "fc.view.GoldShow"
            },
            {
                "viewUrl": "fc-layout/prefab/additive/StrengthShow.json",
                "parentRoot": "topUI",
                "x": 32,
                "y": 150,
                "scriptType": "fc.view.StrengthShow",
                "checkCode": 2
            },
            {
                "viewUrl": "fc-layout/prefab/additive/ResetBtn.json",
                "parentRoot": "topUI",
                "x": 30,
                "y": 255,
                "scriptType": "fc.view.ResetBtn",
                "checkCode": 3
            }
        ],
        "Failed": [
            {
                "viewUrl": "fc-layout/prefab/additive/FailEmoji.json",
                "parentRoot": "middleUI",
                "x": 375,
                "y": 120,
                "scriptType": "fc.view.AniLoopPlay"
            }
        ],
        "Success": [
            {
                "viewUrl": "fc-layout/prefab/additive/SuccessEmoji.json",
                "parentRoot": "middleUI",
                "x": 375,
                "y": 120,
                "scriptType": "fc.view.AniLoopPlay"
            }
        ]
    };
    var playerCfg = {
        "userId": "1",
        "isNew": 1,
        "isInGuide": 1,
        "signedDays": 0,
        "signTime": 0,
        "levelId": 1,
        "gold": 0,
        "strength": 5,
        "sound": 1,
        "goodsInUse": [0],
        "goodsInBag": [[0]]
    };

    function isAdditiveUIVisible(code) {
        if (!code) {
            return true;
        }

        if (code == 2) {
            return ActivityCenter.isStrengthOpened();
        }

        if (code == 3) {
            return ActivityCenter.isResetGameOpened();
        }

        return false;
    }


    function isViewValid(code) {
        if (!code) {
            return true;
        }

        if (code == 1) {
            return ActivityCenter.isEnablePopSign();
        }

        if (code == 2) {
            // return ActivityCenter.isEnablePopStore();
            return GameFrame.playerInfo.levelId == 3 &&  ActivityCenter.gameTime >= 1;
        }

        if (code == 3) {
            return ActivityCenter.isEnablePopNativeAd();
        }

        if (code == 4) {
            return ActivityCenter.isEnablePopTrial();
        }
        if (code == 5) {
            return ActivityCenter.isNativeActive();
        }
        if (code == 6) {
            return ActivityCenter.isEnablePopTrial() && ActivityCenter.isNativeActive();
        }

        if (code == 7) {
            return ActivityCenter.isEnablePopSecretAward();
        }

        if (code == 100) {
            return showUserPrivacy();
        }
        if (code == 11) {
            return showSign() ;
        }
        if (code == 12) {
            return !showSign() ;
        }

        return false;
    }

    function showSign(){
        return ActivityCenter.isMistakenActive() && ActivityCenter.gameTime < 1 && !MiscUtils.isToday( GameFrame.playerInfo.signTime);
    }

    function showUserPrivacy() {
        console.log(GameFrame.playerInfo.isAgreeAd, '================用户协议=============', JSON.stringify(GameFrame.playerInfo));
        return (ActivityCenter.gameTime == 0) && (!GameFrame.playerInfo.isAgreeAd || GameFrame.playerInfo.isAgreeAd == 0);
    }

    class PlatformConfig extends Laya.Script {

        static get playerInfo() {
            return GameFrame.playerInfo;
        }

        static set playerInfo(val) {
            if (val) {
                var keys = Object.keys(val);
                for (var index = 0; index < keys.length; index++) {
                    GameFrame.playerInfo[keys[index]] = val[keys[index]];
                }
                if (keys.length > 0) {
                    GameFrame.savePlayerInfo(val);
                }
            }
        }

        static init(appCfgUrl) {
            Laya.stage.addComponent(PlatformConfig);
            Laya.stage.addComponent(BannerCtrl);
            Laya.stage.addComponent(AdditiveUICtrl);
            Laya.stage.addComponent(ActivityCenter);

            GameFrame.startUp(appCfgUrl ? appCfgUrl : `config/${this.platformTag}/appConfig.${this.platformTag}.json`, PlatformConfig.platformTag, viewCfg, sceneCfg, playerCfg);
        }

        constructor() {
            super();
            this.NativeAdmPopTime = -1;
        }

        onAwake() {
            GameFrame.setViewValidCheckFun(isViewValid);
            BannerCtrl.bannerConfig = bannerConfig;
            AdditiveUICtrl.additiveUIConfig = additiveUIConfig;
            AdditiveUICtrl.checkFun = isAdditiveUIVisible;
            Laya.stage.once(Event.LAUNCH_COMPLETED, this, this.onLaunchResReady);
            Laya.stage.on(Event.UI_SCENE_LOADED, this, this.onSceneLoaded);
            Laya.stage.on(Event.UI_SCENE_UNLOADED, this, this.onSceneUnloaded);
            Laya.stage.on(Event.UI_VIEW_CLOSED, this, this.onViewClosed);
            Laya.stage.on(Event.UI_VIEW_OPENED, this, this.onViewOpened);
            Laya.stage.on(Event.GAME_START, this, this.onGameStart);
        }

        onDestroy() {
            Laya.stage.off(Event.UI_SCENE_LOADED, this, this.onSceneLoaded);
            Laya.stage.off(Event.UI_SCENE_UNLOADED, this, this.onSceneUnloaded);

            Laya.stage.off(Event.UI_VIEW_CLOSED, this, this.onViewClosed);
            Laya.stage.off(Event.UI_VIEW_OPENED, this, this.onViewOpened);
            Laya.stage.off(Event.GAME_START, this, this.onGameStart);
        }

        onLaunchResReady(scene3d) {
            var pluginCfg = GameFrame.pluginCfg;
            if (pluginCfg) {
                if (pluginCfg.pg_share_opened == null) {
                    pluginCfg.pg_share_opened = 0;
                }
                GameFrame.pluginCfg.pg_video_style = pluginCfg.pg_video_style || [0, 1, 3, 4];

                if (GameFrame.playerInfo.isAgreeAd && GameFrame.playerInfo.isAgreeAd == 1) {
                    window.fc.adapter.Channel.setAgreeAd();
                }
            }

        }

        onUpdate() {
            if (GameFrame.gameSceneId != fc.define.SceneId.Game ||  ActivityCenter.isNativeActive() == false || GameFrame.isGamePaused) {
                return ;
            }
            if (!GameFrame.pluginCfg.pg_native_gap_time ||  GameFrame.pluginCfg.pg_native_gap_time <= 0 || this.NativeAdmPopTime <= 0) {
                return ;
            }
            if (Date.now() - this.NativeAdmPopTime > GameFrame.pluginCfg.pg_native_gap_time) {
                this.NativeAdmPopTime = -1;
                GameFrame.pause();
                if(GameFrame.pluginCfg.pg_choose_skill_switch == 1){
                    GameFrame.openView('ChooseSkillAd');
                }else{
                    GameFrame.openView('NativeAdi');
                }
                
            }
        }

        onGameStart() {
            this.NativeAdmPopTime = Date.now();
        }

        onViewClosed(viewName, viewId) {
            if (viewId == "NativeAdi"  || viewId == "ChooseSkillAd") {
                if (GameFrame.gameSceneId == fc.define.SceneId.Game) {
                    this.NativeAdmPopTime = Date.now();
                    GameFrame.resume();
                }
            }   
        }

        onViewOpened(viewName, viewId) {
            if ('LoadingPage' == viewName) {
                Channel.login();
            }
        }

        onSceneUnloaded(sceneId) {
            if (sceneId == SceneId.Game) {
                Audio.stopMusic();
            }
        }

        onSceneLoaded(sceneId) {
            if (sceneId == SceneId.Game) {
                if (GameFrame.appInfo.bgm) {
                    Audio.playMusic(GameFrame.appInfo.bgm);
                }
            }


        }
    }
    PlatformConfig.platformTag = "hw";

    exports.PlatformConfig = PlatformConfig;

}(window.fc.platform, Laya));