window.fc = window.fc || {};
window.fc.view = window.fc.view || {};
(function (exports, Laya) {
    'use strict';

    var GameFrame = fc.framework.GameFrame;
    var Channel = fc.framework.Channel;
    var MiscUtils = fc.framework.MiscUtils;
    var BannerCtrl = fc.plugin.BannerCtrl;

    var Event = fc.define.Event;

    function linkView2Script(script, view, keys) {

        var viewRoot;
        viewRoot = view.getChildByName("fullUI");
        if (viewRoot) {
            for (let index = keys.length - 1; index >= 0; index--) {
                const element = viewRoot.getChildByName(keys[index]);
                if (element) {
                    script[keys[index]] = element;
                    keys.splice(index, 1);
                }
            }
            if (keys.length == 0) {
                return;
            }
        }

        viewRoot = view.getChildByName("topUI");
        if (viewRoot) {
            for (let index = keys.length - 1; index >= 0; index--) {
                const element = viewRoot.getChildByName(keys[index]);
                if (element) {
                    script[keys[index]] = element;
                    keys.splice(index, 1);
                }
            }
            if (keys.length == 0) {
                return;
            }
        }

        viewRoot = view.getChildByName("middleUI");
        if (viewRoot) {
            for (let index = keys.length - 1; index >= 0; index--) {
                const element = viewRoot.getChildByName(keys[index]);
                if (element) {
                    script[keys[index]] = element;
                    keys.splice(index, 1);
                }
            }
            if (keys.length == 0) {
                return;
            }
        }

        viewRoot = view.getChildByName("bottomUI");
        if (viewRoot) {
            for (let index = keys.length - 1; index >= 0; index--) {
                const element = viewRoot.getChildByName(keys[index]);
                if (element) {
                    script[keys[index]] = element;
                    keys.splice(index, 1);
                }
            }
            if (keys.length == 0) {
                return;
            }
        }

        viewRoot = view;
        if (viewRoot) {
            for (let index = keys.length - 1; index >= 0; index--) {
                const element = viewRoot.getChildByName(keys[index]);
                if (element) {
                    script[keys[index]] = element;
                    keys.splice(index, 1);
                }
            }
            if (keys.length == 0) {
                return;
            }
        }
    }

    function isEnableRelive() {
        var pluginCfg = GameFrame.pluginCfg;
        if (!pluginCfg) {
            return false;
        }
        if (!pluginCfg.pg_revive_share_num && !pluginCfg.pg_revive_video_num) {
            return false;
        }
        if (!pluginCfg.pg_revive_per_game) {
            return false;
        }
        if (GameFrame.relivedTime >= pluginCfg.pg_revive_per_game && pluginCfg.pg_revive_per_game > 0) {
            return false;
        }
        var reliveType = getReliveType();
        return !!reliveType;
    }

    function getReliveType() {
        var pluginCfg = GameFrame.pluginCfg;
        if (!pluginCfg) {
            return 0;
        }
        var relivedTime = getRelivedTimeToday();
        if (pluginCfg.pg_revive_video_num) {
            if (pluginCfg.pg_revive_video_num == -1 && Channel.isVideoEnable()) {
                return 1;
            }
            else if (relivedTime < pluginCfg.pg_revive_video_num && Channel.isVideoEnable()) {
                return 1;
            }
        }
        if (pluginCfg.pg_revive_share_num) {
            if (pluginCfg.pg_revive_share_num == -1 && Channel.isShareEnable()) {
                return 2;
            }
            else if (relivedTime < pluginCfg.pg_revive_share_num && Channel.isShareEnable()) {
                return 2;
            }
        }
        return 0;
    }

    function getRelivedTimeToday() {
        var reliveInfo = GameFrame.getLocalStorage("fc_reliveInfo");
        if (!reliveInfo) {
            return 0;
        }
        else {
            var info = reliveInfo.split(",");
            var timestamp = info[info.length - 1];
            timestamp = Number(timestamp);
            if (MiscUtils.isToday(timestamp)) {
                return Number(info[0]);
            }
            else {
                return 0;
            }
        }
    }

    function accRelivedTimeToday() {
        var reliveInfo = GameFrame.getLocalStorage("fc_reliveInfo");
        if (!reliveInfo) {
            GameFrame.setLocalStorage("fc_reliveInfo", `1,${Date.now()}`);
        }
        else {
            var info = reliveInfo.split(",");
            var relivedTime = Number(info[0]);
            relivedTime++;
            GameFrame.setLocalStorage("fc_reliveInfo", `${relivedTime},${Date.now()}`);
        }
    }

    function getSpRecoverTimestamp() {
        var spRecoverTimestamp = GameFrame.getLocalStorage("fc_spRecoverTimestamp");
        if (!spRecoverTimestamp) {
            GameFrame.setLocalStorage("fc_spRecoverTimestamp", Date.now());
            return 0;
        }
        else {
            return Number(spRecoverTimestamp);
        }
    }

    function updateSpRecoverTimestamp(timestampe) {
        GameFrame.setLocalStorage("fc_spRecoverTimestamp", timestampe ? timestampe : Date.now());
    }

    function video2Gold() {
        return GameFrame.pluginCfg && GameFrame.pluginCfg.pg_video_2_gold ? GameFrame.pluginCfg.pg_video_2_gold : 1000;
    }

    function video2Strength() {
        return GameFrame.pluginCfg && GameFrame.pluginCfg.pg_video_2_sp ? GameFrame.pluginCfg.pg_video_2_sp : 10;
    }

    function getMaxStrength() {
        var pluginCfg = GameFrame.pluginCfg;
        if (!pluginCfg || !pluginCfg.pg_sp_max) {
            return -1;
        }
        return pluginCfg.pg_sp_max;
    }

    function getStrengthRecoverInterval() {
        var pluginCfg = GameFrame.pluginCfg;
        if (!pluginCfg || !pluginCfg.pg_sp_recover_interval) {
            return 60000;
        }
        return pluginCfg.pg_sp_recover_interval * 60000;
    }

    function getStrengthRecoverPerTime() {
        var pluginCfg = GameFrame.pluginCfg;
        if (!pluginCfg || !pluginCfg.pg_sp_recover_per_time) {
            return 0;
        }
        return pluginCfg.pg_sp_recover_per_time;
    }

    function isMistakenActive() {
        var pluginCfg = GameFrame.pluginCfg;
        return pluginCfg &&
            pluginCfg.pg_switch &&
            GameFrame.appInfo.version != pluginCfg.pg_version &&
            GameFrame.playerInfo.levelId >= pluginCfg.pg_ad_lv;
    }

    function isVideoMistakenActive() {
        return isMistakenActive() && GameFrame.pluginCfg.pg_ad_switch;
    }
    function isNativeMistakenActive() {
        return isMistakenActive() && GameFrame.pluginCfg.pg_native_btn_switch != 0;
    }
    function isNativeTouchMistakenActive() {
        return isMistakenActive() && GameFrame.pluginCfg.pg_native_touch_switch != 0 && !isNativeMistakenActive();
    }
    function isNativeMoveMistakenActive() {
        return isMistakenActive() && GameFrame.pluginCfg.pg_native_move_switch != 0;
    }

    function isVideoImgBtnMistakenActive() {
        return isMistakenActive() && GameFrame.pluginCfg.pg_ad_switch && GameFrame.pluginCfg.pg_ad_img_switch != 0;
    }

    function isNativeActive() {
        return isMistakenActive() && GameFrame.playerInfo.levelId >=  GameFrame.pluginCfg.pg_ad_native_lv;
    }

    function isHomeAutoShowGameBox() {
        return isMistakenActive() && GameFrame.pluginCfg.pg_auto_show_gamebox && GameFrame.pluginCfg.pg_auto_show_gamebox == 1;
    }

    //原生广告关闭按钮概率进入原生广告
    function isNativeCloseBtnOpenAd(){
        var probability = GameFrame.pluginCfg.pg_closebtn_nativead || 0
        return isMistakenActive() && probability > 0 && Math.random() * 100 < probability ;
    }

   //神秘大奖界面关闭按钮概率进去原生广告
    function isAwardCloseBtnOpenAd(){
        var probability = GameFrame.pluginCfg.pg_awardbtn_nativead || 0
        return isMistakenActive() && probability > 0 && Math.random() * 100 < probability ;
    }

    function getBtnMistakenStyle() {
        return GameFrame.pluginCfg && GameFrame.pluginCfg.pg_button_show_style ? GameFrame.pluginCfg.pg_button_show_style : 0;
    }

    function getBtnMoveDuaration() {
        return GameFrame.pluginCfg && GameFrame.pluginCfg.pg_text_move_time ? GameFrame.pluginCfg.pg_text_move_time : 0;
    }

    function getMistakenBannerDelay() {
        return GameFrame.pluginCfg && GameFrame.pluginCfg.pg_mistaken_banner_delay ? GameFrame.pluginCfg.pg_mistaken_banner_delay : 1000;
    }

    function getMistakenBannerDuaration() {
        return GameFrame.pluginCfg && GameFrame.pluginCfg.pg_mistaken_banner_duaration ? GameFrame.pluginCfg.pg_mistaken_banner_duaration : 3000;
    }

    function getBtnDelayTime() {
        return GameFrame.pluginCfg ? GameFrame.pluginCfg.pg_button_delay_time : 0;
    }

    function getKnockEnergeDecPerFrame() {
        return GameFrame.pluginCfg.pg_fastclick_progress_back;// : 0.1;
    }

    function getKnockEnergeAccPerClick() {
        return GameFrame.pluginCfg.pg_fastclick_progress_add;// : 0.15;
    }

    function isInAdKnockEnergeRange(val) {
        return val > GameFrame.pluginCfg.pg_fastclick_ad_percent[0] && val < GameFrame.pluginCfg.pg_fastclick_ad_percent[1];
    }

    function isEnableCancelVideoModal() {
        var pluginCfg = GameFrame.pluginCfg;
        return pluginCfg &&
            pluginCfg.pg_version != GameFrame.appInfo.version &&
            pluginCfg.pg_switch &&
            pluginCfg.pg_video_cancel_switch == 1;
    }

    function showModal(resultHandler) {
        Channel.showModal(
            GameFrame.langCfg.videocanceltitle,
            GameFrame.langCfg.videocanceldesc,
            GameFrame.langCfg.ok,
            GameFrame.langCfg.cancel,
            resultHandler);
    }

    function isVip() {
        return GameFrame.playerInfo.isVip == 1;
    }

    function isVipValid() {
        var playerInfo = GameFrame.playerInfo;
        return playerInfo.vipTimestamp != null && Date.now() - playerInfo.vipTimestamp < 3600 * 24000;
    }

    class IntroducePage extends fc.plugin.BaseView {

        constructor() {
            super();

            this.introduce = null;
        }

        onAwake() {
            super.onAwake();

            linkView2Script(this, this.owner, ["introduce"]);
            if (GameFrame.pluginCfg.introduceCfg) {
                this.introduce.skin = GameFrame.pluginCfg.introduceCfg;
            }
            this.introduce.on(Laya.Event.CLICK, this, this.closeIntroduce);
        }

        onDestroy() {
            super.onDestroy();
            this.introduce.off(Laya.Event.CLICK, this, this.closeIntroduce);
        }

        closeIntroduce() {
            GameFrame.closeView(this.owner);
        }
    }
    Laya.ClassUtils.regClass("fc.view.IntroducePage", IntroducePage);

    class LoadingPage extends fc.plugin.BaseView {

        constructor() {
            super();

            this.gameTip = null;
            this.loadingLab = null;
            this.loadingImg = null;
            this.loadingProgress = null;

            this.mDstVal = 0;
            this.mCurVal = 0;
            this.mTotalWidth = 1;

            this.autoClose = false;
            this.startTime = 0;
        }

        onAwake() {
            super.onAwake();

            linkView2Script(this, this.owner, ["gameTip", "loadingLab", "loadingImg", "loadingProgress"]);
            
        
            if (fc.platform.PlatformConfig.platformTag.indexOf("iptv") >= 0) {
                this.gameTip && (this.gameTip.visible = false);
            }

            if (this.loadingImg) {
                this.mTotalWidth = this.loadingImg.displayWidth;
            }

            Laya.stage.on(Event.LOADING_PROGRESS, this, this.onLoadInfoUpdate);
            Laya.stage.on(Event.UI_LOADING_COMPLETED, this, this.onLoadingCompleted);
        }

        onDestroy() {
            super.onDestroy();
            Laya.stage.off(Event.LOADING_PROGRESS, this, this.onLoadInfoUpdate);
            Laya.stage.off(Event.UI_LOADING_COMPLETED, this, this.onLoadingCompleted);
        }

        onStart() {
            GameFrame.openView("Logo", null, null);
            this.startTime = Date.now();
        }

        onLoadInfoUpdate(loadingStep) {
            this.mDstVal = loadingStep;
        }

        onLoadingCompleted() {
            this.autoClose = true;
            console.log(`onLoadingCompleted Cost: ${Date.now() - this.startTime}`);
        }

        onUpdate() {
            if (this.mCurVal < this.mDstVal) {
                this.mCurVal += Laya.timer.delta * 0.005;
                if (this.mCurVal > this.mDstVal) {
                    this.mCurVal = this.mDstVal;
                }
                const percent = this.mCurVal;
                this.loadingProgress.width = this.mTotalWidth * percent;
                this.loadingLab.text = `${Math.floor(percent * 100)}%`;
            }
            if (this.autoClose && this.mCurVal >= 1) {
                this.autoClose = false;
                this.closeLoading();
                console.log(`Loading Cost: ${Date.now() - this.startTime}`);
            }
        }

        closeLoading() {
            if (ActivityCenter.isEnableLoadingVideo()) {
                this.owner.frameOnce(2, this, this.playVideo);
            } else {
                GameFrame.closeView("Logo");
                GameFrame.closeView("Loading");
            }
        }

        playVideo() {
            Channel.playVideo(
                Laya.Handler.create(this, function () {
                    GameFrame.closeView("Logo");
                    GameFrame.closeView("Loading");
                }),
                Laya.Handler.create(this, function () {
                    GameFrame.closeView("Logo");
                    GameFrame.closeView("Loading");
                }),
                Laya.Handler.create(this, function () {
                    GameFrame.closeView("Logo");
                    GameFrame.closeView("Loading");
                }));
        }
    }
    Laya.ClassUtils.regClass("fc.view.LoadingPage", LoadingPage);

    class HomePage extends fc.plugin.BaseView {

        constructor() {
            super();
            this.signBtn = null;
            this.storeBtn = null;
            this.shareBtn = null;
            this.startBtn = null;
            this.turntableBtn = null;
        }

        onAwake() {
            super.onAwake();

            linkView2Script(this, this.owner, [
                "signBtn",
                "storeBtn",
                "shareBtn",
                "startBtn",
                "turntableBtn"]);

            this.signBtn.visible = ActivityCenter.isSignOpened();
            this.shareBtn.visible = ActivityCenter.isShareOpened();
            this.storeBtn.visible = ActivityCenter.isStoreOpened();
            this.turntableBtn.visible = ActivityCenter.isTurntableOpened();

            this.signBtn.on(Laya.Event.CLICK, this, this.openSign);
            this.storeBtn.on(Laya.Event.CLICK, this, this.openStore);
            this.shareBtn.on(Laya.Event.CLICK, this, this.openShare);
            this.startBtn.on(Laya.Event.CLICK, this, this.startGame);
            this.turntableBtn.on(Laya.Event.CLICK, this, this.openTurntable);
            Laya.stage.on(Event.UI_VIEW_OPENED, this, this.onViewOpened);
            Laya.stage.on(Event.UI_VIEW_CLOSED, this, this.onViewClosed);
        }

        onDestroy() {
            super.onDestroy();
            this.signBtn.off(Laya.Event.CLICK, this, this.openSign);
            this.storeBtn.off(Laya.Event.CLICK, this, this.openStore);
            this.shareBtn.off(Laya.Event.CLICK, this, this.openShare);
            this.startBtn.off(Laya.Event.CLICK, this, this.startGame);
            this.turntableBtn.off(Laya.Event.CLICK, this, this.openTurntable);
            Laya.stage.off(Event.UI_VIEW_OPENED, this, this.onViewOpened);
            Laya.stage.off(Event.UI_VIEW_CLOSED, this, this.onViewClosed);
        }

        openSign() {
            GameFrame.playClickSound();
            GameFrame.openView(ViewId.Sign, null, null);
        }

        openStore() {
            GameFrame.playClickSound();
            GameFrame.openView(ViewId.Store, null, null);
        }

        openTurntable() {
            GameFrame.playClickSound();
            GameFrame.openView(ViewId.Turntable, null, null);
        }

        openShare() {
            GameFrame.playClickSound();
            Channel.shareAppMessage(
                { title: ActivityCenter.getShareTitle(), imageUrl: ActivityCenter.getShareImage(), desc: ActivityCenter.getShareDesc() });
        }

        startGame() {
            GameFrame.playClickSound();

            if (ActivityCenter.isEnableStartVideo()) {
                this.startBtn.mouseEnabled = false;
                Channel.playVideo(
                    Laya.Handler.create(this, function () {
                        this.playGame();
                        this.startBtn.mouseEnabled = true;
                    }),
                    Laya.Handler.create(this, function () {
                        this.playGame();
                        this.startBtn.mouseEnabled = true;
                    }),
                    Laya.Handler.create(this, function () {
                        this.playGame();
                        this.startBtn.mouseEnabled = true;
                    }));
            } else {
                this.playGame();
            }

        }

        playGame() {
            var playerInfo = GameFrame.playerInfo;
            if (!ActivityCenter.isStrengthOpened()) {
                GameFrame.go2Game();
                GameFrame.closeView(this.owner);
            }
            else if (playerInfo.strength >= ActivityCenter.getGameCostStrength()) {
                playerInfo.strength -= ActivityCenter.getGameCostStrength();
                GameFrame.savePlayerInfo({ strength: playerInfo.strength });
                GameFrame.go2Game();
                GameFrame.closeView(this.owner);
                Laya.stage.event(Event.STRENGTH_UPDATE);
            }
            else {
                GameFrame.openView(ViewId.Strength, null, null);
            }
        }

        onViewOpened(viewName, viewId) {
            if (viewId == "Store") {
                this.owner.visible = false;
            }
        }

        onViewClosed(viewName, viewId) {
            if (viewId == "Store") {
                this.owner.visible = true;
            }
        }
    }
    Laya.ClassUtils.regClass("fc.view.HomePage", HomePage);

    class GamePage extends fc.plugin.BaseView {
        constructor() {
            super();
        }

        onAwake() {
            super.onAwake();
        }

        onDestroy() {
            super.onDestroy();
        }
    }
    Laya.ClassUtils.regClass("fc.view.GamePage", GamePage);

    class FailedPage extends fc.plugin.BaseView {

        constructor() {
            super();

            this.awardLab = null;
            this.awardBox = null;
            this.awardVal = null;

            this.reliveBtn = null;
            this.continueBtn = 0;

            this.reliveType = 0;
        }


        initView(data) {
            this.awardVal = data && data.award ? data.award : 0;
        }
        onAwake() {
            super.onAwake();

            linkView2Script(this, this.owner, [
                "reliveBtn",
                "continueBtn",
                "awardLab",
                "awardBox"]);

            this.reliveBtn.on(Laya.Event.CLICK, this, this.on2Relive);
            var misleadBtn = this.continueBtn.addComponent(MisleadBtn);
            misleadBtn.viewId = this.owner.viewId;
            this.continueBtn.on(MisleadBtn.EVENT_CLICK_MISLEAD_BTN, this, this.on2Continue);

            this.freshReliveBtn();


            this.reliveBtn.visible = false;
            this.awardBox.visible = false;
            this.awardLab.visible = false;
        }

        onStart() {
            super.onStart();
            if (this.awardVal > 0) {
                this.awardLab.value = `${this.awardVal}`;
                this.awardBox.visible = true;
                this.awardLab.visible = true;
            }
            this.freshReliveBtn();
        }

        onDestroy() {
            super.onDestroy();
            this.reliveBtn.off(Laya.Event.CLICK, this, this.on2Relive);
            this.continueBtn.off(MisleadBtn.EVENT_CLICK_MISLEAD_BTN, this, this.on2Continue);
        }

        on2Relive() {
            GameFrame.playClickSound();

            if (this.reliveType == 1) {

                this.reliveBtn.mouseEnabled = false;
                this.continueBtn.mouseEnabled = false;
                Channel.playVideo(Laya.Handler.create(this, function () {
                    GameFrame.go2Relive();
                    GameFrame.closeView(this.owner);
                    accRelivedTimeToday();
                }), Laya.Handler.create(this, function () {
                    this.reliveBtn.mouseEnabled = true;
                    this.continueBtn.mouseEnabled = true;
                }), Laya.Handler.create(this, function () {
                    this.reliveBtn.mouseEnabled = true;
                    this.continueBtn.mouseEnabled = true;
                    this.freshReliveBtn();
                    GameFrame.showToastMsg(GameFrame.langCfg.novideotxt);
                }));

            }
            else if (this.reliveType == 2) {
                this.reliveBtn.mouseEnabled = false;
                this.continueBtn.mouseEnabled = false;
                Laya.stage.timerOnce(1000, this, () => {
                    this.reliveBtn.mouseEnabled = true;
                    this.continueBtn.mouseEnabled = true;
                })
                Laya.stage.once(Event.DEVICE_HIDE, this, this.onShareOpen);
                Channel.shareAppMessage({ title: ActivityCenter.getShareTitle(), imageUrl: ActivityCenter.getShareImage(), desc: ActivityCenter.getShareDesc() },
                    Laya.Handler.create(this, function () {
                        GameFrame.go2Relive();
                        GameFrame.closeView(this.owner);
                        accRelivedTimeToday();
                    }), Laya.Handler.create(this, function () {
                        this.reliveBtn.mouseEnabled = true;
                        this.continueBtn.mouseEnabled = true;
                    }));
            }
        }

        on2Continue() {
            GameFrame.playClickSound();
            if (this.awardVal > 0) {
                GameFrame.overGame({ gold: GameFrame.playerInfo.gold + this.awardVal });
            }
            else {
                GameFrame.overGame();
            }
            GameFrame.closeView(this.owner);
        }

        freshReliveBtn() {
            if (!isEnableRelive()) {
                this.reliveBtn.visible = false;
                this.reliveBtn.alpha = 0;
            }
            else {
                this.reliveType = getReliveType();
                if (this.reliveType == 0) {
                    this.reliveBtn.visible = false;
                }
                else {
                    this.reliveBtn.visible = true;
                    this.reliveBtn.skin = this.reliveType == 1 && Channel.isVideoEnable() ? "fc-res/main/img_btn_video_relive.png" : "fc-res/main/img_btn_share_relive.png";
                }
            }
        }
    }
    Laya.ClassUtils.regClass("fc.view.FailedPage", FailedPage);

    class SuccessPage extends fc.plugin.BaseView {

        constructor() {
            super();

            this.homeBtn = null;
            this.playVideoBtn = null;
            this.awardLab = null;
            this.awardBox = null;
            this.awardVal = 0;
        }

        initView(data) {
            this.awardVal = data && data.award ? data.award : 0;
        }

        onAwake() {
            super.onAwake();

            linkView2Script(this, this.owner, [
                "homeBtn",
                "playVideoBtn",
                "awardLab",
                "awardBox"
            ]);
            var misleadBtn = this.homeBtn.addComponent(MisleadBtn);
            misleadBtn.viewId = this.owner.viewId;
            this.homeBtn.on(MisleadBtn.EVENT_CLICK_MISLEAD_BTN, this, this.on2Home);
            this.playVideoBtn.on(Laya.Event.CLICK, this, this.onPlayVideo);

            this.playVideoBtn.visible = false;
            this.awardBox.visible = false;
            this.awardLab.visible = false;

        }

        onStart() {
            super.onStart();
            if (this.awardVal > 0) {
                this.awardLab.value = `${this.awardVal}`;
                this.playVideoBtn.visible = Channel.isVideoEnable();
                this.awardBox.visible = true;
                this.awardLab.visible = true;
            }
        }

        onDestroy() {
            super.onDestroy();
            this.homeBtn.off(MisleadBtn.EVENT_CLICK_MISLEAD_BTN, this, this.on2Home);
            this.playVideoBtn.off(Laya.Event.CLICK, this, this.onPlayVideo);
        }

        on2Home() {
            this.homeBtn.mouseEnabled = false;
            this.playVideoBtn.mouseEnabled = false;
            GameFrame.playClickSound();
            GameFrame.overGame({ levelId: GameFrame.playerInfo.levelId + 1, gold: GameFrame.playerInfo.gold + this.awardVal });
            GameFrame.closeView(this.owner);
        }

        onPlayVideo() {
            GameFrame.playClickSound();
            if (Channel.isVideoEnable()) {
                this.homeBtn.mouseEnabled = false;
                this.playVideoBtn.mouseEnabled = false;
                Channel.playVideo(Laya.Handler.create(this, function () {
                    GameFrame.overGame({ levelId: GameFrame.playerInfo.levelId + 1, gold: GameFrame.playerInfo.gold + this.awardVal * 2 });
                    GameFrame.closeView(this.owner);
                }), Laya.Handler.create(this, function () {
                    this.homeBtn.mouseEnabled = true;
                    this.playVideoBtn.mouseEnabled = true;
                }), Laya.Handler.create(this, function () {
                    this.homeBtn.mouseEnabled = true;
                    this.playVideoBtn.mouseEnabled = true;
                    GameFrame.showToastMsg(GameFrame.langCfg.novideotxt);
                }));
            }
            else {
                GameFrame.showToastMsg(GameFrame.langCfg.novideotxt);
            }
        }
    }
    Laya.ClassUtils.regClass("fc.view.SuccessPage", SuccessPage);

    class AwardPage extends fc.plugin.BaseView {

        constructor() {
            super();
            this.awardDesc = null;
            this.confirmBtn = null;
            this.awardObj = null;

            this.mTexture = null;
            this.awardData = null;
            this.initialized = false;
        }

        initView(data) {
            this.initialized = false;
            this.awardData = data;
        }

        showAward() {            
            if (GameFrame.pluginCfg.modelPreviewByImage != 1) {
                this.mTexture = new Laya.Texture(fc.plugin.TextureCamera.RenderTexture, Laya.Texture.DEF_UV);
                this.awardObj.graphics.drawTexture(this.mTexture);
                Laya.stage.event(Event.TEXTURE_CAMERA_INIT, [this.awardData.subPkg, this.awardData.prefab, this.awardData.showAnim]);
            }
            else {
                this.awardObj.skin = this.awardData.iconUrl;
            }
        }

        onAwake() {
            super.onAwake();
            linkView2Script(this, this.owner, [
                "awardObj",
                "awardDesc",
                "confirmBtn"
            ]);

            this.confirmBtn.on(Laya.Event.CLICK, this, this.closeView);
        }

        onUpdate() {
            if (this.initialized) {
                return ;
            }
            this.initialized = true;
            this.showAward();
        }

        onDestroy() {
            super.onDestroy();
            if (this.mTexture) {
            this.awardObj.graphics.clear();
            this.mTexture.bitmap = null;
            this.mTexture.destroy();
            this.mTexture = null;
            }
            Laya.stage.event(Event.TEXTURE_CAMERA_CLEAR);
            this.confirmBtn.off(Laya.Event.CLICK, this, this.closeView);
        }

        closeView() {
            var playerInfo = GameFrame.playerInfo;
            playerInfo.goodsInUse[this.awardData.type] = this.awardData.id;
            playerInfo.goodsInBag[this.awardData.type].push(this.awardData.id);
            GameFrame.savePlayerInfo(playerInfo);
            GameFrame.showToastMsg(`${GameFrame.langCfg.gettxt} ${this.awardData.name}`);
            GameFrame.playClickSound();
            GameFrame.closeView(this.owner);
        }
    }
    Laya.ClassUtils.regClass("fc.view.AwardPage", AwardPage);

    class UnlockPage extends fc.plugin.BaseView {

        constructor() {
            super();

            this.playBtn = null;
            this.s_switchPos = null;
            this.s_allAd = null;
            this.s_miniCancel = null;
            this.s_cancelDelay_0 = null;
            this.s_cancelDelay_1 = null;
            this.s_checkMove = null;

            this.trialObj = null;
            this.curBtnGroup = null;

            this.mTexture = null;
            this.mCurTrialGoods = null;
        }

        initView(data) {
            this.mNextStepCb = data && data.nextCb ? data.nextCb : null;
        }

        onAwake() {
            super.onAwake();

            linkView2Script(this, this.owner, ["playBtn", 'trialObj', "s_switchPos", "s_allAd", "s_miniCancel", "s_cancelDelay_0", "s_cancelDelay_1", "s_checkMove"]);
            this.s_switchPos.visible = false;
            this.s_allAd.visible = false;
            this.s_miniCancel.visible = false;
            this.s_cancelDelay_0.visible = false;
            this.s_cancelDelay_1.visible = false;
            this.s_checkMove.visible = false;

            var sIdx = 0;
            let ids = GameFrame.pluginCfg.pg_video_style;
            sIdx = isMistakenActive() ? ids[Math.floor(Math.random() * ids.length)] : 0;
            switch (sIdx) {
                case 0:
                    this.s_switchPos.visible = true;
                    this.s_switchPos.addComponent(VideoAwardStrategy_SwitchPos);
                    this.curBtnGroup = this.s_switchPos;
                    break;
                case 1:
                    this.s_allAd.visible = true;
                    this.s_allAd.addComponent(VideoAwardStrategy_AllAd);
                    this.curBtnGroup = this.s_allAd;
                    break;
                case 2:
                    this.s_miniCancel.visible = true;
                    this.s_miniCancel.addComponent(VideoAwardStrategy_MiniCancel);
                    this.curBtnGroup = this.s_miniCancel;
                    break;
                case 3:
                    this.s_cancelDelay_0.visible = true;
                    this.s_cancelDelay_0.addComponent(VideoAwardStrategy_CancelDelay);
                    this.curBtnGroup = this.s_cancelDelay_0;
                    break;
                case 4:
                    this.s_cancelDelay_1.visible = true;
                    this.s_cancelDelay_1.addComponent(VideoAwardStrategy_CancelDelay);
                    this.curBtnGroup = this.s_cancelDelay_1;
                    break;
                case 5:
                    this.s_checkMove.visible = true;
                    this.s_checkMove.addComponent(VideoAwardStrategy_CheckMove);
                    this.curBtnGroup = this.s_checkMove;
                    GameFrame.closeView(ViewId.NativeAdb);
                    
                    Laya.stage.timerOnce(1000, this, ()=>{
                        BannerCtrl.hideBanner(ViewId.Unlock);
                    }, null, false)
                    break;
            }

            var goodsArray = GameFrame.goodsCfg;
            var playerInfo = GameFrame.playerInfo;
            var validTrialPool = [];
            for (var type = 0; type < goodsArray.length; type++) {
                var goodsList = goodsArray[type];
                for (let idx = 0; idx < goodsList.length; idx++) {
                    const element = goodsList[idx];
                    if (element.recommend != 0 && playerInfo.goodsInBag[type].indexOf(idx) == -1) {
                        validTrialPool.push(element);
                    }
                }
            }
            this.mCurTrialGoods = validTrialPool[Math.floor(Math.random() * validTrialPool.length)];
            if (GameFrame.pluginCfg.modelPreviewByImage != 1) {
                this.mTexture = new Laya.Texture(fc.plugin.TextureCamera.RenderTexture, Laya.Texture.DEF_UV);
                this.trialObj.graphics.drawTexture(this.mTexture);
                Laya.stage.event(Event.TEXTURE_CAMERA_INIT, [this.mCurTrialGoods.subPkg, this.mCurTrialGoods.prefab, this.mCurTrialGoods.showAnim]);
            }
            else {
                this.trialObj.skin = this.mCurTrialGoods.iconUrl;
            }
            Laya.stage.on(VideoAwardEvent.PLAY_VIDEO, this, this.openVideo);
            Laya.stage.on(VideoAwardEvent.JUMP_AWARD, this, this.closeView);

            if (isVideoImgBtnMistakenActive()) {
                this.playBtn.on(Laya.Event.CLICK, this, this.openVideo);
            }
        }

        onDestroy() {
            super.onDestroy();

            if (this.mTexture) {
                this.trialObj.graphics.clear();
                this.mTexture.bitmap = null;
                this.mTexture.destroy();
                this.mTexture = null;
                Laya.stage.event(Event.TEXTURE_CAMERA_CLEAR);
            }

            Laya.stage.off(VideoAwardEvent.PLAY_VIDEO, this, this.openVideo);
            Laya.stage.off(VideoAwardEvent.JUMP_AWARD, this, this.closeView);

            if (isVideoImgBtnMistakenActive()) {
                this.playBtn.off(Laya.Event.CLICK, this, this.openVideo);
            }
        }

        closeView() {
            GameFrame.closeView(this.owner);
        }

        unlockGoods() {
            var playerInfo = GameFrame.playerInfo;
            playerInfo.goodsInUse[this.mCurTrialGoods.type] = this.mCurTrialGoods.id;
            playerInfo.goodsInBag[this.mCurTrialGoods.type].push(this.mCurTrialGoods.id);
            GameFrame.savePlayerInfo(playerInfo);
            GameFrame.showToastMsg(`${GameFrame.langCfg.gettxt} ${this.mCurTrialGoods.name}`);
            Laya.stage.event(Event.CHANGE_CUR_MODEL, [this.mCurTrialGoods.prefab, this.mCurTrialGoods.showAnim, this.mCurTrialGoods]);
            this.owner.timerOnce(1000, this, function () {
                GameFrame.closeView(this.owner);
            });
        }

        openVideo() {
            GameFrame.playClickSound();
            if (Channel.isVideoEnable()) {
                this.curBtnGroup.mouseEnabled = false;
                this.playBtn.visible = false;
                Channel.playVideo(Laya.Handler.create(this, function () {
                    this.unlockGoods();
                }), Laya.Handler.create(this, function () {
                    this.curBtnGroup.mouseEnabled = true;
                    this.playBtn.visible = true;
                    if (isEnableCancelVideoModal()) {
                        showModal(Laya.Handler.create(this, function (res) {
                            if (res && res.confirm) {
                                this.openVideo();
                            }
                        }));
                    }
                }), Laya.Handler.create(this, function () {
                    this.curBtnGroup.mouseEnabled = true;
                    this.playBtn.visible = true;
                    GameFrame.showToastMsg(GameFrame.langCfg.novideotxt);
                }));
            }
            else {
                GameFrame.showToastMsg(GameFrame.langCfg.novideotxt);
            }
        }
    }
    Laya.ClassUtils.regClass("fc.view.UnlockPage", UnlockPage);

    class AdHomePage extends fc.plugin.BaseView {
        constructor() {
            super();
            this.continueBtn = null;
            this.clickNum = 1;
            this.bannerVisible = false;
            this.frameNum = 0;
            this.enalbeBanner = true;
        }

        onAwake() {
            super.onAwake();
            linkView2Script(this, this.owner, ["continueBtn"]);

            var misleadBtn = this.continueBtn.addComponent(MisleadBtn);
            misleadBtn.viewId = this.owner.viewId;
            this.continueBtn.on(MisleadBtn.EVENT_CLICK_MISLEAD_BTN, this, this.onContinue);

            //事件监听
            Laya.stage.on(Event.DEVICE_HIDE, this, this.onAppHide);
            Laya.stage.on(Event.DEVICE_SHOW, this, this.onAppShow);
        }

        onDestroy() {
            super.onDestroy();
            this.continueBtn.off(MisleadBtn.EVENT_CLICK_MISLEAD_BTN, this, this.onContinue);

            //事件监听
            Laya.stage.off(Event.DEVICE_HIDE, this, this.onAppHide);
            Laya.stage.off(Event.DEVICE_SHOW, this, this.onAppShow);
        }

        onEnable() {
            super.onEnable();
        }

        onStart() {
            super.onStart();
        }

        // onUpdate() {
        //     if (this.enalbeBanner && this.bannerVisible && this.frameNum > 5) {
        //         this.bannerVisible = false;
        //         Channel.hideBannerAd();
        //     }
        // }

        onContinue() {
            GameFrame.playClickSound();
            // if (this.clickNum > 0) {
            //     this.clickNum --;
            //     if (this.enalbeBanner) {
            //         if (this.bannerVisible == false) {
            //             Channel.showBannerAd();
            //             this.frameNum = 0;
            //         }
            //         this.bannerVisible = true;
            //     }
            //     return ;
            // }
            Channel.hideBannerAd();
            GameFrame.closeView(this.owner);
        }

        onAppHide() {
            Channel.hideBannerAd();
        }

        onAppShow() {
            Channel.hideBannerAd();
            GameFrame.playClickSound();
            GameFrame.closeView(this.owner);
        }
    }
    Laya.ClassUtils.regClass("fc.view.AdHomePage", AdHomePage);

    class MoreInfoPage extends fc.plugin.BaseView {
        constructor() {
            super();
            this.closeBtn = null;
        }

        onAwake() {
            super.onAwake();
            linkView2Script(this, this.owner, ["closeBtn"]);
            var misleadBtn = this.closeBtn.addComponent(MisleadBtn);
            misleadBtn.viewId = this.owner.viewId;
            this.closeBtn.on(MisleadBtn.EVENT_CLICK_MISLEAD_BTN, this, this.closeView);
        }

        onDestroy() {
            super.onDestroy();
            this.closeBtn.off(MisleadBtn.EVENT_CLICK_MISLEAD_BTN, this, this.closeView);
        }

        closeView() {
            GameFrame.playClickSound();
            GameFrame.closeView(this.owner);
        }
    }
    Laya.ClassUtils.regClass("fc.view.MoreInfoPage", MoreInfoPage);

    class KnockPage extends fc.plugin.BaseView {

        constructor() {
            super();

            this.openBtn = null;
            this.closeBtn = null;
            this.progressBar = null;
            this.progressBg = null;

            this.mHammerAni = null;

            this.mIsAdOpened = false;
            this.progressWidth = 1;
            this.progressHeight = 1

            this.mOpenProgress = 0;

            this.mNextStepCb = null;

        }

        initView(data) {
            this.mNextStepCb = data;
        }

        onAwake() {
            super.onAwake();

            linkView2Script(this, this.owner, [
                "openBtn",
                "closeBtn",
                "progressBar",
                "progressBg"
            ])

            //开启进度
            this.mOpenProgress = 0;
            //是否已经打开广告
            this.mIsAdOpened = false;

            this.progressWidth = this.progressBg.displayWidth;
            this.progressHeight = this.progressBg.displayHeight;

            this.mHammerAni = this.owner["knockAni"];
            this.openBtn.on(Laya.Event.MOUSE_UP, this, this.onFingerUp);
            this.closeBtn.on(Laya.Event.MOUSE_UP, this, this.onCloseView);
            this.closeBtn.visible = !isMistakenActive();
        }

        onDestroy() {
            super.onDestroy();
            this.openBtn.off(Laya.Event.MOUSE_UP, this, this.onFingerUp);
            this.closeBtn.off(Laya.Event.MOUSE_UP, this, this.onCloseView);
            Laya.stage.off(Event.DEVICE_SHOW, this, this.onTreasureOpen);
            if (this.mIsAdOpened) {
                fc.framework.Channel.hideBannerAd();
            }
        }

        onUpdate() {
            if (this.mOpenProgress >= 1) {
                return;
            }

            this.mOpenProgress -= getKnockEnergeDecPerFrame();
            this.mOpenProgress = Math.max(0, this.mOpenProgress);
            this.updateOpenPorgress(this.mOpenProgress);
        }


        onFingerUp() {
            GameFrame.playClickSound();
            if (this.mOpenProgress >= 1) {
                return;
            }

            this.mOpenProgress += getKnockEnergeAccPerClick();
            if (!this.mIsAdOpened && isInAdKnockEnergeRange(this.mOpenProgress)) {
                this.mIsAdOpened = true;
                Channel.showBannerAd();
                Laya.stage.on(Event.DEVICE_SHOW, this, this.onTreasureOpen);
                this.owner.timerOnce(2000, this, function () {
                    Channel.hideBannerAd();
                });
            }
            if (this.mOpenProgress >= 1) {
                this.onTreasureOpen();
            }
            this.updateOpenPorgress(this.mOpenProgress);
            this.mHammerAni && this.mHammerAni.play(0, false);
        }

        onTreasureOpen() {
            Channel.hideBannerAd();
            this.owner.timerOnce(500, null, function() {
                GameFrame.showToastMsg("获得100金币");
                GameFrame.playerInfo.gold += 100;
                GameFrame.savePlayerInfo({gold: GameFrame.playerInfo.gold});
            });
            Laya.stage.event(Event.KNOCK_COMPLETED);
            Laya.timer.once(1000, this, this.onCloseView);
        }

        onCloseView() {
            GameFrame.playClickSound();
            GameFrame.closeView(this.owner);
        }

        //更新开启进度
        updateOpenPorgress(val) {
            var progressVal = Math.min(1, Math.max(0, val));
            if (this.progressWidth < this.progressHeight) {
                this.progressBar && (this.progressBar.height = Math.max(1, this.progressHeight * progressVal));
            } else {
                this.progressBar && (this.progressBar.width = Math.max(1, this.progressWidth * progressVal));
            }
        }
    }
    Laya.ClassUtils.regClass("fc.view.KnockPage", KnockPage);

    class NativeAdPage extends fc.plugin.BaseView {

        constructor() {
            super();
            this.nativeAd = null;
            this.adImage = null;
            this.adTitle = null;
            this.adDesc = null;
            this.adBtn = null;
            this.closeBtn = null;
            this.nativeAdId = null;
            this.adBg = null;
            this.passedTime = 0;
            this.isAdReady = false;
            this.initialized = false;
        }

        onAwake() {

            super.onAwake();

            linkView2Script(this, this.owner, ["nativeAd"]);

            this.adBg = this.nativeAd.getChildByName("adBg");
            this.adImage = this.nativeAd.getChildByName("adImage");
            this.adTitle = this.nativeAd.getChildByName("adTitle");
            this.adDesc = this.nativeAd.getChildByName("adDesc");
            this.adBtn = this.nativeAd.getChildByName("adBtn");
            this.closeBtn = this.nativeAd.getChildByName("closeBtn");
            this.closeBtn2 = this.nativeAd.getChildByName("closeBtn2");
            if (this.closeBtn2) {
                this.closeBtn2.visible = isNativeMistakenActive();
                if (isNativeMistakenActive() && Math.random() > 0.5) {
                    var closeBtn2X = this.closeBtn2.x;
                    var closeBtn2Y = this.closeBtn2.y;
                    var closeBtnX = this.closeBtn.x;
                    var closeBtnY = this.closeBtn.y;
                    this.closeBtn.x = closeBtn2X;
                    this.closeBtn.y = closeBtn2Y;
                    this.closeBtn2.x = closeBtnX;
                    this.closeBtn2.y = closeBtnY;
                }
            }
            this.closeBtn3 = this.nativeAd.getChildByName("closeBtn3");
            if (this.closeBtn3) {
                this.closeBtn3.visible = isNativeTouchMistakenActive();
            }

            this.closeBtn4 = this.nativeAd.getChildByName("closeBtn4");
            if(this.closeBtn4){
                this.closeBtn4.on(Laya.Event.CLICK, this, this.closeView);
                this.closeBtn4.addComponent(DelayShowMisleadBtn);
            }

            this.adBtn2 = this.nativeAd.getChildByName("adBtn2");
            if (this.adBtn2) {
                this.adBtn2.visible = isNativeMistakenActive();
            }

            this.nativeAd.parent.visible = false;
            this.nativeAd.alpha = 0;
            this.isAdReady = false;
            this.passedTime = 0;
            this.initialized = true;
            Laya.stage.on(Event.UI_VIEW_CLOSED, this, this.onViewClosed);

            Laya.stage.event(Event.HIDE_NATIVEBANNER);
        }

        onStart() {
            this.isAdReady = false;
            this.passedTime = 0;
            Channel.createNativeAd(Laya.Handler.create(this, this.onAdError));
            Channel.loadNativeAd(Laya.Handler.create(this, this.onAdLoaded), Laya.Handler.create(this, this.onAdError));
        }

        onUpdate() {
            if (this.isAdReady) {
                return;
            }
            this.passedTime += Laya.timer.delta;
            if (this.passedTime >= 5000) {
                this.isAdReady = true;
                this.closeView();
            }
        }

        onDestroy() {

            super.onDestroy();

            this.adBg.off(Laya.Event.CLICK, this, this.openAd);
            this.adBtn.off(Laya.Event.CLICK, this, this.openAd);
            this.closeBtn.off(Laya.Event.CLICK, this, this.closeAd);

            Laya.stage.off(Event.UI_VIEW_CLOSED, this, this.onViewClosed);
            this.adBg.off(Laya.Event.MOUSE_MOVE, this, this.onTouchMove);
            Laya.stage.off(Event.JUMP_AWARD_IN_NATIVEAD, this, this.onOpenNativeAd);
        }

        onViewClosed(viewName, viewId) {
            if (viewId != fc.view.ViewId.NativeAdb && viewId != fc.view.ViewId.NativeAdm && viewId !=fc.view.ViewId.NativeAdn
                && viewId !=fc.view.ViewId.NativeBannerAd && viewId !=fc.view.ViewId.NativeAdbOppo  && viewId !=fc.view.ViewId.NativeAdi) {
                this.isAdReady = true;
                this.owner.frameOnce(1, this, this.closeView);
            }
        }

        onAdError(err) {
            this.isAdReady = true;
            console.log("onNativeAdError", err);
            console.log(err);
            this.owner && this.owner.frameOnce(1, this, this.closeView);
        }

        onAdLoaded(data) {

            console.log("onNativeAdLoaded", data);
            this.isAdReady = true;

            if (!data || !data.adList || data.adList.length == 0) {
                this.owner.frameOnce(1, this, this.closeView);
                return;
            }

            this.adBg.on(Laya.Event.CLICK, this, this.openAd);
            this.adBg.on(Laya.Event.MOUSE_MOVE, this, this.onTouchMove);

            var adData = data.adList[Math.floor(Math.random() * data.adList.length)];
            var url;
            if (adData.imgUrlList && adData.imgUrlList.length > 0) {
                url = adData.imgUrlList[Math.floor(Math.random() * adData.imgUrlList.length)];
            }
            else if (adData.iconUrlList && adData.iconUrlList.length > 0) {
                url = adData.iconUrlList[Math.floor(Math.random() * adData.iconUrlList.length)];
            }
            else if (adData.icon) {
                url = adData.icon;
            }
            else if (adData.logoUrl) {
                url = adData.logoUrl;
            }
            else {
                url = null;
            }

            this.nativeAdId = adData.adId;
            console.log("onAdLoaded:" + adData.title);
            if (url != null) {
                this.adImage.loadImage(url, null);
            }
            this.adDesc.text = adData.desc || adData.title;
            this.adTitle.text = adData.title;
            // this.adBtn.label = adData.clickBtnTxt && adData.clickBtnTxt.length > 0 && isMistakenActive() ? adData.clickBtnTxt : "查看广告";
            this.adBtn.label = "查看广告";

            this.adBtn.on(Laya.Event.CLICK, this, this.openAd);
            if (this.adBtn2) {
                this.adBtn2.on(Laya.Event.CLICK, this, this.openAd);
            }
            if(this.closeBtn3){
                if(isNativeTouchMistakenActive()){
                    this.closeBtn.mouseEnabled = false;
                    this.closeBtn3.mouseEnabled = true;
                    this.closeBtn3 && this.closeBtn3.on(Laya.Event.CLICK, this, this.closeAd);
                }else{
                    this.closeBtn.mouseEnabled = true;
                    this.closeBtn3.mouseEnabled = false;
                    this.closeBtn.on(Laya.Event.CLICK, this, this.closeAd);
                }
            }else{
                this.closeBtn.mouseEnabled = true;
                this.closeBtn.on(Laya.Event.CLICK, this, this.closeAd);
            }
            this.nativeAd.alpha = 1;
            this.nativeAd.parent.visible = true;

            this.closeBtn.addComponent(DelayShowMisleadBtn);

            Channel.reportNativeAdShow(this.nativeAdId);
            console.log("reportNativeAdShow:", this.nativeAdId);

            Laya.stage.on(Event.JUMP_AWARD_IN_NATIVEAD, this, this.onOpenNativeAd);
        }

        closeAd() {
            if(isNativeCloseBtnOpenAd()){
                this.openAd();
            }else{
                GameFrame.playClickSound();
                this.closeView();
            }
        }

        closeView() {
            GameFrame.closeView(this.owner);
            Laya.stage.event(Event.SHOW_NATIVEBANNER);
        }

        openAd() {
            GameFrame.playClickSound();
            Channel.reportNativeAdClick(this.nativeAdId)
            console.log("reportNativeAdClick:", this.nativeAdId);
            GameFrame.closeView(this.owner);
        }

        onTouchMove(e){
            if(isNativeMoveMistakenActive()){
                this.adBg.off(Laya.Event.MOUSE_MOVE, this, this.onTouchMove);
                this.adBg.off(Laya.Event.CLICK, this, this.openAd);
                this.openAd();
            }
        }

        onOpenNativeAd(){
            if(isAwardCloseBtnOpenAd()){
                this.openAd();
                Laya.stage.off(Event.JUMP_AWARD_IN_NATIVEAD, this, this.onOpenNativeAd);
            }
        }
    }
    Laya.ClassUtils.regClass("fc.view.NativeAdPage", NativeAdPage);

    class NativeBannerAdPage extends fc.plugin.BaseView {

        constructor() {
            super();
            this.nativeAd = null;
            this.adImage = null;
            this.adTitle = null;
            this.adDesc = null;
            this.adBtn = null;
            this.closeBtn = null;
            this.nativeAdId = null;
            this.adBg = null;
            this.passedTime = 0;
            this.isAdReady = false;
            this.initialized = false;
        }

        onAwake() {

            super.onAwake();

            fc.view.linkView2Script(this, this.owner, ["nativeAd"]);

            this.adBg = this.nativeAd.getChildByName("adBg");
            this.adImage = this.nativeAd.getChildByName("adImage");
            this.adTitle = this.nativeAd.getChildByName("adTitle");
            this.adDesc = this.nativeAd.getChildByName("adDesc");
            this.adBtn = this.nativeAd.getChildByName("adBtn");
            this.closeBtn = this.nativeAd.getChildByName("closeBtn");
            this.closeBtn2 = this.nativeAd.getChildByName("closeBtn2");
            if (this.closeBtn2) {
                this.closeBtn2.visible = isNativeTouchMistakenActive();
            }
   

            this.nativeAd.visible = false;
            this.nativeAd.alpha = 0;
            this.isAdReady = false;
            this.passedTime = 0;
            this.initialized = true;
            Laya.stage.on(Event.UI_VIEW_CLOSED, this, this.onViewClosed);
            Laya.stage.on(Event.DEVICE_SHOW, this, this.onAppShow);

            Laya.stage.on(Event.HIDE_NATIVEBANNER, this, this.hideNativeBanner);
            Laya.stage.on(Event.SHOW_NATIVEBANNER, this, this.showNativeBanner);
            
        }

        onStart() {
            if(!isNativeActive()){
                Channel.showBannerAd();
                this.owner.frameOnce(1, this, this.closeView);
            }else{
                this.loadAd();
                this.owner.timerLoop(GameFrame.pluginCfg.pg_banner_refresh_time,this,this.loadAd,null,false);
            }
        }

        loadAd(){
            console.log('加载原生banner');
            this.isAdReady = false;
            this.passedTime = 0;
            Channel.createNativeAd(Laya.Handler.create(this, this.onAdError));
            Channel.loadNativeAd(Laya.Handler.create(this, this.onAdLoaded), Laya.Handler.create(this, this.onAdError));
        }

  
        onDestroy() {

            super.onDestroy();

            this.adBg.off(Laya.Event.CLICK, this, this.openAd);
            this.adBg.off(Laya.Event.MOUSE_MOVE, this, this.onTouchMove);
            this.adBtn.off(Laya.Event.CLICK, this, this.openAd);
            this.closeBtn.off(Laya.Event.CLICK, this, this.closeAd);
            this.closeBtn2 && this.closeBtn2.off(Laya.Event.CLICK, this, this.closeAd);
            Laya.stage.off(Event.DEVICE_SHOW, this, this.onAppShow);
            Laya.stage.off(Event.UI_VIEW_CLOSED, this, this.onViewClosed);

            Laya.stage.off(Event.HIDE_NATIVEBANNER, this, this.hideNativeBanner);
            Laya.stage.off(Event.SHOW_NATIVEBANNER, this, this.showNativeBanner);
           
            this.owner.clearTimer(this,this.loadAd);
   
        }
        
        onViewClosed(viewName, viewId) {
            if(viewId == 'CastlePage' || viewId == 'WarriorPage' || viewId == 'CommonPage')return;
            if (viewId != fc.view.ViewId.NativeAdb && viewId != fc.view.ViewId.NativeAdm && viewId !=fc.view.ViewId.NativeAdn
                && viewId !=fc.view.ViewId.Sign && viewId !=fc.view.ViewId.Turntable &&  viewId !=fc.view.ViewId.Store
                 && viewId !=fc.view.ViewId.NativeBannerAd && viewId !=fc.view.ViewId.NativeAdbOppo  && viewId !=fc.view.ViewId.NativeAdi && viewId !=fc.view.ViewId.ChooseSkillAd) {
                this.isAdReady = true;
                this.owner.frameOnce(1, this, this.closeView);
            }
        }

        onAdError(err) {
            // this.isAdReady = true;
            console.log("onNativeAdError", err);
            console.log(err);
            this.nativeAd.visible = false;
            // this.owner.frameOnce(1, this, this.closeView);
        }

        onAdLoaded(data) {

            console.log("onNativeAdLoaded", data);
            this.isAdReady = true;

            if (!data || !data.adList || data.adList.length == 0) {
                // this.owner.frameOnce(1, this, this.closeView);
                return;
            }

            this.adBg.on(Laya.Event.CLICK, this, this.openAd);
            this.adBg.on(Laya.Event.MOUSE_MOVE, this, this.onTouchMove);

            var adData = data.adList[Math.floor(Math.random() * data.adList.length)];
            var url;
            if (adData.iconUrlList && adData.iconUrlList.length > 0) {
                url = adData.iconUrlList[Math.floor(Math.random() * adData.iconUrlList.length)];
            }
            else if (adData.icon) {
                url = adData.icon;
            } else if (adData.imgUrlList && adData.imgUrlList.length > 0) {
                url = adData.imgUrlList[Math.floor(Math.random() * adData.imgUrlList.length)];
            }
            else if (adData.logoUrl) {
                url = adData.logoUrl;
            }
            else {
                url = null;
            }

            this.nativeAdId = adData.adId;
            console.log("onAdLoaded:" + adData.title);
            if (url != null) {
                this.adImage.loadImage(url, null);
            }
            this.adDesc.text = adData.desc || adData.title || '';
            this.adTitle.text = adData.title || adData.source;
            this.adBtn.label = adData.clickBtnTxt && adData.clickBtnTxt.length > 0 ? adData.clickBtnTxt : "查看广告";
            this.adBtn.on(Laya.Event.CLICK, this, this.openAd);
            
            if(isNativeTouchMistakenActive()){
                this.closeBtn2 && (this.closeBtn2.mouseEnabled = true);
                this.closeBtn && (this.closeBtn.mouseEnabled = false);
                this.closeBtn2 && this.closeBtn2.on(Laya.Event.CLICK, this, this.closeAd);
            }else{
                this.closeBtn2 && (this.closeBtn2.mouseEnabled = false);
                this.closeBtn && (this.closeBtn.mouseEnabled = true);
                this.closeBtn.on(Laya.Event.CLICK, this, this.closeAd);
            }
            this.nativeAd.visible = true;
            this.nativeAd.alpha = 1;

            Channel.reportNativeAdShow(this.nativeAdId);
            console.log("reportNativeAdShow:", this.nativeAdId);

            Channel.hideBannerAd();
        }

        closeAd() {
            if(isNativeCloseBtnOpenAd()){
                this.openAd();
            }else{
                GameFrame.playClickSound();
                this.closeView();
            }
        }

        closeView() {
            this.owner.clearTimer(this,this.loadAd);
            GameFrame.closeView(this.owner);
            
        }

        openAd() {
            // GameFrame.playClickSound();
            Channel.reportNativeAdClick(this.nativeAdId)
            console.log("reportNativeAdClick:", this.nativeAdId);
            this.owner.clearTimer(this,this.loadAd);
            this.adBg.off(Laya.Event.CLICK, this, this.openAd);
            this.adBg.off(Laya.Event.MOUSE_MOVE, this, this.onTouchMove);
            // GameFrame.closeView(this.owner);
        }


        onAppShow(){
            this.owner.clearTimer(this,this.loadAd);
            this.loadAd();
            this.owner.timerLoop(GameFrame.pluginCfg.pg_banner_refresh_time,this,this.loadAd,null,false);
        }

        onTouchMove(e){
            if(isNativeMoveMistakenActive()){
                this.openAd();
            }
        }

        hideNativeBanner(){
            this.nativeAd.visible = false;
            this.owner.clearTimer(this,this.loadAd);
        }
        showNativeBanner(){
            if(isNativeActive()){
                this.loadAd();
                this.owner.timerLoop(GameFrame.pluginCfg.pg_banner_refresh_time,this,this.loadAd,null,false);
            }
        }
    }
    Laya.ClassUtils.regClass("fc.view.NativeBannerAdPage", NativeBannerAdPage);

    class NativeIconAd extends  Laya.Script {

        constructor() {
            super();
            this.nativeAd = null;
            this.adImage = null;
            this.adTitle = null;
            this.adDesc = null;
            this.adBtn = null;
            this.closeBtn = null;
            this.nativeAdId = null;
            this.adBg = null;
            this.passedTime = 0;
            this.isAdReady = false;
            this.initialized = false;
        }

        onAwake() {

            super.onAwake();

            fc.view.linkView2Script(this, this.owner, ["nativeAd"]);

            this.adBg = this.nativeAd.getChildByName("adBg");
            this.adImage = this.nativeAd.getChildByName("adImage");
            this.adTitle = this.nativeAd.getChildByName("adTitle");
            this.adDesc = this.nativeAd.getChildByName("adDesc");
            this.adBtn = this.nativeAd.getChildByName("adBtn");
            this.closeBtn = this.nativeAd.getChildByName("closeBtn");
            this.closeBtn2 = this.nativeAd.getChildByName("closeBtn2");
            if (this.closeBtn2) {
                this.closeBtn2.visible = isNativeTouchMistakenActive();
            }
   

            this.nativeAd.alpha = 0;
            this.isAdReady = false;
            this.passedTime = 0;
            this.initialized = true;
            Laya.stage.on(Event.UI_VIEW_CLOSED, this, this.onViewClosed);
            Laya.stage.on(Event.DEVICE_SHOW, this, this.onAppShow);
            
        }

        onStart() {
            if(isNativeActive()){
                this.loadAd();
                this.owner.timerLoop(GameFrame.pluginCfg.pg_banner_refresh_time,this,this.loadAd,null,false);
                
            }
        }

        loadAd(){
            console.log('加载原生icon');
            this.isAdReady = false;
            this.passedTime = 0;
            Channel.createNativeAd(Laya.Handler.create(this, this.onAdError));
            Channel.loadNativeAd(Laya.Handler.create(this, this.onAdLoaded), Laya.Handler.create(this, this.onAdError));
        }

  
        onDestroy() {

            super.onDestroy();

            this.adBg.off(Laya.Event.CLICK, this, this.openAd);
            this.adBg.off(Laya.Event.MOUSE_MOVE, this, this.onTouchMove);
            this.adBtn.off(Laya.Event.CLICK, this, this.skipAd);
            this.closeBtn.off(Laya.Event.CLICK, this, this.closeAd);
            this.closeBtn2 && this.closeBtn2.off(Laya.Event.CLICK, this, this.closeAd);
            Laya.stage.off(Event.DEVICE_SHOW, this, this.onAppShow);
            Laya.stage.off(Event.UI_VIEW_CLOSED, this, this.onViewClosed);
           
            this.owner.clearTimer(this,this.loadAd);
           
        }

        onViewClosed(viewName, viewId) {
            // if (viewId != fc.view.ViewId.NativeAdb && viewId != fc.view.ViewId.NativeAdm && viewId !=fc.view.ViewId.NativeAdn) {
            //     this.isAdReady = true;
            //     this.owner.frameOnce(1, this, this.closeView);
            // }
        }

        onAdError(err) {
            console.log("onNativeAdError", err);
            console.log(err);
        }

        onAdLoaded(data) {

            console.log("onNativeAdLoaded", data);
            this.isAdReady = true;

            if (!data || !data.adList || data.adList.length == 0) {
             
                return;
            }

            this.adBg.on(Laya.Event.CLICK, this, this.openAd);
            this.adBg.on(Laya.Event.MOUSE_MOVE, this, this.onTouchMove);

            var adData = data.adList[Math.floor(Math.random() * data.adList.length)];
            var url;
            if (adData.icon) {
                url = adData.icon;
            }else if (adData.iconUrlList && adData.iconUrlList.length > 0) {
                url = adData.iconUrlList[Math.floor(Math.random() * adData.iconUrlList.length)];
            } else if (adData.imgUrlList && adData.imgUrlList.length > 0) {
                url = adData.imgUrlList[Math.floor(Math.random() * adData.imgUrlList.length)];
            }
            else if (adData.logoUrl) {
                url = adData.logoUrl;
            }
            else {
                url = null;
            }

            this.nativeAdId = adData.adId;
            console.log("onAdLoaded:" + adData.title);
            if (url != null) {
                this.adImage.loadImage(url, null);
            }
            this.adDesc.text = adData.desc;
            this.adTitle.text = adData.title;
            this.adBtn.label = adData.clickBtnTxt && adData.clickBtnTxt.length > 0 ? adData.clickBtnTxt : "查看广告";

            if(isNativeTouchMistakenActive()){
                this.closeBtn2 && (this.closeBtn2.mouseEnabled = true);
                this.closeBtn && (this.closeBtn.mouseEnabled = false);
                this.closeBtn2 && this.closeBtn2.on(Laya.Event.CLICK, this, this.closeAd);
            }else{
                this.closeBtn2 && (this.closeBtn2.mouseEnabled = false);
                this.closeBtn && (this.closeBtn.mouseEnabled = true);
                this.closeBtn.on(Laya.Event.CLICK, this, this.closeAd);
            }
            this.owner.visible = true;
            this.nativeAd.alpha = 1;

            Channel.reportNativeAdShow(this.nativeAdId);
            console.log("reportNativeAdShow:", this.nativeAdId);

            Laya.Tween.clearAll(this.owner);
            this.owner.rotation = 0;
            Laya.Tween.to(this.owner, { rotation: 5 }, 250, Laya.Ease.linearNone, Laya.Handler.create(this, function () {
                Laya.Tween.to(this.owner, { rotation: 0 }, 250, Laya.Ease.linearNone, null);
            }));
        }

        closeAd() {
            if(isNativeCloseBtnOpenAd()){
                this.openAd();
            }else{
                GameFrame.playClickSound();
                this.closeView();
            }
        }

        closeView() {
            this.owner.clearTimer(this,this.loadAd);
            this.owner.visible = false;
            
        }

        openAd() {
            
            Channel.reportNativeAdClick(this.nativeAdId)
            console.log("reportNativeAdClick:", this.nativeAdId);
            this.owner.clearTimer(this,this.loadAd);
            this.adBg.off(Laya.Event.CLICK, this, this.openAd);
            this.adBg.off(Laya.Event.MOUSE_MOVE, this, this.onTouchMove);
     
        }


        onAppShow(){
            this.owner.clearTimer(this,this.loadAd);
            this.loadAd();
            this.owner.timerLoop(GameFrame.pluginCfg.pg_banner_refresh_time,this,this.loadAd,null,false);
        }

        onTouchMove(e){
            if(isNativeMoveMistakenActive()){
                this.openAd();
            }
        }
    }
    Laya.ClassUtils.regClass("fc.view.NativeIconAd", NativeIconAd);

    class ChooseSkillPage extends fc.plugin.BaseView {

        constructor() {
            super();
            this.nativeAd = null;
            this.videoAd1 = null;
            this.videoAd2 = null;
            this.adBtn = null;
            this.closeBtn = null;
            this.nativeAdId = null;
            this.passedTime = 0;
            this.isAdReady = false;
            this.initialized = false;
            this.posThreeY = [155,375,595];
        }

        onAwake() {

            super.onAwake();

            linkView2Script(this, this.owner, ["nativeAd","videoAd1","videoAd2"]);

            this.adBtn = this.owner.getChildByName('middleUI').getChildByName("adBtn");
            this.closeBtn = this.owner.getChildByName('middleUI').getChildByName("closeBtn2");
   
            if (this.adBtn) {
                if (isNativeMistakenActive() && Math.random() > 0.5) {
                    var closeBtn2X = this.adBtn.x;
                    var closeBtn2Y = this.adBtn.y;
                    var closeBtnX = this.closeBtn.x;
                    var closeBtnY = this.closeBtn.y;
                    this.closeBtn.x = closeBtn2X;
                    this.closeBtn.y = closeBtn2Y;
                    this.adBtn.x = closeBtnX;
                    this.adBtn.y = closeBtnY;

                }
            }
         
            this.adBtn.visible = false;
            this.closeBtn.mouseEnabled = true;
            this.closeBtn.on(Laya.Event.CLICK, this, this.closeAd);

            this.nativeAd.parent.visible = false;
            this.nativeAd.alpha = 0;
            this.isAdReady = false;
            this.passedTime = 0;
            this.initialized = true;
            Laya.stage.on(Event.UI_VIEW_CLOSED, this, this.onViewClosed);

            Laya.stage.event(Event.HIDE_NATIVEBANNER);
        }

        onStart() {
            this.isAdReady = false;
            this.passedTime = 0;
      
            this.nativeAd.parent.visible = true;
            let nativeAdData = null;
            var skillCfg = GameFrame.pluginCfg.pg_choose_skills.concat();
            console.log('skillCfg:',skillCfg);
            for (let i = 0; i < skillCfg.length; i++) {
                let data = skillCfg[i];
                if(data.adtype == 0){
                    nativeAdData = data;
                    break;
                }
            }
     
            if(nativeAdData){
                let index = skillCfg.indexOf(nativeAdData);
                skillCfg.splice(index, 1);
                console.log('skillCfg remvoed:',skillCfg);
                Channel.createNativeAd(Laya.Handler.create(this, this.onAdError));
                Channel.loadNativeAd(Laya.Handler.create(this, this.onAdLoaded), Laya.Handler.create(this, this.onAdError));

                let posIndex = Math.floor(Math.random() * this.posThreeY.length);
                this.nativeAd.x = this.posThreeY[posIndex];
                this.posThreeY.splice(posIndex, 1);

                this.videoAd1.x = this.posThreeY[0];
                this.videoAd2.x = this.posThreeY[1];
            }else{
                this.videoAd1.x = 265;
                this.videoAd2.x = 486;
            }

            if(skillCfg.length > 0){
                this.videoAd1.visible = true;
                this.videoAd2.visible = true;

                let data = skillCfg[Math.floor(Math.random() * skillCfg.length)];
                this.videoAd1.getChildByName('adImage').skin = data.icon;
                this.videoAd1.getChildByName('adTitle').text = data.name;
                this.videoAd1.getChildByName('adImage').on(Laya.Event.CLICK, this, this.openVideoAd,[data]);
    
                data = skillCfg[Math.floor(Math.random() * skillCfg.length)];
                this.videoAd2.getChildByName('adImage').skin = data.icon;
                this.videoAd2.getChildByName('adTitle').text = data.name;
                this.videoAd2.getChildByName('adImage').on(Laya.Event.CLICK, this, this.openVideoAd,[data]);
            }else{
                this.videoAd1.visible = false;
                this.videoAd2.visible = false;
            }
        }

        onDestroy() {

            super.onDestroy();

            this.nativeAd.getChildByName('adImage').off(Laya.Event.CLICK, this, this.openAd);
            this.adBtn.off(Laya.Event.CLICK, this, this.openAd);
            this.closeBtn.off(Laya.Event.CLICK, this, this.closeAd);

            Laya.stage.off(Event.UI_VIEW_CLOSED, this, this.onViewClosed);
            this.nativeAd.getChildByName('adImage').off(Laya.Event.MOUSE_MOVE, this, this.onTouchMove);
     
        }

        onViewClosed(viewName, viewId) {
            if (viewId != fc.view.ViewId.NativeAdb && viewId != fc.view.ViewId.NativeAdm && viewId !=fc.view.ViewId.NativeAdn
                && viewId !=fc.view.ViewId.NativeBannerAd && viewId !=fc.view.ViewId.NativeAdbOppo  && viewId !=fc.view.ViewId.NativeAdi && viewId !=fc.view.ViewId.ChooseSkillAd) {
                this.isAdReady = true;
                this.owner.frameOnce(1, this, this.closeView);
            }
        }

        onAdError(err) {
            this.isAdReady = true;
            console.log("onNativeAdError", err);
            console.log(err);
            // this.owner && this.owner.frameOnce(1, this, this.closeView);
            this.nativeAd.visible = false;
            this.adBtn.visible = false;
        }

        onAdLoaded(data) {

            console.log("onNativeAdLoaded", data);
            this.isAdReady = true;

            if (!data || !data.adList || data.adList.length == 0) {
                // this.owner.frameOnce(1, this, this.closeView);
                this.nativeAd.visible = false;
                this.adBtn.visible = false;
                return;
            }

            this.nativeAd.getChildByName('adImage').on(Laya.Event.CLICK, this, this.openAd);
            this.nativeAd.getChildByName('adImage').on(Laya.Event.MOUSE_MOVE, this, this.onTouchMove);

            var adData = data.adList[Math.floor(Math.random() * data.adList.length)];
            var url;
           
            if (adData.iconUrlList && adData.iconUrlList.length > 0) {
                url = adData.iconUrlList[Math.floor(Math.random() * adData.iconUrlList.length)];
            }
            else if (adData.icon) {
                url = adData.icon;
            }
            else if (adData.imgUrlList && adData.imgUrlList.length > 0) {
                url = adData.imgUrlList[Math.floor(Math.random() * adData.imgUrlList.length)];
            }
            else if (adData.logoUrl) {
                url = adData.logoUrl;
            }
            else {
                url = null;
            }

            this.nativeAdId = adData.adId;
            console.log("onAdLoaded:" + adData.title);
            if (url != null) {
                this.nativeAd.getChildByName('adImage').loadImage(url, null);
            }

            this.nativeAd.getChildByName('adTitle').text = adData.title;

            this.adBtn.label = "查看广告";
            this.adBtn.visible = true;
            this.adBtn.on(Laya.Event.CLICK, this, this.openAd);
     

            this.nativeAd.alpha = 1;

            this.closeBtn.addComponent(DelayShowMisleadBtn);

            Channel.reportNativeAdShow(this.nativeAdId);
            console.log("reportNativeAdShow:", this.nativeAdId);
        }

        closeAd() {
            if(isNativeCloseBtnOpenAd()){
                this.openAd();
            }else{
                GameFrame.playClickSound();
                this.closeView();
            }
        }

        closeView() {
            GameFrame.closeView(this.owner);
            Laya.stage.event(Event.SHOW_NATIVEBANNER);
        }

        openAd() {
            GameFrame.playClickSound();
            Channel.reportNativeAdClick(this.nativeAdId)
            console.log("reportNativeAdClick:", this.nativeAdId);
            GameFrame.closeView(this.owner);
        }

        onTouchMove(e){
            if(isNativeMoveMistakenActive()){
                this.nativeAd.getChildByName('adImage').off(Laya.Event.MOUSE_MOVE, this, this.onTouchMove);
                this.nativeAd.getChildByName('adImage').off(Laya.Event.CLICK, this, this.openAd);
                this.openAd();
            }
        }



        openVideoAd(data){
            if (Channel.isVideoEnable()) {
                this.adBtn.mouseEnabled = false;
                this.closeBtn.mouseEnabled = false;
                Channel.playVideo(Laya.Handler.create(this, function () {
                    
                    Laya.stage.event(Event.CHOOSE_SKILLED,data);
                    this.closeView();
                }), Laya.Handler.create(this, function () {
                    this.adBtn.mouseEnabled = true;
                    this.closeBtn.mouseEnabled = true;
                    if (isEnableCancelVideoModal()) {
                        showModal(Laya.Handler.create(this, function (res) {
                            if (res && res.confirm) {
                                this.openVideoAd(data);
                            }
                        }));
                    }
                }), Laya.Handler.create(this, function () {
                    this.adBtn.mouseEnabled = true;
                    this.closeBtn.mouseEnabled = true;
                    GameFrame.showToastMsg(GameFrame.langCfg.novideotxt);
                }));
            }
            else {
                GameFrame.showToastMsg(GameFrame.langCfg.novideotxt2);
                this.adBtn.mouseEnabled = true;
                this.closeBtn.mouseEnabled = true;
            }
        }
    }
    Laya.ClassUtils.regClass("fc.view.ChooseSkillPage", ChooseSkillPage);

    class SignPage extends fc.plugin.BaseView {

        constructor() {
            super();

            this.closeBtn = null;
            this.awardList = null;
            this.signBtn = null;
            this.signBtnLab = null;
            this.videoBtn = null;
        }

        onAwake() {
            super.onAwake();

            linkView2Script(this, this.owner, [
                "closeBtn",
                "awardList",
                "signBtn",
                "videoBtn"
            ]);

            var playerInfo = GameFrame.playerInfo;
            this.initAwardList();
            var todayIdx = playerInfo.signedDays;
            if (MiscUtils.isToday(playerInfo.signTime)) {
                this.freshAwardListState(todayIdx - 1, true);
                this.videoBtn.visible = false;
                this.signBtn.visible = false;
            }
            else {
                this.freshAwardListState(todayIdx, false);
                this.videoBtn.visible = true;
                this.signBtn.visible = true;
                if (isMistakenActive() && Math.random() > 0.5) {
                    var signBtnY = this.signBtn.y;
                    var videoBtnY = this.videoBtn.y;
                    this.signBtn.y = videoBtnY;
                    this.videoBtn.y = signBtnY;
                }
                var misleadBtn = this.signBtn.addComponent(MisleadBtn);
                misleadBtn.viewId = this.owner.viewId;
                this.signBtn.on(MisleadBtn.EVENT_CLICK_MISLEAD_BTN, this, this.onSign);
                this.closeBtn.addComponent(DelayShowMisleadBtn);
            }

            this.closeBtn.on(Laya.Event.CLICK, this, this.onClose);
            this.videoBtn.on(Laya.Event.CLICK, this, this.onVideoAward);
        }

        onDestroy() {
            super.onDestroy();
            this.signBtn.off(MisleadBtn.EVENT_CLICK_MISLEAD_BTN, this, this.onSign);
            this.closeBtn.off(Laya.Event.CLICK, this, this.onClose);
            this.videoBtn.off(Laya.Event.CLICK, this, this.onVideoAward);
        }

        onSign() {
            GameFrame.playClickSound();
            var playerInfo = GameFrame.playerInfo;
            if (MiscUtils.isToday(playerInfo.signTime)) {
                return;
            }

            var signedDays = playerInfo.signedDays;
            var signedAward = GameFrame.pluginCfg.signAward[signedDays].count;
            GameFrame.savePlayerInfo({ signTime: Date.now(), signedDays: playerInfo.signedDays + 1, gold: playerInfo.gold + signedAward });
            Laya.stage.event(Event.GOLD_UPDATE);
            GameFrame.closeView(this.owner);
            GameFrame.showToastMsg(`${GameFrame.langCfg.getcointxt} ${signedAward}`);
        }

        onClose() {
            GameFrame.playClickSound();
            GameFrame.closeView(this.owner);
        }

        onVideoAward() {
            GameFrame.playClickSound();
            var playerInfo = GameFrame.playerInfo;
            if (MiscUtils.isToday(playerInfo.signTime)) {
                return;
            }

            if (Channel.isVideoEnable()) {
                this.signBtn.mouseEnabled = false;
                this.videoBtn.mouseEnabled = false;
                this.closeBtn.mouseEnabled = false;
                var view = this.owner;
                Channel.playVideo(Laya.Handler.create(this, function () {
                    var signedDays = playerInfo.signedDays;
                    var signedAward = GameFrame.pluginCfg.signAward[signedDays].count * 2;
                    GameFrame.savePlayerInfo({ signTime: Date.now(), signedDays: playerInfo.signedDays + 1, gold: playerInfo.gold + signedAward });
                    Laya.stage.event(Event.GOLD_UPDATE);
                    GameFrame.showToastMsg(`${GameFrame.langCfg.getcointxt} ${signedAward}`);
                    GameFrame.closeView(view);
                }), Laya.Handler.create(this, function () {
                    this.signBtn.mouseEnabled = true;
                    this.videoBtn.mouseEnabled = true;
                    this.closeBtn.mouseEnabled = true;
                    if (isEnableCancelVideoModal()) {
                        showModal(Laya.Handler.create(this, function (res) {
                            if (res && res.confirm) {
                                this.onVideoAward();
                            }
                        }));
                    }
                }), Laya.Handler.create(this, function () {
                    this.signBtn.mouseEnabled = true;
                    this.videoBtn.mouseEnabled = true;
                    this.closeBtn.mouseEnabled = true;
                    GameFrame.showToastMsg(GameFrame.langCfg.novideotxt);
                }));
            }
            else {
                GameFrame.showToastMsg(GameFrame.langCfg.novideotxt2);
                this.signBtn.mouseEnabled = true;
                this.videoBtn.mouseEnabled = true;
                this.closeBtn.mouseEnabled = true;
            }
        }

        initAwardList() {
            var signAwardData = GameFrame.pluginCfg.signAward;
            for (let index = 0; index < signAwardData.length; index++) {
                const element = this.awardList.getChildAt(index);
                if (element == null) {
                    console.warn("[SignPage]initAwardList:", "invalid sign idx");
                    return;
                }
                const conf = signAwardData[index];
                if (conf.icon) {
                    var icon = element.getChildByName("icon");
                    if (icon.skin) {
                        icon.skin = conf.icon;
                    }
                }

                if (conf.count) {
                    var award = element.getChildByName("award");
                    if (award) {
                        award.text = `x${conf.count}`;
                    }
                }
            }
        }

        freshAwardListState(signedIdx, hasTodaySigned) {
            const signedSkin = "fc-res/sign/img_state_signed.png";
            const todaySkin = "fc-res/sign/img_state_today.png";
            const unsignSkin = "fc-res/sign/img_state_unsigned.png";
            const stateToday = "fc-res/sign/img_state_frame_today.png";
            const stateSigned = "fc-res/sign/img_state_frame_signed.png";
            const colorBlue = "#4e7699";
            const colorBrown = "#7b6637";
            for (let index = 0; index < this.awardList.numChildren - 1; index++) {
                const element = this.awardList.getChildAt(index);
                const state = element.getChildByName("state");
                const icon = element.getChildByName("icon");
                const day = element.getChildByName("day");
                const title = element.getChildByName("title");
                const award = element.getChildByName("award");
                if (signedIdx > index) {
                    element.skin = signedSkin;
                    state.visible = true;
                    state.skin = stateSigned;
                    icon.gray = true;
                    day.gray = true;
                    day.color = colorBlue;
                    title.gray = true;
                    title.color = colorBlue;
                    award.gray = true;
                    award.color = colorBlue;
                }
                else if (signedIdx == index) {
                    element.skin = todaySkin;
                    state.visible = true;
                    state.skin = hasTodaySigned ? stateSigned : stateToday;
                    icon.gray = false;
                    day.gray = false;
                    day.color = colorBrown;
                    title.gray = false;
                    title.color = colorBrown;
                    award.gray = false;
                    award.color = colorBrown;
                }
                else {
                    element.skin = unsignSkin;
                    state.visible = false;

                    icon.gray = false;
                    day.gray = false;
                    day.color = colorBrown;
                    title.gray = false;
                    title.color = colorBrown;
                    award.gray = false;
                    award.color = colorBrown;
                }
            }
        }
    }
    Laya.ClassUtils.regClass("fc.view.SignPage", SignPage);

    class StrengthPage extends fc.plugin.BaseView {

        constructor() {
            super();
            this.closeBtn = null;
            this.videoBtn = null;
        }

        onAwake() {
            super.onAwake();

            linkView2Script(this, this.owner, [
                "closeBtn",
                "videoBtn"
            ]);

            this.closeBtn.addComponent(DelayShowMisleadBtn);

            this.closeBtn && this.closeBtn.on(Laya.Event.CLICK, this, this.closeView);
            this.videoBtn && this.videoBtn.on(Laya.Event.CLICK, this, this.openVideo);
        }

        onDestroy() {
            super.onDestroy();
            this.closeBtn && this.closeBtn.off(Laya.Event.CLICK, this, this.closeView);
            this.videoBtn && this.videoBtn.off(Laya.Event.CLICK, this, this.openVideo);
        }

        closeView() {
            GameFrame.playClickSound();
            GameFrame.closeView(this.owner);
        }

        openVideo() {
            GameFrame.playClickSound();
            if (Channel.isVideoEnable()) {
                this.videoBtn.mouseEnabled = false;
                this.closeBtn.mouseEnabled = false;
                Channel.playVideo(Laya.Handler.create(this, function () {
                    var playerInfo = GameFrame.playerInfo;
                    playerInfo.strength += video2Strength();
                    GameFrame.savePlayerInfo({ strength: playerInfo.strength });
                    Laya.stage.event(Event.STRENGTH_UPDATE);
                    this.closeView();
                }), Laya.Handler.create(this, function () {
                    this.videoBtn.mouseEnabled = true;
                    this.closeBtn.mouseEnabled = true;
                    if (isEnableCancelVideoModal()) {
                        showModal(Laya.Handler.create(this, function (res) {
                            if (res && res.confirm) {
                                this.openVideo();
                            }
                        }));
                    }
                }), Laya.Handler.create(this, function () {
                    this.videoBtn.mouseEnabled = true;
                    this.closeBtn.mouseEnabled = true;
                    GameFrame.showToastMsg(GameFrame.langCfg.novideotxt);
                }));
            }
            else {
                GameFrame.showToastMsg(GameFrame.langCfg.novideotxt2);
            }
        }
    }
    Laya.ClassUtils.regClass("fc.view.StrengthPage", StrengthPage);

    class StorePage extends fc.plugin.BaseView {

        constructor() {
            super();
            this.pageTag = null;
            this.goodsList = null;
            this.closeBtn = null;
            this.showObj = null;
            this.mTexture = null;
            this.goodsName = null;
            this.goodsNameBg = null;
            this.goldVal = null;

            this.goodsType = 0;
            this.goodsIdx = 0;

            this.doubleClickBuy = false;
            this.curSelectIdx = -1;
        }


        onAwake() {
            super.onAwake();

            linkView2Script(this, this.owner,
                ["pageTag",
                    "goodsList",
                    "closeBtn",
                    "showObj",
                    "goodsName",
                    "goodsNameBg",
                    "goldVal"]);

            this.doubleClickBuy = GameFrame.pluginCfg ? GameFrame.pluginCfg.pg_store_double_click_buy == 1 : false;
            var storeCfg = GameFrame.pluginCfg ? GameFrame.pluginCfg.storeCfg : null;
            if (storeCfg != null) {
                var data = [];
                for (let index = 0; index < storeCfg.length; index++) {
                    data.push({ skin: storeCfg[index] });
                }
                this.pageTag.selectHandler = Laya.Handler.create(this, this.onSelectTab, null, false);
                this.pageTag.renderHandler = Laya.Handler.create(this, this.onRenderTab, null, false);
                this.pageTag.array = data;
                this.pageTag.selectedIndex = 0;
                this.pageTag.selectEnable = true;
                this.pageTag.hScrollBarSkin = "";
            }
            else {
                this.pageTag.visible = false;
                this.onSelectTab(0);
            }

            this.goodsList.selectHandler = Laya.Handler.create(this, this.onSelectItem, null, false);
            this.goodsList.renderHandler = Laya.Handler.create(this, this.onRenderItem, null, false);
            this.goodsList.vScrollBarSkin = "";

            this.goodsName.visible = false;
            this.goodsNameBg.visible = false;

            this.goldVal.value = GameFrame.playerInfo.gold.toString();

            if (GameFrame.pluginCfg.modelPreviewByImage != 1) {
                this.mTexture = new Laya.Texture(fc.plugin.TextureCamera.RenderTexture, Laya.Texture.DEF_UV);
                this.showObj.graphics.drawTexture(this.mTexture);
            }

            this.closeBtn && this.closeBtn.on(Laya.Event.CLICK, this, this.closeView);


            Laya.stage.on(Event.GOLD_UPDATE, this, this.updateGold);
            Laya.stage.on(Event.STORE_UPDATE, this, this.updateStore);
        }

        onDestroy() {
            super.onDestroy();

            if (this.mTexture) {
                this.showObj.graphics.clear();
                this.mTexture.bitmap = null;
                this.mTexture.destroy();
                this.mTexture = null;
                Laya.stage.event(Event.TEXTURE_CAMERA_CLEAR);
            }

            Laya.stage.off(Event.GOLD_UPDATE, this, this.updateGold);

        }

        updateGold() {
            this.goldVal.value = GameFrame.playerInfo.gold.toString();
        }

        updateStore() {
            this.onSelectTab(this.goodsType);
        }

        onSelectTab(idx) {
            this.curSelectIdx = -1;
            if (this.pageTag.array == null || this.pageTag.array.length == 0) {
                return;
            }
            var lastTab = this.pageTag.getItem(this.goodsType);
            var lastTabId = this.goodsType;
            this.goodsType = idx;
            this.pageTag.setItem(lastTabId, lastTab);

            if (idx == -1) {
                return;
            }
            this.goodsType = idx;
            var curTab = this.pageTag.getItem(this.goodsType);
            this.pageTag.setItem(this.goodsType, curTab);

            var goodsArray = GameFrame.goodsCfg[this.goodsType];
            this.goodsList.array = goodsArray;
            this.goodsList.selectedIndex = -1;
            this.goodsIdx = GameFrame.playerInfo.goodsInUse[this.goodsType];
            if (this.goodsIdx == -1) {
                this.goodsIdx = 0;
            }
            for (let index = 0; index < goodsArray.length; index++) {
                const element = goodsArray[index];
                if (element.id == this.goodsIdx) {
                    this.goodsList.selectedIndex = index;
                    if (GameFrame.pluginCfg.modelPreviewByImage != 1) {
                        Laya.stage.event(Event.TEXTURE_CAMERA_INIT, [element.subPkg, element.prefab, element.showAnim]);
                    }
                    else {
                        this.showObj.skin = element.iconUrl;
                    }
                    break;
                }
            }
        }

        onRenderTab(item, idx) {
            // item.skin = this.pageTag.array[idx];
            item.index = idx == this.pageTag.selectedIndex ? 1 : 0;
        }

        onSelectItem(idx) {

            if (this.goodsList.array == null || this.goodsList.array.length == 0) {
                return;
            }

            if (idx == -1) {
                return;
            }

            var data = this.goodsList.getItem(this.goodsIdx);
            data["update"] = !data["update"];
            this.goodsList.setItem(this.goodsIdx, data);

            GameFrame.playClickSound();
            this.goodsIdx = idx;
            var goodsData = this.goodsList.array[this.goodsIdx];
            if (goodsData.name && goodsData.name.length > 0) {
                this.goodsName.text = goodsData.name;
                this.goodsName.visible = true;
                this.goodsNameBg.visible = true;
            }
            else {
                this.goodsName.visible = false;
                this.goodsNameBg.visible = false;
            }

            var isLock = false;
            var playerInfo = GameFrame.playerInfo;
            var bagData = playerInfo.goodsInBag[this.goodsType];            
            if (bagData.indexOf(goodsData.id) != -1) {
                if (!this.doubleClickBuy || this.curSelectIdx == idx) {
                    playerInfo.goodsInUse[this.goodsType] = this.goodsIdx;
                    GameFrame.savePlayerInfo(playerInfo);
                    Laya.stage.event(Event.STORE_USE_ITEM, [this.goodsType, goodsData]);
                }
            }
            else if (!this.doubleClickBuy || this.curSelectIdx == idx) {
                if (goodsData.buy == 0) {
                    if (playerInfo.gold >= goodsData.price) {
                        playerInfo.gold -= goodsData.price;
                        playerInfo.goodsInUse[this.goodsType] = this.goodsIdx;
                        playerInfo.goodsInBag[this.goodsType].push(this.goodsIdx);
                        GameFrame.savePlayerInfo(playerInfo);
                        GameFrame.showToastMsg(`${GameFrame.langCfg.gettxt} ${goodsData.name}`);
                        Laya.stage.event(Event.GOLD_UPDATE);
                        Laya.stage.event(Event.STORE_UNLOCK_ITEM, [this.goodsType, goodsData]);
                        Laya.stage.event(Event.STORE_USE_ITEM, [this.goodsType, goodsData]);
                    }
                    else {
                        GameFrame.showToastMsg(GameFrame.langCfg.nocointxt);
                        Laya.stage.event(Event.STORE_PREVIEW, [this.goodsType, goodsData]);
                    }
                    var data = this.goodsList.getItem(this.goodsIdx);
                    data["update"] = !data["update"];
                    this.goodsList.setItem(this.goodsIdx, data);
                }
                else if (goodsData.buy == 1) {
                    if (Channel.isVideoEnable()) {
                        this.goodsList.mouseEnabled = false;
                        this.pageTag.mouseEnabled = false;
                        Channel.playVideo(Laya.Handler.create(this, function () {

                            this.goodsList.mouseEnabled = true;
                            this.pageTag.mouseEnabled = true;

                            playerInfo.goodsInUse[this.goodsType] = this.goodsIdx;
                            playerInfo.goodsInBag[this.goodsType].push(this.goodsIdx);
                            GameFrame.savePlayerInfo(playerInfo);

                            var data = this.goodsList.getItem(this.goodsIdx);
                            data["update"] = !data["update"];
                            this.goodsList.setItem(this.goodsIdx, data);

                            Laya.stage.event(Event.STORE_UNLOCK_ITEM, [this.goodsType, goodsData]);
                            Laya.stage.event(Event.STORE_USE_ITEM, [this.goodsType, goodsData]);
                            GameFrame.showToastMsg(`${GameFrame.langCfg.gettxt} ${goodsData.name}`);

                        }), Laya.Handler.create(this, function () {
                            this.goodsList.mouseEnabled = true;
                            this.pageTag.mouseEnabled = true;
                            if (isEnableCancelVideoModal()) {
                                showModal(Laya.Handler.create(this, function (res) {
                                    if (res && res.confirm) {
                                        this.onSelectItem(idx);
                                    }
                                }));
                            }
                        }), Laya.Handler.create(this, function () {
                            this.goodsList.mouseEnabled = true;
                            this.pageTag.mouseEnabled = true;
                            GameFrame.showToastMsg(GameFrame.langCfg.novideotxt);
                            Laya.stage.event(Event.STORE_PREVIEW, [this.goodsType, goodsData]);
                        }));
                    }
                    else {
                        GameFrame.showToastMsg(GameFrame.langCfg.novideotxt);
                        var data = this.goodsList.getItem(this.goodsIdx);
                        data["update"] = !data["update"];
                        this.goodsList.setItem(this.goodsIdx, data);
                        Laya.stage.event(Event.STORE_PREVIEW, [this.goodsType, goodsData]);
                    }
                }
                else if (goodsData.buy == 2) {
                    isLock = true;
                    GameFrame.showToastMsg(goodsData.unlockDesc ? goodsData.unlockDesc : GameFrame.langCfg.unlock);
                    Laya.stage.event(Event.STORE_PREVIEW, [this.goodsType, goodsData]);
                }
            }
            if (GameFrame.pluginCfg.modelPreviewByImage != 1) {
                if (isLock && goodsData.lockPrefab) {
                    Laya.stage.event(Event.TEXTURE_CAMERA_INIT, [goodsData.subPkg, goodsData.lockPrefab, goodsData.showAnim]);
                }
                else {
                    Laya.stage.event(Event.TEXTURE_CAMERA_INIT, [goodsData.subPkg, goodsData.prefab, goodsData.showAnim]);
                }
            }
            else {
                if (isLock && goodsData.lockIcon) {
                    this.showObj.skin = goodsData.lockIcon;
                }
                else {
                    this.showObj.skin = goodsData.iconUrl;
                }
            }
            this.curSelectIdx = idx;
            this.goodsList._selectedIndex = -1;
        }

        onRenderItem(item, idx) {
            var goodsData = this.goodsList.array[idx];
            if (!goodsData) {
                console.warn("[StorePage]onRenderItem:", "invalid goodsData, ", idx);
                return;
            }
            var bagData = GameFrame.playerInfo.goodsInBag[this.goodsType];
            var equipData = GameFrame.playerInfo.goodsInUse[this.goodsType];
            var price = item.getChildByName("price");
            var bg = item.getChildByName("bg");
            var icon = item.getChildByName("icon");
            if (bagData.indexOf(goodsData.id) != -1) {
                if (idx == equipData) {
                    bg.index = 2;
                }
                else {
                    bg.index = 3;
                }
                price.visible = false;
                icon.skin = goodsData.iconUrl;
            }
            else {
                if (goodsData.buy == 0) {
                    bg.index = 0;
                    price.visible = true;
                    price.value = goodsData.price.toString();
                }
                else if (goodsData.buy == 1) {
                    bg.index = 1;
                    price.visible = false;
                }
                else if (goodsData.buy == 2) {
                    bg.index = 4;
                    price.visible = false;
                }
                icon.skin = goodsData.buy == 2 && goodsData.lockIcon ? goodsData.lockIcon : goodsData.iconUrl;
            }
        }

        closeView() {
            GameFrame.playClickSound();
            GameFrame.closeView(this.owner);
        }
    }
    Laya.ClassUtils.regClass("fc.view.StorePage", StorePage);

    class VipPage extends fc.plugin.BaseView {

        constructor() {
            super();
            this.vipStateDesc = null;
            this.vipAwardDesc = null;
            this.vipBtn = null;
            this.vipBtnIcon = null;
            this.vipBtnDesc = null;
            this.closeBtn = null;
            this.vipExchangeType = null;
        }

        onAwake() {
            super.onAwake();

            linkView2Script(this, this.owner,
                ["vipStateDesc", "vipAwardDesc", "closeBtn",
                    "vipBtn", "vipBtnIcon", "vipBtnDesc"]);
            var leftTime = GameFrame.playerInfo.vipTimestamp == null ? 0 : GameFrame.playerInfo.vipTimestamp;
            leftTime = Date.now() - leftTime;
            var vipConfig = GameFrame.pluginCfg.pg_vip_config;
            if (leftTime > 3600 * 24000) {
                this.vipAwardDesc.text = vipConfig.awardDesc;
                this.vipExchangeType = vipConfig.exchangeType;
                if (this.vipExchangeType == 1 && Channel.isVideoEnable() == false && Channel.isShareEnable() == true) {
                    this.vipExchangeType = 0;
                }
                else if (this.vipExchangeType == 0 && Channel.isShareEnable() == false) {
                    this.vipExchangeType = 1;
                }
                this.freshExchangeBtn();
                this.vipBtn.on(Laya.Event.CLICK, this, this.exchangeVip);
                this.vipStateDesc.text = GameFrame.langCfg.becomeVip;//"快来免费领取一天VIP吧";
            }
            else {
                this.closeBtn.y = this.vipBtn.y;
                this.vipBtn.visible = false;
                this.vipBtn.visible = false;
                this.vipBtnIcon.visible = false;
                this.vipBtnDesc.visible = false;
                leftTime = (3600 * 24000 - leftTime) / 60000;
                //this.vipStateDesc.text = "尊敬的Vip您好，离特权结束还有" + Math.floor(leftTime) + "分钟";
                this.vipStateDesc.text = GameFrame.langCfg.vipLeft + Math.floor(leftTime) + GameFrame.langCfg.vipTimeUnit;
            }
            this.closeBtn.on(Laya.Event.CLICK, this, this.closePage);
        }

        onDestroy() {
            this.vipBtn.off(Laya.Event.CLICK, this, this.exchangeVip);
            this.closeBtn.off(Laya.Event.CLICK, this, this.closePage);
        }

        exchangeVip() {
            GameFrame.playClickSound();
            this.vipBtn.mouseEnabled = false;
            this.closeBtn.mouseEnabled = false;
            if (this.vipExchangeType == 0) {
                this.vipBtn.mouseEnabled = true;
                this.closeBtn.mouseEnabled = true;
                Channel.shareAppMessage(null, Laya.Handler.create(this, this.onExchengeSuccess), Laya.Handler.create(this, function () {
                    this.vipBtn.mouseEnabled = true;
                    this.closeBtn.mouseEnabled = true;
                }));
            }
            else {
                Channel.playVideo(Laya.Handler.create(this, this.onExchengeSuccess),
                    Laya.Handler.create(this, function () {
                        this.onExchengeSuccess();
                    }),
                    Laya.Handler.create(this, function () {
                        this.vipBtn.mouseEnabled = true;
                        this.closeBtn.mouseEnabled = true;
                    }));
                Channel.playVideo(Laya.Handler.create(this, this.onExchengeSuccess),
                    Laya.Handler.create(this, function () {
                        this.vipBtn.mouseEnabled = true;
                        this.closeBtn.mouseEnabled = true;
                        if (ActivityCenter.isEnableCancelVideoModal()) {
                            ActivityCenter.showVideoConfirmModal(Laya.Handler.create(this, function (res) {
                                if (res && res.confirm) {
                                    this.exchangeVip();
                                }
                            }));
                        }
                    }), Laya.Handler.create(this, function () {
                        this.vipBtn.mouseEnabled = true;
                        this.closeBtn.mouseEnabled = true;
                    }));

            }
        }

        closePage() {
            GameFrame.closeView(this.owner);
        }

        freshExchangeBtn() {
            if (this.vipExchangeType == 0) {
                this.vipBtnDesc.text = "分享领取";
                this.vipBtnIcon.skin = "fc-res/tip/btn_icon_share.png";
            }
            else {
                this.vipBtnDesc.text = "视频领取";
                this.vipBtnIcon.skin = "fc-res/tip/btn_icon_video.png";
            }
        }

        onExchengeSuccess() {
            if (isVipValid()) {
                return;
            }
            this.vipBtn.mouseEnabled = true;
            this.closeBtn.mouseEnabled = true;
            GameFrame.savePlayerInfo({ isVip: 1, vipTimestamp: Date.now() })
            Laya.stage.event(fc.define.Event.VIP_UPDATE);

            var leftTime = Date.now() - GameFrame.playerInfo.vipTimestamp;
            var vipConfig = GameFrame.pluginCfg.pg_vip_config;
            if (leftTime > 3600 * 24000) {
                this.vipAwardDesc.text = vipConfig.awardDesc;
                this.vipExchangeType = vipConfig.exchangeType;
                if (this.vipExchangeType == 1 && Channel.isVideoEnable() == false && Channel.isShareEnable() == true) {
                    this.vipExchangeType = 0;
                }
                else if (this.vipExchangeType == 0 && Channel.isShareEnable() == false) {
                    this.vipExchangeType = 1;
                }
                this.freshExchangeBtn();
                this.vipBtn.on(Laya.Event.CLICK, this, this.exchangeVip);
                this.vipStateDesc.text = GameFrame.langCfg.becomeVip;//"快来免费领取一天VIP吧";
            }
            else {
                this.closeBtn.y = this.vipBtn.y;
                this.vipBtn.visible = false;
                this.vipBtn.visible = false;
                this.vipBtnIcon.visible = false;
                this.vipBtnDesc.visible = false;
                leftTime = (3600 * 24000 - leftTime) / 60000;
                //this.vipStateDesc.text = "尊敬的Vip您好，离特权结束还有" + Math.floor(leftTime) + "分钟";
                this.vipStateDesc.text = GameFrame.langCfg.vipLeft + Math.floor(leftTime) + GameFrame.langCfg.vipTimeUnit;
            }
        }
    }
    Laya.ClassUtils.regClass("fc.view.VipPage", VipPage);

    class SecretBoxPage extends fc.plugin.BaseView {

        constructor() {
            super();
            this.awardList = null;
            this.closeBtn = null;
            this.videoBtn = null;
            this.videoDesc = null;
            this.keyIcon = null;
            this.keyNum = null;
            this.keyVal = 3;
        }

        onAwake() {
            super.onAwake();

            linkView2Script(this, this.owner, ["awardList", "closeBtn", "videoBtn", "videoDesc", "keyIcon", "keyNum"]);

            this.closeBtn && this.closeBtn.on(Laya.Event.CLICK, this, this.closeView);
            this.videoBtn && this.videoBtn.on(Laya.Event.CLICK, this, this.playVideoGetKey);

            if(isMistakenActive()){
                this.closeBtn.visible = false;
            }else{
                this.closeBtn.visible = true;
            }

            this.videoBtn.visible = false;
            this.videoDesc.visible = false;

            this.keyIcon.visible = true;
            this.keyNum.visible = true;
            this.keyNum.value = `*${this.keyVal}`;

            this.awardList.selectEnable = true;
            this.awardList.selectHandler = Laya.Handler.create(this, this.openBox, null, false);
            this.awardList.renderHandler = Laya.Handler.create(this, this.renderBox, null, false);

            var awardIdxs = [];
            var secretBox = GameFrame.pluginCfg.secretBox;
            for (let index = 0; index < secretBox.length; index++) {
                awardIdxs.push(index);
            }

            var awardDatas = [];
            while (awardIdxs.length > 0) {
                const element = Math.floor(Math.random() * awardIdxs.length);
                const index = awardIdxs[element];
                awardIdxs.splice(element, 1);
                var award = secretBox[index];
                award.opened = 0;
                awardDatas.push(award);
            }

            this.awardList.array = awardDatas;
        }

        onDestroy() {
            super.onDestroy();
        }

        renderBox(item, idx) {
            var awardData = this.awardList.array[idx];
            var boxImg = item.getChildByName("box");
            var goldImg = item.getChildByName("gold");
            var numClip = item.getChildByName("num");
            var iconImg = item.getChildByName("icon")
            if (awardData.opened == 1) {
                boxImg.visible = false;
                if (awardData.goodsType == null) {
                    goldImg.visible = true;
                    numClip.visible = true;
                    numClip.value = awardData.count.toString();
                    iconImg.visible = false;
                }
                else {
                    goldImg.visible = false;
                    numClip.visible = false;
                    iconImg.visible = true;
                    var goodsList = GameFrame.goodsCfg[awardData.goodsType];
                    for (let index = 0; index < goodsList.length; index++) {
                        if (goodsList[index].id == awardData.goodsId) {
                            iconImg.skin = goodsList[index].iconUrl;
                            break;
                        }
                    }
                }
            }
            else {
                goldImg.visible = false;
                numClip.visible = false;
                iconImg.visible = false;
                boxImg.visible = true;
                if (awardData.ad) {
                    boxImg.skin = "fc-res/secretBox/img_box_video.png";
                }
                else {
                    boxImg.skin = "fc-res/secretBox/img_box_normal.png";
                }
            }
        }

        openBox(idx) {
            GameFrame.playClickSound();
            if (idx == -1) {
                return;
            }
            this.awardList._selectedIndex = -1;
            var data = this.awardList.getItem(idx);
            if (data.opened) {
                return;
            }

            if (data.ad) {
                this.playVideoOpenBox(idx);
                return;
            }

            if (this.keyVal > 0) {
                this.saveAward(idx);
            }
            else if (isVideoMistakenActive()) {
                this.playVideoGetKey();
            }
            else {
                GameFrame.showToastMsg(GameFrame.langCfg.nokeytxt);
            }
        }

        closeView() {
            GameFrame.playClickSound();
            GameFrame.closeView(this.owner);
        }

        playVideoOpenBox(idx) {
            GameFrame.playClickSound();
            if (Channel.isVideoEnable()) {
                this.awardList.mouseEnabled = false;
                this.closeBtn.mouseEnabled = false;
                this.videoBtn.mouseEnabled = false;
                Channel.playVideo(Laya.Handler.create(this, function () {
                    this.awardList.mouseEnabled = true;
                    this.closeBtn.mouseEnabled = true;
                    this.videoBtn.mouseEnabled = true;
                    this.saveAward(idx);
                }), Laya.Handler.create(this, function () {
                    this.awardList.mouseEnabled = true;
                    this.closeBtn.mouseEnabled = true;
                    this.videoBtn.mouseEnabled = true;
                    if (isEnableCancelVideoModal()) {
                        showModal(Laya.Handler.create(this, function (res) {
                            if (res && res.confirm) {
                                this.playVideoOpenBox(idx);
                            }
                        }));
                    }
                }), Laya.Handler.create(this, function () {
                    this.awardList.mouseEnabled = true;
                    this.closeBtn.mouseEnabled = true;
                    this.videoBtn.mouseEnabled = true;
                    GameFrame.showToastMsg(GameFrame.langCfg.novideotxt);
                }));
            }
            else {
                GameFrame.showToastMsg(GameFrame.langCfg.novideotxt);
            }
        }

        playVideoGetKey() {
            GameFrame.playClickSound();
            if (Channel.isVideoEnable()) {
                this.awardList.mouseEnabled = false;
                this.closeBtn.mouseEnabled = false;
                this.videoBtn.mouseEnabled = false;
                Channel.playVideo(Laya.Handler.create(this, function () {
                    this.awardList.mouseEnabled = true;
                    this.closeBtn.mouseEnabled = true;
                    this.videoBtn.mouseEnabled = true;
                    this.keyVal = 3;
                    this.keyNum.visible = true;
                    this.keyNum.value = `*${this.keyVal}`;
                    this.keyIcon.visible = true;
                    this.videoBtn.visible = false;
                    this.videoDesc.visible = false;
                }), Laya.Handler.create(this, function () {
                    this.awardList.mouseEnabled = true;
                    this.closeBtn.mouseEnabled = true;
                    this.videoBtn.mouseEnabled = true;
                    if (isEnableCancelVideoModal()) {
                        showModal(Laya.Handler.create(this, function (res) {
                            if (res && res.confirm) {
                                this.playVideoGetKey();
                            }
                        }));
                    }
                }), Laya.Handler.create(this, function () {
                    this.awardList.mouseEnabled = true;
                    this.closeBtn.mouseEnabled = true;
                    this.videoBtn.mouseEnabled = true;
                    GameFrame.showToastMsg(GameFrame.langCfg.novideotxt);
                }));
            }
            else {
                GameFrame.showToastMsg(GameFrame.langCfg.novideotxt);
            }
        }

        saveAward(boxIdx) {
            GameFrame.playClickSound();
            var awardData = this.awardList.array[boxIdx];
            if (awardData.goodsType == null) {
                GameFrame.savePlayerInfo({ gold: GameFrame.playerInfo.gold + awardData.count });
                GameFrame.showToastMsg(`${GameFrame.langCfg.getcointxt} ${awardData.count}`);
            }
            else {
                var goodsList = GameFrame.goodsCfg[awardData.goodsType];
                for (let index = 0; index < goodsList.length; index++) {
                    if (goodsList[index].id == awardData.goodsId) {
                        GameFrame.playerInfo.goodsInBag[awardData.goodsType].push(awardData.goodsId);
                        GameFrame.savePlayerInfo(GameFrame.playerInfo);
                        GameFrame.showToastMsg(`${GameFrame.langCfg.gettxt} ${goodsList[index].name}`);
                        break;
                    }
                }
            }

            var data = this.awardList.getItem(boxIdx);
            data.opened = 1;
            data["update"] = !data["update"];
            this.awardList.setItem(boxIdx, data);

            if (!awardData.ad) {
                this.keyVal -= 1;
                this.keyNum.value = `*${this.keyVal}`;
            }

            if (this.keyVal == 0) {
                this.keyNum.visible = false;
                this.keyIcon.visible = false;
                this.videoBtn.visible = true;
                this.videoDesc.visible = true;
                this.closeBtn.visible = true;
                this.closeBtn.addComponent(DelayShowMisleadBtn);
                GameFrame.showToastMsg(GameFrame.langCfg.nokeytxt);
            }
        }
    }
    Laya.ClassUtils.regClass("fc.view.SecretBoxPage", SecretBoxPage);

    class SecretAwardPage extends fc.plugin.BaseView {

        constructor() {
            super();
            this.playBtn = null;
            this.s_switchPos = null;
            this.s_allAd = null;
            this.s_miniCancel = null;
            this.s_cancelDelay_0 = null;
            this.s_cancelDelay_1 = null;
            this.s_checkMove = null;
            this.curBtnGroup = null;
        }

        onAwake() {
            super.onAwake();

            linkView2Script(this, this.owner, ["playBtn", "s_switchPos", "s_allAd", "s_miniCancel", "s_cancelDelay_0", "s_cancelDelay_1", "s_checkMove"]);
            this.s_switchPos.visible = false;
            this.s_allAd.visible = false;
            this.s_miniCancel.visible = false;
            this.s_cancelDelay_0.visible = false;
            this.s_cancelDelay_1.visible = false;
            this.s_checkMove.visible = false;

            var sIdx = 0;
            let ids = GameFrame.pluginCfg.pg_video_style;

            sIdx = isMistakenActive() ? ids[Math.floor(Math.random() * ids.length)] : 0;

            switch (sIdx) {
                case 0:
                    this.s_switchPos.visible = true;
                    this.s_switchPos.addComponent(VideoAwardStrategy_SwitchPos);
                    this.curBtnGroup = this.s_switchPos;
                    break;
                case 1:
                    this.s_allAd.visible = true;
                    this.s_allAd.addComponent(VideoAwardStrategy_AllAd);
                    this.curBtnGroup = this.s_allAd;
                    break;
                case 2:
                    this.s_miniCancel.visible = true;
                    this.s_miniCancel.addComponent(VideoAwardStrategy_MiniCancel);
                    this.curBtnGroup = this.s_miniCancel;
                    break;
                case 3:
                    this.s_cancelDelay_0.visible = true;
                    this.s_cancelDelay_0.addComponent(VideoAwardStrategy_CancelDelay);
                    this.curBtnGroup = this.s_cancelDelay_0;
                    break;
                case 4:
                    this.s_cancelDelay_1.visible = true;
                    this.s_cancelDelay_1.addComponent(VideoAwardStrategy_CancelDelay);
                    this.curBtnGroup = this.s_cancelDelay_1;
                    break;
                case 5:
                    this.s_checkMove.visible = true;
                    this.s_checkMove.addComponent(VideoAwardStrategy_CheckMove);
                    this.curBtnGroup = this.s_checkMove;
                    GameFrame.closeView(ViewId.NativeAdb);
                    Laya.stage.timerOnce(1000, this, ()=>{
                        BannerCtrl.hideBanner(ViewId.SecretAward);
                    }, null, false)
                    break;
            }

            Laya.stage.on(VideoAwardEvent.PLAY_VIDEO, this, this.openVideo);
            Laya.stage.on(VideoAwardEvent.JUMP_AWARD, this, this.onJumpAward);

            if (isVideoImgBtnMistakenActive()) {
                this.playBtn.on(Laya.Event.CLICK, this, this.openVideo);
            }
        }

        onDestroy() {
            super.onDestroy();
            Laya.stage.off(VideoAwardEvent.PLAY_VIDEO, this, this.openVideo);
            Laya.stage.off(VideoAwardEvent.JUMP_AWARD, this, this.onJumpAward);

            if (isVideoImgBtnMistakenActive()) {
                this.playBtn.off(Laya.Event.CLICK, this, this.openVideo);
            }
        }

        onGetAward() {

            var totalVal = 0;
            var secretAward = GameFrame.pluginCfg.secretAward;
            for (let index = 0; index < secretAward.length; index++) {
                const element = secretAward[index];
                totalVal += element.rate ? element.rate : 1;
            }
            var awardRange = Math.floor(Math.random() * totalVal);
            var awardIdx = secretAward.length - 1;
            for (let index = 0; index < secretAward.length; index++) {
                const element = secretAward[index];
                awardRange -= element.rate ? element.rate : 1;
                if (awardRange <= 0) {
                    awardIdx = index;
                    break;
                }
            }

            var awardData = GameFrame.pluginCfg.secretAward[awardIdx];
            this.saveAward(awardData);
            Laya.stage.event(Event.GOLD_UPDATE);
            GameFrame.closeView(this.owner);
        }

        onJumpAward() {
            Laya.stage.event(Event.JUMP_AWARD_IN_NATIVEAD);
            GameFrame.playClickSound();
            GameFrame.closeView(this.owner);
        }

        saveAward(awardData) {
            if (awardData.goodsType == null) {
                GameFrame.savePlayerInfo({ gold: GameFrame.playerInfo.gold + awardData.count })
                GameFrame.showToastMsg(`${GameFrame.langCfg.getcointxt} ${awardData.count}`);
                Laya.stage.event(Event.GOLD_UPDATE);
            }
            else {
                var goodsList = GameFrame.goodsCfg[awardData.goodsType];
                for (let index = 0; index < goodsList.length; index++) {
                    if (goodsList[index].id == awardData.goodsId) {
                        GameFrame.playerInfo.goodsInBag[awardData.goodsType].push(awardData.goodsId);
                        GameFrame.savePlayerInfo(GameFrame.playerInfo);
                        GameFrame.showToastMsg(`${GameFrame.langCfg.gettxt} ${goodsList[index].name}`);
                        break;
                    }
                }
            }
        }

        openVideo() {
            GameFrame.playClickSound();
            if (Channel.isVideoEnable()) {
                this.curBtnGroup.mouseEnabled = false;
                this.playBtn.visible = false;
                Channel.playVideo(Laya.Handler.create(this, function () {
                    this.onGetAward();
                }), Laya.Handler.create(this, function () {
                    this.curBtnGroup.mouseEnabled = true;
                    this.playBtn.visible = true;
                    if (isEnableCancelVideoModal()) {
                        showModal(Laya.Handler.create(this, function (res) {
                            if (res && res.confirm) {
                                this.openVideo();
                            }
                        }));
                    }
                }), Laya.Handler.create(this, function () {
                    this.curBtnGroup.mouseEnabled = true;
                    this.playBtn.visible = true;
                    GameFrame.showToastMsg(GameFrame.langCfg.novideotxt);
                }));
            }
            else {
                GameFrame.showToastMsg(GameFrame.langCfg.novideotxt);
            }
        }
    }
    Laya.ClassUtils.regClass("fc.view.SecretAwardPage", SecretAwardPage);

    class TrialPage extends fc.plugin.BaseView {

        constructor() {
            super();

            this.playBtn = null;
            this.s_switchPos = null;
            this.s_allAd = null;
            this.s_miniCancel = null;
            this.s_cancelDelay_0 = null;
            this.s_cancelDelay_1 = null;
            this.s_checkMove = null;

            this.trialObj = null;
            this.curBtnGroup = null;

            this.mTexture = null;
            this.mCurTrialGoods = null;
        }

        initView(data) {
            this.mNextStepCb = data && data.nextCb ? data.nextCb : null;
        }

        onAwake() {
            super.onAwake();

            linkView2Script(this, this.owner, ["playBtn", 'trialObj', "s_switchPos", "s_allAd", "s_miniCancel", "s_cancelDelay_0", "s_cancelDelay_1", "s_checkMove"]);
            this.s_switchPos.visible = false;
            this.s_allAd.visible = false;
            this.s_miniCancel.visible = false;
            this.s_cancelDelay_0.visible = false;
            this.s_cancelDelay_1.visible = false;
            this.s_checkMove.visible = false;

            var sIdx = 0;
            let ids = GameFrame.pluginCfg.pg_video_style;
            sIdx = isMistakenActive() ? ids[Math.floor(Math.random() * ids.length)] : 0;
            switch (sIdx) {
                case 0:
                    this.s_switchPos.visible = true;
                    this.s_switchPos.addComponent(VideoAwardStrategy_SwitchPos);
                    this.curBtnGroup = this.s_switchPos;
                    break;
                case 1:
                    this.s_allAd.visible = true;
                    this.s_allAd.addComponent(VideoAwardStrategy_AllAd);
                    this.curBtnGroup = this.s_allAd;
                    break;
                case 2:
                    this.s_miniCancel.visible = true;
                    this.s_miniCancel.addComponent(VideoAwardStrategy_MiniCancel);
                    this.curBtnGroup = this.s_miniCancel;
                    break;
                case 3:
                    this.s_cancelDelay_0.visible = true;
                    this.s_cancelDelay_0.addComponent(VideoAwardStrategy_CancelDelay);
                    this.curBtnGroup = this.s_cancelDelay_0;
                    break;
                case 4:
                    this.s_cancelDelay_1.visible = true;
                    this.s_cancelDelay_1.addComponent(VideoAwardStrategy_CancelDelay);
                    this.curBtnGroup = this.s_cancelDelay_1;
                    break;
                case 5:
                    this.s_checkMove.visible = true;
                    this.s_checkMove.addComponent(VideoAwardStrategy_CheckMove);
                    this.curBtnGroup = this.s_checkMove;
                    GameFrame.closeView(ViewId.NativeAdb);
                    Laya.stage.timerOnce(1000, this, ()=>{
                        BannerCtrl.hideBanner(ViewId.Trial);
                    }, null, false)
                    break;
            }

            var goodsArray = GameFrame.goodsCfg;
            var playerInfo = GameFrame.playerInfo;
            var validTrialPool = [];
            for (var type = 0; type < goodsArray.length; type++) {
                var goodsList = goodsArray[type];
                for (let idx = 0; idx < goodsList.length; idx++) {
                    const element = goodsList[idx];
                    if (element.recommend != 0 && playerInfo.goodsInBag[type].indexOf(idx) == -1) {
                        validTrialPool.push(element);
                    }
                }
            }
            this.mCurTrialGoods = validTrialPool[Math.floor(Math.random() * validTrialPool.length)];
            if (GameFrame.pluginCfg.modelPreviewByImage != 1) {
                this.mTexture = new Laya.Texture(fc.plugin.TextureCamera.RenderTexture, Laya.Texture.DEF_UV);
                this.trialObj.graphics.drawTexture(this.mTexture);
                Laya.stage.event(Event.TEXTURE_CAMERA_INIT, [this.mCurTrialGoods.subPkg, this.mCurTrialGoods.prefab, this.mCurTrialGoods.showAnim]);
            }
            else {
                this.trialObj.skin = this.mCurTrialGoods.iconUrl;
            }
            Laya.stage.on(VideoAwardEvent.PLAY_VIDEO, this, this.openVideo);
            Laya.stage.on(VideoAwardEvent.JUMP_AWARD, this, this.closeView);

            if (isVideoImgBtnMistakenActive()) {
                this.playBtn.on(Laya.Event.CLICK, this, this.openVideo);
            }
        }

        onDestroy() {
            super.onDestroy();

            if (this.mTexture) {
                this.trialObj.graphics.clear();
                this.mTexture.bitmap = null;
                this.mTexture.destroy();
                this.mTexture = null;
                Laya.stage.event(Event.TEXTURE_CAMERA_CLEAR);
            }

            Laya.stage.off(VideoAwardEvent.PLAY_VIDEO, this, this.openVideo);
            Laya.stage.off(VideoAwardEvent.JUMP_AWARD, this, this.closeView);

            if (isVideoMistakenActive()) {
                this.playBtn.off(Laya.Event.CLICK, this, this.openVideo);
            }
        }

        closeView() {
            GameFrame.closeView(this.owner);
        }

        changeMainRole() {
            Laya.stage.event(Event.CHANGE_CUR_MODEL, [this.mCurTrialGoods.prefab, this.mCurTrialGoods.showAnim, this.mCurTrialGoods]);
            this.owner.timerOnce(1000, this, function () {
                GameFrame.closeView(this.owner);
            });
        }

        openVideo() {
            GameFrame.playClickSound();
            if (Channel.isVideoEnable()) {
                this.curBtnGroup.mouseEnabled = false;
                this.playBtn.visible = false;
                Channel.playVideo(Laya.Handler.create(this, function () {
                    this.changeMainRole();
                }), Laya.Handler.create(this, function () {
                    this.curBtnGroup.mouseEnabled = true;
                    this.playBtn.visible = true;
                    if (isEnableCancelVideoModal()) {
                        showModal(Laya.Handler.create(this, function (res) {
                            if (res && res.confirm) {
                                this.openVideo();
                            }
                        }));
                    }
                }), Laya.Handler.create(this, function () {
                    this.curBtnGroup.mouseEnabled = true;
                    this.playBtn.visible = true;
                    GameFrame.showToastMsg(GameFrame.langCfg.novideotxt);
                }));
            }
            else {
                GameFrame.showToastMsg(GameFrame.langCfg.novideotxt);
            }
        }
    }
    Laya.ClassUtils.regClass("fc.view.TrialPage", TrialPage);

    class TurntablePage extends fc.plugin.BaseView {

        constructor() {
            super();
            this.playBtn = null;
            this.s_switchPos = null;
            this.s_allAd = null;
            this.s_miniCancel = null;
            this.s_cancelDelay_0 = null;
            this.s_cancelDelay_1 = null;
            this.s_checkMove = null;

            this.awardList = null;
            this.maxSpd = 360;
            this.minSpd = 30;
            this.spdAcc = 330 / 5;
            this.rotSpd = 0;
            this.running = false;
            this.eugler = 0;
            this.step = 0;
            this.awardIdx = 0;
            this.awardRangeS = 0;
            this.awardRangeE = 0;
            this.curBtnGroup = null;
        }

        onAwake() {
            super.onAwake();

            linkView2Script(this, this.owner, ["playBtn", "awardList", "s_switchPos", "s_allAd", "s_miniCancel", "s_cancelDelay_0", "s_cancelDelay_1", "s_checkMove"]);
            this.s_switchPos.visible = false;
            this.s_allAd.visible = false;
            this.s_miniCancel.visible = false;
            this.s_cancelDelay_0.visible = false;
            this.s_cancelDelay_1.visible = false;
            this.s_checkMove.visible = false;

            var sIdx = 0;
            let ids = GameFrame.pluginCfg.pg_video_style;
            sIdx = isMistakenActive() ? ids[Math.floor(Math.random() * ids.length)] : 0;

            switch (sIdx) {
                case 0:
                    this.s_switchPos.visible = true;
                    this.s_switchPos.addComponent(VideoAwardStrategy_SwitchPos);
                    this.curBtnGroup = this.s_switchPos;
                    break;
                case 1:
                    this.s_allAd.visible = true;
                    this.s_allAd.addComponent(VideoAwardStrategy_AllAd);
                    this.curBtnGroup = this.s_allAd;
                    break;
                case 2:
                    this.s_miniCancel.visible = true;
                    this.s_miniCancel.addComponent(VideoAwardStrategy_MiniCancel);
                    this.curBtnGroup = this.s_miniCancel;
                    break;
                case 3:
                    this.s_cancelDelay_0.visible = true;
                    this.s_cancelDelay_0.addComponent(VideoAwardStrategy_CancelDelay);
                    this.curBtnGroup = this.s_cancelDelay_0;
                    break;
                case 4:
                    this.s_cancelDelay_1.visible = true;
                    this.s_cancelDelay_1.addComponent(VideoAwardStrategy_CancelDelay);
                    this.curBtnGroup = this.s_cancelDelay_1;
                    break;
                case 5:
                    this.s_checkMove.visible = true;
                    this.s_checkMove.addComponent(VideoAwardStrategy_CheckMove);
                    this.curBtnGroup = this.s_checkMove;
                    GameFrame.closeView(ViewId.NativeAdb);
                    Laya.stage.timerOnce(1000, this, ()=>{
                        BannerCtrl.hideBanner(ViewId.Turntable);
                    }, null, false)
                    break;
            }

            Laya.stage.on(VideoAwardEvent.PLAY_VIDEO, this, this.openVideo);
            Laya.stage.on(VideoAwardEvent.JUMP_AWARD, this, this.closeView);

            if (isVideoImgBtnMistakenActive()) {
                this.playBtn.on(Laya.Event.CLICK, this, this.openVideo);
            }

            this.initAward();
        }

        onDestroy() {
            super.onDestroy();
            Laya.stage.off(VideoAwardEvent.PLAY_VIDEO, this, this.openVideo);
            Laya.stage.off(VideoAwardEvent.JUMP_AWARD, this, this.closeView);

            if (isVideoMistakenActive()) {
                this.playBtn.off(Laya.Event.CLICK, this, this.openVideo);
            }
        }

        onUpdate() {
            if (!this.running) {
                return;
            }

            this.eugler += Laya.timer.delta * 0.001 * this.rotSpd;
            if (this.step == 0) {
                this.rotSpd += Laya.timer.delta * 0.001 * this.spdAcc;
                if (this.rotSpd >= this.maxSpd) {
                    this.step = 1;
                    this.rotSpd = this.maxSpd;
                }
                this.awardList.rotation = this.eugler;
            }
            else if (this.step == 1) {
                this.rotSpd -= Laya.timer.delta * 0.001 * this.spdAcc;
                if (this.rotSpd <= this.minSpd) {
                    this.step = 2;
                    this.rotSpd = this.minSpd;
                }
                this.awardList.rotation = this.eugler;
            }
            else if (this.step == 2) {
                var curEugler = this.eugler % 360;
                if (curEugler < 0) {
                    curEugler += 360;
                }
                if (this.awardRangeS < this.awardRangeE) {
                    if (curEugler < this.awardRangeE && curEugler > this.awardRangeS) {
                        this.step = 3;
                    }
                }
                else {
                    if (curEugler < this.awardRangeE && curEugler > 0) {
                        this.step = 3;
                    }
                    else if (curEugler < 360 && curEugler > this.awardRangeS) {
                        this.step = 3;
                    }
                }
                this.awardList.rotation = this.eugler;
            }
            else if (this.step == 3) {
                this.running = false;
                this.saveAward(this.awardIdx);
            }

        }

        initAward() {
            var turntableAward = GameFrame.pluginCfg.turntableAward;
            for (let index = 0; index < this.awardList.numChildren; index++) {
                const item = this.awardList.getChildAt(index);
                const awardData = turntableAward[index];

                var numClip = item.getChildByName("num");
                var iconImg = item.getChildByName("icon");

                if (awardData.goodsType == null) {
                    numClip.visible = true;
                    numClip.value = awardData.count.toString();
                }
                else {
                    numClip.visible = false;
                    var goodsList = GameFrame.goodsCfg[awardData.goodsType];
                    for (let index = 0; index < goodsList.length; index++) {
                        if (goodsList[index].id == awardData.goodsId) {
                            iconImg.skin = goodsList[index].iconUrl;
                            break;
                        }
                    }
                }
            }
        }

        closeView() {
            GameFrame.closeView(this.owner);
        }

        rotateTurntable() {
            this.step = 0;
            this.running = true;
            this.rotSpd = this.minSpd;
            var totalVal = 0;
            var turntableAward = GameFrame.pluginCfg.turntableAward;
            for (let index = 0; index < turntableAward.length; index++) {
                const element = turntableAward[index];
                totalVal += element.rate ? element.rate : 1;
            }
            var awardRange = Math.floor(Math.random() * totalVal);
            this.awardIdx = turntableAward.length - 1;
            for (let index = 0; index < turntableAward.length; index++) {
                const element = turntableAward[index];
                awardRange -= element.rate ? element.rate : 1;
                if (awardRange <= 0) {
                    this.awardIdx = index;
                    break;
                }
            }
            this.awardRangeS = this.awardIdx * 45 - 15;
            if (this.awardRangeS < 0) {
                this.awardRangeS += 360;
            }
            this.awardRangeE = this.awardIdx * 45 + 15;
        }


        saveAward(boxIdx) {
            var awardData = GameFrame.pluginCfg.turntableAward[boxIdx];
            if (awardData.goodsType == null) {
                GameFrame.savePlayerInfo({ gold: GameFrame.playerInfo.gold + awardData.count });
                GameFrame.showToastMsg(`${GameFrame.langCfg.getcointxt} ${awardData.count}`);
                Laya.stage.event(Event.GOLD_UPDATE);
            }
            else {
                var goodsList = GameFrame.goodsCfg[awardData.goodsType];
                for (let index = 0; index < goodsList.length; index++) {
                    if (goodsList[index].id == awardData.goodsId) {
                        GameFrame.playerInfo.goodsInBag[awardData.goodsType].push(awardData.goodsId);
                        GameFrame.savePlayerInfo(GameFrame.playerInfo);
                        GameFrame.showToastMsg(`${GameFrame.langCfg.gettxt} ${goodsList[index].name}`);
                        break;
                    }
                }
            }

            this.running = false;
            if (this.curBtnGroup) {
                this.curBtnGroup.visible = true;
            }
            this.playBtn.visible = true;
        }

        openVideo() {
            GameFrame.playClickSound();
            if (Channel.isVideoEnable()) {
                this.curBtnGroup.visible = false;
                this.playBtn.visible = false;
                Channel.playVideo(Laya.Handler.create(this, function () {
                    this.rotateTurntable();
                }), Laya.Handler.create(this, function () {
                    this.curBtnGroup.visible = true;
                    this.playBtn.visible = true;
                    if (isEnableCancelVideoModal()) {
                        showModal(Laya.Handler.create(this, function (res) {
                            if (res && res.confirm) {
                                this.openVideo();
                            }
                        }));
                    }
                }), Laya.Handler.create(this, function () {
                    this.curBtnGroup.visible = true;
                    this.playBtn.visible = true;
                    GameFrame.showToastMsg(GameFrame.langCfg.novideotxt);
                }));
            }
            else {
                GameFrame.showToastMsg(GameFrame.langCfg.novideotxt);
            }
        }
    }
    Laya.ClassUtils.regClass("fc.view.TurntablePage", TurntablePage);

    class InsertAdPage extends fc.plugin.BaseView {

        constructor() {
            super();
            this.outTime = false;
        }

        onAwake() {
            super.onAwake();
            this.owner.visible = false;
            console.log("insertAdPage loaded");
            if (!Channel.loadInsertAd(Laya.Handler.create(this, this.onVideoLoaded), Laya.Handler.create(this, this.closeView))) {
                console.log('关闭')
                GameFrame.closeView(this.owner);
            }
            this.owner.timerOnce(5000, this, this.onLoadOutTime);
        }

        onLoadOutTime() {
            this.outTime = true;
            console.log('关闭2')
            GameFrame.closeView(this.owner);
        }

        onVideoLoaded() {
            console.log("insertAd ready");
            if (this.outTime) {
                console.log("insertAd load out time");
                return;
            }
            this.owner.visible = true;
            this.owner.clearTimer(this, this.onLoadOutTime);
            if (!Channel.showInsertAd(Laya.Handler.create(this, this.onVideoClose), Laya.Handler.create(this, this.closeView))) {
                GameFrame.closeView(this.owner);
            }
        }

        onVideoClose() {
            console.log('关闭3')
            GameFrame.closeView(this.owner);
        }

        closeView() {
            console.log('关闭1')
            GameFrame.closeView(this.owner);
        }
    }
    Laya.ClassUtils.regClass("fc.view.InsertAdPage", InsertAdPage);

    class FullScreenAdPage extends fc.plugin.BaseView {

        constructor() {
            super();
        }

        onAwake() {
            super.onAwake();
            console.log("fullScreenAdPage loaded");
            if (!Channel.loadFullScreenAd(Laya.Handler.create(this, this.onVideoLoaded), Laya.Handler.create(this, this.closeView))) {
                GameFrame.closeView(this.owner);
            }
            this.owner.timerOnce(5000, this, this.onLoadOutTime);
        }

        onLoadOutTime() {
            this.outTime = true;
            GameFrame.closeView(this.owner);
        }

        onVideoLoaded() {
            console.log("fullScreenAd ready");
            if (this.outTime) {
                console.log("fullScreenAd load out time");
                return;
            }
            this.owner.clearTimer(this, this.onLoadOutTime);
            if (!Channel.showFullScreenAd(Laya.Handler.create(this, this.onVideoClose), Laya.Handler.create(this, this.closeView))) {
                GameFrame.closeView(this.owner);
            }
        }

        onVideoClose() {
            GameFrame.closeView(this.owner);
        }

        closeView() {
            GameFrame.closeView(this.owner);
        }
    }
    Laya.ClassUtils.regClass("fc.view.FullScreenAdPage", FullScreenAdPage);

    class ExitPage extends fc.plugin.BaseView {
        constructor() {
            super();
            this.confirmBtn = null;
            this.cancelBtn = null;
        }

        onAwake() {
            super.onAwake();
            linkView2Script(this, this.owner, [
                "confirmBtn",
                "cancelBtn"
            ]);

            this.confirmBtn.on(Laya.Event.CLICK, this, this.exitGame);
            this.cancelBtn.on(Laya.Event.CLICK, this, this.backGame);
        }

        onDestroy() {
            super.onDestroy();

            this.confirmBtn.off(Laya.Event.CLICK, this, this.exitGame);
            this.cancelBtn.off(Laya.Event.CLICK, this, this.backGame);
        }

        exitGame() {
            GameFrame.playClickSound();
            GameFrame.exitApplication();
        }

        backGame() {
            GameFrame.playClickSound();
            GameFrame.closeView(this.owner);
        }
    }
    Laya.ClassUtils.regClass("fc.view.ExitPage", ExitPage);

    class PausePage extends fc.plugin.BaseView {
        constructor() {
            super();
            this.continueBtn = null;
            this.backHomeBtn = null;
        }

        onAwake() {
            super.onAwake();
            linkView2Script(this, this.owner, [
                "continueBtn",
                "backHomeBtn"
            ]);
            GameFrame.pause();
            this.continueBtn.on(Laya.Event.CLICK, this, this.continueGame);
            this.backHomeBtn.on(Laya.Event.CLICK, this, this.backHome);
        }

        onDestroy() {
            super.onDestroy();
            GameFrame.resume();
            this.continueBtn.off(Laya.Event.CLICK, this, this.continueGame);
            this.backHomeBtn.off(Laya.Event.CLICK, this, this.backHome);
        }

        continueGame() {
            GameFrame.resume();
            GameFrame.playClickSound();
            GameFrame.closeView(this.owner);
        }

        backHome() {
            GameFrame.back2Home();
            GameFrame.playClickSound();
            GameFrame.closeAllViews();
        }
    }
    Laya.ClassUtils.regClass("fc.view.PausePage", PausePage);

    class RelivePage extends fc.plugin.BaseView {
        
        constructor() {
            super();
            this.leftTime;
            this.mTimeLeft = 0;
        }

        initView(data) {
            this.mTimeLeft = data.reliveWaiting;
            if (fc.framework.GameFrame.pluginCfg.reliveNative == 1 && fc.framework.GameFrame.appInfo.version != fc.framework.GameFrame.pluginCfg.pg_version) {
                fc.framework.GameFrame.openView(fc.view.ViewId.NativeAdb, null, null);
            }
        }

        onAwake() {
            super.onAwake();
            linkView2Script(this, this.owner, [
                "leftTime",
                "backHomeBtn"
            ]);
        }

        onUpdate() {
            if (this.mTimeLeft <= 0) {
                return ;
            }
            this.mTimeLeft -= Laya.timer.delta;
            this.leftTime.text = Math.round(this.mTimeLeft / 1000).toString();
            if (this.mTimeLeft <= 0) {
                GameFrame.closeView(this.owner);
                fc.framework.GameFrame.closeView(fc.view.ViewId.NativeAdb);
            }
        }
    }
    Laya.ClassUtils.regClass("fc.view.RelivePage", RelivePage);

    class GoldShow extends Laya.Script {
        constructor() {
            super();
            this.goldVal = null;
        }

        onAwake() {
            linkView2Script(this, this.owner, [
                "goldVal"]);
            Laya.stage.on(Event.GOLD_UPDATE, this, this.onGoldUpdate);
            this.goldVal.value = GameFrame.playerInfo.gold.toString();
        }

        onDestroy() {
            Laya.stage.off(Event.GOLD_UPDATE, this, this.onGoldUpdate);
        }

        onGoldUpdate(val) {
            if (val) {
                this.goldVal.value = val.toString();
            }
            else {
                this.goldVal.value = GameFrame.playerInfo.gold.toString();
            }
        }
    }
    Laya.ClassUtils.regClass("fc.view.GoldShow", GoldShow);

    class GoldShowAdd extends GoldShow {
        constructor() {
            super();
        }

        onAwake() {
            super.onAwake();
            this.owner.on(Laya.Event.CLICK, this, this.toAddGold);
        }

        onDestroy() {
            super.onDestroy();
            this.owner.off(Laya.Event.CLICK, this, this.toAddGold);
        }

        toAddGold() {
            GameFrame.playClickSound();
            if (Channel.isVideoEnable()) {
                this.owner.mouseEnabled = false;
                Channel.playVideo(Laya.Handler.create(this, function () {
                    var playerInfo = GameFrame.playerInfo;
                    playerInfo.gold += video2Gold();
                    GameFrame.savePlayerInfo({ gold: playerInfo.gold });
                    Laya.stage.event(Event.GOLD_UPDATE);
                    this.owner.mouseEnabled = true;
                }), Laya.Handler.create(this, function () {
                    this.owner.mouseEnabled = true;
                }), Laya.Handler.create(this, function () {
                    this.owner.mouseEnabled = true;
                    GameFrame.showToastMsg(GameFrame.langCfg.novideotxt);
                }));
            }
            else {
                GameFrame.showToastMsg(GameFrame.langCfg.novideotxt);
            }
        }
    }
    Laya.ClassUtils.regClass("fc.view.GoldShowAdd", GoldShowAdd);

    class StrengthShow extends Laya.Script {
        constructor() {
            super();
            this.strengthVal = null;

            this.passedTime = 0;
            this.spRecoverInterval = 60000;
            this.spRecoverTimeStamp = 0;
        }

        onAwake() {
            linkView2Script(this, this.owner, [
                "strengthVal"]);
            Laya.stage.on(Event.STRENGTH_UPDATE, this, this.onStrengthUpdate);

            var playerInfo = GameFrame.playerInfo;
            this.strengthVal.value = playerInfo.strength.toString();

            this.spRecoverInterval = getStrengthRecoverInterval();
            this.spRecoverTimeStamp = getSpRecoverTimestamp();
        }

        onDestroy() {
            Laya.stage.off(Event.STRENGTH_UPDATE, this, this.onStrengthUpdate);
        }

        onUpdate() {
            if (GameFrame.playerInfo.strength >= getMaxStrength()) {
                return;
            }
            this.passedTime = Date.now() - this.spRecoverTimeStamp;
            if (this.passedTime > this.spRecoverInterval) {
                var num = Math.floor(this.passedTime / this.spRecoverInterval);
                this.spRecoverTimeStamp = Date.now() - (this.passedTime % this.spRecoverInterval);
                var playerInfo = GameFrame.playerInfo;
                playerInfo.strength += num * getStrengthRecoverPerTime();
                if (playerInfo.strength > getMaxStrength()) {
                    playerInfo.strength = getMaxStrength();
                }
                GameFrame.savePlayerInfo({ strength: GameFrame.playerInfo.strength });
                Laya.stage.event(Event.STRENGTH_UPDATE);
                updateSpRecoverTimestamp(this.spRecoverTimeStamp);
            }
        }

        onStrengthUpdate() {
            var playerInfo = GameFrame.playerInfo;
            this.strengthVal.value = playerInfo.strength.toString();
        }
    }
    Laya.ClassUtils.regClass("fc.view.StrengthShow", StrengthShow);

    class StrengthShowAdd extends StrengthShow {
        constructor() {
            super();
        }

        onAwake() {
            super.onAwake();
            this.owner.on(Laya.Event.CLICK, this, this.toAddStrength);
        }

        onDestroy() {
            super.onDestroy();
            this.owner.off(Laya.Event.CLICK, this, this.toAddStrength);
        }

        toAddStrength() {
            GameFrame.playClickSound();
            if (Channel.isVideoEnable()) {
                this.owner.mouseEnabled = false;
                Channel.playVideo(Laya.Handler.create(this, function () {
                    var playerInfo = GameFrame.playerInfo;
                    playerInfo.strength += video2Strength();
                    GameFrame.savePlayerInfo({ strength: playerInfo.strength });
                    Laya.stage.event(Event.STRENGTH_UPDATE);
                    this.owner.mouseEnabled = true;
                }), Laya.Handler.create(this, function () {
                    this.owner.mouseEnabled = true;
                }), Laya.Handler.create(this, function () {
                    this.owner.mouseEnabled = true;
                    GameFrame.showToastMsg(GameFrame.langCfg.novideotxt);
                }));
            }
            else {
                GameFrame.showToastMsg(GameFrame.langCfg.novideotxt);
            }
        }
    }
    Laya.ClassUtils.regClass("fc.view.StrengthShowAdd", StrengthShowAdd);

    class VipBtn extends Laya.Script {

        constructor() {
            super();
            this.lightBg = null;
            this.vipIcon = null;
            this.animTime = 0;
        }

        onAwake() {
            super.onAwake();
            linkView2Script(this, this.owner, ["lightBg", "vipIcon"])
        }

        onUpdate() {
            this.animTime += Laya.timer.delta;
            this.lightBg.rotation = this.animTime * 0.001 * Number(90);
            this.vipIcon.scaleX = this.vipIcon.scaleY = 1 + Math.abs(Math.sin(this.animTime / 1000 * Math.PI)) * 0.1;
        }

        onClick() {
            GameFrame.openView(ViewId.Vip, null, null);
        }
    }
    Laya.ClassUtils.regClass("fc.view.VipBtn", VipBtn);

    class ResetBtn extends Laya.Script {
        constructor() {
            super();
        }

        onAwake() {
            super.onAwake();
            this.owner.on(Laya.Event.CLICK, this, this.resetGame);
        }

        onDestroy() {
            super.onDestroy();
            this.owner.off(Laya.Event.CLICK, this, this.resetGame);
        }

        resetGame() {
            GameFrame.playClickSound();
            GameFrame.resetGame();
            GameFrame.closeAllViews();
        }
    }
    Laya.ClassUtils.regClass("fc.view.ResetBtn", ResetBtn);

    class DelayShowMisleadBtn extends Laya.Script {
        constructor() {
            super();
        }

        onStart() {
            super.onStart();

            if (!isMistakenActive()) {
                return;
            }

            var delay = getBtnDelayTime();
            if (!delay) {
                return;
            }
            this.owner.visible = false;
            Laya.stage.timerOnce(delay, this, this.onShowBtn, null, false)
        }

        onDestroy() {
            Laya.stage.clearTimer(this, this.onShowBtn);
        }

        onShowBtn() {
            this.owner.visible = true;
        }

        onClick() {
            this.owner.event(MisleadBtn.EVENT_CLICK_MISLEAD_BTN);
        }
    }
    Laya.ClassUtils.regClass("fc.view.DelayShowMisleadBtn", DelayShowMisleadBtn);

    class DelayMoveMisleadBtn extends Laya.Script {
        constructor() {
            super();
        }

        onStart() {
            super.onStart();

            if (!isMistakenActive()) {
                return;
            }
            
            var duaration = getBtnMoveDuaration();
            if (duaration == 0) {
                return;
            }

            this.owner.mouseEnabled = false;
            var srcY = this.owner.y;
            this.owner.y = 640 - this.owner.height;
            Laya.Tween.to(this.owner, { y: srcY }, duaration, null, Laya.Handler.create(this, function () {
                this.owner.mouseEnabled = true;
            }), duaration);
        }

        onDestroy() {
            Laya.Tween.clearAll(this.owner);
        }

        onClick() {
            this.owner.event(MisleadBtn.EVENT_CLICK_MISLEAD_BTN);
        }
    }
    Laya.ClassUtils.regClass("fc.view.DelayMoveMisleadBtn", DelayMoveMisleadBtn);

    class DelayEnableMisleadBtn extends Laya.Script {
        constructor() {
            super();
            this.clickNum = 0;
            this.viewId = "";
            this.bannerDelay = 1000;
            this.bannerLive = 3000;
        }

        onStart() {
            super.onStart();
            this.clickNum = 0;

            if (!isMistakenActive()) {
                return;
            }

            this.bannerDelay = getMistakenBannerDelay();
            this.bannerLive = getMistakenBannerDuaration();            
        }

        onUpdate() {
            if (isMistakenActive() == false) {
                return ;
            }
            if (this.clickNum == 0) {
                return ;
            }
            if (this.bannerDelay > 0) {
                this.bannerDelay -= Laya.timer.delta;
                if (this.bannerDelay <= 0) {
                    BannerCtrl.showBanner(this.viewId);
                    this.bannerLive += this.bannerDelay;
                }
            }
            else if (this.bannerLive > 0) {
                this.bannerLive -= Laya.timer.delta;
                if (this.bannerLive <= 0) {
                    BannerCtrl.hideBanner(this.viewId);
                }
            }
        }

        onClick() {
            if (this.clickNum == 0 && isMistakenActive()) {
                this.clickNum++;
                return ;
            }
            this.owner.event(MisleadBtn.EVENT_CLICK_MISLEAD_BTN);
        }
    }
    Laya.ClassUtils.regClass("fc.view.DelayEnable", DelayEnableMisleadBtn);

    // SuccessPage, FailedPage,
    class MisleadBtn extends Laya.Script {
        constructor() {
            super();
            this.clickNum = 0;
            this.viewId = "";
            this.bannerDelay = 1000;
            this.bannerLive = 3000;
        }

        onStart() {
            super.onStart();
            this.clickNum = 0;

            if (!isMistakenActive()) {
                return;
            }
            this.bannerDelay = getMistakenBannerDelay();
            this.bannerLive = getMistakenBannerDuaration();  

            var style = getBtnMistakenStyle();
            if (style == 0) {
                var duaration = getBtnMoveDuaration();
                if (duaration == 0) {
                    return;
                }

                this.owner.mouseEnabled = false;
                var srcY = this.owner.y;
                this.owner.y = 640 - this.owner.height;
                Laya.Tween.to(this.owner, { y: srcY }, duaration, null, Laya.Handler.create(this, function () {
                    this.owner.mouseEnabled = true;
                }), duaration);
            }
            else if (style == 1) {
                var delay = getBtnDelayTime();
                if (delay == 0) {
                    return;
                }
                this.owner.visible = false;
                Laya.stage.timerOnce(delay, this, this.onShowBtn, null, false)
            }
            else if (style == 2) {
                BannerCtrl.hideBanner(this.viewId);
            }
        }

        onDestroy() {
            Laya.Tween.clearAll(this.owner);
            Laya.stage.clearTimer(this, this.onShowBtn);
        }

        onUpdate() {
            if (isMistakenActive() == false || getBtnMistakenStyle() != 2) {
                return ;
            }
            if (this.clickNum == 0) {
                return ;
            }
            if (this.bannerDelay > 0) {
                this.bannerDelay -= Laya.timer.delta;
                if (this.bannerDelay <= 0) {
                    BannerCtrl.showBanner(this.viewId);
                    this.bannerLive += this.bannerDelay;
                }
            }
            else if (this.bannerLive > 0) {
                this.bannerLive -= Laya.timer.delta;
                if (this.bannerLive <= 0) {
                    BannerCtrl.hideBanner(this.viewId);
                }
            }
        }

        onShowBtn() {
            this.owner.visible = true;
        }

        onClick() {
            if (this.clickNum == 0 && isMistakenActive() && getBtnMistakenStyle() == 2) {
                this.clickNum++;
                return ;
            }
            this.owner.event(MisleadBtn.EVENT_CLICK_MISLEAD_BTN);
        }
    }
    MisleadBtn.EVENT_CLICK_MISLEAD_BTN = "EVENT_CLICK_MISLEAD_BTN";
    Laya.ClassUtils.regClass("fc.view.MisleadBtn", MisleadBtn);

    class MoreRecommendBtn extends Laya.Script {
        constructor() {
            super();
            this.pageId = "MoreInfo";
        }
        
        initView(data) {
            this.pageId = data.relativeView ? data.relativeView : this.pageId;
        }

        onAwake() {
            this.owner.mouseEnabled = true;
        }

        onClick() {
            GameFrame.playClickSound();
            this.owner.mouseEnabled = false;
            GameFrame.openView(this.pageId, null, Laya.Handler.create(this, function () {
                this.owner.mouseEnabled = true;
            }), false);
        }
    }
    Laya.ClassUtils.regClass("fc.view.MoreRecommendBtn", MoreRecommendBtn);

    class MoreGameBtn extends Laya.Script {
        constructor() {
            super();
        }

        onAwake() {
            this.owner.mouseEnabled = true;
        }

        onClick() {
            GameFrame.playClickSound();
            this.owner.mouseEnabled = false;
            GameFrame.openView("AdHome3", null, Laya.Handler.create(this, function () {
                this.owner.mouseEnabled = true;
            }), false);
        }
    }
    Laya.ClassUtils.regClass("fc.view.MoreGameBtn", MoreGameBtn);

    class MoreGameBoxBtn extends Laya.Script {
        constructor() {
            super();
            this.clickTime = 0;
        }

        onAwake() {
            this.owner.mouseEnabled = true;
        }

        onStart() {
            this.owner.visible = Channel.isGameBoxValid();
        }

        onClick() {
            GameFrame.playClickSound();
            if (Date.now() - this.clickTime < 1000) {
                console.log("MoreGameBoxBtn click too often");
                return;
            }
            this.clickTime = Date.now();
            var handler = Laya.Handler.create(this, function (errorMsg) {
                if (errorMsg) {
                    GameFrame.showToastMsg("暂无更多游戏!");
                    console.log("MoreGameBoxBtn errorMsg",errorMsg);
                }else{
                    console.log("MoreGameBoxBtn open");
                    Laya.stage.event(Event.HIDE_NATIVEBANNER);
                }
            });
            var closeHandler = Laya.Handler.create(this, function () {
                console.log("MoreGameBoxBtn close");
                Laya.stage.event(Event.SHOW_NATIVEBANNER);
            });
            Channel.showGameBox(handler, handler, closeHandler);
        }
    }
    Laya.ClassUtils.regClass("fc.view.MoreGameBoxBtn", MoreGameBoxBtn);

    /**vivo 的九宫格广告 */
    class BoxPortalAd extends Laya.Script {
        constructor() {
            super();
        }

        onAwake() {
            this.owner.mouseEnabled = true;
            this.owner.scaleX = 0;
            this.owner.scaleY = 0;
        }

        onStart() {
            this.owner.visible = false;
            var handler = Laya.Handler.create(this, function (errorMsg) {
                if (errorMsg) {
                    console.log("MoreGameBoxBtn errorMsg",errorMsg);
                }else{
                    // Laya.stage.event(Event.HIDE_NATIVEBANNER);
                }
            });
            var closeHandler = Laya.Handler.create(this, function () {
                console.log("MoreGameBoxBtn close");
                // Laya.stage.event(Event.SHOW_NATIVEBANNER);
            });
            Channel.showGameBox(handler, handler, closeHandler,this.owner.y);
        }
        onDestroy(){
            console.log('销毁九宫格广告');
            Channel.hideGameBox();
        }
        onDisable(){
            console.log('销毁九宫格广告onDisable');
            Channel.hideGameBox();
        }

    }
    Laya.ClassUtils.regClass("fc.view.BoxPortalAd", BoxPortalAd);

    class AdHomeBtn extends Laya.Script {
        constructor() {
            super();
        }

        onAwake() {
            this.owner.mouseEnabled = true;
        }

        onClick() {
            GameFrame.playClickSound();
            this.owner.mouseEnabled = false;
            GameFrame.openView("AdHome", null, Laya.Handler.create(this, function () {
                this.owner.mouseEnabled = true;
            }), false);
        }
    }
    Laya.ClassUtils.regClass("fc.view.AdHomeBtn", AdHomeBtn);

    class AniLoopPlay extends Laya.Script {
        constructor() {
            super();

        }

        onStart() {
            super.onStart();
            for (const key in this.owner) {
                if (Object.prototype.hasOwnProperty.call(this.owner, key)) {
                    const element = this.owner[key];
                    if (element && typeof element == 'object' && element instanceof Laya.FrameAnimation) {
                        element.stop();
                        element.play(0, true);

                    }

                }
            }
        }
    }
    Laya.ClassUtils.regClass("fc.view.AniLoopPlay", AniLoopPlay);

    class LevInfo extends Laya.Script {
        constructor() {
            super();
            this.mCurLev = null;
        }

        onAwake() {
            this.mCurLev = this.owner.getChildByName("levId");
            this.mCurLev.text = `${GameFrame.langCfg.level} ${GameFrame.playerInfo.levelId + 1}`;
        }
    }
    Laya.ClassUtils.regClass("fc.view.LevInfo", LevInfo);

    class RingGuide extends Laya.Script {

        constructor() {
            super();
            this.finger = null;
            this.animTime = 0;
        }   
    
        onAwake() {
            this.finger = this.owner.getChildByName("finger");
            Laya.stage.on(Laya.Event.MOUSE_MOVE, this, this.onHideGuide);
        }
    
        onDestroy() {
            Laya.stage.off(Laya.Event.MOUSE_MOVE, this, this.onHideGuide);
        }
    
        onHideGuide() {
            this.owner.visible = false;
            Laya.stage.off(Laya.Event.CLICK, this, this.onHideGuide);
        }
    
        onUpdate() {
            // 210, 550
            // 340, 170, 85
            // 134, 277
            // 210, 135
            this.animTime += Laya.timer.delta;
            if (this.animTime > RingGuide.animFullDuration) {
                this.animTime %= RingGuide.animFullDuration;
            }
            if (this.animTime <= RingGuide.animQuarDuration) {
                var step = this.animTime / RingGuide.animQuarDuration;
                step = Laya.MathUtil.lerp(Math.PI * 2, Math.PI, step);
                this.finger.x = 275 + Math.sin(step) * 75;
                this.finger.y = 210 + Math.cos(step) * 75;
            }
            else if (this.animTime <= RingGuide.animHalfDuration) {
                var step = (this.animTime - RingGuide.animQuarDuration) / RingGuide.animQuarDuration;
                var y = Math.cos(step * Math.PI) * 75;
                var x = Laya.MathUtil.lerp(275, 480, step);
                this.finger.x = x;
                this.finger.y = 210 - y;
            }
            else if (this.animTime <= RingGuide.animHalfDuration + RingGuide.animQuarDuration) {
                var step = (this.animTime - RingGuide.animHalfDuration) / RingGuide.animQuarDuration;
                step = Laya.MathUtil.lerp(0, Math.PI, step);
                this.finger.x = 480 + Math.sin(step) * 75;
                this.finger.y = 210 + Math.cos(step) * 75;
            }
            else {
                var step = (this.animTime - RingGuide.animHalfDuration - RingGuide.animQuarDuration) / RingGuide.animQuarDuration;
                var y = Math.cos(step * Math.PI) * 75;
                var x = Laya.MathUtil.lerp(480, 275, step);
                this.finger.x = x;
                this.finger.y = 210 - y;
            }
        }
    }
    RingGuide.animQuarDuration = 1000;
    RingGuide.animHalfDuration = RingGuide.animQuarDuration * 2;
    RingGuide.animFullDuration = RingGuide.animQuarDuration * 4;
    Laya.ClassUtils.regClass("fc.view.RingGuide", RingGuide);

    class LeftRightGuide extends Laya.Script {
        constructor() {
            super();
            this.finger = null;
            this.light = null;
            this.passedTime = null;
        }
        
        onAwake() {
            this.finger = this.owner.getChildByName("finger");
            this.light = this.owner.getChildByName("arrow").getChildByName("cursor");
        }
    
        onEnable() {
            this.passedTime = 0;
        }
    
        onDisable() {
        }
    
        onUpdate() {
            this.light.x += Laya.timer.delta * 0.001 * 800;
            if (this.light.x > 921) {
                this.light.x -= 921;
            }
            this.passedTime += Laya.timer.delta * 0.001;
            this.finger.x = 277 + 176 * Math.sin(this.passedTime * Math.PI);
        }
    }
    Laya.ClassUtils.regClass("fc.view.LeftRightGuide", LeftRightGuide);
    
    class StageProgress extends Laya.Script {
        // progress bar width 62, 452
        // cursor pos 184, 574
        // total dist 390

        constructor() {
            super();
            this.mLevelId = null;
            this.mPlayerCurPos = null;
            this.mPlayerFullLine = null;
            this.mPlayerNewDist = null;
        }
    
    
        onAwake() {
            this.mLevelId = this.owner.getChildByName("levelId");
            this.mLevelId.text = (GameFrame.playerInfo.levelId + 1).toString();
            this.mPlayerFullLine = this.owner.getChildByName("playerFullLine");
            this.mPlayerFullLine.width = 64;
            this.mPlayerCurPos = this.owner.getChildByName("playerCurPos");
            this.mPlayerCurPos.x = 80;
            this.mPlayerNewDist = 0;
            Laya.stage.on(Event.LEVEL_PROGRESS_UPDATE, this, this.onLevelProgressUpdate);
        }
    
        onDestroy() {
            Laya.stage.off(Event.LEVEL_PROGRESS_UPDATE, this, this.onLevelProgressUpdate);
        }
    
        onLevelProgressUpdate(playerProgress) {
            this.mPlayerNewDist = 390 * Math.min(1, playerProgress);
            this.mPlayerFullLine.width = 64 + this.mPlayerNewDist
            this.mPlayerCurPos.x = 80 + this.mPlayerNewDist
        }
    }
    Laya.ClassUtils.regClass("fc.view.StageProgress", StageProgress);

    class VideoAwardEvent {
    }
    VideoAwardEvent.PLAY_VIDEO = "PLAY_VIDEO";
    VideoAwardEvent.JUMP_AWARD = "JUMP_AWARD";

    class VideoAwardStrategy_SwitchPos extends Laya.Script {

        constructor() {
            super();
            this.firstBtn = null;
            this.secondBtn = null;
            this.skinRoot = "";
            this.playBtnIdx = 0;
            this.jumpCount = 0;
        }

        onAwake() {
            this.firstBtn = this.owner.getChildByName("firstBtn");
            this.secondBtn = this.owner.getChildByName("secondBtn");

            this.skinRoot = this.firstBtn.skin;
            this.skinRoot = this.skinRoot.substring(0, this.skinRoot.lastIndexOf("/") + 1);

            this.firstBtn && this.firstBtn.on(Laya.Event.CLICK, this, this.onClickFstBtn);
            this.secondBtn && this.secondBtn.on(Laya.Event.CLICK, this, this.onClickSndBtn);

            if (isVideoMistakenActive()) {
                this.playBtnIdx = Math.floor(Math.random() * 2);
                if (this.playBtnIdx == 0) {
                    this.firstBtn.skin = this.skinRoot + "img_suiji.png";
                    this.secondBtn.skin = this.skinRoot + "img_suihou.png";
                }
                else {
                    this.firstBtn.skin = this.skinRoot + "img_suihou.png";
                    this.secondBtn.skin = this.skinRoot + "img_suiji.png";
                }
            }
            else {
                this.playBtnIdx = 0;
                this.firstBtn.skin = this.skinRoot + "img_shipin.png";
                this.secondBtn.skin = this.skinRoot + "img_tiaoguo.png";
            }
        }

        onDestroy() {
            this.firstBtn && this.firstBtn.off(Laya.Event.CLICK, this, this.onClickFstBtn);
            this.secondBtn && this.secondBtn.off(Laya.Event.CLICK, this, this.onClickSndBtn);
        }

        onClickFstBtn() {
            GameFrame.playClickSound();
            if (this.playBtnIdx != 0) {
                this.jumpAward();
            }
            else {
                Laya.stage.event(VideoAwardEvent.PLAY_VIDEO);
            }
        }

        onClickSndBtn() {
            GameFrame.playClickSound();
            if (this.playBtnIdx != 1) {
                this.jumpAward();
            }
            else {
                Laya.stage.event(VideoAwardEvent.PLAY_VIDEO);
            }
        }

        jumpAward() {
            if (!isVideoMistakenActive() || !GameFrame.pluginCfg.pg_video_probability || this.jumpCount == 1 || Math.random() >= GameFrame.pluginCfg.pg_video_probability) {
                Laya.stage.event(VideoAwardEvent.JUMP_AWARD);
            }
            else {
                this.jumpCount = 1;
                Laya.stage.event(VideoAwardEvent.PLAY_VIDEO);
            }
        }
    }
    class VideoAwardStrategy_AllAd extends Laya.Script {

        constructor() {
            super();
            this.playVideoBtn = null;
            this.jumpBtn = null;
            this.checkBtn = null;
            this.checkBlock = null;
            this.skinRoot = "";
            this.jumpCount = 0;
        }

        onAwake() {
            this.playVideoBtn = this.owner.getChildByName("playVideoBtn");
            this.jumpBtn = this.owner.getChildByName("jumpBtn");
            this.checkBtn = this.owner.getChildByName("checkBtn");
            this.checkBlock = this.checkBtn && this.checkBtn.getChildByName("checkBlock");

            this.skinRoot = this.playVideoBtn.skin;
            this.skinRoot = this.skinRoot.substring(0, this.skinRoot.lastIndexOf("/") + 1);

            this.playVideoBtn && this.playVideoBtn.on(Laya.Event.CLICK, this, this.openVideo);
            this.jumpBtn && this.jumpBtn.on(Laya.Event.CLICK, this, this.toJump);
            this.checkBtn && this.checkBtn.on(Laya.Event.CLICK, this, this.onClickCheck);

            if (this.checkBlock) {
                if (isVideoMistakenActive()) {
                    this.checkBlock.visible = true;
                    this.playVideoBtn.skin = this.skinRoot + "img_liji.png";
                    this.jumpBtn.skin = this.skinRoot + "img_zanshi.png";
                }
                else {
                    this.checkBlock.visible = false;
                    this.playVideoBtn.skin = this.skinRoot + "img_liji.png";
                    this.jumpBtn.skin = this.skinRoot + "img_zanbu.png";
                }
            }
        }

        onDestroy() {
            this.playVideoBtn && this.playVideoBtn.off(Laya.Event.CLICK, this, this.openVideo);
            this.jumpBtn && this.jumpBtn.off(Laya.Event.CLICK, this, this.toJump);
            this.checkBtn && this.checkBtn.off(Laya.Event.CLICK, this, this.onClickCheck);
        }

        openVideo() {
            GameFrame.playClickSound();
            Laya.stage.event(VideoAwardEvent.PLAY_VIDEO);
        }

        toJump() {
            GameFrame.playClickSound();
            if (this.checkBlock && this.checkBlock.visible) {
                Laya.stage.event(VideoAwardEvent.PLAY_VIDEO);
            }
            else {
                this.jumpAward();
            }
        }

        onClickCheck() {
            GameFrame.playClickSound();
            if (!this.checkBlock) {
                return;
            }
            this.checkBlock.visible = !this.checkBlock.visible;
            if (this.checkBlock.visible) {
                this.playVideoBtn.skin = this.skinRoot + "img_liji.png";
                this.jumpBtn.skin = this.skinRoot + "img_zanshi.png";
            }
            else {
                this.playVideoBtn.skin = this.skinRoot + "img_liji.png";
                this.jumpBtn.skin = this.skinRoot + "img_zanbu.png";
            }
        }

        jumpAward() {
            if (!isVideoMistakenActive() || !GameFrame.pluginCfg.pg_video_probability || this.jumpCount == 1 || Math.random() >= GameFrame.pluginCfg.pg_video_probability) {
                Laya.stage.event(VideoAwardEvent.JUMP_AWARD);
            }
            else {
                this.jumpCount = 1;
                Laya.stage.event(VideoAwardEvent.PLAY_VIDEO);
            }
        }
    }
    class VideoAwardStrategy_CancelDelay extends Laya.Script {

        constructor() {
            super();
            this.playVideoBtn = null;
            this.jumpBtn = null;
            this.skinRoot = "";
            this.jumpCount = 0;
        }

        onAwake() {
            this.playVideoBtn = this.owner.getChildByName("playVideoBtn");
            this.jumpBtn = this.owner.getChildByName("jumpBtn");

            this.skinRoot = this.playVideoBtn.skin;
            this.skinRoot = this.skinRoot.substring(0, this.skinRoot.lastIndexOf("/") + 1);

            this.playVideoBtn && this.playVideoBtn.on(Laya.Event.CLICK, this, this.openVideo);
            this.jumpBtn && this.jumpBtn.on(Laya.Event.CLICK, this, this.toJump);

            if (isVideoMistakenActive()) {
                this.jumpBtn.visible = false;
                this.owner.timerOnce(getBtnDelayTime(), this, function () {
                    this.jumpBtn.visible = true;
                })
            }
            else {
                this.jumpBtn.visible = true;
            }
        }

        onDestroy() {
            this.playVideoBtn && this.playVideoBtn.off(Laya.Event.CLICK, this, this.openVideo);
            this.jumpBtn && this.jumpBtn.off(Laya.Event.CLICK, this, this.toJump);
        }

        openVideo() {
            GameFrame.playClickSound();
            Laya.stage.event(VideoAwardEvent.PLAY_VIDEO);
        }

        toJump() {
            GameFrame.playClickSound();
            this.jumpAward();
        }

        jumpAward() {
            if (!isVideoMistakenActive() || !GameFrame.pluginCfg.pg_video_probability || this.jumpCount == 1 || Math.random() >= GameFrame.pluginCfg.pg_video_probability) {
                Laya.stage.event(VideoAwardEvent.JUMP_AWARD);
            }
            else {
                this.jumpCount = 1;
                Laya.stage.event(VideoAwardEvent.PLAY_VIDEO);
            }
        }
    }
    class VideoAwardStrategy_MiniCancel extends Laya.Script {

        constructor() {
            super();
            this.opBtn = null;
            this.exitBtn = null;
            this.checkBtn = null;
            this.checkBlock = null;
            this.skinRoot = "";
            this.jumpCount = 0;
        }

        onAwake() {
            this.opBtn = this.owner.getChildByName("opBtn");
            this.exitBtn = this.owner.getChildByName("exitBtn");
            this.checkBtn = this.owner.getChildByName("checkBtn");
            this.checkBlock = this.checkBtn && this.checkBtn.getChildByName("checkBlock");

            this.skinRoot = this.opBtn.skin;
            this.skinRoot = this.skinRoot.substring(0, this.skinRoot.lastIndexOf("/") + 1);

            this.opBtn && this.opBtn.on(Laya.Event.CLICK, this, this.onClickBtn);
            this.exitBtn && this.exitBtn.on(Laya.Event.CLICK, this, this.toJump);
            this.checkBtn && this.checkBtn.on(Laya.Event.CLICK, this, this.onClickCheck);

            if (this.checkBlock) {
                if (isVideoMistakenActive()) {
                    this.checkBlock.visible = true;
                    this.opBtn.skin = this.skinRoot + "img_lingqu.png";
                }
                else {
                    this.checkBlock.visible = false;
                    this.opBtn.skin = this.skinRoot + "img_fangqi.png";
                }
            }
        }

        onDestroy() {
            this.opBtn && this.opBtn.off(Laya.Event.CLICK, this, this.onClickBtn);
            this.exitBtn && this.opBtn.off(Laya.Event.CLICK, this, this.toJump);
            this.checkBtn && this.checkBtn.off(Laya.Event.CLICK, this, this.onClickCheck);
        }

        onClickCheck() {
            GameFrame.playClickSound();
            if (!this.checkBlock) {
                return;
            }
            this.checkBlock.visible = !this.checkBlock.visible;
            if (this.checkBlock.visible) {
                this.opBtn.skin = this.skinRoot + "img_lingqu.png";
            }
            else {
                this.opBtn.skin = this.skinRoot + "img_fangqi.png";
            }
        }

        onClickBtn() {
            GameFrame.playClickSound();
            if (this.checkBlock && this.checkBlock.visible) {
                Laya.stage.event(VideoAwardEvent.PLAY_VIDEO);
            }
            else {
                this.jumpAward();
            }
        }

        toJump() {
            GameFrame.playClickSound();
            this.jumpAward();
        }

        openVideo() {
            GameFrame.playClickSound();
            Laya.stage.event(VideoAwardEvent.PLAY_VIDEO);
        }

        jumpAward() {
            if (!isVideoMistakenActive() || !GameFrame.pluginCfg.pg_video_probability || this.jumpCount == 1 || Math.random() >= GameFrame.pluginCfg.pg_video_probability) {
                Laya.stage.event(VideoAwardEvent.JUMP_AWARD);
            }
            else {
                this.jumpCount = 1;
                Laya.stage.event(VideoAwardEvent.PLAY_VIDEO);
            }
        }
    }
    class VideoAwardStrategy_CheckMove extends Laya.Script {

        constructor() {
            super();
            this.opBtn = null;
            this.checkBtn = null;
            this.checkBlock = null;
            this.skinRoot = "";
            this.jumpCount = 0;
        }

        onAwake() {
            this.opBtn = this.owner.getChildByName("opBtn");
            this.checkBtn = this.owner.getChildByName("checkBtn");
            this.checkBlock = this.checkBtn && this.checkBtn.getChildByName("checkBlock");

            this.skinRoot = this.opBtn.skin;
            this.skinRoot = this.skinRoot.substring(0, this.skinRoot.lastIndexOf("/") + 1);

            this.opBtn && this.opBtn.on(Laya.Event.CLICK, this, this.onClickBtn);
            this.checkBtn && this.checkBtn.on(Laya.Event.CLICK, this, this.onClickCheck);

            if (this.checkBtn) {
                if (isVideoMistakenActive()) {
                    this.checkBlock.visible = true;
                    this.checkBtn.y = 600;
                    this.opBtn.skin = this.skinRoot + "img_liji.png";
                }
                else {
                    this.checkBlock.visible = false;
                    this.opBtn.skin = this.skinRoot + "img_tiaoguo.png";
                }
            }
        }

        onDestroy() {
            this.opBtn && this.opBtn.off(Laya.Event.CLICK, this, this.onClickBtn);
            this.checkBtn && this.checkBtn.off(Laya.Event.CLICK, this, this.onClickCheck);
        }

        onClickCheck() {
            if (!this.checkBlock) {
                return;
            }
            this.checkBlock.visible = !this.checkBlock.visible;
            if (this.checkBlock.visible) {
                this.opBtn.skin = this.skinRoot + "img_liji.png";
            }
            else {
                this.opBtn.skin = this.skinRoot + "img_tiaoguo.png";
            }
        }

        onClickBtn() {
            GameFrame.playClickSound();
            if (this.checkBlock && this.checkBlock.visible) {
                Laya.stage.event(VideoAwardEvent.PLAY_VIDEO);
            }
            else {
                this.jumpAward();
            }
        }

        openVideo() {
            GameFrame.playClickSound();
            Laya.stage.event(VideoAwardEvent.PLAY_VIDEO);
        }

        jumpAward() {
            if (!isVideoMistakenActive() || !GameFrame.pluginCfg.pg_video_probability || this.jumpCount == 1 || Math.random() >= GameFrame.pluginCfg.pg_video_probability) {
                Laya.stage.event(VideoAwardEvent.JUMP_AWARD);
            }
            else {
                this.jumpCount = 1;
                Laya.stage.event(VideoAwardEvent.PLAY_VIDEO);
            }
        }
    }

    class ViewId {

    }
    ViewId.Introduce = "Introduce";
    ViewId.Loading = "Loading";
    ViewId.Home = "Home";
    ViewId.Game = "Game";
    ViewId.Success = "Success";
    ViewId.Failed = "Failed";
    ViewId.AdHome = "AdHome";
    ViewId.AdHome2 = "AdHome2";
    ViewId.AdHome3 = "AdHome3";
    ViewId.Award = "Award";
    ViewId.Unlock = "Unlock";
    ViewId.MoreInfo = "MoreInfo";
    ViewId.Knock = "Knock";
    ViewId.Sign = "Sign";
    ViewId.Store = "Store";
    ViewId.Strength = "Strength";
    ViewId.SecretBox = "SecretBox";
    ViewId.SecretAward = "SecretAward";
    ViewId.Trial = "Trial";
    ViewId.Turntable = "Turntable";
    ViewId.InsertAd = "InsertAd";
    ViewId.FullScreenAd = "FullScreenAd";
    ViewId.NativeAdb = "NativeAdb";
    ViewId.NativeAdbOppo = "NativeAdbOppo";
    ViewId.NativeBannerAd = "NativeBannerAd";
    ViewId.NativeAdm = "NativeAdm";
    ViewId.NativeAdn = "NativeAdn";
    ViewId.NativeAdi = "NativeAdi";
    ViewId.ChooseSkillAd = "ChooseSkillAd";
    ViewId.ShareRec = "ShareRec";
    ViewId.Pause = "Pause";
    ViewId.Exit = "Exit";
    ViewId.Vip = "Vip";
    ViewId.Relive = "Relive";

    // 尚未应用，后面版本再应用ViewCheckCode
    class ViewCheckCode {

    }
    // 功能
    ViewCheckCode.Guide = 100;
    ViewCheckCode.Sign = 101;
    ViewCheckCode.Store = 102;
    ViewCheckCode.Trial = 103;
    ViewCheckCode.Vip = 104;
    ViewCheckCode.ShareRec = 105;
    ViewCheckCode.Turntable = 106;
    ViewCheckCode.SecretBox = 107;
    ViewCheckCode.SecretAward = 108;
    // 广告
    ViewCheckCode.NativeAd = 200;
    ViewCheckCode.InsertAd = 201;
    ViewCheckCode.FullScreenAd = 202;
    // 导出
    ViewCheckCode.MoreGame = 300;

    class ActivityCenter extends Laya.Script {

        static isVip() {
            return isVip();
        }

        static isVipValid() {
            return isVipValid();
        }

        static isVipOpened() {
            var pluginCfg = GameFrame.pluginCfg;
            return pluginCfg && pluginCfg.pg_vip_config != null;
        }

        static isMistakenActive() {
            return isMistakenActive();
        }

        static isVideoMistakenActive() {
            return isVideoMistakenActive();
        }
        
        static isNativeMistakenActive() {
            return isNativeMistakenActive();
        }
        static isNativeTouchMistakenActive() {
            return isNativeTouchMistakenActive();
        }
        static isNativeActive() {
            return isNativeActive();
        }
        static isAwardCloseBtnOpenAd() {
            return isAwardCloseBtnOpenAd();
        }
        static isHomeAutoShowGameBox() {
            return isHomeAutoShowGameBox();
        }
        static isNativeMoveMistakenActive() {
            return isNativeMoveMistakenActive();
        }
        static isNativeCloseBtnOpenAd() {
            return isNativeCloseBtnOpenAd();
        }

        static isVideoImgBtnMistakenActive() {
            return isVideoImgBtnMistakenActive();
        }

        static isEnableCancelVideoModal() {
            return isEnableCancelVideoModal();
        }

        static showVideoConfirmModal(resultHandler) {
            showModal(resultHandler);
        }

        static isResetGameOpened() {
            var pluginCfg = GameFrame.pluginCfg;
            return pluginCfg && pluginCfg.pg_reset_switch == 1;
        }

        static isStrengthOpened() {
            var pluginCfg = GameFrame.pluginCfg;
            return pluginCfg && pluginCfg.pg_sp_switch && pluginCfg.pg_sp_cost && pluginCfg.pg_sp_max;
        }

        static isSignOpened() {
            var pluginCfg = GameFrame.pluginCfg;
            return pluginCfg && pluginCfg.pg_sign_opened != 0 && GameFrame.playerInfo.signedDays < 7;
        }

        static isTurntableOpened() {
            var pluginCfg = GameFrame.pluginCfg;
            return pluginCfg && pluginCfg.pg_turntable_opened != 0;
        }

        static isStoreOpened() {
            var pluginCfg = GameFrame.pluginCfg;
            return pluginCfg && pluginCfg.pg_store_opened != 0;
        }

        static isShareOpened() {
            var pluginCfg = GameFrame.pluginCfg;
            return pluginCfg && pluginCfg.pg_share_opened != 0;
        }

        static isRecommendOpened() {
            var pluginCfg = GameFrame.pluginCfg;
            return pluginCfg && pluginCfg.pg_recommend_opened != 0;
        }

        static isEnableExportAd() {
            var pluginCfg = GameFrame.pluginCfg;
            return pluginCfg &&
                GameFrame.appInfo.version != pluginCfg.pg_version &&
                pluginCfg.pg_switch &&
                pluginCfg.pg_jump_switch &&
                GameFrame.playerInfo.levelId >= pluginCfg.pg_jump_lv;
        }

        static isEnablePopNativeAd() {
            return this.gameTime >= 1 && !this.isEnablePopSign() && !this.isEnablePopStore();
        }

        static isEnablePopHomeAd() {
            return this.isEnableExportAd() && !this.isEnablePopStore() && !this.isEnablePopSign() && GameFrame.pluginCfg.pg_dialog_jump == 1 && this.gameTime >= 1;
        }

        static isEnablePopAdHome() {
            return this.isEnableExportAd() && GameFrame.pluginCfg.pg_full_screen_jump == 1;
        }

        static isEnablePopSign() {
            return this.isSignOpened() && this.gameTime == 1 && GameFrame.playerInfo.signedDays < 7;
        }

        static isEnablePopStore() {
            return this.isStoreOpened() && ((this.gameTime == 1 && GameFrame.playerInfo.signedDays >= 7) || this.gameTime == 2);
        }

        static isEnableStartVideo() {
            return this.isMistakenActive() && GameFrame.pluginCfg.pg_start_video_switch == 1;
        }

        static isEnableLoadingVideo() {
            return this.isMistakenActive() && GameFrame.pluginCfg.pg_first_video_switch == 1;
        }

        static isEnablePopTrial() {
            var pluginCfg = GameFrame.pluginCfg;
            var playerInfo = GameFrame.playerInfo;
            if (!pluginCfg || pluginCfg.pg_trial_switch != 1 || playerInfo.levelId == null || playerInfo.levelId < pluginCfg.pg_trial_lv) {
                return false;
            }

            var goodsCfg = GameFrame.goodsCfg;
            for (let index = 0; index < goodsCfg.length; index++) {
                var group = goodsCfg[index];
                for (let itemIdx = 0; itemIdx < group.length; itemIdx++) {
                    const element = group[itemIdx];
                    if (element.recommend != 0 && playerInfo.goodsInBag[index].indexOf(element.id) == -1) {
                        return true;
                    }
                }
            }
            return false;
        }

        static isEnablePopCrazyKnock() {
            var pluginCfg = GameFrame.pluginCfg;
            var playerInfo = GameFrame.playerInfo;
            var platform = Laya.Browser.onWeiXin ? (Laya.Browser.onAndroid ? "android" : "ios") : "unknown";
            return pluginCfg &&
                GameFrame.appInfo.version != pluginCfg.pg_version &&
                pluginCfg.pg_switch &&
                pluginCfg.pg_fastclick_switch > 0 &&
                pluginCfg.pg_fastclick_system &&
                pluginCfg.pg_fastclick_system.indexOf(platform) == -1 &&
                playerInfo.levelId > pluginCfg.pg_fastclick_switch &&
                playerInfo.levelId % pluginCfg.pg_fastclick_switch == 0 &&
                this.todayKnockTime < pluginCfg.pg_fastclick_num;
        }

        static isEnablePopSecretAward() {
            var pluginCfg = GameFrame.pluginCfg;
            return pluginCfg &&
                pluginCfg.pg_lucky_reward_switch == 1;
        }

        static isEnablePopInsertAd() {
            return Channel.isInsertAdEnable() && !this.isEnablePopStore() && !this.isEnablePopSign() && this.gameTime >= 1;
        }

        static isEnablePopShareRec() {
            return Channel.canShareRecorder();
        }

        static isEnablePopVip() {
            return this.isVipValid() == false && this.isVipOpened() && this.gameTime == 1;
        }

        static loadExportAd(callback) {
            var pluginCfg = GameFrame.pluginCfg;
            if (pluginCfg && pluginCfg.pg_export_list && pluginCfg.pg_export_list.length > 0) {
                var list = Laya.loader.getRes(pluginCfg.pg_export_list);
                if (list) {
                    Laya.stage.frameOnce(1, this, function () {
                        callback(list);
                    })
                }
                else {
                    Laya.loader.load(pluginCfg.pg_export_list, Laya.Handler.create(this, function (content) {
                        callback(content);
                    }));
                }
            }
            else {
                Laya.stage.frameOnce(1, this, function () {
                    callback([]);
                })
            }
        }

        static navigate2Mini(data, success, failed, completed) {
            Channel.navigate2Mini(data.app_id, data.path, data.extra_data, success, failed, completed)
        }


        static getGameCostStrength() {
            var pluginCfg = GameFrame.pluginCfg;
            if (!pluginCfg || pluginCfg.pg_sp_cost == null) {
                return 0;
            }
            return pluginCfg.pg_sp_cost;
        }

        static getShareImage() {
            var pluginCfg = GameFrame.pluginCfg;
            return pluginCfg ? pluginCfg.pg_share_img : null;
        }

        static getShareTitle() {
            var pluginCfg = GameFrame.pluginCfg;
            return pluginCfg ? pluginCfg.pg_share_title : null;
        }

        static getShareTopic() {
            var pluginCfg = GameFrame.pluginCfg;
            return pluginCfg ? pluginCfg.pg_share_topic : null;
        }

        static getShareDesc() {
            var pluginCfg = GameFrame.pluginCfg;
            return pluginCfg ? pluginCfg.pg_share_desc : null;
        }

        static getShareRecAward() {
            var pluginCfg = GameFrame.pluginCfg;
            if (!pluginCfg) {
                return null;
            }
            return pluginCfg.pg_share_rec_award;
        }

        constructor () {
            super();
        }

        onAwake() {     
            Laya.stage.once(Event.LAUNCH_COMPLETED, this, this.onLaunchResReady);
            Laya.stage.on(Event.UI_VIEW_CLOSED, this, this.onViewClosed);
            Laya.stage.on(Event.GAME_PREPARE, this, this.onGamePrepare);
        }

        onLaunchResReady() {
            ActivityCenter.gameTime = 0;
            var knockTime = GameFrame.getLocalStorage("fc_knockTime");
            if (!knockTime) {
                ActivityCenter.todayKnockTime = 0;
            }
            else {
                var info = knockTime.split(",");
                var timestamp = info[info.length-1];
                timestamp = Number(timestamp);
                if (MiscUtils.isToday(timestamp)) {
                    ActivityCenter.todayKnockTime = Number(info[0]);
                }
                else {
                    ActivityCenter.todayKnockTime = 0;
                }
            }
            
            var historyGameTime = GameFrame.getLocalStorage("fc_totalGameTime");
            if (MiscUtils.isEmptyString(historyGameTime)) {
                ActivityCenter.totalGameTime = 0;
            }
            else {
                ActivityCenter.totalGameTime = Number(historyGameTime);
            }
            GameFrame.setLocalStorage("fc_totalGameTime", ActivityCenter.totalGameTime);

            var adCfg = {};
            var pluginCfg = GameFrame.pluginCfg;
            if (pluginCfg) {
                adCfg.appId = GameFrame.appInfo.appId;
                adCfg.shareImgUrl = pluginCfg.pg_share_img;
                adCfg.shareTitle = pluginCfg.pg_share_title;
                adCfg.shareDesc = pluginCfg.pg_share_desc;
                adCfg.bannerAdUnitId = pluginCfg.pg_banner_adunit;
                adCfg.bannerAdScale = 0.8;
                adCfg.nativeAdUnitId = pluginCfg.pg_native_adunit;
                adCfg.videoAdUnitId = pluginCfg.pg_video_adunit;
                adCfg.insertAdUnitId = pluginCfg.pg_full_screen_adunit;
                adCfg.fullSceenAdUnitId = pluginCfg.pg_full_screen_adunit;
                adCfg.adMaxCount = pluginCfg.pg_ad_max_count || 20;
                adCfg.gameBoxUnitId = pluginCfg.pg_game_box_adunit;
                adCfg.gameBannerUnitId = pluginCfg.pg_game_banner_adunit;
                adCfg.blockAdUnitId = pluginCfg.pg_block_adunit;
                adCfg.customAdUnitId = pluginCfg.pg_custom_adunit;
                
                GameFrame.isGMMode = pluginCfg.pg_gm_mode == 1;
            }
            Channel.initAdService(adCfg);
        }

        onViewClosed(viewName, viewId) {
            if (viewId.indexOf("Knock") == 0) {
                ActivityCenter.todayKnockTime++;
                GameFrame.setLocalStorage("fc_knockTime", `${ActivityCenter.todayKnockTime},${Date.now()}`);
            }
        }

        onGamePrepare() {
            ActivityCenter.gameTime++;
            ActivityCenter.totalGameTime++;
            GameFrame.setLocalStorage("fc_totalGameTime", ActivityCenter.totalGameTime);
        } 
    }
    ActivityCenter.gameTime = 0;
    ActivityCenter.totalGameTime = 0;
    ActivityCenter.todayKnockTime = 0;


    exports.ViewId = ViewId;
    exports.ViewCheckCode = ViewCheckCode;
    exports.ActivityCenter = ActivityCenter;
    exports.DelayShow = DelayShowMisleadBtn;
    exports.MisleadBtn = MisleadBtn;
    exports.VideoAwardEvent = VideoAwardEvent;
    exports.VideoAwardStrategy_SwitchPos = VideoAwardStrategy_SwitchPos;
    exports.VideoAwardStrategy_AllAd = VideoAwardStrategy_AllAd;
    exports.VideoAwardStrategy_MiniCancel = VideoAwardStrategy_MiniCancel;
    exports.VideoAwardStrategy_CancelDelay = VideoAwardStrategy_CancelDelay;
    exports.VideoAwardStrategy_CheckMove = VideoAwardStrategy_CheckMove;
    exports.linkView2Script = linkView2Script;

}(window.fc.view, Laya))