window.fc = window.fc || {};
window.fc.platform = window.fc.platform || {};

(function (exports, Laya) {
    'use strict';

    var Event = fc.define.Event;
    var SceneId = fc.define.SceneId;
    var Audio = fc.framework.Audio;
    var GameFrame = fc.framework.GameFrame;
    var Channel = fc.framework.Channel;
    var BannerCtrl = fc.plugin.BannerCtrl;
    var AdditiveUICtrl = fc.plugin.AdditiveUICtrl;
    var ActivityCenter = fc.view.ActivityCenter;

    class VidewAwardEvent {
    }
    VidewAwardEvent.PLAY_VIDEO = "PLAY_VIDEO";
    VidewAwardEvent.JUMP_AWARD = "JUMP_AWARD";
    class VideoAwardStrategy_AllAd extends Laya.Script {

        constructor() {
            super();
            this.playVideoBtn = null;
            this.jumpBtn = null;
            this.checkBtn = null;
            this.checkBlock = null;
            this.skinRoot = "";
        }

        onAwake() {
            this.playVideoBtn = this.owner.getChildByName("playVideoBtn");
            this.jumpBtn = this.owner.getChildByName("jumpBtn");
            this.checkBtn = this.owner.getChildByName("checkBtn");
            this.checkBlock = this.checkBtn && this.checkBtn.getChildByName("checkBlock");
            this.checkBtn.addComponent(fc.view.DelayShow);

            this.skinRoot = this.playVideoBtn.skin;
            this.skinRoot = this.skinRoot.substring(0, this.skinRoot.lastIndexOf("/") + 1);

            this.playVideoBtn && this.playVideoBtn.on(Laya.Event.CLICK, this, this.openVideo);
            this.jumpBtn && this.jumpBtn.on(Laya.Event.CLICK, this, this.toJump);
            this.checkBtn && this.checkBtn.on(Laya.Event.CLICK, this, this.onClickCheck);

            if (this.checkBlock) {
                if (ActivityCenter.isMistakenActive()) {
                    if (Number(PlatformConfig.adZsCfg.test_touch_switch) == 1) {
                        this.checkBlock.visible = true;
                        this.playVideoBtn.skin = this.skinRoot + "img_shiyong_1.png";
                        this.jumpBtn.skin = this.skinRoot + "img_zanshi.png";
                    }
                    else {
                        this.checkBlock.visible = true;
                        this.playVideoBtn.skin = this.skinRoot + "img_shiyong.png";
                        this.jumpBtn.skin = this.skinRoot + "img_zanbu.png";
                    }
                } else {
                    this.checkBlock.visible = false;
                    this.playVideoBtn.skin = this.skinRoot + "img_zhijie.png";
                    this.jumpBtn.skin = this.skinRoot + "img_zanbu.png";
                }
            }
        }

        onDestroy() {
            this.playVideoBtn && this.playVideoBtn.off(Laya.Event.CLICK, this, this.openVideo);
            this.jumpBtn && this.jumpBtn.off(Laya.Event.CLICK, this, this.toJump);
            this.checkBtn && this.checkBtn.off(Laya.Event.CLICK, this, this.onClickCheck);
        }

        openVideo() {
            GameFrame.playClickSound();

            if (this.checkBlock && this.checkBlock.visible) {
                Laya.stage.event(VidewAwardEvent.PLAY_VIDEO);
            }
            else {
                Laya.stage.event(VidewAwardEvent.JUMP_AWARD);
            }
        }

        toJump() {
            GameFrame.playClickSound();
            if (this.checkBlock && this.checkBlock.visible) {
                Laya.stage.event(VidewAwardEvent.PLAY_VIDEO);
            }
            else {
                Laya.stage.event(VidewAwardEvent.JUMP_AWARD);
            }
        }

        onClickCheck() {
            GameFrame.playClickSound();
            if (!this.checkBlock) {
                return;
            }
            this.checkBlock.visible = !this.checkBlock.visible;
            if (this.checkBlock.visible) {
                this.playVideoBtn.skin = this.skinRoot + "img_shiyong.png";
                this.jumpBtn.skin = this.skinRoot + "img_zanshi.png";
            }
            else {
                this.playVideoBtn.skin = this.skinRoot + "img_zhijie.png";
                this.jumpBtn.skin = this.skinRoot + "img_zanbu.png";
            }
        }
    }


    class TrialPage extends fc.plugin.BaseView {

        constructor() {
            super();

            this.playBtn = null;
            this.s_switchPos = null;
            this.s_allAd = null;
            this.s_miniCancel = null;
            this.s_cancelDelay_0 = null;
            this.s_cancelDelay_1 = null;
            this.s_checkMove = null;

            this.trialObj = null;
            this.curBtnGroup = null;

            this.mTexture = null;
            this.mCurTrialGoods = null;
        }

        initView(data) {
            this.mNextStepCb = data && data.nextCb ? data.nextCb : null;
        }

        onAwake() {
            super.onAwake();

            fc.view.linkView2Script(this, this.owner, ["playBtn", 'trialObj', "s_switchPos", "s_allAd", "s_miniCancel", "s_cancelDelay_0", "s_cancelDelay_1", "s_checkMove"]);
            this.s_switchPos.visible = false;
            this.s_allAd.visible = false;
            this.s_miniCancel.visible = false;
            this.s_cancelDelay_0.visible = false;
            this.s_cancelDelay_1.visible = false;
            this.s_checkMove.visible = false;

            this.s_allAd.visible = true;
            this.s_allAd.addComponent(VideoAwardStrategy_AllAd);
            this.curBtnGroup = this.s_allAd;

            var goodsArray = GameFrame.goodsCfg;
            var playerInfo = GameFrame.playerInfo;
            var validTrialPool = [];
            for (var type = 0; type < goodsArray.length; type++) {
                var goodsList = goodsArray[type];
                for (let idx = 0; idx < goodsList.length; idx++) {
                    const element = goodsList[idx];
                    if (element.recommend != 0 && playerInfo.goodsInBag[type].indexOf(idx) == -1) {
                        validTrialPool.push(element);
                    }
                }
            }
            this.mCurTrialGoods = validTrialPool[Math.floor(Math.random() * validTrialPool.length)];
            if (GameFrame.pluginCfg.modelPreviewByImage != 1) {
                this.mTexture = new Laya.Texture(fc.plugin.TextureCamera.RenderTexture, Laya.Texture.DEF_UV);
                this.trialObj.graphics.drawTexture(this.mTexture);
                Laya.stage.event(Event.TEXTURE_CAMERA_INIT, [this.mCurTrialGoods.subPkg, this.mCurTrialGoods.prefab, this.mCurTrialGoods.showAnim]);
            }
            else {
                this.trialObj.skin = this.mCurTrialGoods.iconUrl;
            }

            Laya.stage.on(VidewAwardEvent.PLAY_VIDEO, this, this.openVideo);
            Laya.stage.on(VidewAwardEvent.JUMP_AWARD, this, this.closeView);


            if (ActivityCenter.isMistakenActive()) {
                this.playBtn.on(Laya.Event.CLICK, this, this.openVideo);
            }
        }

        onDestroy() {
            super.onDestroy();

            if (this.mTexture) {
                this.trialObj.graphics.clear();
                this.mTexture.bitmap = null;
                this.mTexture.destroy();
                this.mTexture = null;
                Laya.stage.event(Event.TEXTURE_CAMERA_CLEAR);
            }

            Laya.stage.off(VidewAwardEvent.PLAY_VIDEO, this, this.openVideo);
            Laya.stage.off(VidewAwardEvent.JUMP_AWARD, this, this.closeView);

            if (ActivityCenter.isMistakenActive()) {
                this.playBtn.off(Laya.Event.CLICK, this, this.openVideo);
            }
        }

        closeView() {
            GameFrame.closeView(this.owner);
        }

        changeMainRole() {
            Laya.stage.event(Event.CHANGE_CUR_MODEL, [this.mCurTrialGoods.prefab, this.mCurTrialGoods.showAnim, this.mCurTrialGoods]);
            this.owner.timerOnce(1000, this, function () {
                GameFrame.closeView(this.owner);
            });
        }

        openVideo() {
            GameFrame.playClickSound();
            if (Channel.isVideoEnable()) {
                this.curBtnGroup.mouseEnabled = false;
                this.playBtn.visible = false;
                Channel.playVideo(Laya.Handler.create(this, function () {
                    this.changeMainRole();
                }), Laya.Handler.create(this, function () {
                    this.curBtnGroup.mouseEnabled = true;
                    this.playBtn.visible = true;
                    if (ActivityCenter.isEnableCancelVideoModal()) {
                        ActivityCenter.showVideoConfirmModal(Laya.Handler.create(this, function (res) {
                            if (res && res.confirm) {
                                this.openVideo();
                            }
                        }));
                    }
                }), Laya.Handler.create(this, function () {
                    this.curBtnGroup.mouseEnabled = true;
                    this.playBtn.visible = true;
                    GameFrame.showToastMsg(GameFrame.langCfg.novideotxt);
                }));
            }
            else {
                GameFrame.showToastMsg(GameFrame.langCfg.novideotxt);
            }
        }
    }
    Laya.ClassUtils.regClass("fc.zsks.TrialPage", TrialPage);
    function getBtnDelayTime() {
        return GameFrame.pluginCfg ? GameFrame.pluginCfg.pg_button_delay_time : 0;
    }
    class VideoAwardStrategy_CancelDelay extends Laya.Script {

        constructor() {
            super();
            this.playVideoBtn = null;
            this.jumpBtn = null;
            this.skinRoot = "";
        }

        onAwake() {
            this.playVideoBtn = this.owner.getChildByName("playVideoBtn");
            this.jumpBtn = this.owner.getChildByName("jumpBtn");

            this.skinRoot = this.playVideoBtn.skin;
            this.skinRoot = this.skinRoot.substring(0, this.skinRoot.lastIndexOf("/") + 1);

            this.playVideoBtn && this.playVideoBtn.on(Laya.Event.CLICK, this, this.openVideo);
            this.jumpBtn && this.jumpBtn.on(Laya.Event.CLICK, this, this.toJump);

            if (ActivityCenter.isMistakenActive()) {
                this.jumpBtn.visible = false;
                this.owner.timerOnce(getBtnDelayTime(), this, function () {
                    this.jumpBtn.visible = true;
                })

            }
            else {
                this.jumpBtn.visible = true;
            }
        }

        onDestroy() {
            this.playVideoBtn && this.playVideoBtn.off(Laya.Event.CLICK, this, this.openVideo);
            this.jumpBtn && this.jumpBtn.off(Laya.Event.CLICK, this, this.toJump);
        }

        openVideo() {
            GameFrame.playClickSound();
            Laya.stage.event(VidewAwardEvent.PLAY_VIDEO);
        }

        toJump() {
            GameFrame.playClickSound();
            Laya.stage.event(VidewAwardEvent.JUMP_AWARD);
        }
    }

    class SecretAwardPage extends fc.plugin.BaseView {

        constructor() {
            super();
            this.playBtn = null;
            this.s_switchPos = null;
            this.s_allAd = null;
            this.s_miniCancel = null;
            this.s_cancelDelay_0 = null;
            this.s_cancelDelay_1 = null;
            this.s_checkMove = null;
            this.curBtnGroup = null;
        }

        onAwake() {
            super.onAwake();

            fc.view.linkView2Script(this, this.owner, ["playBtn", "s_switchPos", "s_allAd", "s_miniCancel", "s_cancelDelay_0", "s_cancelDelay_1", "s_checkMove"]);
            this.s_switchPos.visible = false;
            this.s_allAd.visible = false;
            this.s_miniCancel.visible = false;
            this.s_cancelDelay_0.visible = false;
            this.s_cancelDelay_1.visible = false;
            this.s_checkMove.visible = false;

            this.s_cancelDelay_1.visible = true;
            this.s_cancelDelay_1.addComponent(VideoAwardStrategy_CancelDelay);
            this.curBtnGroup = this.s_cancelDelay_1;

            Laya.stage.on(VidewAwardEvent.PLAY_VIDEO, this, this.openVideo);
            Laya.stage.on(VidewAwardEvent.JUMP_AWARD, this, this.onJumpAward);

            if (ActivityCenter.isMistakenActive()) {
                this.playBtn.on(Laya.Event.CLICK, this, this.openVideo);
            }
        }

        onDestroy() {
            super.onDestroy();
            Laya.stage.off(VidewAwardEvent.PLAY_VIDEO, this, this.openVideo);
            Laya.stage.off(VidewAwardEvent.JUMP_AWARD, this, this.onJumpAward);

            if (ActivityCenter.isMistakenActive()) {
                this.playBtn.off(Laya.Event.CLICK, this, this.openVideo);
            }
        }

        onGetAward() {

            var totalVal = 0;
            var secretAward = GameFrame.pluginCfg.secretAward;
            for (let index = 0; index < secretAward.length; index++) {
                const element = secretAward[index];
                totalVal += element.rate ? element.rate : 1;
            }
            var awardRange = Math.floor(Math.random() * totalVal);
            var awardIdx = secretAward.length - 1;
            for (let index = 0; index < secretAward.length; index++) {
                const element = secretAward[index];
                awardRange -= element.rate ? element.rate : 1;
                if (awardRange <= 0) {
                    awardIdx = index;
                    break;
                }
            }

            var awardData = GameFrame.pluginCfg.secretAward[awardIdx];
            this.saveAward(awardData);
            Laya.stage.event(Event.GOLD_UPDATE);
            GameFrame.closeView(this.owner);
        }

        onJumpAward() {
            GameFrame.playClickSound();
            GameFrame.closeView(this.owner);
        }

        saveAward(awardData) {
            if (awardData.goodsType == null) {
                GameFrame.savePlayerInfo({ gold: GameFrame.playerInfo.gold + awardData.count })
                GameFrame.showToastMsg(`${GameFrame.langCfg.getcointxt} ${awardData.count}`);
                Laya.stage.event(Event.GOLD_UPDATE);
            }
            else {
                var goodsList = GameFrame.goodsData[awardData.goodsType];
                for (let index = 0; index < goodsList.length; index++) {
                    if (goodsList[index].id == awardData.goodsId) {
                        GameFrame.playerInfo.goodsInBag[awardData.goodsType].push(awardData.goodsId);
                        GameFrame.savePlayerInfo(GameFrame.playerInfo);
                        GameFrame.showToastMsg(`${GameFrame.langCfg.gettxt} ${goodsList[index].name}`);
                        break;
                    }
                }
            }
        }

        openVideo() {
            GameFrame.playClickSound();
            if (Channel.isVideoEnable()) {
                this.curBtnGroup.mouseEnabled = false;
                this.playBtn.visible = false;
                Channel.playVideo(Laya.Handler.create(this, function () {
                    this.onGetAward();
                }), Laya.Handler.create(this, function () {
                    this.curBtnGroup.mouseEnabled = true;
                    this.playBtn.visible = true;
                    if (ActivityCenter.isEnableCancelVideoModal()) {
                        ActivityCenter.showVideoConfirmModal(Laya.Handler.create(this, function (res) {
                            if (res && res.confirm) {
                                this.openVideo();
                            }
                        }));
                    }
                }), Laya.Handler.create(this, function () {
                    this.curBtnGroup.mouseEnabled = true;
                    this.playBtn.visible = true;
                    GameFrame.showToastMsg(GameFrame.langCfg.novideotxt);
                }));
            }
            else {
                GameFrame.showToastMsg(GameFrame.langCfg.novideotxt);
            }
        }
    }
    Laya.ClassUtils.regClass("fc.zsks.SecretAwardPage", SecretAwardPage);

    class SecretBoxPage extends fc.plugin.BaseView {

        constructor() {
            super();
            this.awardList = null;
            this.closeBtn = null;
            this.videoBtn = null;
            this.goldBtn = null;
            this.videoDesc = null;
            this.keyIcon = null;
            this.keyNum = null;
            this.keyVal = 3;
        }

        onAwake() {
            super.onAwake();

            fc.view.linkView2Script(this, this.owner, ["awardList", "closeBtn", "videoBtn", "goldBtn", "videoDesc", "keyIcon", "keyNum"]);

            this.closeBtn && this.closeBtn.on(Laya.Event.CLICK, this, this.closeView);
            this.videoBtn && this.videoBtn.on(Laya.Event.CLICK, this, this.playVideoGetKey);
            this.goldBtn && this.goldBtn.on(Laya.Event.CLICK, this, this.playVideoGetGold);

            this.closeBtn.visible = false;
            if (ActivityCenter.isMistakenActive()) {
                this.closeBtn.visible = false;
            } else {
                this.closeBtn.visible = true;
            }

            this.videoBtn.visible = false;
            this.goldBtn.visible = false;
            this.videoDesc.visible = false;

            this.keyIcon.visible = true;
            this.keyNum.visible = true;
            this.keyNum.value = `*${this.keyVal}`;

            this.awardList.selectEnable = true;
            this.awardList.selectHandler = Laya.Handler.create(this, this.openBox, null, false);
            this.awardList.renderHandler = Laya.Handler.create(this, this.renderBox, null, false);

            var awardIdxs = [];
            var secretBox = GameFrame.pluginCfg.secretBox;
            for (let index = 0; index < secretBox.length; index++) {
                awardIdxs.push(index);
            }

            var awardDatas = [];
            while (awardIdxs.length > 0) {
                const element = Math.floor(Math.random() * awardIdxs.length);
                const index = awardIdxs[element];
                awardIdxs.splice(element, 1);
                var award = secretBox[index];
                award.opened = 0;
                awardDatas.push(award);
            }

            this.awardList.array = awardDatas;
        }

        onDestroy() {
            super.onDestroy();
        }

        renderBox(item, idx) {
            var awardData = this.awardList.array[idx];
            var boxImg = item.getChildByName("box");
            var goldImg = item.getChildByName("gold");
            var numClip = item.getChildByName("num");
            var iconImg = item.getChildByName("icon")
            if (awardData.opened == 1) {
                boxImg.visible = false;
                if (awardData.goodsType == null) {
                    goldImg.visible = true;
                    numClip.visible = true;
                    numClip.value = awardData.count.toString();
                    iconImg.visible = false;
                }
                else {
                    goldImg.visible = false;
                    numClip.visible = false;
                    iconImg.visible = true;
                    var goodsList = GameFrame.goodsData[awardData.goodsType];
                    for (let index = 0; index < goodsList.length; index++) {
                        if (goodsList[index].id == awardData.goodsId) {
                            iconImg.skin = goodsList[index].iconUrl;
                            break;
                        }
                    }
                }
            }
            else {
                goldImg.visible = false;
                numClip.visible = false;
                iconImg.visible = false;
                boxImg.visible = true;
                if (awardData.ad) {
                    boxImg.skin = "fc-res/secretBox/img_box_video.png";
                }
                else {
                    boxImg.skin = "fc-res/secretBox/img_box_normal.png";
                }
            }
        }

        openBox(idx) {
            GameFrame.playClickSound();
            if (idx == -1) {
                return;
            }
            this.awardList._selectedIndex = -1;
            var data = this.awardList.getItem(idx);
            if (data.opened) {
                return;
            }

            if (data.ad) {
                this.playVideoOpenBox(idx);
                return;
            }

            if (this.keyVal > 0) {
                this.saveAward(idx);
            }
            else if (ActivityCenter.isMistakenActive()) {
                this.playVideoGetKey();
            }
            else {
                GameFrame.showToastMsg(GameFrame.langCfg.nokeytxt);
            }
        }

        closeView() {
            GameFrame.playClickSound();
            GameFrame.closeView(this.owner);
        }

        playVideoOpenBox(idx) {
            GameFrame.playClickSound();
            if (Channel.isVideoEnable()) {
                this.awardList.mouseEnabled = false;
                this.closeBtn.mouseEnabled = false;
                this.videoBtn.mouseEnabled = false;
                this.goldBtn.mouseEnabled = true;
                Channel.playVideo(Laya.Handler.create(this, function () {
                    this.awardList.mouseEnabled = true;
                    this.closeBtn.mouseEnabled = true;
                    this.videoBtn.mouseEnabled = true;
                    this.goldBtn.mouseEnabled = true;
                    this.saveAward(idx);
                }), Laya.Handler.create(this, function () {
                    this.awardList.mouseEnabled = true;
                    this.closeBtn.mouseEnabled = true;
                    this.videoBtn.mouseEnabled = true;
                    this.goldBtn.mouseEnabled = true;

                    if (ActivityCenter.isEnableCancelVideoModal()) {
                        ActivityCenter.showVideoConfirmModal(Laya.Handler.create(this, function (res) {
                            if (res && res.confirm) {
                                this.playVideoOpenBox(idx);
                            }
                        }));
                    }
                }), Laya.Handler.create(this, function () {
                    this.awardList.mouseEnabled = true;
                    this.closeBtn.mouseEnabled = true;
                    this.videoBtn.mouseEnabled = true;
                    this.goldBtn.mouseEnabled = true;
                    GameFrame.showToastMsg(GameFrame.langCfg.novideotxt);
                }));
            }
            else {
                GameFrame.showToastMsg(GameFrame.langCfg.novideotxt);
            }
        }

        playVideoGetGold() {
            GameFrame.playClickSound();

            if (Channel.isVideoEnable()) {
                this.awardList.mouseEnabled = false;
                this.closeBtn.mouseEnabled = false;
                this.videoBtn.mouseEnabled = false;
                this.goldBtn.mouseEnabled = true;
                Channel.playVideo(Laya.Handler.create(this, function () {
                    this.awardList.mouseEnabled = true;
                    this.closeBtn.mouseEnabled = true;
                    this.videoBtn.mouseEnabled = true;
                    this.goldBtn.mouseEnabled = true;

                    var playerInfo = GameFrame.playerInfo;
                    let gold = GameFrame.pluginCfg && GameFrame.pluginCfg.pg_video_2_gold ? GameFrame.pluginCfg.pg_video_2_gold : 500;
                    playerInfo.gold += gold
                    GameFrame.savePlayerInfo({ gold: playerInfo.gold });
                    Laya.stage.event(Event.GOLD_UPDATE);

                    GameFrame.showToastMsg(`${GameFrame.langCfg.getcointxt} ${gold}`);

                    GameFrame.closeView(this.owner);

                }), Laya.Handler.create(this, function () {
                    this.awardList.mouseEnabled = true;
                    this.closeBtn.mouseEnabled = true;
                    this.videoBtn.mouseEnabled = true;
                    this.goldBtn.mouseEnabled = true;

                    if (ActivityCenter.isEnableCancelVideoModal()) {
                        ActivityCenter.showVideoConfirmModal(Laya.Handler.create(this, function (res) {
                            if (res && res.confirm) {

                                this.playVideoGetGold();
                            }
                        }));
                    }
                }), Laya.Handler.create(this, function () {
                    this.awardList.mouseEnabled = true;
                    this.closeBtn.mouseEnabled = true;
                    this.videoBtn.mouseEnabled = true;
                    this.goldBtn.mouseEnabled = true;
                    GameFrame.showToastMsg(GameFrame.langCfg.novideotxt);
                }));
            }
            else {
                GameFrame.showToastMsg(GameFrame.langCfg.novideotxt);
            }
        }
        playVideoGetKey() {
            GameFrame.playClickSound();
            if (Channel.isVideoEnable()) {
                this.awardList.mouseEnabled = false;
                this.closeBtn.mouseEnabled = false;
                this.videoBtn.mouseEnabled = false;
                this.goldBtn.mouseEnabled = false;
                Channel.playVideo(Laya.Handler.create(this, function () {
                    this.awardList.mouseEnabled = true;
                    this.closeBtn.mouseEnabled = true;
                    this.videoBtn.mouseEnabled = true;
                    this.goldBtn.mouseEnabled = true;
                    this.keyVal = 3;
                    this.keyNum.visible = true;
                    this.keyNum.value = `*${this.keyVal}`;
                    this.keyIcon.visible = true;
                    this.videoBtn.visible = false;
                    this.videoDesc.visible = false;
                }), Laya.Handler.create(this, function () {
                    this.awardList.mouseEnabled = true;
                    this.closeBtn.mouseEnabled = true;
                    this.videoBtn.mouseEnabled = true;
                    this.goldBtn.mouseEnabled = true;
                    if (ActivityCenter.isEnableCancelVideoModal()) {
                        ActivityCenter.showVideoConfirmModal(Laya.Handler.create(this, function (res) {
                            if (res && res.confirm) {
                                this.playVideoGetKey();
                            }
                        }));
                    }

                }), Laya.Handler.create(this, function () {
                    this.awardList.mouseEnabled = true;
                    this.closeBtn.mouseEnabled = true;
                    this.videoBtn.mouseEnabled = true;
                    this.goldBtn.mouseEnabled = true;
                    GameFrame.showToastMsg(GameFrame.langCfg.novideotxt);
                }));
            }
            else {
                GameFrame.showToastMsg(GameFrame.langCfg.novideotxt);
            }
        }

        saveAward(boxIdx) {
            GameFrame.playClickSound();
            var awardData = this.awardList.array[boxIdx];
            if (awardData.goodsType == null) {
                GameFrame.savePlayerInfo({ gold: GameFrame.playerInfo.gold + awardData.count });
                GameFrame.showToastMsg(`${GameFrame.langCfg.getcointxt} ${awardData.count}`);
            }
            else {
                var goodsList = GameFrame.goodsData[awardData.goodsType];
                for (let index = 0; index < goodsList.length; index++) {
                    if (goodsList[index].id == awardData.goodsId) {
                        GameFrame.playerInfo.goodsInBag[awardData.goodsType].push(awardData.goodsId);
                        GameFrame.savePlayerInfo(GameFrame.playerInfo);
                        GameFrame.showToastMsg(`${GameFrame.langCfg.gettxt} ${goodsList[index].name}`);
                        break;
                    }
                }
            }

            var data = this.awardList.getItem(boxIdx);
            data.opened = 1;
            data["update"] = !data["update"];
            this.awardList.setItem(boxIdx, data);

            if (!awardData.ad) {
                this.keyVal -= 1;
                this.keyNum.value = `*${this.keyVal}`;
            }

            if (this.keyVal == 0) {
                this.keyNum.visible = false;
                this.keyIcon.visible = false;
                this.videoBtn.visible = true;
                this.goldBtn.visible = true;
                this.videoDesc.visible = true;

                this.closeBtn.visible = true;
                this.closeBtn.addComponent(fc.view.DelayShow);
                GameFrame.showToastMsg(GameFrame.langCfg.nokeytxt);
            }
        }
    }
    Laya.ClassUtils.regClass("fc.zsks.SecretBoxPage", SecretBoxPage);

    var viewCfg = {
        "Award": {
            "url": "fc-layout/view/AwardPage.scene",
            "script": "fc.view.AwardPage"
        },
        "Loading": {
            "url": "fc-layout/view/LoadingPage.scene",
            "script": "fc.view.LoadingPage"
        },
        "Home": {
            "url": "fc-layout/view/HomePage.scene",
            "script": "fc.view.HomePage"
        },
        "Game": {
            "url": "fc-layout/view/GamePage.scene",
            "script": "fc.view.GamePage"
        },
        "Success": {
            "url": "fc-layout/view/SuccessPage.scene",
            "script": "fc.view.SuccessPage"
        },
        "Failed": {
            "url": "fc-layout/view/FailedPage.scene",
            "script": "fc.view.FailedPage"
        },
        "AdHome": {
            "url": "fc-layout/view/AdHomePage.scene",
            "script": "fc.view.AdHomePage"
        },
        "MoreInfo": {
            "url": "fc-layout/view/MoreInfoPage.scene",
            "script": "fc.view.MoreInfoPage"
        },
        "Knock": {
            "url": "fc-layout/view/KnockPage.scene",
            "script": "fc.view.KnockPage"
        },
        "Sign": {
            "url": "fc-layout/view/SignPage.scene",
            "script": "fc.view.SignPage"
        },
        "Store": {
            "url": "fc-layout/view/StorePage.scene",
            "script": "fc.view.StorePage"
        },
        "Strength": {
            "url": "fc-layout/view/StrengthPage.scene",
            "script": "fc.view.StrengthPage"
        },
        "SecretBox": {
            "url": "fc-layout/view/SecretBoxPageZsKs.scene",
            "script": "fc.zsks.SecretBoxPage"
        },
        "SecretAward": {
            "url": "fc-layout/view/SecretAwardPageZsKs.scene",
            "script": "fc.zsks.SecretAwardPage"
        },
        "ShareRec": {
            "url": "fc-layout/view/ShareRecPage.scene",
            "script": "fc.view.ShareRecPage"
        },
        "Trial": {
            "url": "fc-layout/view/TrialPageZsKs.scene",
            "script": "fc.zsks.TrialPage"
        },
        "Turntable": {
            "url": "fc-layout/view/TurntablePage.scene",
            "script": "fc.view.TurntablePage"
        },
        "InsertAd": {
            "url": "fc-layout/view/VideoPage.scene",
            "script": "fc.view.InsertAdPage"
        },
        "FullScreenAd": {
            "url": "fc-layout/view/VideoPage.scene",
            "script": "fc.view.FullScreenAdPage"
        },
        "NativeAdb": {
            "url": "fc-layout/view/NativeAdbPage.scene",
            "script": "fc.view.NativeAdPage"
        },
        "NativeAdm": {
            "url": "fc-layout/view/NativeAdmPage.scene",
            "script": "fc.view.NativeAdPage"
        }
    };
    var sceneCfg = {
        "loading": {
            "type": "sync",
            "array": [
                {
                    "viewId": "Loading"
                }
            ]
        },
        "home": {
            "type": "sync",
            "array": [
                {
                    "viewId": "Store",
                    "checkCode": 2,
                    "zOrder": 2
                },
                {
                    "viewId": "Home",
                    "zOrder": 1
                },
                {
                    "viewId": "Sign",
                    "checkCode": 1,
                    "zOrder": 2
                }
            ]
        },
        "prepare": {
            "type": "sequence",
            "array": [
                {
                    "viewId": "Trial",
                    "checkCode": 4
                }
            ]
        },
        "game": {
            "type": "sync",
            "array": [
                {
                    "viewId": "Game"
                }
            ]
        },
        "failed": {
            "type": "sequence",
            "array": [
                {
                    "viewId": "ShareRec"
                },
                {
                    "viewId": "SecretBox",
                    "checkCode": 7
                },
                {
                    "viewId": "Failed"
                }
            ]
        },
        "success": {
            "type": "sequence",
            "array": [
                {
                    "viewId": "SecretAward",
                    "checkCode": 7
                },
                {
                    "viewId": "ShareRec"
                },
                {
                    "viewId": "Success"
                }
            ]
        },
        "over": {
            "type": "sequence",
            "array": [

            ]
        }
    };
    var bannerConfig = {
        "Sign": { "delay": 10 },
        "Trial": { "delay": 10 },
        "Failed": { "delay": 10 },
        "Success": { "delay": 10 },
        "Home": { "delay": 10 },
        "SecretBox": { "delay": 10 },
        "ShareRec": { "delay": 10 },
        "SecretAward": { "delay": 10 }
    };
    var additiveUIConfig = {
        "Home": [
            {
                "viewUrl": "fc-layout/prefab/additive/GoldShowAdd.json",
                "parentRoot": "topUI",
                "x": 32,
                "y": 326,
                "scriptType": "fc.view.GoldShowAdd"
            },
            {
                "viewUrl": "fc-layout/prefab/additive/StrengthShowAdd.json",
                "parentRoot": "topUI",
                "x": 310,
                "y": 326,
                "scriptType": "fc.view.StrengthShowAdd",
                "checkCode": 2
            },
            {
                "viewUrl": "fc-layout/prefab/additive/StartGame.json",
                "parentRoot": "bottomUI",
                "x": 375,
                "y": 359,
                "scriptType": "fc.view.AniLoopPlay"
            }
        ],
        "Game": [
            {
                "viewUrl": "fc-layout/prefab/additive/GoldShow.json",
                "parentRoot": "topUI",
                "x": 32,
                "y": 326,
                "scriptType": "fc.view.GoldShow"
            },
            {
                "viewUrl": "fc-layout/prefab/additive/StrengthShow.json",
                "parentRoot": "topUI",
                "x": 310,
                "y": 326,
                "scriptType": "fc.view.StrengthShow",
                "checkCode": 2
            },
            {
                "viewUrl": "fc-layout/prefab/additive/ResetBtn.json",
                "parentRoot": "topUI",
                "x": 30,
                "y": 420,
                "scriptType": "fc.view.ResetBtn",
                "checkCode": 3
            }
        ],
        "Failed": [
            {
                "viewUrl": "fc-layout/prefab/additive/FailEmoji.json",
                "parentRoot": "middleUI",
                "x": 375,
                "y": 120,
                "scriptType": "fc.view.AniLoopPlay"
            }
        ],
        "Success": [
            {
                "viewUrl": "fc-layout/prefab/additive/SuccessEmoji.json",
                "parentRoot": "middleUI",
                "x": 375,
                "y": 120,
                "scriptType": "fc.view.AniLoopPlay"
            }
        ]
    }
    var playerCfg = {
        "userId": "1",
        "isNew": 1,
        "isInGuide": 1,
        "signedDays": 0,
        "signTime": 0,
        "levelId": 1,
        "gold": 0,
        "strength": 5,
        "sound": 1,
        "goodsInUse": [0],
        "goodsInBag": [[0]]
    };

    function isAdditiveUIVisible(code) {
        if (!code) {
            return true;
        }

        if (code == 1) {
            return ActivityCenter.isEnableExportAd();
        }

        if (code == 2) {
            return ActivityCenter.isStrengthOpened();
        }

        if (code == 3) {
            return ActivityCenter.isResetGameOpened();
        }

        return false;
    }

    function isViewValid(code) {
        if (!code) {
            return true;
        }

        if (code == 1) {
            return ActivityCenter.isEnablePopSign();
        }

        if (code == 2) {
            return ActivityCenter.isEnablePopStore();
        }

        if (code == 3) {
            return ActivityCenter.isEnablePopHomeAd();
        }

        if (code == 4) {
            return ActivityCenter.isEnablePopTrial();
        }

        if (code == 5) {
            return ActivityCenter.isEnablePopCrazyKnock();
        }

        if (code == 6) {
            return ActivityCenter.isEnablePopAdHome();
        }

        if (code == 7) {
            return ActivityCenter.isEnablePopSecretAward();
        }

        if (code == 104) {
            return ActivityCenter.isEnablePopVip();
        }

        if (code == 105) {
            return ActivityCenter.isEnablePopShareRec();
        }

        return false;
    }

    class ShareRecPage extends fc.plugin.BaseView {

        constructor() {
            super();
            this.shareBtn = null;
            this.awardLab = null;
            this.closeBtn = null;
            this.shareImg = null;
            this.shareBtn2 = null;
            this.hasShared = false;
            this.shareAward = null;
        }

        onAwake() {
            super.onAwake();
            fc.view.linkView2Script(this, this.owner, [
                "shareBtn",
                "awardLab",
                "closeBtn",
                "shareImg",
                "shareBtn2"
            ]);

            this.shareImg.loadImage(ActivityCenter.getShareImage(), Laya.Handler.create(this, this.onImageReady, [this.shareImg.width]));
            this.shareAward = ActivityCenter.getShareRecAward();
            if (this.shareAward == null || (this.shareAward instanceof Object) == false) {
                this.awardLab.visible = false;
            }
            else if (this.shareAward.type == 0 && this.shareAward.count) {
                this.awardLab.text = `分享录屏可以获得${this.shareAward.count}金币`;
            }
            else if (this.shareAward.type == 1 && this.shareAward.goodsType != null && this.shareAward.goodsId != null &&
                GameFrame.playerInfo.goodsInBag[this.shareAward.goodsType].indexOf(this.shareAward.goodsId) == -1) {
                this.awardLab.text = `分享录屏免费获得一个道具`;
            }
            else {
                this.awardLab.visible = false;
            }
            this.closeBtn.addComponent(fc.view.DelayShow);
            this.shareBtn.on(Laya.Event.CLICK, this, this.shareRec);
            this.closeBtn.on(Laya.Event.CLICK, this, this.closeView);
            this.shareBtn2.on(Laya.Event.CLICK, this, this.shareRec);
        }

        onDestroy() {
            super.onDestroy();

            this.shareBtn.off(Laya.Event.CLICK, this, this.shareRec);
            this.closeBtn.off(Laya.Event.CLICK, this, this.closeView);
            this.shareBtn2.off(Laya.Event.CLICK, this, this.shareRec);
        }

        shareRec() {
            GameFrame.playClickSound();
            if (Channel.canShareRecorder() == false) {
                GameFrame.showToastMsg(GameFrame.langCfg.sharetimeshort);
                return;
            }
            if (this.hasShared) {
                return;
            }
            Channel.shareAppMessage(
                {},

                Laya.Handler.create(this, function () {
                    this.hasShared = true;
                    Laya.stage.on(Event.DEVICE_SHOW, this, this.onShareReturn);
                }),
                Laya.Handler.create(this, function () {
                    this.hasShared = false;
                    GameFrame.showToastMsg(GameFrame.langCfg.sharefailtxt);
                }));
        }

        closeView() {
            GameFrame.playClickSound();
            GameFrame.closeView(this.owner);
        }

        onShareReturn() {
            var playerInfo = GameFrame.playerInfo;
            if (this.shareAward == null || (this.shareAward instanceof Object) == false) {
                this.awardLab.visible = false;
            }
            else if (this.shareAward.type == 0 && this.shareAward.count) {
                playerInfo.gold += this.shareAward.count;
                GameFrame.savePlayerInfo({ gold: playerInfo.gold });
                GameFrame.showToastMsg(GameFrame.langCfg.getcointxt + ":" + this.shareAward.count);
                Laya.stage.event(Event.GOLD_UPDATE);
            }
            else if (this.shareAward.type == 1 && this.shareAward.goodsType != null && this.shareAward.goodsId != null
                && playerInfo.goodsInBag != null
                && playerInfo.goodsInBag[this.shareAward.goodsType].indexOf(this.shareAward.goodsId) == -1) {
                playerInfo.goodsInBag[this.shareAward.goodsType].push(this.shareAward.goodsId);
                GameFrame.savePlayerInfo({ goodsInBag: playerInfo.goodsInBag });
            }
            this.owner.timerOnce(1000, this, function () {
                GameFrame.closeView(this.owner);
            });
        }
    }
    Laya.ClassUtils.regClass("fc.view.ShareRecPage", ShareRecPage);

    class RecorderCtrl extends Laya.Script {

        onAwake() {
            Laya.stage.on(Event.GAME_START, this, this.onGameStart);
            Laya.stage.on(Event.GAME_RELIVE_START, this, this.onGameReliveStart);
            Laya.stage.on(Event.GAME_SHOW_WIN, this, this.onGameSuccess);
            Laya.stage.on(Event.GAME_SHOW_FAIL, this, this.onGameFailed);
            Laya.stage.on(Event.GAME_RESET, this, this.onGameReset);
        }

        onDestroy() {
            Laya.stage.off(Event.GAME_START, this, this.onGameStart);
            Laya.stage.off(Event.GAME_RELIVE_START, this, this.onGameReliveStart);
            Laya.stage.off(Event.GAME_SHOW_WIN, this, this.onGameSuccess);
            Laya.stage.off(Event.GAME_SHOW_FAIL, this, this.onGameFailed);
            Laya.stage.off(Event.GAME_RESET, this, this.onGameReset);
        }

        onGameStart() {
            Channel.startRecorder();
        }

        onGameReliveStart() {
            Channel.startRecorder();
        }

        onGameSuccess() {
            Channel.stopRecorder();
        }

        onGameFailed() {
            Channel.stopRecorder();
        }

        onGameReset() {
            Channel.stopRecorder();
        }
    }

    class PlatformConfig extends Laya.Script {

        static get playerInfo() {
            return GameFrame.playerInfo;
        }

        static set playerInfo(val) {
            if (val) {
                var keys = Object.keys(val);
                for (var index = 0; index < keys.length; index++) {
                    GameFrame.playerInfo[keys[index]] = val[keys[index]];
                }
                if (keys.length > 0) {
                    GameFrame.savePlayerInfo(val);
                }
            }
        }

        static init(appCfgUrl) {
            PlatformConfig.appCfgUrl = appCfgUrl;
            Channel.login();//指色的登陆
            Channel.loadCfg(this.loadCfgSuccess);//指色的配置
        }

        static loadCfgSuccess(cfg) {
            if (cfg) {
                PlatformConfig.adZsCfg = cfg;
                Laya.stage.addComponent(PlatformConfig);
                Laya.stage.addComponent(RecorderCtrl);
                Laya.stage.addComponent(BannerCtrl);
                Laya.stage.addComponent(AdditiveUICtrl);
                Laya.stage.addComponent(ActivityCenter);
                GameFrame.startUp(PlatformConfig.appCfgUrl ? PlatformConfig.appCfgUrl : `config/${PlatformConfig.platformTag}/appConfig.${PlatformConfig.platformTag}.json`, PlatformConfig.platformTag, viewCfg, sceneCfg, playerCfg);
            }
        }

        constructor() {
            super();
        }

        onAwake() {

            GameFrame.setViewValidCheckFun(isViewValid);
            BannerCtrl.bannerConfig = bannerConfig;
            AdditiveUICtrl.additiveUIConfig = additiveUIConfig;
            AdditiveUICtrl.checkFun = isAdditiveUIVisible;


            Laya.stage.once(Event.LAUNCH_COMPLETED, this, this.onLaunchResReady);
            Laya.stage.on(Event.APP_JUMP_CANCEL, this, this.onJumpCancel);
            Laya.stage.on(Event.UI_SCENE_LOADED, this, this.onSceneLoaded);
            Laya.stage.on(Event.UI_SCENE_UNLOADED, this, this.onSceneUnloaded);
        }

        onDestroy() {
            Laya.stage.off(Event.APP_JUMP_CANCEL, this, this.onJumpCancel);
            Laya.stage.off(Event.UI_SCENE_LOADED, this, this.onSceneLoaded);
            Laya.stage.off(Event.UI_SCENE_UNLOADED, this, this.onSceneUnloaded);
        }

        onLaunchResReady(scene3d) {
            var pluginCfg = GameFrame.pluginCfg;
            if (pluginCfg) {
                GameFrame.pluginCfg.pg_video_style = pluginCfg.pg_video_style || [0, 1, 2, 3, 4, 5];

                pluginCfg.pg_version = PlatformConfig.adZsCfg.zs_version;
                pluginCfg.pg_video_adunit = PlatformConfig.adZsCfg.zs_video_adunit;
                pluginCfg.pg_switch = 1;
                pluginCfg.pg_button_show_style = 1;
                pluginCfg.pg_text_move_time = 0;
                pluginCfg.pg_lucky_reward_switch = PlatformConfig.adZsCfg.zs_luckybox_switch ? Number(PlatformConfig.adZsCfg.zs_luckybox_switch) : 0;
                pluginCfg.pg_button_delay_time = PlatformConfig.adZsCfg.zs_button_delay_time ? Number(PlatformConfig.adZsCfg.zs_button_delay_time) : 0;
            }

            Channel.login();
        }

        onJumpCancel() {
            GameFrame.openView(fc.view.ViewId.AdHome, null, null);
        }

        onSceneUnloaded(sceneId) {
            if (sceneId == SceneId.Game) {
                Audio.stopMusic();
            }
        }

        onSceneLoaded(sceneId) {
            if (sceneId == SceneId.Game) {
                if (GameFrame.appInfo.bgm) {
                    Audio.playMusic(GameFrame.appInfo.bgm);
                }
            }
        }
    }
    PlatformConfig.platformTag = "zsks";
    PlatformConfig.appCfgUrl = "";
    PlatformConfig.adZsCfg = {};//指色的配置

    exports.PlatformConfig = PlatformConfig;

}(window.fc.platform, Laya));