window.fc = window.fc || {};
window.fc.platform = window.fc.platform || {};
window.wx = window.qq;
(function (exports, Laya) {
    'use strict';

    var Event = fc.define.Event;
    var SceneId = fc.define.SceneId;
    var Audio = fc.framework.Audio;
    var GameFrame = fc.framework.GameFrame;
    var MiscUtils = fc.framework.MiscUtils;
    var Channel = fc.framework.Channel;
    var BlockAdCtrl = fc.plugin.BlockAdCtrl;
    var AdditiveUICtrl = fc.plugin.AdditiveUICtrl;
    var ActivityCenter = fc.view.ActivityCenter;

    class BannerView extends Laya.Script {

        constructor() {
            super();
            this.config = null;
            this.otherViews = [];
            this.passedTime = 0;
            this.uiReady = false;
            this.uiVisible = true;
            this.uiHasShow = false;

            this.showNativeHandler = null;
            this.hideNativeHandler = null;
        }

        onStart() {
            if (!this.owner.viewId) {
                return;
            }

            this.config = BannerCtrl.bannerConfig[this.owner.viewId];
            this.uiReady = this.config && !this.config.delay;
            this.uiVisible = this.config && this.config.visible != false;

            if (!this.config) {
                return;
            }

            var userSetting = BannerCtrl.userSetting[this.owner.viewId];
            if (userSetting) {
                this.uiVisible = this.uiVisible && !!userSetting.visible;
                BannerCtrl.userSetting[this.owner.viewId] = null;
            }

            zs.laya.banner.WxBannerMgr.Instance.updateBanner(true, null, null, 1, true);
        }

        onEnable() {
            Laya.stage.on(Event.UI_VIEW_CLOSED, this, this.onViewClosed);
            Laya.stage.on(Event.UI_VIEW_OPENED, this, this.onViewOpened);
            Laya.stage.on(BannerCtrl.EVENT_SHOW_BANNER, this, this.onShowBanner);
            Laya.stage.on(BannerCtrl.EVENT_HIDE_BANNER, this, this.onHideBanner);
            this.showBanner();
        }

        onDisable() {
            Laya.stage.off(Event.UI_VIEW_CLOSED, this, this.onViewClosed);
            Laya.stage.off(Event.UI_VIEW_OPENED, this, this.onViewOpened);
            Laya.stage.off(BannerCtrl.EVENT_SHOW_BANNER, this, this.onShowBanner);
            Laya.stage.off(BannerCtrl.EVENT_HIDE_BANNER, this, this.onHideBanner);
            this.hideBanner();
        }

        onDestroy() {
            Laya.stage.off(Event.UI_VIEW_CLOSED, this, this.onViewClosed);
            Laya.stage.off(Event.UI_VIEW_OPENED, this, this.onViewOpened);
            Laya.stage.off(BannerCtrl.EVENT_SHOW_BANNER, this, this.onShowBanner);
            Laya.stage.off(BannerCtrl.EVENT_HIDE_BANNER, this, this.onHideBanner);
            this.hideBanner();
        }

        onUpdate() {
            if (!this.config || !this.owner.viewId) {
                return;
            }
            if (this.uiReady) {
                return;
            }
            if (!this.config.delay) {
                this.uiReady = true;
                this.showBanner();
            }
            else if (this.config.delay >= 0) {
                this.passedTime += Laya.timer.delta;
                if (this.passedTime > this.config.delay) {
                    this.uiReady = true;
                    this.showBanner();
                }
            }
        }

        onViewOpened(viewName, viewId, view) {
            if (view.zOrder < this.owner.zOrder) {
                return;
            }
            this.otherViews.push(viewId);
            this.hideBanner();
        }

        onViewClosed(viewName, viewId) {
            var idx = this.otherViews.indexOf(viewId);
            if (idx < 0) {
                return;
            }
            this.otherViews.splice(idx, 1);
            this.showBanner();
        }

        onShowBanner(viewId) {
            if (viewId != this.owner.viewId) {
                return;
            }
            this.uiVisible = true;
            this.showBanner();
        }

        onHideBanner(viewId) {
            if (viewId != this.owner.viewId) {
                return;
            }
            this.uiVisible = false;
            this.hideBanner();
        }

        showBanner() {
            if (this.otherViews.length == 0 && this.uiVisible && this.uiHasShow == false && this.uiReady) {
                this.uiHasShow = true;
                // Channel.showBannerAd();
                zs.laya.banner.WxBannerMgr.Instance.showBanner();
            }
        }

        hideBanner() {
            if (this.uiHasShow) {
                // Channel.hideBannerAd();
                zs.laya.banner.WxBannerMgr.Instance.hideAll();
                this.uiHasShow = false;
            }
        }
    }

    class BannerCtrl extends Laya.Script {

        static showBanner(viewId) {
            this.userSetting[viewId] = this.userSetting[viewId] || {};
            this.userSetting[viewId].visible = true;
            Laya.stage.event(BannerCtrl.EVENT_SHOW_BANNER, viewId);
        }

        static hideBanner(viewId) {
            if (!this.bannerConfig) {
                return;
            }
            this.userSetting[viewId] = this.bannerConfig[viewId] || {};
            this.userSetting[viewId].visible = false;
            Laya.stage.event(BannerCtrl.EVENT_HIDE_BANNER, viewId);
        }

        constructor() {
            super();
        }

        onAwake() {
            Laya.stage.once(Event.LAUNCH_COMPLETED, this, this.onLaunchResReady);
            Laya.stage.on(Event.UI_VIEW_OPENED, this, this.onViewOpened);
        }

        onDestroy() {
            Laya.stage.off(Event.UI_VIEW_OPENED, this, this.onViewOpened);
        }

        onLaunchResReady() {
            var pluginCfg = GameFrame.pluginCfg;
            if (pluginCfg) {
                if (pluginCfg.bannerConfig) {
                    BannerCtrl.bannerConfig = pluginCfg.bannerConfig;
                }
                BannerCtrl.bannerRefreshInterval = pluginCfg.pg_banner_refresh_time;
            }
        }

        onViewOpened(viewName, viewId, view) {
            view.addComponent(BannerView);
        }
    }
    BannerCtrl.EVENT_SHOW_BANNER = "BANNERCTRL_EVENT_SHOW_BANNER";
    BannerCtrl.EVENT_HIDE_BANNER = "BANNERCTRL_EVENT_HIDE_BANNER";
    BannerCtrl.userSetting = {};
    BannerCtrl.bannerConfig = {};
    BannerCtrl.bannerCreateTime = 0;
    BannerCtrl.bannerRefreshInterval = 30000;
    Laya.ClassUtils.regClass("fc.plugin.BannerCtrl", BannerCtrl);

    class KnockPage extends fc.plugin.BaseView {

        constructor() {
            super();

            this.box = null;
            this.openBtn = null;
            this.closeBtn = null;
            this.againBtn = null;
            this.progressBar = null;
            this.progressBg = null;

            this.mHammerAni = null;

            this.mIsAdOpened = false;
            this.progressWidth = 1;
            this.progressHeight = 1

            this.mOpenProgress = 0;
            this.mKncokCount = 0;

            this.mNextStepCb = null;

        }

        initView(data) {
            this.mNextStepCb = data;
        }

        onAwake() {
            super.onAwake();

            fc.view.linkView2Script(this, this.owner, [
                "box",
                "openBtn",
                "closeBtn",
                "progressBar",
                "progressBg",
                "againBtn"
            ])

            //开启进度
            this.mOpenProgress = 0;
            //是否已经打开广告
            this.mIsAdOpened = false;

            this.progressWidth = this.progressBg.displayWidth;
            this.progressHeight = this.progressBg.displayHeight;

            this.mHammerAni = this.owner["knockAni"];
            this.openBtn.on(Laya.Event.MOUSE_UP, this, this.onFingerUp);
            this.closeBtn.on(Laya.Event.MOUSE_UP, this, this.onCloseView);
            this.againBtn.on(Laya.Event.CLICK, this, this.onAgainKnock);
            this.againBtn.getChildByName('okBtn').on(Laya.Event.CLICK, this, this.onJump);
            this.closeBtn.visible = !isMistakenActive();
        }

        onDestroy() {
            super.onDestroy();
            this.openBtn.off(Laya.Event.MOUSE_UP, this, this.onFingerUp);
            this.closeBtn.off(Laya.Event.MOUSE_UP, this, this.onCloseView);
            Laya.stage.off(Event.DEVICE_SHOW, this, this.onTreasureOpen);
            this.againBtn.off(Laya.Event.CLICK, this, this.onAgainKnock);
            this.againBtn.getChildByName('okBtn').off(Laya.Event.CLICK, this, this.onJump);
            if (this.mIsAdOpened) {
                // fc.framework.Channel.hideBannerAd();
                // zs.laya.banner.WxBannerMgr.Instance.hideAll();
            }
        }

        onUpdate() {
            if (this.mOpenProgress >= 1) {
                return;
            }

            this.mOpenProgress -= getKnockEnergeDecPerFrame();
            this.mOpenProgress = Math.max(0, this.mOpenProgress);
            this.updateOpenPorgress(this.mOpenProgress);
        }


        onFingerUp() {
            GameFrame.playClickSound();
            if (this.mOpenProgress >= 1) {
                return;
            }

            this.mOpenProgress += getKnockEnergeAccPerClick();
            if (!this.mIsAdOpened && isInAdKnockEnergeRange(this.mOpenProgress)) {
                this.mIsAdOpened = true;

                var handler = Laya.Handler.create(this, function (errorMsg) {
                    if (errorMsg) {
                        GameFrame.showToastMsg(errorMsg);
                    }
                    // this.onTreasureOpen();
                });
                Channel.showGameBox(handler, handler, handler);

                Laya.stage.on(Event.DEVICE_SHOW, this, this.onTreasureOpen);
                // this.owner.timerOnce(2000, this, function () {
                // Channel.hideBannerAd();
                //    zs.laya.banner.WxBannerMgr.Instance.hideAll();
                // });
            }
            if (this.mOpenProgress >= 1) {
                this.onTreasureOpen();
            }
            this.updateOpenPorgress(this.mOpenProgress);
            this.mHammerAni && this.mHammerAni.play(0, false);
        }

        onTreasureOpen() {
            
            for (let index = 0; index < 20; index++) {
                DropItem.create(
                    this.owner.parent, 
                    "fc-res/main/img_coins.png", 
                    this.box.x + Math.floor(Math.random() * 10) * 0.1 * 400 - 200,
                    this.box.y + Math.floor(Math.random() * 10) * 0.1 * 400,
                    0, 0, index * 50);
            }
            this.owner.timerOnce(2000, null, function() {
                GameFrame.showToastMsg("获得100金币");
                GameFrame.playerInfo.gold += 100;
                GameFrame.savePlayerInfo({gold: GameFrame.playerInfo.gold});
            });
            
            Laya.stage.event(Event.KNOCK_COMPLETED);
            this.mKncokCount++;
            if (this.mKncokCount == 1 && PlatformConfig.adZsCfg.zs_ready_click_two == 1) {
                this.againBtn.visible = true;
                this.openBtn.mouseEnabled = false;
            } else {
                Laya.timer.once(500, this, this.onCloseView);
            }

        }

        onCloseView() {
            GameFrame.playClickSound();
            GameFrame.closeView(this.owner);
        }

        //更新开启进度
        updateOpenPorgress(val) {
            var progressVal = Math.min(1, Math.max(0, val));
            if (this.progressWidth < this.progressHeight) {
                this.progressBar && (this.progressBar.height = Math.max(1, this.progressHeight * progressVal));
            } else {
                this.progressBar && (this.progressBar.width = Math.max(1, this.progressWidth * progressVal));
            }
        }

        onAgainKnock() {
            this.mOpenProgress = 0;
            this.updateOpenPorgress(this.mOpenProgress);
            this.againBtn.visible = false;
            this.openBtn.mouseEnabled = true;
            this.mIsAdOpened = false;
        }

        onJump() {
            this.onCloseView();
        }
    }
    Laya.ClassUtils.regClass("fc.zs.KnockPage", KnockPage);

    class KnockBannerPage extends fc.plugin.BaseView {

        constructor() {
            super();

            this.box = null;
            this.openBtn = null;
            this.closeBtn = null;
            this.againBtn = null;
            this.progressBar = null;
            this.progressBg = null;

            this.mHammerAni = null;

            this.mIsAdOpened = false;
            this.progressWidth = 1;
            this.progressHeight = 1

            this.mOpenProgress = 0;
            this.mKncokCount = 0;

            this.mNextStepCb = null;

        }

        initView(data) {
            this.mNextStepCb = data;
        }

        onAwake() {
            super.onAwake();

            fc.view.linkView2Script(this, this.owner, [
                "box",
                "openBtn",
                "closeBtn",
                "progressBar",
                "progressBg",
                "againBtn"
            ])

            //开启进度
            this.mOpenProgress = 0;
            //是否已经打开广告
            this.mIsAdOpened = false;

            this.progressWidth = this.progressBg.displayWidth;
            this.progressHeight = this.progressBg.displayHeight;

            this.mHammerAni = this.owner["knockAni"];
            this.openBtn.on(Laya.Event.MOUSE_UP, this, this.onFingerUp);
            this.closeBtn.on(Laya.Event.MOUSE_UP, this, this.onCloseView);
            this.againBtn.on(Laya.Event.CLICK, this, this.onAgainKnock);
            this.againBtn.getChildByName('okBtn').on(Laya.Event.CLICK, this, this.onJump);
            this.closeBtn.visible = !isMistakenActive();
        }

        onDestroy() {
            super.onDestroy();
            this.openBtn.off(Laya.Event.MOUSE_UP, this, this.onFingerUp);
            this.closeBtn.off(Laya.Event.MOUSE_UP, this, this.onCloseView);
            Laya.stage.off(Event.DEVICE_SHOW, this, this.onTreasureOpen);
            this.againBtn.off(Laya.Event.CLICK, this, this.onAgainKnock);
            this.againBtn.getChildByName('okBtn').off(Laya.Event.CLICK, this, this.onJump);
            if (this.mIsAdOpened) {
                // fc.framework.Channel.hideBannerAd();
                // zs.laya.banner.WxBannerMgr.Instance.hideAll();
            }
        }

        onUpdate() {
            if (this.mOpenProgress >= 1) {
                return;
            }

            this.mOpenProgress -= getKnockEnergeDecPerFrame();
            this.mOpenProgress = Math.max(0, this.mOpenProgress);
            this.updateOpenPorgress(this.mOpenProgress);
        }


        onFingerUp() {
            GameFrame.playClickSound();
            if (this.mOpenProgress >= 1) {
                return;
            }

            this.mOpenProgress += getKnockEnergeAccPerClick();
            if (!this.mIsAdOpened && isInAdKnockEnergeRange(this.mOpenProgress)) {
                this.mIsAdOpened = true;
                this.knockBnnaer();
            }
            if (this.mOpenProgress >= 1) {
                this.onTreasureOpen();
            }
            this.updateOpenPorgress(this.mOpenProgress);
            this.mHammerAni && this.mHammerAni.play(0, false);
        }

        knockBnnaer() {
            //广告时间来了
    
            if (GameFrame.pluginCfg.pg_click_video_switch == 0) {
                zs.laya.banner.WxBannerMgr.Instance.showBanner();
                Laya.stage.on(fc.define.Event.DEVICE_SHOW, this, this.onKnockOpen);
                this.owner.timerOnce(2000, this, function () {
                    zs.laya.banner.WxBannerMgr.Instance.hideAll();
                });
            } else {
                Laya.MouseManager.enabled = false;
                Channel.playVideo(Laya.Handler.create(this, () => {
                    Laya.MouseManager.enabled = true;
                    this.onKnockOpen();
                }), Laya.Handler.create(this, () => {
                    Laya.MouseManager.enabled = true;
                    this.onKnockOpen();
                }), Laya.Handler.create(this, () => {
                    Laya.MouseManager.enabled = true;
                    this.onKnockOpen();
                }));
            }
    
            console.log('显示banner');
    
    
        }

        onKnockOpen() {

            zs.laya.banner.WxBannerMgr.Instance.hideAll();

            Laya.stage.off(fc.define.Event.DEVICE_SHOW, this, this.onKnockOpen);
            console.log('点击了广告回到游戏');
    
            this.onTreasureOpen();
    
    
        }

        onTreasureOpen() {
            
            for (let index = 0; index < 20; index++) {
                DropItem.create(
                    this.owner.parent, 
                    "fc-res/main/img_coins.png", 
                    this.box.x + Math.floor(Math.random() * 10) * 0.1 * 400 - 200,
                    this.box.y + Math.floor(Math.random() * 10) * 0.1 * 400,
                    0, 0, index * 50);
            }
            this.owner.timerOnce(2000, null, function() {
                GameFrame.showToastMsg("获得100金币");
                GameFrame.playerInfo.gold += 100;
                GameFrame.savePlayerInfo({gold: GameFrame.playerInfo.gold});
            });

            Laya.stage.event(Event.KNOCK_COMPLETED);
            this.mKncokCount++;
            if (this.mKncokCount == 1 && PlatformConfig.adZsCfg.zs_ready_click_two == 1) {
                this.againBtn.visible = true;
                this.openBtn.mouseEnabled = false;
            } else {
                Laya.timer.once(500, this, this.onCloseView);
            }

        }

        onCloseView() {
            GameFrame.playClickSound();
            GameFrame.closeView(this.owner);
        }

        //更新开启进度
        updateOpenPorgress(val) {
            var progressVal = Math.min(1, Math.max(0, val));
            if (this.progressWidth < this.progressHeight) {
                this.progressBar && (this.progressBar.height = Math.max(1, this.progressHeight * progressVal));
            } else {
                this.progressBar && (this.progressBar.width = Math.max(1, this.progressWidth * progressVal));
            }
        }

        onAgainKnock() {
            this.mOpenProgress = 0;
            this.updateOpenPorgress(this.mOpenProgress);
            this.againBtn.visible = false;
            this.openBtn.mouseEnabled = true;
            this.mIsAdOpened = false;
        }

        onJump() {
            this.onCloseView();
        }
    }
    Laya.ClassUtils.regClass("fc.zs.KnockBannerPage", KnockBannerPage);


    class VidewAwardEvent {
    }
    VidewAwardEvent.PLAY_VIDEO = "PLAY_VIDEO";
    VidewAwardEvent.JUMP_AWARD = "JUMP_AWARD";

    class VideoAwardStrategy_CancelDelay extends Laya.Script {

        constructor() {
            super();
            this.playVideoBtn = null;
            this.jumpBtn = null;
            this.skinRoot = "";
        }

        onAwake() {
            this.playVideoBtn = this.owner.getChildByName("playVideoBtn");
            this.jumpBtn = this.owner.getChildByName("jumpBtn");

            this.skinRoot = this.playVideoBtn.skin;
            this.skinRoot = this.skinRoot.substring(0, this.skinRoot.lastIndexOf("/") + 1);

            this.playVideoBtn && this.playVideoBtn.on(Laya.Event.CLICK, this, this.openVideo);
            this.jumpBtn && this.jumpBtn.on(Laya.Event.CLICK, this, this.toJump);

            if (ActivityCenter.isMistakenActive()) {
                this.jumpBtn.visible = false;
                this.owner.timerOnce(getBtnDelayTime(), this, function () {
                    this.jumpBtn.visible = true;
                })

            }
            else {
                this.jumpBtn.visible = true;
            }
        }

        onDestroy() {
            this.playVideoBtn && this.playVideoBtn.off(Laya.Event.CLICK, this, this.openVideo);
            this.jumpBtn && this.jumpBtn.off(Laya.Event.CLICK, this, this.toJump);
        }

        openVideo() {
            GameFrame.playClickSound();
            Laya.stage.event(VidewAwardEvent.PLAY_VIDEO);
        }

        toJump() {
            GameFrame.playClickSound();
            if (ActivityCenter.isMistakenActive() && PlatformConfig.adZsCfg.zs_start_video_switch && Number(PlatformConfig.adZsCfg.zs_start_video_switch) == 1) {
                Laya.stage.event(VidewAwardEvent.PLAY_VIDEO);
            } else {
                Laya.stage.event(VidewAwardEvent.JUMP_AWARD);
            }

        }
    }

    class TrialPage extends fc.plugin.BaseView {

        constructor() {
            super();

            this.playBtn = null;
            this.s_switchPos = null;
            this.s_allAd = null;
            this.s_miniCancel = null;
            this.s_cancelDelay_0 = null;
            this.s_cancelDelay_1 = null;
            this.s_checkMove = null;

            this.trialObj = null;
            this.curBtnGroup = null;

            this.mTexture = null;
            this.mCurTrialGoods = null;
        }

        initView(data) {
            this.mNextStepCb = data && data.nextCb ? data.nextCb : null;
        }

        onAwake() {
            super.onAwake();

            fc.view.linkView2Script(this, this.owner, ["playBtn", 'trialObj', "s_switchPos", "s_allAd", "s_miniCancel", "s_cancelDelay_0", "s_cancelDelay_1", "s_checkMove"]);
            this.s_switchPos.visible = false;
            this.s_allAd.visible = false;
            this.s_miniCancel.visible = false;
            this.s_cancelDelay_0.visible = false;
            this.s_cancelDelay_1.visible = false;
            this.s_checkMove.visible = false;


            this.s_cancelDelay_0.visible = true;
            this.s_cancelDelay_0.addComponent(VideoAwardStrategy_CancelDelay);
            this.curBtnGroup = this.s_cancelDelay_0;

            var goodsArray = GameFrame.goodsCfg;
            var playerInfo = GameFrame.playerInfo;
            var validTrialPool = [];
            for (var type = 0; type < goodsArray.length; type++) {
                var goodsList = goodsArray[type];
                for (let idx = 0; idx < goodsList.length; idx++) {
                    const element = goodsList[idx];
                    if (element.recommend != 0 && playerInfo.goodsInBag[type].indexOf(idx) == -1) {
                        validTrialPool.push(element);
                    }
                }
            }
            this.mCurTrialGoods = validTrialPool[Math.floor(Math.random() * validTrialPool.length)];
            if (GameFrame.pluginCfg.modelPreviewByImage != 1) {
                this.mTexture = new Laya.Texture(fc.plugin.TextureCamera.RenderTexture, Laya.Texture.DEF_UV);
                this.trialObj.graphics.drawTexture(this.mTexture);
                Laya.stage.event(Event.TEXTURE_CAMERA_INIT, [this.mCurTrialGoods.subPkg, this.mCurTrialGoods.prefab, this.mCurTrialGoods.showAnim]);
            }
            else {
                this.trialObj.skin = this.mCurTrialGoods.iconUrl;
            }

            Laya.stage.on(VidewAwardEvent.PLAY_VIDEO, this, this.openVideo);
            Laya.stage.on(VidewAwardEvent.JUMP_AWARD, this, this.closeView);

            if (ActivityCenter.isMistakenActive()) {
                this.playBtn.on(Laya.Event.CLICK, this, this.openVideo);
            }
        }

        onDestroy() {
            super.onDestroy();

            if (this.mTexture) {
                this.trialObj.graphics.clear();
                this.mTexture.bitmap = null;
                this.mTexture.destroy();
                this.mTexture = null;
                Laya.stage.event(Event.TEXTURE_CAMERA_CLEAR);
            }

            Laya.stage.off(VidewAwardEvent.PLAY_VIDEO, this, this.openVideo);
            Laya.stage.off(VidewAwardEvent.JUMP_AWARD, this, this.closeView);

            if (ActivityCenter.isMistakenActive()) {
                this.playBtn.off(Laya.Event.CLICK, this, this.openVideo);
            }
        }

        closeView() {
            GameFrame.closeView(this.owner);
        }

        changeMainRole() {
            Laya.stage.event(Event.CHANGE_CUR_MODEL, [this.mCurTrialGoods.prefab, this.mCurTrialGoods.showAnim, this.mCurTrialGoods]);
            this.owner.timerOnce(1000, this, function () {
                GameFrame.closeView(this.owner);
            });
        }

        openVideo() {
            GameFrame.playClickSound();
            if (Channel.isVideoEnable()) {
                this.curBtnGroup.mouseEnabled = false;
                this.playBtn.visible = false;
                Channel.playVideo(Laya.Handler.create(this, function () {
                    this.changeMainRole();
                }), Laya.Handler.create(this, function () {
                    this.curBtnGroup.mouseEnabled = true;
                    this.playBtn.visible = true;
                    if (ActivityCenter.isEnableCancelVideoModal()) {
                        ActivityCenter.showVideoConfirmModal(Laya.Handler.create(this, function (res) {
                            if (res && res.confirm) {
                                this.openVideo();
                            } else {
                                this.closeView();
                            }
                        }));
                    } else {
                        this.closeView();
                    }
                }), Laya.Handler.create(this, function () {
                    this.curBtnGroup.mouseEnabled = true;
                    this.playBtn.visible = true;
                    GameFrame.showToastMsg(GameFrame.langCfg.novideotxt);
                    this.closeView();
                }));
            }
            else {
                GameFrame.showToastMsg(GameFrame.langCfg.novideotxt);
                this.closeView();
            }
        }
    }
    Laya.ClassUtils.regClass("fc.zsqq.TrialPage", TrialPage);
    
    class DropItemFactory {
        get Skin() {
            return this.mSkin;
        }

        get Sign() {
            return this.mSign;
        }

        constructor(skin) {
            this.mSkin = skin;
            this.mSign = "fc_dropItem_" + this.mSkin;
        }

        create() {
            return Laya.Pool.getItemByCreateFun(this.mSign, this.createFun, this);
        }

        createFun() {
            return new Laya.Image(this.mSkin);
        }
    }

    class DropItem extends Laya.Script {

        static create(root, skin, srcX, srcY, dstX, dstY, delay) {
            var factory = this.sFactoryMap[skin];
            if (factory == null) {
                this.sFactoryMap[skin] = factory = new DropItemFactory(skin);
            }
            var dropItem = factory.create();
            dropItem.zOrder = 10;
            root.addChild(dropItem);
            var script = dropItem.getComponent(DropItem);
            if (script == null) {
                script = dropItem.addComponent(DropItem);
            }
            script.init(factory.Sign, srcX, srcY, dstX, dstY, delay);
            return dropItem;
        }

        get ItemImage() {
            return this.owner;
        }

        constructor() {
            super();
            this.mSign = "";
        }
        
        init(sign, srcX, srcY, dstX, dstY, delay) {
            this.ItemImage.visible = true;
            this.ItemImage.x = srcX;
            this.ItemImage.y = srcY;
            this.mSign = sign;
            Laya.Tween.to(this.ItemImage, 
                {x: dstX, y: dstY}, 
                500, 
                Laya.Ease.strongIn, 
                Laya.Handler.create(this, this.onReachDst), delay, false);
        }

        onReachDst() {
            this.ItemImage.visible = false;
            this.ItemImage.removeSelf();
            Laya.Pool.recover(this.mSign, this.ItemImage);
        }
    }
    DropItem.sFactoryMap = {};

    function getBtnDelayTime() {
        return GameFrame.pluginCfg ? GameFrame.pluginCfg.pg_button_delay_time : 0;
    }


    function getKnockEnergeDecPerFrame() {
        return GameFrame.pluginCfg.pg_fastclick_progress_back;// : 0.1;
    }

    function getKnockEnergeAccPerClick() {
        return GameFrame.pluginCfg.pg_fastclick_progress_add;// : 0.15;
    }

    function isInAdKnockEnergeRange(val) {
        return val > GameFrame.pluginCfg.pg_fastclick_ad_percent[0] && val < GameFrame.pluginCfg.pg_fastclick_ad_percent[1];
    }

    var viewCfg = {
        "Award": {
            "url": "fc-layout/view/AwardPage.scene",
            "script": "fc.view.AwardPage"
        },
        "Loading": {
            "url": "fc-layout/view/LoadingPage.scene",
            "script": "fc.view.LoadingPage"
        },
        "Home": {
            "url": "fc-layout/view/HomePage.scene",
            "script": "fc.view.HomePage"
        },
        "Game": {
            "url": "fc-layout/view/GamePage.scene",
            "script": "fc.view.GamePage"
        },
        "Success": {
            "url": "fc-layout/view/SuccessPageZsqq.scene",
            "script": "fc.view.SuccessPage"
        },
        "Failed": {
            "url": "fc-layout/view/FailedPageZsqq.scene",
            "script": "fc.view.FailedPage"
        },
        "AdHome": {
            "url": "fc-layout/view/AdHomePage.scene",
            "script": "fc.view.AdHomePage"
        },
        "MoreInfo": {
            "url": "fc-layout/view/MoreInfoPage.scene",
            "script": "fc.view.MoreInfoPage"
        },
        "Knock": {
            "url": "fc-layout/view/KnockPageZsqq.scene",
            "script": "fc.zs.KnockPage"
        },
        "KnockBanner": {
            "url": "fc-layout/view/KnockBannerPageZsqq.scene",
            "script": "fc.zs.KnockBannerPage"
        },
        "Sign": {
            "url": "fc-layout/view/SignPage.scene",
            "script": "fc.view.SignPage"
        },
        "Store": {
            "url": "fc-layout/view/StorePage.scene",
            "script": "fc.view.StorePage"
        },
        "Strength": {
            "url": "fc-layout/view/StrengthPage.scene",
            "script": "fc.view.StrengthPage"
        },
        "SecretBox": {
            "url": "fc-layout/view/SecretBoxPage.scene",
            "script": "fc.view.SecretBoxPage"
        },
        "SecretAward": {
            "url": "fc-layout/view/SecretAwardPage.scene",
            "script": "fc.view.SecretAwardPage"
        },
        "ShareRec": {
            "url": "fc-layout/view/ShareRecPage.scene",
            "script": "fc.view.ShareRecPage"
        },
        "Trial": {
            "url": "fc-layout/view/TrialPageZsqq.scene",
            "script": "fc.zsqq.TrialPage"
        },
        "Turntable": {
            "url": "fc-layout/view/TurntablePage.scene",
            "script": "fc.view.TurntablePage"
        },
        "InsertAd": {
            "url": "fc-layout/view/VideoPage.scene",
            "script": "fc.view.InsertAdPage"
        },
        "FullScreenAd": {
            "url": "fc-layout/view/VideoPage.scene",
            "script": "fc.view.FullScreenAdPage"
        },
        "NativeAdb": {
            "url": "fc-layout/view/NativeAdbPage.scene",
            "script": "fc.view.NativeAdPage"
        },
        "NativeAdm": {
            "url": "fc-layout/view/NativeAdmPage.scene",
            "script": "fc.view.NativeAdPage"
        }
    };
    var sceneCfg = {
        "loading": {
            "type": "sync",
            "array": [
                {
                    "viewId": "Loading"
                }
            ]
        },
        "home": {
            "type": "sync",
            "array": [
               
                {
                    "viewId": "InsertAd",
                    "checkCode": 3,
                    "zOrder": 2
                },
                {
                    "viewId": "Home",
                    "zOrder": 1
                },
                {
                    "viewId": "Sign",
                    "checkCode": 1,
                    "zOrder": 2
                },
                {
                    "viewId": "Store",
                    "checkCode": 2,
                    "delay": 300,
                    "zOrder": 2
                }
            ]
        },
        "prepare": {
            "type": "sequence",
            "array": [
                {
                    "viewId": "Trial",
                    "checkCode": 4
                }
            ]
        },
        "game": {
            "type": "sync",
            "array": [
                {
                    "viewId": "Game"
                }
            ]
        },
        "success": {
            "type": "sequence",
            "array": [
                {
                    "viewId": "Knock",
                    "checkCode": 5
                },
                {
                    "viewId": "SecretAward",
                    "checkCode": 6
                },
                {
                    "viewId": "Success"
                }
            ]
        },

        "failed": {
            "type": "sequence",
            "array": [
                {
                    "viewId": "Knock",
                    "checkCode": 5
                },
                {
                    "viewId": "SecretBox",
                    "checkCode": 6
                },
                {
                    "viewId": "Failed"
                }
            ]
        },
        "over": {
            "type": "sequence",
            "array": [
            ]
        }
    };
    var blockConfig = {
        "Home": {"x": 0, "y": 115, "size": 5, "delay": 100},
        "Game": {"x": 0, "y": 115, "size": 5, "delay": 100},
        "Failed": {"x": 0, "y": 430, "size": 5, "delay": 100},
        "Success": {"x": 0, "y": 430, "size": 5, "delay": 100}
    }
    var bannerConfig = {
        "Home": { "delay": 100 },
        "Game": { "delay": 100 },
        "Sign": { "delay": 100 },
        "Trial": { "delay": 100 },
        "Failed": { "delay": 100 },
        "Success": { "delay": 100 },
        "SecretBox": { "delay": 100 },
        "SecretAward": { "delay": 100 },
        "Turntable": { "delay": 100 }
    };
    var additiveUIConfig = {
        "Home": [
            {
                "viewUrl": "fc-layout/prefab/additive/MoreRecommendBtn.json",
                "parentRoot": "middleUI",
                "x": 42,
                "y": 331,
                "scriptType": "fc.view.MoreGameBoxBtn",
                "checkCode": 5
            },
            {
                "viewUrl": "fc-layout/prefab/additive/GoldShowAdd.json",
                "parentRoot": "topUI",
                "x": 32,
                "y": 160,
                "scriptType": "fc.view.GoldShowAdd"
            },
            {
                "viewUrl": "fc-layout/prefab/additive/StrengthShowAdd.json",
                "parentRoot": "topUI",
                "x": 310,
                "y": 160,
                "scriptType": "fc.view.StrengthShowAdd",
                "checkCode": 2
            },
            {
                "viewUrl": "fc-layout/prefab/additive/StartGame.json",
                "parentRoot": "bottomUI",
                "x": 375,
                "y": 359,
                "scriptType": "fc.view.AniLoopPlay"
            }
        ],
        "Game": [
            {
                "viewUrl": "fc-layout/prefab/additive/ResetBtn.json",
                "parentRoot": "topUI",
                "x": 30,
                "y": 420,
                "scriptType": "fc.view.ResetBtn",
                "checkCode": 3
            },
            {
                "viewUrl": "fc-layout/prefab/additive/GoldShow.json",
                "parentRoot": "topUI",
                "x": 32,
                "y": 160,
                "scriptType": "fc.view.GoldShow"
            },
            {
                "viewUrl": "fc-layout/prefab/additive/StrengthShow.json",
                "parentRoot": "topUI",
                "x": 310,
                "y": 160,
                "scriptType": "fc.view.StrengthShow",
                "checkCode": 2
            }
        ],
        "Failed": [
            {
                "viewUrl": "fc-layout/prefab/additive/FailEmoji.json",
                "parentRoot": "topUI",
                "x": 375,
                "y": 500,
                "scriptType": "fc.view.AniLoopPlay"
            }
        ],
        "Success": [
            {
                "viewUrl": "fc-layout/prefab/additive/SuccessEmoji.json",
                "parentRoot": "topUI",
                "x": 375,
                "y": 550,
                "scriptType": "fc.view.AniLoopPlay"
            }
        ]
    };
    var playerCfg = {
        "userId": "1",
        "isNew": 1,
        "isInGuide": 1,
        "signedDays": 0,
        "signTime": 0,
        "levelId": 1,
        "gold": 0,
        "strength": 5,
        "sound": 1,
        "goodsInUse": [0],
        "goodsInBag": [[0]]
    };

    function isAdditiveUIVisible(code) {

        if (!code) {
            return true;
        }

        if (code == 2) {
            return ActivityCenter.isStrengthOpened();
        }

        if (code == 3) {
            return ActivityCenter.isResetGameOpened();
        }

        if (code == 5) {
            return isEnableMoreGameBtn();
        }

        return false;
    }

    function isViewValid(code) {
        if (!code) {
            return true;
        }

        if (code == 1) {
            return ActivityCenter.isEnablePopSign();
        }

        if (code == 2) {
            return ActivityCenter.isEnablePopStore();
        }

        if (code == 3) {
            return ActivityCenter.isEnablePopInsertAd();
        }
        
        if (code == 4) {
            return ActivityCenter.isEnablePopTrial();
        }
        
        if (code == 5) {
            return isEnablePopCrazyKnock();
        }
        if (code == 6) {
            return isMistakenActive();
        }
        if (code == 7) {
            return ActivityCenter.isEnablePopNativeAd();
        }

        if (code == 104) {
            return ActivityCenter.isEnablePopVip();
        }

        return false;
    }

    function isEnablePopCrazyKnock() {
        if (!isMistakenActive()) {
            return false;
        }
        var playerInfo = GameFrame.playerInfo;
        let clickNum = GameFrame.pluginCfg.pg_fastclick_num;

        if (fc.framework.MiscUtils.isNumber(clickNum)) {

            //如果是-1则是无限制
            if (clickNum == -1) return true;

            if (Number(clickNum) > Number(ActivityCenter.todayKnockTime)) return true;
        }

        if (clickNum && clickNum.length > 0) {
            if (clickNum.length == 1 && clickNum[0] == -1)
                return true;

            var index = clickNum.indexOf(playerInfo.levelId);
            if (index != -1) {
                return true;
            }
        }

        return false;
    }
    function isMistakenActive() {
        var pluginCfg = PlatformConfig.adZsCfg;
        return pluginCfg &&
            GameFrame.appInfo.version != pluginCfg.zs_version && PlatformConfig.adZsCfg.zs_switch == 1;
    }

    function isEnableMoreGameBtn() {
        var pluginCfg = PlatformConfig.adZsCfg;
        return isMistakenActive() && pluginCfg && PlatformConfig.adZsCfg.zs_box_switch == 1;
    }


    class PlatformConfig extends Laya.Script {

        static get playerInfo() {
            return GameFrame.playerInfo;
        }

        static set playerInfo(val) {
            if (val) {
                var keys = Object.keys(val);
                for (var index = 0; index < keys.length; index++) {
                    GameFrame.playerInfo[keys[index]] = val[keys[index]];
                }
                if (keys.length > 0) {
                    GameFrame.savePlayerInfo(val);
                }
            }
        }
        static init(appCfgUrl) {
            PlatformConfig.appCfgUrl = appCfgUrl;
            Laya.loader.load(this.appCfgUrl ? this.appCfgUrl : `config/${this.platformTag}/appConfig.${this.platformTag}.json`, Laya.Handler.create(this, this.onAppInfoReady));
        }

        static onAppInfoReady(appInfo) {
            GameFrame.appInfo = appInfo.appInfo;
            zs.sdk.initSdkByConf({ channel: "qq", appId: GameFrame.appInfo.appId, secret: GameFrame.appInfo.zsSecret });
            Channel.login();//指色的登陆
            Channel.loadCfg(this.loadCfgSuccess);//指色的配置
        }

        static loadCfgSuccess(cfg) {
            if (cfg) {
                PlatformConfig.adZsCfg = cfg;

                PlatformConfig.adZsCfg.zs_ready_click_two = Number(PlatformConfig.adZsCfg.zs_ready_click_two || 0);

                Laya.stage.addComponent(PlatformConfig);
                Laya.stage.addComponent(BannerCtrl);
                Laya.stage.addComponent(BlockAdCtrl);
                Laya.stage.addComponent(AdditiveUICtrl);
                Laya.stage.addComponent(ActivityCenter);
                GameFrame.startUp(PlatformConfig.appCfgUrl ? PlatformConfig.appCfgUrl : `config/${PlatformConfig.platformTag}/appConfig.${PlatformConfig.platformTag}.json`, PlatformConfig.platformTag, viewCfg, sceneCfg, playerCfg);
            }
        }

        onAwake() {
            GameFrame.setViewValidCheckFun(isViewValid);
            BannerCtrl.bannerConfig = bannerConfig;
            BlockAdCtrl.blockConfig = blockConfig;
            AdditiveUICtrl.additiveUIConfig = additiveUIConfig;
            AdditiveUICtrl.checkFun = isAdditiveUIVisible;
            Laya.stage.once(Event.LAUNCH_COMPLETED, this, this.onLaunchResReady);
            Laya.stage.on(Event.UI_SCENE_LOADED, this, this.onSceneLoaded);
            Laya.stage.on(Event.UI_SCENE_UNLOADED, this, this.onSceneUnloaded);
            Laya.stage.on(Event.UI_VIEW_CLOSED, this, this.onViewClosed);
        }

        onDestroy() {
            Laya.stage.off(Event.UI_SCENE_LOADED, this, this.onSceneLoaded);
            Laya.stage.off(Event.UI_SCENE_UNLOADED, this, this.onSceneUnloaded);
            Laya.stage.off(Event.UI_VIEW_CLOSED, this, this.onViewClosed);
        }

        onLaunchResReady(scene3d) {
            var pluginCfg = GameFrame.pluginCfg;
            if (pluginCfg) {
                GameFrame.pluginCfg.pg_video_style = pluginCfg.pg_video_style || [0, 1, 2, 3, 4, 5];

                // pluginCfg.pg_banner_adunit = ['4f93ec28ea2e19f0ea053bfefb3f72b0','d5e7fae70405768eecad8c29f141e12b','d7a55b113e514f86b29f86f27c0bc244'];
                // if (PlatformConfig.adZsCfg.zs_banner_adunit) {
                //     pluginCfg.pg_banner_adunit.push(PlatformConfig.adZsCfg.zs_banner_adunit);
                // }
                // if (PlatformConfig.adZsCfg.zs_banner_adunit2) {
                //     pluginCfg.pg_banner_adunit.push(PlatformConfig.adZsCfg.zs_banner_adunit2);
                // }
                // if (PlatformConfig.adZsCfg.zs_banner_adunit3) {
                //     pluginCfg.pg_banner_adunit.push(PlatformConfig.adZsCfg.zs_banner_adunit3);
                // }

                // pluginCfg.pg_video_adunit = 'e628b2d74fcbe488bfcfdcb5b3bdfd18';//PlatformConfig.adZsCfg.zs_video_adunit;
                // pluginCfg.pg_full_screen_adunit = 'e2183498c3871db200a6582766653861';//PlatformConfig.adZsCfg.zs_full_screen_adunit;

                // pluginCfg.pg_game_box_adunit = '74675f54311a6d8e3392951b04c61b31';//PlatformConfig.adZsCfg.zs_box_adunit;
                // pluginCfg.pg_block_adunit = 'ba62b1ad3da1c839e523b24c53946a7a';//PlatformConfig.adZsCfg.zs_blockAdUnitId;

                pluginCfg.pg_banner_refresh_time = PlatformConfig.adZsCfg.zs_banner_refresh_time;
                pluginCfg.pg_lucky_reward_switch = Number(PlatformConfig.adZsCfg.zs_luckybox_switch);
                pluginCfg.zs_box_switch = Number(PlatformConfig.adZsCfg.zs_box_switch);

                pluginCfg.pg_switch = PlatformConfig.adZsCfg.zs_switch;
                pluginCfg.pg_version = PlatformConfig.adZsCfg.zs_version;
                pluginCfg.pg_button_show_style = 1;
                pluginCfg.pg_text_move_time = PlatformConfig.adZsCfg.zs_banner_vertical_enable == 1 ? 2000 : 0;
                pluginCfg.pg_button_delay_time = PlatformConfig.adZsCfg.zs_button_delay_time ? Number(PlatformConfig.adZsCfg.zs_button_delay_time) : 2000;

                pluginCfg.pg_fastclick_num = PlatformConfig.adZsCfg.zs_click_award_num;
                // pluginCfg.pg_fastclick_progress_add = PlatformConfig.adZsCfg.zs_click_award_add ;
                // pluginCfg.pg_fastclick_progress_back = PlatformConfig.adZsCfg.zs_click_award_back;
                // pluginCfg.pg_fastclick_ad_percent = PlatformConfig.adZsCfg.zs_click_award_percent;
                pluginCfg.pg_click_video_switch = Number(PlatformConfig.adZsCfg.zs_click_video_switch || 0);




                PlatformConfig.adZsCfg.zs_banner_paly_time = 15;
                PlatformConfig.adZsCfg.zs_banner_time = 5;

                zs.laya.banner.WxBannerMgr.Instance.setAdUnitId(pluginCfg.pg_banner_adunit);
            }
        }

        onSceneUnloaded(sceneId) {
            if (sceneId == SceneId.Game) {
                Audio.stopMusic();
            }
        }

        onSceneLoaded(sceneId) {
            if (sceneId == SceneId.Game) {
                if (GameFrame.appInfo.bgm) {
                    Audio.playMusic(GameFrame.appInfo.bgm);
                }
            }
        }

        onViewClosed(viewName, viewId) {
            if (viewId == "Failed" || viewId == "Success") {
                if (isEnableMoreGameBtn()) {
                    Channel.showGameBox(null, null, null);
                }
            }
        }
    }
    PlatformConfig.platformTag = "zsqq";
    PlatformConfig.appCfgUrl = "";
    PlatformConfig.adZsCfg = {};//指色的配置

    exports.PlatformConfig = PlatformConfig;
    exports.isEnablePopCrazyKnock = isEnablePopCrazyKnock;



}(window.fc.platform, Laya));