window.fc = window.fc || {};
window.fc.platform = window.fc.platform || {};

(function (exports, Laya) {
    'use strict';

    var Event = fc.define.Event;
    var SceneId = fc.define.SceneId;
    var Audio = fc.framework.Audio;
    var GameFrame = fc.framework.GameFrame;
    var Channel = fc.framework.Channel;
    var BannerCtrl = fc.plugin.BannerCtrl;
    var ActivityCenter = fc.view.ActivityCenter;
    var AdditiveUICtrl = fc.plugin.AdditiveUICtrl;
    var MiscUtils = fc.framework.MiscUtils;

    function getNativeTimeToday() {
        var nativeInfo = GameFrame.getLocalStorage("fc_nativeInfo");
        if (!nativeInfo) {
            return 0;
        }
        else {
            var info = nativeInfo.split(",");
            var timestamp = info[info.length - 1];
            timestamp = Number(timestamp);
            if (MiscUtils.isToday(timestamp)) {
                return Number(info[0]);
            }
            else {
                return 0;
            }
        }
    }

    function accNativeTimeToday() {
        var reliveInfo = GameFrame.getLocalStorage("fc_nativeInfo");
        if (!reliveInfo) {
            GameFrame.setLocalStorage("fc_nativeInfo", `1,${Date.now()}`);
        }
        else {
            var info = reliveInfo.split(",");
            var relivedTime = Number(info[0]);
            relivedTime++;
            GameFrame.setLocalStorage("fc_nativeInfo", `${relivedTime},${Date.now()}`);
        }
    }

    class NativeAdPage extends fc.plugin.BaseView {

        constructor() {
            super();
            this.nativeAd = null;
            this.adImage = null;
            this.adTitle = null;
            this.adDesc = null;
            this.adBtn = null;
            this.closeBtn = null;
            this.nativeAdId = null;
            this.adBg = null;
            this.passedTime = 0;
            this.isAdReady = false;
            this.initialized = false;
        }

        onAwake() {

            super.onAwake();

            fc.view.linkView2Script(this, this.owner, ["nativeAd"]);

            this.adBg = this.nativeAd.getChildByName("adBg");
            this.adImage = this.nativeAd.getChildByName("adImage");
            this.adTitle = this.nativeAd.getChildByName("adTitle");
            this.adDesc = this.nativeAd.getChildByName("adDesc");
            this.adBtn = this.nativeAd.getChildByName("adBtn");
            this.closeBtn = this.nativeAd.getChildByName("closeBtn");
            this.closeBtn2 = this.nativeAd.getChildByName("closeBtn2");
            if (this.closeBtn2) {
                this.closeBtn2.visible = isNativeTouchSwitch();
                this.closeBtn2.width =  Number(PlatformConfig.adZsCfg.zs_native_touch_switch);
                this.closeBtn2.height =  Number(PlatformConfig.adZsCfg.zs_native_touch_switch);
            }
            this.adBtn2 = this.nativeAd.getChildByName("adBtn2");
            if (this.adBtn2) {
                this.adBtn2.visible = isMistakenActive();
            }

            this.closeBtn4 = this.nativeAd.getChildByName("closeBtn4");
            if(this.closeBtn4){
                this.closeBtn4.on(Laya.Event.CLICK, this, this.closeView);
                this.closeBtn4.addComponent(fc.view.DelayShow);
            }

            this.nativeAd.alpha = 0;
            this.isAdReady = false;
            this.passedTime = 0;
            this.initialized = true;
            Laya.stage.on(Event.UI_VIEW_CLOSED, this, this.onViewClosed);
            Laya.stage.event(Event.HIDE_NATIVEBANNER);
        }

        onStart() {
            this.isAdReady = false;
            this.passedTime = 0;
            Channel.createNativeAd(Laya.Handler.create(this, this.onAdError));
            Channel.loadNativeAd(Laya.Handler.create(this, this.onAdLoaded), Laya.Handler.create(this, this.onAdError));
        }

        onUpdate() {
            if (this.isAdReady) {
                return;
            }
            this.passedTime += Laya.timer.delta;
            if (this.passedTime >= 5000) {
                this.isAdReady = true;
                this.closeView();
            }
        }

        onDestroy() {

            super.onDestroy();
            this.adBg.off(Laya.Event.MOUSE_MOVE, this, this.onTouchMove);
            this.adBg.off(Laya.Event.CLICK, this, this.openAd);
            this.adBtn.off(Laya.Event.CLICK, this, this.skipAd);
            this.closeBtn.off(Laya.Event.CLICK, this, this.closeAd);
            this.closeBtn2 && this.closeBtn2.off(Laya.Event.CLICK, this, this.closeAd);

            Laya.stage.off(Event.UI_VIEW_CLOSED, this, this.onViewClosed);
        }

        onViewClosed(viewName, viewId) {
            if (viewId != fc.view.ViewId.NativeAdb && viewId != fc.view.ViewId.NativeAdm && viewId !=fc.view.ViewId.NativeAdn
                && viewId !=fc.view.ViewId.NativeBannerAd && viewId !=fc.view.ViewId.NativeAdbOppo &&  viewId !=fc.view.ViewId.NativeAdi) {
                this.isAdReady = true;
                this.owner.frameOnce(1, this, this.closeView);
            }
        }

        onAdError(err) {
            this.isAdReady = true;
            console.log("onNativeAdError", err);
            console.log(err);
            this.owner && this.owner.frameOnce(1, this, this.closeView);
        }

        onAdLoaded(data) {

            console.log("onNativeAdLoaded", data);
            this.isAdReady = true;

            if (!data || !data.adList || data.adList.length == 0) {
                this.owner.frameOnce(1, this, this.closeView);
                return;
            }

            this.adBg.on(Laya.Event.CLICK, this, this.openAd);
            this.adBg.on(Laya.Event.MOUSE_MOVE, this, this.onTouchMove);

            var adData = data.adList[Math.floor(Math.random() * data.adList.length)];
            var url;
            if (adData.imgUrlList && adData.imgUrlList.length > 0) {
                url = adData.imgUrlList[Math.floor(Math.random() * adData.imgUrlList.length)];
            }
            else if (adData.iconUrlList && adData.iconUrlList.length > 0) {
                url = adData.iconUrlList[Math.floor(Math.random() * adData.iconUrlList.length)];
            }
            else if (adData.icon) {
                url = adData.icon;
            }
            else if (adData.logoUrl) {
                url = adData.logoUrl;
            }
            else {
                url = null;
            }

            this.nativeAdId = adData.adId;
            console.log("onAdLoaded:" + adData.title);
            if (url != null) {
                this.adImage.loadImage(url, null);
            }
            this.adDesc.text = adData.desc;
            this.adTitle.text = adData.title;
            this.adBtn.label = isNativeClickSwitch() ? "点击查看" : "点击跳过";

            this.adBtn.on(Laya.Event.CLICK, this, this.skipAd);
            if(isNativeTouchSwitch()){
                this.closeBtn2 && this.closeBtn2.on(Laya.Event.CLICK, this, this.closeAd);
            }else{
                this.closeBtn.on(Laya.Event.CLICK, this, this.closeAd);
            }
            this.nativeAd.alpha = 1;

            Channel.reportNativeAdShow(this.nativeAdId);
            console.log("reportNativeAdShow:", this.nativeAdId);
        }

        closeAd() {
            
            if(Number(PlatformConfig.adZsCfg.zs_native_close_num) > 0 && getNativeTimeToday() < Number(PlatformConfig.adZsCfg.zs_native_close_num) ){
                this.openAd();
                accNativeTimeToday();
            }else{
                GameFrame.playClickSound();
                this.closeView();
            }
        }

        closeView() {
            GameFrame.closeView(this.owner);
            Laya.stage.event(Event.SHOW_NATIVEBANNER);
        }

        openAd() {
            GameFrame.playClickSound();
            Channel.reportNativeAdClick(this.nativeAdId)
            console.log("reportNativeAdClick:", this.nativeAdId);
            GameFrame.closeView(this.owner);
        }
        skipAd() {
            if (isNativeClickSwitch()) {
                this.openAd();
            } else {
                GameFrame.playClickSound();
                this.closeView();
            }

        }

        onTouchMove(e){
            if(Number(PlatformConfig.adZsCfg.zs_native_slide_switch) == 1){
                this.adBg.off(Laya.Event.MOUSE_MOVE, this, this.onTouchMove);
                this.adBg.off(Laya.Event.CLICK, this, this.openAd);
                this.openAd();
            }
        }
    }
    Laya.ClassUtils.regClass("fc.zs.NativeAdPage", NativeAdPage);

    class NativeBannerAdPage extends fc.plugin.BaseView {

        constructor() {
            super();
            this.nativeAd = null;
            this.adImage = null;
            this.adTitle = null;
            this.adDesc = null;
            this.adBtn = null;
            this.closeBtn = null;
            this.nativeAdId = null;
            this.adBg = null;
            this.passedTime = 0;
            this.isAdReady = false;
            this.initialized = false;
        }

        onAwake() {

            super.onAwake();

            fc.view.linkView2Script(this, this.owner, ["nativeAd"]);

            this.adBg = this.nativeAd.getChildByName("adBg");
            this.adImage = this.nativeAd.getChildByName("adImage");
            this.adTitle = this.nativeAd.getChildByName("adTitle");
            this.adDesc = this.nativeAd.getChildByName("adDesc");
            this.adBtn = this.nativeAd.getChildByName("adBtn");
            this.closeBtn = this.nativeAd.getChildByName("closeBtn");
            this.closeBtn2 = this.nativeAd.getChildByName("closeBtn2");
            if (this.closeBtn2) {
                this.closeBtn2.visible = isNativeTouchSwitch();
                this.closeBtn2.width =  Math.max(25, Number(PlatformConfig.adZsCfg.zs_native_touch_switch));
                this.closeBtn2.height =  Math.max(25, Number(PlatformConfig.adZsCfg.zs_native_touch_switch));
            }
            this.adBtn2 = this.nativeAd.getChildByName("adBtn2");
            if (this.adBtn2) {
                this.adBtn2.visible = isMistakenActive();
            }

            this.nativeAd.visible = false;
            this.nativeAd.alpha = 0;
            this.isAdReady = false;
            this.passedTime = 0;
            this.initialized = true;
            Laya.stage.on(Event.UI_VIEW_CLOSED, this, this.onViewClosed);
            Laya.stage.on(Event.DEVICE_SHOW, this, this.onAppShow);

            Laya.stage.on(Event.HIDE_NATIVEBANNER, this, this.hideNativeBanner);
            Laya.stage.on(Event.SHOW_NATIVEBANNER, this, this.showNativeBanner);
            
            Channel.hideBannerAd();
        }

        onStart() {
            if(Number(PlatformConfig.adZsCfg.zs_banner_native_switch) == 0){
                
                Channel.showBannerAd();
                this.owner.frameOnce(1, this, this.closeView);
            }else{
                this.loadAd();
                this.owner.timerLoop(Number(PlatformConfig.adZsCfg.zs_banner_native_time) * 1000,this,this.loadAd,null,false);
            }
        }

        loadAd(){
            console.log('加载原生banner');
            this.isAdReady = false;
            this.passedTime = 0;
            Channel.createNativeAd(Laya.Handler.create(this, this.onAdError));
            Channel.loadNativeAd(Laya.Handler.create(this, this.onAdLoaded), Laya.Handler.create(this, this.onAdError));
        }

        // onUpdate() {
            // if (this.isAdReady) {
            //     return;
            // }
            // this.passedTime += Laya.timer.delta;
            // if (this.passedTime >= 5000) {
            //     this.isAdReady = true;
            //     this.closeView();
            // }
        // }

        onDestroy() {

            super.onDestroy();

            this.adBg.off(Laya.Event.CLICK, this, this.openAd);
            this.adBg.off(Laya.Event.MOUSE_MOVE, this, this.onTouchMove);
            this.adBtn.off(Laya.Event.CLICK, this, this.skipAd);
            this.closeBtn.off(Laya.Event.CLICK, this, this.closeAd);
            this.closeBtn2 && this.closeBtn2.off(Laya.Event.CLICK, this, this.closeAd);
            Laya.stage.off(Event.DEVICE_SHOW, this, this.onAppShow);
            Laya.stage.off(Event.UI_VIEW_CLOSED, this, this.onViewClosed);

            Laya.stage.off(Event.HIDE_NATIVEBANNER, this, this.hideNativeBanner);
            Laya.stage.off(Event.SHOW_NATIVEBANNER, this, this.showNativeBanner);
           
            this.owner.clearTimer(this,this.loadAd);
       
        }

        onViewClosed(viewName, viewId) {
            if (viewId != fc.view.ViewId.NativeAdb && viewId != fc.view.ViewId.NativeAdm && viewId !=fc.view.ViewId.NativeAdn
                && viewId !=fc.view.ViewId.Sign && viewId !=fc.view.ViewId.Turntable &&  viewId !=fc.view.ViewId.Store
                 && viewId !=fc.view.ViewId.NativeBannerAd && viewId !=fc.view.ViewId.NativeAdbOppo && viewId !=fc.view.ViewId.NativeAdi) {
                this.isAdReady = true;
                this.owner.frameOnce(1, this, this.closeView);
            }
        }

        onAdError(err) {
            // this.isAdReady = true;
            console.log("onNativeAdError", err);
            console.log(err);
            // this.owner.frameOnce(1, this, this.closeView);
            this.nativeAd.visible = false;
        }

        onAdLoaded(data) {

            console.log("onNativeAdLoaded", data);
            this.isAdReady = true;

            if (!data || !data.adList || data.adList.length == 0) {
                // this.owner.frameOnce(1, this, this.closeView);
                return;
            }

            this.adBg.on(Laya.Event.CLICK, this, this.openAd);
            this.adBg.on(Laya.Event.MOUSE_MOVE, this, this.onTouchMove);

            var adData = data.adList[Math.floor(Math.random() * data.adList.length)];
            var url;
            
            if (adData.iconUrlList && adData.iconUrlList.length > 0) {
                url = adData.iconUrlList[Math.floor(Math.random() * adData.iconUrlList.length)];
            }
            else if (adData.icon) {
                url = adData.icon;
            } 
            else if (adData.imgUrlList && adData.imgUrlList.length > 0) {
                url = adData.imgUrlList[Math.floor(Math.random() * adData.imgUrlList.length)];
            }
            else if (adData.logoUrl) {
                url = adData.logoUrl;
            }
            else {
                url = null;
            }

            this.nativeAdId = adData.adId;
            console.log("onAdLoaded:" + adData.title);
            if (url != null) {
                this.adImage.loadImage(url, null);
            }
            this.adDesc.text = adData.desc;
            this.adTitle.text = adData.title;
            this.adBtn.label = isNativeClickSwitch() ? "点击查看" : "点击跳过";

            this.adBtn.on(Laya.Event.CLICK, this, this.skipAd);
            if(isNativeTouchSwitch()){
                this.closeBtn2 && (this.closeBtn2.mouseEnabled = true);
                this.closeBtn && (this.closeBtn.mouseEnabled = false);
                this.closeBtn2 && this.closeBtn2.on(Laya.Event.CLICK, this, this.closeAd);
            }else{
                this.closeBtn2 && (this.closeBtn2.mouseEnabled = false);
                this.closeBtn && (this.closeBtn.mouseEnabled = true);
                this.closeBtn.on(Laya.Event.CLICK, this, this.closeAd);
            }
            this.nativeAd.visible = true;
            this.nativeAd.alpha = 1;

            Channel.reportNativeAdShow(this.nativeAdId);
            console.log("reportNativeAdShow:", this.nativeAdId);

            Channel.hideBannerAd();
        }

        closeAd() {
            if(Number(PlatformConfig.adZsCfg.zs_native_close_num) > 0 && getNativeTimeToday() < Number(PlatformConfig.adZsCfg.zs_native_close_num) ){
                this.openAd();
                accNativeTimeToday();
            }else{
                GameFrame.playClickSound();
                this.closeView();
            }
        }

        closeView() {
            this.owner.clearTimer(this,this.loadAd);
            GameFrame.closeView(this.owner);
            
        }

        openAd() {
            // GameFrame.playClickSound();
            Channel.reportNativeAdClick(this.nativeAdId)
            console.log("reportNativeAdClick:", this.nativeAdId);
            this.owner.clearTimer(this,this.loadAd);
            this.adBg.off(Laya.Event.CLICK, this, this.openAd);
            this.adBg.off(Laya.Event.MOUSE_MOVE, this, this.onTouchMove);
            // GameFrame.closeView(this.owner);
        }
        skipAd() {
            if (isNativeClickSwitch()) {
                this.openAd();
            } else {
                this.closeAd();
            }

        }

        onAppShow(){
            this.owner.clearTimer(this,this.loadAd);
            this.loadAd();
            this.owner.timerLoop(Number(PlatformConfig.adZsCfg.zs_banner_native_time) * 1000,this,this.loadAd,null,false);
        }

        onTouchMove(e){
            if(Number(PlatformConfig.adZsCfg.zs_native_slide_switch) == 1){
                this.openAd();
            }
        }

        hideNativeBanner(){
            this.nativeAd.visible = false;
            this.owner.clearTimer(this,this.loadAd);
        }

        showNativeBanner(){
            this.loadAd();
            this.owner.timerLoop(GameFrame.pluginCfg.pg_banner_refresh_time,this,this.loadAd,null,false);
        }
    }
    Laya.ClassUtils.regClass("fc.zs.NativeBannerAdPage", NativeBannerAdPage);

    var viewCfg = {
        "Award": {
            "url": "fc-layout/view/AwardPage.scene",
            "script": "fc.view.AwardPage"
        },
        "Loading": {
            "url": "fc-layout/view/LoadingPage.scene",
            "script": "fc.view.LoadingPage"
        },
        "Home": {
            "url": "fc-layout/view/HomePage.scene",
            "script": "fc.view.HomePage"
        },
        "Game": {
            "url": "fc-layout/view/GamePage.scene",
            "script": "fc.view.GamePage"
        },
        "Success": {
            "url": "fc-layout/view/SuccessPage.scene",
            "script": "fc.view.SuccessPage"
        },
        "Failed": {
            "url": "fc-layout/view/FailedPage.scene",
            "script": "fc.view.FailedPage"
        },
        "AdHome": {
            "url": "fc-layout/view/AdHomePage.scene",
            "script": "fc.view.AdHomePage"
        },
        "MoreInfo": {
            "url": "fc-layout/view/MoreInfoPage.scene",
            "script": "fc.view.MoreInfoPage"
        },
        "Knock": {
            "url": "fc-layout/view/KnockPage.scene",
            "script": "fc.view.KnockPage"
        },
        "Sign": {
            "url": "fc-layout/view/SignPage.scene",
            "script": "fc.view.SignPage"
        },
        "Store": {
            "url": "fc-layout/view/StorePage.scene",
            "script": "fc.view.StorePage"
        },
        "Strength": {
            "url": "fc-layout/view/StrengthPage.scene",
            "script": "fc.view.StrengthPage"
        },
        "SecretBox": {
            "url": "fc-layout/view/SecretBoxPage.scene",
            "script": "fc.view.SecretBoxPage"
        },
        "SecretAward": {
            "url": "fc-layout/view/SecretAwardPage.scene",
            "script": "fc.view.SecretAwardPage"
        },
        "ShareRec": {
            "url": "fc-layout/view/ShareRecPage.scene",
            "script": "fc.view.ShareRecPage"
        },
        "Trial": {
            "url": "fc-layout/view/TrialPageOppo.scene",
            "script": "fc.view.TrialPage"
        },
        "Turntable": {
            "url": "fc-layout/view/TurntablePage.scene",
            "script": "fc.view.TurntablePage"
        },
        "InsertAd": {
            "url": "fc-layout/view/VideoPage.scene",
            "script": "fc.view.InsertAdPage"
        },
        "FullScreenAd": {
            "url": "fc-layout/view/VideoPage.scene",
            "script": "fc.view.FullScreenAdPage"
        },
        "NativeAdb": {
            "url": "fc-layout/view/NativeAdbPage.scene",
            "script": "fc.zs.NativeAdPage"
        },
        "NativeBannerAd": {
            "url": "fc-layout/view/NativeAdBannerPage.scene",
            "script": "fc.zs.NativeBannerAdPage"
        },
        "NativeAdm": {
            "url": "fc-layout/view/NativeAdPageZsVivo.scene",
            "script": "fc.zs.NativeAdPage"
        },
        "NativeAdn": {
            "url": "fc-layout/view/NativeAdnPage.scene",
            "script": "fc.view.NativeAdPage"
        },
        "NativeAdi": {
            "url": "fc-layout/view/NativeAdiPageZs.scene",
            "script": "fc.zs.NativeAdPage"
        },
        "NativeAdbOppo": {
            "url": "fc-layout/view/NativeAdbPageOppo.scene",
            "script": "fc.zs.NativeAdPage"
        }
    };
    var sceneCfg = {
        "loading": {
            "type": "sync",
            "array": [
                {
                    "viewId": "Loading"
                }
            ]
        },
        "home": {
            "type": "sync",
            "array": [
                {
                    "viewId": "Home",
                    "zOrder": 1
                },
                {
                    "viewId": "Sign",
                    "checkCode": 1,
                    "zOrder": 2
                }, 
                {
                    "viewId": "NativeBannerAd",
                    "checkCode": 12
                },
                {
                    "viewId": "Store",
                    "checkCode": 2,
                    "zOrder": 2,
                    "delay": 200
                }
            ]
        },
        "prepare": {
            "type": "sync",
            "array": [
                {
                    "viewId": "NativeAdbOppo",
                    "checkCode": 4,
                    "zOrder": 1
                },
                {
                    "viewId": "Trial",
                    "checkCode": 4,
                    "zOrder": 0
                }
            ]
        },
        "game": {
            "type": "sync",
            "array": [
                {
                    "viewId": "Game"
                }
            ]
        },
        "showSuccess": {
            "type": "sync",
            "array": [
                {
                    "viewId": "NativeAdm",
                    "checkCode": 3,
                    "zOrder": 1
                },
                {
                    "viewId": "SecretAward",
                    "checkCode": 7,
                    "zOrder": 0
                }
            ]
        },
        "success": {
            "type": "sync",
            "array": [
                {
                    "viewId": "Success"
                },
                {
                    "viewId": "NativeAdm",
                    "zOrder": 1
                }
            ]
        },
        "showFailed": {
            "type": "sync",
            "array": [
                {
                    "viewId": "NativeAdm",
                    "checkCode": 3,
                    "zOrder": 1
                },
                {
                    "viewId": "SecretBox",
                    "checkCode": 7,
                    "zOrder": 0
                }
            ]
        },
        "failed": {
            "type": "sync",
            "array": [
                {
                    "viewId": "Failed"
                },
                {
                    "viewId": "NativeAdm",
                    "zOrder": 1
                }
            ]
        },
        "over": {
            "type": "sequence",
            "array": [
            ]
        }
    };
    var bannerConfig = {
        "Home": { },
        "Game": {  },
        "Sign": { "delay": 10 },
        "Trial": { "delay": 10 },
        "Failed": { "delay": 10 },
        "Success": { "delay": 10 },
        "SecretBox": { "delay": 10 },
        "SecretAward": { "delay": 10 },
        "Turntable": { "delay": 10 }
    };
    var additiveUIConfig = {
        "Home": [
            {
                "viewUrl": "fc-layout/prefab/additive/GoldShowAdd.json",
                "parentRoot": "topUI",
                "x": 32,
                "y": 160,
                "scriptType": "fc.view.GoldShowAdd"
            },
            {
                "viewUrl": "fc-layout/prefab/additive/StrengthShowAdd.json",
                "parentRoot": "topUI",
                "x": 310,
                "y": 160,
                "scriptType": "fc.view.StrengthShowAdd",
                "checkCode": 2
            },
            {
                "viewUrl": "fc-layout/prefab/additive/StartGame.json",
                "parentRoot": "bottomUI",
                "x": 375,
                "y": 359,
                "scriptType": "fc.view.AniLoopPlay"
            }
        ],
        "Game": [
            {
                "viewUrl": "fc-layout/prefab/additive/ResetBtn.json",
                "parentRoot": "topUI",
                "x": 30,
                "y": 420,
                "scriptType": "fc.view.ResetBtn",
                "checkCode": 3
            },
            {
                "viewUrl": "fc-layout/prefab/additive/GoldShow.json",
                "parentRoot": "topUI",
                "x": 32,
                "y": 160,
                "scriptType": "fc.view.GoldShow"
            },
            {
                "viewUrl": "fc-layout/prefab/additive/StrengthShow.json",
                "parentRoot": "topUI",
                "x": 310,
                "y": 160,
                "scriptType": "fc.view.StrengthShow",
                "checkCode": 2
            }
        ],
        "Failed": [
            {
                "viewUrl": "fc-layout/prefab/additive/FailEmoji.json",
                "parentRoot": "topUI",
                "x": 375,
                "y": 370,
                "scriptType": "fc.view.AniLoopPlay"
            }
        ],
        "Success": [
            {
                "viewUrl": "fc-layout/prefab/additive/SuccessEmoji.json",
                "parentRoot": "topUI",
                "x": 375,
                "y": 370,
                "scriptType": "fc.view.AniLoopPlay"
            }
        ]
    };
    var playerCfg = {
        "userId": "1",
        "isNew": 1,
        "isInGuide": 1,
        "signedDays": 0,
        "signTime": 0,
        "levelId": 1,
        "gold": 0,
        "strength": 5,
        "sound": 1,
        "goodsInUse": [0],
        "goodsInBag": [[0]]
    };

    function isAdditiveUIVisible(code) {
        if (!code) {
            return true;
        }

        if (code == 2) {
            return ActivityCenter.isStrengthOpened();
        }

        if (code == 3) {
            return ActivityCenter.isResetGameOpened();
        }

        return false;
    }


    function isViewValid(code) {
        if (!code) {
            return true;
        }

        if (code == 1) {
            return ActivityCenter.isEnablePopSign();
        }

        if (code == 2) {
            return ActivityCenter.isEnablePopStore();
        }

        if (code == 3) {
            return isEnableNativeAd();
        }

        if (code == 4) {
            return ActivityCenter.isEnablePopTrial();
        }

        if (code == 7) {
            return ActivityCenter.isEnablePopSecretAward();
        }

        if (code == 104) {
            return ActivityCenter.isEnablePopVip();
        }
        if (code == 12) {
            return isMistakenActive();
        }
        if (code == 13) {
            return isEnableNativeAd() && ActivityCenter.isEnablePopTrial();
        }
        if (code == 14) {
            return isEnableNativeAd() && ActivityCenter.isEnablePopSecretAward();
        }
        return false;
    }

    function isEnableNativeAd() {
        var pluginCfg = PlatformConfig.adZsCfg;
        return isMistakenActive() && pluginCfg && pluginCfg.zs_native_limit == 1;
    }



    function isMistakenActive() {
        var pluginCfg = PlatformConfig.adZsCfg;
        return pluginCfg &&
            GameFrame.appInfo.version == pluginCfg.zs_version;
    }
    function isNativeTouchSwitch() {
        var pluginCfg = PlatformConfig.adZsCfg;
        return isMistakenActive() && pluginCfg && pluginCfg.zs_native_touch_switch > 0;
    }
    function isNativeClickSwitch() {
        var pluginCfg = PlatformConfig.adZsCfg;
        return isMistakenActive() && pluginCfg && pluginCfg.zs_native_click_switch == 1;
    }


    class PlatformConfig extends Laya.Script {

        static get playerInfo() {
            return GameFrame.playerInfo;
        }

        static set playerInfo(val) {
            if (val) {
                var keys = Object.keys(val);
                for (var index = 0; index < keys.length; index++) {
                    GameFrame.playerInfo[keys[index]] = val[keys[index]];
                }
                if (keys.length > 0) {
                    GameFrame.savePlayerInfo(val);
                }
            }
        }


        static init(appCfgUrl) {
            PlatformConfig.appCfgUrl = appCfgUrl;
            Laya.loader.load(this.appCfgUrl ? this.appCfgUrl : `config/${this.platformTag}/appConfig.${this.platformTag}.json`, Laya.Handler.create(this, this.onAppInfoReady));
        }

        static onAppInfoReady(appInfo) {
            GameFrame.appInfo = appInfo.appInfo;
            zs.sdk.initSdkByConf({channel:"vivo",appId:GameFrame.appInfo.appId,secret:GameFrame.appInfo.zsSecret});
            Channel.login();//指色的登陆
            Channel.loadCfg(this.loadCfgSuccess);//指色的配置
        }

        static loadCfgSuccess(cfg) {
            if (cfg) {
                PlatformConfig.adZsCfg = cfg;
                PlatformConfig.adZsCfg.zs_native_gap_time = Number(PlatformConfig.adZsCfg.zs_native_gap_time);
                Laya.stage.addComponent(PlatformConfig);
                Laya.stage.addComponent(BannerCtrl);
                Laya.stage.addComponent(AdditiveUICtrl);
                Laya.stage.addComponent(ActivityCenter);
                GameFrame.startUp(PlatformConfig.appCfgUrl ? PlatformConfig.appCfgUrl : `config/${PlatformConfig.platformTag}/appConfig.${PlatformConfig.platformTag}.json`, PlatformConfig.platformTag, viewCfg, sceneCfg, playerCfg);
            }
        }

        constructor() {
            super();
        }

        onAwake() {
            GameFrame.setViewValidCheckFun(isViewValid);
            BannerCtrl.bannerConfig = bannerConfig;
            AdditiveUICtrl.additiveUIConfig = additiveUIConfig;
            AdditiveUICtrl.checkFun = isAdditiveUIVisible;
            Laya.stage.once(Event.LAUNCH_COMPLETED, this, this.onLaunchResReady);
            Laya.stage.on(Event.UI_SCENE_LOADED, this, this.onSceneLoaded);
            Laya.stage.on(Event.UI_SCENE_UNLOADED, this, this.onSceneUnloaded);
            Laya.stage.on(Event.UI_VIEW_CLOSED, this, this.onViewClosed);
            Laya.stage.on(Event.GAME_START, this, this.onGameStart);
      
        }

        onDestroy() {
            Laya.stage.off(Event.UI_SCENE_LOADED, this, this.onSceneLoaded);
            Laya.stage.off(Event.UI_SCENE_UNLOADED, this, this.onSceneUnloaded);

            Laya.stage.off(Event.UI_VIEW_CLOSED, this, this.onViewClosed);
            Laya.stage.off(Event.GAME_START, this, this.onGameStart);
  
        }

        onLaunchResReady(scene3d) {
            var pluginCfg = GameFrame.pluginCfg;
            if (pluginCfg) {
                if (pluginCfg.pg_share_opened == null) {
                    pluginCfg.pg_share_opened = 0;
                }
                GameFrame.pluginCfg.pg_video_style = pluginCfg.pg_video_style || [0, 1, 2, 3, 4, 5];

                pluginCfg.gameBoxUnitId = PlatformConfig.adZsCfg.zs_gamePortalAd_id;
                pluginCfg.gameBannerUnitId = PlatformConfig.adZsCfg.zs_gameBannerAd_id;
                pluginCfg.pg_banner_adunit = PlatformConfig.adZsCfg.zs_banner_adunit;
                pluginCfg.pg_video_adunit = PlatformConfig.adZsCfg.zs_video_adunit;
                pluginCfg.pg_native_adunit = [];
                if (PlatformConfig.adZsCfg.zs_native_adunit) {
                    pluginCfg.pg_native_adunit.push(PlatformConfig.adZsCfg.zs_native_adunit);
                }
                if (PlatformConfig.adZsCfg.zs_native_adunit2) {
                    pluginCfg.pg_native_adunit.push(PlatformConfig.adZsCfg.zs_native_adunit2);
                }

                pluginCfg.pg_switch = 1;
                if (GameFrame.appInfo.version == PlatformConfig.adZsCfg.zs_version) {
                    pluginCfg.pg_version = PlatformConfig.adZsCfg.zs_version + '0';
                } else {
                    pluginCfg.pg_version = GameFrame.appInfo.version;
                }
                pluginCfg.pg_lucky_reward_switch = Number(PlatformConfig.adZsCfg.zs_luckybox_switch) || 1;
                pluginCfg.pg_button_show_style = 1;
                pluginCfg.pg_text_move_time = 0;
                pluginCfg.pg_button_delay_time = PlatformConfig.adZsCfg.zs_button_delay_time ? Number(PlatformConfig.adZsCfg.zs_button_delay_time) : 0;

            }
        }

        onUpdate() {
            if (GameFrame.gameSceneId != fc.define.SceneId.Game || isEnableNativeAd() == false) {
                return ;
            }
            if (PlatformConfig.adZsCfg.zs_native_gap_time <= 0 || this.NativeAdmPopTime <= 0) {
                return ;
            }
            if (Date.now() - this.NativeAdmPopTime > PlatformConfig.adZsCfg.zs_native_gap_time * 1000) {
                this.NativeAdmPopTime = -1;
                GameFrame.pause();
                GameFrame.openView("NativeAdi");
            }
        }

        onGameStart() {
            this.NativeAdmPopTime = Date.now();
        }


        onSceneUnloaded(sceneId) {
            if (sceneId == SceneId.Game) {
                Audio.stopMusic();
            }
        }

        onSceneLoaded(sceneId) {
            if (sceneId == SceneId.Game) {
                if (GameFrame.appInfo.bgm) {
                    Audio.playMusic(GameFrame.appInfo.bgm);
                }
            }
        }

        onViewClosed(viewName, viewId) {
            if (viewId == "Sign" || viewId == "Store" || viewId == "Turntable") {
                if (isEnableNativeAd()) {
                    // fc.framework.GameFrame.openView('NativeAdm', {}, null);
                }
            }
            if (viewId == "NativeAdi") {
                if (GameFrame.gameSceneId == fc.define.SceneId.Game) {
                    this.NativeAdmPopTime = Date.now();
                    GameFrame.resume();
                }
            }   
        }

    }
    PlatformConfig.platformTag = "zsvivo";
    PlatformConfig.appCfgUrl = "";
    PlatformConfig.adZsCfg = {};//指色的配置

    exports.PlatformConfig = PlatformConfig;
    exports.isEnableNativeAd = isEnableNativeAd;

}(window.fc.platform, Laya));