(function () {
  'use strict';

  var Vector3 = Laya.Vector3;
  class Util {
      static getTransformUp(transform, out) {
          out = out ? out : new Laya.Vector3();
          transform.getUp(out);
          Vector3.normalize(out, out);
          return out;
      }
      static getTransformForward(transform, out) {
          out = out ? out : new Laya.Vector3();
          transform.getForward(out);
          Vector3.normalize(out, out);
          return out;
      }
      static getTransformRight(transform, out) {
          out = out ? out : new Laya.Vector3();
          transform.getRight(out);
          Vector3.normalize(out, out);
          return out;
      }
      static vector3Scale(vector3, scale, out) {
          out = out ? out : new Vector3();
          Vector3.scale(vector3, scale, out);
          return out;
      }
      static vector3Subtract(a, b, out) {
          out = out ? out : new Vector3();
          Vector3.subtract(a, b, out);
          return out;
      }
      static vector3Add(a, b, out) {
          out = out ? out : new Vector3();
          Vector3.add(a, b, out);
          return out;
      }
      static vector3Normalize(a, out) {
          out = out ? out : new Vector3();
          Vector3.normalize(a, out);
          return out;
      }
      static magnitude(vector3) {
          return Math.sqrt(this.sqrMagnitude(vector3));
      }
      static magHorizontal(first, second) {
          return Math.sqrt((first.x - second.x) * (first.x - second.x) + (first.z - second.z) * (first.z - second.z));
      }
      static mag(x1, y1, x2, y2) {
          return Math.sqrt((x1 - x2) * (x1 - x2) + (y1 - y2) * (y1 - y2));
      }
      static sqrMagnitude(vector3) {
          return vector3.x * vector3.x + vector3.y * vector3.y + vector3.z * vector3.z;
      }
      static normalize(point) {
          var magSqr = point.x * point.x + point.y * point.y;
          if (magSqr == 1.0)
              return new Laya.Point(point.x, point.y);
          if (magSqr == 0.0) {
              return new Laya.Point(point.x, point.y);
          }
          let out = new Laya.Point();
          var invsqrt = 1.0 / Math.sqrt(magSqr);
          out.x = point.x * invsqrt;
          out.y = point.y * invsqrt;
          return out;
      }
      ;
      static normalizeSelf(vector3) {
          var magSqr = vector3.x * vector3.x + vector3.z * vector3.z;
          if (magSqr == 1.0)
              return vector3;
          if (magSqr == 0.0) {
              return vector3;
          }
          var invsqrt = 1.0 / Math.sqrt(magSqr);
          vector3.x = vector3.x * invsqrt;
          vector3.z = vector3.z * invsqrt;
          return vector3;
      }
      ;
      static mul(point, num) {
          let out = new Laya.Point();
          out.x = point.x * num;
          out.y = point.y * num;
          return out;
      }
      static magSqr(point) {
          return point.x * point.x + point.y * point.y;
      }
      static add(point, addPoint) {
          let out = new Laya.Point();
          out.x = point.x + addPoint.x;
          out.y = point.y + addPoint.y;
          return new Laya.Point(out.x, out.y);
      }
      static dot(a, b) {
          return a.x * b.x + a.y * b.y;
      }
      static distanceVector2(targetPoint, sourcePoint) {
          let x = targetPoint.x - sourcePoint.x;
          let y = targetPoint.y - sourcePoint.y;
          return x * x + y * y;
      }
      static distanceSquared(targetPoint, sourcePoint) {
          let x = targetPoint.x - sourcePoint.x;
          let y = targetPoint.y - sourcePoint.y;
          return x * x + y * y;
      }
      static rotateY(vector3, radians) {
          var sin = Math.sin(radians);
          var cos = Math.cos(radians);
          var x = vector3.x;
          vector3.x = cos * x - sin * vector3.z;
          vector3.z = sin * x + cos * vector3.z;
          return vector3;
      }
      ;
      static angleY(target, source) {
          let x = target.x - source.x;
          let y = target.z - source.z;
          return Math.atan2(x, y) * 180 / Math.PI;
      }
      static getTargetAngle(firstPos, endPos) {
          var angle = 0;
          var dir = new Laya.Vector3();
          Laya.Vector3.subtract(endPos, firstPos, dir);
          angle = Math.atan2(dir.x, dir.z) * 180 / Math.PI;
          return angle;
      }
      static angleY2(vec) {
          return Math.atan2(vec.x, vec.z) * 180 / Math.PI;
      }
      static angleX(target, source) {
          let x = target.y - source.y;
          let y = target.z - source.z;
          return Math.atan2(-x, y) * 180 / Math.PI;
      }
      static angleZ(target, source) {
          let x = target.x - source.x;
          let y = target.y - source.y;
          return Math.atan2(-x, y) * 180 / Math.PI;
      }
      static angleZ2(vec) {
          return Math.atan2(-vec.x, vec.y) * 180 / Math.PI;
      }
      static angleDirection(direction, up) {
          Laya.Vector3.normalize(direction, direction);
          var cosVal = Laya.Vector3.dot(direction, up);
          var pitch = Math.acos(cosVal);
          pitch = 90 - pitch * 180 / Math.PI;
          return pitch;
      }
      static getVectorRotation(source, target) {
          var desQuaternion = new Laya.Quaternion();
          let normalizeDir = Util.vectorDirection(source, target);
          Laya.Quaternion.rotationLookAt(normalizeDir, Util.UP, desQuaternion);
          var desAngle = Util.ZERO.clone();
          desQuaternion.getYawPitchRoll(desAngle);
          var yy = this.angleY(source, target);
          var destRotation = new Laya.Vector3(-(desAngle.y * 180 / Math.PI), yy, 0);
          return destRotation;
      }
      static cross(a, b) {
          return a.x * b.y - a.y * b.x;
      }
      static angle(a, b) {
          var magSqr1 = this.magSqr(a);
          var magSqr2 = this.magSqr(b);
          if (magSqr1 === 0 || magSqr2 === 0) {
              return 0.0;
          }
          var dot = this.dot(a, b);
          var theta = dot / (Math.sqrt(magSqr1 * magSqr2));
          theta = this.clampf(theta, -1.0, 1.0);
          return Math.acos(theta);
      }
      ;
      static clampf(value, min_inclusive, max_inclusive) {
          if (min_inclusive > max_inclusive) {
              var temp = min_inclusive;
              min_inclusive = max_inclusive;
              max_inclusive = temp;
          }
          return value < min_inclusive ? min_inclusive : value < max_inclusive ? value : max_inclusive;
      }
      ;
      static clamp(value, min = Number.NEGATIVE_INFINITY, max = Number.POSITIVE_INFINITY) {
          if (min > max) {
              var temp = min;
              min = max;
              max = temp;
          }
          return Math.max(min, Math.min(max, value));
      }
      static clamp01(value) {
          return Math.max(0, Math.min(1, value));
      }
      static random(min, max) {
          return Math.random() * (max - min) + min << 0;
      }
      static probabilityCanHappen(num) {
          let random = this.random(0, 100);
          return random <= num;
      }
      static scaleTo(target, startScale, endScale, duration, caller, callback) {
          Laya.Tween.clearAll(target);
          Laya.Tween.to(target, { scaleX: startScale, scaleY: startScale }, duration * 0.5, null, Laya.Handler.create(this, () => {
              Laya.Tween.to(target, { scaleX: endScale, scaleY: endScale }, duration * 0.5, null, Laya.Handler.create(caller, callback));
          }));
      }
      static scaleToDefault(target, caller, callback) {
          this.scaleTo(target, 0.8, 1, 200, caller, callback);
          this.delayMouseEnabled(target);
      }
      static scaleToDefault2(target, caller, callback) {
          this.scaleTo(target, 0.8, 1, 200, caller, callback);
      }
      static delayMouseEnabled(node, delay = 500) {
          node.mouseEnabled = false;
          Laya.stage.timerOnce(delay, this, () => {
              node && (node.mouseEnabled = true);
          });
      }
      static ComputeCollision(w, h, r, rx, ry) {
          var dx = Math.min(rx, w * 0.5);
          var dx1 = Math.max(dx, -w * 0.5);
          var dy = Math.min(ry, h * 0.5);
          var dy1 = Math.max(dy, -h * 0.5);
          return (dx1 - rx) * (dx1 - rx) + (dy1 - ry) * (dy1 - ry) <= r * r;
      }
      static IsCircleIntersectTriangle(x, y, r, x1, y1, x2, y2, x3, y3) {
          if (this.PointinTriangle1(new Laya.Vector3(x, 0, y), new Laya.Vector3(x1, 0, y1), new Laya.Vector3(x2, 0, y2), new Laya.Vector3(x3, 0, y3)))
              return true;
          if (this.IsCircleIntersectLineSeg(x, y, r, x1, y1, x2, y2))
              return true;
          if (this.IsCircleIntersectLineSeg(x, y, r, x2, y2, x3, y3))
              return true;
          if (this.IsCircleIntersectLineSeg(x, y, r, x3, y3, x1, y1))
              return true;
          return false;
      }
      static SameSide(A, B, C, P) {
          let AB = new Laya.Vector3();
          let AC = new Laya.Vector3();
          let AP = new Laya.Vector3();
          Laya.Vector3.subtract(B, A, AB);
          Laya.Vector3.subtract(C, A, AC);
          Laya.Vector3.subtract(P, A, AP);
          let v1 = new Laya.Vector3();
          let v2 = new Laya.Vector3();
          Laya.Vector3.cross(AB, AC, v1);
          Laya.Vector3.cross(AB, AP, v2);
          let num = Laya.Vector3.dot(v1, v2);
          return num >= 0;
      }
      static PointinTriangle1(A, B, C, P) {
          return this.SameSide(A, B, C, P) && this.SameSide(B, C, A, P) && this.SameSide(C, A, B, P);
      }
      static IsCircleIntersectLineSeg(x, y, r, x1, y1, x2, y2) {
          var vx1 = x - x1;
          var vy1 = y - y1;
          var vx2 = x2 - x1;
          var vy2 = y2 - y1;
          if (Math.abs(vx2) < 0.00001 && Math.abs(vy2) < 0.00001)
              return false;
          var len = Math.sqrt(vx2 * vx2 + vy2 * vy2);
          vx2 /= len;
          vy2 /= len;
          var u = vx1 * vx2 + vy1 * vy2;
          var x0 = 0;
          var y0 = 0;
          if (u <= 0) {
              x0 = x1;
              y0 = y1;
          }
          else if (u >= len) {
              x0 = x2;
              y0 = y2;
          }
          else {
              x0 = x1 + vx2 * u;
              y0 = y1 + vy2 * u;
          }
          return (x - x0) * (x - x0) + (y - y0) * (y - y0) <= r * r;
      }
      static computeDistanceBetweenPointAndLine(x, y, x1, y1, x2, y2) {
          var a = y2 - y1;
          var b = x1 - x2;
          var c = x2 * y1 - x1 * y2;
          return Math.abs(a * x + b * y + c) / Math.sqrt(a * a + b * b);
      }
      static detectCircleAndRectCollision(circle, rect) {
          var ABLength = Laya.Vector3.distance(rect.vector2, rect.vector1);
          var DALength = Laya.Vector3.distance(rect.vector1, rect.vector4);
          var halfABLength = ABLength / 2;
          var halfDALength = DALength / 2;
          var vectorAB = Util.vector3Subtract(rect.vector2, rect.vector1);
          var vectorAC = Util.vector3Subtract(rect.vector3, rect.vector1);
          var vectorAD = Util.vector3Subtract(rect.vector4, rect.vector1);
          var rectCenterX = rect.x + vectorAC.x / 2;
          var rectCenterY = rect.y + vectorAC.z / 2;
          var ABCenterX = rect.x + vectorAB.x / 2;
          var ABCenterY = rect.y + vectorAB.z / 2;
          var ADCenterX = rect.x + vectorAD.x / 2;
          var ADCenterY = rect.y + vectorAD.z / 2;
          var d1 = this.computeDistanceBetweenPointAndLine(circle.mtransform.position.x, circle.mtransform.position.z, rectCenterX, rectCenterY, ADCenterX, ADCenterY);
          if (d1 > (halfDALength + circle.radius))
              return false;
          var d2 = this.computeDistanceBetweenPointAndLine(circle.mtransform.position.x, circle.mtransform.position.z, rectCenterX, rectCenterY, ABCenterX, ABCenterY);
          if (d2 > (halfABLength + circle.radius))
              return false;
          if ((d1 <= halfDALength) ||
              (d2 <= halfABLength))
              return true;
          return false;
      }
      static segmentsIntr(a, b, c, d) {
          var area_abc = (a.x - c.x) * (b.y - c.y) - (a.y - c.y) * (b.x - c.x);
          var area_abd = (a.x - d.x) * (b.y - d.y) - (a.y - d.y) * (b.x - d.x);
          if (area_abc * area_abd >= 0) {
              return false;
          }
          var area_cda = (c.x - a.x) * (d.y - a.y) - (c.y - a.y) * (d.x - a.x);
          var area_cdb = area_cda + area_abc - area_abd;
          if (area_cda * area_cdb >= 0) {
              return false;
          }
          var t = area_cda / (area_abd - area_abc);
          var dx = t * (b.x - a.x), dy = t * (b.y - a.y);
          return { x: a.x + dx, y: a.y + dy };
      }
      static judgeIntersect(x1, y1, x2, y2, x3, y3, x4, y4) {
          if (!(Math.min(x1, x2) <= Math.max(x3, x4) && Math.min(y3, y4) <= Math.max(y1, y2) && Math.min(x3, x4) <= Math.max(x1, x2) && Math.min(y1, y2) <= Math.max(y3, y4)))
              return false;
          var u, v, w, z;
          u = (x3 - x1) * (y2 - y1) - (x2 - x1) * (y3 - y1);
          v = (x4 - x1) * (y2 - y1) - (x2 - x1) * (y4 - y1);
          w = (x1 - x3) * (y4 - y3) - (x4 - x3) * (y1 - y3);
          z = (x2 - x3) * (y4 - y3) - (x4 - x3) * (y2 - y3);
          return (u * v <= 0.00000001 && w * z <= 0.00000001);
      }
      static vectorDirection(srcVec, desVec) {
          var subtractPos = new Laya.Vector3(0, 0, 0);
          Laya.Vector3.subtract(desVec, srcVec, subtractPos);
          var normalizeDir = new Laya.Vector3(0, 0, 0);
          Laya.Vector3.normalize(subtractPos, normalizeDir);
          return normalizeDir;
      }
      static lerp(from, to, t) {
          return from + (to - from) * Math.min(1, t);
      }
      static IsNumber(val) {
          var regPos = /^\d+(\.\d+)?$/;
          var regNeg = /^(-(([0-9]+\.[0-9]*[1-9][0-9]*)|([0-9]*[1-9][0-9]*\.[0-9]+)|([0-9]*[1-9][0-9]*)))$/;
          if (regPos.test(val) || regNeg.test(val)) {
              return true;
          }
          else {
              return false;
          }
      }
      static padding(num, length) {
          for (var len = (num + "").length; len < length; len = num.length) {
              num = "0" + num;
          }
          return num;
      }
      static Reflect(vec, normal) {
          let out = new Laya.Vector3();
          Laya.Vector3.normalize(vec, vec);
          Laya.Vector3.normalize(normal, normal);
          Laya.Vector3.scale(normal, 2.0 * Laya.Vector3.dot(vec, normal), out);
          Laya.Vector3.subtract(vec, out, out);
          return out;
      }
      static GetVerticalDir(_dir) {
          if (_dir.z == 0) {
              return new Laya.Vector3(0, 0, -1);
          }
          else if (_dir.x == 0) {
              return new Laya.Vector3(1, 0, 0);
          }
          else {
              let out = new Laya.Vector3();
              Laya.Vector3.normalize(new Laya.Vector3(-_dir.z / _dir.x, 0, 1), out);
              return out;
          }
      }
      static WorldToScreen2(camera, point) {
          var pointA = this.InverseTransformPoint(camera.transform, point);
          var distance = pointA.z;
          var out = new Laya.Vector3();
          camera.viewport.project(point, camera.projectionViewMatrix, out);
          var value = new Laya.Vector3(out.x / Laya.stage.clientScaleX, out.y / Laya.stage.clientScaleY, distance);
          return value;
      }
      static ScreenToWorld(camera, point) {
          var halfFOV = (camera.fieldOfView * 0.5) * Math.PI / 180;
          let height = point.z * Math.tan(halfFOV);
          let width = height * camera.aspectRatio;
          let lowerLeft = this.GetLowerLeft(camera.transform, point.z, width, height);
          let v = this.GetScreenScale(width, height);
          var value = new Laya.Vector3();
          var lowerLeftA = this.InverseTransformPoint(camera.transform, lowerLeft);
          value = new Laya.Vector3(-point.x / v.x, point.y / v.y, 0);
          Laya.Vector3.add(lowerLeftA, value, value);
          value = this.TransformPoint(camera.transform, value);
          return value;
      }
      static GetScreenScale(width, height) {
          var v = new Laya.Vector3();
          v.x = Laya.stage.width / width / 2;
          v.y = Laya.stage.height / height / 2;
          return v;
      }
      static GetLowerLeft(transform, distance, width, height) {
          var lowerLeft = new Laya.Vector3();
          var right = new Laya.Vector3();
          transform.getRight(right);
          Laya.Vector3.normalize(right, right);
          var xx = new Laya.Vector3(right.x * width, right.y * width, right.z * width);
          Laya.Vector3.add(transform.position, xx, lowerLeft);
          var up = new Laya.Vector3();
          transform.getUp(up);
          Laya.Vector3.normalize(up, up);
          var yy = new Laya.Vector3(up.x * height, up.y * height, up.z * height);
          Laya.Vector3.subtract(lowerLeft, yy, lowerLeft);
          var forward = new Laya.Vector3();
          transform.getForward(forward);
          Laya.Vector3.normalize(forward, forward);
          var zz = new Laya.Vector3(forward.x * distance, forward.y * distance, forward.z * distance);
          Laya.Vector3.subtract(lowerLeft, zz, lowerLeft);
          return lowerLeft;
      }
      static InverseTransformPoint(origin, point) {
          var xx = new Laya.Vector3();
          origin.getRight(xx);
          var yy = new Laya.Vector3();
          origin.getUp(yy);
          var zz = new Laya.Vector3();
          origin.getForward(zz);
          var zz1 = new Laya.Vector3(-zz.x, -zz.y, -zz.z);
          var x = this.ProjectDistance(point, origin.position, xx);
          var y = this.ProjectDistance(point, origin.position, yy);
          var z = this.ProjectDistance(point, origin.position, zz1);
          var value = new Laya.Vector3(x, y, z);
          return value;
      }
      static TransformPoint(origin, point) {
          var value = new Laya.Vector3();
          Laya.Vector3.transformQuat(point, origin.rotation, value);
          Laya.Vector3.add(value, origin.position, value);
          return value;
      }
      static ProjectDistance(A, C, B) {
          var CA = new Laya.Vector3();
          Laya.Vector3.subtract(A, C, CA);
          var angle = this.Angle2(CA, B) * Math.PI / 180;
          var distance = Laya.Vector3.distance(A, C);
          distance *= Math.cos(angle);
          return distance;
      }
      static Angle2(ma, mb) {
          var v1 = (ma.x * mb.x) + (ma.y * mb.y) + (ma.z * mb.z);
          var ma_val = Math.sqrt(ma.x * ma.x + ma.y * ma.y + ma.z * ma.z);
          var mb_val = Math.sqrt(mb.x * mb.x + mb.y * mb.y + mb.z * mb.z);
          var cosM = v1 / (ma_val * mb_val);
          if (cosM < -1)
              cosM = -1;
          if (cosM > 1)
              cosM = 1;
          var angleAMB = Math.acos(cosM) * 180 / Math.PI;
          return angleAMB;
      }
      static IsEqual(num1, num2) {
          let absX = Math.abs(num1 - num2);
          return absX < 0.00001;
      }
      static changMat(matname, meshRenderer) {
          let npcmat = Laya.Loader.getRes(matname);
          if (!npcmat) {
              Laya.Material.load(matname, Laya.Handler.create(null, function (mat) {
                  if (mat && meshRenderer) {
                      meshRenderer.material = mat;
                  }
              }));
          }
          else {
              meshRenderer && (meshRenderer.material = npcmat);
          }
      }
      static changShareMat(matname, meshRenderer) {
          let npcmat = Laya.Loader.getRes(matname);
          if (!npcmat) {
              Laya.Material.load(matname, Laya.Handler.create(null, function (mat) {
                  if (mat && meshRenderer) {
                      meshRenderer.sharedMaterial = mat;
                  }
              }));
          }
          else {
              meshRenderer && (meshRenderer.sharedMaterial = npcmat);
          }
      }
      static mutation(first, second) {
          if (first.length != second.length)
              return false;
          var a = first.toLowerCase();
          var b = second.toLowerCase();
          for (var i = 0; i < b.length; i++) {
              if (a.indexOf(b[i]) === -1) {
                  return false;
              }
          }
          return true;
      }
      static isCollision(x1, y1, x2, y2, w, h) {
          if (x1 >= x2 && x1 <= x2 + w && y1 >= y2 && y1 <= y2 + h) {
              return true;
          }
          return false;
      }
      static isCollisionWithRect(x1, y1, w1, h1, x2, y2, w2, h2) {
          if (x1 >= x2 && x1 >= x2 + w2) {
              return false;
          }
          else if (x1 <= x2 && x1 + w1 <= x2) {
              return false;
          }
          else if (y1 >= y2 && y1 >= y2 + h2) {
              return false;
          }
          else if (y1 <= y2 && y1 + h1 <= y2) {
              return false;
          }
          return true;
      }
      static activeAllChild(node, isActive) {
          if (!node)
              return;
          for (let i = 0; i < node.numChildren; i++) {
              let child = node.getChildAt(i);
              child.active = isActive;
          }
      }
      static visibleAllChild(node, isActive) {
          if (!node)
              return;
          for (let i = 0; i < node.numChildren; i++) {
              let child = node.getChildAt(i);
              child.visible = isActive;
          }
      }
      static clearSprite(sprite3d) {
          if (!sprite3d)
              return;
          while (sprite3d.numChildren > 0) {
              let el = sprite3d.getChildAt(0);
              let scripts3d = sprite3d.getComponents(Laya.Script3D);
              if (scripts3d) {
                  for (let index = 0; index < scripts3d.length; index++) {
                      let script = scripts3d[index];
                      if (script) {
                          script.destroy();
                      }
                  }
              }
              let scripts = sprite3d.getComponents(Laya.Script);
              if (scripts) {
                  for (let index = 0; index < scripts.length; index++) {
                      let script = scripts[index];
                      if (script) {
                          script.destroy();
                      }
                  }
              }
              el.active = false;
              el.removeSelf();
              el.destroy(true);
          }
      }
      static unique(array) {
          var temp = [];
          var index = [];
          var l = array.length;
          for (var i = 0; i < l; i++) {
              for (var j = i + 1; j < l; j++) {
                  if (array[i] === array[j]) {
                      i++;
                      j = i;
                  }
              }
              temp.push(array[i]);
              index.push(i);
          }
          return temp;
      }
      static screen2worldPosByName(checkObjName, scenee3d, camera, screenPos, worldPos, collisionGroup = Laya.Physics3DUtils.COLLISIONFILTERGROUP_ALLFILTER, canCollideWith = Laya.Physics3DUtils.COLLISIONFILTERGROUP_ALLFILTER) {
          let _startray = new Laya.Ray(new Laya.Vector3(0, 0, 0), new Laya.Vector3(0, 0, 0));
          let _outHitResult = [];
          let screenPoint = new Laya.Vector2(screenPos.x / (Laya.stage.width * Laya.stage.scaleX), screenPos.y / (Laya.stage.height * Laya.stage.scaleY));
          camera.normalizedViewportPointToRay(screenPoint, _startray);
          scenee3d.physicsSimulation.rayCastAll(_startray, _outHitResult, 10000, collisionGroup, canCollideWith);
          for (let i = 0, n = _outHitResult.length; i < n; i++) {
              let collisionName = _outHitResult[i].collider.owner.name;
              if (collisionName.indexOf(checkObjName) != -1) {
                  worldPos.setValue(_outHitResult[i].point.x, _outHitResult[i].point.y, _outHitResult[i].point.z);
                  return true;
              }
          }
          return false;
      }
      static screen2worldPos(scenee3d, camera, screenPos, worldPos, collisionGroup = Laya.Physics3DUtils.COLLISIONFILTERGROUP_ALLFILTER, canCollideWith = Laya.Physics3DUtils.COLLISIONFILTERGROUP_ALLFILTER) {
          let _startray = new Laya.Ray(new Laya.Vector3(0, 0, 0), new Laya.Vector3(0, 0, 0));
          let _outHitResult = new Laya.HitResult();
          screenPos.setValue(screenPos.x / (Laya.stage.width * Laya.stage.scaleX), screenPos.y / (Laya.stage.height * Laya.stage.scaleY));
          camera.normalizedViewportPointToRay(screenPos, _startray);
          scenee3d.physicsSimulation.rayCast(_startray, _outHitResult, 10000, collisionGroup, canCollideWith);
          if (_outHitResult.succeeded) {
              console.log('碰撞的：', _outHitResult.collider.owner.name);
              worldPos.setValue(_outHitResult.point.x, _outHitResult.point.y, _outHitResult.point.z);
              console.log("worldPos", worldPos);
          }
      }
      static raySpriteByName(checkObjName, scenee3d, camera, screenPos, collisionGroup = Laya.Physics3DUtils.COLLISIONFILTERGROUP_ALLFILTER, canCollideWith = Laya.Physics3DUtils.COLLISIONFILTERGROUP_ALLFILTER) {
          let _startray = new Laya.Ray(new Laya.Vector3(0, 0, 0), new Laya.Vector3(0, 0, 0));
          let _outHitResult = [];
          let screenPoint = new Laya.Vector2(screenPos.x / (Laya.stage.width * Laya.stage.scaleX), screenPos.y / (Laya.stage.height * Laya.stage.scaleY));
          camera.normalizedViewportPointToRay(screenPoint, _startray);
          scenee3d.physicsSimulation.rayCastAll(_startray, _outHitResult, 10000, collisionGroup, canCollideWith);
          for (let i = 0, n = _outHitResult.length; i < n; i++) {
              let collisionName = _outHitResult[i].collider.owner.name;
              if (collisionName.indexOf(checkObjName) != -1) {
                  return _outHitResult[i].collider.owner;
              }
          }
          return null;
      }
      static rayCheckSprite(scenee3d, from, to, worldPos, collisionGroup = Laya.Physics3DUtils.COLLISIONFILTERGROUP_ALLFILTER, canCollideWith = Laya.Physics3DUtils.COLLISIONFILTERGROUP_ALLFILTER) {
          let direction = Util.vectorDirection(from, to);
          let _startray = new Laya.Ray(from, direction);
          let _outHitResult = new Laya.HitResult();
          ;
          scenee3d.physicsSimulation.rayCast(_startray, _outHitResult, 10000, collisionGroup, canCollideWith);
          if (_outHitResult && _outHitResult.succeeded) {
              worldPos.setValue(_outHitResult.point.x, _outHitResult.point.y, _outHitResult.point.z);
              return _outHitResult.collider.owner;
          }
          return null;
      }
      static shapeCheckSprite(scenee3d, from, to, worldPos, collisionGroup = Laya.Physics3DUtils.COLLISIONFILTERGROUP_ALLFILTER, canCollideWith = Laya.Physics3DUtils.COLLISIONFILTERGROUP_ALLFILTER) {
          let _outHitResult = new Laya.HitResult();
          var boxCollider = new Laya.BoxColliderShape(1, 1, 1);
          scenee3d.physicsSimulation.shapeCast(boxCollider, from, to, _outHitResult, null, null, collisionGroup, canCollideWith);
          if (_outHitResult && _outHitResult.succeeded) {
              worldPos.setValue(_outHitResult.point.x, _outHitResult.point.y, _outHitResult.point.z);
              return _outHitResult.collider.owner;
          }
          return null;
      }
      static getBaseURL(name, subPachage, suffix) {
          subPachage = subPachage ? subPachage : "3dres";
          let finalUrl = subPachage + "/Conventional/" + name;
          if (suffix != null && suffix.trim().length > 0) {
              finalUrl += "." + suffix;
          }
          return finalUrl;
      }
      static formatter(num, digits) {
          const si = [
              { value: 1, symbol: "" },
              { value: 1E3, symbol: "K" },
              { value: 1E6, symbol: "M" },
              { value: 1E9, symbol: "G" },
              { value: 1E12, symbol: "T" },
              { value: 1E15, symbol: "P" },
              { value: 1E18, symbol: "E" }
          ];
          const rx = /\.0+$|(\.[0-9]*[1-9])0+$/;
          let i;
          for (i = si.length - 1; i > 0; i--) {
              if (num >= si[i].value) {
                  break;
              }
          }
          return (num / si[i].value).toFixed(digits).replace(rx, "$1") + si[i].symbol;
      }
      static drawText3d(mTitleText, mTitleLabel, mTitleMat, txt) {
          mTitleLabel.width = 256;
          mTitleLabel.height = 128;
          mTitleLabel.align = "center";
          mTitleLabel.valign = "middle";
          mTitleLabel.fontSize = 30;
          mTitleLabel.font = "Microsoft YaHei";
          mTitleLabel.color = "#000000";
          mTitleLabel.text = txt;
          mTitleMat.albedoTexture = mTitleText;
          Laya.stage.frameOnce(2, this, function () {
              mTitleLabel.drawToTexture(256, 128, 0, 0, mTitleText);
          }, null, false);
      }
  }
  Util.UP = new Laya.Vector3(0, 1, 0);
  Util.DOWN = new Laya.Vector3(0, -1, 0);
  Util.LEFT = new Laya.Vector3(1, 0, 0);
  Util.RIGHT = new Laya.Vector3(-1, 0, 0);
  Util.FORWARD = new Laya.Vector3(0, 0, 1);
  Util.ONE = new Laya.Vector3(1, 1, 1);
  Util.ZERO = new Laya.Vector3(0, 0, 0);
  Util.RADIUS_TO_ANGLE = 180 / Math.PI;
  Util.ANGLE_TO_RADIUS = Math.PI / 180;
  Util.signAngle = function (a, b) {
      let angle = this.angle(a, b);
      return this.cross(a, b) < 0 ? -angle : angle;
  };

  class GameData {
      constructor() {
          this.stageCfg = [];
          this.curStageId = 1;
          this.gold = 0;
          this.stageGold = 0;
          this.atkUp = 0;
          this.isShowVideoForSolider = false;
          this.isShowVideoForBrick = false;
          this.selectRoleId = 0;
          this.gameMode = 0;
          this.gameFinish = false;
          this.stageCount = 1;
      }
      set roleId(val) {
          this.selectRoleId = val;
      }
      set gameGold(val) {
          this.gold = this.gameGold;
          this.gold += val;
          fc.framework.GameFrame.savePlayerInfo({ gold: this.gold });
      }
      get gameGold() {
          var localgold = fc.framework.GameFrame.playerInfo.gold;
          if (Util.IsNumber(localgold)) {
              return Number(localgold);
          }
          return 0;
      }
      get warriors() {
          return fc.framework.GameFrame.playerInfo.warriors;
      }
      set warriors(val) {
          let count = Math.max(1, val);
          fc.framework.GameFrame.savePlayerInfo({ warriors: count });
      }
      getClaim() {
          return fc.framework.GameFrame.playerInfo.claim;
      }
      setClaim(val, index) {
          let claim = fc.framework.GameFrame.playerInfo.claim;
          claim[index] = val;
          fc.framework.GameFrame.savePlayerInfo({ claim: claim });
      }
      getGameTime(type) {
          let localName = "gametimes" + type;
          var localstage = Laya.LocalStorage.getItem(localName);
          let lv = 0;
          if (Util.IsNumber(localstage)) {
              lv = Math.max(0, Number(localstage));
          }
          return lv;
      }
      setGameTime(val, type) {
          let lv = Math.max(0, val);
          let localName = "gametimes" + type;
          Laya.LocalStorage.setItem(localName, lv.toString());
      }
      getWarriorLv(warriorType) {
          let localName = "warriorlv" + warriorType;
          var localstage = Laya.LocalStorage.getItem(localName);
          let lv = 0;
          if (Util.IsNumber(localstage)) {
              lv = Math.max(0, Number(localstage));
          }
          return lv;
      }
      setWarriorLv(val, warriorType) {
          let lv = Math.max(0, val);
          let localName = "warriorlv" + warriorType;
          var localstage = Laya.LocalStorage.getItem(localName);
          if (Util.IsNumber(localstage)) {
              lv = Math.max(lv, Number(localstage));
          }
          Laya.LocalStorage.setItem(localName, lv.toString());
      }
      getCastleLv(castletype) {
          let localName = "castleLv" + castletype;
          var localstage = Laya.LocalStorage.getItem(localName);
          let lv = 0;
          if (Util.IsNumber(localstage)) {
              lv = Math.max(0, Number(localstage));
          }
          return lv;
      }
      setCastleLv(val, castletype) {
          let lv = Math.max(0, val);
          let localName = "castleLv" + castletype;
          var localstage = Laya.LocalStorage.getItem(localName);
          if (Util.IsNumber(localstage)) {
              lv = Math.max(lv, Number(localstage));
          }
          Laya.LocalStorage.setItem(localName, lv.toString());
      }
      getCommonLv(castletype) {
          let localName = "commonLv" + castletype;
          var localstage = Laya.LocalStorage.getItem(localName);
          let lv = 0;
          if (Util.IsNumber(localstage)) {
              lv = Math.max(0, Number(localstage));
          }
          return lv;
      }
      setCommonLv(val, castletype) {
          let lv = Math.max(0, val);
          let localName = "commonLv" + castletype;
          var localstage = Laya.LocalStorage.getItem(localName);
          if (Util.IsNumber(localstage)) {
              lv = Math.max(lv, Number(localstage));
          }
          Laya.LocalStorage.setItem(localName, lv.toString());
      }
      getWpVideoCount(weaponName) {
          let localName = "robotgunweaponvideo" + weaponName;
          var localstage = Laya.LocalStorage.getItem(localName);
          let lv = 0;
          if (Util.IsNumber(localstage)) {
              lv = Math.max(0, Number(localstage));
          }
          return lv;
      }
      setWpVideoCount(val, weaponName) {
          let lv = Math.max(0, val);
          let localName = "robotgunweaponvideo" + weaponName;
          var localstage = Laya.LocalStorage.getItem(localName);
          if (Util.IsNumber(localstage)) {
              lv = Math.max(lv, Number(localstage));
          }
          Laya.LocalStorage.setItem(localName, lv.toString());
      }
      get curStage() {
          return this.curStageId;
      }
      set curStage(val) {
          let stage = Math.max(1, val);
          var localstage = Laya.LocalStorage.getItem("growempirestage");
          if (Util.IsNumber(localstage)) {
              stage = Math.max(stage, Number(localstage));
          }
          Laya.LocalStorage.setItem("growempirestage", stage.toString());
          this.curStageId = stage;
          fc.framework.GameFrame.savePlayerInfo({ levelId: stage });
      }
      static initStageData(mapdata) {
          if (mapdata) {
              this.data.stageCfg = mapdata.stagedatas;
              this.data.stageCount = this.data.stageCfg.length;
          }
      }
      static initCastleCfg(castledata) {
          if (castledata) {
              this.data.castleCfg = castledata;
          }
      }
      static initCommonCfg(commondata) {
          if (commondata) {
              this.data.commonCfg = commondata;
          }
      }
      static initEnemyCfg(data) {
          if (data) {
              this.data.enemyData = data.enemydatas;
          }
      }
      static initWarriorCfg(data) {
          if (data) {
              this.data.warriorData = data.warriordatas;
          }
      }
      static updatePlayerInfo(playerInfo) {
          if (playerInfo) {
              this.data.gold = this.data.gameGold;
              this.data.selectRoleId = fc.framework.GameFrame.playerInfo.goodsInUse[0];
              this.data.curStage = fc.framework.GameFrame.playerInfo.levelId;
              console.log('==========更新用户信息===========', fc.framework.GameFrame.playerInfo.goodsInBag[0]);
          }
      }
      getStageData(id) {
          id = Math.min(id, this.stageCfg.length);
          return this.stageCfg[id - 1];
      }
      getWarriorData(name) {
          let data = null;
          this.warriorData.forEach(function (info) {
              if (name == info.name) {
                  data = info;
                  return;
              }
          });
          return data;
      }
      getEnemyData(name) {
          let data = null;
          this.enemyData.forEach(function (info) {
              if (name == info.name) {
                  data = info;
                  return;
              }
          });
          return data;
      }
  }
  GameData.data = new GameData();

  class EventConst {
  }
  EventConst.LOADRES_SUCCESS = "LOADRES_SUCCESS";
  EventConst.SHOW_SUCCESS = "SHOW_SUCCESS";
  EventConst.SHOW_FAILED = "SHOW_FAILED";
  EventConst.CHANGE_CUR_MODEL = "CHANGE_CUR_MODEL";
  EventConst.SHOW_GUIDE = "SHOW_GUIDE";
  EventConst.HIDE_GUIDE = "Hide_GUIDE";
  EventConst.MOUSE_DOWN = "MOUSE_DOWN";
  EventConst.Game_Update_Gold = "Game_Update_Gold";
  EventConst.Game_Update_Progress = "Game_Update_Progress";
  EventConst.Battle_Start = "Battle_Start";
  EventConst.Update_Total_Hp = "Update_Total_Hp";
  EventConst.Soldier_LevelUp = "Soldier_LevelUp";
  EventConst.Update_Soldier_UI = "Update_Soldier_UI";
  EventConst.Game_Count_Tip = "Game_Count_Tip";
  EventConst.Update_LevelUi_Pos = "Update_LevelUi_Pos";
  EventConst.Player_Relive = "Player_Relive";
  EventConst.Show_Video_Btn = "Show_Video_Btn";
  EventConst.Video_Get_Solider = "Video_Get_Solider";
  EventConst.Video_Get_Brick = "Video_Get_Brick";
  EventConst.Game_Refresh = "Game_Refresh";
  EventConst.RENEW_MAX_HP = "RENEW_MAX_HP";
  EventConst.SHOW_BUILD_TIP = "SHOW_BUILD_TIP";
  EventConst.HIDE_BUILD_TIP = "HIDE_BUILD_TIP";
  EventConst.Update_Scenario = "Update_Scenario";
  EventConst.Update_Archer = "Update_Archer";
  EventConst.Update_Hp = "Update_Hp";
  EventConst.Update_Tower = "Update_Tower";
  EventConst.Update_Catapult = "Update_Catapult";
  EventConst.Fire_Tower = "Fire_Tower";
  EventConst.Fire_Catapult = "Fire_Catapult";
  EventConst.Strong_Power_Start = "Strong_Power_Start";
  EventConst.Strong_Power_End = "Strong_Power_End";
  EventConst.Update_XianFeng = "Update_XianFeng";
  EventConst.Update_Online = "Update_Online";
  EventConst.Close_Door = "Close_Door";
  EventConst.Open_Door = "Open_Door";

  class ResConst {
  }
  ResConst.Sound_Click = "sound/click.mp3";
  ResConst.Sprite3d_BlueRoot = "Blue";
  ResConst.Sprite3d_RedRoot = "Red";
  ResConst.Sprite3d_Player = "Role";
  ResConst.Sprite3d_Role_Cowboy = "Role_Cowboy";
  ResConst.Sprite3d_Role_Elephant = "Role_Elephant";
  ResConst.Sprite3d_BulletBlue = "ef_arrow_blue_01";
  ResConst.Sprite3d_BulletRed = "ef_arrow_red_01";
  ResConst.Sprite3d_Scene = "Scene";
  ResConst.Sprite3d_Arrow = "Arrow";
  ResConst.Sprite3d_Ballista_Arrow = "Ballista_Arrow";
  ResConst.Sprite3d_Javelin = "Javelin";
  ResConst.Sprite3d_StoneBomb = "ef_blast_01";
  ResConst.MaterialPath = "3dres/Conventional/Assets/Materials/";
  ResConst.RoleMate = ['CH_Spartan_01.lmat', 'CH_Spartan_02.lmat'];
  ResConst.EnemyCfgName = ['DaoBing', 'QiangBing', 'BiaoQiangBing', 'QiBing', 'GongChengChui', 'DaXiang'];
  ResConst.WarriorCfgName = ['DaoBing', 'QiangBing', 'BiaoQiangBing', 'QiBing', 'XianFeng'];
  ResConst.WarriorLvName = [
      ['农夫', '民兵', '士兵', '甲兵', '重甲兵'],
      ['民兵农夫', '民兵枪兵', '普通枪兵', '重甲枪兵', '铁甲枪兵'],
      ['标枪民兵', '标枪士兵', '标枪力士', '标枪投手'],
      ['轻骑兵', '精锐骑兵', '重骑兵']
  ];
  var ScriptType;
  (function (ScriptType) {
      ScriptType[ScriptType["idle"] = 0] = "idle";
      ScriptType[ScriptType["run"] = 1] = "run";
      ScriptType[ScriptType["runback"] = 2] = "runback";
      ScriptType[ScriptType["pursuerrun"] = 3] = "pursuerrun";
      ScriptType[ScriptType["atk"] = 4] = "atk";
      ScriptType[ScriptType["atkwall"] = 5] = "atkwall";
      ScriptType[ScriptType["win"] = 6] = "win";
      ScriptType[ScriptType["unknow"] = 7] = "unknow";
  })(ScriptType || (ScriptType = {}));
  var TeamType;
  (function (TeamType) {
      TeamType[TeamType["Blue"] = 0] = "Blue";
      TeamType[TeamType["Red"] = 1] = "Red";
      TeamType[TeamType["Neutral"] = 2] = "Neutral";
  })(TeamType || (TeamType = {}));
  var CommonCfg;
  (function (CommonCfg) {
      CommonCfg["HaoJiao"] = "HaoJiao";
      CommonCfg["GongJianJuli"] = "GongJianJuli";
      CommonCfg["ChengQiang"] = "ChengQiang";
      CommonCfg["JinBi"] = "JinBi";
      CommonCfg["GongJianSudu"] = "GongJianSudu";
  })(CommonCfg || (CommonCfg = {}));
  var CastleCfg;
  (function (CastleCfg) {
      CastleCfg["Wall"] = "Wall";
      CastleCfg["Archer"] = "Archer";
      CastleCfg["Tower"] = "Tower";
      CastleCfg["Catapult"] = "Catapult";
  })(CastleCfg || (CastleCfg = {}));
  var WarriorType;
  (function (WarriorType) {
      WarriorType[WarriorType["DaoBing"] = 0] = "DaoBing";
      WarriorType[WarriorType["QiangBing"] = 1] = "QiangBing";
      WarriorType[WarriorType["BiaoQiangBing"] = 2] = "BiaoQiangBing";
      WarriorType[WarriorType["QiBing"] = 3] = "QiBing";
      WarriorType[WarriorType["XianFeng"] = 4] = "XianFeng";
      WarriorType[WarriorType["GongJian"] = 5] = "GongJian";
  })(WarriorType || (WarriorType = {}));
  var EnemyType;
  (function (EnemyType) {
      EnemyType[EnemyType["DaoBing"] = 0] = "DaoBing";
      EnemyType[EnemyType["QiangBing"] = 1] = "QiangBing";
      EnemyType[EnemyType["BiaoQiangBing"] = 2] = "BiaoQiangBing";
      EnemyType[EnemyType["QiBing"] = 3] = "QiBing";
      EnemyType[EnemyType["GongChengChui"] = 4] = "GongChengChui";
      EnemyType[EnemyType["DaXiang"] = 5] = "DaXiang";
  })(EnemyType || (EnemyType = {}));
  var GameTimeType;
  (function (GameTimeType) {
      GameTimeType[GameTimeType["Online"] = 0] = "Online";
      GameTimeType[GameTimeType["Offline"] = 1] = "Offline";
  })(GameTimeType || (GameTimeType = {}));

  class CastleShow extends Laya.Script {
      constructor() {
          super();
          this.mHp = 0;
          this.mTotalHp = 0;
      }
      onAwake() {
          super.onAwake();
          Laya.stage.on(EventConst.Update_Hp, this, this.updateHp);
          Laya.stage.on(EventConst.Update_Scenario, this, this.initHp);
          Laya.stage.on(EventConst.RENEW_MAX_HP, this, this.onMaxHp);
          this.owner.mouseEnabled = false;
      }
      updateHp(damage) {
          this.mHp -= damage;
          this.mHp = Math.max(0, this.mHp);
          this.hpVal && (this.hpVal.text = this.mHp + '/' + this.mTotalHp);
          this.panelProgress.width = (this.mHp / this.mTotalHp) * 150;
      }
      onStart() {
          super.onStart();
          this.owner.mouseEnabled = false;
          this.initHp();
          this.updateHp(0);
      }
      initHp() {
          let bounslv = Math.max(1, GameData.data.getCommonLv(CommonCfg.ChengQiang));
          let bunsdata = GameData.data.commonCfg.ChengQiang;
          let lv = Math.max(1, GameData.data.getCastleLv(CastleCfg.Wall));
          let data = GameData.data.castleCfg.Wall;
          this.mHp = data.hp + (lv - 1) * data.hpadd;
          this.mHp = Math.floor(this.mHp * (1 + ((bounslv - 1) * bunsdata.bonus1)));
          this.mTotalHp = this.mHp;
          this.hpVal && (this.hpVal.text = this.mHp + '/' + this.mTotalHp);
      }
      onMaxHp() {
          this.mHp = this.mTotalHp;
          this.updateHp(0);
      }
      onDisable() {
          super.onDisable();
          Laya.stage.off(EventConst.Update_Hp, this, this.updateHp);
          Laya.stage.off(EventConst.Update_Scenario, this, this.initHp);
          Laya.stage.off(EventConst.RENEW_MAX_HP, this, this.onMaxHp);
      }
      onDestroy() {
          super.onDisable();
          Laya.stage.off(EventConst.Update_Hp, this, this.updateHp);
          Laya.stage.off(EventConst.Update_Scenario, this, this.initHp);
          Laya.stage.off(EventConst.RENEW_MAX_HP, this, this.onMaxHp);
      }
  }

  class WaveProgress extends Laya.Script {
      constructor() {
          super();
      }
      onAwake() {
          super.onAwake();
          this.owner.mouseEnabled = false;
          Laya.stage.on(EventConst.Game_Update_Progress, this, this.updateProgress);
      }
      onStart() {
          super.onStart();
          this.waveLabel && (this.waveLabel.text = '公元' + GameData.data.curStage + '年');
      }
      onDisable() {
          super.onDisable();
          Laya.stage.off(EventConst.Game_Update_Progress, this, this.updateProgress);
      }
      onDestroy() {
          super.onDestroy();
          Laya.stage.off(EventConst.Game_Update_Progress, this, this.updateProgress);
      }
      updateProgress(progress) {
          progress = Math.min(1, progress);
          let width = progress * 180;
          this.progressImg && (this.progressImg.width = width);
      }
  }

  class ResourceMgr {
      static preLoadRes(resRootPath, resList) {
          resList.forEach(function (cfgUrl) {
              ResourceMgr.urls.push(ResourceMgr.get3dPrefabUrl(resRootPath, cfgUrl));
          });
      }
      static preCreateRes() {
          if (this.urls.length > 0) {
              Laya.loader.create(this.urls, Laya.Handler.create(this, () => {
                  Laya.stage.event(EventConst.LOADRES_SUCCESS);
              }));
          }
          else {
              Laya.stage.event(EventConst.LOADRES_SUCCESS);
          }
      }
      static get3dPrefabUrl(resRootPath, filename) {
          return resRootPath + filename + ".lh";
      }
      static loadSprite3d(resRootPath, url) {
          return Laya.loader.getRes(this.get3dPrefabUrl(resRootPath, url));
      }
      static getSprite3d(original) {
          var fc_sign = original.url;
          var self = this;
          return Laya.Pool.getItemByCreateFun(fc_sign, function () {
              var sprite3d = Laya.Sprite3D.instantiate(original, null, false, new Laya.Vector3(0, 0, 0));
              sprite3d["_fc_sign"] = fc_sign;
              return sprite3d;
          });
      }
      static recoverSprite3d(sprite3d) {
          sprite3d.removeSelf();
          if (sprite3d["_fc_sign"] == null) {
              return;
          }
          Laya.Pool.recover(sprite3d["_fc_sign"], sprite3d);
      }
      static load3dResAsyn(resPath, onCompleted, onProgress) {
          Laya.loader.create(resPath, onCompleted, onProgress);
      }
      static loadSprite3dAsyn(resRootPath, resName, onCompleted, onProgress) {
          if (resName instanceof Array) {
              var spriteUrls = [];
              for (let index = 0; index < resName.length; index++) {
                  spriteUrls.push(this.get3dPrefabUrl(resRootPath, resName[index]));
              }
              this.load3dResAsyn(spriteUrls, onCompleted, onProgress);
          }
          else {
              var spriteUrl = this.get3dPrefabUrl(resRootPath, resName);
              this.load3dResAsyn(spriteUrl, onCompleted, onProgress);
          }
      }
  }
  ResourceMgr.threeDResRootPath2 = "3dres2/Conventional/";
  ResourceMgr.threeDResRootPath3 = "3dres3/Conventional/";
  ResourceMgr.res3List = [];
  ResourceMgr.res2List = [];
  ResourceMgr.efEesList = [
      'ef_blast_01'
  ];
  ResourceMgr.urls = [];

  class CameraCtr extends Laya.Script3D {
      constructor() {
          super();
          this.dir = new Laya.Vector3();
          this.isMouseDown = false;
          this.lastMouseX = 0;
          this.lastMouseY = 0;
          this.touchId = -1;
          this.movex = 0;
          this.camera = null;
          this.cameraApe = null;
          this.cameraTransform = null;
          this.defaultCamPosition = new Laya.Vector3();
          this.defaultCamRotation = new Laya.Vector3();
          this.curCamPosition = new Laya.Vector3();
          this.curCamRotation = new Laya.Vector3();
          this.offsetRotation = new Laya.Vector3();
          this.offsetPosition = new Laya.Vector3();
          this.animator = null;
          this.nodeAnimator = null;
          this.directionLight = null;
          this.isHomeing = false;
      }
      initCameraByPlayer(player) {
          this.target = player;
          if (!player)
              return;
          this.cameraApe.transform.position = this.defaultCamPosition.clone();
          this.defaultCamPosition.cloneTo(this.curCamPosition);
          Laya.Vector3.subtract(this.curCamPosition, player.mtransform.position, this.offsetPosition);
          this.cameraApe.transform.rotationEuler = this.defaultCamRotation.clone();
          this.defaultCamRotation.cloneTo(this.curCamRotation);
          Laya.Vector3.subtract(this.curCamRotation, player.mtransform.rotationEuler, this.offsetRotation);
      }
      resetCamera() {
          if (fc.platform.PlatformConfig.platformTag != 'iptv') {
              this.cameraTransform.position = this.defaultCamPosition.clone();
          }
          else {
              this.cameraTransform.position = new Laya.Vector3(0, 0, -5);
          }
          this.cameraTransform.rotationEuler = this.defaultCamRotation.clone();
      }
      updateCameraByPlayer() {
      }
      updateRotation(isTurnAngle) {
          let localRotationEulerY = -isTurnAngle;
          Laya.Tween.to(this.cameraApe.transform, {
              localRotationEulerY: localRotationEulerY
          }, 500, null, null);
      }
      world2ScreenPos(worldPos, screenPos) {
          let screenPoint = new Laya.Vector4();
          this.camera.viewport.project(worldPos, this.camera.projectionViewMatrix, screenPoint);
          screenPos.x = screenPoint.x / Laya.stage.clientScaleX;
          screenPos.y = screenPoint.y / Laya.stage.clientScaleY;
      }
      camera2PlayerDir() {
          if (this.target) {
              return Util.vectorDirection(this.camera.transform.position, this.target.mtransform.position);
          }
          else {
              return Util.FORWARD;
          }
      }
      onAwake() {
          this.cameraApe = this.owner;
          this.camera = this.cameraApe.getChildByName("Main Camera");
          this.animator = this.camera.getComponent(Laya.Animator);
          this.animator && (this.animator.speed = 0);
          this.nodeAnimator = this.cameraApe.getComponent(Laya.Animator);
          this.nodeAnimator && (this.nodeAnimator.speed = 0);
          this.cameraTransform = this.owner.transform;
          this.cameraApe.transform.position.cloneTo(this.defaultCamPosition);
          this.defaultCamPosition.cloneTo(this.curCamPosition);
          this.cameraApe.transform.rotationEuler.cloneTo(this.defaultCamRotation);
          this.defaultCamRotation.cloneTo(this.curCamRotation);
          this.directionLight = this.cameraApe.getChildByName("Directional Light");
          this.directionLight.shadowMode = Laya.ShadowMode.None;
          this.directionLight.shadowDistance = 20;
          this.directionLight.shadowResolution = 1024;
          this.directionLight.shadowCascadesMode = Laya.ShadowCascadesMode.NoCascades;
          this.directionLight.shadowNormalBias = 4;
      }
      setShadowMode(mode) {
      }
      onEnable() {
          Laya.stage.on(Laya.Event.MOUSE_DOWN, this, this.onTouchStart);
          Laya.stage.on(Laya.Event.MOUSE_MOVE, this, this.onTouchMove);
          Laya.stage.on(Laya.Event.MOUSE_UP, this, this.onTouchEnd);
          Laya.stage.on(Laya.Event.MOUSE_OUT, this, this.onTouchEnd);
      }
      onStart() {
          this.width = Laya.stage.width * Laya.stage.scaleX;
      }
      onDisable() {
          Laya.stage.off(Laya.Event.MOUSE_DOWN, this, this.onTouchStart);
          Laya.stage.off(Laya.Event.MOUSE_MOVE, this, this.onTouchMove);
          Laya.stage.off(Laya.Event.MOUSE_UP, this, this.onTouchEnd);
          Laya.stage.off(Laya.Event.MOUSE_OUT, this, this.onTouchEnd);
      }
      onDestroy() {
          this.camera = null;
          this.cameraApe = null;
      }
      onAnimStateEnter(stateName, animator) {
      }
      onAnimStateUpdate(stateName, animator) {
      }
      onAnimStateExit(stateName, animator) {
          switch (stateName) {
              case "cameragame":
                  {
                      Laya.stage.event(EventConst.Battle_Start);
                      this.animator.speed = 0;
                  }
                  break;
              case "camerahome":
                  {
                      this.animator.speed = 0;
                  }
                  break;
              default:
                  break;
          }
      }
      playHomeAni() {
          this.animator && (this.play("camerahome"));
          this.isHomeing = true;
          this.resetCamera();
      }
      playGameAni() {
          this.isHomeing = false;
          this.animator && (this.play("cameragame"));
          Laya.stage.timerOnce(500, this, () => {
              Laya.stage.event(EventConst.Battle_Start);
          }, null, false);
      }
      play(newName) {
          if (!this.animator || !newName)
              return;
          this.animator.play(newName, 0, 0);
          this.curClipName = newName;
          this.animator.speed = 1;
      }
      onTouchStart(e) {
          if (this.isMouseDown)
              return;
          this.isMouseDown = true;
          this.lastMouseX = Laya.stage.mouseX;
          this.lastMouseY = Laya.stage.mouseY;
      }
      onTouchMove(e) {
          let offsetX = Laya.stage.mouseX - this.lastMouseX;
          if (this.movex == 0) {
              this.movex = offsetX;
          }
          else {
              if (offsetX > 0 && offsetX < 10 && this.movex < 0) {
                  console.log('反向1');
                  return;
              }
              else if (offsetX < 0 && offsetX > -10 && this.movex > 0) {
                  console.log('反向2');
                  return;
              }
              this.movex = offsetX;
          }
          let moveX = offsetX * 20 / this.width;
          this.lastMouseX = Laya.stage.mouseX;
          this.lastMouseY = Laya.stage.mouseY;
          this.dir.setValue(moveX, 0, 0);
          this.touchHandlerX(this.dir);
      }
      onTouchEnd(e) {
          if (!this.isMouseDown)
              return;
          this.isMouseDown = false;
          this.touchId = -1;
          this.dir.setValue(0, 0, 0);
          this.touchHandlerX(this.dir);
      }
      touchHandlerX(dir) {
          let offsetX = Util.clamp(dir.x, -0.25, 0.25);
          let z = this.cameraTransform.localPositionZ - offsetX;
          z = Math.max(-9, Math.min(12, z));
          this.cameraTransform.localPositionZ = z;
      }
      keyHandlerX(dir) {
          let offsetX = dir.x;
          let z = this.cameraTransform.localPositionZ - offsetX;
          z = Math.max(-9, Math.min(12, z));
          Laya.Tween.clearAll(this.cameraTransform);
          Laya.Tween.to(this.cameraTransform, { localPositionZ: z }, 500, null, null);
      }
      setPositionZ(z) {
          z = Math.max(-9, Math.min(12, z));
          Laya.Tween.clearAll(this.cameraTransform);
          Laya.Tween.to(this.cameraTransform, { localPositionZ: z }, 500, null, null);
      }
  }

  class AnimatorController extends Laya.Script {
      onAwake() {
          this.animator = this.owner.getComponent(Laya.Animator);
          this.animator.speed = 0;
      }
      onStart() {
      }
      onEnable() {
          super.onEnable();
      }
      onDisable() {
          super.onDestroy();
      }
      onAnimEvt(...args) {
      }
      onAnimStateEnter(stateName, animator) {
      }
      onAnimStateUpdate(stateName, animator) {
      }
      onAnimStateExit(stateName, animator) {
      }
      onDestroy() {
          this.animator = null;
          this.character = null;
          this.curClipName = null;
      }
      stop() { }
      play(newName) {
          if (!this.animator || !newName || this.curClipName == newName)
              return;
          if (!this.character)
              return;
          console.log(newName);
          this.animator.play(newName, 0, 0);
          this.curClipName = newName;
          this.animator.speed = 1;
      }
  }
  AnimatorController.atk = "attack";
  AnimatorController.atked = "atked";
  AnimatorController.run = "running";
  AnimatorController.die = "death";
  AnimatorController.win = "win";
  AnimatorController.idle = "idle";

  class BaseScript extends Laya.Script3D {
      constructor() {
          super();
      }
      onAwake() {
          super.onAwake();
          this.sprite3d = this.owner;
          this.mtransform = this.sprite3d.transform;
      }
      initBase() {
          this.sprite3d = this.owner;
      }
      set position(pos) {
          this.sprite3d.transform.position = pos;
      }
      get position() {
          return this.sprite3d.transform.position;
      }
      set localPosition(pos) {
          this.sprite3d.transform.localPosition = pos;
      }
      get localPosition() {
          return this.sprite3d.transform.localPosition;
      }
      set localPositionX(val) {
          this.sprite3d.transform.localPositionX = val;
      }
      get localPositionX() {
          return this.sprite3d.transform.localPositionX;
      }
      set localPositionY(val) {
          this.sprite3d.transform.localPositionY = val;
      }
      get localPositionY() {
          return this.sprite3d.transform.localPositionY;
      }
      set localPositionZ(val) {
          this.sprite3d.transform.localPositionZ = val;
      }
      get localPositionZ() {
          return this.sprite3d.transform.localPositionZ;
      }
      translate(translation, isLocal) {
          this.sprite3d.transform.translate(translation, isLocal);
      }
      setRotate(rot, isLocal, isRadian) {
          this.sprite3d.transform.rotate(rot, isLocal, isRadian);
      }
      get transform() {
          return this.sprite3d.transform;
      }
      set rotationEuler(euler) {
          this.sprite3d.transform.rotationEuler = euler;
      }
      get rotationEuler() {
          return this.sprite3d.transform.rotationEuler;
      }
      get rotation() {
          return this.sprite3d.transform.rotation;
      }
      set rotation(rot) {
          this.sprite3d.transform.rotation = rot;
      }
      setScale(scale) {
          this.sprite3d.transform.setWorldLossyScale(scale);
      }
      getScale() {
          return this.sprite3d.transform.getWorldLossyScale();
      }
      getObj() {
          return this.sprite3d;
      }
      show() {
          this.sprite3d.active = true;
      }
      hide() {
          this.sprite3d.active = false;
      }
      get active() {
          return this.sprite3d.active;
      }
      set active(active) {
          this.sprite3d.active = active;
      }
      destroy() {
          super.destroy();
      }
  }

  var GameFrame = fc.framework.GameFrame;
  class GameObject extends BaseScript {
      constructor() {
          super(...arguments);
          this.isPause = true;
          this.dir = new Laya.Vector3();
          this.moveDir = new Laya.Vector3(0, 0, 0);
      }
      setScene3d(scene3d) {
          this.scene3d = scene3d;
      }
      onAwake() {
          super.onAwake();
          this.sprite3d = this.owner;
          this.mtransform = this.sprite3d.transform;
      }
      onEnable() {
          super.onEnable();
          this.initEvent();
      }
      onDisable() {
          super.onDisable();
          this.removeEvent();
      }
      onStart() {
          super.onStart();
          this.setScene3d(GameFrame.scene3d);
      }
      initEvent() {
      }
      removeEvent() {
      }
      setColliderEnable(isEnable) {
          if (!this.owner)
              return;
          this.owner.frameOnce(1, this, () => {
              this.rigidbody3d && (this.rigidbody3d.enabled = isEnable);
              this.collider3d && (this.collider3d.enabled = isEnable);
          });
      }
      activeSprite(isEnable) {
      }
      on3dTriggerEnter(other, self) {
      }
      on3dTriggerExit(other, self) {
      }
      on3dTriggerStay(other, self) {
      }
      fcCollisionEnter(script, collision) {
          console.log('碰撞到', collision.other.owner.name);
      }
      fcCollisionStay(script, collision) {
      }
      fcCollisionExit(script, collision) {
      }
  }

  class CollisionScript3d extends Laya.Script3D {
      constructor() {
          super();
          this.listener = null;
      }
      static Init(sprite3d, listener) {
          var trigger = sprite3d.getComponent(CollisionScript3d);
          if (!trigger) {
              trigger = sprite3d.addComponent(CollisionScript3d);
          }
          trigger.listener = listener;
      }
      onCollisionEnter(collision) {
          if (this.listener != null && this.listener.fcCollisionEnter) {
              this.listener.fcCollisionEnter(this, collision);
          }
      }
      onCollisionExit(collision) {
          if (this.listener != null && this.listener.fcCollisionExit) {
              this.listener.fcCollisionExit(this, collision);
          }
      }
      onCollisionStay(collision) {
          if (this.listener != null && this.listener.fcCollisionStay) {
              this.listener.fcCollisionStay(this, collision);
          }
      }
      onDestroy() {
          this.listener = null;
      }
  }

  class CollisionConst {
  }
  CollisionConst.GROUP_BULLET = Laya.Physics3DUtils.COLLISIONFILTERGROUP_CHARACTERFILTER;
  CollisionConst.GROUP_Soldier = Laya.Physics3DUtils.COLLISIONFILTERGROUP_CUSTOMFILTER1;
  CollisionConst.GROUP_RAY = Laya.Physics3DUtils.COLLISIONFILTERGROUP_CUSTOMFILTER2;
  CollisionConst.GROUP_Wall = Laya.Physics3DUtils.COLLISIONFILTERGROUP_CUSTOMFILTER3;
  CollisionConst.GROUP_Aider = Laya.Physics3DUtils.COLLISIONFILTERGROUP_CUSTOMFILTER5;
  CollisionConst.CAN_BULLET = CollisionConst.GROUP_Soldier | CollisionConst.GROUP_Wall | CollisionConst.GROUP_Aider;
  CollisionConst.CAN_Soldier = CollisionConst.GROUP_RAY | CollisionConst.GROUP_BULLET;
  CollisionConst.CAN_Wall = CollisionConst.GROUP_BULLET;
  CollisionConst.CAN_Aider = CollisionConst.GROUP_BULLET | CollisionConst.GROUP_RAY;
  CollisionConst.CAN_RAY = Laya.Physics3DUtils.COLLISIONFILTERGROUP_ALLFILTER ^ CollisionConst.GROUP_BULLET ^ CollisionConst.GROUP_Wall;

  var TriggerScript3d = fc.plugin.Trigger3dListener;
  class Character extends GameObject {
      constructor() {
          super(...arguments);
          this.healthBar = null;
          this.teamType = TeamType.Neutral;
          this.atkIntval = 0;
          this.atkIntvalTemp = 0;
          this.mHp = 0;
          this.mTotalHp = 0;
          this.atkDamage = 0;
          this.moveSpeed = 4;
          this.atkRang = 20;
          this.shotRang = 30;
          this.shotSpeed = 20;
          this.atkradius = 2;
          this.damageRang = 2;
          this.atkedInterval = 0;
          this.isMoveing = false;
          this.isColliderEnmey = false;
          this.atkedTarget = null;
          this.hpPoint = null;
          this.isAtkedLock = false;
          this.nickName = "游客" + Util.random(10000, 9000000);
          this.isTouching = false;
          this.movevelocity = new Laya.Vector3;
      }
      getTouch() {
          return this.isTouching;
      }
      setTouch(val) {
          this.isTouching = val;
      }
      onAwake() {
          this.sprite3d = this.owner;
          this.mtransform = this.sprite3d.transform;
      }
      onStart() {
          super.onStart();
          this.rigidbody3d = this.sprite3d.getComponent(Laya.Rigidbody3D);
          this.collider3d = this.sprite3d.getComponent(Laya.PhysicsCollider);
          if (this.rigidbody3d) {
              TriggerScript3d.add(this.rigidbody3d, this);
              CollisionScript3d.Init(this.sprite3d, this);
          }
          else if (this.collider3d) {
              TriggerScript3d.add(this.collider3d, this);
          }
          this.animatorController = this.owner.getComponent(AnimatorController);
      }
      updateLv() { }
      reset() { }
      onRun() {
      }
      onIdle() {
      }
      getBoxCount() {
          return 0;
      }
      initEvent() {
          Laya.stage.on(Character.EVENT_PAUSE, this, this.onPause);
          Laya.stage.on(Character.EVENT_RESUME, this, this.onResume);
      }
      removeEvent() {
          Laya.stage.off(Character.EVENT_PAUSE, this, this.onPause);
          Laya.stage.off(Character.EVENT_RESUME, this, this.onResume);
      }
      onResume() {
          this.isPause = false;
      }
      onPause() {
          this.isPause = true;
      }
      onDie() {
          this.onPause();
      }
      onWin() {
          this.onPause();
      }
      onAtk(atkedObj) {
      }
      onDamage(damage) {
      }
      on3dTriggerEnter(other, self) {
          super.on3dTriggerEnter(other, self);
      }
      on3dTriggerExit(other, self) {
          super.on3dTriggerExit(other, self);
      }
      fcCollisionEnter(script, collision) {
          super.fcCollisionEnter(script, collision);
      }
      playIdleAni() {
      }
      playMageAni() {
      }
      playShotAni() {
      }
      playUpgradeAni() {
      }
      playCollectAni() {
      }
      playAtkAni() {
      }
      playRunAni() {
      }
      playWinAni() {
      }
      playDieAni() {
      }
      setPath(path) {
          this.mtransform.position = path.clone();
      }
      setRotation(angle) {
          let rotation = new Laya.Vector3(this.mtransform.rotationEuler.x, angle, this.mtransform.rotationEuler.z);
          this.mtransform.rotationEuler = rotation.clone();
          this.mtransform.getForward(this.dir);
      }
      setRotationTo(angleDelta) {
          let rotation = new Laya.Vector3();
          Laya.Vector3.add(this.mtransform.rotationEuler, angleDelta, rotation);
          this.mtransform.rotationEuler = rotation.clone();
          this.mtransform.getForward(this.dir);
      }
      isLife() {
          return !this.isPause && this.mHp > 0;
      }
      move(dir) {
      }
      getNearEnemyOfAtkRang(character = null) {
          return null;
      }
      getNearEnemyOfRang(character = null) {
          return null;
      }
      getRandomEnemyOfRang() {
          return;
      }
      getEnemiesOfRang(rang) {
          return [];
      }
      getNearObjectOfRang() {
          return;
      }
      getRandomObjectOfRang() {
          return;
      }
      rotationRole(angle) {
      }
      isInAtkRang(character) {
          if (!character)
              return false;
          let newDisSqr = Laya.Vector3.distanceSquared(character.mtransform.position, this.mtransform.position);
          let rang = Math.max(this.atkRang, character.atkRang);
          if (character.nickName == 'BiaoQiangBing' || this.nickName == 'BiaoQiangBing') {
              rang = this.atkRang;
          }
          return (newDisSqr <= (rang) * (rang));
      }
      collectBrick(brick3d) {
      }
      rayCheckWall(dir, rang) {
          if (!dir || dir == null) {
              return null;
          }
          var from = new Laya.Vector3(0, 0, 0);
          Laya.Vector3.normalize(dir, from);
          var to = new Laya.Vector3(0, 0, 0);
          var raylenght = new Laya.Vector3(0, 0, 0);
          Laya.Vector3.scale(from, -rang, raylenght);
          Laya.Vector3.add(this.mtransform.position.clone(), raylenght, to);
          this.mtransform.position.cloneTo(from);
          from.y = 1;
          to.y = 1;
          var hitResults = [];
          var boxCollider = new Laya.BoxColliderShape(0.5, 0.5, 0.5);
          this.scene3d.physicsSimulation.shapeCastAll(boxCollider, from, to, hitResults, null, null, CollisionConst.GROUP_RAY, CollisionConst.CAN_RAY);
          var lineSprite = this.scene3d.addChild(new Laya.PixelLineSprite3D(1));
          lineSprite.addLine(from, to, Laya.Color.RED, Laya.Color.RED);
          for (let i = 0, n = hitResults.length; i < n; i++) {
              console.log('射线检测到的：', hitResults[i].collider.owner.name, '碰撞法线：', hitResults[i].normal);
              var otherSprite3d = hitResults[i].collider.owner;
              if (otherSprite3d && otherSprite3d.active) {
                  var character = otherSprite3d.getComponent(Character);
                  if (!character || character != this) {
                      return hitResults[i];
                  }
              }
          }
          return null;
      }
      intersectsWall(dir, rang, targetObj) {
          if (!dir || dir == null) {
              return false;
          }
          if (!targetObj.meshRenderer)
              return false;
          var from = new Laya.Vector3(0, 0, 0);
          Laya.Vector3.normalize(dir, from);
          var to = new Laya.Vector3(0, 0, 0);
          var raylenght = new Laya.Vector3(0, 0, 0);
          Laya.Vector3.scale(from, -rang, raylenght);
          Laya.Vector3.add(this.mtransform.position.clone(), raylenght, to);
          this.mtransform.position.cloneTo(from);
          from.y = 1;
          to.y = 1;
          console.log(targetObj.name);
          let boundBox = targetObj.meshRenderer.bounds._boundBox;
          let intersects = Util.judgeIntersect(from.x, from.z, to.x, to.z, boundBox.max.x, boundBox.max.z, boundBox.min.x, boundBox.min.z);
          return intersects;
      }
  }
  Character.EVENT_PAUSE = "pause";
  Character.EVENT_RESUME = "resume";
  Character.EVENT_REVIVE = "relive";
  Character.EVENT_DIE = "die";
  Character.EVENT_ATKED = "atked";
  Character.EVENT_CHECKED = "checked";
  Character.EVENT_CHANGESTATE = "changestate";
  Character.EVENT_FIRE = "fire";

  var Resource = fc.framework.Resource;
  var ObjectPool = fc.plugin.ObjectPool;
  class EffectMgr {
      constructor() {
          this.effectList = [];
          this.spriteEffectList = [];
          this.spriteOtherEfList = [];
      }
      static Instance() {
          if (this.instance == null)
              this.instance = new EffectMgr();
          return this.instance;
      }
      init(parent) {
          this.parent = parent;
      }
      createSpriteEffect(name, isSceneRoot = true) {
          let original = Resource.loadSprite3d(name);
          let clone3d = ObjectPool.getSprite3d(original);
          clone3d.name = name;
          if (isSceneRoot && this.parent) {
              this.parent.addChild(clone3d);
          }
          clone3d.active = true;
          let index = this.spriteEffectList.indexOf(clone3d);
          if (index == -1)
              this.spriteEffectList[this.spriteEffectList.length] = clone3d;
          return clone3d;
      }
      removeSpriteEffect(ape, isRecover = true) {
          let index = this.spriteEffectList.indexOf(ape);
          if (index != -1)
              this.spriteEffectList.splice(index, 1);
          if (isRecover) {
              ape.active = false;
              ObjectPool.recoverSprite3d(ape);
          }
          else {
              ape.removeSelf();
              ape.destroy(true);
          }
      }
      createSpriteEf2(resRootPath, name, isSceneRoot = true) {
          let original = ResourceMgr.loadSprite3d(resRootPath, name);
          if (original == null)
              return null;
          let clone3d = ResourceMgr.getSprite3d(original);
          if (clone3d == null)
              return null;
          clone3d.name = name;
          if (isSceneRoot && this.parent) {
              this.parent.addChild(clone3d);
          }
          clone3d.active = true;
          let index = this.spriteOtherEfList.indexOf(clone3d);
          if (index == -1)
              this.spriteOtherEfList[this.spriteOtherEfList.length] = clone3d;
          return clone3d;
      }
      removeSpriteE2(ape, isRecover = true) {
          let index = this.spriteOtherEfList.indexOf(ape);
          if (index != -1)
              this.spriteOtherEfList.splice(index, 1);
          if (isRecover) {
              ape.active = false;
              ResourceMgr.recoverSprite3d(ape);
          }
          else {
              ape.removeSelf();
              ape.destroy(true);
          }
      }
      addEffect(effect) {
          if (!effect)
              return;
          let index = this.effectList.indexOf(effect);
          if (index == -1)
              this.effectList[this.effectList.length] = effect;
      }
      removeEffect(effect) {
          let index = this.effectList.indexOf(effect);
          if (index != -1)
              this.effectList.splice(index, 1);
          let sprite3d = effect.sprite3d;
          sprite3d.removeSelf();
          sprite3d.destroy(true);
          effect.destroy();
      }
      onDestroy() {
          if (this.effectList.length > 0) {
              for (let i = this.effectList.length - 1; i >= 0; i--) {
                  const effect = this.effectList.pop();
                  let sprite3d = effect.sprite3d;
                  effect.destroy();
                  sprite3d.active = false;
                  sprite3d.removeSelf();
                  sprite3d.destroy(true);
              }
              this.effectList = [];
          }
          if (this.spriteEffectList.length > 0) {
              for (let i = this.spriteEffectList.length - 1; i >= 0; i--) {
                  const effect = this.spriteEffectList.pop();
                  ObjectPool.recoverSprite3d(effect);
              }
              this.spriteEffectList = [];
          }
          if (this.spriteOtherEfList.length > 0) {
              for (let i = this.spriteOtherEfList.length - 1; i >= 0; i--) {
                  const effect = this.spriteOtherEfList.pop();
                  ResourceMgr.recoverSprite3d(effect);
              }
              this.spriteOtherEfList = [];
          }
      }
  }
  EffectMgr.instance = null;

  var Resource$1 = fc.framework.Resource;
  var ObjectPool$1 = fc.plugin.ObjectPool;
  class BulletMgr {
      constructor() {
          this.bulletList = [];
      }
      static Instance() {
          if (this.instance == null)
              this.instance = new BulletMgr();
          return this.instance;
      }
      init(parent) {
          this.parent = parent;
      }
      createBullet(name) {
          var original = Resource$1.loadSprite3d(name);
          let clone3d = ObjectPool$1.getSprite3d(original);
          clone3d.name = name;
          clone3d.active = true;
          this.parent.addChild(clone3d);
          return clone3d;
      }
      addBullet(bullet) {
          if (!bullet)
              return;
          let index = this.bulletList.indexOf(bullet);
          if (index == -1)
              this.bulletList[this.bulletList.length] = bullet;
      }
      getBulletList() {
          return this.bulletList;
      }
      removeBullet(bullet, isRecover = true) {
          let index = this.bulletList.indexOf(bullet);
          if (index != -1)
              this.bulletList.splice(index, 1);
          let bullet3d = bullet.bullet3d;
          bullet.destroy();
          if (isRecover) {
              bullet3d.active = false;
              ObjectPool$1.recoverSprite3d(bullet3d);
          }
          else {
              bullet3d.removeSelf();
              bullet3d.destroy(true);
          }
      }
      onDestroy() {
          if (this.bulletList.length > 0) {
              for (let i = this.bulletList.length - 1; i >= 0; i--) {
                  const bullet = this.bulletList.pop();
                  let sprite3d = bullet.bullet3d;
                  bullet.destroy();
                  sprite3d.active = false;
                  sprite3d.removeSelf();
                  sprite3d.destroy(true);
              }
              this.bulletList = [];
          }
      }
  }
  BulletMgr.instance = null;

  var Resource$2 = fc.framework.Resource;
  var ObjectPool$2 = fc.plugin.ObjectPool;
  class GoldMgr {
      constructor() {
          this.goldList = [];
      }
      static Instance() {
          if (this.instance == null)
              this.instance = new GoldMgr();
          return this.instance;
      }
      init(parent) {
          this.parent = parent;
      }
      createGold(name) {
          let original = Resource$2.loadSprite3d(name);
          let clone3d = ObjectPool$2.getSprite3d(original);
          clone3d.name = name;
          this.parent.addChild(clone3d);
          clone3d.active = true;
          let index = this.goldList.indexOf(clone3d);
          if (index == -1)
              this.goldList[this.goldList.length] = clone3d;
          return clone3d;
      }
      removeGold(ape) {
          let index = this.goldList.indexOf(ape);
          if (index != -1)
              this.goldList.splice(index, 1);
          ape.active = false;
          ObjectPool$2.recoverSprite3d(ape);
      }
      onDestroy() {
          if (this.goldList.length > 0) {
              for (let i = this.goldList.length - 1; i >= 0; i--) {
                  const effect = this.goldList.pop();
                  ObjectPool$2.recoverSprite3d(effect);
              }
              this.goldList = [];
          }
      }
      getGoldList() {
          return this.goldList;
      }
      addGoldsY(count, name, radius, diePosition) {
          var angleRate = Math.floor(360 / count);
          angleRate = angleRate * Math.PI / 180;
          for (let i = 0; i < count; i++) {
              const gold = this.createGold(name);
              gold.transform.position = diePosition.clone();
              let offsetX = Math.sin(angleRate * i) * radius;
              let offsetZ = Math.cos(angleRate * i) * radius;
              let x = gold.transform.position.x + offsetX;
              let y = gold.transform.position.y + 2;
              let yy = gold.transform.position.y + 0.5;
              let z = gold.transform.position.z + offsetZ;
              Laya.Tween.to(gold.transform, {
                  localPositionX: x - offsetX * 0.5, localPositionY: y, localPositionZ: z - offsetZ * 0.5
              }, 150, Laya.Ease.strongIn, Laya.Handler.create(this, () => {
                  Laya.Tween.to(gold.transform, {
                      localPositionX: x, localPositionY: yy, localPositionZ: z
                  }, 150, Laya.Ease.strongIn, null);
              }));
          }
      }
      addGoldsZ(parent, count, name, radius, centerPosition) {
          var angleRate = Math.floor(360 / count);
          angleRate = angleRate * Math.PI / 180;
          for (let i = 0; i < count; i++) {
              const gold = this.createGold(name);
              parent.addChild(gold);
              if (gold && gold.getComponent(Laya.PhysicsCollider)) {
                  let collider = gold.getComponent(Laya.PhysicsCollider);
                  collider.enabled = false;
              }
              gold.transform.position = centerPosition.clone();
              let offsetX = Math.sin(angleRate * i) * radius;
              let offsetY = Math.cos(angleRate * i) * radius;
              let x = centerPosition.x + offsetX;
              let y = centerPosition.y + offsetY;
              gold.transform.localPosition = new Laya.Vector3(x, y, centerPosition.z);
              Laya.Tween.to(gold.transform, {
                  localPositionX: x, localPositionY: y
              }, 200, Laya.Ease.strongIn, null);
          }
      }
  }
  GoldMgr.instance = null;

  class AudioManager {
      constructor() {
          this.playSfxing = false;
      }
      static get Instance() {
          if (AudioManager.instance == null)
              AudioManager.instance = new AudioManager();
          return AudioManager.instance;
      }
      stopMusic() {
          fc.framework.Device.stopMusic();
      }
      playSound(res, loop = false) {
          if (fc.platform.PlatformConfig.platformTag == 'iptv' || fc.platform.PlatformConfig.platformTag == 'native') {
              res = res.replace('.mp3', '.ogg');
          }
          fc.framework.Audio.playSound(res, loop);
      }
      stopSound(res) {
          if (fc.platform.PlatformConfig.platformTag == 'iptv' || fc.platform.PlatformConfig.platformTag == 'native') {
              res = res.replace('.mp3', '.ogg');
          }
          fc.framework.Audio.stopSound(res);
      }
      stopSfx(sfx) {
          if (fc.platform.PlatformConfig.platformTag == 'iptv' || fc.platform.PlatformConfig.platformTag == 'native') {
              sfx = sfx.replace('.mp3', '.ogg');
          }
          fc.framework.Audio.stopSound(AudioManager.AudioUrl + sfx);
      }
      playSfx(sfx, loop = false) {
          if (fc.platform.PlatformConfig.platformTag == 'iptv' || fc.platform.PlatformConfig.platformTag == 'native') {
              sfx = sfx.replace('.mp3', '.ogg');
          }
          fc.framework.Audio.playSound(AudioManager.AudioUrl + sfx, loop);
      }
      playSfx2(sfx, loop = false) {
          if (fc.platform.PlatformConfig.platformTag == 'iptv' || fc.platform.PlatformConfig.platformTag == 'native') {
              sfx = sfx.replace('.mp3', '.ogg');
          }
          if (!this.playSfxing) {
              fc.framework.Audio.playSound(AudioManager.AudioUrl + sfx, loop);
              this.playSfxing = true;
              Laya.stage.timerOnce(50, this, () => {
                  this.playSfxing = false;
              }, null, false);
          }
      }
  }
  AudioManager.AudioUrl = "sound/";

  var GameFrame$1 = fc.framework.GameFrame;
  class CastlePageUI extends fc.plugin.BaseView {
      constructor() {
          super();
      }
      onAwake() {
          super.onAwake();
          this.upgradeBox = this.owner.getChildByName('middleUI').getChildByName('upgradeBox');
          this.closeBtn = this.upgradeBox.getChildByName('closeBtn');
          this.closeBtn.on(Laya.Event.CLICK, this, this.closeView);
          this.mWall = this.upgradeBox.getChildByName('Wall');
          this.mArcher = this.upgradeBox.getChildByName('Archer');
          this.mTower = this.upgradeBox.getChildByName('Tower');
          this.mCatapult = this.upgradeBox.getChildByName('Catapult');
          this.mXianFeng = this.upgradeBox.getChildByName('XianFeng');
          this.mTower.visible = false;
          this.mCatapult.visible = false;
          this.mXianFeng.visible = false;
          this.mTower.getChildByName('upgradeBtn').visible = false;
          this.mCatapult.getChildByName('upgradeBtn').visible = false;
          this.mXianFeng.getChildByName('upgradeBtn').visible = false;
      }
      closeView() {
          GameFrame$1.closeView(this.owner);
          GameFrame$1.playClickSound();
      }
      onStart() {
          super.onStart();
          this.owner.zOrder = 2;
          this.initWall();
          this.initArcher();
          this.initTower();
          this.initCatapult();
          this.initXianFeng();
          this.refeshList();
      }
      refeshList() {
          let lv = Math.max(1, GameData.data.getCastleLv(CastleCfg.Wall));
          if (lv >= 4) {
              this.mTower.visible = true;
              this.mTower.getChildByName('upgradeBtn').visible = true;
          }
          if (lv >= 24) {
              this.mCatapult.visible = true;
              this.mCatapult.getChildByName('upgradeBtn').visible = true;
          }
          let xflv = Math.max(0, GameData.data.getWarriorLv(WarriorType.XianFeng));
          if (xflv >= 1) {
              this.mXianFeng.visible = true;
              this.mXianFeng.getChildByName('upgradeBtn').visible = true;
          }
      }
      initWall() {
          let panelProgress = this.mWall.getChildByName('panelProgress');
          let upgradeBtn = this.mWall.getChildByName('upgradeBtn');
          let lvVal = this.mWall.getChildByName('lv');
          let goldVal = this.mWall.getChildByName('goldVal');
          let lv = Math.max(1, GameData.data.getCastleLv(CastleCfg.Wall));
          let data = GameData.data.castleCfg.Wall;
          let gold = data.upgrade + Math.floor(((1 + (lv - 1)) * (lv - 1) * 0.5) * data.upgradeadd);
          goldVal.value = gold.toString();
          panelProgress.width = (lv / data.maxlv) * 215;
          upgradeBtn.on(Laya.Event.CLICK, this, this.upgradeWall);
          lvVal.text = '级别 ' + lv + (lv >= data.maxlv ? ' 最大' : '');
      }
      upgradeWall() {
          Util.delayMouseEnabled(this.mWall, 200);
          GameFrame$1.playClickSound();
          let lv = Math.max(1, GameData.data.getCastleLv(CastleCfg.Wall));
          let data = GameData.data.castleCfg.Wall;
          if (lv >= data.maxlv) {
              return;
          }
          let gold = data.upgrade + Math.floor(((1 + (lv - 1)) * (lv - 1) * 0.5) * data.upgradeadd);
          if (GameData.data.gameGold >= gold) {
              GameData.data.gameGold = -gold;
              Laya.stage.event(fc.define.Event.GOLD_UPDATE);
              GameData.data.setCastleLv(lv + 1, CastleCfg.Wall);
              GameMgr.Instance().updateScenario();
              this.refeshList();
              this.initWall();
          }
          else {
              GameFrame$1.showToastMsg(GameFrame$1['langCfg']['nocointxt']);
          }
      }
      initArcher() {
          let panelProgress = this.mArcher.getChildByName('panelProgress');
          let upgradeBtn = this.mArcher.getChildByName('upgradeBtn');
          let lvVal = this.mArcher.getChildByName('lv');
          let goldVal = this.mArcher.getChildByName('goldVal');
          let lv = Math.max(1, GameData.data.getCastleLv(CastleCfg.Archer));
          let data = GameData.data.castleCfg.Archer;
          let gold = data.upgrade + Math.floor(((1 + (lv - 1)) * (lv - 1) * 0.5) * data.upgradeadd);
          goldVal.value = gold.toString();
          panelProgress.width = (lv / data.maxlv) * 215;
          upgradeBtn.on(Laya.Event.CLICK, this, this.upgradeArcher);
          lvVal.text = '级别 ' + lv + (lv >= data.maxlv ? ' 最大' : '');
      }
      upgradeArcher() {
          Util.delayMouseEnabled(this.mArcher, 200);
          GameFrame$1.playClickSound();
          let lv = Math.max(1, GameData.data.getCastleLv(CastleCfg.Archer));
          let data = GameData.data.castleCfg.Archer;
          if (lv >= data.maxlv) {
              return;
          }
          let gold = data.upgrade + Math.floor(((1 + (lv - 1)) * (lv - 1) * 0.5) * data.upgradeadd);
          if (GameData.data.gameGold >= gold) {
              GameData.data.gameGold = -gold;
              Laya.stage.event(fc.define.Event.GOLD_UPDATE);
              GameData.data.setCastleLv(lv + 1, CastleCfg.Archer);
              GameMgr.Instance().addArcher();
              Laya.stage.event(EventConst.Update_Archer);
              this.initArcher();
          }
          else {
              GameFrame$1.showToastMsg(GameFrame$1['langCfg']['nocointxt']);
          }
      }
      initTower() {
          let panelProgress = this.mTower.getChildByName('panelProgress');
          let upgradeBtn = this.mTower.getChildByName('upgradeBtn');
          let lvVal = this.mTower.getChildByName('lv');
          let goldVal = this.mTower.getChildByName('goldVal');
          let lv = Math.max(1, GameData.data.getCastleLv(CastleCfg.Tower));
          let data = GameData.data.castleCfg.Tower;
          let gold = data.upgrade + Math.floor(((1 + (lv - 1)) * (lv - 1) * 0.5) * data.upgradeadd);
          goldVal.value = gold.toString();
          panelProgress.width = (lv / data.maxlv) * 215;
          upgradeBtn.on(Laya.Event.CLICK, this, this.upgradeTower);
          lvVal.text = '级别 ' + lv + (lv >= data.maxlv ? ' 最大' : '');
      }
      upgradeTower() {
          Util.delayMouseEnabled(this.mTower, 200);
          GameFrame$1.playClickSound();
          let lv = Math.max(1, GameData.data.getCastleLv(CastleCfg.Tower));
          let data = GameData.data.castleCfg.Tower;
          if (lv >= data.maxlv) {
              return;
          }
          let gold = data.upgrade + Math.floor(((1 + (lv - 1)) * (lv - 1) * 0.5) * data.upgradeadd);
          if (GameData.data.gameGold >= gold) {
              GameData.data.gameGold = -gold;
              Laya.stage.event(fc.define.Event.GOLD_UPDATE);
              GameData.data.setCastleLv(lv + 1, CastleCfg.Tower);
              GameMgr.Instance().addTower();
              this.initTower();
          }
          else {
              GameFrame$1.showToastMsg(GameFrame$1['langCfg']['nocointxt']);
          }
      }
      initCatapult() {
          let panelProgress = this.mCatapult.getChildByName('panelProgress');
          let upgradeBtn = this.mCatapult.getChildByName('upgradeBtn');
          let lvVal = this.mCatapult.getChildByName('lv');
          let goldVal = this.mCatapult.getChildByName('goldVal');
          let lv = Math.max(1, GameData.data.getCastleLv(CastleCfg.Catapult));
          let data = GameData.data.castleCfg.Catapult;
          let gold = data.upgrade + Math.floor(((1 + (lv - 1)) * (lv - 1) * 0.5) * data.upgradeadd);
          goldVal.value = gold.toString();
          panelProgress.width = (lv / data.maxlv) * 215;
          upgradeBtn.on(Laya.Event.CLICK, this, this.upgradeCatapult);
          lvVal.text = '级别 ' + lv + (lv >= data.maxlv ? ' 最大' : '');
      }
      upgradeCatapult() {
          Util.delayMouseEnabled(this.mCatapult, 200);
          GameFrame$1.playClickSound();
          let lv = Math.max(1, GameData.data.getCastleLv(CastleCfg.Catapult));
          let data = GameData.data.castleCfg.Catapult;
          if (lv >= data.maxlv) {
              return;
          }
          let gold = data.upgrade + Math.floor(((1 + (lv - 1)) * (lv - 1) * 0.5) * data.upgradeadd);
          if (GameData.data.gameGold >= gold) {
              GameData.data.gameGold = -gold;
              Laya.stage.event(fc.define.Event.GOLD_UPDATE);
              GameData.data.setCastleLv(lv + 1, CastleCfg.Catapult);
              Laya.stage.event(EventConst.Update_Catapult);
              this.initCatapult();
          }
          else {
              GameFrame$1.showToastMsg(GameFrame$1['langCfg']['nocointxt']);
          }
      }
      initXianFeng() {
          let panelProgress = this.mXianFeng.getChildByName('panelProgress');
          let upgradeBtn = this.mXianFeng.getChildByName('upgradeBtn');
          let lvVal = this.mXianFeng.getChildByName('lv');
          let goldVal = this.mXianFeng.getChildByName('goldVal');
          let lv = Math.max(1, GameData.data.getWarriorLv(WarriorType.XianFeng));
          let data = GameMgr.Instance().getSoldierData(WarriorType.XianFeng, lv);
          let gold = data.upgrade;
          goldVal.value = gold.toString();
          panelProgress.width = (lv / data.lv_max) * 215;
          lvVal.text = '级别 ' + lv + (lv >= data.lv_max ? ' 最大' : '');
          upgradeBtn.on(Laya.Event.CLICK, this, this.upgradeXianFeng);
      }
      upgradeXianFeng() {
          Util.delayMouseEnabled(this.mXianFeng, 200);
          GameFrame$1.playClickSound();
          let lv = Math.max(1, GameData.data.getWarriorLv(WarriorType.XianFeng));
          let data = GameMgr.Instance().getSoldierData(WarriorType.XianFeng, lv);
          if (lv >= data.lv_max) {
              return;
          }
          let gold = data.upgrade;
          if (GameData.data.gameGold >= gold) {
              GameData.data.gameGold = -gold;
              Laya.stage.event(fc.define.Event.GOLD_UPDATE);
              GameData.data.setWarriorLv(lv + 1, WarriorType.XianFeng);
              Laya.stage.event(EventConst.Update_XianFeng);
              GameMgr.Instance().addXianFeng();
              this.initXianFeng();
          }
          else {
              GameFrame$1.showToastMsg(GameFrame$1['langCfg']['nocointxt']);
          }
      }
      onDisable() {
          super.onDisable();
      }
      onDestroy() {
          super.onDestroy();
      }
  }

  var GameFrame$2 = fc.framework.GameFrame;
  class WarriorPageUI extends fc.plugin.BaseView {
      constructor() {
          super();
      }
      onAwake() {
          super.onAwake();
          this.upgradeBox = this.owner.getChildByName('middleUI').getChildByName('upgradeBox');
          this.closeBtn = this.upgradeBox.getChildByName('closeBtn');
          this.closeBtn.on(Laya.Event.CLICK, this, this.closeView);
          this.mDaoBing = this.upgradeBox.getChildByName('DaoBing');
          this.mQiangBing = this.upgradeBox.getChildByName('QiangBing');
          this.mBiaoQiangBing = this.upgradeBox.getChildByName('BiaoQiangBing');
          this.mQiBing = this.upgradeBox.getChildByName('QiBing');
          this.mQiangBing.disabled = true;
          this.mBiaoQiangBing.disabled = true;
          this.mQiBing.disabled = true;
      }
      closeView() {
          GameFrame$2.closeView(this.owner);
          GameFrame$2.playClickSound();
      }
      onStart() {
          super.onStart();
          this.owner.zOrder = 2;
          this.initDaoBing(this.mDaoBing, WarriorType.DaoBing);
          this.initDaoBing(this.mQiangBing, WarriorType.QiangBing);
          this.initDaoBing(this.mBiaoQiangBing, WarriorType.BiaoQiangBing);
          this.initDaoBing(this.mQiBing, WarriorType.QiBing);
          this.refeshList();
      }
      refeshList() {
          let warriors = GameData.data.warriors;
          if (warriors >= 2) {
              this.mQiangBing.disabled = false;
          }
          if (warriors >= 3) {
              this.mBiaoQiangBing.disabled = false;
          }
          if (warriors >= 4) {
              this.mQiBing.disabled = false;
          }
      }
      initDaoBing(box, type) {
          let hpProgress = box.getChildByName('attrBox').getChildByName('hp').getChildByName('panelProgress');
          let atkProgress = box.getChildByName('attrBox').getChildByName('atk').getChildByName('panelProgress');
          let defProgress = box.getChildByName('attrBox').getChildByName('def').getChildByName('panelProgress');
          let hpVal = box.getChildByName('attrBox').getChildByName('hp').getChildByName('hpVal');
          let atkVal = box.getChildByName('attrBox').getChildByName('atk').getChildByName('hpVal');
          let defVal = box.getChildByName('attrBox').getChildByName('def').getChildByName('hpVal');
          let upgradeBtn = box.getChildByName('gold');
          let name = box.getChildByName('name');
          let goldVal = box.getChildByName('gold').getChildByName('goldVal');
          let lv = Math.max(1, GameData.data.getWarriorLv(type));
          let data = GameMgr.Instance().getSoldierData(type, lv);
          let gold = data.upgrade;
          goldVal.value = gold.toString();
          hpProgress.width = (lv / data.lv_max) * 108;
          atkProgress.width = (lv / data.lv_max) * 108;
          defProgress.width = (lv / data.lv_max) * 108;
          hpVal.value = data.hp.toString();
          atkVal.value = data.atk.toString();
          defVal.value = data.defense.toString();
          let stage = 1 + Math.floor((lv - 1) / 3);
          stage = Math.min(ResConst.WarriorLvName[type].length, stage);
          name.text = ResConst.WarriorLvName[type][stage - 1];
          for (let index = 0; index < ResConst.WarriorLvName[type].length; index++) {
              let warrior = box.getChildByName('warriorBox').getChildByName('warrior' + index);
              if (index <= stage - 1) {
                  warrior.disabled = false;
                  if (index == stage - 1) {
                      warrior.getChildByName('glow').visible = true;
                      warrior.getChildByName('uppro').width = 29 * (1 + (lv - 1) % 3);
                  }
                  else {
                      warrior.getChildByName('glow').visible = false;
                      warrior.getChildByName('uppro').width = 87;
                  }
              }
              else {
                  warrior.disabled = true;
                  warrior.getChildByName('glow').visible = false;
                  warrior.getChildByName('uppro').width = 0;
              }
          }
          upgradeBtn.on(Laya.Event.CLICK, this, this.upgradeDaoBing, [box, type]);
      }
      upgradeDaoBing(box, type) {
          if (box.disabled)
              return;
          GameFrame$2.playClickSound();
          let lv = Math.max(1, GameData.data.getWarriorLv(type));
          let data = GameMgr.Instance().getSoldierData(type, lv);
          if (lv >= data.lv_max) {
              GameFrame$2.showToastMsg('已经到最大级别');
              return;
          }
          let gold = data.upgrade;
          if (GameData.data.gameGold >= gold) {
              GameData.data.gameGold = -gold;
              Laya.stage.event(fc.define.Event.GOLD_UPDATE);
              GameData.data.setWarriorLv(lv + 1, type);
              this.initDaoBing(box, type);
              let stage = 1 + Math.floor(lv / 3);
              if (type == WarriorType.DaoBing) {
                  if (stage == 2 && lv % 3 == 0) {
                      GameData.data.warriors = 2;
                  }
              }
              if (type == WarriorType.QiangBing) {
                  if (stage == 3 && lv % 3 == 0) {
                      GameData.data.warriors = 3;
                  }
                  if (stage == 4 && lv % 3 == 0) {
                      GameData.data.warriors = 4;
                  }
              }
              if (lv % 3 == 0) {
                  fc.framework.GameFrame.openView('UnlockPage', { warriorType: type }, null);
              }
              this.refeshList();
          }
          else {
              GameFrame$2.showToastMsg(GameFrame$2['langCfg']['nocointxt']);
          }
      }
      onDisable() {
          super.onDisable();
      }
      onDestroy() {
          super.onDestroy();
      }
  }

  var GameFrame$3 = fc.framework.GameFrame;
  class UnlockPageUI extends fc.plugin.BaseView {
      constructor() {
          super();
          this.warriorType = -1;
      }
      initView(data) {
          if (data) {
              this.warriorType = data.warriorType;
          }
      }
      onAwake() {
          super.onAwake();
          this.unlockBox = this.owner.getChildByName('middleUI').getChildByName('unlockBox');
          this.closeBtn = this.unlockBox.getChildByName('closeBtn');
          this.closeBtn.on(Laya.Event.CLICK, this, this.closeView);
          AudioManager.Instance.playSfx('level up.mp3');
      }
      closeView() {
          GameFrame$3.closeView(this.owner);
          GameFrame$3.playClickSound();
      }
      onStart() {
          super.onStart();
          this.owner.zOrder = 3;
          this.init(this.warriorType);
      }
      init(type) {
          let name = this.unlockBox.getChildByName('name');
          let icon = this.unlockBox.getChildByName('icon');
          let lv = Math.max(1, GameData.data.getWarriorLv(type));
          let stage = 1 + Math.floor((lv - 1) / 3);
          stage = Math.min(ResConst.WarriorLvName[type].length, stage);
          name.text = ResConst.WarriorLvName[type][stage - 1];
          icon.skin = 'icon/' + type + '_' + (stage - 1) + '.png';
      }
      onDisable() {
          super.onDisable();
      }
      onDestroy() {
          super.onDestroy();
      }
  }

  var GameFrame$4 = fc.framework.GameFrame;
  class CommonPageUI extends fc.plugin.BaseView {
      constructor() {
          super();
      }
      onAwake() {
          super.onAwake();
          this.upgradeBox = this.owner.getChildByName('middleUI').getChildByName('upgradeBox');
          this.closeBtn = this.upgradeBox.getChildByName('closeBtn');
          this.closeBtn.on(Laya.Event.CLICK, this, this.closeView);
          this.mHaoJiao = this.upgradeBox.getChildByName('HaoJiao');
          this.mGongJianJuli = this.upgradeBox.getChildByName('GongJianJuli');
          this.mChengQiang = this.upgradeBox.getChildByName('ChengQiang');
          this.mJinBi = this.upgradeBox.getChildByName('JinBi');
          this.mGongJianSudu = this.upgradeBox.getChildByName('GongJianSudu');
      }
      closeView() {
          GameFrame$4.closeView(this.owner);
          GameFrame$4.playClickSound();
      }
      onStart() {
          super.onStart();
          this.owner.zOrder = 2;
          this.init(this.mHaoJiao, CommonCfg.HaoJiao);
          this.init(this.mGongJianJuli, CommonCfg.GongJianJuli);
          this.init(this.mChengQiang, CommonCfg.ChengQiang);
          this.init(this.mJinBi, CommonCfg.JinBi);
          this.init(this.mGongJianSudu, CommonCfg.GongJianSudu);
      }
      init(box, type) {
          let bonus1 = box.getChildByName('bonus1');
          let bonus2 = box.getChildByName('bonus2');
          let panelProgress1 = bonus1.getChildByName('panelProgress');
          let tipVal1 = bonus1.getChildByName('tip');
          let bonusval1 = bonus1.getChildByName('bonusval');
          let panelProgress2 = bonus2.getChildByName('panelProgress');
          let tipVal2 = bonus2.getChildByName('tip');
          let bonusval2 = bonus2.getChildByName('bonusval');
          let upgradeBtn = box.getChildByName('upgradeBtn');
          let lvVal = box.getChildByName('lv');
          let goldVal = box.getChildByName('goldVal');
          let name = box.getChildByName('name');
          let desc = box.getChildByName('desc');
          let lv = Math.max(1, GameData.data.getCommonLv(type));
          let data = GameData.data.commonCfg[type];
          let gold = data.upgrade + Math.floor(((1 + (lv - 1)) * (lv - 1) * 0.5) * data.upgradeadd);
          goldVal.value = gold.toString();
          panelProgress1.width = (lv / data.maxlv) * 132;
          panelProgress2.width = (lv / data.maxlv) * 132;
          upgradeBtn.on(Laya.Event.CLICK, this, this.upgrade, [box, type]);
          lvVal.text = (lv - 1) + (lv >= data.maxlv ? ' 最大' : '');
          if (data.bonus2 == 0) {
              bonus2.visible = false;
          }
          tipVal1.text = data.bonus1txt;
          tipVal2.text = data.bonus2txt;
          bonusval1.text = "+" + (((lv - 1) * data.bonus1) * 100).toFixed(0) + "%";
          bonusval2.text = "-" + (((lv - 1) * data.bonus2) / 1000).toFixed(2) + '秒';
          desc.text = data.desc;
          name.text = data.title;
      }
      upgrade(box, type) {
          GameFrame$4.playClickSound();
          let lv = Math.max(1, GameData.data.getCommonLv(type));
          let data = GameData.data.commonCfg[type];
          if (lv >= data.maxlv) {
              GameFrame$4.showToastMsg('已到最大级别');
              return;
          }
          let gold = data.upgrade + Math.floor(((1 + (lv - 1)) * (lv - 1) * 0.5) * data.upgradeadd);
          if (GameData.data.gameGold >= gold) {
              GameData.data.gameGold = -gold;
              Laya.stage.event(fc.define.Event.GOLD_UPDATE);
              GameData.data.setCommonLv(lv + 1, type);
              this.init(box, type);
              Laya.stage.event(EventConst.Update_Scenario);
              Laya.stage.event(EventConst.Update_Archer);
          }
          else {
              GameFrame$4.showToastMsg(GameFrame$4['langCfg']['nocointxt']);
          }
      }
      onDisable() {
          super.onDisable();
      }
      onDestroy() {
          super.onDestroy();
      }
  }

  var GameFrame$5 = fc.framework.GameFrame;
  class OnlinePageUI extends fc.plugin.BaseView {
      constructor() {
          super();
      }
      onAwake() {
          super.onAwake();
          this.upgradeBox = this.owner.getChildByName('middleUI').getChildByName('upgradeBox');
          this.closeBtn = this.upgradeBox.getChildByName('closeBtn');
          this.closeBtn.on(Laya.Event.CLICK, this, this.closeView);
      }
      closeView() {
          GameFrame$5.closeView(this.owner);
          GameFrame$5.playClickSound();
      }
      onStart() {
          super.onStart();
          this.owner.zOrder = 2;
          this.initUI();
      }
      initUI() {
          let datas = fc.framework.GameFrame.gameCfg.onlineCfg;
          for (let index = 0; index < datas.length; index++) {
              let data = datas[index];
              let onlineUi = this.upgradeBox.getChildByName('Online' + index);
              this.initOnlineUi(onlineUi, data);
              onlineUi.disabled = true;
              let claimed = fc.framework.GameFrame.playerInfo.claim[index];
              if (claimed == 0 && GameData.data.getGameTime(GameTimeType.Online) >= data.buy) {
                  onlineUi.disabled = false;
              }
              if (claimed == 1) {
                  let upgradeBtn = onlineUi.getChildByName('upgradeBtn');
                  upgradeBtn.alpha = 0;
              }
          }
      }
      initOnlineUi(box, data) {
          let upgradeBtn = box.getChildByName('upgradeBtn');
          let descVal = box.getChildByName('desc');
          let goldVal = box.getChildByName('goldVal');
          descVal.text = data.name;
          let gold = data.price + fc.framework.GameFrame.playerInfo.claimround * 0.2 * data.price;
          goldVal.value = gold.toString();
          upgradeBtn.on(Laya.Event.CLICK, this, this.upgradeWall, [box, gold, data.id]);
      }
      upgradeWall(box, gold, id) {
          if (box.disabled)
              return;
          Util.delayMouseEnabled(box, 200);
          GameFrame$5.playClickSound();
          GameData.data.gameGold = gold;
          Laya.stage.event(fc.define.Event.GOLD_UPDATE);
          GameFrame$5.showToastMsg('恭喜您获得' + gold + '金币');
          box.disabled = true;
          let upgradeBtn = box.getChildByName('upgradeBtn');
          upgradeBtn.alpha = 0;
          GameData.data.setClaim(1, id);
          let isSuccess = true;
          for (let index = 0; index < GameData.data.getClaim().length; index++) {
              let cliamed = GameData.data.getClaim()[index];
              if (cliamed == 0) {
                  isSuccess = false;
                  break;
              }
          }
          if (isSuccess) {
              GameData.data.setGameTime(0, GameTimeType.Online);
              Laya.stage.event(EventConst.Update_Online, 0);
              let count = fc.framework.GameFrame.playerInfo.claimround + 1;
              fc.framework.GameFrame.savePlayerInfo({ claimround: count });
              fc.framework.GameFrame.savePlayerInfo({ claim: [0, 0, 0, 0, 0] });
          }
      }
      onDisable() {
          super.onDisable();
      }
      onDestroy() {
          super.onDestroy();
      }
  }

  var GameFrame$6 = fc.framework.GameFrame;
  class OfflineAwardPageUI extends fc.plugin.BaseView {
      constructor() {
          super();
          this.gold = 0;
      }
      onAwake() {
          super.onAwake();
          this.unlockBox = this.owner.getChildByName('middleUI').getChildByName('unlockBox');
          this.closeBtn = this.unlockBox.getChildByName('closeBtn');
          this.closeBtn.on(Laya.Event.CLICK, this, this.closeView);
          AudioManager.Instance.playSfx('level up.mp3');
      }
      closeView() {
          GameData.data.gameGold = this.gold;
          GameFrame$6.showToastMsg('恭喜您获得' + this.gold + '金币');
          Laya.stage.event(fc.define.Event.GOLD_UPDATE);
          GameData.data.setGameTime(Date.now(), GameTimeType.Offline);
          GameFrame$6.playClickSound();
          GameFrame$6.closeView(this.owner);
      }
      onStart() {
          super.onStart();
          this.owner.zOrder = 3;
          let goldVal = this.unlockBox.getChildByName('goldVal');
          this.gold = Math.max(100, Math.floor((Date.now() - GameData.data.getGameTime(GameTimeType.Offline)) / 60000) * 100);
          goldVal.value = this.gold.toString();
      }
      onDisable() {
          super.onDisable();
      }
      onDestroy() {
          super.onDestroy();
      }
  }

  var EventId = fc.define.Event;
  var GameFrame$7 = fc.framework.GameFrame;
  class GameLogic extends Laya.Script {
      constructor() {
          super();
          this.isShowMaxPage = false;
          this.isShowKonckPage = false;
          this.isStartGameTime = 0;
          this.createEnmeyTime = 0;
          this.aiinterval = 0;
          this.isInitScene = false;
          this.efEesList = [];
          this.isStartBattle = false;
      }
      onAwake() {
          Laya.MouseManager.enabled = false;
          Laya.ClassUtils.regClass("fc.custom.CastlePageUI", CastlePageUI);
          Laya.ClassUtils.regClass("fc.custom.WarriorPageUI", WarriorPageUI);
          Laya.ClassUtils.regClass("fc.custom.UnlockPageUI", UnlockPageUI);
          Laya.ClassUtils.regClass("fc.custom.CommonPageUI", CommonPageUI);
          Laya.ClassUtils.regClass("fc.custom.OnlinePageUI", OnlinePageUI);
          Laya.ClassUtils.regClass("fc.custom.OfflineAwardPageUI", OfflineAwardPageUI);
          if (Laya.Browser.onWeiXin || Laya.Browser.onMiniGame || Laya.Browser.onQGMiniGame || Laya.Browser.onVVMiniGame || Laya.Browser.onQQMiniGame) {
          }
          this.mInitialize = false;
          this.mOnGameShow = false;
          Laya.stage.once(EventId.LAUNCH_COMPLETED, this, this.onGameLaunchReady);
          Laya.stage.on(EventId.GAME_HOME, this, this.onGameHome);
          Laya.stage.on(EventId.GAME_PREPARE, this, this.onGamePrepare);
          Laya.stage.on(EventId.GAME_START, this, this.onGameStart);
          Laya.stage.on(EventId.GAME_OVER, this, this.onGameOver);
          Laya.stage.on(EventId.GAME_RELIVE, this, this.onGameRelive);
          Laya.stage.on(EventId.GAME_RESET, this, this.onGameReset);
          Laya.stage.on(EventId.UI_VIEW_OPENED, this, this.onViewOpened);
          Laya.stage.on(EventId.UI_VIEW_CLOSED, this, this.onViewClosed);
          Laya.stage.on(EventId.STORE_USE_ITEM, this, this.onUseItem);
          Laya.stage.on(EventId.CHANGE_CUR_MODEL, this, this.onChangeMainRole);
          Laya.stage.on(EventConst.SHOW_SUCCESS, this, this.onGameSuccess);
          Laya.stage.on(EventConst.SHOW_FAILED, this, this.onGameFailed);
          Laya.stage.on(EventConst.LOADRES_SUCCESS, this, this.load3dResSucess);
          Laya.stage.on(EventConst.Game_Refresh, this, this.onRefest);
          Laya.stage.on(EventConst.Battle_Start, this, this.battleStart);
          Laya.stage.on(EventId.GAME_WAKEUP, this, this.onWakeUp);
          Laya.stage.on(EventId.GAME_SLEEP, this, this.onSleep);
          Laya.stage.on(EventId.GAME_BACK_HOME, this, this.onBackHome);
          Laya.stage.on(EventId.DEVICE_HIDE, this, this.onAppHide);
          Laya.stage.on(EventId.DEVICE_SHOW, this, this.onAppShow);
      }
      onDestroy() {
          GameData.data.setGameTime(Date.now(), GameTimeType.Offline);
          console.log('退出游戏1');
      }
      onWakeUp() {
          if (GameFrame$7.gameSceneId == fc.define.SceneId.Game) {
              Laya.stage.event(Character.EVENT_RESUME);
              HealthBar.onShowAll(true);
          }
      }
      onSleep() {
          Laya.stage.event(Character.EVENT_PAUSE);
          HealthBar.onShowAll(false);
      }
      onAppHide() {
          Laya.stage.clearTimer(this, this.updateOnline);
          GameData.data.setGameTime(Date.now(), GameTimeType.Offline);
          console.log('退出游戏2');
      }
      onAppShow() {
      }
      onBackHome() {
          this.efEesList = [];
          this.mOnGameShow = true;
          HealthBar.onShowAll(false);
          GameLogic.cameraCtr.initCameraByPlayer(null);
          Laya.stage.event(Character.EVENT_PAUSE);
          GameFrame$7.closeView("Game");
          GameFrame$7.closeView("MainGame");
          this.onGameHome();
          GameData.data.gameFinish = true;
      }
      onUpdate() {
          if (false == this.mInitialize ||
              this.mOnGameShow ||
              GameFrame$7.isGamePaused) {
              return;
          }
          if (GameFrame$7.gameSceneId != fc.define.SceneId.Game) {
              return;
          }
          if (GameData.data.gameFinish)
              return;
          if (Laya.timer.delta > 600)
              return;
          this.createEnemyWarrior();
          if (fc.view.ActivityCenter.isMistakenActive() && !this.isShowMaxPage && fc.framework.GameFrame.pluginCfg.pg_video2hp > 0) {
              let progress = (GameMgr.Instance().getScenario().mHp) / GameMgr.Instance().getScenario().mTotalHp;
              if (progress < fc.framework.GameFrame.pluginCfg.pg_video2hp * 0.1) {
                  fc.framework.GameFrame.openView('VideoForHp', {}, null);
                  this.isShowMaxPage = true;
              }
          }
      }
      onGameLaunchReady() {
          console.log("GameLogic", EventId.LAUNCH_COMPLETED);
          this.mInitialize = true;
          var stageCfg = Laya.loader.getRes(fc.platform.PlatformConfig.platformTag == 'iptv' ? "config/stageCfgiptv.json" : "config/stageCfg.json");
          GameData.initStageData(stageCfg);
          var CastleCfg = Laya.loader.getRes("config/castleCfg.json");
          GameData.initCastleCfg(CastleCfg);
          var WarriorCfg = Laya.loader.getRes("config/warriorCfg.json");
          GameData.initWarriorCfg(WarriorCfg);
          var EnemyCfg = Laya.loader.getRes("config/enemyCfg.json");
          GameData.initEnemyCfg(EnemyCfg);
          var CommonCfg = Laya.loader.getRes("config/commonCfg.json");
          GameData.initCommonCfg(CommonCfg);
          GameData.updatePlayerInfo(GameFrame$7.playerInfo);
          this.init();
          ResourceMgr.preLoadRes(ResourceMgr.threeDResRootPath3, ResourceMgr.res3List);
          ResourceMgr.preLoadRes(ResourceMgr.threeDResRootPath2, ResourceMgr.res2List);
          ResourceMgr.preCreateRes();
      }
      load3dResSucess() {
          GameFrame$7.finishLoading();
          this.isInitScene = true;
          this.efEesList = ResourceMgr.efEesList.concat();
          Laya.MouseManager.enabled = true;
      }
      preLoadEffect() {
          this.efEesList.forEach(function (cfgUrl) {
              let ef = EffectMgr.Instance().createSpriteEffect(cfgUrl, true);
              ef.transform.position = new Laya.Vector3(-3, 0, -10);
              Laya.Tween.to(GameFrame$7.scene3d, {}, 2000, null, Laya.Handler.create(this, (ef) => {
                  EffectMgr.Instance().removeSpriteEffect(ef);
              }, [ef]));
          });
      }
      onGameHome() {
          Laya.MouseManager.enabled = false;
          this.mOnGameShow = false;
          this.initGame();
          this.preLoadEffect();
          GameData.data.gameFinish = true;
          this.isInitScene = false;
      }
      onGamePrepare() {
          GameData.data.gameFinish = false;
          GameFrame$7.startGame();
      }
      onGameStart() {
          this.isStartGameTime = Date.now();
          GameLogic.cameraCtr.playGameAni();
      }
      battleStart() {
          Laya.stage.event(Character.EVENT_RESUME);
          this.isStartBattle = false;
          this.createEnmeyTime = 0;
          Laya.stage.timerLoop(3000, this, this.setCameraPos, null, false);
      }
      onGameOver() {
          GameData.data.curStage = fc.framework.GameFrame.playerInfo.levelId;
          GameData.data.gameMode = 0;
          GameData.data.gold = GameData.data.gameGold;
          this.efEesList = [];
      }
      onGameReset() {
      }
      onGameRelive() {
      }
      onGameSuccess() {
          Laya.stage.clearTimer(this, this.setCameraPos);
          var playerInfo = GameFrame$7.playerInfo;
          if (playerInfo.isInGuide == 1) {
              GameFrame$7.savePlayerInfo({ isInGuide: 0, isNew: 0 });
          }
          this.mOnGameShow = true;
          GameData.data.gameFinish = true;
          Laya.stage.mouseEnabled = false;
          AudioManager.Instance.playSfx('victoria.mp3');
          GameLogic.cameraCtr.initCameraByPlayer(null);
          this.owner.timerOnce(500, this, () => {
              HealthBar.onShowAll(false);
              Laya.stage.mouseEnabled = true;
              Laya.stage.event(Character.EVENT_PAUSE);
              let count = 100 * GameData.data.curStage;
              GameFrame$7.showSuccessed({ award: count });
              GameFrame$7.closeView("Game");
              GameFrame$7.closeView("MainGame");
          });
      }
      onGameFailed() {
          Laya.stage.clearTimer(this, this.setCameraPos);
          if (GameFrame$7.playerInfo.isNew == 1) {
              GameFrame$7.savePlayerInfo({ isNew: 0 });
          }
          this.mOnGameShow = true;
          GameData.data.gameFinish = true;
          fc.framework.Device.vibrateLong();
          AudioManager.Instance.playSfx('defeat.mp3');
          GameLogic.cameraCtr.initCameraByPlayer(null);
          this.owner.timerOnce(500, this, () => {
              HealthBar.onShowAll(false);
              Laya.stage.mouseEnabled = true;
              Laya.stage.event(Character.EVENT_PAUSE);
              GameFrame$7.showFailed({ award: 100 });
              GameFrame$7.closeView("Game");
              GameFrame$7.closeView("MainGame");
          });
      }
      onViewClosed(viewName, viewId) {
          console.log("onViewClosed", viewName);
          if ('StorePage' == viewName || 'TrialPage' == viewName) {
              Laya.stage.event(fc.define.Event.TEXTURE_CAMERA_CLEAR);
          }
          if (viewName.indexOf('HomePage') != -1) {
              GameFrame$7.closeView("MainHome");
          }
          if (viewId == 'VideoForHp' || viewId == 'VideoForBrick') {
              HealthBar.onShowAll(true);
          }
      }
      onViewOpened(viewName, viewId) {
          console.log("onViewOpened", viewName);
          if (viewId == 'VideoForHp' || viewId == 'VideoForBrick') {
              HealthBar.onShowAll(false);
          }
      }
      onUseItem(type, data) {
      }
      onChangeMainRole(prefab) {
      }
      init() {
          Laya.MouseManager.multiTouchEnabled = false;
          GameFrame$7.scene3d.input.multiTouchEnabled = false;
          GameMgr.Instance().init(GameFrame$7.scene3d);
          BulletMgr.Instance().init(GameFrame$7.scene3d);
          EffectMgr.Instance().init(GameFrame$7.scene3d);
          GoldMgr.Instance().init(GameFrame$7.scene3d);
          GameLogic.cameraCtr = GameFrame$7.scene3d.getChildByName("CameraNode").addComponent(CameraCtr);
          GameLogic.camera = GameLogic.cameraCtr.camera;
          Laya.stage.timerLoop(60000, this, this.updateOnline, null, false);
      }
      updateOnline() {
          let online = GameData.data.getGameTime(GameTimeType.Online) + 1;
          Laya.stage.event(EventConst.Update_Online, online);
          GameData.data.setGameTime(online, GameTimeType.Online);
          GameData.data.setGameTime(Date.now(), GameTimeType.Offline);
      }
      initGame() {
          this.destroyGame();
          GameLogic.cameraCtr.playHomeAni();
          GameData.data.gameFinish = false;
          GameData.data.isShowVideoForSolider = false;
          GameData.data.isShowVideoForBrick = false;
          this.aiinterval = GameFrame$7.gameCfg.aiinterval + Math.min(2000, GameData.data.curStage * 10);
          this.isShowMaxPage = false;
          this.isShowKonckPage = false;
          this.isStartBattle = false;
          GameMgr.Instance().initStageById(GameData.data.curStage);
          GameMgr.Instance().initMainHome();
          Laya.MouseManager.enabled = true;
      }
      onRefest() {
          this.destroyGame();
          GameData.data.gameFinish = false;
          GameMgr.Instance().initStageById(GameData.data.curStage);
          GameMgr.Instance().initMainHome();
          Laya.MouseManager.enabled = false;
          Laya.MouseManager.enabled = true;
      }
      destroyGame() {
          GameMgr.Instance().onDestroy();
          EffectMgr.Instance().onDestroy();
          GoldMgr.Instance().onDestroy();
          BulletMgr.Instance().onDestroy();
          HealthBar.onRecoverAll();
          HealthBar.onDestroyAll();
      }
      createEnemyWarrior() {
          if (GameMgr.Instance().enemyLave < 1 && GameMgr.Instance().getEnemyList().length < 1) {
              Laya.stage.event(EventConst.SHOW_SUCCESS);
          }
          if (GameMgr.Instance().enemyLave < 1)
              return;
          if (GameMgr.Instance().getEnemyList().length > 20)
              return;
          if (GameMgr.Instance().getPlayerList().length > 0 || this.isStartBattle) {
              this.isStartBattle = true;
              if (Date.now() - this.createEnmeyTime > this.aiinterval) {
                  GameMgr.Instance().createEnemySoldier();
                  this.createEnmeyTime = Date.now();
                  let progress = GameMgr.Instance().enemyLave / GameMgr.Instance().enemyTotal;
                  Laya.stage.event(EventConst.Game_Update_Progress, progress);
              }
          }
      }
      setCameraPos() {
          let z = this.getPlayerCenterPos().z;
          GameLogic.cameraCtr.setPositionZ(z);
      }
      getPlayerCenterPos() {
          let characterList = GameMgr.Instance().getPlayerList().concat();
          ;
          let pos = Util.ZERO.clone();
          let x = 0;
          let z = 0;
          let y = 0;
          let count = 0;
          for (let i = characterList.length - 1; i >= -1; i--) {
              const character = characterList[i];
              if (character && character.transform.position.z > 1) {
                  x += character.transform.position.x;
                  y += character.transform.position.y;
                  z += character.transform.position.z;
                  count++;
              }
          }
          if (count > 0) {
              pos.setValue(x / count, y / count, z / count);
          }
          return pos;
      }
  }

  class HealthBar {
      constructor(nickname, hp, isAi) {
          this._outPos = new Laya.Vector4();
          this.width = 64;
          Laya.loader.create("game/prefab/hp.json", Laya.Handler.create(this, this.onPrefabReady, [nickname, hp, isAi]), null, Laya.Loader.PREFAB);
      }
      static getHealthBarInPool(nickname, hp, isAi) {
          let healthBar = null;
          if (this.healthBarPool.length > 0)
              healthBar = this.healthBarPool.pop();
          else
              healthBar = new HealthBar(nickname, hp, isAi);
          return healthBar;
      }
      static getHealthBar(nickname, hp, isAi = true) {
          HealthBar.parent = HealthBar.parent || Laya.stage;
          let healthBar = this.getHealthBarInPool(nickname, hp, isAi);
          if (healthBar.hpBar) {
              healthBar.isAi = isAi;
              healthBar.hpProgress.width = healthBar.width;
              if (isAi) {
                  healthBar.hpProgress.getChildByName('red').visible = false;
                  healthBar.hpProgress.getChildByName('blue').visible = true;
              }
              else {
                  healthBar.hpProgress.getChildByName('red').visible = true;
                  healthBar.hpProgress.getChildByName('blue').visible = false;
              }
              healthBar.nickname && (healthBar.nickname.text = nickname);
              healthBar.hpTxt && (healthBar.hpTxt.value = String(Math.floor(hp)));
              healthBar.hpBar.scaleX = 0.6;
              healthBar.hpBar.scaleY = 0.6;
          }
          healthBar.setActive(false);
          this.healthBars.push(healthBar);
          return healthBar;
      }
      static recover(healthBar) {
          if (!healthBar)
              return;
          this.recovertoPool(healthBar);
          let index = this.healthBars.indexOf(healthBar);
          if (index != -1)
              this.healthBars.splice(index, 1);
      }
      static recovertoPool(healthBar) {
          if (!healthBar)
              return;
          let index = this.healthBarPool.indexOf(healthBar);
          if (index == -1)
              this.healthBarPool[this.healthBarPool.length] = healthBar;
          healthBar.setActive(false);
      }
      static setParent(parent) {
          this.parent = parent;
      }
      onPrefabReady(nickname, hp, isAi, prefab) {
          this.hpBar = prefab.create();
          this.hpBar.visible = false;
          HealthBar.parent.addChild(this.hpBar);
          this.hpProgress = this.hpBar.getChildByName('progress');
          this.nickname = this.hpBar.getChildByName('nickname');
          this.hpTxt = this.hpBar.getChildByName('hpTxt');
          this.hpProgress.width = this.width;
          this.nickname && (this.nickname.text = nickname);
          this.hpTxt && (this.hpTxt.value = String(Math.floor(hp)));
          if (isAi) {
              this.hpProgress.getChildByName('red').visible = false;
              this.hpProgress.getChildByName('blue').visible = true;
          }
          else {
              this.hpProgress.getChildByName('red').visible = true;
              this.hpProgress.getChildByName('blue').visible = false;
          }
          this.hpBar.scaleX = 0.6;
          this.hpBar.scaleY = 0.6;
      }
      setActive(val) {
          this.hpBar && (this.hpBar.visible = val);
      }
      onDestroy() {
          this.hpBar.destroy(true);
          this.hpBar = null;
      }
      updatePosition(position) {
          if (!this.hpBar)
              return;
          GameLogic.camera.viewport.project(position, GameLogic.camera.projectionViewMatrix, this._outPos);
          let x = this._outPos.x / Laya.stage.clientScaleX;
          let y = this._outPos.y / Laya.stage.clientScaleY;
          this.hpBar.pos(x, y);
      }
      static onRecoverAll() {
          for (let i = this.healthBars.length - 1; i >= 0; i--) {
              const healthBar = this.healthBars.pop();
              this.recovertoPool(healthBar);
          }
          this.healthBars = [];
      }
      static onDestroyAll() {
          for (let i = this.healthBarPool.length - 1; i >= 0; i--) {
              const healthBar = this.healthBarPool.pop();
              healthBar.onDestroy();
          }
          this.healthBarPool = [];
      }
      static onShowAll(show) {
          let length = this.healthBars.length;
          for (let i = 0; i < length; i++) {
              const healthBar = this.healthBars[i];
              healthBar.setActive(show);
          }
      }
      updateHp(hpPercent, hp) {
          if (this.hpBar) {
              hpPercent = Math.max(0.001, Math.min(hpPercent, 1));
              this.hpProgress.width = this.width * hpPercent;
              this.hpTxt && (this.hpTxt.value = String(hp));
              hp <= 0 && this.setActive(false);
          }
      }
  }
  HealthBar.healthBarPool = [];
  HealthBar.healthBars = [];

  class IBullet extends Laya.Script3D {
      constructor() {
          super(...arguments);
          this.dir = new Laya.Vector3(0, 0, 0);
          this.startPos = new Laya.Vector3(0, 0, 0);
          this.lifeDis = 0;
          this.speed = 0;
          this.rotationEuler = new Laya.Vector3(0, 0, 0);
          this.position = new Laya.Vector3(0, 0, 0);
      }
      onAwake() {
          this.bullet3d = this.owner;
      }
      onStart() {
          this.rigidbody3d = this.bullet3d.getComponent(Laya.Rigidbody3D);
          this.rigidbody3d.angularFactor = new Laya.Vector3(0, 0, 1);
          this.rigidbody3d.angularVelocity = new Laya.Vector3(0, 0, 0);
          this.rigidbody3d.linearVelocity = new Laya.Vector3(0, 0, 0);
          this.rigidbody3d.collisionGroup = CollisionConst.GROUP_BULLET;
          this.rigidbody3d.canCollideWith = CollisionConst.CAN_BULLET;
          this.rigidbody3d.sleepLinearVelocity = 0;
          this.rigidbody3d.sleepAngularVelocity = 0;
          this.rigidbody3d.wakeUp();
          this.rigidbody3d.enabled = true;
          this.rigidbody3d.isTrigger = true;
          this.rigidbody3d.isKinematic = true;
          this.rigidbody3d.overrideGravity = true;
          this.rigidbody3d.gravity = new Laya.Vector3(0, 0, 0);
          this.mTransform = this.bullet3d.transform;
          this.mTransform.rotationEuler = this.rotationEuler.clone();
          this.mTransform.position = this.position.clone();
          this.initEffect();
      }
      init(firer, atked, dir, startPos) {
          this.firer = firer;
          this.atked = atked;
          this.dir = dir.clone();
          this.startPos = startPos.clone();
      }
      initEffect() {
      }
      onTriggerEnter(other) {
      }
      onCollisionEnter(collision) {
          var otherSprite3d = collision.other.owner;
          console.log(otherSprite3d.name, 'onCollisionEnter.name');
      }
      setColliderEnable(isEnable) {
          this.rigidbody3d && (this.rigidbody3d.enabled = isEnable);
      }
      addHitEffect() {
          this.setColliderEnable(false);
      }
      removeBullet() {
      }
  }

  class Bezier {
      static draw_bezier_curves(points, count, step) {
          var bezier_curves_points = new Array();
          var t = 0;
          do {
              var temp_point = this.bezier_interpolation_func(t, points, count);
              t += step;
              bezier_curves_points.push(temp_point);
          } while (t <= 1 && count > 1);
          return bezier_curves_points;
      }
      static bezier_interpolation_func(t, points, count) {
          var result = new Laya.Vector3();
          var part = [];
          var sum_x = 0, sum_y = 0, sum_z = 0;
          for (let i = 0; i < count; i++) {
              var tmp;
              var n_order = count - 1;
              tmp = this.calc_combination_number(n_order, i);
              sum_x += (tmp * points[i].x * Math.pow((1 - t), n_order - i) * Math.pow(t, i));
              sum_y += (tmp * points[i].y * Math.pow((1 - t), n_order - i) * Math.pow(t, i));
              sum_z += (tmp * points[i].z * Math.pow((1 - t), n_order - i) * Math.pow(t, i));
          }
          result.x = sum_x;
          result.y = sum_y;
          result.z = sum_z;
          return result;
      }
      static calc_combination_number(n, k) {
          var result = [];
          for (let i = 1; i <= n; i++) {
              result[i] = 1;
              for (let j = i - 1; j >= 1; j--)
                  result[j] += result[j - 1];
              result[0] = 1;
          }
          return result[k];
      }
      static creatorTwoBezierPoint(p0, p1, p2, pointCount) {
          var vertices = [], t = 0;
          for (var i = 0; i < pointCount; i++) {
              let x = Math.pow(1 - t, 2) * p0.x + 2 * t * (1 - t) * p1.x + Math.pow(t, 2) * p2.x;
              let y = Math.pow(1 - t, 2) * p0.y + 2 * t * (1 - t) * p1.y + Math.pow(t, 2) * p2.y;
              let z = Math.pow(1 - t, 2) * p0.z + 2 * t * (1 - t) * p1.z + Math.pow(t, 2) * p2.z;
              vertices.push(new Laya.Vector3(x, y, z));
              t += 1.0 / pointCount;
          }
          return vertices;
      }
  }

  class Arrow extends IBullet {
      constructor() {
          super(...arguments);
          this.targetPos = Util.ZERO.clone();
          this.isEnemy = false;
      }
      init(firer, atked, dir, startPos) {
          super.init(firer, atked, dir, startPos);
          this.lifeDis = firer.shotRang;
          this.speed = firer.shotSpeed;
          this.isEnemy = firer.teamType == TeamType.Red;
          this.targetPos = atked.mtransform.position.clone();
      }
      onStart() {
          super.onStart();
          this.flyTrack();
      }
      rotationBullet() {
          if (!this.atked || !this.atked.isLife())
              return;
          let targetPos = this.atked.mtransform.position.clone();
          targetPos.y += 3.5;
          let targetDir = Util.vectorDirection(this.bullet3d.transform.position.clone(), targetPos);
          let angley = Util.angleY2(targetDir);
          let rotation = new Laya.Vector3(-Util.angleDirection(targetDir, Util.UP), angley, 0);
          this.mTransform.rotationEuler = rotation;
          this.dir = targetDir;
      }
      onLateUpdate() {
          if (!this.dir)
              return;
          if (this.lifeDis <= 0 || (!this.atked || !this.atked.isLife())) {
              this.removeBullet();
              this.lifeDis = 1000;
              this.dir = null;
              return;
          }
      }
      removeBullet() {
          super.removeBullet();
          Laya.Tween.clearAll(this.mTransform.position);
          let trail2 = this.bullet3d.getChildByName('Trail').getChildByName('Trail');
          trail2 && trail2.clear();
          BulletMgr.Instance().removeBullet(this);
      }
      onTriggerEnter(other) {
          var otherSprite3d = other.owner;
          let robotObj = null;
          robotObj = otherSprite3d.getComponent(Character);
          if (robotObj && robotObj.teamType != this.firer.teamType) {
              console.log(otherSprite3d.name, 'onTriggerEnter.name');
              if (robotObj.sprite3d.active && robotObj.isLife()) {
                  robotObj.onDamage(this.firer.atkDamage);
                  this.setColliderEnable(false);
                  this.lifeDis = 0;
                  AudioManager.Instance.playSfx('ArrowHitDirt0' + Util.random(1, 6) + '.mp3');
              }
          }
      }
      flyTrack() {
          var curPos = this.mTransform.position.clone();
          let targetPos = this.targetPos.clone();
          Laya.Vector3.add(targetPos, this.dir, targetPos);
          var positions = Bezier.creatorTwoBezierPoint(curPos, new Laya.Vector3((curPos.x + this.targetPos.x) * 0.5, Math.max(3, curPos.y), (curPos.z + this.targetPos.z) * 0.5), targetPos, 10);
          var hander = new Laya.Handler(this, function () {
              this.mTransform.position = this.mTransform.position;
          });
          let dist = Laya.Vector3.distance(curPos, this.targetPos);
          let flytime = 1000 * dist / this.speed;
          flytime = Math.max(flytime, 250);
          var totleTime = 0;
          var tweenTime = flytime / positions.length;
          var d = flytime / (positions.length * (positions.length - 1));
          var curTime = tweenTime;
          for (let i = 0; i < positions.length; i++) {
              Laya.Tween.to(this.mTransform.position, {
                  y: positions[i].y,
                  z: positions[i].z,
                  x: positions[i].x,
                  update: hander
              }, curTime, null, Laya.Handler.create(this, (i) => {
                  if (i == positions.length - 1) {
                      this.mTransform.position = this.targetPos.clone();
                      Laya.stage.frameOnce(1, this, () => {
                          this.setColliderEnable(false);
                          Laya.stage.frameOnce(10, this, () => {
                              this.addHitEffect();
                          }, null, false);
                      }, null, false);
                  }
                  else {
                      let targetDir = Util.vectorDirection(this.mTransform.position, positions[i + 1]);
                      let angley = Util.angleY2(targetDir);
                      let rotation = new Laya.Vector3(-Util.angleDirection(targetDir, Util.UP), this.mTransform.rotationEuler.y, 0);
                      this.mTransform.rotationEuler = rotation;
                  }
              }, [i]), totleTime);
              totleTime += curTime;
              curTime += d;
          }
      }
      addHitEffect() {
          super.addHitEffect();
          this.removeBullet();
      }
  }

  var AnimListener = fc.plugin.AnimStateListener;
  class SoldierAnimatorController extends AnimatorController {
      onAwake() {
          this.animator = this.owner.getComponent(Laya.Animator);
          this.animator.speed = 0;
      }
      onStart() {
          AnimListener.add(this.animator, SoldierAnimatorController.ci, this);
          AnimListener.add(this.animator, SoldierAnimatorController.kan, this);
          AnimListener.add(this.animator, SoldierAnimatorController.tou, this);
          AnimListener.add(this.animator, SoldierAnimatorController.shejian, this);
      }
      onEnable() {
          super.onEnable();
          if (this.character == null) {
          }
      }
      onDisable() {
          this.onDestroy();
          if (this.animator) {
              AnimListener.remove(this.animator, SoldierAnimatorController.ci, this);
              AnimListener.remove(this.animator, SoldierAnimatorController.kan, this);
              AnimListener.remove(this.animator, SoldierAnimatorController.tou, this);
              AnimListener.remove(this.animator, SoldierAnimatorController.shejian, this);
          }
      }
      onAnimEvt(...args) {
      }
      onAnimStateEnter(stateName, animator) {
      }
      onAnimStateUpdate(stateName, animator) {
      }
      onAnimStateExit(stateName, animator) {
          switch (stateName) {
              case SoldierAnimatorController.ci:
              case SoldierAnimatorController.tou:
              case SoldierAnimatorController.kan:
              case SoldierAnimatorController.shejian:
                  this.curClipName = null;
                  this.play(SoldierAnimatorController.idle);
                  break;
              default:
                  break;
          }
      }
      onDestroy() {
          this.animator = null;
          this.character = null;
          this.curClipName = null;
      }
      play(newName) {
          if (!this.animator || !newName || this.curClipName == newName)
              return;
          if (!this.character)
              return;
          this.animator.play(newName, 0, 0);
          this.curClipName = newName;
          this.animator.speed = 1;
      }
  }
  SoldierAnimatorController.ci = "ci";
  SoldierAnimatorController.kan = "kan";
  SoldierAnimatorController.tou = "tou";
  SoldierAnimatorController.shejian = "shejian";

  class GongJian extends Character {
      constructor() {
          super(...arguments);
          this.soldierController = null;
          this.mAtkedEf = null;
          this.mEfName = '';
      }
      onAwake() {
          super.onAwake();
          Laya.stage.on(EventConst.Update_Archer, this, this.updateLv);
      }
      onEnable() {
          super.onEnable();
      }
      onPause() {
          this.isPause = true;
          this.playIdleAni();
      }
      onStart() {
          super.onStart();
          this.collider3d = this.sprite3d.getComponent(Laya.PhysicsCollider);
          this.collider3d.enabled = true;
          this.collider3d.isTrigger = true;
          this.setColliderEnable(false);
          this.collider3d.collisionGroup = CollisionConst.GROUP_Soldier;
          this.collider3d.canCollideWith = CollisionConst.CAN_Soldier;
          this.animatorController = this.sprite3d.getComponent(SoldierAnimatorController);
          if (this.animatorController) {
              this.animatorController.destroy();
              this.animatorController = null;
          }
          this.animatorController = this.sprite3d.addComponent(SoldierAnimatorController);
          this.animatorController.character = this;
          this.hpPoint = this.sprite3d.getChildByName('hp');
          this.updateLv();
          let hand = this.sprite3d.getChildByName('Bip001').getChildByName('Bip001 Pelvis').getChildByName('Bip001 Spine').getChildByName('Bip001 Spine1').getChildByName('Bip001 Neck').getChildByName('Bip001 Head').getChildByName('head_point');
          let shield = this.sprite3d.getChildByName('Bip001').getChildByName('Bip001 Pelvis').getChildByName('Bip001 Spine').getChildByName('Bip001 Spine1').getChildByName('Bip001 Neck').getChildByName('Bip001 L Clavicle').getChildByName('Bip001 L UpperArm').getChildByName('Bip001 L Forearm').getChildByName('Bip001 L Hand').getChildByName('Left_shield');
          let weapons = this.sprite3d.getChildByName('Bip001').getChildByName('Bip001 Pelvis').getChildByName('Bip001 Spine').getChildByName('Bip001 Spine1').getChildByName('Bip001 Neck').getChildByName('Bip001 R Clavicle').getChildByName('Bip001 R UpperArm').getChildByName('Bip001 R Forearm').getChildByName('Bip001 R Hand').getChildByName('Right_weapons');
          this.mShotPoint = weapons;
          Util.activeAllChild(hand, false);
          Util.activeAllChild(shield, false);
          Util.activeAllChild(weapons, false);
          shield.getChildByName('Arrow_02').active = true;
          hand.getChildByName('Helmet_01').active = true;
          if (this.mAtkedEf) {
              this.mAtkedEf.removeSelf();
              this.mAtkedEf.destroy();
          }
          this.playIdleAni();
      }
      updateLv() {
          let lv = 0;
          if (this.teamType == TeamType.Red) {
          }
          else {
              Util.changMat(ResConst.MaterialPath + ResConst.RoleMate[1], this.sprite3d.getChildByName('CH_Spartan_Melee_01').skinnedMeshRenderer);
              lv = Math.max(1, GameData.data.getCastleLv(CastleCfg.Archer));
              if (lv <= 1) {
                  GameData.data.setCastleLv(1, CastleCfg.Archer);
              }
              let archerdata = GameData.data.castleCfg.Archer;
              this.mHp = 1000;
              this.mTotalHp = this.mHp;
              this.moveSpeed = 0;
              let bounslv = Math.max(1, GameData.data.getCommonLv(CommonCfg.GongJianJuli));
              let data = GameData.data.commonCfg.GongJianJuli;
              this.atkRang = archerdata.atkdist + ((bounslv - 1) * data.bonus1) * archerdata.atkdist;
              let bounslv1 = Math.max(1, GameData.data.getCommonLv(CommonCfg.GongJianSudu));
              let data2 = GameData.data.commonCfg.GongJianSudu;
              let interval = archerdata.interval - ((bounslv1 - 1) * data2.bonus1) * archerdata.interval;
              interval = Math.max(1000, interval);
              this.atkIntvalTemp = interval;
              this.atkIntval = 0;
              this.atkDamage = archerdata.atk + Math.floor(lv * archerdata.atkadd);
              this.shotRang = archerdata.atkdist;
              this.shotSpeed = archerdata.shotspeed;
              this.damageRang = archerdata.atkdist;
          }
      }
      onResume() {
          super.onResume();
          if (!this.healthBar) {
          }
      }
      initEvent() {
          super.initEvent();
      }
      removeEvent() {
          super.removeEvent();
          Laya.stage.off(EventConst.Update_Archer, this, this.updateLv);
      }
      playIdleAni() {
          this.animatorController && (this.animatorController.play(SoldierAnimatorController.idle));
      }
      playAtkAni() {
          this.animatorController && (this.animatorController.play(SoldierAnimatorController.shejian));
      }
      playRunAni() {
          this.animatorController && (this.animatorController.play(SoldierAnimatorController.run));
      }
      playDieAni() {
          this.animatorController && (this.animatorController.play(SoldierAnimatorController.die));
      }
      onUpdate() {
          if (!this.isLife())
              return;
          if (GameData.data.gameFinish)
              return;
          this.atkIntval -= Laya.timer.delta;
          if (this.atkIntval <= 0) {
              let target = null;
              if (this.atkedTarget && this.atkedTarget.isLife() && this.isInAtkRang(this.atkedTarget)) {
                  target = this.atkedTarget;
              }
              else {
                  target = this.getNearEnemyOfAtkRang();
                  this.atkedTarget = target;
              }
              if (this.isLife()) {
                  if (target) {
                      this.atkIntval = this.atkIntvalTemp;
                      let angle = Util.angleY(this.atkedTarget.mtransform.position.clone(), this.mtransform.position.clone());
                      let angleY = this.sprite3d.transform.rotationEuler.y;
                      if (angle != angleY) {
                          this.sprite3d.transform.rotate(new Laya.Vector3(0, angle - angleY, 0), false, false);
                      }
                      this.onAtk(this.atkedTarget);
                  }
                  else {
                      this.playIdleAni();
                  }
              }
          }
      }
      move(dir) {
      }
      rotationRole(angle) {
          let angleY = this.sprite3d.transform.rotationEuler.y;
          let rotateY = angle - angleY;
          if (rotateY != 0) {
              this.sprite3d.transform.rotate(new Laya.Vector3(0, rotateY, 0), false, false);
          }
      }
      onAtk(atkedObj) {
          this.atkedTarget = atkedObj;
          this.playAtkAni();
          this.owner.timerOnce(100, this, this.createBullet, null, false);
      }
      createBullet() {
          AudioManager.Instance.playSfx('arrow' + Util.random(1, 3) + '.mp3');
          let name = 'Arrow';
          let bullet3d = BulletMgr.Instance().createBullet(name);
          let bullet = bullet3d.getComponent(IBullet);
          if (bullet != null) {
              bullet.destroy();
          }
          bullet = bullet3d.addComponent(Arrow);
          BulletMgr.Instance().addBullet(bullet);
          bullet3d.transform.setWorldLossyScale(new Laya.Vector3(2, 2, 1));
          bullet.firer = this;
          bullet.atked = this.atkedTarget;
          let shotDir = Util.vectorDirection(this.mtransform.position.clone(), this.atkedTarget.mtransform.position.clone());
          bullet.init(this, this.atkedTarget, shotDir, this.mShotPoint.transform.position.clone());
          bullet.rotationEuler = this.mtransform.rotationEuler.clone();
          bullet.position = this.mShotPoint.transform.position.clone();
      }
      onDamage(damage) {
      }
      onDestroy() {
          if (this.mAtkedEf) {
              EffectMgr.Instance().removeSpriteEffect(this.mAtkedEf, true);
          }
      }
      addDamageEf() {
      }
      getNearEnemyOfAtkRang(role = null) {
          let characterList = [];
          if (this.teamType == TeamType.Red) {
              characterList = GameMgr.Instance().getPlayerList().concat();
          }
          else {
              characterList = GameMgr.Instance().getEnemyList().concat();
          }
          let target = null;
          let disSqr = this.atkRang * this.atkRang;
          for (let i = characterList.length - 1; i >= 0; i--) {
              const character = characterList[i];
              if (character == this || !character.isLife() || !character.sprite3d.active)
                  continue;
              let newDisSqr = Laya.Vector3.distanceSquared(character.mtransform.position, this.mtransform.position);
              if (newDisSqr <= disSqr && character != role) {
                  if (target == null || disSqr > newDisSqr) {
                      target = character;
                      disSqr = newDisSqr;
                  }
              }
          }
          return target;
      }
      getNearEnemyOfRang(role = null) {
          let characterList = [];
          if (this.teamType == TeamType.Red) {
              characterList = GameMgr.Instance().getPlayerList().concat();
          }
          else {
              characterList = GameMgr.Instance().getEnemyList().concat();
          }
          let target = null;
          let disSqr = 100000;
          for (let i = characterList.length - 1; i >= 0; i--) {
              const character = characterList[i];
              if (character == this || !character.isLife() || !character.sprite3d.active)
                  continue;
              let newDisSqr = Laya.Vector3.distanceSquared(character.mtransform.position, this.mtransform.position);
              if (newDisSqr <= disSqr && character != role) {
                  if (target == null || disSqr > newDisSqr) {
                      target = character;
                      disSqr = newDisSqr;
                  }
              }
          }
          return target;
      }
      getRandomEnemyOfRang(role = null) {
          let characterList = [];
          if (this.teamType == TeamType.Red) {
              characterList = GameMgr.Instance().getPlayerList().concat();
          }
          else {
              characterList = GameMgr.Instance().getEnemyList().concat();
          }
          let target = characterList[Util.random(0, characterList.length)];
          return target;
      }
      rayCheckWall(dir, rang) {
          if (!dir || dir == null) {
              return null;
          }
          var from = new Laya.Vector3(0, 0, 0);
          Laya.Vector3.normalize(dir, from);
          var to = new Laya.Vector3(0, 0, 0);
          var raylenght = new Laya.Vector3(0, 0, 0);
          Laya.Vector3.scale(from, -rang, raylenght);
          Laya.Vector3.add(this.mtransform.position.clone(), raylenght, to);
          this.mtransform.position.cloneTo(from);
          from.y = 1;
          to.y = 1;
          var hitResults = [];
          this.scene3d.physicsSimulation.raycastAllFromTo(from, to, hitResults, CollisionConst.GROUP_RAY, CollisionConst.CAN_RAY);
          for (let i = 0, n = hitResults.length; i < n; i++) {
              var otherSprite3d = hitResults[i].collider.owner;
              if (otherSprite3d && otherSprite3d.active) {
                  var character = otherSprite3d.getComponent(Character);
                  if (!character || (character != this && (character.teamType == this.teamType))) {
                      let hitResult = hitResults[i];
                      return hitResult;
                  }
              }
          }
          return null;
      }
  }

  class IState {
      Enter(ai) {
      }
      Update(ai) {
      }
      Exit(ai) {
      }
  }

  class Atk extends IState {
      constructor() {
          super(...arguments);
          this.atkIntval = 0;
          this.behavior = ScriptType.atk;
      }
      Enter(ai) {
          console.log("攻击状态");
          this.atkIntval = 0;
      }
      Update(ai) {
          if (!ai.character.isLife())
              return;
          this.atkIntval -= Laya.timer.delta;
          if (this.atkIntval <= 0) {
              let target = null;
              ai.atkTarget = ai.character.getNearEnemyOfRang();
              if (ai.atkTarget && ai.atkTarget.isLife() && ai.character.isInAtkRang(ai.atkTarget)) {
                  target = ai.atkTarget;
              }
              if (ai.character && ai.character.isLife()) {
                  if (target) {
                      this.atkIntval = ai.character.atkIntval;
                      let angle = Util.angleY(target.mtransform.position.clone(), ai.character.mtransform.position.clone());
                      ai.character.rotationRole(angle);
                      ai.character.onAtk(ai.atkTarget);
                      if (!ai.character.isInAtkRang(ai.atkTarget)) {
                          ai.changeState(new PursuerRun());
                      }
                  }
                  else {
                      ai.changeState(new PursuerRun());
                  }
              }
              else {
                  ai.character && ai.character.move(null);
              }
          }
      }
  }

  class RunBack extends IState {
      constructor() {
          super(...arguments);
          this.targetPosition = null;
          this.behavior = ScriptType.runback;
      }
      Enter(ai) {
          ai.character.move(null);
          ai.character.playRunAni();
          let dir = Util.vectorDirection(this.targetPosition, ai.character.mtransform.position);
          dir.y = 0;
          Laya.Vector3.normalize(dir, ai.dir);
          let angle = Math.atan2(-ai.dir.x, -ai.dir.z) * 180 / Math.PI;
          ai.character.rotationRole(angle);
      }
      Update(ai) {
          let character = ai.character;
          if (character && character.isLife()) {
              if (ai.character.teamType == TeamType.Blue && ai.character.getNearEnemyOfRang()) {
                  ai.changeState(new PursuerRun());
              }
              else {
                  let dir = ai.dir;
                  dir.y = 0;
                  Laya.Vector3.normalize(dir, dir);
                  character.move(ai.dir);
                  let curPosition = character.mtransform.position;
                  let disSqr = Laya.Vector3.distanceSquared(curPosition, this.targetPosition);
                  if (disSqr < 0.1) {
                      ai.character.move(null);
                      ai.character.playIdleAni();
                      if (ai.character.teamType == TeamType.Red) {
                          ai.changeState(new PursuerRun());
                      }
                  }
              }
          }
          else {
              character.move(null);
          }
      }
      Exit(ai) {
          ai.character.sprite3d.offAllCaller(this);
      }
  }

  class AtkWall extends IState {
      constructor() {
          super(...arguments);
          this.atkIntval = 0;
          this.behavior = ScriptType.atkwall;
      }
      Enter(ai) {
          console.log("攻击状态");
          this.atkIntval = ai.character.atkIntval * 0.1;
      }
      Update(ai) {
          if (!ai.character.isLife())
              return;
          this.atkIntval -= Laya.timer.delta;
          if (this.atkIntval <= 0) {
              let target = null;
              if (ai.atkTarget && ai.atkTarget.isLife() && ai.character.transform.position.z - ai.atkTarget.transform.position.z < ai.character.atkRang) {
                  target = ai.atkTarget;
              }
              else {
                  target = GameMgr.Instance().getScenario();
                  ai.atkTarget = target;
              }
              if (ai.character && ai.character.isLife()) {
                  if (target) {
                      this.atkIntval = ai.character.atkIntval;
                      ai.character.onAtk(ai.atkTarget);
                      if (ai.character.getNearEnemyOfRang()) {
                          if (fc.platform.PlatformConfig.platformTag == 'iptv') {
                              let runBack = new PursuerRun();
                              ai.changeState(runBack);
                          }
                          else {
                              let runBack = new RunBack();
                              runBack.targetPosition = new Laya.Vector3(Util.clamp(ai.character.transform.position.x, -2, 2), 0, Util.clamp(ai.character.transform.position.z - 9, 5, 16));
                              ai.changeState(runBack);
                          }
                      }
                  }
                  else {
                      ai.character && ai.character.move(null);
                  }
              }
              else {
                  ai.character && ai.character.move(null);
              }
          }
      }
  }

  class Run extends IState {
      constructor() {
          super(...arguments);
          this.targetPosition = null;
          this.behavior = ScriptType.run;
      }
      Enter(ai) {
          ai.character.move(null);
          ai.character.playRunAni();
          let dir = Util.vectorDirection(this.targetPosition, ai.character.mtransform.position);
          dir.y = 0;
          Laya.Vector3.normalize(dir, ai.dir);
          let angle = Math.atan2(-ai.dir.x, -ai.dir.z) * 180 / Math.PI;
          ai.character.rotationRole(angle);
      }
      Update(ai) {
          let character = ai.character;
          if (character && character.isLife()) {
              if (ai.character.getNearEnemyOfRang()) {
                  ai.changeState(new PursuerRun());
              }
              else {
                  character.move(ai.dir);
                  let curPosition = character.mtransform.position;
                  let disSqr = Laya.Vector3.distanceSquared(curPosition, this.targetPosition);
                  if (disSqr < (character.atkRang) * (character.atkRang)) {
                      ai.changeState(new AtkWall());
                      ai.character.move(null);
                  }
              }
          }
          else {
              character.move(null);
          }
      }
      Exit(ai) {
          ai.character.sprite3d.offAllCaller(this);
      }
  }

  class PursuerRun extends IState {
      constructor() {
          super(...arguments);
          this.isRotationDiring = false;
          this.targetPosition = null;
          this.behavior = ScriptType.pursuerrun;
          this.mWall3D = null;
      }
      Enter(ai) {
          ai.character.move(null);
          ai.character.playRunAni();
          this.isRotationDiring = false;
      }
      Update(ai) {
          let character = ai.character;
          if (!character.isLife())
              return;
          if (character && character.isLife()) {
              let target = null;
              if (ai.atkTarget && ai.atkTarget.isLife()) {
                  target = ai.atkTarget;
              }
              else {
                  if (Util.probabilityCanHappen(5)) {
                      target = ai.character.getNearEnemyOfRang();
                  }
                  else {
                      target = ai.character.getNearEnemyOfRang();
                  }
                  ai.atkTarget = target;
                  if (ai.atkTarget && ai.atkTarget instanceof Character) {
                      ai.atkTarget.isAtkedLock = true;
                  }
              }
              if (!ai.atkTarget || !ai.atkTarget.isLife()) {
                  ai.character && ai.character.move(null);
                  if (ai.character.teamType == TeamType.Red) {
                      let run = new Run();
                      run.targetPosition = new Laya.Vector3(Util.clamp(ai.character.transform.position.x, -2, 2), 0, 0);
                      ai.changeState(run);
                  }
                  else {
                      let runBack = new RunBack();
                      runBack.targetPosition = new Laya.Vector3(ai.character.transform.position.x, 0, Util.clamp(ai.character.transform.position.z - 3, 5.01, 16));
                      ai.changeState(runBack);
                  }
                  return;
              }
              if (fc.platform.PlatformConfig.platformTag != 'iptv') {
                  if (ai.character.teamType == TeamType.Red && ai.character.nickName != 'BiaoQiangBing' && ai.character.transform.position.z < ai.character.atkRang) {
                      ai.character && ai.character.move(null);
                      ai.changeState(new AtkWall());
                      return;
                  }
              }
              let dir = Util.vectorDirection(ai.atkTarget.mtransform.position.clone(), character.mtransform.position);
              dir.y = 0;
              Laya.Vector3.normalize(dir, dir);
              let hitResult = character.rayCheckWall(dir, 0.8);
              if (hitResult && hitResult.normal) {
                  if (!this.isRotationDiring) {
                      let getVerticalDir = Util.GetVerticalDir(hitResult.normal);
                      if (Laya.Vector3.dot(dir, getVerticalDir) < 0) {
                          getVerticalDir.x = -getVerticalDir.x;
                          getVerticalDir.z = -getVerticalDir.z;
                      }
                      Laya.Vector3.normalize(getVerticalDir, getVerticalDir);
                      character.move(getVerticalDir);
                      let angle = Math.atan2(-getVerticalDir.x, -getVerticalDir.z) * 180 / Math.PI;
                      ai.character.rotationRole(angle);
                      this.isRotationDiring = true;
                      if (!hitResult.collider.owner.getComponent(Character)) {
                          this.mWall3D = hitResult.collider.owner;
                      }
                      else {
                          this.mWall3D = null;
                      }
                  }
                  else {
                      let hit = character.rayCheckWall(character.moveDir, 1);
                      if (hit && !hit.collider.owner.getComponent(Character)) {
                          character.moveDir.x = -character.moveDir.x;
                          character.moveDir.z = -character.moveDir.z;
                          character.move(character.moveDir);
                          let angle = Math.atan2(-character.moveDir.x, -character.moveDir.z) * 180 / Math.PI;
                          ai.character.rotationRole(angle);
                      }
                  }
              }
              else {
                  if (this.isRotationDiring) {
                      if (this.mWall3D) {
                          if (!character.intersectsWall(dir, 2, this.mWall3D)) {
                              character.dir = dir.clone();
                              character.move(character.dir);
                              let angle = Math.atan2(-dir.x, -dir.z) * 180 / Math.PI;
                              ai.character.rotationRole(angle);
                              this.isRotationDiring = false;
                              this.mWall3D = null;
                          }
                      }
                      else {
                          character.dir = dir.clone();
                          character.move(character.dir);
                          let angle = Math.atan2(-dir.x, -dir.z) * 180 / Math.PI;
                          ai.character.rotationRole(angle);
                          this.isRotationDiring = false;
                      }
                  }
                  else {
                      character.dir = dir.clone();
                      character.move(character.dir);
                      let angle = Math.atan2(-dir.x, -dir.z) * 180 / Math.PI;
                      ai.character.rotationRole(angle);
                      this.isRotationDiring = false;
                  }
              }
              let curPosition = character.mtransform.position;
              let enemyPosition = ai.atkTarget.mtransform.position;
              let disSqr = Laya.Vector3.distanceSquared(curPosition, enemyPosition);
              let rang = Math.max(character.atkRang, ai.atkTarget.atkRang);
              if (character.nickName == 'BiaoQiangBing' || ai.atkTarget.nickName == 'BiaoQiangBing') {
                  rang = character.atkRang;
              }
              if (disSqr < (rang) * (rang)) {
                  ai.changeState(new Atk());
                  ai.character.move(null);
              }
          }
          else {
              ai.character && ai.character.move(null);
          }
      }
      Exit(ai) {
          ai.character.sprite3d.offAllCaller(this);
      }
  }

  class Idle extends IState {
      constructor() {
          super(...arguments);
          this.idleTime = 0;
          this.behavior = ScriptType.idle;
      }
      Enter(ai) {
          ai.character.playIdleAni();
          this.idleTime = 0;
          ai.character.isPause = false;
          ai.character.move(null);
      }
      Update(ai) {
          this.idleTime -= Laya.timer.delta;
          if (this.idleTime <= 0) {
              if (ai.character.teamType == TeamType.Red && ai.character.nickName == 'BiaoQiangBing') {
                  let runBack = new RunBack();
                  runBack.targetPosition = new Laya.Vector3(ai.character.transform.position.x, 0, 21.5);
                  ai.changeState(runBack);
              }
              else {
                  ai.changeState(new PursuerRun());
              }
          }
      }
  }

  class ElementController extends Laya.Script3D {
      constructor() {
          super(...arguments);
          this.character = null;
          this.dir = new Laya.Vector3(0, 0, 1);
          this.curState = null;
          this.atkTarget = null;
      }
      onStart() {
          this.character = this.owner.getComponent(Character);
          this.sprite3d = this.owner;
          this.mTransform = this.sprite3d.transform;
          this.setBehaviorAndChangeState(this.behavior);
      }
      setBehavior(behavior) {
          this.behavior = behavior;
      }
      setBehaviorAndChangeState(behavior) {
          this.behavior = behavior;
          switch (this.behavior) {
              case ScriptType.run:
                  this.changeState(new Run());
                  break;
              case ScriptType.runback:
                  this.changeState(new RunBack());
                  break;
              case ScriptType.pursuerrun:
                  this.changeState(new PursuerRun());
                  break;
              case ScriptType.idle:
                  this.changeState(new Idle());
                  break;
              case ScriptType.atk:
                  this.changeState(new Atk());
                  break;
              default:
                  break;
          }
      }
      onEnable() {
          this.initEvent();
      }
      onDisable() {
          this.removeEvent();
      }
      initEvent() {
          this.owner.on(Character.EVENT_REVIVE, this, this.reset);
          this.owner.on(Character.EVENT_DIE, this, this.onDie);
          this.owner.on(Character.EVENT_CHANGESTATE, this, this.setBehaviorAndChangeState);
      }
      removeEvent() {
          this.owner.off(Character.EVENT_REVIVE, this, this.reset);
          this.owner.off(Character.EVENT_DIE, this, this.onDie);
          this.owner.off(Character.EVENT_CHANGESTATE, this, this.setBehaviorAndChangeState);
      }
      reset() {
          this.changeState(this.curState);
      }
      onDie() {
          this.changeState(null);
      }
      onUpdate() {
          if (this.curState == null || this.character == null || this.character.isPause)
              return;
          this.curState.Update(this);
      }
      changeState(newState) {
          if (this.curState && newState && newState.behavior == this.curState.behavior)
              return;
          if (this.curState != null) {
              this.curState.Exit(this);
              this.curState = null;
          }
          if (!newState)
              return;
          this.curState = newState;
          this.curState.Enter(this);
          this.behavior = this.curState.behavior;
      }
  }

  class QiBing extends Character {
      constructor() {
          super(...arguments);
          this.soldierController = null;
          this.mAtkedEf = null;
          this.mEfName = '';
      }
      onAwake() {
          super.onAwake();
      }
      onEnable() {
          super.onEnable();
      }
      onPause() {
          this.isPause = true;
          this.playIdleAni();
      }
      onStart() {
          super.onStart();
          this.collider3d = this.sprite3d.getComponent(Laya.PhysicsCollider);
          this.collider3d.enabled = true;
          this.collider3d.isTrigger = true;
          this.collider3d.collisionGroup = CollisionConst.GROUP_Soldier;
          this.collider3d.canCollideWith = CollisionConst.CAN_Soldier;
          this.animatorController = this.sprite3d.getChildByName('Cowboy').getComponent(AnimatorController);
          if (this.animatorController) {
              this.animatorController.destroy();
              this.animatorController = null;
          }
          this.animatorController = this.sprite3d.getChildByName('Cowboy').addComponent(AnimatorController);
          this.animatorController.character = this;
          this.hpPoint = this.sprite3d.getChildByName('hp');
          this.role3d = this.sprite3d.getChildByName('Cowboy').getChildByName('Anim_Container').getChildByName('CH_Cowboy_01_skMESH').getChildByName('TrajectorySHJnt').getChildByName('ROOTSHJnt').getChildByName('Spine_01SHJnt').getChildByName('point').getChildByName('Role_Cowboy');
          this.roleAnimatorController = this.role3d.getComponent(SoldierAnimatorController);
          if (this.roleAnimatorController) {
              this.roleAnimatorController.destroy();
              this.roleAnimatorController = null;
          }
          this.roleAnimatorController = this.role3d.addComponent(SoldierAnimatorController);
          this.roleAnimatorController.character = this;
          let lv = 0;
          if (this.teamType == TeamType.Red) {
              Util.changMat(ResConst.MaterialPath + ResConst.RoleMate[0], this.role3d.getChildByName('Spartan_Cowboy_01').skinnedMeshRenderer);
              this.enemyData = GameMgr.Instance().getEnemySoldierData(EnemyType.QiBing);
              lv = 1 + Math.floor((GameData.data.curStage - this.enemyData.upgrade) / this.enemyData.upgradeadd);
              lv = Util.clamp(lv, 1, this.enemyData.lv_max);
              this.mHp = this.enemyData.hp + this.enemyData.defense;
              this.mTotalHp = this.mHp;
              this.moveSpeed = this.enemyData.speed;
              this.atkRang = this.enemyData.atkdist;
              this.atkIntvalTemp = this.enemyData.interval;
              this.atkIntval = this.enemyData.interval;
              this.atkDamage = this.enemyData.atk;
              this.shotRang = 10;
              this.shotSpeed = this.enemyData.shotspeed;
              this.damageRang = this.enemyData.atkdist - Util.random(1, 4) * 0.1;
          }
          else {
              Util.changMat(ResConst.MaterialPath + ResConst.RoleMate[1], this.role3d.getChildByName('Spartan_Cowboy_01').skinnedMeshRenderer);
              lv = Math.max(1, GameData.data.getWarriorLv(WarriorType.QiBing));
              if (lv <= 1) {
                  GameData.data.setWarriorLv(1, WarriorType.QiBing);
              }
              this.soldierData = GameMgr.Instance().getSoldierData(WarriorType.QiBing, lv);
              this.mHp = this.soldierData.hp + this.soldierData.defense;
              this.mTotalHp = this.mHp;
              this.moveSpeed = this.soldierData.speed;
              this.atkRang = this.soldierData.atkdist;
              this.atkIntvalTemp = this.soldierData.interval;
              this.atkIntval = this.soldierData.interval;
              this.atkDamage = this.soldierData.atk;
              this.shotRang = 10;
              this.shotSpeed = this.soldierData.shotspeed;
              this.damageRang = this.soldierData.atkdist - Util.random(1, 4) * 0.1;
          }
          let stage = 1 + Math.floor((lv - 1) / 3);
          stage = Math.min(ResConst.WarriorLvName[3].length, stage);
          let hand = this.role3d.getChildByName('Bip001').getChildByName('Bip001 Pelvis').getChildByName('Bip001 Spine').getChildByName('Bip001 Spine1').getChildByName('Bip001 Neck').getChildByName('Bip001 Head').getChildByName('head_point');
          let shield = this.role3d.getChildByName('Bip001').getChildByName('Bip001 Pelvis').getChildByName('Bip001 Spine').getChildByName('Bip001 Spine1').getChildByName('Bip001 Neck').getChildByName('Bip001 L Clavicle').getChildByName('Bip001 L UpperArm').getChildByName('Bip001 L Forearm').getChildByName('Bip001 L Hand').getChildByName('Left_shield');
          let weapons = this.role3d.getChildByName('Bip001').getChildByName('Bip001 Pelvis').getChildByName('Bip001 Spine').getChildByName('Bip001 Spine1').getChildByName('Bip001 Neck').getChildByName('Bip001 R Clavicle').getChildByName('Bip001 R UpperArm').getChildByName('Bip001 R Forearm').getChildByName('Bip001 R Hand').getChildByName('Right_weapons');
          Util.activeAllChild(hand, false);
          Util.activeAllChild(shield, false);
          Util.activeAllChild(weapons, false);
          switch (stage) {
              case 1:
                  hand.getChildByName('Helmet_01').active = true;
                  weapons.getChildByName('Spear').active = true;
                  break;
              case 2:
                  hand.getChildByName('Helmet_02').active = true;
                  weapons.getChildByName('Spear').active = true;
                  break;
              case 3:
                  hand.getChildByName('Helmet_03').active = true;
                  weapons.getChildByName('Spear').active = true;
                  break;
              default:
                  break;
          }
          if (this.teamType != TeamType.Neutral) {
              this.soldierController = this.sprite3d.addComponent(ElementController);
              this.soldierController.setBehavior(ScriptType.idle);
          }
          if (this.mAtkedEf) {
              this.mAtkedEf.removeSelf();
              this.mAtkedEf.destroy();
          }
          if (!this.healthBar) {
              this.healthBar = HealthBar.getHealthBar(this.nickName, this.mTotalHp, this.teamType == TeamType.Red);
              this.onResume();
          }
          this.healthBar.setActive(false);
      }
      onResume() {
          super.onResume();
      }
      initEvent() {
          super.initEvent();
      }
      removeEvent() {
          super.removeEvent();
      }
      playIdleAni() {
          this.animatorController && (this.animatorController.play(AnimatorController.idle));
      }
      playAtkAni() {
          this.roleAnimatorController && (this.roleAnimatorController.play(SoldierAnimatorController.ci));
      }
      playRunAni() {
          this.animatorController && (this.animatorController.play(AnimatorController.run));
      }
      playDieAni() {
          this.animatorController && (this.animatorController.play(AnimatorController.die));
      }
      onUpdate() {
          if (!this.isLife())
              return;
          if (GameData.data.gameFinish)
              return;
          if (Laya.timer.delta > 500)
              return;
          let position = this.mtransform.position.clone();
          let movevelocity = this.movevelocity.clone();
          position.z += movevelocity.z;
          position.x += movevelocity.x;
          this.mtransform.position = position;
          if (this.moveDir.x == 0 && this.moveDir.z == 0) {
              this.playIdleAni();
              this.move(null);
          }
          else {
              this.playRunAni();
          }
          this.atkedInterval -= Laya.timer.delta;
      }
      onLateUpdate() {
          if (!this.isLife())
              return;
          if (GameData.data.gameFinish)
              return;
          this.healthBar && this.healthBar.updatePosition(this.hpPoint.transform.position);
      }
      move(dir) {
          if (!dir || dir == null) {
              this.movevelocity.x = 0;
              this.movevelocity.z = 0;
              this.moveDir.setValue(0, 0, 0);
              return;
          }
          if ((dir.x == 0 && dir.z == 0))
              return;
          Laya.Vector3.normalize(dir, dir);
          this.moveDir.x = dir.x;
          this.moveDir.z = dir.z;
          let curSpeed = this.moveSpeed * Laya.timer.delta * 0.001;
          this.movevelocity.x = -dir.x * curSpeed;
          this.movevelocity.y = 0;
          this.movevelocity.z = -dir.z * curSpeed;
      }
      rotationRole(angle) {
          let angleY = this.sprite3d.transform.rotationEuler.y;
          let rotateY = angle - angleY;
          if (rotateY != 0) {
              this.sprite3d.transform.rotate(new Laya.Vector3(0, rotateY, 0), false, false);
          }
      }
      onAtk(atkedObj) {
          this.atkedTarget = atkedObj;
          this.playAtkAni();
          if (atkedObj instanceof QiangBing) {
              AudioManager.Instance.playSfx2('qiangvsqiang.mp3');
          }
          else if (atkedObj instanceof DaoBing) {
              AudioManager.Instance.playSfx2('daovsqiang.mp3');
          }
          else if (atkedObj instanceof QiBing) {
              AudioManager.Instance.playSfx2('qiangvsqiang.mp3');
          }
          else {
              AudioManager.Instance.playSfx2('daojian.mp3');
          }
          this.healthBar.setActive(true);
          atkedObj.onDamage(this.atkDamage);
      }
      onDamage(damage) {
          if (this.atkedInterval > 0)
              return;
          if (GameData.data.gameFinish || !this.isLife())
              return;
          if (this.teamType == TeamType.Red) {
              let bounslv = Math.max(1, GameData.data.getCommonLv(CommonCfg.HaoJiao));
              let bunsdata = GameData.data.commonCfg.HaoJiao;
              damage = Math.floor(damage * (1 + GameData.data.atkUp * (bounslv) * bunsdata.bonus1));
          }
          this.mHp -= damage;
          this.atkedInterval = 0;
          console.log('剩余血量', this.mHp);
          this.addDamageEf();
          this.healthBar.updateHp(this.mHp / this.mTotalHp, Math.max(0, this.mHp));
          this.healthBar.setActive(true);
          if (this.mHp <= 0) {
              this.healthBar.setActive(false);
              HealthBar.recover(this.healthBar);
              this.playDieAni();
              if (this.teamType == TeamType.Blue) {
                  GameMgr.Instance().playerToRemoveList(this);
                  Laya.stage.timerOnce(1000, this, () => {
                      GameMgr.Instance().removePlayer(this);
                  }, null, false);
              }
              else {
                  GameMgr.Instance().enemyToRemoveList(this);
                  Laya.stage.timerOnce(1000, this, () => {
                      GameMgr.Instance().removeEnemy(this);
                  }, null, false);
              }
              if (this.teamType == TeamType.Red) {
                  let bounslv = Math.max(1, GameData.data.getCommonLv(CommonCfg.JinBi));
                  let bunsdata = GameData.data.commonCfg.JinBi;
                  let gold = this.enemyData.gold + GameData.data.curStage * this.enemyData.goldadd * 0.5;
                  gold = Math.floor(gold * (1 + ((bounslv - 1) * bunsdata.bonus1)));
                  Laya.stage.event(EventConst.Game_Update_Gold, gold);
              }
          }
      }
      onDestroy() {
          if (this.mAtkedEf) {
              EffectMgr.Instance().removeSpriteEffect(this.mAtkedEf, true);
          }
      }
      addDamageEf() {
      }
      getNearEnemyOfAtkRang(role = null) {
          let characterList = [];
          if (this.teamType == TeamType.Red) {
              characterList = GameMgr.Instance().getPlayerList().concat();
          }
          else {
              characterList = GameMgr.Instance().getEnemyList().concat();
          }
          let target = null;
          let disSqr = this.atkRang * this.atkRang;
          for (let i = characterList.length - 1; i >= 0; i--) {
              const character = characterList[i];
              if (character == this || !character.isLife() || !character.sprite3d.active)
                  continue;
              let newDisSqr = Laya.Vector3.distanceSquared(character.mtransform.position, this.mtransform.position);
              if (newDisSqr <= disSqr && character != role) {
                  if (target == null || disSqr > newDisSqr) {
                      target = character;
                      disSqr = newDisSqr;
                  }
              }
          }
          return target;
      }
      getNearEnemyOfRang(role = null) {
          let characterList = [];
          if (this.teamType == TeamType.Red) {
              characterList = GameMgr.Instance().getPlayerList().concat();
          }
          else {
              characterList = GameMgr.Instance().getEnemyList().concat();
          }
          let target = null;
          let disSqr = 100000;
          for (let i = characterList.length - 1; i >= 0; i--) {
              const character = characterList[i];
              if (character == this || !character.isLife() || !character.sprite3d.active)
                  continue;
              if (this.teamType == TeamType.Blue && character.transform.position.z > 22) {
                  continue;
              }
              let newDisSqr = Laya.Vector3.distanceSquared(character.mtransform.position, this.mtransform.position);
              if (newDisSqr <= disSqr && character != role) {
                  if (target == null || disSqr > newDisSqr) {
                      target = character;
                      disSqr = newDisSqr;
                  }
              }
          }
          return target;
      }
      getRandomEnemyOfRang(role = null) {
          let characterList = [];
          if (this.teamType == TeamType.Red) {
              characterList = GameMgr.Instance().getPlayerList().concat();
          }
          else {
              characterList = GameMgr.Instance().getEnemyList().concat();
          }
          let target = characterList[Util.random(0, characterList.length)];
          return target;
      }
      rayCheckWall(dir, rang) {
          if (!dir || dir == null) {
              return null;
          }
          var from = new Laya.Vector3(0, 0, 0);
          Laya.Vector3.normalize(dir, from);
          var to = new Laya.Vector3(0, 0, 0);
          var raylenght = new Laya.Vector3(0, 0, 0);
          Laya.Vector3.scale(from, -rang, raylenght);
          Laya.Vector3.add(this.mtransform.position.clone(), raylenght, to);
          this.mtransform.position.cloneTo(from);
          from.y = 1;
          to.y = 1;
          var hitResults = [];
          this.scene3d.physicsSimulation.raycastAllFromTo(from, to, hitResults, CollisionConst.GROUP_RAY, CollisionConst.CAN_RAY);
          for (let i = 0, n = hitResults.length; i < n; i++) {
              var otherSprite3d = hitResults[i].collider.owner;
              if (otherSprite3d && otherSprite3d.active) {
                  var character = otherSprite3d.getComponent(Character);
                  if (!character || (character != this && (character.teamType == this.teamType))) {
                      let hitResult = hitResults[i];
                      return hitResult;
                  }
              }
          }
          return null;
      }
  }

  class QiangBing extends Character {
      constructor() {
          super(...arguments);
          this.soldierController = null;
          this.mAtkedEf = null;
          this.mEfName = '';
      }
      onAwake() {
          super.onAwake();
      }
      onEnable() {
          super.onEnable();
      }
      onPause() {
          this.isPause = true;
          this.playIdleAni();
      }
      onStart() {
          super.onStart();
          this.collider3d = this.sprite3d.getComponent(Laya.PhysicsCollider);
          this.collider3d.enabled = true;
          this.collider3d.isTrigger = true;
          this.collider3d.collisionGroup = CollisionConst.GROUP_Soldier;
          this.collider3d.canCollideWith = CollisionConst.CAN_Soldier;
          this.animatorController = this.sprite3d.getComponent(SoldierAnimatorController);
          if (this.animatorController) {
              this.animatorController.destroy();
              this.animatorController = null;
          }
          this.animatorController = this.sprite3d.addComponent(SoldierAnimatorController);
          this.animatorController.character = this;
          this.hpPoint = this.sprite3d.getChildByName('hp');
          let lv = 0;
          if (this.teamType == TeamType.Red) {
              Util.changMat(ResConst.MaterialPath + ResConst.RoleMate[0], this.sprite3d.getChildByName('CH_Spartan_Melee_01').skinnedMeshRenderer);
              this.enemyData = GameMgr.Instance().getEnemySoldierData(EnemyType.QiangBing);
              lv = 1 + Math.floor((GameData.data.curStage - this.enemyData.upgrade) / this.enemyData.upgradeadd);
              lv = Util.clamp(lv, 1, this.enemyData.lv_max);
              this.mHp = this.enemyData.hp + this.enemyData.defense;
              this.mTotalHp = this.mHp;
              this.moveSpeed = this.enemyData.speed;
              this.atkRang = this.enemyData.atkdist;
              this.atkIntvalTemp = this.enemyData.interval;
              this.atkIntval = this.enemyData.interval;
              this.atkDamage = this.enemyData.atk;
              this.shotRang = 10;
              this.shotSpeed = this.enemyData.shotspeed;
              this.damageRang = this.enemyData.atkdist - Util.random(1, 4) * 0.1;
          }
          else {
              Util.changMat(ResConst.MaterialPath + ResConst.RoleMate[1], this.sprite3d.getChildByName('CH_Spartan_Melee_01').skinnedMeshRenderer);
              lv = Math.max(1, GameData.data.getWarriorLv(WarriorType.QiangBing));
              if (lv <= 1) {
                  GameData.data.setWarriorLv(1, WarriorType.QiangBing);
              }
              this.soldierData = GameMgr.Instance().getSoldierData(WarriorType.QiangBing, lv);
              this.mHp = this.soldierData.hp + this.soldierData.defense;
              this.mTotalHp = this.mHp;
              this.moveSpeed = this.soldierData.speed;
              this.atkRang = this.soldierData.atkdist;
              this.atkIntvalTemp = this.soldierData.interval;
              this.atkIntval = this.soldierData.interval;
              this.atkDamage = this.soldierData.atk;
              this.shotRang = 10;
              this.shotSpeed = this.soldierData.shotspeed;
              this.damageRang = this.soldierData.atkdist - Util.random(1, 4) * 0.1;
          }
          let stage = 1 + Math.floor((lv - 1) / 3);
          stage = Math.min(ResConst.WarriorLvName[1].length, stage);
          let hand = this.sprite3d.getChildByName('Bip001').getChildByName('Bip001 Pelvis').getChildByName('Bip001 Spine').getChildByName('Bip001 Spine1').getChildByName('Bip001 Neck').getChildByName('Bip001 Head').getChildByName('head_point');
          let shield = this.sprite3d.getChildByName('Bip001').getChildByName('Bip001 Pelvis').getChildByName('Bip001 Spine').getChildByName('Bip001 Spine1').getChildByName('Bip001 Neck').getChildByName('Bip001 L Clavicle').getChildByName('Bip001 L UpperArm').getChildByName('Bip001 L Forearm').getChildByName('Bip001 L Hand').getChildByName('Left_shield');
          let weapons = this.sprite3d.getChildByName('Bip001').getChildByName('Bip001 Pelvis').getChildByName('Bip001 Spine').getChildByName('Bip001 Spine1').getChildByName('Bip001 Neck').getChildByName('Bip001 R Clavicle').getChildByName('Bip001 R UpperArm').getChildByName('Bip001 R Forearm').getChildByName('Bip001 R Hand').getChildByName('Right_weapons');
          Util.activeAllChild(hand, false);
          Util.activeAllChild(shield, false);
          Util.activeAllChild(weapons, false);
          switch (stage) {
              case 1:
                  hand.getChildByName('Helmet_01').active = true;
                  weapons.getChildByName('Fork').active = true;
                  break;
              case 2:
                  hand.getChildByName('Helmet_01').active = true;
                  weapons.getChildByName('Spear').active = true;
                  break;
              case 3:
                  hand.getChildByName('Helmet_02').active = true;
                  weapons.getChildByName('Spear').active = true;
                  shield.getChildByName('Steele_01').active = true;
                  break;
              case 4:
                  hand.getChildByName('Helmet_03').active = true;
                  weapons.getChildByName('Spear').active = true;
                  shield.getChildByName('Steele_01').active = true;
                  break;
              case 5:
                  hand.getChildByName('Helmet_03').active = true;
                  weapons.getChildByName('Spear').active = true;
                  shield.getChildByName('Steele_02').active = true;
                  break;
              default:
                  break;
          }
          if (this.teamType != TeamType.Neutral) {
              this.soldierController = this.sprite3d.addComponent(ElementController);
              this.soldierController.setBehavior(ScriptType.idle);
          }
          if (this.mAtkedEf) {
              this.mAtkedEf.removeSelf();
              this.mAtkedEf.destroy();
          }
          if (!this.healthBar) {
              this.healthBar = HealthBar.getHealthBar(this.nickName, this.mTotalHp, this.teamType == TeamType.Red);
              this.onResume();
          }
          this.healthBar.setActive(false);
      }
      onResume() {
          super.onResume();
      }
      initEvent() {
          super.initEvent();
      }
      removeEvent() {
          super.removeEvent();
      }
      playIdleAni() {
          this.animatorController && (this.animatorController.play(SoldierAnimatorController.idle));
      }
      playAtkAni() {
          this.animatorController && (this.animatorController.play(SoldierAnimatorController.ci));
      }
      playRunAni() {
          this.animatorController && (this.animatorController.play(SoldierAnimatorController.run));
      }
      playDieAni() {
          this.animatorController && (this.animatorController.play(SoldierAnimatorController.die));
      }
      onUpdate() {
          if (!this.isLife())
              return;
          if (GameData.data.gameFinish)
              return;
          if (Laya.timer.delta > 500)
              return;
          let position = this.mtransform.position.clone();
          let movevelocity = this.movevelocity.clone();
          position.z += movevelocity.z;
          position.x += movevelocity.x;
          this.mtransform.position = position;
          if (this.moveDir.x == 0 && this.moveDir.z == 0) {
              this.move(null);
          }
          else {
              this.playRunAni();
          }
          this.atkedInterval -= Laya.timer.delta;
      }
      onLateUpdate() {
          if (!this.isLife())
              return;
          if (GameData.data.gameFinish)
              return;
          this.healthBar && this.healthBar.updatePosition(this.hpPoint.transform.position);
      }
      move(dir) {
          if (!dir || dir == null) {
              this.movevelocity.x = 0;
              this.movevelocity.z = 0;
              this.moveDir.setValue(0, 0, 0);
              return;
          }
          if ((dir.x == 0 && dir.z == 0))
              return;
          Laya.Vector3.normalize(dir, dir);
          this.moveDir.x = dir.x;
          this.moveDir.z = dir.z;
          let curSpeed = this.moveSpeed * Laya.timer.delta * 0.001;
          this.movevelocity.x = -dir.x * curSpeed;
          this.movevelocity.y = 0;
          this.movevelocity.z = -dir.z * curSpeed;
      }
      rotationRole(angle) {
          let angleY = this.sprite3d.transform.rotationEuler.y;
          let rotateY = angle - angleY;
          if (rotateY != 0) {
              this.sprite3d.transform.rotate(new Laya.Vector3(0, rotateY, 0), false, false);
          }
      }
      onAtk(atkedObj) {
          this.atkedTarget = atkedObj;
          this.playAtkAni();
          if (atkedObj instanceof QiangBing) {
              AudioManager.Instance.playSfx2('qiangvsqiang.mp3');
          }
          else if (atkedObj instanceof DaoBing) {
              AudioManager.Instance.playSfx2('daovsqiang.mp3');
          }
          else if (atkedObj instanceof QiBing) {
              AudioManager.Instance.playSfx2('qiangvsqiang.mp3');
          }
          else {
              AudioManager.Instance.playSfx2('daojian.mp3');
          }
          this.healthBar.setActive(true);
          atkedObj.onDamage(this.atkDamage);
      }
      onDamage(damage) {
          if (this.atkedInterval > 0)
              return;
          if (GameData.data.gameFinish || !this.isLife())
              return;
          if (this.teamType == TeamType.Red) {
              let bounslv = Math.max(1, GameData.data.getCommonLv(CommonCfg.HaoJiao));
              let bunsdata = GameData.data.commonCfg.HaoJiao;
              damage = Math.floor(damage * (1 + GameData.data.atkUp * (bounslv) * bunsdata.bonus1));
          }
          this.mHp -= damage;
          this.atkedInterval = 0;
          console.log('剩余血量', this.mHp);
          this.addDamageEf();
          this.healthBar.updateHp(this.mHp / this.mTotalHp, Math.max(0, this.mHp));
          this.healthBar.setActive(true);
          if (this.mHp <= 0) {
              this.healthBar.setActive(false);
              HealthBar.recover(this.healthBar);
              this.playDieAni();
              if (this.teamType == TeamType.Blue) {
                  GameMgr.Instance().playerToRemoveList(this);
                  Laya.stage.timerOnce(1000, this, () => {
                      GameMgr.Instance().removePlayer(this);
                  }, null, false);
              }
              else {
                  GameMgr.Instance().enemyToRemoveList(this);
                  Laya.stage.timerOnce(1000, this, () => {
                      GameMgr.Instance().removeEnemy(this);
                  }, null, false);
              }
              if (this.teamType == TeamType.Red) {
                  let bounslv = Math.max(1, GameData.data.getCommonLv(CommonCfg.JinBi));
                  let bunsdata = GameData.data.commonCfg.JinBi;
                  let gold = this.enemyData.gold + GameData.data.curStage * this.enemyData.goldadd * 0.5;
                  gold = Math.floor(gold * (1 + ((bounslv - 1) * bunsdata.bonus1)));
                  Laya.stage.event(EventConst.Game_Update_Gold, gold);
              }
          }
      }
      onDestroy() {
          if (this.mAtkedEf) {
              EffectMgr.Instance().removeSpriteEffect(this.mAtkedEf, true);
          }
      }
      addDamageEf() {
      }
      getNearEnemyOfAtkRang(role = null) {
          let characterList = [];
          if (this.teamType == TeamType.Red) {
              characterList = GameMgr.Instance().getPlayerList().concat();
          }
          else {
              characterList = GameMgr.Instance().getEnemyList().concat();
          }
          let target = null;
          let disSqr = this.atkRang * this.atkRang;
          for (let i = characterList.length - 1; i >= 0; i--) {
              const character = characterList[i];
              if (character == this || !character.isLife() || !character.sprite3d.active)
                  continue;
              if (this.teamType == TeamType.Blue && character.transform.position.z > 22) {
                  continue;
              }
              let newDisSqr = Laya.Vector3.distanceSquared(character.mtransform.position, this.mtransform.position);
              if (newDisSqr <= disSqr && character != role) {
                  if (target == null || disSqr > newDisSqr) {
                      target = character;
                      disSqr = newDisSqr;
                  }
              }
          }
          return target;
      }
      getNearEnemyOfRang(role = null) {
          let characterList = [];
          if (this.teamType == TeamType.Red) {
              characterList = GameMgr.Instance().getPlayerList().concat();
          }
          else {
              characterList = GameMgr.Instance().getEnemyList().concat();
          }
          let target = null;
          let disSqr = 100000;
          for (let i = characterList.length - 1; i >= 0; i--) {
              const character = characterList[i];
              if (character == this || !character.isLife() || !character.sprite3d.active)
                  continue;
              let newDisSqr = Laya.Vector3.distanceSquared(character.mtransform.position, this.mtransform.position);
              if (newDisSqr <= disSqr && character != role) {
                  if (target == null || disSqr > newDisSqr) {
                      target = character;
                      disSqr = newDisSqr;
                  }
              }
          }
          return target;
      }
      getRandomEnemyOfRang(role = null) {
          let characterList = [];
          if (this.teamType == TeamType.Red) {
              characterList = GameMgr.Instance().getPlayerList().concat();
          }
          else {
              characterList = GameMgr.Instance().getEnemyList().concat();
          }
          let target = characterList[Util.random(0, characterList.length)];
          return target;
      }
      rayCheckWall(dir, rang) {
          if (!dir || dir == null) {
              return null;
          }
          var from = new Laya.Vector3(0, 0, 0);
          Laya.Vector3.normalize(dir, from);
          var to = new Laya.Vector3(0, 0, 0);
          var raylenght = new Laya.Vector3(0, 0, 0);
          Laya.Vector3.scale(from, -rang, raylenght);
          Laya.Vector3.add(this.mtransform.position.clone(), raylenght, to);
          this.mtransform.position.cloneTo(from);
          from.y = 1;
          to.y = 1;
          var hitResults = [];
          this.scene3d.physicsSimulation.raycastAllFromTo(from, to, hitResults, CollisionConst.GROUP_RAY, CollisionConst.CAN_RAY);
          for (let i = 0, n = hitResults.length; i < n; i++) {
              var otherSprite3d = hitResults[i].collider.owner;
              if (otherSprite3d && otherSprite3d.active) {
                  var character = otherSprite3d.getComponent(Character);
                  if (!character || (character != this && (character.teamType == this.teamType))) {
                      let hitResult = hitResults[i];
                      return hitResult;
                  }
              }
          }
          return null;
      }
  }

  class DaoBing extends Character {
      constructor() {
          super(...arguments);
          this.soldierController = null;
          this.mAtkedEf = null;
          this.mEfName = '';
      }
      onAwake() {
          super.onAwake();
      }
      onEnable() {
          super.onEnable();
      }
      onPause() {
          this.isPause = true;
          this.playIdleAni();
      }
      onStart() {
          super.onStart();
          this.collider3d = this.sprite3d.getComponent(Laya.PhysicsCollider);
          this.collider3d.enabled = true;
          this.collider3d.isTrigger = true;
          this.collider3d.collisionGroup = CollisionConst.GROUP_Soldier;
          this.collider3d.canCollideWith = CollisionConst.CAN_Soldier;
          this.animatorController = this.sprite3d.getComponent(SoldierAnimatorController);
          if (this.animatorController) {
              this.animatorController.destroy();
              this.animatorController = null;
          }
          this.animatorController = this.sprite3d.addComponent(SoldierAnimatorController);
          this.animatorController.character = this;
          this.hpPoint = this.sprite3d.getChildByName('hp');
          let lv = 0;
          if (this.teamType == TeamType.Red) {
              Util.changMat(ResConst.MaterialPath + ResConst.RoleMate[0], this.sprite3d.getChildByName('CH_Spartan_Melee_01').skinnedMeshRenderer);
              this.enemyData = GameMgr.Instance().getEnemySoldierData(EnemyType.DaoBing);
              lv = 1 + Math.floor((GameData.data.curStage - this.enemyData.upgrade) / this.enemyData.upgradeadd);
              lv = Util.clamp(lv, 1, this.enemyData.lv_max);
              this.mHp = this.enemyData.hp + this.enemyData.defense;
              this.mTotalHp = this.mHp;
              this.moveSpeed = this.enemyData.speed;
              this.atkRang = this.enemyData.atkdist;
              this.atkIntvalTemp = this.enemyData.interval;
              this.atkIntval = this.enemyData.interval;
              this.atkDamage = this.enemyData.atk;
              this.shotRang = 10;
              this.shotSpeed = this.enemyData.shotspeed;
              this.damageRang = this.enemyData.atkdist - Util.random(1, 4) * 0.1;
          }
          else {
              Util.changMat(ResConst.MaterialPath + ResConst.RoleMate[1], this.sprite3d.getChildByName('CH_Spartan_Melee_01').skinnedMeshRenderer);
              lv = Math.max(1, GameData.data.getWarriorLv(WarriorType.DaoBing));
              if (lv <= 1) {
                  GameData.data.setWarriorLv(1, WarriorType.DaoBing);
              }
              this.soldierData = GameMgr.Instance().getSoldierData(WarriorType.DaoBing, lv);
              this.mHp = this.soldierData.hp + this.soldierData.defense;
              this.mTotalHp = this.mHp;
              this.moveSpeed = this.soldierData.speed;
              this.atkRang = this.soldierData.atkdist;
              this.atkIntvalTemp = this.soldierData.interval;
              this.atkIntval = this.soldierData.interval;
              this.atkDamage = this.soldierData.atk;
              this.shotRang = 10;
              this.shotSpeed = this.soldierData.shotspeed;
              this.damageRang = this.soldierData.atkdist - Util.random(1, 4) * 0.1;
          }
          let stage = 1 + Math.floor((lv - 1) / 3);
          stage = Math.min(ResConst.WarriorLvName[0].length, stage);
          let hand = this.sprite3d.getChildByName('Bip001').getChildByName('Bip001 Pelvis').getChildByName('Bip001 Spine').getChildByName('Bip001 Spine1').getChildByName('Bip001 Neck').getChildByName('Bip001 Head').getChildByName('head_point');
          let shield = this.sprite3d.getChildByName('Bip001').getChildByName('Bip001 Pelvis').getChildByName('Bip001 Spine').getChildByName('Bip001 Spine1').getChildByName('Bip001 Neck').getChildByName('Bip001 L Clavicle').getChildByName('Bip001 L UpperArm').getChildByName('Bip001 L Forearm').getChildByName('Bip001 L Hand').getChildByName('Left_shield');
          let weapons = this.sprite3d.getChildByName('Bip001').getChildByName('Bip001 Pelvis').getChildByName('Bip001 Spine').getChildByName('Bip001 Spine1').getChildByName('Bip001 Neck').getChildByName('Bip001 R Clavicle').getChildByName('Bip001 R UpperArm').getChildByName('Bip001 R Forearm').getChildByName('Bip001 R Hand').getChildByName('Right_weapons');
          Util.activeAllChild(hand, false);
          Util.activeAllChild(shield, false);
          Util.activeAllChild(weapons, false);
          switch (stage) {
              case 1:
                  hand.getChildByName('Helmet_01').active = true;
                  weapons.getChildByName('Sword').active = true;
                  break;
              case 2:
                  hand.getChildByName('Helmet_01').active = true;
                  weapons.getChildByName('Sword').active = true;
                  shield.getChildByName('Steele_01').active = true;
                  break;
              case 3:
                  hand.getChildByName('Helmet_01').active = true;
                  weapons.getChildByName('Knife').active = true;
                  shield.getChildByName('Steele_01').active = true;
                  break;
              case 4:
                  hand.getChildByName('Helmet_02').active = true;
                  weapons.getChildByName('Knife').active = true;
                  shield.getChildByName('Steele_02').active = true;
                  break;
              case 5:
                  hand.getChildByName('Helmet_03').active = true;
                  weapons.getChildByName('Knife').active = true;
                  shield.getChildByName('Steele_02').active = true;
                  break;
              default:
                  break;
          }
          if (this.teamType != TeamType.Neutral) {
              this.soldierController = this.sprite3d.addComponent(ElementController);
              this.soldierController.setBehavior(ScriptType.idle);
          }
          if (this.mAtkedEf) {
              this.mAtkedEf.removeSelf();
              this.mAtkedEf.destroy();
          }
          if (!this.healthBar) {
              this.healthBar = HealthBar.getHealthBar(this.nickName, this.mTotalHp, this.teamType == TeamType.Red);
              this.onResume();
          }
          this.healthBar.setActive(false);
      }
      onResume() {
          super.onResume();
      }
      initEvent() {
          super.initEvent();
      }
      removeEvent() {
          super.removeEvent();
      }
      playIdleAni() {
          this.animatorController && (this.animatorController.play(SoldierAnimatorController.idle));
      }
      playAtkAni() {
          this.animatorController && (this.animatorController.play(SoldierAnimatorController.kan));
      }
      playRunAni() {
          this.animatorController && (this.animatorController.play(SoldierAnimatorController.run));
      }
      playDieAni() {
          this.animatorController && (this.animatorController.play(SoldierAnimatorController.die));
      }
      onUpdate() {
          if (!this.isLife())
              return;
          if (GameData.data.gameFinish)
              return;
          if (Laya.timer.delta > 500)
              return;
          let position = this.mtransform.position.clone();
          let movevelocity = this.movevelocity.clone();
          position.z += movevelocity.z;
          position.x += movevelocity.x;
          this.mtransform.position = position;
          if (this.moveDir.x == 0 && this.moveDir.z == 0) {
              this.move(null);
          }
          else {
              this.playRunAni();
          }
          this.atkedInterval -= Laya.timer.delta;
      }
      onLateUpdate() {
          if (!this.isLife())
              return;
          if (GameData.data.gameFinish)
              return;
          this.healthBar && this.healthBar.updatePosition(this.hpPoint.transform.position);
      }
      move(dir) {
          if (!dir || dir == null) {
              this.movevelocity.x = 0;
              this.movevelocity.z = 0;
              this.moveDir.setValue(0, 0, 0);
              return;
          }
          if ((dir.x == 0 && dir.z == 0))
              return;
          Laya.Vector3.normalize(dir, dir);
          this.moveDir.x = dir.x;
          this.moveDir.z = dir.z;
          let curSpeed = this.moveSpeed * Laya.timer.delta * 0.001;
          this.movevelocity.x = -dir.x * curSpeed;
          this.movevelocity.y = 0;
          this.movevelocity.z = -dir.z * curSpeed;
      }
      rotationRole(angle) {
          let angleY = this.sprite3d.transform.rotationEuler.y;
          let rotateY = angle - angleY;
          if (rotateY != 0) {
              this.sprite3d.transform.rotate(new Laya.Vector3(0, rotateY, 0), false, false);
          }
      }
      onAtk(atkedObj) {
          this.atkedTarget = atkedObj;
          this.playAtkAni();
          if (atkedObj instanceof QiangBing) {
              AudioManager.Instance.playSfx2('daovsqiang.mp3');
          }
          else if (atkedObj instanceof DaoBing) {
              AudioManager.Instance.playSfx2('daovsdao.mp3');
          }
          else if (atkedObj instanceof QiBing) {
              AudioManager.Instance.playSfx2('daovsqiang.mp3');
          }
          else {
              AudioManager.Instance.playSfx2('daojian.mp3');
          }
          this.healthBar.setActive(true);
          atkedObj.onDamage(this.atkDamage);
      }
      onDamage(damage) {
          if (this.atkedInterval > 0)
              return;
          if (GameData.data.gameFinish || !this.isLife())
              return;
          if (this.teamType == TeamType.Red) {
              let bounslv = Math.max(1, GameData.data.getCommonLv(CommonCfg.HaoJiao));
              let bunsdata = GameData.data.commonCfg.HaoJiao;
              damage = Math.floor(damage * (1 + GameData.data.atkUp * (bounslv) * bunsdata.bonus1));
          }
          this.mHp -= damage;
          this.atkedInterval = 0;
          console.log('剩余血量', this.mHp);
          this.addDamageEf();
          this.healthBar.updateHp(this.mHp / this.mTotalHp, Math.max(0, this.mHp));
          this.healthBar.setActive(true);
          if (this.mHp <= 0) {
              this.healthBar.setActive(false);
              HealthBar.recover(this.healthBar);
              this.setColliderEnable(false);
              this.playDieAni();
              if (this.teamType == TeamType.Blue) {
                  GameMgr.Instance().playerToRemoveList(this);
                  Laya.stage.timerOnce(1000, this, () => {
                      GameMgr.Instance().removePlayer(this);
                  }, null, false);
              }
              else {
                  GameMgr.Instance().enemyToRemoveList(this);
                  Laya.stage.timerOnce(1000, this, () => {
                      GameMgr.Instance().removeEnemy(this);
                  }, null, false);
              }
              if (this.teamType == TeamType.Red) {
                  let bounslv = Math.max(1, GameData.data.getCommonLv(CommonCfg.JinBi));
                  let bunsdata = GameData.data.commonCfg.JinBi;
                  let gold = this.enemyData.gold + GameData.data.curStage * this.enemyData.goldadd * 0.5;
                  gold = Math.floor(gold * (1 + ((bounslv - 1) * bunsdata.bonus1)));
                  Laya.stage.event(EventConst.Game_Update_Gold, gold);
              }
          }
      }
      onDestroy() {
          if (this.mAtkedEf) {
              EffectMgr.Instance().removeSpriteEffect(this.mAtkedEf, true);
          }
      }
      addDamageEf() {
      }
      getNearEnemyOfAtkRang(role = null) {
          let characterList = [];
          if (this.teamType == TeamType.Red) {
              characterList = GameMgr.Instance().getPlayerList().concat();
          }
          else {
              characterList = GameMgr.Instance().getEnemyList().concat();
          }
          let target = null;
          let disSqr = this.atkRang * this.atkRang;
          for (let i = characterList.length - 1; i >= 0; i--) {
              const character = characterList[i];
              if (character == this || !character.isLife() || !character.sprite3d.active)
                  continue;
              let newDisSqr = Laya.Vector3.distanceSquared(character.mtransform.position, this.mtransform.position);
              if (newDisSqr <= disSqr && character != role) {
                  if (target == null || disSqr > newDisSqr) {
                      target = character;
                      disSqr = newDisSqr;
                  }
              }
          }
          return target;
      }
      getNearEnemyOfRang(role = null) {
          let characterList = [];
          if (this.teamType == TeamType.Red) {
              characterList = GameMgr.Instance().getPlayerList().concat();
          }
          else {
              characterList = GameMgr.Instance().getEnemyList().concat();
          }
          let target = null;
          let disSqr = 100000;
          for (let i = characterList.length - 1; i >= 0; i--) {
              const character = characterList[i];
              if (character == this || !character.isLife() || !character.sprite3d.active)
                  continue;
              if (this.teamType == TeamType.Blue && character.transform.position.z > 22) {
                  continue;
              }
              let newDisSqr = Laya.Vector3.distanceSquared(character.mtransform.position, this.mtransform.position);
              if (newDisSqr <= disSqr && character != role) {
                  if (target == null || disSqr > newDisSqr) {
                      target = character;
                      disSqr = newDisSqr;
                  }
              }
          }
          return target;
      }
      getRandomEnemyOfRang(role = null) {
          let characterList = [];
          if (this.teamType == TeamType.Red) {
              characterList = GameMgr.Instance().getPlayerList().concat();
          }
          else {
              characterList = GameMgr.Instance().getEnemyList().concat();
          }
          let target = characterList[Util.random(0, characterList.length)];
          return target;
      }
      rayCheckWall(dir, rang) {
          if (!dir || dir == null) {
              return null;
          }
          var from = new Laya.Vector3(0, 0, 0);
          Laya.Vector3.normalize(dir, from);
          var to = new Laya.Vector3(0, 0, 0);
          var raylenght = new Laya.Vector3(0, 0, 0);
          Laya.Vector3.scale(from, -rang, raylenght);
          Laya.Vector3.add(this.mtransform.position.clone(), raylenght, to);
          this.mtransform.position.cloneTo(from);
          from.y = 1;
          to.y = 1;
          var hitResults = [];
          this.scene3d.physicsSimulation.raycastAllFromTo(from, to, hitResults, CollisionConst.GROUP_RAY, CollisionConst.CAN_RAY);
          for (let i = 0, n = hitResults.length; i < n; i++) {
              var otherSprite3d = hitResults[i].collider.owner;
              if (otherSprite3d && otherSprite3d.active) {
                  var character = otherSprite3d.getComponent(Character);
                  if (!character || (character != this && (character.teamType == this.teamType))) {
                      let hitResult = hitResults[i];
                      return hitResult;
                  }
              }
          }
          return null;
      }
  }

  class BiaoQiangBing extends Character {
      constructor() {
          super(...arguments);
          this.soldierController = null;
          this.mAtkedEf = null;
          this.mEfName = '';
      }
      onAwake() {
          super.onAwake();
      }
      onEnable() {
          super.onEnable();
      }
      onPause() {
          this.isPause = true;
          this.playIdleAni();
      }
      onStart() {
          super.onStart();
          this.collider3d = this.sprite3d.getComponent(Laya.PhysicsCollider);
          this.collider3d.enabled = true;
          this.collider3d.isTrigger = true;
          this.collider3d.collisionGroup = CollisionConst.GROUP_Soldier;
          this.collider3d.canCollideWith = CollisionConst.CAN_Soldier;
          this.animatorController = this.sprite3d.getComponent(SoldierAnimatorController);
          if (this.animatorController) {
              this.animatorController.destroy();
              this.animatorController = null;
          }
          this.animatorController = this.sprite3d.addComponent(SoldierAnimatorController);
          this.animatorController.character = this;
          this.hpPoint = this.sprite3d.getChildByName('hp');
          let lv = 0;
          if (this.teamType == TeamType.Red) {
              Util.changMat(ResConst.MaterialPath + ResConst.RoleMate[0], this.sprite3d.getChildByName('CH_Spartan_Melee_01').skinnedMeshRenderer);
              this.enemyData = GameMgr.Instance().getEnemySoldierData(EnemyType.BiaoQiangBing);
              lv = 1 + Math.floor((GameData.data.curStage - this.enemyData.upgrade) / this.enemyData.upgradeadd);
              lv = Util.clamp(lv, 1, this.enemyData.lv_max);
              this.mHp = this.enemyData.hp + this.enemyData.defense;
              this.mTotalHp = this.mHp;
              this.moveSpeed = this.enemyData.speed;
              this.atkRang = this.enemyData.atkdist;
              this.atkIntvalTemp = this.enemyData.interval;
              this.atkIntval = this.enemyData.interval;
              this.atkDamage = this.enemyData.atk;
              this.shotRang = 10;
              this.shotSpeed = this.enemyData.shotspeed;
              this.damageRang = this.enemyData.atkdist - Util.random(1, 4) * 0.1;
              this.nickName = this.enemyData.name;
          }
          else {
              Util.changMat(ResConst.MaterialPath + ResConst.RoleMate[1], this.sprite3d.getChildByName('CH_Spartan_Melee_01').skinnedMeshRenderer);
              lv = Math.max(1, GameData.data.getWarriorLv(WarriorType.BiaoQiangBing));
              if (lv <= 1) {
                  GameData.data.setWarriorLv(1, WarriorType.BiaoQiangBing);
              }
              this.soldierData = GameMgr.Instance().getSoldierData(WarriorType.BiaoQiangBing, lv);
              this.mHp = this.soldierData.hp + this.soldierData.defense;
              this.mTotalHp = this.mHp;
              this.moveSpeed = this.soldierData.speed;
              this.atkRang = this.soldierData.atkdist;
              this.atkIntvalTemp = this.soldierData.interval;
              this.atkIntval = this.soldierData.interval;
              this.atkDamage = this.soldierData.atk;
              this.shotRang = 30;
              this.shotSpeed = this.soldierData.shotspeed;
              this.damageRang = this.soldierData.atkdist - Util.random(1, 4) * 0.1;
              this.nickName = this.soldierData.name;
          }
          let stage = 1 + Math.floor((lv - 1) / 3);
          stage = Math.min(ResConst.WarriorLvName[2].length, stage);
          let hand = this.sprite3d.getChildByName('Bip001').getChildByName('Bip001 Pelvis').getChildByName('Bip001 Spine').getChildByName('Bip001 Spine1').getChildByName('Bip001 Neck').getChildByName('Bip001 Head').getChildByName('head_point');
          let shield = this.sprite3d.getChildByName('Bip001').getChildByName('Bip001 Pelvis').getChildByName('Bip001 Spine').getChildByName('Bip001 Spine1').getChildByName('Bip001 Neck').getChildByName('Bip001 L Clavicle').getChildByName('Bip001 L UpperArm').getChildByName('Bip001 L Forearm').getChildByName('Bip001 L Hand').getChildByName('Left_shield');
          let weapons = this.sprite3d.getChildByName('Bip001').getChildByName('Bip001 Pelvis').getChildByName('Bip001 Spine').getChildByName('Bip001 Spine1').getChildByName('Bip001 Neck').getChildByName('Bip001 R Clavicle').getChildByName('Bip001 R UpperArm').getChildByName('Bip001 R Forearm').getChildByName('Bip001 R Hand').getChildByName('Right_weapons');
          this.mShotPoint = weapons;
          Util.activeAllChild(hand, false);
          Util.activeAllChild(shield, false);
          Util.activeAllChild(weapons, false);
          switch (stage) {
              case 1:
                  hand.getChildByName('Helmet_01').active = true;
                  weapons.getChildByName('Javelin').active = true;
                  break;
              case 2:
                  hand.getChildByName('Helmet_02').active = true;
                  weapons.getChildByName('Javelin').active = true;
                  break;
              case 3:
                  hand.getChildByName('Helmet_03').active = true;
                  weapons.getChildByName('Javelin').active = true;
                  break;
              case 4:
                  hand.getChildByName('Helmet_03').active = true;
                  weapons.getChildByName('Javelin').active = true;
                  shield.getChildByName('Steele_01').active = true;
                  break;
              default:
                  break;
          }
          if (this.teamType != TeamType.Neutral) {
              this.soldierController = this.sprite3d.addComponent(ElementController);
              this.soldierController.setBehavior(ScriptType.idle);
          }
          if (this.mAtkedEf) {
              this.mAtkedEf.removeSelf();
              this.mAtkedEf.destroy();
          }
          if (!this.healthBar) {
              this.healthBar = HealthBar.getHealthBar(this.nickName, this.mTotalHp, this.teamType == TeamType.Red);
              this.onResume();
          }
          this.healthBar.setActive(false);
      }
      onResume() {
          super.onResume();
      }
      initEvent() {
          super.initEvent();
      }
      removeEvent() {
          super.removeEvent();
      }
      playIdleAni() {
          this.animatorController && (this.animatorController.play(SoldierAnimatorController.idle));
      }
      playAtkAni() {
          this.animatorController && (this.animatorController.play(SoldierAnimatorController.tou));
      }
      playRunAni() {
          this.animatorController && (this.animatorController.play(SoldierAnimatorController.run));
      }
      playDieAni() {
          this.animatorController && (this.animatorController.play(SoldierAnimatorController.die));
      }
      onUpdate() {
          if (!this.isLife())
              return;
          if (GameData.data.gameFinish)
              return;
          if (Laya.timer.delta > 500)
              return;
          let position = this.mtransform.position.clone();
          let movevelocity = this.movevelocity.clone();
          position.z += movevelocity.z;
          position.x += movevelocity.x;
          this.mtransform.position = position;
          if (this.moveDir.x == 0 && this.moveDir.z == 0) {
              this.move(null);
          }
          else {
              this.playRunAni();
          }
          this.atkedInterval -= Laya.timer.delta;
      }
      onLateUpdate() {
          if (!this.isLife())
              return;
          if (GameData.data.gameFinish)
              return;
          this.healthBar && this.healthBar.updatePosition(this.hpPoint.transform.position);
      }
      move(dir) {
          if (!dir || dir == null) {
              this.movevelocity.x = 0;
              this.movevelocity.z = 0;
              this.moveDir.setValue(0, 0, 0);
              return;
          }
          if ((dir.x == 0 && dir.z == 0))
              return;
          Laya.Vector3.normalize(dir, dir);
          this.moveDir.x = dir.x;
          this.moveDir.z = dir.z;
          let curSpeed = this.moveSpeed * Laya.timer.delta * 0.001;
          this.movevelocity.x = -dir.x * curSpeed;
          this.movevelocity.y = 0;
          this.movevelocity.z = -dir.z * curSpeed;
      }
      rotationRole(angle) {
          let angleY = this.sprite3d.transform.rotationEuler.y;
          let rotateY = angle - angleY;
          if (rotateY != 0) {
              this.sprite3d.transform.rotate(new Laya.Vector3(0, rotateY, 0), false, false);
          }
      }
      onAtk(atkedObj) {
          this.atkedTarget = atkedObj;
          this.playAtkAni();
          this.healthBar.setActive(true);
          this.owner.timerOnce(100, this, this.createBullet, null, false);
      }
      createBullet() {
          if (!this.isLife())
              return;
          AudioManager.Instance.playSfx('biaoqiang.mp3');
          let name = 'Javelin';
          let bullet3d = BulletMgr.Instance().createBullet(name);
          let bullet = bullet3d.getComponent(IBullet);
          if (bullet != null) {
              bullet.destroy();
          }
          bullet = bullet3d.addComponent(Arrow);
          BulletMgr.Instance().addBullet(bullet);
          bullet3d.transform.setWorldLossyScale(new Laya.Vector3(1, 1, 1));
          bullet.firer = this;
          bullet.atked = this.atkedTarget;
          let shotDir = Util.vectorDirection(this.mtransform.position.clone(), this.atkedTarget.mtransform.position.clone());
          bullet.init(this, this.atkedTarget, shotDir, this.mShotPoint.transform.position.clone());
          bullet.rotationEuler = this.mtransform.rotationEuler.clone();
          bullet.position = this.mShotPoint.transform.position.clone();
      }
      onDamage(damage) {
          if (this.atkedInterval > 0)
              return;
          if (GameData.data.gameFinish || !this.isLife())
              return;
          if (this.teamType == TeamType.Red) {
              let bounslv = Math.max(1, GameData.data.getCommonLv(CommonCfg.HaoJiao));
              let bunsdata = GameData.data.commonCfg.HaoJiao;
              damage = Math.floor(damage * (1 + GameData.data.atkUp * (bounslv) * bunsdata.bonus1));
          }
          this.mHp -= damage;
          this.atkedInterval = 0;
          console.log('剩余血量', this.mHp);
          this.addDamageEf();
          this.healthBar.updateHp(this.mHp / this.mTotalHp, Math.max(0, this.mHp));
          this.healthBar.setActive(true);
          if (this.mHp <= 0) {
              this.healthBar.setActive(false);
              HealthBar.recover(this.healthBar);
              this.playDieAni();
              if (this.teamType == TeamType.Blue) {
                  GameMgr.Instance().playerToRemoveList(this);
                  Laya.stage.timerOnce(1000, this, () => {
                      GameMgr.Instance().removePlayer(this);
                  }, null, false);
              }
              else {
                  GameMgr.Instance().enemyToRemoveList(this);
                  Laya.stage.timerOnce(1000, this, () => {
                      GameMgr.Instance().removeEnemy(this);
                  }, null, false);
              }
              if (this.teamType == TeamType.Red) {
                  let bounslv = Math.max(1, GameData.data.getCommonLv(CommonCfg.JinBi));
                  let bunsdata = GameData.data.commonCfg.JinBi;
                  let gold = this.enemyData.gold + GameData.data.curStage * this.enemyData.goldadd * 0.5;
                  gold = Math.floor(gold * (1 + ((bounslv - 1) * bunsdata.bonus1)));
                  Laya.stage.event(EventConst.Game_Update_Gold, gold);
              }
          }
      }
      onDestroy() {
          if (this.mAtkedEf) {
              EffectMgr.Instance().removeSpriteEffect(this.mAtkedEf, true);
          }
      }
      addDamageEf() {
      }
      getNearEnemyOfAtkRang(role = null) {
          let characterList = [];
          if (this.teamType == TeamType.Red) {
              characterList = GameMgr.Instance().getPlayerList().concat();
          }
          else {
              characterList = GameMgr.Instance().getEnemyList().concat();
          }
          let target = null;
          let disSqr = this.atkRang * this.atkRang;
          for (let i = characterList.length - 1; i >= 0; i--) {
              const character = characterList[i];
              if (character == this || !character.isLife() || !character.sprite3d.active)
                  continue;
              if (this.teamType == TeamType.Blue && character.transform.position.z > 22) {
                  continue;
              }
              let newDisSqr = Laya.Vector3.distanceSquared(character.mtransform.position, this.mtransform.position);
              if (newDisSqr <= disSqr && character != role) {
                  if (target == null || disSqr > newDisSqr) {
                      target = character;
                      disSqr = newDisSqr;
                  }
              }
          }
          return target;
      }
      getNearEnemyOfRang(role = null) {
          let characterList = [];
          if (this.teamType == TeamType.Red) {
              characterList = GameMgr.Instance().getPlayerList().concat();
          }
          else {
              characterList = GameMgr.Instance().getEnemyList().concat();
          }
          let target = null;
          let disSqr = 100000;
          for (let i = characterList.length - 1; i >= 0; i--) {
              const character = characterList[i];
              if (character == this || !character.isLife() || !character.sprite3d.active)
                  continue;
              let newDisSqr = Laya.Vector3.distanceSquared(character.mtransform.position, this.mtransform.position);
              if (newDisSqr <= disSqr && character != role) {
                  if (target == null || disSqr > newDisSqr) {
                      target = character;
                      disSqr = newDisSqr;
                  }
              }
          }
          return target;
      }
      getRandomEnemyOfRang(role = null) {
          let characterList = [];
          if (this.teamType == TeamType.Red) {
              characterList = GameMgr.Instance().getPlayerList().concat();
          }
          else {
              characterList = GameMgr.Instance().getEnemyList().concat();
          }
          let target = characterList[Util.random(0, characterList.length)];
          return target;
      }
      rayCheckWall(dir, rang) {
          if (!dir || dir == null) {
              return null;
          }
          var from = new Laya.Vector3(0, 0, 0);
          Laya.Vector3.normalize(dir, from);
          var to = new Laya.Vector3(0, 0, 0);
          var raylenght = new Laya.Vector3(0, 0, 0);
          Laya.Vector3.scale(from, -rang, raylenght);
          Laya.Vector3.add(this.mtransform.position.clone(), raylenght, to);
          this.mtransform.position.cloneTo(from);
          from.y = 1;
          to.y = 1;
          var hitResults = [];
          this.scene3d.physicsSimulation.raycastAllFromTo(from, to, hitResults, CollisionConst.GROUP_RAY, CollisionConst.CAN_RAY);
          for (let i = 0, n = hitResults.length; i < n; i++) {
              var otherSprite3d = hitResults[i].collider.owner;
              if (otherSprite3d && otherSprite3d.active) {
                  var character = otherSprite3d.getComponent(Character);
                  if (!character || (character != this && (character.teamType == this.teamType))) {
                      let hitResult = hitResults[i];
                      return hitResult;
                  }
              }
          }
          return null;
      }
  }

  class XianFeng extends Character {
      constructor() {
          super(...arguments);
          this.soldierController = null;
          this.mAtkedEf = null;
          this.mEfName = '';
      }
      onAwake() {
          super.onAwake();
      }
      onEnable() {
          super.onEnable();
      }
      onPause() {
          this.isPause = true;
          this.playIdleAni();
      }
      onStart() {
          super.onStart();
          this.collider3d = this.sprite3d.getComponent(Laya.PhysicsCollider);
          this.collider3d.enabled = true;
          this.collider3d.isTrigger = true;
          this.collider3d.collisionGroup = CollisionConst.GROUP_Soldier;
          this.collider3d.canCollideWith = CollisionConst.CAN_Soldier;
          this.animatorController = this.sprite3d.getComponent(SoldierAnimatorController);
          if (this.animatorController) {
              this.animatorController.destroy();
              this.animatorController = null;
          }
          this.animatorController = this.sprite3d.addComponent(SoldierAnimatorController);
          this.animatorController.character = this;
          this.hpPoint = this.sprite3d.getChildByName('hp');
          this.updateLv();
          if (this.mAtkedEf) {
              this.mAtkedEf.removeSelf();
              this.mAtkedEf.destroy();
          }
          this.playIdleAni();
      }
      updateLv() {
          let lv = 0;
          if (this.teamType == TeamType.Red) {
          }
          else {
              Util.changMat(ResConst.MaterialPath + ResConst.RoleMate[1], this.sprite3d.getChildByName('CH_Spartan_Melee_01').skinnedMeshRenderer);
              lv = Math.max(1, GameData.data.getWarriorLv(WarriorType.XianFeng));
              if (lv <= 1) {
                  GameData.data.setWarriorLv(1, WarriorType.XianFeng);
              }
              this.soldierData = GameMgr.Instance().getSoldierData(WarriorType.XianFeng, lv);
              this.mHp = this.soldierData.hp + this.soldierData.defense;
              this.mTotalHp = this.mHp;
              this.moveSpeed = this.soldierData.speed;
              this.atkRang = this.soldierData.atkdist;
              this.atkIntvalTemp = this.soldierData.interval;
              this.atkIntval = this.soldierData.interval;
              this.atkDamage = this.soldierData.atk;
              this.shotRang = 10;
              this.shotSpeed = this.soldierData.shotspeed;
              this.damageRang = this.soldierData.atkdist - Util.random(1, 4) * 0.1;
          }
          let stage = 1 + Math.floor((lv - 1) / 8);
          stage = Math.min(4, stage);
          let hand = this.sprite3d.getChildByName('Bip001').getChildByName('Bip001 Pelvis').getChildByName('Bip001 Spine').getChildByName('Bip001 Spine1').getChildByName('Bip001 Neck').getChildByName('Bip001 Head').getChildByName('head_point');
          let shield = this.sprite3d.getChildByName('Bip001').getChildByName('Bip001 Pelvis').getChildByName('Bip001 Spine').getChildByName('Bip001 Spine1').getChildByName('Bip001 Neck').getChildByName('Bip001 L Clavicle').getChildByName('Bip001 L UpperArm').getChildByName('Bip001 L Forearm').getChildByName('Bip001 L Hand').getChildByName('Left_shield');
          let weapons = this.sprite3d.getChildByName('Bip001').getChildByName('Bip001 Pelvis').getChildByName('Bip001 Spine').getChildByName('Bip001 Spine1').getChildByName('Bip001 Neck').getChildByName('Bip001 R Clavicle').getChildByName('Bip001 R UpperArm').getChildByName('Bip001 R Forearm').getChildByName('Bip001 R Hand').getChildByName('Right_weapons');
          Util.activeAllChild(hand, false);
          Util.activeAllChild(shield, false);
          Util.activeAllChild(weapons, false);
          switch (stage) {
              case 1:
                  hand.getChildByName('Helmet_01').active = true;
                  weapons.getChildByName('Sword').active = true;
                  shield.getChildByName('Steele_01').active = true;
                  break;
              case 2:
                  hand.getChildByName('Helmet_01').active = true;
                  weapons.getChildByName('Knife').active = true;
                  shield.getChildByName('Steele_01').active = true;
                  break;
              case 3:
                  hand.getChildByName('Helmet_02').active = true;
                  weapons.getChildByName('Knife').active = true;
                  shield.getChildByName('Steele_02').active = true;
                  break;
              case 4:
                  hand.getChildByName('Helmet_03').active = true;
                  weapons.getChildByName('Knife').active = true;
                  shield.getChildByName('Steele_02').active = true;
                  break;
              default:
                  break;
          }
      }
      onResume() {
          super.onResume();
          if (!this.healthBar) {
              this.healthBar = HealthBar.getHealthBar(this.nickName, this.mTotalHp, this.teamType == TeamType.Red);
              this.healthBar.setActive(false);
              if (this.teamType != TeamType.Neutral) {
                  this.soldierController = this.sprite3d.addComponent(ElementController);
                  this.soldierController.setBehavior(ScriptType.idle);
              }
          }
      }
      initEvent() {
          super.initEvent();
          Laya.stage.on(EventConst.Update_XianFeng, this, this.updateLv);
      }
      removeEvent() {
          super.removeEvent();
          Laya.stage.off(EventConst.Update_XianFeng, this, this.updateLv);
      }
      playIdleAni() {
          this.animatorController && (this.animatorController.play(SoldierAnimatorController.idle));
      }
      playAtkAni() {
          this.animatorController && (this.animatorController.play(SoldierAnimatorController.kan));
      }
      playRunAni() {
          this.animatorController && (this.animatorController.play(SoldierAnimatorController.run));
      }
      playDieAni() {
          this.animatorController && (this.animatorController.play(SoldierAnimatorController.die));
      }
      onUpdate() {
          if (!this.isLife())
              return;
          if (GameData.data.gameFinish)
              return;
          if (Laya.timer.delta > 500)
              return;
          let position = this.mtransform.position.clone();
          let movevelocity = this.movevelocity.clone();
          position.z += movevelocity.z;
          position.x += movevelocity.x;
          this.mtransform.position = position;
          if (this.moveDir.x == 0 && this.moveDir.z == 0) {
              this.move(null);
          }
          else {
              this.playRunAni();
          }
          this.atkedInterval -= Laya.timer.delta;
      }
      onLateUpdate() {
          if (!this.isLife())
              return;
          if (GameData.data.gameFinish)
              return;
          this.healthBar && this.healthBar.updatePosition(this.hpPoint.transform.position);
      }
      move(dir) {
          if (!dir || dir == null) {
              this.movevelocity.x = 0;
              this.movevelocity.z = 0;
              this.moveDir.setValue(0, 0, 0);
              return;
          }
          if ((dir.x == 0 && dir.z == 0))
              return;
          Laya.Vector3.normalize(dir, dir);
          this.moveDir.x = dir.x;
          this.moveDir.z = dir.z;
          let curSpeed = this.moveSpeed * Laya.timer.delta * 0.001;
          this.movevelocity.x = -dir.x * curSpeed;
          this.movevelocity.y = 0;
          this.movevelocity.z = -dir.z * curSpeed;
      }
      rotationRole(angle) {
          let angleY = this.sprite3d.transform.rotationEuler.y;
          let rotateY = angle - angleY;
          if (rotateY != 0) {
              this.sprite3d.transform.rotate(new Laya.Vector3(0, rotateY, 0), false, false);
          }
      }
      onAtk(atkedObj) {
          this.atkedTarget = atkedObj;
          this.playAtkAni();
          AudioManager.Instance.playSfx('daojian.mp3');
          this.healthBar.setActive(true);
          atkedObj.onDamage(this.atkDamage);
      }
      onDamage(damage) {
          if (this.atkedInterval > 0)
              return;
          if (GameData.data.gameFinish || !this.isLife())
              return;
          this.mHp -= damage;
          this.atkedInterval = 0;
          console.log('剩余血量', this.mHp);
          this.addDamageEf();
          this.healthBar.updateHp(this.mHp / this.mTotalHp, Math.max(0, this.mHp));
          this.healthBar.setActive(true);
          if (this.mHp <= 0) {
              this.healthBar.setActive(false);
              HealthBar.recover(this.healthBar);
              this.playDieAni();
              if (this.teamType == TeamType.Blue) {
                  GameMgr.Instance().playerToRemoveList(this);
                  Laya.stage.timerOnce(1000, this, () => {
                      GameMgr.Instance().removePlayer(this);
                  }, null, false);
              }
              else {
                  GameMgr.Instance().enemyToRemoveList(this);
                  Laya.stage.timerOnce(1000, this, () => {
                      GameMgr.Instance().removeEnemy(this);
                  }, null, false);
              }
              if (this.teamType == TeamType.Red) {
                  Laya.stage.event(EventConst.Game_Update_Gold, this.enemyData.gold);
              }
          }
      }
      onDestroy() {
          if (this.mAtkedEf) {
              EffectMgr.Instance().removeSpriteEffect(this.mAtkedEf, true);
          }
      }
      addDamageEf() {
      }
      getNearEnemyOfAtkRang(role = null) {
          let characterList = [];
          if (this.teamType == TeamType.Red) {
              characterList = GameMgr.Instance().getPlayerList().concat();
          }
          else {
              characterList = GameMgr.Instance().getEnemyList().concat();
          }
          let target = null;
          let disSqr = this.atkRang * this.atkRang;
          for (let i = characterList.length - 1; i >= 0; i--) {
              const character = characterList[i];
              if (character == this || !character.isLife() || !character.sprite3d.active)
                  continue;
              let newDisSqr = Laya.Vector3.distanceSquared(character.mtransform.position, this.mtransform.position);
              if (newDisSqr <= disSqr && character != role) {
                  if (target == null || disSqr > newDisSqr) {
                      target = character;
                      disSqr = newDisSqr;
                  }
              }
          }
          return target;
      }
      getNearEnemyOfRang(role = null) {
          let characterList = [];
          if (this.teamType == TeamType.Red) {
              characterList = GameMgr.Instance().getPlayerList().concat();
          }
          else {
              characterList = GameMgr.Instance().getEnemyList().concat();
          }
          let target = null;
          let disSqr = 1000;
          for (let i = characterList.length - 1; i >= 0; i--) {
              const character = characterList[i];
              if (character == this || !character.isLife() || !character.sprite3d.active)
                  continue;
              let newDisSqr = Laya.Vector3.distanceSquared(character.mtransform.position, this.mtransform.position);
              if (newDisSqr <= disSqr && character != role) {
                  if (target == null || disSqr > newDisSqr) {
                      target = character;
                      disSqr = newDisSqr;
                  }
              }
          }
          return target;
      }
      getRandomEnemyOfRang(role = null) {
          let characterList = [];
          if (this.teamType == TeamType.Red) {
              characterList = GameMgr.Instance().getPlayerList().concat();
          }
          else {
              characterList = GameMgr.Instance().getEnemyList().concat();
          }
          let target = characterList[Util.random(0, characterList.length)];
          return target;
      }
      rayCheckWall(dir, rang) {
          if (!dir || dir == null) {
              return null;
          }
          var from = new Laya.Vector3(0, 0, 0);
          Laya.Vector3.normalize(dir, from);
          var to = new Laya.Vector3(0, 0, 0);
          var raylenght = new Laya.Vector3(0, 0, 0);
          Laya.Vector3.scale(from, -rang, raylenght);
          Laya.Vector3.add(this.mtransform.position.clone(), raylenght, to);
          this.mtransform.position.cloneTo(from);
          from.y = 1;
          to.y = 1;
          var hitResults = [];
          this.scene3d.physicsSimulation.raycastAllFromTo(from, to, hitResults, CollisionConst.GROUP_RAY, CollisionConst.CAN_RAY);
          for (let i = 0, n = hitResults.length; i < n; i++) {
              var otherSprite3d = hitResults[i].collider.owner;
              if (otherSprite3d && otherSprite3d.active) {
                  var character = otherSprite3d.getComponent(Character);
                  if (!character || (character != this && (character.teamType == this.teamType))) {
                      let hitResult = hitResults[i];
                      return hitResult;
                  }
              }
          }
          return null;
      }
  }

  class GongChengChui extends Character {
      constructor() {
          super(...arguments);
          this.soldierController = null;
          this.mAtkedEf = null;
          this.mEfName = '';
      }
      onAwake() {
          super.onAwake();
      }
      onEnable() {
          super.onEnable();
      }
      onPause() {
          this.isPause = true;
          this.playIdleAni();
      }
      onStart() {
          super.onStart();
          this.collider3d = this.sprite3d.getComponent(Laya.PhysicsCollider);
          this.collider3d.enabled = true;
          this.collider3d.isTrigger = true;
          this.collider3d.collisionGroup = CollisionConst.GROUP_Aider;
          this.collider3d.canCollideWith = CollisionConst.CAN_Aider;
          this.animatorController = this.sprite3d.getComponent(AnimatorController);
          if (this.animatorController) {
              this.animatorController.destroy();
              this.animatorController = null;
          }
          this.animatorController = this.sprite3d.addComponent(AnimatorController);
          this.animatorController.character = this;
          this.hpPoint = this.sprite3d.getChildByName('hp');
          if (this.teamType == TeamType.Red) {
              this.enemyData = GameMgr.Instance().getEnemySoldierData(EnemyType.GongChengChui);
              this.mHp = this.enemyData.hp + this.enemyData.defense;
              this.mTotalHp = this.mHp;
              this.moveSpeed = this.enemyData.speed;
              this.atkIntvalTemp = this.enemyData.interval;
              this.atkIntval = 0;
              this.atkDamage = this.enemyData.atk;
              this.atkRang = this.enemyData.atkdist;
          }
          else {
          }
          if (this.mAtkedEf) {
              this.mAtkedEf.removeSelf();
              this.mAtkedEf.destroy();
          }
          if (!this.healthBar) {
              this.healthBar = HealthBar.getHealthBar(this.nickName, this.mTotalHp, this.teamType == TeamType.Red);
              this.onResume();
          }
          this.healthBar.setActive(false);
      }
      onResume() {
          super.onResume();
      }
      initEvent() {
          super.initEvent();
      }
      removeEvent() {
          super.removeEvent();
      }
      playIdleAni() {
      }
      playAtkAni() {
          this.animatorController && (this.animatorController.play(AnimatorController.atk));
          this.animatorController.curClipName = null;
      }
      playRunAni() {
          this.animatorController && (this.animatorController.play(AnimatorController.run));
      }
      playDieAni() {
          this.animatorController && (this.animatorController.play(AnimatorController.die));
      }
      onUpdate() {
          if (!this.isLife())
              return;
          if (GameData.data.gameFinish)
              return;
          this.transform.getForward(this.dir);
          let hitResult = this.rayCheckWall(this.dir, this.atkRang);
          if (this.transform.position.z - this.atkRang > 0 && (!hitResult || hitResult.collider.owner.name.indexOf('BatteringRam') == -1)) {
              let curSpeed = this.moveSpeed * Laya.timer.delta * 0.001;
              let position = this.mtransform.position.clone();
              position.z -= curSpeed;
              this.mtransform.position = position;
          }
          if (this.transform.position.z - this.atkRang <= 0) {
              this.atkIntval -= Laya.timer.delta;
              if (this.atkIntval <= 0) {
                  if (this.isLife() && GameMgr.Instance().getScenario().isLife()) {
                      this.atkIntval = this.atkIntvalTemp;
                      this.playAtkAni();
                      GameMgr.Instance().getScenario().onDamage(this.atkDamage);
                  }
              }
          }
          else {
              this.playRunAni();
          }
      }
      onLateUpdate() {
          if (!this.isLife())
              return;
          if (GameData.data.gameFinish)
              return;
          this.healthBar && this.healthBar.updatePosition(this.hpPoint.transform.position);
      }
      move(dir) {
      }
      rotationRole(angle) {
          let angleY = this.sprite3d.transform.rotationEuler.y;
          let rotateY = angle - angleY;
          if (rotateY != 0) {
              this.sprite3d.transform.rotate(new Laya.Vector3(0, rotateY, 0), false, false);
          }
      }
      onAtk(atkedObj) {
      }
      onDamage(damage) {
          if (this.atkedInterval > 0)
              return;
          if (GameData.data.gameFinish || !this.isLife())
              return;
          if (this.teamType == TeamType.Red) {
              let bounslv = Math.max(1, GameData.data.getCommonLv(CommonCfg.HaoJiao));
              let bunsdata = GameData.data.commonCfg.HaoJiao;
              damage = Math.floor(damage * (1 + GameData.data.atkUp * (bounslv) * bunsdata.bonus1));
          }
          this.mHp -= damage;
          this.atkedInterval = 0;
          console.log('剩余血量', this.mHp);
          this.addDamageEf();
          this.healthBar.updateHp(this.mHp / this.mTotalHp, Math.max(0, this.mHp));
          this.healthBar.setActive(true);
          if (this.mHp <= 0) {
              this.healthBar.setActive(false);
              HealthBar.recover(this.healthBar);
              this.playDieAni();
              if (this.teamType == TeamType.Blue) {
                  GameMgr.Instance().playerToRemoveList(this);
                  Laya.stage.timerOnce(1000, this, () => {
                      GameMgr.Instance().removePlayer(this);
                  }, null, false);
              }
              else {
                  GameMgr.Instance().enemyToRemoveList(this);
                  Laya.stage.timerOnce(1000, this, () => {
                      GameMgr.Instance().removeEnemy(this);
                  }, null, false);
              }
              if (this.teamType == TeamType.Red) {
                  let bounslv = Math.max(1, GameData.data.getCommonLv(CommonCfg.JinBi));
                  let bunsdata = GameData.data.commonCfg.JinBi;
                  let gold = this.enemyData.gold + GameData.data.curStage * this.enemyData.goldadd * 0.5;
                  gold = Math.floor(gold * (1 + ((bounslv - 1) * bunsdata.bonus1)));
                  Laya.stage.event(EventConst.Game_Update_Gold, gold);
              }
          }
      }
      onDestroy() {
          if (this.mAtkedEf) {
              EffectMgr.Instance().removeSpriteEffect(this.mAtkedEf, true);
          }
      }
      addDamageEf() {
      }
      getNearEnemyOfAtkRang(role = null) {
          let characterList = [];
          if (this.teamType == TeamType.Red) {
              characterList = GameMgr.Instance().getPlayerList().concat();
          }
          else {
              characterList = GameMgr.Instance().getEnemyList().concat();
          }
          let target = null;
          let disSqr = this.atkRang * this.atkRang;
          for (let i = characterList.length - 1; i >= 0; i--) {
              const character = characterList[i];
              if (character == this || !character.isLife() || !character.sprite3d.active)
                  continue;
              let newDisSqr = Laya.Vector3.distanceSquared(character.mtransform.position, this.mtransform.position);
              if (newDisSqr <= disSqr && character != role) {
                  if (target == null || disSqr > newDisSqr) {
                      target = character;
                      disSqr = newDisSqr;
                  }
              }
          }
          return target;
      }
      getNearEnemyOfRang(role = null) {
          let characterList = [];
          if (this.teamType == TeamType.Red) {
              characterList = GameMgr.Instance().getPlayerList().concat();
          }
          else {
              characterList = GameMgr.Instance().getEnemyList().concat();
          }
          let target = null;
          let disSqr = 100000;
          for (let i = characterList.length - 1; i >= 0; i--) {
              const character = characterList[i];
              if (character == this || !character.isLife() || !character.sprite3d.active || character.isAtkedLock)
                  continue;
              let newDisSqr = Laya.Vector3.distanceSquared(character.mtransform.position, this.mtransform.position);
              if (newDisSqr <= disSqr && character != role) {
                  if (target == null || disSqr > newDisSqr) {
                      target = character;
                      disSqr = newDisSqr;
                  }
              }
          }
          return target;
      }
      getRandomEnemyOfRang(role = null) {
          let characterList = [];
          if (this.teamType == TeamType.Red) {
              characterList = GameMgr.Instance().getPlayerList().concat();
          }
          else {
              characterList = GameMgr.Instance().getEnemyList().concat();
          }
          let target = characterList[Util.random(0, characterList.length)];
          return target;
      }
      rayCheckWall(dir, rang) {
          if (!dir || dir == null) {
              return null;
          }
          var from = new Laya.Vector3(0, 0, 0);
          Laya.Vector3.normalize(dir, from);
          var to = new Laya.Vector3(0, 0, 0);
          var raylenght = new Laya.Vector3(0, 0, 0);
          Laya.Vector3.scale(from, -rang, raylenght);
          Laya.Vector3.add(this.mtransform.position.clone(), raylenght, to);
          this.mtransform.position.cloneTo(from);
          from.y = 1;
          to.y = 1;
          var hitResults = [];
          this.scene3d.physicsSimulation.raycastAllFromTo(from, to, hitResults, CollisionConst.GROUP_RAY, CollisionConst.CAN_RAY);
          for (let i = 0, n = hitResults.length; i < n; i++) {
              var otherSprite3d = hitResults[i].collider.owner;
              if (otherSprite3d && otherSprite3d.active) {
                  var character = otherSprite3d.getComponent(Character);
                  if (!character || (character != this && (character.teamType == this.teamType))) {
                      let hitResult = hitResults[i];
                      return hitResult;
                  }
              }
          }
          return null;
      }
  }

  var AnimListener$1 = fc.plugin.AnimStateListener;
  class DaXiangAnimatorController extends AnimatorController {
      onAwake() {
          this.animator = this.owner.getComponent(Laya.Animator);
          this.animator.speed = 0;
      }
      onStart() {
          AnimListener$1.add(this.animator, DaXiangAnimatorController.attack, this);
      }
      onEnable() {
          super.onEnable();
          if (this.character == null) {
          }
      }
      onDisable() {
          this.onDestroy();
          if (this.animator) {
              AnimListener$1.remove(this.animator, DaXiangAnimatorController.attack, this);
          }
      }
      onAnimEvt(...args) {
      }
      onAnimStateEnter(stateName, animator) {
      }
      onAnimStateUpdate(stateName, animator) {
      }
      onAnimStateExit(stateName, animator) {
          switch (stateName) {
              case DaXiangAnimatorController.attack:
                  this.curClipName = null;
                  this.play(DaXiangAnimatorController.idle);
                  break;
              default:
                  break;
          }
      }
      onDestroy() {
          this.animator = null;
          this.character = null;
          this.curClipName = null;
      }
      play(newName) {
          if (!this.animator || !newName || this.curClipName == newName)
              return;
          if (!this.character)
              return;
          this.animator.play(newName, 0, 0);
          this.curClipName = newName;
          this.animator.speed = 1;
      }
  }
  DaXiangAnimatorController.attack = "attack";

  class DaXiang extends Character {
      constructor() {
          super(...arguments);
          this.soldierController = null;
          this.mAtkedEf = null;
          this.mEfName = '';
          this.daxiangDamage = 0;
          this.shotIntval = 0;
          this.shotIntvalTemp = 0;
          this.atkDist = 0;
      }
      onAwake() {
          super.onAwake();
      }
      onEnable() {
          super.onEnable();
      }
      onPause() {
          this.isPause = true;
          this.playIdleAni();
      }
      onStart() {
          super.onStart();
          this.collider3d = this.sprite3d.getComponent(Laya.PhysicsCollider);
          this.collider3d.enabled = true;
          this.collider3d.isTrigger = true;
          this.collider3d.collisionGroup = CollisionConst.GROUP_Aider;
          this.collider3d.canCollideWith = CollisionConst.CAN_Aider;
          this.animatorController = this.sprite3d.getChildByName('Elephant').getComponent(DaXiangAnimatorController);
          if (this.animatorController) {
              this.animatorController.destroy();
              this.animatorController = null;
          }
          this.animatorController = this.sprite3d.getChildByName('Elephant').addComponent(DaXiangAnimatorController);
          this.animatorController.character = this;
          this.hpPoint = this.sprite3d.getChildByName('hp');
          this.role3d = this.sprite3d.getChildByName('Role');
          this.roleAnimatorController = this.role3d.getComponent(SoldierAnimatorController);
          if (this.roleAnimatorController) {
              this.roleAnimatorController.destroy();
              this.roleAnimatorController = null;
          }
          this.roleAnimatorController = this.role3d.addComponent(SoldierAnimatorController);
          this.roleAnimatorController.character = this;
          let lv = 0;
          if (this.teamType == TeamType.Red) {
              Util.changMat(ResConst.MaterialPath + ResConst.RoleMate[0], this.role3d.getChildByName('CH_Spartan_Melee_01').skinnedMeshRenderer);
              this.enemyData = GameMgr.Instance().getEnemySoldierData(EnemyType.DaXiang);
              lv = 1 + Math.floor((GameData.data.curStage - this.enemyData.upgrade) / this.enemyData.upgradeadd);
              lv = Util.clamp(lv, 1, this.enemyData.lv_max);
              this.mHp = this.enemyData.hp + this.enemyData.defense;
              this.mTotalHp = this.mHp;
              this.moveSpeed = this.enemyData.speed;
              this.atkIntvalTemp = this.enemyData.interval;
              this.atkIntval = 0;
              this.daxiangDamage = this.enemyData.atk;
              this.atkRang = this.enemyData.atkdist;
              this.damageRang = this.enemyData.atkdist;
              let soldierData = GameMgr.Instance().getEnemySoldierData(EnemyType.BiaoQiangBing);
              this.atkDist = soldierData.atkdist;
              this.shotRang = soldierData.atkdist;
              this.shotSpeed = soldierData.shotspeed;
              this.shotIntvalTemp = soldierData.interval;
              this.shotIntval = 0;
              this.atkDamage = soldierData.atk;
          }
          else {
          }
          let stage = 1 + Math.floor((lv - 1) / 3);
          stage = Math.min(3, stage);
          let hand = this.role3d.getChildByName('Bip001').getChildByName('Bip001 Pelvis').getChildByName('Bip001 Spine').getChildByName('Bip001 Spine1').getChildByName('Bip001 Neck').getChildByName('Bip001 Head').getChildByName('head_point');
          let shield = this.role3d.getChildByName('Bip001').getChildByName('Bip001 Pelvis').getChildByName('Bip001 Spine').getChildByName('Bip001 Spine1').getChildByName('Bip001 Neck').getChildByName('Bip001 L Clavicle').getChildByName('Bip001 L UpperArm').getChildByName('Bip001 L Forearm').getChildByName('Bip001 L Hand').getChildByName('Left_shield');
          let weapons = this.role3d.getChildByName('Bip001').getChildByName('Bip001 Pelvis').getChildByName('Bip001 Spine').getChildByName('Bip001 Spine1').getChildByName('Bip001 Neck').getChildByName('Bip001 R Clavicle').getChildByName('Bip001 R UpperArm').getChildByName('Bip001 R Forearm').getChildByName('Bip001 R Hand').getChildByName('Right_weapons');
          this.mShotPoint = weapons;
          Util.activeAllChild(hand, false);
          Util.activeAllChild(shield, false);
          Util.activeAllChild(weapons, false);
          switch (stage) {
              case 1:
                  hand.getChildByName('Helmet_01').active = true;
                  shield.getChildByName('Arrow_02').active = true;
                  break;
              case 2:
                  hand.getChildByName('Helmet_02').active = true;
                  shield.getChildByName('Arrow_02').active = true;
                  break;
              case 3:
                  hand.getChildByName('Helmet_03').active = true;
                  shield.getChildByName('Arrow_02').active = true;
                  break;
              default:
                  break;
          }
          if (this.mAtkedEf) {
              this.mAtkedEf.removeSelf();
              this.mAtkedEf.destroy();
          }
          if (!this.healthBar) {
              this.healthBar = HealthBar.getHealthBar(this.nickName, this.mTotalHp, this.teamType == TeamType.Red);
              this.onResume();
          }
          this.healthBar.setActive(false);
      }
      onResume() {
          super.onResume();
      }
      initEvent() {
          super.initEvent();
      }
      removeEvent() {
          super.removeEvent();
      }
      playIdleAni() {
          this.animatorController && (this.animatorController.play(DaXiangAnimatorController.idle));
      }
      playRoleIdleAni() {
          this.roleAnimatorController && (this.roleAnimatorController.play(SoldierAnimatorController.idle));
      }
      playRoleAtkAni() {
          this.roleAnimatorController && (this.roleAnimatorController.play(SoldierAnimatorController.shejian));
      }
      playAtkAni() {
          this.animatorController && (this.animatorController.play(DaXiangAnimatorController.attack));
      }
      playRunAni() {
          this.animatorController && (this.animatorController.play(DaXiangAnimatorController.run));
      }
      playDieAni() {
          this.animatorController && (this.animatorController.play(DaXiangAnimatorController.die));
          this.roleAnimatorController && (this.roleAnimatorController.play(SoldierAnimatorController.die));
      }
      onUpdate() {
          if (!this.isLife())
              return;
          if (GameData.data.gameFinish)
              return;
          this.shotIntval -= Laya.timer.delta;
          if (this.shotIntval <= 0) {
              let target = null;
              if (this.atkedTarget && this.atkedTarget.isLife() && this.isInAtkRang(this.atkedTarget)) {
                  target = this.atkedTarget;
              }
              else {
                  target = this.getNearEnemyOfAtkRang();
                  this.atkedTarget = target;
              }
              if (this.isLife()) {
                  if (target) {
                      this.shotIntval = this.shotIntvalTemp;
                      let angle = Util.angleY(this.atkedTarget.mtransform.position.clone(), this.role3d.transform.position.clone());
                      let angleY = this.role3d.transform.rotationEuler.y;
                      if (angle != angleY) {
                          this.role3d.transform.rotate(new Laya.Vector3(0, angle - angleY, 0), false, false);
                      }
                      this.onAtk(this.atkedTarget);
                  }
                  else {
                      this.playRoleIdleAni();
                  }
              }
          }
          this.transform.getForward(this.dir);
          if (this.transform.position.z - this.atkRang > 0 && !this.rayCheckWall(this.dir, this.atkRang)) {
              let curSpeed = this.moveSpeed * Laya.timer.delta * 0.001;
              let position = this.mtransform.position.clone();
              position.z -= curSpeed;
              this.mtransform.position = position;
          }
          if (this.transform.position.z - this.atkRang <= 0) {
              this.atkIntval -= Laya.timer.delta;
              if (this.atkIntval <= 0) {
                  if (this.isLife() && GameMgr.Instance().getScenario().isLife()) {
                      this.atkIntval = this.atkIntvalTemp;
                      this.playAtkAni();
                      GameMgr.Instance().getScenario().onDamage(this.daxiangDamage);
                  }
              }
          }
          else {
              this.playRunAni();
          }
      }
      onLateUpdate() {
          if (!this.isLife())
              return;
          if (GameData.data.gameFinish)
              return;
          this.healthBar && this.healthBar.updatePosition(this.hpPoint.transform.position);
      }
      move(dir) {
      }
      rotationRole(angle) {
          let angleY = this.sprite3d.transform.rotationEuler.y;
          let rotateY = angle - angleY;
          if (rotateY != 0) {
              this.sprite3d.transform.rotate(new Laya.Vector3(0, rotateY, 0), false, false);
          }
      }
      onAtk(atkedObj) {
          this.atkedTarget = atkedObj;
          this.playRoleAtkAni();
          this.owner.timerOnce(100, this, this.createBullet, null, false);
      }
      createBullet() {
          AudioManager.Instance.playSfx('arrow1.mp3');
          let name = 'Arrow';
          let bullet3d = BulletMgr.Instance().createBullet(name);
          let bullet = bullet3d.getComponent(IBullet);
          if (bullet != null) {
              bullet.destroy();
          }
          bullet = bullet3d.addComponent(Arrow);
          BulletMgr.Instance().addBullet(bullet);
          bullet3d.transform.setWorldLossyScale(new Laya.Vector3(2, 2, 1));
          bullet.firer = this;
          bullet.atked = this.atkedTarget;
          let shotDir = Util.vectorDirection(this.mtransform.position.clone(), this.atkedTarget.mtransform.position.clone());
          bullet.init(this, this.atkedTarget, shotDir, this.mShotPoint.transform.position.clone());
          bullet.rotationEuler = this.mtransform.rotationEuler.clone();
          bullet.position = this.mShotPoint.transform.position.clone();
      }
      onDamage(damage) {
          if (this.atkedInterval > 0)
              return;
          if (GameData.data.gameFinish || !this.isLife())
              return;
          if (this.teamType == TeamType.Red) {
              let bounslv = Math.max(1, GameData.data.getCommonLv(CommonCfg.HaoJiao));
              let bunsdata = GameData.data.commonCfg.HaoJiao;
              damage = Math.floor(damage * (1 + GameData.data.atkUp * (bounslv) * bunsdata.bonus1));
          }
          this.mHp -= damage;
          this.atkedInterval = 0;
          console.log('剩余血量', this.mHp);
          this.addDamageEf();
          this.healthBar.updateHp(this.mHp / this.mTotalHp, Math.max(0, this.mHp));
          this.healthBar.setActive(true);
          if (this.mHp <= 0) {
              this.healthBar.setActive(false);
              HealthBar.recover(this.healthBar);
              this.playDieAni();
              if (this.teamType == TeamType.Blue) {
                  GameMgr.Instance().playerToRemoveList(this);
                  Laya.stage.timerOnce(1000, this, () => {
                      GameMgr.Instance().removePlayer(this);
                  }, null, false);
              }
              else {
                  AudioManager.Instance.playSfx('elephant_die.mp3');
                  GameMgr.Instance().enemyToRemoveList(this);
                  Laya.stage.timerOnce(1000, this, () => {
                      GameMgr.Instance().removeEnemy(this);
                  }, null, false);
              }
              if (this.teamType == TeamType.Red) {
                  let bounslv = Math.max(1, GameData.data.getCommonLv(CommonCfg.JinBi));
                  let bunsdata = GameData.data.commonCfg.JinBi;
                  let gold = this.enemyData.gold + GameData.data.curStage * this.enemyData.goldadd * 0.5;
                  gold = Math.floor(gold * (1 + ((bounslv - 1) * bunsdata.bonus1)));
                  Laya.stage.event(EventConst.Game_Update_Gold, gold);
              }
          }
      }
      onDestroy() {
          if (this.mAtkedEf) {
              EffectMgr.Instance().removeSpriteEffect(this.mAtkedEf, true);
          }
      }
      addDamageEf() {
      }
      isInAtkRang(character) {
          if (!character)
              return false;
          let newDisSqr = Laya.Vector3.distanceSquared(character.mtransform.position, this.mtransform.position);
          let rang = Math.max(this.atkDist, character.atkRang);
          if (character.nickName == 'BiaoQiangBing') {
              rang = this.atkDist;
          }
          return (newDisSqr <= (rang) * (rang));
      }
      getNearEnemyOfAtkRang(role = null) {
          let characterList = [];
          if (this.teamType == TeamType.Red) {
              characterList = GameMgr.Instance().getPlayerList().concat();
          }
          else {
              characterList = GameMgr.Instance().getEnemyList().concat();
          }
          let target = null;
          let disSqr = this.atkDist * this.atkDist;
          for (let i = characterList.length - 1; i >= 0; i--) {
              const character = characterList[i];
              if (character == this || !character.isLife() || !character.sprite3d.active)
                  continue;
              let newDisSqr = Laya.Vector3.distanceSquared(character.mtransform.position, this.mtransform.position);
              if (newDisSqr <= disSqr && character != role) {
                  if (target == null || disSqr > newDisSqr) {
                      target = character;
                      disSqr = newDisSqr;
                  }
              }
          }
          return target;
      }
      getNearEnemyOfRang(role = null) {
          let characterList = [];
          if (this.teamType == TeamType.Red) {
              characterList = GameMgr.Instance().getPlayerList().concat();
          }
          else {
              characterList = GameMgr.Instance().getEnemyList().concat();
          }
          let target = null;
          let disSqr = 100000;
          for (let i = characterList.length - 1; i >= 0; i--) {
              const character = characterList[i];
              if (character == this || !character.isLife() || !character.sprite3d.active || character.isAtkedLock)
                  continue;
              let newDisSqr = Laya.Vector3.distanceSquared(character.mtransform.position, this.mtransform.position);
              if (newDisSqr <= disSqr && character != role) {
                  if (target == null || disSqr > newDisSqr) {
                      target = character;
                      disSqr = newDisSqr;
                  }
              }
          }
          return target;
      }
      getRandomEnemyOfRang(role = null) {
          let characterList = [];
          if (this.teamType == TeamType.Red) {
              characterList = GameMgr.Instance().getPlayerList().concat();
          }
          else {
              characterList = GameMgr.Instance().getEnemyList().concat();
          }
          let target = characterList[Util.random(0, characterList.length)];
          return target;
      }
      rayCheckWall(dir, rang) {
          if (!dir || dir == null) {
              return null;
          }
          var from = new Laya.Vector3(0, 0, 0);
          Laya.Vector3.normalize(dir, from);
          var to = new Laya.Vector3(0, 0, 0);
          var raylenght = new Laya.Vector3(0, 0, 0);
          Laya.Vector3.scale(from, -rang, raylenght);
          Laya.Vector3.add(this.mtransform.position.clone(), raylenght, to);
          this.mtransform.position.cloneTo(from);
          from.y = 1;
          to.y = 1;
          var hitResults = [];
          this.scene3d.physicsSimulation.raycastAllFromTo(from, to, hitResults, CollisionConst.GROUP_RAY, CollisionConst.CAN_RAY);
          for (let i = 0, n = hitResults.length; i < n; i++) {
              var otherSprite3d = hitResults[i].collider.owner;
              if (otherSprite3d && otherSprite3d.active) {
                  var character = otherSprite3d.getComponent(Character);
                  if (!character || (character != this && (character.teamType == this.teamType))) {
                      let hitResult = hitResults[i];
                      return hitResult;
                  }
              }
          }
          return null;
      }
  }

  class Scenario extends Character {
      onAwake() {
          super.onAwake();
          Laya.stage.on(EventConst.Update_Scenario, this, this.updateLv);
          Laya.stage.on(EventConst.RENEW_MAX_HP, this, this.onMaxHp);
          this.teamType = TeamType.Blue;
      }
      onDestroy() {
          super.onDestroy();
          Laya.stage.off(EventConst.Update_Scenario, this, this.updateLv);
          Laya.stage.off(EventConst.RENEW_MAX_HP, this, this.onMaxHp);
      }
      onStart() {
          super.onStart();
          this.collider3d = this.sprite3d.getComponent(Laya.PhysicsCollider);
          this.collider3d.enabled = true;
          this.collider3d.isTrigger = true;
          this.collider3d.collisionGroup = CollisionConst.GROUP_Wall;
          this.collider3d.canCollideWith = CollisionConst.CAN_Wall;
          this.setColliderEnable(true);
          this.updateLv();
      }
      updateLv() {
          let lv = Math.max(1, GameData.data.getCastleLv(CastleCfg.Wall));
          if (lv <= 1) {
              GameData.data.setCastleLv(1, CastleCfg.Wall);
          }
          let data = GameData.data.castleCfg.Wall;
          this.mHp = data.hp + (lv - 1) * data.hpadd;
          let bounslv = Math.max(1, GameData.data.getCommonLv(CommonCfg.ChengQiang));
          let bunsdata = GameData.data.commonCfg.ChengQiang;
          this.mHp = Math.floor(this.mHp * (1 + ((bounslv - 1) * bunsdata.bonus1)));
          this.mTotalHp = this.mHp;
          let stage = 1 + Math.floor((lv - 1) / 5);
          stage = Math.min(12, stage);
          Util.activeAllChild(this.sprite3d.getChildByName('Back_Human'), false);
          for (let index = 0; index < stage; index++) {
              this.sprite3d.getChildByName('Back_Human').getChildAt(index).active = true;
          }
      }
      onDamage(damage) {
          if (GameData.data.gameFinish || !this.isLife())
              return;
          this.mHp -= damage;
          Laya.stage.event(EventConst.Update_Hp, damage);
          console.log('剩余血量', this.mHp);
          if (this.mHp <= 0) {
              Laya.stage.event(EventConst.SHOW_FAILED);
          }
      }
      onMaxHp() {
          this.mHp = this.mTotalHp;
      }
  }

  class BallistaArrow extends IBullet {
      init(firer, atked, dir, startPos) {
          super.init(firer, atked, dir, startPos);
          this.lifeDis = firer.shotRang;
          this.speed = firer.shotSpeed;
      }
      onUpdate() {
          let dt = Laya.timer.delta / Laya.timer.scale;
          if (!this.dir)
              return;
          let curVelocity = new Laya.Vector3(0, 0, 0);
          ;
          var curSpeed = this.speed * dt * 0.001;
          Laya.Vector3.scale(this.dir, curSpeed, curVelocity);
          this.lifeDis -= Math.abs(Laya.Vector3.scalarLength(curVelocity));
          this.mTransform.translate(curVelocity, false);
      }
      onLateUpdate() {
          if (!this.dir)
              return;
          if (this.lifeDis <= 0) {
              this.removeBullet();
              this.lifeDis = 1000;
              this.dir = null;
              return;
          }
      }
      removeBullet() {
          super.removeBullet();
          let trail2 = this.bullet3d.getChildByName('Trail').getChildByName('Trail');
          trail2 && trail2.clear();
          BulletMgr.Instance().removeBullet(this);
      }
      onTriggerEnter(other) {
          var otherSprite3d = other.owner;
          console.log(otherSprite3d.name, 'onTriggerEnter.name');
          let robotObj = null;
          robotObj = otherSprite3d.getComponent(Character);
          if (robotObj && robotObj.teamType != this.firer.teamType) {
              if (robotObj.sprite3d.active && robotObj.isLife()) {
                  robotObj.onDamage(this.firer.atkDamage);
                  this.setColliderEnable(false);
                  this.lifeDis = 0;
              }
          }
      }
  }

  var AnimListener$2 = fc.plugin.AnimStateListener;
  class TowerAnimatorController extends AnimatorController {
      onAwake() {
          this.animator = this.owner.getComponent(Laya.Animator);
          this.animator.speed = 0;
      }
      onStart() {
          AnimListener$2.add(this.animator, TowerAnimatorController.fire, this);
      }
      onEnable() {
          super.onEnable();
          if (this.character == null) {
          }
      }
      onDisable() {
          this.onDestroy();
          if (this.animator) {
              AnimListener$2.remove(this.animator, TowerAnimatorController.fire, this);
          }
      }
      onAnimEvt(...args) {
      }
      onAnimStateEnter(stateName, animator) {
      }
      onAnimStateUpdate(stateName, animator) {
      }
      onAnimStateExit(stateName, animator) {
          switch (stateName) {
              case TowerAnimatorController.fire:
                  this.curClipName = null;
                  this.play(TowerAnimatorController.ready);
                  break;
              default:
                  break;
          }
      }
      onDestroy() {
          this.animator = null;
          this.character = null;
          this.curClipName = null;
      }
      play(newName) {
          if (!this.animator || !newName || this.curClipName == newName)
              return;
          if (!this.character)
              return;
          this.animator.play(newName, 0, 0);
          this.curClipName = newName;
          this.animator.speed = 1;
      }
  }
  TowerAnimatorController.ready = "Tower_0";
  TowerAnimatorController.fire = "Tower_01";

  class Tower extends Character {
      constructor() {
          super(...arguments);
          this.shotCount = 1;
      }
      onAwake() {
          super.onAwake();
      }
      onEnable() {
          super.onEnable();
      }
      onPause() {
          this.isPause = true;
          this.playIdleAni();
      }
      onStart() {
          super.onStart();
          this.setColliderEnable(false);
          this.animatorController = this.sprite3d.getComponent(TowerAnimatorController);
          if (this.animatorController) {
              this.animatorController.destroy();
              this.animatorController = null;
          }
          this.animatorController = this.sprite3d.addComponent(TowerAnimatorController);
          this.animatorController.character = this;
          this.mBallista3d = this.sprite3d.getChildByName('ballista');
          this.mShotPoint = this.mBallista3d.getChildByName('Ballista.root').getChildByName('Armature.ballista').getChildByName('BoltPosition (1)');
          this.updateLv();
          this.playIdleAni();
      }
      updateLv() {
          let lv = 0;
          if (this.teamType == TeamType.Red) {
          }
          else {
              lv = Math.max(1, GameData.data.getCastleLv(CastleCfg.Tower));
              if (lv <= 1) {
                  GameData.data.setCastleLv(1, CastleCfg.Tower);
              }
              let data = GameData.data.castleCfg.Tower;
              this.mHp = 1000;
              this.mTotalHp = this.mHp;
              this.atkRang = data.atkdist;
              this.atkDamage = data.atk * (fc.platform.PlatformConfig.platformTag == 'iptv' ? 2 : 1) + (lv - 1) * data.atkadd;
              this.shotRang = data.atkdist + 2;
              this.shotSpeed = data.shotspeed;
              this.shotCount = data.count;
          }
          let stage = 1 + (lv - 1) % 4;
          for (let index = 1; index < 5; index++) {
              if (index <= stage) {
                  this.mBallista3d.getChildByName(index + 'Up').active = true;
              }
              else {
                  this.mBallista3d.getChildByName(index + 'Up').active = false;
              }
          }
      }
      onResume() {
          super.onResume();
          if (!this.healthBar) {
          }
      }
      initEvent() {
          super.initEvent();
          Laya.stage.on(EventConst.Update_Tower, this, this.updateLv);
          Laya.stage.on(EventConst.Fire_Tower, this, this.fireatk);
      }
      removeEvent() {
          super.removeEvent();
          Laya.stage.off(EventConst.Update_Tower, this, this.updateLv);
          Laya.stage.off(EventConst.Fire_Tower, this, this.fireatk);
      }
      playIdleAni() {
          this.animatorController && (this.animatorController.play(TowerAnimatorController.ready));
      }
      playAtkAni() {
          this.animatorController && (this.animatorController.play(TowerAnimatorController.fire));
      }
      playRunAni() {
      }
      playDieAni() {
      }
      move(dir) {
      }
      rotationRole(angle) {
          let angleY = this.sprite3d.transform.rotationEuler.y;
          let rotateY = angle - angleY;
          if (rotateY != 0) {
              this.sprite3d.transform.rotate(new Laya.Vector3(0, rotateY, 0), false, false);
          }
      }
      fireatk() {
          this.onAtk(null);
      }
      onAtk(atkedObj) {
          if (this.atkedTarget && this.atkedTarget.isLife() && this.isInAtkRang(this.atkedTarget)) {
              atkedObj = this.atkedTarget;
          }
          else {
              atkedObj = this.getNearEnemyOfAtkRang();
              this.atkedTarget = atkedObj;
          }
          if (this.isLife()) {
              if (atkedObj) {
                  let angle = Util.angleY(this.atkedTarget.mtransform.position.clone(), this.mtransform.position.clone());
                  let angleY = this.mBallista3d.transform.rotationEuler.y;
                  if (angle != angleY) {
                      this.mBallista3d.transform.rotate(new Laya.Vector3(0, angle - angleY, 0), false, false);
                  }
                  this.atkedTarget = atkedObj;
                  this.playAtkAni();
                  this.owner.timerOnce(10, this, this.createBullet, null, false);
              }
          }
      }
      createBullet() {
          AudioManager.Instance.playSfx('BowShoot01.mp3');
          for (let index = 0; index < this.shotCount; index++) {
              let name = 'Ballista_Arrow';
              let bullet3d = BulletMgr.Instance().createBullet(name);
              let bullet = bullet3d.getComponent(IBullet);
              if (bullet != null) {
                  bullet.destroy();
              }
              bullet = bullet3d.addComponent(BallistaArrow);
              BulletMgr.Instance().addBullet(bullet);
              bullet3d.transform.setWorldLossyScale(new Laya.Vector3(1, 1, 1));
              bullet.firer = this;
              bullet.atked = this.atkedTarget;
              let targetPos = this.atkedTarget.transform.position.clone();
              let totalWidth = 0.5 * (this.shotCount - 1);
              let start = -(totalWidth * 0.5);
              let x = start + 0.5 * index;
              targetPos.x += x;
              targetPos.y = this.atkedTarget.transform.position.y + 1;
              let shotDir = Util.vectorDirection(this.mShotPoint.transform.position.clone(), targetPos);
              bullet.init(this, this.atkedTarget, shotDir, this.mShotPoint.transform.position.clone());
              bullet.rotationEuler = this.mBallista3d.transform.rotationEuler.clone();
              bullet.position = this.mShotPoint.transform.position.clone();
          }
      }
      onDamage(damage) {
      }
      onDestroy() {
          super.onDestroy();
      }
      addDamageEf() {
      }
      getNearEnemyOfAtkRang(role = null) {
          let characterList = [];
          if (this.teamType == TeamType.Red) {
              characterList = GameMgr.Instance().getPlayerList().concat();
          }
          else {
              characterList = GameMgr.Instance().getEnemyList().concat();
          }
          let target = null;
          let disSqr = this.atkRang * this.atkRang;
          for (let i = characterList.length - 1; i >= 0; i--) {
              const character = characterList[i];
              if (character == this || !character.isLife() || !character.sprite3d.active)
                  continue;
              let newDisSqr = Laya.Vector3.distanceSquared(character.mtransform.position, this.mtransform.position);
              if (newDisSqr <= disSqr && character != role) {
                  if (target == null || disSqr > newDisSqr) {
                      target = character;
                      disSqr = newDisSqr;
                  }
              }
          }
          return target;
      }
      getNearEnemyOfRang(role = null) {
          let characterList = [];
          if (this.teamType == TeamType.Red) {
              characterList = GameMgr.Instance().getPlayerList().concat();
          }
          else {
              characterList = GameMgr.Instance().getEnemyList().concat();
          }
          let target = null;
          let disSqr = 100000;
          for (let i = characterList.length - 1; i >= 0; i--) {
              const character = characterList[i];
              if (character == this || !character.isLife() || !character.sprite3d.active || character.isAtkedLock)
                  continue;
              let newDisSqr = Laya.Vector3.distanceSquared(character.mtransform.position, this.mtransform.position);
              if (newDisSqr <= disSqr && character != role) {
                  if (target == null || disSqr > newDisSqr) {
                      target = character;
                      disSqr = newDisSqr;
                  }
              }
          }
          return target;
      }
      getRandomEnemyOfRang(role = null) {
          let characterList = [];
          if (this.teamType == TeamType.Red) {
              characterList = GameMgr.Instance().getPlayerList().concat();
          }
          else {
              characterList = GameMgr.Instance().getEnemyList().concat();
          }
          let target = characterList[Util.random(0, characterList.length)];
          return target;
      }
      rayCheckWall(dir, rang) {
          if (!dir || dir == null) {
              return null;
          }
          var from = new Laya.Vector3(0, 0, 0);
          Laya.Vector3.normalize(dir, from);
          var to = new Laya.Vector3(0, 0, 0);
          var raylenght = new Laya.Vector3(0, 0, 0);
          Laya.Vector3.scale(from, -rang, raylenght);
          Laya.Vector3.add(this.mtransform.position.clone(), raylenght, to);
          this.mtransform.position.cloneTo(from);
          from.y = 1;
          to.y = 1;
          var hitResults = [];
          this.scene3d.physicsSimulation.raycastAllFromTo(from, to, hitResults, CollisionConst.GROUP_RAY, CollisionConst.CAN_RAY);
          for (let i = 0, n = hitResults.length; i < n; i++) {
              var otherSprite3d = hitResults[i].collider.owner;
              if (otherSprite3d && otherSprite3d.active) {
                  var character = otherSprite3d.getComponent(Character);
                  if (!character || (character != this && (character.teamType == this.teamType))) {
                      let hitResult = hitResults[i];
                      return hitResult;
                  }
              }
          }
          return null;
      }
  }

  class WallAnimatorController extends AnimatorController {
      onAwake() {
          this.animator = this.owner.getComponent(Laya.Animator);
          this.animator.speed = 0;
      }
      onStart() {
      }
      onEnable() {
          super.onEnable();
          if (this.character == null) {
          }
      }
      onDisable() {
          this.onDestroy();
          if (this.animator) {
          }
      }
      onAnimEvt(...args) {
      }
      onAnimStateEnter(stateName, animator) {
      }
      onAnimStateUpdate(stateName, animator) {
      }
      onAnimStateExit(stateName, animator) {
      }
      onDestroy() {
          this.animator = null;
          this.character = null;
          this.curClipName = null;
      }
      play(newName) {
          if (!this.animator || !newName || this.curClipName == newName)
              return;
          if (!this.character)
              return;
          this.animator.play(newName, 0, 0);
          this.curClipName = newName;
          this.animator.speed = 1;
      }
  }
  WallAnimatorController.open = "open";
  WallAnimatorController.close = "close";
  WallAnimatorController.collapse = "collapse";

  class Wall extends Character {
      constructor() {
          super(...arguments);
          this.isCloseDooring = false;
      }
      onAwake() {
          super.onAwake();
      }
      onEnable() {
          super.onEnable();
      }
      onPause() {
          this.isPause = true;
          this.playIdleAni();
      }
      onStart() {
          super.onStart();
          this.animatorController = this.sprite3d.getComponent(WallAnimatorController);
          if (this.animatorController) {
              this.animatorController.destroy();
              this.animatorController = null;
          }
          this.animatorController = this.sprite3d.addComponent(WallAnimatorController);
          this.animatorController.character = this;
          this.updateLv();
          this.playIdleAni();
      }
      updateLv() {
          let lv = 0;
          if (this.teamType == TeamType.Red) {
          }
          else {
              lv = Math.max(1, GameData.data.getCastleLv(CastleCfg.Wall));
              if (lv <= 1) {
                  GameData.data.setCastleLv(1, CastleCfg.Wall);
              }
          }
          let stage = 1 + Math.floor((lv - 1) / 20);
          let up = 0;
          let count = 2;
          if (stage == 1) {
              up = 1 + Math.floor((lv - 1) / 10);
              up = Math.min(2, up);
              count = 2;
          }
          else if (stage == 2) {
              up = 1 + Math.floor((lv - 20 * (stage - 1) - 1) / 4);
              up = Math.min(5, up);
              count = 5;
          }
          else if (stage == 3) {
              up = 1 + Math.floor((lv - 20 * (stage - 1) - 1) / 5);
              up = Math.min(4, up);
              count = 4;
          }
          else if (stage >= 4) {
              up = 1 + Math.floor((lv - 20 * (4 - 1) - 1) / 5);
              up = Math.min(4, up);
              count = 4;
          }
          for (let index = 1; index < count + 1; index++) {
              if (index <= up) {
                  this.sprite3d.getChildByName('UP' + index).active = true;
              }
              else {
                  this.sprite3d.getChildByName('UP' + index).active = false;
              }
          }
      }
      onResume() {
          super.onResume();
          if (!this.healthBar) {
          }
      }
      initEvent() {
          super.initEvent();
          Laya.stage.on(EventConst.Open_Door, this, this.playOpenAni);
          Laya.stage.on(EventConst.Close_Door, this, this.closeDoor);
          Laya.stage.on(EventConst.SHOW_FAILED, this, this.playCollapseAni);
      }
      removeEvent() {
          super.removeEvent();
          Laya.stage.off(EventConst.Open_Door, this, this.playOpenAni);
          Laya.stage.off(EventConst.Close_Door, this, this.closeDoor);
          Laya.stage.off(EventConst.SHOW_FAILED, this, this.playCollapseAni);
      }
      playIdleAni() {
          this.animatorController && (this.animatorController.play(WallAnimatorController.idle));
      }
      playOpenAni() {
          AudioManager.Instance.playSfx('open_door.mp3');
          this.animatorController && (this.animatorController.play(WallAnimatorController.open));
      }
      playCloseAni() {
          this.animatorController && (this.animatorController.play(WallAnimatorController.close));
      }
      playCollapseAni() {
          this.animatorController && (this.animatorController.play(WallAnimatorController.collapse));
      }
      closeDoor() {
          this.isCloseDooring = true;
      }
      onUpdate() {
          if (this.isCloseDooring && this.isEnableClose()) {
              this.playCloseAni();
              this.isCloseDooring = false;
          }
      }
      move(dir) {
      }
      rotationRole(angle) {
          let angleY = this.sprite3d.transform.rotationEuler.y;
          let rotateY = angle - angleY;
          if (rotateY != 0) {
              this.sprite3d.transform.rotate(new Laya.Vector3(0, rotateY, 0), false, false);
          }
      }
      onAtk(atkedObj) {
      }
      onDestroy() {
          super.onDestroy();
      }
      addDamageEf() {
      }
      isEnableClose() {
          let characterList = GameMgr.Instance().getPlayerList().concat();
          ;
          let ret = true;
          for (let i = characterList.length - 1; i >= 0; i--) {
              const character = characterList[i];
              if (character.transform.position.z < 1) {
                  ret = false;
              }
          }
          return ret;
      }
      getNearEnemyOfAtkRang(role = null) {
          let characterList = [];
          if (this.teamType == TeamType.Red) {
              characterList = GameMgr.Instance().getPlayerList().concat();
          }
          else {
              characterList = GameMgr.Instance().getEnemyList().concat();
          }
          let target = null;
          let disSqr = this.atkRang * this.atkRang;
          for (let i = characterList.length - 1; i >= 0; i--) {
              const character = characterList[i];
              if (character == this || !character.isLife() || !character.sprite3d.active)
                  continue;
              let newDisSqr = Laya.Vector3.distanceSquared(character.mtransform.position, this.mtransform.position);
              if (newDisSqr <= disSqr && character != role) {
                  if (target == null || disSqr > newDisSqr) {
                      target = character;
                      disSqr = newDisSqr;
                  }
              }
          }
          return target;
      }
      getNearEnemyOfRang(role = null) {
          let characterList = [];
          if (this.teamType == TeamType.Red) {
              characterList = GameMgr.Instance().getPlayerList().concat();
          }
          else {
              characterList = GameMgr.Instance().getEnemyList().concat();
          }
          let target = null;
          let disSqr = 100000;
          for (let i = characterList.length - 1; i >= 0; i--) {
              const character = characterList[i];
              if (character == this || !character.isLife() || !character.sprite3d.active || character.isAtkedLock)
                  continue;
              let newDisSqr = Laya.Vector3.distanceSquared(character.mtransform.position, this.mtransform.position);
              if (newDisSqr <= disSqr && character != role) {
                  if (target == null || disSqr > newDisSqr) {
                      target = character;
                      disSqr = newDisSqr;
                  }
              }
          }
          return target;
      }
      getRandomEnemyOfRang(role = null) {
          let characterList = [];
          if (this.teamType == TeamType.Red) {
              characterList = GameMgr.Instance().getPlayerList().concat();
          }
          else {
              characterList = GameMgr.Instance().getEnemyList().concat();
          }
          let target = characterList[Util.random(0, characterList.length)];
          return target;
      }
      rayCheckWall(dir, rang) {
          if (!dir || dir == null) {
              return null;
          }
          var from = new Laya.Vector3(0, 0, 0);
          Laya.Vector3.normalize(dir, from);
          var to = new Laya.Vector3(0, 0, 0);
          var raylenght = new Laya.Vector3(0, 0, 0);
          Laya.Vector3.scale(from, -rang, raylenght);
          Laya.Vector3.add(this.mtransform.position.clone(), raylenght, to);
          this.mtransform.position.cloneTo(from);
          from.y = 1;
          to.y = 1;
          var hitResults = [];
          this.scene3d.physicsSimulation.raycastAllFromTo(from, to, hitResults, CollisionConst.GROUP_RAY, CollisionConst.CAN_RAY);
          for (let i = 0, n = hitResults.length; i < n; i++) {
              var otherSprite3d = hitResults[i].collider.owner;
              if (otherSprite3d && otherSprite3d.active) {
                  var character = otherSprite3d.getComponent(Character);
                  if (!character || (character != this && (character.teamType == this.teamType))) {
                      let hitResult = hitResults[i];
                      return hitResult;
                  }
              }
          }
          return null;
      }
  }

  class Bomb extends IBullet {
      constructor() {
          super(...arguments);
          this.tipEffect = null;
          this.targetPos = Util.ZERO.clone();
          this.isEnemy = false;
          this.atkDamage = 0;
          this.bombDamageRang = 0;
      }
      init(firer, atked, dir, startPos) {
          super.init(firer, atked, dir, startPos);
          this.lifeDis = firer.shotRang;
          this.speed = firer.shotSpeed;
          this.atkDamage = firer.atkDamage;
          this.bombDamageRang = firer.atkradius;
          this.isEnemy = firer.teamType == TeamType.Red;
          this.targetPos = atked.mtransform.position.clone();
      }
      onStart() {
          super.onStart();
          this.flyTrack();
      }
      flyTrack() {
          var curPos = this.mTransform.position.clone();
          if (!this.isEnemy) {
              this.mTransform.position = curPos;
          }
          var positions = Bezier.creatorTwoBezierPoint(curPos, new Laya.Vector3((curPos.x + this.targetPos.x) * 0.5, Math.max(10, curPos.y), (curPos.z + this.targetPos.z) * 0.5), this.targetPos.clone(), 10);
          var hander = new Laya.Handler(this, function () {
              this.mTransform.position = this.mTransform.position;
          });
          let dist = Laya.Vector3.distance(curPos, this.targetPos);
          let flytime = 1000 * dist / this.speed;
          flytime = Math.max(flytime, 500);
          var totleTime = 0;
          var tweenTime = flytime / positions.length;
          var d = flytime / (positions.length * (positions.length - 1));
          var curTime = tweenTime;
          for (let i = 0; i < positions.length; i++) {
              Laya.Tween.to(this.mTransform.position, {
                  y: positions[i].y,
                  z: positions[i].z,
                  x: positions[i].x,
                  update: hander
              }, curTime, null, Laya.Handler.create(this, (i) => {
                  if (i == positions.length - 1) {
                      this.mTransform.position = this.targetPos.clone();
                      this.onDamage();
                      this.addHitEffect();
                  }
                  else {
                      let targetDir = Util.vectorDirection(this.mTransform.position, positions[i + 1]);
                      let angley = Util.angleY2(targetDir);
                      let rotation = new Laya.Vector3(-Util.angleDirection(targetDir, Util.UP), this.mTransform.rotationEuler.y, 0);
                      this.mTransform.rotationEuler = rotation;
                  }
              }, [i]), totleTime);
              totleTime += curTime;
              curTime += d;
          }
      }
      rotationBullet() {
          if (!this.atked || !this.atked.isLife())
              return;
          let targetPos = this.atked.mtransform.position.clone();
          targetPos.y += 3.5;
          let targetDir = Util.vectorDirection(this.bullet3d.transform.position.clone(), targetPos);
          let angley = Util.angleY2(targetDir);
          let rotation = new Laya.Vector3(-Util.angleDirection(targetDir, Util.UP), angley, 0);
          this.mTransform.rotationEuler = rotation;
          this.dir = targetDir;
      }
      removeBullet() {
          super.removeBullet();
          if (this.tipEffect) {
              EffectMgr.Instance().removeSpriteEffect(this.tipEffect);
          }
          BulletMgr.Instance().removeBullet(this);
      }
      initEffect() {
          super.initEffect();
          this.addBombTipEffect();
      }
      addHitEffect() {
          let efname = 'ef_blast_01';
          AudioManager.Instance.playSfx("etfx_explosion_rocket.mp3");
          let ef = EffectMgr.Instance().createSpriteEffect(efname);
          ef.transform.position = this.targetPos;
          this.bullet3d.active = false;
          Laya.Tween.to(ef.transform, {}, 1500, null, Laya.Handler.create(this, (ef) => {
              EffectMgr.Instance().removeSpriteEffect(ef);
              this.removeBullet();
          }, [ef]));
      }
      addBombTipEffect() {
          if (this.isEnemy) {
              this.tipEffect = EffectMgr.Instance().createSpriteEffect('ef_range_01');
              this.tipEffect.transform.position = this.targetPos;
              Laya.Tween.to(this.tipEffect.transform, {}, 2500, null, Laya.Handler.create(this, (ef) => {
                  EffectMgr.Instance().removeSpriteEffect(ef);
                  this.tipEffect = null;
              }, [this.tipEffect]));
          }
      }
      onDamage() {
          if (this.isEnemy) {
          }
          else {
              AudioManager.Instance.playSfx('FireBallStart.mp3');
              let list = GameMgr.Instance().getEnmeyByRang(this.bombDamageRang, this.mTransform.position);
              for (let index = 0; index < list.length; index++) {
                  let enemy = list[index];
                  enemy.onDamage(this.atkDamage);
              }
          }
      }
  }

  class CatapultAnimatorController extends AnimatorController {
      onAwake() {
          this.animator = this.owner.getComponent(Laya.Animator);
          this.animator.speed = 0;
      }
      onStart() {
      }
      onEnable() {
          super.onEnable();
          if (this.character == null) {
          }
      }
      onDisable() {
          this.onDestroy();
          if (this.animator) {
          }
      }
      onAnimEvt(...args) {
      }
      onAnimStateEnter(stateName, animator) {
      }
      onAnimStateUpdate(stateName, animator) {
      }
      onAnimStateExit(stateName, animator) {
      }
      onDestroy() {
          this.animator = null;
          this.character = null;
          this.curClipName = null;
      }
      play(newName) {
          if (!this.animator || !newName)
              return;
          if (!this.character)
              return;
          this.animator.play(newName, 0, 0);
          this.curClipName = newName;
          this.animator.speed = 1;
      }
  }
  CatapultAnimatorController.catapult = "catapult";

  class TouShiJi extends Character {
      constructor() {
          super(...arguments);
          this.soldierController = null;
          this.mAtkedEf = null;
          this.mEfName = '';
      }
      onAwake() {
          super.onAwake();
      }
      onEnable() {
          super.onEnable();
      }
      onPause() {
          this.isPause = true;
          this.playIdleAni();
      }
      onStart() {
          super.onStart();
          this.setColliderEnable(false);
          this.animatorController = this.sprite3d.getChildByName('Catapult_0').getComponent(CatapultAnimatorController);
          if (this.animatorController) {
              this.animatorController.destroy();
              this.animatorController = null;
          }
          this.animatorController = this.sprite3d.getChildByName('Catapult_0').addComponent(CatapultAnimatorController);
          this.animatorController.character = this;
          this.hpPoint = this.sprite3d.getChildByName('hp');
          this.updateLv();
          this.mShotPoint = this.sprite3d.getChildByName('Catapult_0').getChildByName('BoulderPosition');
          if (this.mAtkedEf) {
              this.mAtkedEf.removeSelf();
              this.mAtkedEf.destroy();
          }
          this.playIdleAni();
      }
      updateLv() {
          let lv = 0;
          if (this.teamType == TeamType.Red) {
          }
          else {
              lv = Math.max(1, GameData.data.getCastleLv(CastleCfg.Catapult));
              if (lv <= 1) {
                  GameData.data.setCastleLv(1, CastleCfg.Catapult);
              }
              let archerdata = GameData.data.castleCfg.Catapult;
              this.mHp = 1000;
              this.mTotalHp = this.mHp;
              this.moveSpeed = 0;
              this.atkRang = archerdata.atkdist;
              this.atkIntvalTemp = 0;
              this.atkIntval = 0;
              this.atkDamage = archerdata.atk * (fc.platform.PlatformConfig.platformTag == 'iptv' ? 2 : 1) + (lv - 1) * archerdata.atkadd;
              this.shotRang = archerdata.atkdist;
              this.shotSpeed = archerdata.shotspeed;
              this.damageRang = archerdata.atkdist;
              this.atkradius = archerdata.atkradius;
              if (lv < 10) {
                  this.sprite3d.getChildByName('Catapult_0').getChildByName('Up1').active = true;
                  this.sprite3d.getChildByName('Catapult_0').getChildByName('Up2').active = false;
              }
              else {
                  this.sprite3d.getChildByName('Catapult_0').getChildByName('Up1').active = true;
                  this.sprite3d.getChildByName('Catapult_0').getChildByName('Up2').active = true;
              }
          }
      }
      onResume() {
          super.onResume();
          if (!this.healthBar) {
          }
      }
      initEvent() {
          super.initEvent();
          Laya.stage.on(EventConst.Update_Catapult, this, this.updateLv);
          Laya.stage.on(EventConst.Fire_Catapult, this, this.fireatk);
      }
      removeEvent() {
          super.removeEvent();
          Laya.stage.off(EventConst.Update_Catapult, this, this.updateLv);
          Laya.stage.off(EventConst.Fire_Catapult, this, this.fireatk);
      }
      playIdleAni() {
      }
      playAtkAni() {
          AudioManager.Instance.playSfx('catapulta-disparo.mp3');
          this.animatorController && (this.animatorController.play(CatapultAnimatorController.catapult));
      }
      playRunAni() {
      }
      playDieAni() {
      }
      move(dir) {
      }
      rotationRole(angle) {
          let angleY = this.sprite3d.transform.rotationEuler.y;
          let rotateY = angle - angleY;
          if (rotateY != 0) {
              this.sprite3d.transform.rotate(new Laya.Vector3(0, rotateY, 0), false, false);
          }
      }
      fireatk() {
          this.onAtk(null);
      }
      onAtk(atkedObj) {
          if (this.atkedTarget && this.atkedTarget.isLife() && this.isInAtkRang(this.atkedTarget)) {
              atkedObj = this.atkedTarget;
          }
          else {
              atkedObj = this.getNearEnemyOfAtkRang();
              this.atkedTarget = atkedObj;
          }
          if (this.isLife()) {
              if (atkedObj) {
                  this.atkedTarget = atkedObj;
                  this.playAtkAni();
                  this.owner.timerOnce(10, this, this.createBullet, null, false);
              }
          }
      }
      createBullet() {
          AudioManager.Instance.playSfx('archer_attack.mp3');
          let name = 'Bullet_Pistol';
          let bullet3d = BulletMgr.Instance().createBullet(name);
          let bullet = bullet3d.getComponent(IBullet);
          if (bullet != null) {
              bullet.destroy();
          }
          bullet = bullet3d.addComponent(Bomb);
          BulletMgr.Instance().addBullet(bullet);
          bullet3d.transform.setWorldLossyScale(new Laya.Vector3(1, 1, 1));
          bullet.firer = this;
          bullet.atked = this.atkedTarget;
          let shotDir = Util.vectorDirection(this.mtransform.position.clone(), this.atkedTarget.mtransform.position.clone());
          bullet.init(this, this.atkedTarget, shotDir, this.mShotPoint.transform.position.clone());
          bullet.rotationEuler = this.mShotPoint.transform.rotationEuler.clone();
          bullet.position = this.mShotPoint.transform.position.clone();
      }
      onDamage(damage) {
      }
      onDestroy() {
          if (this.mAtkedEf) {
              EffectMgr.Instance().removeSpriteEffect(this.mAtkedEf, true);
          }
      }
      addDamageEf() {
      }
      getNearEnemyOfAtkRang(role = null) {
          let characterList = [];
          if (this.teamType == TeamType.Red) {
              characterList = GameMgr.Instance().getPlayerList().concat();
          }
          else {
              characterList = GameMgr.Instance().getEnemyList().concat();
          }
          let target = null;
          let disSqr = this.atkRang * this.atkRang;
          for (let i = characterList.length - 1; i >= 0; i--) {
              const character = characterList[i];
              if (character == this || !character.isLife() || !character.sprite3d.active)
                  continue;
              let newDisSqr = Laya.Vector3.distanceSquared(character.mtransform.position, this.mtransform.position);
              if (newDisSqr <= disSqr && character != role) {
                  if (target == null || disSqr > newDisSqr) {
                      target = character;
                      disSqr = newDisSqr;
                  }
              }
          }
          return target;
      }
      getNearEnemyOfRang(role = null) {
          let characterList = [];
          if (this.teamType == TeamType.Red) {
              characterList = GameMgr.Instance().getPlayerList().concat();
          }
          else {
              characterList = GameMgr.Instance().getEnemyList().concat();
          }
          let target = null;
          let disSqr = 100000;
          for (let i = characterList.length - 1; i >= 0; i--) {
              const character = characterList[i];
              if (character == this || !character.isLife() || !character.sprite3d.active)
                  continue;
              let newDisSqr = Laya.Vector3.distanceSquared(character.mtransform.position, this.mtransform.position);
              if (newDisSqr <= disSqr && character != role) {
                  if (target == null || disSqr > newDisSqr) {
                      target = character;
                      disSqr = newDisSqr;
                  }
              }
          }
          return target;
      }
      getRandomEnemyOfRang(role = null) {
          let characterList = [];
          if (this.teamType == TeamType.Red) {
              characterList = GameMgr.Instance().getPlayerList().concat();
          }
          else {
              characterList = GameMgr.Instance().getEnemyList().concat();
          }
          let target = characterList[Util.random(0, characterList.length)];
          return target;
      }
      rayCheckWall(dir, rang) {
          if (!dir || dir == null) {
              return null;
          }
          var from = new Laya.Vector3(0, 0, 0);
          Laya.Vector3.normalize(dir, from);
          var to = new Laya.Vector3(0, 0, 0);
          var raylenght = new Laya.Vector3(0, 0, 0);
          Laya.Vector3.scale(from, -rang, raylenght);
          Laya.Vector3.add(this.mtransform.position.clone(), raylenght, to);
          this.mtransform.position.cloneTo(from);
          from.y = 1;
          to.y = 1;
          var hitResults = [];
          this.scene3d.physicsSimulation.raycastAllFromTo(from, to, hitResults, CollisionConst.GROUP_RAY, CollisionConst.CAN_RAY);
          for (let i = 0, n = hitResults.length; i < n; i++) {
              var otherSprite3d = hitResults[i].collider.owner;
              if (otherSprite3d && otherSprite3d.active) {
                  var character = otherSprite3d.getComponent(Character);
                  if (!character || (character != this && (character.teamType == this.teamType))) {
                      let hitResult = hitResults[i];
                      return hitResult;
                  }
              }
          }
          return null;
      }
  }

  var Resource$3 = fc.framework.Resource;
  var ObjectPool$3 = fc.plugin.ObjectPool;
  class GameMgr {
      constructor() {
          this.enemys = [];
          this.enemyList = [];
          this.playerList = [];
          this.removeList = [];
          this.playerStartPos = new Laya.Vector3(0, 0, -8);
          this.npcStartPos = new Laya.Vector3(0, 0, 24);
          this.players = [];
          this.npcs = [];
          this.stageroot3d = null;
          this.mWall = null;
          this.mScenario = null;
          this.mEemeyRoot = null;
          this.mPlayerRoot = null;
          this.mEnemyLave = 0;
          this.mEnemyTotal = 0;
          this.scenario = null;
          this.wall3d = null;
          this.archerList = [];
          this.tempTotal = 0;
          this.playDieSfx = false;
      }
      static Instance() {
          if (this.instance == null)
              this.instance = new GameMgr();
          return this.instance;
      }
      get enemyLave() {
          return this.mEnemyLave;
      }
      get enemyTotal() {
          return this.mEnemyTotal;
      }
      getPlayer() {
          return this.players;
      }
      getNpc() {
          return this.npcs;
      }
      init(parent) {
          this.parent = parent;
      }
      getEnemyList() {
          return this.enemyList;
      }
      getPlayerList() {
          return this.playerList;
      }
      clearRootSprite(sprite3d) {
          if (!sprite3d)
              return;
          this.clearSubSprite(sprite3d);
      }
      clearSubSprite(sprite3d) {
          if (!sprite3d)
              return;
          for (let index = 0; index < sprite3d.numChildren; index++) {
              const element = sprite3d.getChildAt(index);
              element && (this.clearSprite(element, false));
          }
      }
      clearSprite(sprite3d, isRecover = true) {
          if (!sprite3d)
              return;
          while (sprite3d.numChildren > 0) {
              let el = sprite3d.getChildAt(0);
              let scripts3d = el.getComponents(Laya.Script3D);
              if (scripts3d) {
                  for (let index = 0; index < scripts3d.length; index++) {
                      let script = scripts3d[index];
                      if (script) {
                          script.destroy();
                      }
                  }
              }
              let scripts = el.getComponents(Laya.Script);
              if (scripts) {
                  for (let index = 0; index < scripts.length; index++) {
                      let script = scripts[index];
                      if (script) {
                          script.destroy();
                      }
                  }
              }
              el.active = false;
              if (isRecover) {
                  ObjectPool$3.recoverSprite3d(el);
                  sprite3d.removeChild(el);
              }
              else {
                  let rigidBody = el.getComponent(Laya.Rigidbody3D);
                  if (rigidBody) {
                      let meshColliderShape = rigidBody.colliderShape;
                      if (meshColliderShape && meshColliderShape.mesh) {
                          meshColliderShape.mesh.destroy();
                          rigidBody.destroy();
                      }
                  }
                  el.removeSelf();
                  el.destroy(true);
              }
          }
      }
      initStageById(stageId) {
          this.enemyList = [];
          this.playerList = [];
          this.removeList = [];
          this.mEemeyRoot = this.parent.getChildByName('Red');
          this.mPlayerRoot = this.parent.getChildByName('Blue');
          this.clearSprite(this.mEemeyRoot);
          this.clearSprite(this.mPlayerRoot);
          if (GameData.data.curStage > GameData.data.stageCount) {
              let rate = 1 + Math.floor(GameData.data.curStage / 100);
              let maxstage = Math.min(200, GameData.data.curStage);
              if (fc.platform.PlatformConfig.platformTag == 'iptv') {
                  rate = rate + 1;
              }
              this.curStageData = GameData.data.getStageData(Math.floor(maxstage * 0.5));
              if (GameData.data.curStage % 30 == 0) {
                  this.enemys = [0, 0, 0, 0, 0, Math.floor(GameData.data.curStage / 30)];
              }
              else {
                  for (let index = 0; index < this.curStageData.enemys.length; index++) {
                      this.enemys[index] = this.curStageData.enemys[index] * rate;
                  }
              }
          }
          else {
              this.curStageData = GameData.data.getStageData(GameData.data.curStage);
              if (fc.platform.PlatformConfig.platformTag == 'iptv') {
                  for (let index = 0; index < this.curStageData.enemys.length; index++) {
                      this.enemys[index] = this.curStageData.enemys[index] * 2;
                  }
              }
              else {
                  this.enemys = this.curStageData.enemys.concat();
              }
          }
          this.mEnemyLave = 0;
          for (let index = 0; index < this.enemys.length; index++) {
              this.mEnemyLave += this.enemys[index];
          }
          this.tempTotal = this.mEnemyLave;
          this.mEnemyTotal = this.mEnemyLave;
      }
      getScenario() {
          return this.scenario;
      }
      initMainHome() {
          this.stageroot3d = this.parent.getChildByName(ResConst.Sprite3d_Scene);
          this.clearRootSprite(this.stageroot3d);
          this.mScenario = this.stageroot3d.getChildByName('Scenario');
          this.mScenario.active = true;
          this.initScenario();
          this.initWall();
          this.initTower();
          this.initCatapult();
          this.initArcher();
          this.initXianFeng();
      }
      initScenario() {
          this.createOtherSprite3dAsyn(ResourceMgr.threeDResRootPath2, 'Scenario_Humans', 'Scenario_Humans', this.mScenario, Laya.Handler.create(this, (clone3d) => {
              clone3d.transform.position = Util.ZERO.clone();
              clone3d.transform.setWorldLossyScale(new Laya.Vector3(1, 1, 1));
              clone3d.transform.rotationEuler = new Laya.Vector3(0, 0, 0);
              this.removeComponent(clone3d);
              this.scenario = clone3d.addComponent(Scenario);
          }));
      }
      updateScenario() {
          this.addTower();
          this.addCatapult();
          this.initWall();
          Laya.stage.event(EventConst.Update_Scenario);
          let lv = Math.max(1, GameData.data.getCastleLv(CastleCfg.Wall));
          if (lv == 30) {
              GameData.data.setWarriorLv(1, WarriorType.XianFeng);
              this.addXianFeng();
          }
      }
      initTower() {
          let tower3d = this.stageroot3d.getChildByName('Tower');
          let tlv = Math.max(1, GameData.data.getCastleLv(CastleCfg.Tower));
          let towerLv = Math.floor((tlv - 1) / 16);
          towerLv = Math.min(3, towerLv);
          let lv = Math.max(1, GameData.data.getCastleLv(CastleCfg.Wall));
          if (lv < 12 && lv > 3) {
              this.createOtherSprite3dAsyn(ResourceMgr.threeDResRootPath2, 'Tower_' + towerLv, 'Tower_' + towerLv, tower3d, Laya.Handler.create(this, (clone3d) => {
                  clone3d.transform.position = new Laya.Vector3(4.5, 0, -5);
                  clone3d.transform.setWorldLossyScale(new Laya.Vector3(1, 1, 1));
                  clone3d.transform.rotationEuler = new Laya.Vector3(0, 0, 0);
                  this.removeComponent(clone3d);
                  let tower = clone3d.addComponent(Tower);
                  tower.teamType = TeamType.Blue;
              }));
          }
          else if (lv >= 12) {
              this.createOtherSprite3dAsyn(ResourceMgr.threeDResRootPath2, 'Tower_' + towerLv, 'Tower_' + towerLv, tower3d, Laya.Handler.create(this, (clone3d) => {
                  clone3d.transform.position = new Laya.Vector3(4.5, 0, -5);
                  clone3d.transform.setWorldLossyScale(new Laya.Vector3(1, 1, 1));
                  clone3d.transform.rotationEuler = new Laya.Vector3(0, 0, 0);
                  this.removeComponent(clone3d);
                  let tower = clone3d.addComponent(Tower);
                  tower.teamType = TeamType.Blue;
              }));
              this.createOtherSprite3dAsyn(ResourceMgr.threeDResRootPath2, 'Tower_' + towerLv, 'Tower_' + towerLv, tower3d, Laya.Handler.create(this, (clone3d) => {
                  clone3d.transform.position = new Laya.Vector3(-4.5, 0, -5);
                  clone3d.transform.setWorldLossyScale(new Laya.Vector3(1, 1, 1));
                  clone3d.transform.rotationEuler = new Laya.Vector3(0, 0, 0);
                  this.removeComponent(clone3d);
                  let tower = clone3d.addComponent(Tower);
                  tower.teamType = TeamType.Blue;
              }));
          }
      }
      addTower() {
          let tower3d = this.stageroot3d.getChildByName('Tower');
          this.clearSprite(tower3d, true);
          this.initTower();
      }
      initWall() {
          let aider = this.stageroot3d.getChildByName('Wall');
          let lv = Math.max(1, GameData.data.getCastleLv(CastleCfg.Wall));
          let stage = Math.floor((lv - 1) / 20);
          stage = Math.min(3, stage);
          if (this.wall3d) {
              this.removeComponent(this.wall3d);
              this.wall3d.removeSelf();
              this.wall3d.destroy(true);
              this.wall3d = null;
          }
          this.createOtherSprite3dAsyn(ResourceMgr.threeDResRootPath2, 'WallLvl' + stage, 'WallLvl' + stage, aider, Laya.Handler.create(this, (clone3d) => {
              clone3d.transform.position = new Laya.Vector3(0, 0, 0);
              clone3d.transform.setWorldLossyScale(new Laya.Vector3(1, 1, 1));
              clone3d.transform.rotationEuler = new Laya.Vector3(0, 0, 0);
              this.removeComponent(clone3d);
              let wall = clone3d.addComponent(Wall);
              wall.teamType = TeamType.Blue;
              this.wall3d = clone3d;
          }));
          this.resetArcherPos();
      }
      resetArcherPos() {
          for (let i = this.archerList.length - 1; i >= 0; i--) {
              let character = this.archerList[i];
              let x = character.position.x;
              let z = character.position.z;
              let y = 0;
              let walllv = Math.max(1, GameData.data.getCastleLv(CastleCfg.Wall));
              let stage = Math.floor((walllv - 1) / 20);
              stage = Math.min(3, stage);
              switch (stage) {
                  case 0:
                      y = 0;
                      break;
                  case 1:
                      y = 0.66;
                      break;
                  case 2:
                      y = 0.76;
                      break;
                  case 3:
                      y = 1.87;
                      break;
                  default:
                      break;
              }
              character.transform.position = new Laya.Vector3(x, y, z);
          }
      }
      initArcher() {
          this.archerList = [];
          let x = 3;
          let z = -0.3;
          let y = 0;
          let walllv = Math.max(1, GameData.data.getCastleLv(CastleCfg.Wall));
          let stage = Math.floor((walllv - 1) / 20);
          stage = Math.min(3, stage);
          switch (stage) {
              case 0:
                  y = 0;
                  break;
              case 1:
                  y = 0.66;
                  break;
              case 2:
                  y = 0.76;
                  break;
              case 3:
                  y = 1.87;
                  break;
              default:
                  break;
          }
          let lv = Math.max(1, GameData.data.getCastleLv(CastleCfg.Archer));
          let archer = this.stageroot3d.getChildByName('Archer');
          for (let index = 0; index < lv; index++) {
              if (Math.floor((index) / 5) == 0) {
                  x = -3 - index * 1.4;
                  z = -0.3;
              }
              if (Math.floor((index) / 5) == 1) {
                  x = 3 + (index % 5) * 1.4;
                  z = -0.3;
              }
              if (Math.floor((index) / 5) == 2) {
                  x = -3.7 - (index % 5) * 1.4;
                  z = -0.6;
              }
              if (Math.floor((index) / 5) >= 3) {
                  x = 3.7 + (index % 5) * 1.4;
                  z = -0.6;
              }
              let pos = new Laya.Vector3(x, y, z);
              this.createWarror(archer, 'Role', WarriorType.GongJian, pos);
          }
      }
      addArcher() {
          let x = 3;
          let z = -0.3;
          let y = 0;
          let walllv = Math.max(1, GameData.data.getCastleLv(CastleCfg.Wall));
          let stage = Math.floor((walllv - 1) / 20);
          switch (stage) {
              case 0:
                  y = 0;
                  break;
              case 1:
                  y = 0.66;
                  break;
              case 2:
                  y = 0.76;
                  break;
              case 3:
                  y = 1.87;
                  break;
              default:
                  break;
          }
          let index = Math.max(1, GameData.data.getCastleLv(CastleCfg.Archer)) - 1;
          if (Math.floor((index) / 5) == 0) {
              x = -3 - index * 1.4;
              z = -0.3;
          }
          if (Math.floor((index) / 5) == 1) {
              x = 3 + (index % 5) * 1.4;
              z = -0.3;
          }
          if (Math.floor((index) / 5) == 2) {
              x = -3.7 - index % 5 * 1.4;
              z = -0.6;
          }
          if (Math.floor((index) / 5) >= 3) {
              x = 3.7 + (index % 5) * 1.4;
              z = -0.6;
          }
          let pos = new Laya.Vector3(x, y, z);
          let archer = this.stageroot3d.getChildByName('Archer');
          this.createWarror(archer, 'Role', WarriorType.GongJian, pos);
      }
      initCatapult() {
          let lv = Math.max(1, GameData.data.getCastleLv(CastleCfg.Wall));
          if (lv >= 24) {
              let aider = this.stageroot3d.getChildByName('Aider');
              this.createSprite3dAsyn('Catapult', 'Catapult', aider, Laya.Handler.create(this, (clone3d) => {
                  clone3d.transform.position = new Laya.Vector3(-7, 0, -5);
                  clone3d.transform.setWorldLossyScale(new Laya.Vector3(1, 1, 1));
                  clone3d.transform.rotationEuler = new Laya.Vector3(0, 0, 0);
                  this.removeComponent(clone3d);
                  let wall = clone3d.addComponent(TouShiJi);
                  wall.teamType = TeamType.Blue;
              }));
          }
      }
      addCatapult() {
          let lv = Math.max(1, GameData.data.getCastleLv(CastleCfg.Wall));
          if (lv == 24) {
              let aider = this.stageroot3d.getChildByName('Aider');
              this.createSprite3dAsyn('Catapult', 'Catapult', aider, Laya.Handler.create(this, (clone3d) => {
                  clone3d.transform.position = new Laya.Vector3(-7, 0, -5);
                  clone3d.transform.setWorldLossyScale(new Laya.Vector3(1, 1, 1));
                  clone3d.transform.rotationEuler = new Laya.Vector3(0, 0, 0);
                  this.removeComponent(clone3d);
                  let wall = clone3d.addComponent(TouShiJi);
                  wall.teamType = TeamType.Blue;
              }));
          }
      }
      initXianFeng() {
          let lv = Math.max(0, GameData.data.getWarriorLv(WarriorType.XianFeng));
          if (lv == 0)
              return;
          let count = 1 + Math.floor((lv - 1) / 8);
          switch (count) {
              case 1:
                  {
                      let pos = new Laya.Vector3(3, 0, 5);
                      this.createWarror(this.mPlayerRoot, 'Role', WarriorType.XianFeng, pos);
                  }
                  break;
              case 2:
                  {
                      let pos = new Laya.Vector3(3, 0, 5);
                      this.createWarror(this.mPlayerRoot, 'Role', WarriorType.XianFeng, pos);
                      let pos2 = new Laya.Vector3(3, 0, 7);
                      this.createWarror(this.mPlayerRoot, 'Role', WarriorType.XianFeng, pos2);
                  }
                  break;
              case 3:
                  {
                      let pos = new Laya.Vector3(3, 0, 5);
                      this.createWarror(this.mPlayerRoot, 'Role', WarriorType.XianFeng, pos);
                      let pos2 = new Laya.Vector3(3, 0, 7);
                      this.createWarror(this.mPlayerRoot, 'Role', WarriorType.XianFeng, pos2);
                      let pos3 = new Laya.Vector3(-3, 0, 5);
                      this.createWarror(this.mPlayerRoot, 'Role', WarriorType.XianFeng, pos3);
                  }
                  break;
              case 4:
                  {
                      let pos = new Laya.Vector3(3, 0, 5);
                      this.createWarror(this.mPlayerRoot, 'Role', WarriorType.XianFeng, pos);
                      let pos2 = new Laya.Vector3(3, 0, 7);
                      this.createWarror(this.mPlayerRoot, 'Role', WarriorType.XianFeng, pos2);
                      let pos3 = new Laya.Vector3(-3, 0, 5);
                      this.createWarror(this.mPlayerRoot, 'Role', WarriorType.XianFeng, pos3);
                      let pos4 = new Laya.Vector3(-3, 0, 7);
                      this.createWarror(this.mPlayerRoot, 'Role', WarriorType.XianFeng, pos4);
                  }
                  break;
              default:
                  break;
          }
      }
      addXianFeng() {
          let lv = Math.max(1, GameData.data.getWarriorLv(WarriorType.XianFeng));
          let pos = Util.ZERO.clone();
          switch (lv) {
              case 1:
                  {
                      pos = new Laya.Vector3(3, 0, 5);
                  }
                  break;
              case 9:
                  {
                      pos = new Laya.Vector3(3, 0, 7);
                  }
                  break;
              case 17:
                  {
                      pos = new Laya.Vector3(-3, 0, 5);
                  }
                  break;
              case 25:
                  {
                      pos = new Laya.Vector3(-3, 0, 7);
                  }
                  break;
              default:
                  break;
          }
          if (pos.z <= 0)
              return;
          this.createWarror(this.mPlayerRoot, 'Role', WarriorType.XianFeng, pos);
      }
      createWarror(parent, name, type, pos) {
          this.createSprite3dAsyn(name, name, parent, Laya.Handler.create(this, (type, pos, clone3d) => {
              clone3d.transform.position = pos.clone();
              clone3d.transform.setWorldLossyScale(new Laya.Vector3(1, 1, 1));
              clone3d.transform.rotationEuler = new Laya.Vector3(0, 0, 0);
              this.removeComponent(clone3d);
              let soldierSc = null;
              switch (type) {
                  case WarriorType.DaoBing:
                      {
                          soldierSc = clone3d.addComponent(DaoBing);
                          this.playerList.push(soldierSc);
                      }
                      break;
                  case WarriorType.QiangBing:
                      {
                          soldierSc = clone3d.addComponent(QiangBing);
                          this.playerList.push(soldierSc);
                      }
                      break;
                  case WarriorType.BiaoQiangBing:
                      {
                          soldierSc = clone3d.addComponent(BiaoQiangBing);
                          this.playerList.push(soldierSc);
                      }
                      break;
                  case WarriorType.QiBing:
                      soldierSc = clone3d.addComponent(QiBing);
                      this.playerList.push(soldierSc);
                      break;
                  case WarriorType.XianFeng:
                      soldierSc = clone3d.addComponent(XianFeng);
                      this.playerList.push(soldierSc);
                      break;
                  case WarriorType.GongJian:
                      soldierSc = clone3d.addComponent(GongJian);
                      this.archerList.push(soldierSc);
                      break;
                  default:
                      break;
              }
              soldierSc.teamType = TeamType.Blue;
          }, [type, pos]));
      }
      createSprite3dByPosData(posdata, name = '') {
          let resourceName = name.length > 0 ? name : posdata.name;
          console.log('resourceName', resourceName);
          var original = Resource$3.loadSprite3d(resourceName);
          let element3d = ObjectPool$3.getSprite3d(original);
          element3d.name = posdata.name;
          element3d.active = true;
          element3d.transform.position = new Laya.Vector3(posdata.posX / 100, posdata.posY / 100, posdata.posZ / 100);
          element3d.transform.rotation = new Laya.Quaternion(posdata.rotationX / 100, posdata.rotationY / 100, posdata.rotationZ / 100, posdata.rotationW / 100);
          element3d.transform.setWorldLossyScale(new Laya.Vector3(posdata.scaleX / 100, posdata.scaleY / 100, posdata.scaleZ / 100));
          return element3d;
      }
      createSprite3dByRoleData(posdata, name = '') {
          let resourceName = name.length > 0 ? name : posdata.name;
          console.log('resourceName', resourceName);
          var original = Resource$3.loadSprite3d(resourceName);
          let element3d = ObjectPool$3.getSprite3d(original);
          element3d.name = posdata.name;
          element3d.active = true;
          element3d.transform.position = new Laya.Vector3(posdata.posX / 100, posdata.posY / 100, posdata.posZ / 100);
          element3d.transform.rotation = new Laya.Quaternion(posdata.rotationX / 100, posdata.rotationY / 100, posdata.rotationZ / 100, posdata.rotationW / 100);
          element3d.transform.setWorldLossyScale(new Laya.Vector3(1, 1, 1));
          return element3d;
      }
      createSprite3dByRoleDataAsyn(posdata, onCompleted, name = '') {
          let resourceName = name.length > 0 ? name : posdata.name;
          Resource$3.load3dResAsyn(Util.getBaseURL(resourceName, '', 'lh'), Laya.Handler.create(this, (data, onCompleted, original) => {
              let element3d = ObjectPool$3.getSprite3d(original);
              element3d.name = data.name;
              element3d.active = true;
              element3d.transform.position = new Laya.Vector3(data.posX / 100, data.posY / 100, data.posZ / 100);
              element3d.transform.rotation = new Laya.Quaternion(data.rotationX / 100, data.rotationY / 100, data.rotationZ / 100, data.rotationW / 100);
              element3d.transform.setWorldLossyScale(new Laya.Vector3(1, 1, 1));
              if (onCompleted) {
                  onCompleted.runWith(element3d);
              }
          }, [posdata, onCompleted]), null, null);
      }
      createOtherSprite3dByPosDataAsyn(resRootPath, posdata, onCompleted, name = '') {
          let resourceName = name.length > 0 ? name : posdata.name;
          ResourceMgr.loadSprite3dAsyn(resRootPath, resourceName, Laya.Handler.create(this, (data, onCompleted, original) => {
              let element3d = ResourceMgr.getSprite3d(original);
              element3d.name = data.name;
              element3d.active = true;
              element3d.transform.position = new Laya.Vector3(data.posX / 100, data.posY / 100, data.posZ / 100);
              element3d.transform.rotationEuler = new Laya.Vector3(data.rotationX / 100, data.rotationY / 100, data.rotationZ / 100);
              element3d.transform.setWorldLossyScale(new Laya.Vector3(data.scaleX / 100, data.scaleY / 100, data.scaleZ / 100));
              if (onCompleted) {
                  onCompleted.runWith(element3d);
              }
          }, [posdata, onCompleted]), null);
      }
      createSprite3dByPosDataAsyn(posdata, onCompleted, name = '') {
          let resourceName = name.length > 0 ? name : posdata.name;
          Resource$3.load3dResAsyn(Util.getBaseURL(resourceName, '', 'lh'), Laya.Handler.create(this, (data, onCompleted, original) => {
              let element3d = ObjectPool$3.getSprite3d(original);
              element3d.name = data.name;
              element3d.active = true;
              element3d.transform.position = new Laya.Vector3(data.posX / 100, data.posY / 100, data.posZ / 100);
              element3d.transform.rotation = new Laya.Quaternion(data.rotationX / 100, data.rotationY / 100, data.rotationZ / 100, data.rotationW / 100);
              element3d.transform.setWorldLossyScale(new Laya.Vector3(data.scaleX / 100, data.scaleY / 100, data.scaleZ / 100));
              if (onCompleted) {
                  onCompleted.runWith(element3d);
              }
          }, [posdata, onCompleted]), null, null);
      }
      createSprite3d(name, label, parent) {
          let labelname = label.length > 0 ? label : name;
          var original = Resource$3.loadSprite3d(name);
          let clone3d = ObjectPool$3.getSprite3d(original);
          clone3d.name = labelname;
          clone3d.active = true;
          parent.addChild(clone3d);
          return clone3d;
      }
      createSprite3dAsyn(name, label, parent, onCompleted) {
          console.log(name);
          Resource$3.load3dResAsyn(Util.getBaseURL(name, '', 'lh'), Laya.Handler.create(this, (label, onCompleted, original) => {
              let clone3d = ObjectPool$3.getSprite3d(original);
              clone3d.name = label;
              clone3d.active = true;
              parent.addChild(clone3d);
              if (onCompleted) {
                  onCompleted.runWith(clone3d);
              }
          }, [label, onCompleted]), null, null);
      }
      createOtherSprite3dAsyn(resRootPath, name, label, parent, onCompleted) {
          ResourceMgr.loadSprite3dAsyn(resRootPath, name, Laya.Handler.create(this, (label, onCompleted, original) => {
              let clone3d = ResourceMgr.getSprite3d(original);
              clone3d.name = label;
              clone3d.active = true;
              parent.addChild(clone3d);
              if (onCompleted) {
                  onCompleted.runWith(clone3d);
              }
          }, [label, onCompleted]), null);
      }
      createOtherSprite3d(resRootPath, name, label, parent) {
          var original = ResourceMgr.loadSprite3d(resRootPath, name);
          let clone3d = ResourceMgr.getSprite3d(original);
          clone3d.name = label;
          clone3d.active = true;
          parent.addChild(clone3d);
          return clone3d;
      }
      createPlayerSoldier(type) {
          let name = 'Role';
          if (type == WarriorType.QiBing) {
              name = 'Role_Cowboy';
          }
          let count = 2 + GameData.data.warriors;
          let row = 1 + Math.floor((count - 1) / 4);
          let col = 1 + (count - 1) % 4;
          let totalWidth = 0.8 * ((row > 1 ? 4 : col) - 1);
          let start = (totalWidth * 0.5);
          for (let z = 0; z < row; z++) {
              let _col = 4;
              if (z == row - 1) {
                  _col = col;
              }
              let posZ = this.playerStartPos.z - z;
              for (let index = 0; index < _col; index++) {
                  let posX = start - 0.8 * index;
                  let pos = new Laya.Vector3(posX, 0, posZ);
                  this.createWarror(this.mPlayerRoot, name, type, pos);
              }
          }
      }
      createEnemy(parent, name, type, pos) {
          this.createSprite3dAsyn(name, name, parent, Laya.Handler.create(this, (type, pos, clone3d) => {
              clone3d.transform.position = pos.clone();
              clone3d.transform.setWorldLossyScale(new Laya.Vector3(1, 1, 1));
              clone3d.transform.rotationEuler = new Laya.Vector3(0, 180, 0);
              this.removeComponent(clone3d);
              let soldierSc = null;
              switch (type) {
                  case EnemyType.DaoBing:
                      {
                          soldierSc = clone3d.addComponent(DaoBing);
                      }
                      break;
                  case EnemyType.QiangBing:
                      {
                          soldierSc = clone3d.addComponent(QiangBing);
                      }
                      break;
                  case EnemyType.BiaoQiangBing:
                      {
                          soldierSc = clone3d.addComponent(BiaoQiangBing);
                      }
                      break;
                  case EnemyType.QiBing:
                      soldierSc = clone3d.addComponent(QiBing);
                      break;
                  case EnemyType.GongChengChui:
                      soldierSc = clone3d.addComponent(GongChengChui);
                      break;
                  case EnemyType.DaXiang:
                      soldierSc = clone3d.addComponent(DaXiang);
                      break;
                  default:
                      break;
              }
              soldierSc.teamType = TeamType.Red;
              this.enemyList.push(soldierSc);
              this.mEnemyLave -= 1;
          }, [type, pos]));
      }
      createEnemySoldier() {
          if (this.tempTotal < 1)
              return;
          let type = EnemyType.DaoBing;
          let maxcount = 10;
          if (GameData.data.curStage < 6) {
              maxcount = 3;
          }
          else if (GameData.data.curStage < 10 && GameData.data.curStage >= 6) {
              maxcount = 4;
          }
          else if (GameData.data.curStage < 20 && GameData.data.curStage >= 10) {
              maxcount = 5;
          }
          else if (GameData.data.curStage < 50 && GameData.data.curStage >= 20) {
              maxcount = 7;
          }
          else {
              maxcount = 10;
          }
          let count = Util.random(2, maxcount + 1);
          if (GameData.data.curStage % 30 == 0) {
              type = EnemyType.DaXiang;
              count = 1;
              let name = 'Role_Elephant';
              let pos = new Laya.Vector3(0, 0, this.npcStartPos.z);
              this.createEnemy(this.mEemeyRoot, name, type, pos);
          }
          else {
              count = Math.min(this.enemyLave, count);
              let enemyIndex = [];
              let enemyListCount = this.enemys.length - 1;
              for (let i = 0; i < enemyListCount; i++) {
                  if (this.enemys[i] > 0) {
                      enemyIndex.push(i);
                  }
              }
              let row = 1 + Math.floor((count - 1) / 7);
              let col = 1 + (count - 1) % 7;
              let totalWidth = 0.8 * ((row > 1 ? 7 : col) - 1);
              let start = (totalWidth * 0.5);
              let posZ = this.npcStartPos.z;
              for (let z = 0; z < row; z++) {
                  let _col = 7;
                  if (z == row - 1) {
                      _col = col;
                  }
                  posZ += 1;
                  for (let index = 0; index < _col; index++) {
                      if (enemyIndex.length == 0)
                          break;
                      type = enemyIndex[Util.random(0, enemyIndex.length)];
                      this.enemys[type] = this.enemys[type] - 1;
                      enemyIndex = [];
                      for (let i = 0; i < enemyListCount; i++) {
                          if (this.enemys[i] > 0) {
                              enemyIndex.push(i);
                          }
                      }
                      let name = 'Role';
                      if (type == EnemyType.QiBing) {
                          name = 'Role_Cowboy';
                      }
                      let posX = start - 0.8 * index;
                      let pos = new Laya.Vector3(posX, 0, posZ);
                      if (type == EnemyType.GongChengChui) {
                          let baseData = GameData.data.getEnemyData(ResConst.EnemyCfgName[type]);
                          let lv = Math.floor((GameData.data.curStage - baseData.upgrade) / baseData.upgradeadd);
                          lv = Util.clamp(lv, 0, 3);
                          name = 'BatteringRam-' + lv + 'UP';
                          posZ += 4;
                          pos = new Laya.Vector3(0, 0, posZ);
                          posZ += 2;
                      }
                      this.createEnemy(this.mEemeyRoot, name, type, pos);
                  }
              }
          }
          this.tempTotal -= count;
      }
      initPlayer() {
          return null;
      }
      initPlayerAsyn(onCompleted) {
          this.npcs = [];
          this.players = [];
      }
      initVsCharacterAsyn(onCompleted) {
      }
      removeEnemy(character) {
          let index = this.enemyList.indexOf(character);
          index != -1 && this.enemyList.splice(index, 1);
          index = this.removeList.indexOf(character);
          index != -1 && this.removeList.splice(index, 1);
          let sprite3d = character.sprite3d;
          this.removeComponent(sprite3d);
          sprite3d.active = false;
          ObjectPool$3.recoverSprite3d(sprite3d);
      }
      removePlayer(character) {
          let index = this.playerList.indexOf(character);
          index != -1 && this.playerList.splice(index, 1);
          index = this.removeList.indexOf(character);
          index != -1 && this.removeList.splice(index, 1);
          let sprite3d = character.sprite3d;
          this.removeComponent(sprite3d);
          sprite3d.active = false;
          ObjectPool$3.recoverSprite3d(sprite3d);
      }
      removePlayerRole(character) {
          let index = this.players.indexOf(character);
          index != -1 && this.players.splice(index, 1);
          index = this.removeList.indexOf(character);
          index != -1 && this.removeList.splice(index, 1);
          let sprite3d = character.sprite3d;
          this.removeComponent(sprite3d);
          sprite3d.active = false;
          sprite3d.removeSelf();
          sprite3d = null;
      }
      removeNpcRole(character) {
          let index = this.npcs.indexOf(character);
          index != -1 && this.npcs.splice(index, 1);
          index = this.removeList.indexOf(character);
          index != -1 && this.removeList.splice(index, 1);
          let sprite3d = character.sprite3d;
          this.removeComponent(sprite3d);
          sprite3d.active = false;
          sprite3d.removeSelf();
          sprite3d = null;
      }
      enemyToRemoveList(character) {
          let index = this.enemyList.indexOf(character);
          index != -1 && this.enemyList.splice(index, 1);
          index = this.removeList.indexOf(character);
          if (index == -1) {
              this.removeList.push(character);
          }
      }
      playerToRemoveList(character) {
          let index = this.playerList.indexOf(character);
          index != -1 && this.playerList.splice(index, 1);
          index = this.removeList.indexOf(character);
          if (index == -1) {
              this.removeList.push(character);
          }
          if (!this.playDieSfx) {
              AudioManager.Instance.playSfx('die.mp3');
              this.playDieSfx = true;
              Laya.stage.timerOnce(2000, this, () => {
                  this.playDieSfx = false;
              }, null, false);
          }
      }
      onDestroy() {
          for (let i = this.playerList.length - 1; i >= 0; i--) {
              let character = this.playerList[i];
              let sprite3d = character.sprite3d;
              this.playerList[i] = null;
              if (character.healthBar) {
                  character.healthBar.setActive(false);
                  HealthBar.recover(character.healthBar);
              }
              this.removeComponent(sprite3d);
              if (sprite3d && !sprite3d.destroyed) {
                  character.destroy();
                  sprite3d.active = false;
                  character = null;
                  ObjectPool$3.recoverSprite3d(sprite3d);
              }
          }
          for (let i = this.enemyList.length - 1; i >= 0; i--) {
              let character = this.enemyList[i];
              if (character.healthBar) {
                  character.healthBar.setActive(false);
                  HealthBar.recover(character.healthBar);
              }
              let sprite3d = character.sprite3d;
              this.enemyList[i] = null;
              this.removeComponent(sprite3d);
              if (sprite3d && !sprite3d.destroyed) {
                  character.destroy();
                  sprite3d.active = false;
                  character = null;
                  ObjectPool$3.recoverSprite3d(sprite3d);
              }
          }
          for (let i = this.archerList.length - 1; i >= 0; i--) {
              let character = this.archerList[i];
              let sprite3d = character.sprite3d;
              this.archerList[i] = null;
              this.removeComponent(sprite3d);
              if (sprite3d && !sprite3d.destroyed) {
                  character.destroy();
                  sprite3d.active = false;
                  character = null;
                  ObjectPool$3.recoverSprite3d(sprite3d);
              }
          }
          for (let i = this.removeList.length - 1; i >= 0; i--) {
              let character = this.removeList[i];
              if (character.healthBar) {
                  character.healthBar.setActive(false);
                  HealthBar.recover(character.healthBar);
              }
              let sprite3d = character.sprite3d;
              this.removeList[i] = null;
              this.removeComponent(sprite3d);
              if (sprite3d && !sprite3d.destroyed) {
                  sprite3d.removeSelf();
                  sprite3d.destroy(true);
                  character.destroy();
                  sprite3d.active = false;
                  character = null;
                  sprite3d = null;
              }
          }
          this.enemyList = [];
          this.removeList = [];
          this.playerList = [];
          this.archerList = [];
          this.clearRootSprite(this.stageroot3d);
          this.clearSprite(this.mEemeyRoot);
          this.clearSprite(this.mPlayerRoot);
      }
      removeComponent(sprite3d) {
          if (sprite3d) {
              let scripts3d = sprite3d.getComponents(Laya.Script3D);
              if (scripts3d) {
                  for (let index = 0; index < scripts3d.length; index++) {
                      let script = scripts3d[index];
                      if (script) {
                          script.destroy();
                      }
                  }
              }
              let scripts = sprite3d.getComponents(Laya.Script);
              if (scripts) {
                  for (let index = 0; index < scripts.length; index++) {
                      let script = scripts[index];
                      if (script) {
                          script.destroy();
                      }
                  }
              }
          }
      }
      getSoldierData(type, lv) {
          let baseData = GameData.data.getWarriorData(ResConst.WarriorCfgName[type]);
          lv = Math.min(baseData.lv_max, lv);
          let hpRate = 1;
          if (fc.platform.PlatformConfig.platformTag == 'iptv') {
              hpRate = 2;
          }
          let data = {
              name: baseData.name,
              hp: hpRate * (baseData.hp + baseData.hpadd * (lv - 1)),
              hpadd: baseData.hpadd,
              atk: baseData.atk + baseData.atkadd * (lv - 1),
              atkadd: baseData.atkadd,
              defense: baseData.defense + baseData.defenseadd * (lv - 1),
              defenseadd: baseData.defenseadd,
              upgrade: baseData.upgrade * (1 + Math.floor((lv - 1) / 3)) + Math.floor(((1 + lv) * lv * 0.5) * baseData.upgradeadd),
              upgradeadd: baseData.upgradeadd,
              speed: baseData.speed,
              shotspeed: baseData.shotspeed,
              atkdist: baseData.atkdist,
              interval: baseData.interval,
              lv_max: baseData.lv_max
          };
          return data;
      }
      getEnemySoldierData(type) {
          let baseData = GameData.data.getEnemyData(ResConst.EnemyCfgName[type]);
          let lv = 1 + Math.floor((GameData.data.curStage - baseData.upgrade) / baseData.upgradeadd);
          lv = Math.max(1, lv);
          lv = Math.min(baseData.lv_max, lv);
          let rate = 1;
          if (fc.platform.PlatformConfig.platformTag == 'iptv') {
              rate = 2;
          }
          let hpRate = 1;
          if (fc.platform.PlatformConfig.platformTag == 'iptv') {
              hpRate = 2;
          }
          let data = {
              name: baseData.name,
              hp: hpRate * (baseData.hp + baseData.hpadd * (lv - 1)),
              hpadd: baseData.hpadd,
              atk: baseData.atk + baseData.atkadd * (lv - 1),
              atkadd: baseData.atkadd,
              defense: baseData.defense,
              defenseadd: baseData.defenseadd,
              upgrade: baseData.upgrade,
              upgradeadd: baseData.upgradeadd,
              speed: baseData.speed,
              shotspeed: baseData.shotspeed,
              interval: baseData.interval,
              atkdist: baseData.atkdist,
              gold: Math.floor(rate * (baseData.gold + baseData.goldadd * GameData.data.curStage)),
              goldadd: baseData.goldadd,
              lv_max: baseData.lv_max
          };
          return data;
      }
      getEnmeyByRang(rang, centerPos) {
          let enemyList = this.getActiveEnmey();
          let result = [];
          let dis = rang * rang;
          for (let i = enemyList.length - 1; i >= 0; i--) {
              let brick = enemyList[i];
              let newDis = Laya.Vector3.distanceSquared(brick.mtransform.position, centerPos);
              if (newDis <= dis) {
                  result[result.length] = brick;
              }
          }
          if (result.length > 0)
              return result;
          return [];
      }
      getActiveEnmey() {
          let list = [];
          for (let index = 0; index < this.enemyList.length; index++) {
              let enemy = this.enemyList[index];
              if (enemy && enemy.owner.active && enemy.isLife()) {
                  list.push(enemy);
              }
          }
          return list;
      }
  }
  GameMgr.instance = null;

  var GameFrame$8 = fc.framework.GameFrame;
  class HomePageUI extends Laya.Script {
      onAwake() {
      }
      onEnable() {
          this.castleBtn.on(Laya.Event.CLICK, this, this.openCastle);
          this.normalBtn.on(Laya.Event.CLICK, this, this.openNormal);
          this.warriorBtn.on(Laya.Event.CLICK, this, this.openWarrior);
          this.onlineGoldBtn && (this.onlineGoldBtn.on(Laya.Event.CLICK, this, this.openOnlineAward));
          Laya.stage.on(fc.define.Event.GOLD_UPDATE, this, this.updateGold);
          Laya.stage.on(EventConst.Update_Online, this, this.updaeOnline);
          this.updateGold();
          this.initOnline();
      }
      updateGold() {
          this.castleBtn.getChildByName('light').visible = false;
          this.checkCastleUprade(CastleCfg.Wall);
          this.checkCastleUprade(CastleCfg.Archer);
          this.checkCastleUprade(CastleCfg.Tower);
          this.warriorBtn.getChildByName('light').visible = false;
          this.checkWaariorUprade(WarriorType.DaoBing);
          this.checkWaariorUprade(WarriorType.QiangBing);
          this.checkWaariorUprade(WarriorType.BiaoQiangBing);
          this.checkWaariorUprade(WarriorType.QiBing);
      }
      checkWaariorUprade(type) {
          let lv = Math.max(0, GameData.data.getWarriorLv(type));
          let data = GameMgr.Instance().getSoldierData(type, lv);
          if (lv < 1) {
              return;
          }
          let gold = data.upgrade;
          if (GameData.data.gameGold >= gold) {
              this.warriorBtn.getChildByName('light').visible = true;
          }
      }
      checkCastleUprade(type) {
          let lv = Math.max(0, GameData.data.getCastleLv(type));
          if (lv < 1)
              return;
          let data = GameData.data.castleCfg[type];
          let gold = data.upgrade + Math.floor(((1 + (lv - 1)) * (lv - 1) * 0.5) * data.upgradeadd);
          if (GameData.data.gameGold >= gold) {
              this.castleBtn.getChildByName('light').visible = true;
          }
      }
      onDisable() {
          this.castleBtn.off(Laya.Event.CLICK, this, this.openCastle);
          this.normalBtn.off(Laya.Event.CLICK, this, this.openNormal);
          this.warriorBtn.off(Laya.Event.CLICK, this, this.openWarrior);
          Laya.stage.off(fc.define.Event.GOLD_UPDATE, this, this.updateGold);
          this.onlineGoldBtn && (this.onlineGoldBtn.off(Laya.Event.CLICK, this, this.openOnlineAward));
          Laya.stage.off(EventConst.Update_Online, this, this.updaeOnline);
      }
      onStart() {
          if (fc.view.ActivityCenter['gameTime'] < 1 && GameData.data.getGameTime(GameTimeType.Offline) > 0 && fc.platform.PlatformConfig.platformTag == 'iptv') {
              fc.framework.GameFrame.openView('OfflineAwardPage', null, null);
          }
      }
      openCastle() {
          GameFrame$8.playClickSound();
          this.showCustomAd();
          fc.framework.GameFrame.openView('CastlePage', null, null);
      }
      openNormal() {
          GameFrame$8.playClickSound();
          this.showCustomAd();
          fc.framework.GameFrame.openView('CommonPage', null, null);
      }
      openWarrior() {
          GameFrame$8.playClickSound();
          this.showCustomAd();
          fc.framework.GameFrame.openView('WarriorPage', null, null);
      }
      showCustomAd() {
          if (fc.view.ActivityCenter.isMistakenActive() && fc.platform.PlatformConfig.platformTag == 'vivo') {
              fc.framework.GameFrame.openView('CustomAd', null, null);
          }
          if (fc.view.ActivityCenter.isMistakenActive() && (fc.platform.PlatformConfig.platformTag == 'ks' || fc.platform.PlatformConfig.platformTag == 'byte' || fc.platform.PlatformConfig.platformTag == 'uc'
              || fc.platform.PlatformConfig.platformTag == 'xm' || (fc.platform.PlatformConfig.platformTag == 'hw'))) {
              fc.framework.GameFrame.openView('InsertAd', null, null);
          }
      }
      openOnlineAward() {
          GameFrame$8.playClickSound();
          fc.framework.GameFrame.openView('OnlineAwardPage', null, null);
      }
      updaeOnline(min) {
          if (this.onlineGoldBtn) {
              this.onlineGoldBtn.getChildByName('time').text = min + '分钟';
          }
      }
      initOnline() {
          let online = GameData.data.getGameTime(GameTimeType.Online);
          this.updaeOnline(online);
      }
  }

  var GameFrame$9 = fc.framework.GameFrame;
  class GamePlayUI extends Laya.Script {
      constructor() {
          super();
          this.unitCount = 2;
          this.towerCoolTime = 0;
          this.catapultCoolTime = 0;
          this.strongCoolTime = 0;
          this.strongTime = 0;
          this.moveDir = Util.ZERO.clone();
          this.lastClickTime = 0;
          this.unitCoolTimesTotal = 0;
          this.preWidth = 0;
          this.towerLaveCool = 0;
          this.catapultLaveCool = 0;
          this.strongLaveCool = 0;
          this.strongCoutdown = 0;
          this._outPos = new Laya.Vector4();
      }
      onAwake() {
          super.onAwake();
          Laya.stage.on(EventConst.Game_Update_Gold, this, this.updateGold);
          Laya.stage.on(EventConst.SHOW_GUIDE, this, this.showGuide);
          Laya.stage.on(EventConst.HIDE_GUIDE, this, this.hideGuide);
          Laya.stage.on(EventConst.Game_Count_Tip, this, this.updateScoreTip);
          Laya.stage.on(EventConst.SHOW_BUILD_TIP, this, this.showBuildTip);
          Laya.stage.on(EventConst.HIDE_BUILD_TIP, this, this.hideBuildTip);
          this.towerbtn.on(Laya.Event.CLICK, this, this.fireTowerArrow);
          this.bombbtn.on(Laya.Event.CLICK, this, this.fireBomb);
          this.strongbtn.on(Laya.Event.CLICK, this, this.onStrong);
          Laya.stage.on(fc.define.Event.CHOOSE_SKILLED, this, this.chooseSkill);
          this.rightBtn && (this.rightBtn.on(Laya.Event.CLICK, this, this.rightMove));
          this.leftBtn && (this.leftBtn.on(Laya.Event.CLICK, this, this.leftMove));
      }
      onStart() {
          this.unitBox.getChildByName('count').text = this.unitCount.toString();
          this.unitProgress = this.unitBox.getChildByName('timeBox').getChildByName('panelProgress');
          this.unitBox.getChildByName('unitBtn0').on(Laya.Event.CLICK, this, this.createWarrior, [WarriorType.DaoBing]);
          this.unitBox.getChildByName('unitBtn1').on(Laya.Event.CLICK, this, this.createWarrior, [WarriorType.QiangBing]);
          this.unitBox.getChildByName('unitBtn2').on(Laya.Event.CLICK, this, this.createWarrior, [WarriorType.BiaoQiangBing]);
          this.unitBox.getChildByName('unitBtn3').on(Laya.Event.CLICK, this, this.createWarrior, [WarriorType.QiBing]);
          for (let index = 0; index < 4; index++) {
              if (index < GameData.data.warriors) {
                  this.unitBox.getChildByName('unitBtn' + index).visible = true;
              }
              else {
                  this.unitBox.getChildByName('unitBtn' + index).visible = false;
              }
              let icon = this.unitBox.getChildByName('unitBtn' + index).getChildByName('icon');
              let lv = Math.max(1, GameData.data.getWarriorLv(index));
              let stage = 1 + Math.floor((lv - 1) / 3);
              stage = Math.min(ResConst.WarriorLvName[index].length, stage);
              icon.skin = 'icon/' + index + '_' + (stage - 1) + '.png';
          }
          let wallLv = GameData.data.getCastleLv(CastleCfg.Wall);
          if (wallLv > 3) {
              let data = GameData.data.castleCfg.Tower;
              let lv = Math.max(1, GameData.data.getCastleLv(CastleCfg.Tower));
              this.towerCoolTime = Math.max(3000, data.cooling - (lv - 1) * data.coolingdown);
              this.towerbtn.visible = true;
          }
          if (wallLv >= 24) {
              let data = GameData.data.castleCfg.Catapult;
              let lv = Math.max(1, GameData.data.getCastleLv(CastleCfg.Catapult));
              this.catapultCoolTime = Math.max(5000, data.cooling - (lv - 1) * data.coolingdown);
              this.bombbtn.visible = true;
          }
          let bounslv = Math.max(1, GameData.data.getCommonLv(CommonCfg.HaoJiao));
          let bunsdata = GameData.data.commonCfg.HaoJiao;
          this.strongCoolTime = fc.framework.GameFrame.gameCfg.stronginterval - (bounslv - 1) * bunsdata.bonus2;
          this.strongTime = fc.framework.GameFrame.gameCfg.strongtime;
          this.cooltimeDownStrong();
          if (GameFrame$9.playerInfo.isInGuide == 0) {
              this.owner.getChildByName('bottomUI').getChildByName('guideBox').visible = false;
          }
          else {
              this.owner.getChildByName('bottomUI').getChildByName('guideBox').visible = true;
          }
      }
      onEnable() {
          super.onEnable();
      }
      onDisable() {
          super.onDisable();
          Laya.stage.off(EventConst.Game_Update_Gold, this, this.updateGold);
          Laya.stage.off(EventConst.Game_Count_Tip, this, this.updateScoreTip);
          Laya.stage.off(EventConst.SHOW_GUIDE, this, this.showGuide);
          Laya.stage.off(EventConst.HIDE_GUIDE, this, this.hideGuide);
          Laya.stage.off(EventConst.SHOW_BUILD_TIP, this, this.showBuildTip);
          Laya.stage.off(EventConst.HIDE_BUILD_TIP, this, this.hideBuildTip);
          this.towerbtn.off(Laya.Event.CLICK, this, this.fireTowerArrow);
          this.bombbtn.off(Laya.Event.CLICK, this, this.fireBomb);
          this.strongbtn.off(Laya.Event.CLICK, this, this.onStrong);
          Laya.stage.off(fc.define.Event.CHOOSE_SKILLED, this, this.chooseSkill);
          this.unitBox.getChildByName('unitBtn0').off(Laya.Event.CLICK, this, this.createWarrior);
          this.unitBox.getChildByName('unitBtn1').off(Laya.Event.CLICK, this, this.createWarrior);
          this.unitBox.getChildByName('unitBtn2').off(Laya.Event.CLICK, this, this.createWarrior);
          this.unitBox.getChildByName('unitBtn3').off(Laya.Event.CLICK, this, this.createWarrior);
          this.rightBtn && (this.rightBtn.off(Laya.Event.CLICK, this, this.rightMove));
          this.leftBtn && (this.leftBtn.off(Laya.Event.CLICK, this, this.leftMove));
      }
      rightMove() {
          this.moveDir.setValue(-4, 0, 0);
          GameLogic.cameraCtr.keyHandlerX(this.moveDir);
      }
      leftMove() {
          this.moveDir.setValue(4, 0, 0);
          GameLogic.cameraCtr.keyHandlerX(this.moveDir);
      }
      createWarrior(type) {
          Util.delayMouseEnabled(this.unitBox, 150);
          if (this.unitCount > 0) {
              GameFrame$9.playClickSound();
              if (Date.now() - this.lastClickTime > 1000) {
                  GameMgr.Instance().createPlayerSoldier(type);
              }
              else {
                  Laya.stage.timerOnce(1000 - (Date.now() - this.lastClickTime), this, (type) => {
                      GameMgr.Instance().createPlayerSoldier(type);
                  }, [type], false);
              }
              this.lastClickTime = Date.now();
              Laya.stage.event(EventConst.Open_Door);
              Laya.Tween.clearAll(this.unitBox);
              Laya.Tween.to(this.unitBox, {}, 5000, null, Laya.Handler.create(this, () => {
                  Laya.stage.event(EventConst.Close_Door);
              }));
              this.unitCount--;
              this.unitBox.getChildByName('count').text = this.unitCount.toString();
              let width = this.unitProgress.width - 110;
              this.unitProgress.width = width;
              if (this.unitCount == 1) {
                  this.unitCooltimeDown();
              }
              if (this.unitCount == 0) {
                  for (let index = 0; index < 4; index++) {
                      this.unitBox.getChildByName('unitBtn' + index).disabled = true;
                  }
              }
          }
          var playerInfo = GameFrame$9.playerInfo;
          if (playerInfo.isInGuide == 1) {
              GameFrame$9.savePlayerInfo({ isInGuide: 0, isNew: 0 });
              this.owner.getChildByName('bottomUI').getChildByName('guideBox').visible = false;
          }
      }
      unitCooltimeDown() {
          this.unitProgress.clearTimer(this, this.unitCooltime);
          this.unitCoolTimesTotal = fc.framework.GameFrame.gameCfg.unitinterval;
          this.preWidth = (110 / this.unitCoolTimesTotal) * 100;
          this.unitProgress.timerLoop(100, this, this.unitCooltime);
      }
      unitCooltime() {
          let width = this.unitProgress.width + this.preWidth;
          this.unitProgress.width = width;
          this.unitCount = Math.floor(width / 110);
          this.unitBox.getChildByName('count').text = this.unitCount.toString();
          if (this.unitCount > 0) {
              for (let index = 0; index < 4; index++) {
                  this.unitBox.getChildByName('unitBtn' + index).disabled = false;
              }
          }
          if (this.unitProgress.width >= 220) {
              this.unitProgress.width = 220;
              this.unitProgress.clearTimer(this, this.unitCooltime);
          }
      }
      cooltimeDownTower() {
          this.towerbtn.getChildByName('panel').height = 0;
          this.towerbtn.clearTimer(this, this.cooltimeTower);
          this.towerLaveCool = 0;
          this.towerbtn.timerLoop(100, this, this.cooltimeTower);
      }
      cooltimeTower() {
          this.towerLaveCool += 100;
          this.towerbtn.getChildByName('panel').height = 133 * this.towerLaveCool / this.towerCoolTime;
          if (this.towerLaveCool >= this.towerCoolTime) {
              this.towerbtn.clearTimer(this, this.cooltimeTower);
          }
      }
      fireTowerArrow() {
          if (GameMgr.Instance().getEnemyList().length < 1)
              return;
          if (this.towerbtn.getChildByName('panel').height >= 133) {
              GameFrame$9.playClickSound();
              Laya.stage.event(EventConst.Fire_Tower);
              this.cooltimeDownTower();
          }
      }
      cooltimeDownCatapult() {
          this.bombbtn.getChildByName('panel').height = 0;
          this.bombbtn.clearTimer(this, this.cooltimeCatapult);
          this.catapultLaveCool = 0;
          this.bombbtn.timerLoop(100, this, this.cooltimeCatapult);
      }
      cooltimeCatapult() {
          this.catapultLaveCool += 100;
          this.bombbtn.getChildByName('panel').height = 133 * this.catapultLaveCool / this.catapultCoolTime;
          if (this.catapultLaveCool >= this.catapultCoolTime) {
              this.bombbtn.clearTimer(this, this.cooltimeCatapult);
          }
      }
      fireBomb() {
          if (GameMgr.Instance().getEnemyList().length < 1)
              return;
          if (this.bombbtn.getChildByName('panel').height >= 133) {
              GameFrame$9.playClickSound();
              Laya.stage.event(EventConst.Fire_Catapult);
              this.cooltimeDownCatapult();
          }
      }
      cooltimeDownStrong() {
          this.strongbtn.getChildByName('panel').height = 0;
          this.strongbtn.clearTimer(this, this.cooltimeStrong);
          this.strongLaveCool = 0;
          this.strongbtn.timerLoop(100, this, this.cooltimeStrong);
      }
      cooltimeStrong() {
          this.strongLaveCool += 100;
          this.strongbtn.getChildByName('panel').height = 133 * this.strongLaveCool / this.strongCoolTime;
          if (this.strongLaveCool >= this.strongCoolTime) {
              this.strongbtn.clearTimer(this, this.cooltimeStrong);
          }
      }
      coutDownStrong() {
          this.strongBox.visible = true;
          this.strongtip.visible = true;
          AudioManager.Instance.playSfx('haojiao.mp3');
          Laya.stage.event(EventConst.Strong_Power_Start);
          GameData.data.atkUp = 1;
          this.strongBox.getChildByName('panelProgress').width = 180;
          this.strongBox.clearTimer(this, this.coundownStrong);
          this.strongCoutdown = this.strongTime;
          this.strongBox.timerLoop(50, this, this.coundownStrong);
          let val = this.strongBox.getChildByName('val');
          let bounslv = Math.max(1, GameData.data.getCommonLv(CommonCfg.HaoJiao));
          let bunsdata = GameData.data.commonCfg.HaoJiao;
          val.text = "+" + Math.floor((bounslv * bunsdata.bonus1) * 100) + "%伤害";
      }
      coundownStrong() {
          this.strongCoutdown -= 50;
          this.strongBox.getChildByName('panelProgress').width = 180 * this.strongCoutdown / this.strongTime;
          if (this.strongCoutdown <= 0) {
              this.strongBox.clearTimer(this, this.coundownStrong);
              Laya.stage.event(EventConst.Strong_Power_End);
              GameData.data.atkUp = 0;
              this.strongBox.visible = false;
              this.strongtip.visible = false;
          }
      }
      onStrong() {
          if (GameMgr.Instance().getEnemyList().length < 1)
              return;
          if (this.strongbtn.getChildByName('panel').height >= 133) {
              GameFrame$9.playClickSound();
              this.cooltimeDownStrong();
              this.coutDownStrong();
          }
      }
      onLateUpdate() {
          if (!this.strongtip.visible)
              return;
          GameLogic.camera.viewport.project(GameMgr.Instance().getScenario().transform.position, GameLogic.camera.projectionViewMatrix, this._outPos);
          let x = this._outPos.x / Laya.stage.clientScaleX;
          let y = this._outPos.y / Laya.stage.clientScaleY;
          this.strongtip.pos(x, y - 100);
      }
      showGuide() {
      }
      hideGuide() {
      }
      showBuildTip() {
          var playerInfo = fc.framework.GameFrame.playerInfo;
          if (playerInfo.isInGuide == 1) {
          }
      }
      hideBuildTip() {
          var playerInfo = fc.framework.GameFrame.playerInfo;
          if (playerInfo.isInGuide == 1) {
              fc.framework.GameFrame.savePlayerInfo({ isInGuide: 0, isNew: 0 });
          }
      }
      updateGold(gold) {
          GameData.data.gameGold = gold;
          Laya.stage.event(fc.define.Event.GOLD_UPDATE);
      }
      updateScoreTip(count, pos) {
          var tipLabel = Laya.Pool.getItemByCreateFun("tiplabel", this.mTipLabelPrefab.create, this.mTipLabelPrefab);
          Laya.Tween.clearAll(tipLabel);
          tipLabel.visible = true;
          tipLabel.value = (count > 0 ? "+" : "-") + Math.abs(count);
          tipLabel.x = pos.x;
          tipLabel.y = pos.y - 100;
          tipLabel.scaleX = 0.8;
          tipLabel.scaleY = 0.8;
          tipLabel.alpha = 1;
          this.owner.addChild(tipLabel);
          Laya.Tween.to(tipLabel, { y: pos.y - 150, alpha: 0.4 }, 300, null, Laya.Handler.create(this, () => {
              tipLabel.removeSelf();
              Laya.Pool.recover("tiplabel", tipLabel);
          }));
      }
      chooseSkill(data) {
          if (data) {
              if (data.id == 1) {
                  Laya.stage.event(EventConst.RENEW_MAX_HP);
              }
              if (data.id == 2) {
                  this.coutDownStrong();
              }
              if (data.id == 3) {
                  GameMgr.Instance().createPlayerSoldier(WarriorType.DaoBing);
              }
          }
      }
  }

  var GameFrame$a = fc.framework.GameFrame;
  class VideoForHpUI extends Laya.Script {
      constructor() {
          super();
      }
      onAwake() {
          super.onAwake();
          this.videoBtn.on(Laya.Event.CLICK, this, this.toAddGold);
          this.closeBtn.on(Laya.Event.CLICK, this, this.closeView);
      }
      onDestroy() {
          this.closeBtn.off(Laya.Event.CLICK, this, this.closeView);
          this.videoBtn.off(Laya.Event.CLICK, this, this.toAddGold);
      }
      closeView() {
          fc.framework.GameFrame.playClickSound();
          GameFrame$a.closeView(this.owner);
          GameFrame$a.resume();
      }
      onStart() {
          this.owner.zOrder = 2;
          this.closeBtn.addComponent(fc.view.DelayShow);
          GameFrame$a.pause();
      }
      toAddGold() {
          GameFrame$a.playClickSound();
          if (fc.framework.Channel.isVideoEnable()) {
              this.owner.mouseEnabled = false;
              fc.framework.Channel.playVideo(Laya.Handler.create(this, () => {
                  this.owner.mouseEnabled = true;
                  Laya.stage.event(EventConst.RENEW_MAX_HP);
                  this.closeView();
              }), Laya.Handler.create(this, function () {
                  this.owner.mouseEnabled = true;
              }), Laya.Handler.create(this, function () {
                  this.owner.mouseEnabled = true;
                  GameFrame$a.showToastMsg(GameFrame$a['langCfg'].novideotxt);
              }));
          }
          else {
              GameFrame$a.showToastMsg(GameFrame$a['langCfg'].novideotxt);
          }
      }
  }

  class InfiniteSlideGuideBox extends Laya.Script {
      constructor() { super(); }
      onAwake() {
          super.onAwake();
          Laya.stage.on(EventConst.SHOW_GUIDE, this, this.showGuide);
          Laya.stage.on(EventConst.HIDE_GUIDE, this, this.hideGuide);
      }
      onEnable() {
          super.onEnable();
      }
      onDisable() {
          super.onDisable();
          Laya.stage.off(EventConst.SHOW_GUIDE, this, this.showGuide);
          Laya.stage.off(EventConst.HIDE_GUIDE, this, this.hideGuide);
      }
      onStart() {
          this.owner && (this.owner.visible = false);
          for (const key in this.owner) {
              if (Object.prototype.hasOwnProperty.call(this.owner, key)) {
                  const element = this.owner[key];
                  if (element && typeof element == 'object' && element instanceof Laya.FrameAnimation) {
                      let ani = element;
                      ani.stop();
                      ani.play(0, true);
                  }
              }
          }
      }
      showGuide() {
          this.owner.visible = true;
      }
      hideGuide() {
          this.owner.visible = false;
      }
  }

  class LevelLabel extends Laya.Script {
      constructor() {
          super();
      }
      onAwake() {
          super.onAwake();
          this.owner.mouseEnabled = false;
          this.stageLabel && (this.stageLabel.text = '关卡 ' + GameData.data.curStage.toString());
      }
      onStart() {
          super.onStart();
          Laya.stage.event(fc.define.Event.GOLD_UPDATE);
      }
      onDisable() {
          super.onDisable();
      }
      onDestroy() {
          super.onDisable();
      }
  }

  class GameConst {
  }
  GameConst.nickname = [
      "南鸢离梦",
      "北柒陌人",
      "顾紫迎风",
      "欣风雨中",
      "水墨",
      "长烟",
      "剑胆琴心",
      "剑绝诗狂",
      "眉目成诗",
      "南风过熙",
      "盏茶作酒",
      "酒笙清栀",
      "素年凉音",
      "沐兮陌墨",
      "离歌倾城",
      "稳坐江山",
      "各生欢喜",
      "不念过往",
      "花开浅夏",
      "北城南笙",
      "南熙寒笙",
      "北染陌人",
      "浮生未歇",
      "一缕墨香",
      "一顾倾城",
      "浅陌初心",
      "落花倾城",
      "白衣酒客",
      "浅墨入画",
      "初雪未央",
      "日光倾城",
      "北城旧巷",
      "微笑向暖",
      "蝶舞轻盈",
      "静听年华",
      "顾恋轻尘",
      "乍然离场",
      "南街浊酒",
      "冷瞳少女",
      "墨染流云",
      "浅笑离愁",
      "独孤一剑",
      "蒹葭苍苍",
      "玉暖蓝田",
      "鱼书雁信",
      "兰舟独上",
      "染黛如诗",
      "低头浅笑",
      "清雅淡然",
      "浅夏花开",
      "森眸暖光",
      "顾北凉城",
      "终是辜负",
      "南笙一梦",
      "余生安好",
      "紫嫣轻娆",
      "北城不夏",
      "浅若清风",
      "旧巷念人",
      "日光倾城",
      "北葵向暖",
      "倾城之泪",
      "南巷清风",
      "浅若夏沫",
      "南拥夏栀",
      "余生有你",
      "清酒沾衣",
      "十里桃花",
      "眉间朱砂",
      "君已陌路",
      "君临天下",
      "长安故里",
      "温山软水",
      "醉酒当歌",
      "执手温酒",
      "执子之手",
      "一裘白衣",
      "半纸书笺",
      "清明雨上",
      "清茶一壶",
      "戎马万世",
      "花卿灼灼",
      "画眉如黛",
      "北朽暖栀",
      "花葬画沙",
      "北陌深巷",
      "雨落青衣",
      "青青子衿",
      "浅吟悲欢",
      "不问风月"
  ];

  class LevelProgress extends Laya.Script {
      constructor() {
          super();
      }
      onAwake() {
          super.onAwake();
          this.owner.mouseEnabled = false;
          Laya.stage.on(EventConst.Game_Update_Progress, this, this.updateProgress);
      }
      onStart() {
          super.onStart();
          this.playerLabel && (this.playerLabel.text = GameData.data.gameMode == 0 ? '玩家' : '蓝方');
          this.npcLabel && (this.npcLabel.text = GameData.data.gameMode == 0 ? GameConst.nickname[Util.random(0, GameConst.nickname.length)] : '红方');
      }
      onDisable() {
          super.onDisable();
          Laya.stage.off(EventConst.Game_Update_Progress, this, this.updateProgress);
      }
      onDestroy() {
          super.onDestroy();
          Laya.stage.off(EventConst.Game_Update_Progress, this, this.updateProgress);
      }
      updateProgress(progress, team) {
          if (team == TeamType.Blue) {
              progress = Math.min(1, progress);
              let width = progress * 256;
              this.progressImg && (this.progressImg.width = width);
          }
          else {
              progress = Math.min(1, progress);
              let width = progress * 256;
              this.progressNpcImg && (this.progressNpcImg.width = width);
          }
      }
  }

  class StartGame2 extends Laya.Script {
      constructor() {
          super();
      }
      onAwake() {
          super.onAwake();
          this.owner.on(Laya.Event.CLICK, this, this.startGame2);
          if (GameData.data.curStage > 5) {
              this.owner.getChildAt(0).visible = false;
          }
      }
      onDestroy() {
          this.owner.on(Laya.Event.CLICK, this, this.startGame2);
      }
      startGame() {
          if (fc.view.ActivityCenter.isEnableStartVideo()) {
              this.owner.mouseEnabled = false;
              fc.framework.Channel.playVideo(Laya.Handler.create(this, function () {
                  this.playGame();
                  this.owner.mouseEnabled = true;
              }), Laya.Handler.create(this, function () {
                  this.playGame();
                  this.owner.mouseEnabled = true;
              }), Laya.Handler.create(this, function () {
                  this.playGame();
                  this.owner.mouseEnabled = true;
              }));
          }
          else {
              this.playGame();
          }
      }
      playGame() {
          GameData.data.gameMode = 1;
          Laya.stage.event(EventConst.Game_Refresh);
          fc.framework.GameFrame.go2Game();
          fc.framework.GameFrame.closeView('Home');
      }
      startGame2() {
          fc.framework.GameFrame.playClickSound();
          if (GameData.data.curStage > 5) {
              this.startGame();
          }
          else {
              fc.framework.GameFrame.showToastMsg('5关之后解锁');
          }
      }
  }

  class GameConfig {
      constructor() {
      }
      static init() {
          var reg = Laya.ClassUtils.regClass;
          reg("compUI/CastleShow.ts", CastleShow);
          reg("compUI/WaveProgress.ts", WaveProgress);
          reg("compUI/HomePageUI.ts", HomePageUI);
          reg("compUI/GamePlayUI.ts", GamePlayUI);
          reg("compUI/VideoForHpUI.ts", VideoForHpUI);
          reg("compUI/InfiniteSlideGuideBox.ts", InfiniteSlideGuideBox);
          reg("compUI/LevelLabel.ts", LevelLabel);
          reg("compUI/LevelProgress.ts", LevelProgress);
          reg("compUI/StartGame2.ts", StartGame2);
      }
  }
  GameConfig.width = 1334;
  GameConfig.height = 750;
  GameConfig.scaleMode = "fixedauto";
  GameConfig.screenMode = "horizontal";
  GameConfig.alignV = "top";
  GameConfig.alignH = "left";
  GameConfig.startScene = "fc-layout/view/AdHomePage.scene";
  GameConfig.sceneRoot = "";
  GameConfig.debug = false;
  GameConfig.stat = false;
  GameConfig.physicsDebug = false;
  GameConfig.exportSceneToJson = true;
  GameConfig.init();

  var PlatformConfig = fc.platform.PlatformConfig;
  class Main {
      constructor() {
          Config.isAntialias = true;
          Config.useWebGL2 = false;
          if (window["Laya3D"])
              Laya3D.init(GameConfig.width, GameConfig.height);
          else
              Laya.init(GameConfig.width, GameConfig.height, Laya["WebGL"]);
          Laya["Physics"] && Laya["Physics"].enable();
          Laya["DebugPanel"] && Laya["DebugPanel"].enable();
          Laya.stage.bgColor = "#ffffff";
          Laya.stage.scaleMode = GameConfig.scaleMode;
          Laya.stage.screenMode = GameConfig.screenMode;
          Laya.URL.exportSceneToJson = GameConfig.exportSceneToJson;
          if (GameConfig.debug || Laya.Utils.getQueryString("debug") == "true")
              Laya.enableDebugPanel();
          if (GameConfig.physicsDebug && Laya["PhysicsDebugDraw"])
              Laya["PhysicsDebugDraw"].enable();
          if (GameConfig.stat)
              Laya.Stat.show();
          Laya.alertGlobalError(false);
          Laya.ResourceVersion.enable("version.json", Laya.Handler.create(this, this.onVersionLoaded), Laya.ResourceVersion.FILENAME_VERSION);
      }
      onVersionLoaded() {
          Laya.AtlasInfoManager.enable("fileconfig.json", Laya.Handler.create(this, this.onConfigLoaded));
      }
      onConfigLoaded() {
          PlatformConfig.init();
          Laya.stage.addComponent(GameLogic);
      }
  }
  new Main();

}());
//# sourceMappingURL=bundle.js.map
